/*
 * OPN2 Bank Editor by Wohlstand, a free tool for music bank editing
 * Copyright (c) 2017-2020 Vitaly Novichkov <admin@wohlnet.ru>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "ins_names.h"

static const MidiProgram XgSet[] =
{
    {'M',   0,   1,   0, "XG #001 Key Scale Panning", "GrndPnoK"},
    {'M',   0,   1,   1, "XG #001 Key Scale Panning", "BritPnoK"},
    {'M',   0,   1,   2, "XG #001 Key Scale Panning", "ElGrPnoK"},
    {'M',   0,   1,   3, "XG #001 Key Scale Panning", "HnkyTnkK"},
    {'M',   0,   1,   4, "XG #001 Key Scale Panning", "El.Pno1K"},
    {'M',   0,   1,   5, "XG #001 Key Scale Panning", "El.Pno2K"},
    {'M',   0,   1,   6, "XG #001 Key Scale Panning", "Harpsi.K"},
    {'M',   0,   1,   7, "XG #001 Key Scale Panning", "Clavi.K"},
    {'M',   0,   1,  11, "XG #001 Key Scale Panning", "VibesK"},
    {'M',   0,   1,  12, "XG #001 Key Scale Panning", "MarimbaK"},
    {'M',   0,   3,   1, "XG #003 Stereo", "+StBrtPno"},
    {'M',   0,   3,  16, "XG #003 Stereo", "+StDrawOr"},
    {'M',   0,   3,  48, "XG #003 Stereo", "S.Strngs"},
    {'M',   0,   3,  49, "XG #003 Stereo", "S.SlwStr"},
    {'M',   0,   3,  52, "XG #003 Stereo", "S.Choir"},
    {'M',   0,   3,  61, "XG #003 Stereo", "StBrsSec ***"},
    {'M',   0,   6,  34, "XG #006 Single", "PickBa 2"},
    {'M',   0,   6,  39, "XG #006 Single", "MelloSBa"},
    {'M',   0,   6,  60, "XG #006 Single", "FrHrSolo"},
    {'M',   0,   6,  80, "XG #006 Single", "SquarLd2"},
    {'M',   0,   6,  81, "XG #006 Single", "Saw Ld 2"},
    {'M',   0,   8,  40, "XG #008 Slow", "Slow Vln"},
    {'M',   0,   8,  44, "XG #008 Slow", "SlwTrStr"},
    {'M',   0,   8,  48, "XG #008 Slow", "Slow Str"},
    {'M',   0,   8,  49, "XG #008 Slow", "LegatoSt"},
    {'M',   0,   8,  50, "XG #008 Slow", "+Memory"},
    {'M',   0,   8,  64, "XG #008 Slow", "+VgSprnSx"},
    {'M',   0,   8,  80, "XG #008 Slow", "LMSquare"},
    {'M',   0,   8,  81, "XG #008 Slow", "ThickSaw"},
    {'M',   0,   8,  86, "XG #008 Slow", "+FifthLdS"},
    {'M',   0,   8, 102, "XG #008 Slow", "Echoes 2"},
    {'M',   0,  12,   5, "XG #012 Fast Decay", "+ChoEPDcy"},
    {'M',   0,  12,  30, "XG #012 Fast Decay", "DstRthmG **"},
    {'M',   0,  12,  39, "XG #012 Fast Decay", "Seq Bass"},
    {'M',   0,  12,  55, "XG #012 Fast Decay", "LoFi Hit ***"},
    {'M',   0,  12,  62, "XG #012 Fast Decay", "Quack Br"},
    {'M',   0,  12,  98, "XG #012 Fast Decay", "SynDrCmp"},
    {'M',   0,  14,  48, "XG #014 Double Attack", "+SfzStrgs"},
    {'M',   0,  14,  61, "XG #014 Double Attack", "SfrzndBr **"},
    {'M',   0,  14,  98, "XG #014 Double Attack", "Popcorn"},
    {'M',   0,  14, 102, "XG #014 Double Attack", "Echo Pan"},
    {'M',   0,  16,  24, "XG #016 Bright 1", "NylonGt2"},
    {'M',   0,  16,  25, "XG #016 Bright 1", "SteelGt2"},
    {'M',   0,  16,  37, "XG #016 Bright 1", "+BrtSlpSp"},
    {'M',   0,  16,  52, "XG #016 Bright 1", "Ch.Aahs2"},
    {'M',   0,  16,  56, "XG #016 Bright 1", "Trumpet2"},
    {'M',   0,  16,  58, "XG #016 Bright 1", "Tuba 2"},
    {'M',   0,  16,  87, "XG #016 Bright 1", "Big&Low"},
    {'M',   0,  16,  89, "XG #016 Bright 1", "ThickPad"},
    {'M',   0,  17,  56, "XG #017 Bright 2", "BriteTrp"},
    {'M',   0,  17,  89, "XG #017 Bright 2", "Soft Pad"},
    {'M',   0,  18,   0, "XG #018 Dark 1", "MelloGrP"},
    {'M',   0,  18,   4, "XG #018 Dark 1", "MelloEP1"},
    {'M',   0,  18,  26, "XG #018 Dark 1", "MelloGtr"},
    {'M',   0,  18,  33, "XG #018 Dark 1", "FingrDrk"},
    {'M',   0,  18,  38, "XG #018 Dark 1", "SynBa1Dk"},
    {'M',   0,  18,  39, "XG #018 Dark 1", "ClkSynBa"},
    {'M',   0,  18,  50, "XG #018 Dark 1", "+Zephyr"},
    {'M',   0,  18,  57, "XG #018 Dark 1", "Trmbone2"},
    {'M',   0,  18,  61, "XG #018 Dark 1", "MildBrss ***"},
    {'M',   0,  18,  63, "XG #018 Dark 1", "Soft Brs"},
    {'M',   0,  18,  65, "XG #018 Dark 1", "+ASax Lgt"},
    {'M',   0,  18,  80, "XG #018 Dark 1", "Hollow"},
    {'M',   0,  18,  81, "XG #018 Dark 1", "Dyna Saw"},
    {'M',   0,  18,  89, "XG #018 Dark 1", "Sine Pad"},
    {'M',   0,  18,  98, "XG #018 Dark 1", "TinyBell"},
    {'M',   0,  18,  99, "XG #018 Dark 1", "WarmAtms"},
    {'M',   0,  19,  39, "XG #019 Dark 2", "SynBa2Dk"},
    {'M',   0,  19,  80, "XG #019 Dark 2", "Shroud"},
    {'M',   0,  19,  81, "XG #019 Dark 2", "Digi Saw"},
    {'M',   0,  19,  99, "XG #019 Dark 2", "HollwRls"},
    {'M',   0,  20,   1, "XG #020 Resonant", "+ResoBrtP"},
    {'M',   0,  20,  38, "XG #020 Resonant", "FastResB"},
    {'M',   0,  20,  62, "XG #020 Resonant", "RezSynBr"},
    {'M',   0,  20,  81, "XG #020 Resonant", "Big Lead"},
    {'M',   0,  20,  95, "XG #020 Resonant", "Shwimmer"},
    {'M',   0,  21,  36, "XG #021 LFO-Cutoff Freq", "+CsmicSlp"},
    {'M',   0,  21,  38, "XG #021 LFO-Cutoff Freq", "+TL66"},
    {'M',   0,  21,  51, "XG #021 LFO-Cutoff Freq", "+TradeWnd"},
    {'M',   0,  22,  37, "XG #022 Vel-cutoff freq", "+Wah Slap"},
    {'M',   0,  22,  39, "XG #022 Vel-cutoff freq", "+Zealot"},
    {'M',   0,  24,  17, "XG #024 Attack", "70sPcOr1"},
    {'M',   0,  24,  30, "XG #024 Attack", "DistGtr2 **"},
    {'M',   0,  24,  38, "XG #024 Attack", "AcidBass"},
    {'M',   0,  24,  48, "XG #024 Attack", "Arco Str"},
    {'M',   0,  24,  62, "XG #024 Attack", "PolyBrss"},
    {'M',   0,  24,  81, "XG #024 Attack", "HeavySyn"},
    {'M',   0,  24,  85, "XG #024 Attack", "SynthAah"},
    {'M',   0,  25,   6, "XG #025 Release", "Harpsi.2"},
    {'M',   0,  25,  24, "XG #025 Release", "NylonGt3"},
    {'M',   0,  25,  81, "XG #025 Release", "WaspySyn"},
    {'M',   0,  26,  81, "XG #026 Sweep", "Mondo ***"},
    {'M',   0,  27,   7, "XG #027 Resonant Sweep", "ClaviWah"},
    {'M',   0,  27,  33, "XG #027 Resonant Sweep", "FlangeBa"},
    {'M',   0,  27,  36, "XG #027 Resonant Sweep", "ResoSlap"},
    {'M',   0,  27,  38, "XG #027 Resonant Sweep", "ResoBass ***"},
    {'M',   0,  27,  50, "XG #027 Resonant Sweep", "Reso Str"},
    {'M',   0,  27,  62, "XG #027 Resonant Sweep", "SynBrss3"},
    {'M',   0,  27,  81, "XG #027 Resonant Sweep", "RezzySaw ***"},
    {'M',   0,  27,  95, "XG #027 Resonant Sweep", "Converge"},
    {'M',   0,  27,  97, "XG #027 Resonant Sweep", "Prologue"},
    {'M',   0,  28,  34, "XG #028 Muted", "MutePkBa"},
    {'M',   0,  28,  38, "XG #028 Muted", "+MtdPulsB"},
    {'M',   0,  28, 105, "XG #028 Muted", "MuteBnjo"},
    {'M',   0,  29,  38, "XG #029 Complex-FEG", "+SlwAttck"},
    {'M',   0,  29,  62, "XG #029 Complex-FEG", "+AnalgSfz"},
    {'M',   0,  32,   1, "XG #032 Detune 1", "+DtndBrtP"},
    {'M',   0,  32,   2, "XG #032 Detune 1", "Det.CP80"},
    {'M',   0,  32,   4, "XG #032 Detune 1", "Chor.EP1"},
    {'M',   0,  32,   5, "XG #032 Detune 1", "Chor.EP2"},
    {'M',   0,  32,   6, "XG #032 Detune 1", "HarpsiDt"},
    {'M',   0,  32,  16, "XG #032 Detune 1", "DetDrwOr"},
    {'M',   0,  32,  17, "XG #032 Detune 1", "DetPrcOr"},
    {'M',   0,  32,  19, "XG #032 Detune 1", "ChurOrg3"},
    {'M',   0,  32,  20, "XG #032 Detune 1", "+ReedOrDt"},
    {'M',   0,  32,  21, "XG #032 Detune 1", "AccordIt"},
    {'M',   0,  32,  22, "XG #032 Detune 1", "Harmo. 2"},
    {'M',   0,  32,  24, "XG #032 Detune 1", "+NylnGtDt"},
    {'M',   0,  32,  25, "XG #032 Detune 1", "+StlGtrDt"},
    {'M',   0,  32,  26, "XG #032 Detune 1", "Jazz Amp"},
    {'M',   0,  32,  27, "XG #032 Detune 1", "ChorusGt"},
    {'M',   0,  32,  29, "XG #032 Detune 1", "+OdrvGtDt"},
    {'M',   0,  32,  33, "XG #032 Detune 1", "+FngrBaDt"},
    {'M',   0,  32,  35, "XG #032 Detune 1", "Fretles2"},
    {'M',   0,  32,  36, "XG #032 Detune 1", "PunchThm"},
    {'M',   0,  32,  39, "XG #032 Detune 1", "SmthSynB"},
    {'M',   0,  32,  52, "XG #032 Detune 1", "MelChoir"},
    {'M',   0,  32,  56, "XG #032 Detune 1", "Warm Trp"},
    {'M',   0,  32,  60, "XG #032 Detune 1", "FrHorn 2"},
    {'M',   0,  32,  62, "XG #032 Detune 1", "JumpBrss"},
    {'M',   0,  32,  81, "XG #032 Detune 1", "DoublSaw ***"},
    {'M',   0,  32, 104, "XG #032 Detune 1", "DetSitar"},
    {'M',   0,  33,   5, "XG #033 Detune 2", "DX Hard"},
    {'M',   0,  33,  16, "XG #033 Detune 2", "60sDrOr1"},
    {'M',   0,  33,  17, "XG #033 Detune 2", "Lite Org"},
    {'M',   0,  33,  27, "XG #033 Detune 2", "+ChoGtrLt"},
    {'M',   0,  33,  35, "XG #033 Detune 2", "Fretles3"},
    {'M',   0,  34,   5, "XG #034 Detune 3", "DXLegend"},
    {'M',   0,  34,  16, "XG #034 Detune 3", "60sDrOr2"},
    {'M',   0,  34,  35, "XG #034 Detune 3", "Fretles4"},
    {'M',   0,  35,   2, "XG #035 Octave 1", "+Synth CP"},
    {'M',   0,  35,   6, "XG #035 Octave 1", "Harpsi.3"},
    {'M',   0,  35,  15, "XG #035 Octave 1", "Dulcimr2"},
    {'M',   0,  35,  16, "XG #035 Octave 1", "70sDrOr1"},
    {'M',   0,  35,  19, "XG #035 Octave 1", "ChurOrg2"},
    {'M',   0,  35,  25, "XG #035 Octave 1", "12StrGtr"},
    {'M',   0,  35,  30, "XG #035 Octave 1", "DistGtr3 **"},
    {'M',   0,  35,  38, "XG #035 Octave 1", "Clv Bass"},
    {'M',   0,  35,  45, "XG #035 Octave 1", "+Pizz Oct"},
    {'M',   0,  35,  48, "XG #035 Octave 1", "60sStrng"},
    {'M',   0,  35,  50, "XG #035 Octave 1", "Syn Str3 **"},
    {'M',   0,  35,  55, "XG #035 Octave 1", "OrchHit2"},
    {'M',   0,  35,  61, "XG #035 Octave 1", "Tp&TbSec"},
    {'M',   0,  35,  80, "XG #035 Octave 1", "2 Osc ***"},
    {'M',   0,  35,  81, "XG #035 Octave 1", "Toy Lead ***"},
    {'M',   0,  35,  86, "XG #035 Octave 1", "Big Five"},
    {'M',   0,  35,  98, "XG #035 Octave 1", "RndGlock"},
    {'M',   0,  35, 104, "XG #035 Octave 1", "Sitar 2"},
    {'M',   0,  36,  16, "XG #036 Octave 2", "DrawOrg2"},
    {'M',   0,  36,  30, "XG #036 Octave 2", "PowerGt2 **"},
    {'M',   0,  36,  61, "XG #036 Octave 2", "Tp&TbSc2 ***"},
    {'M',   0,  36,  81, "XG #036 Octave 2", "Dim Saw ***"},
    {'M',   0,  37,  16, "XG #037 5th 1", "60sDrOr3"},
    {'M',   0,  37,  17, "XG #037 5th 1", "PercOrg2"},
    {'M',   0,  37,  30, "XG #037 5th 1", "PowerGt1 **"},
    {'M',   0,  37,  60, "XG #037 5th 1", "HornOrch"},
    {'M',   0,  38,  16, "XG #038 5th 2", "Even Bar"},
    {'M',   0,  38,  30, "XG #038 5th 2", "Dst.5ths **"},
    {'M',   0,  39,  50, "XG #039 Bend", "+Monarchy"},
    {'M',   0,  39,  51, "XG #039 Bend", "+WormHole"},
    {'M',   0,  39,  52, "XG #039 Bend", "+Gasp"},
    {'M',   0,  39,  61, "XG #039 Bend", "BrssFall **"},
    {'M',   0,  40,   0, "XG #040 Tutti 1", "PianoStr"},
    {'M',   0,  40,   1, "XG #040 Tutti 1", "+SyPadPno"},
    {'M',   0,  40,   2, "XG #040 Tutti 1", "LayerCP1"},
    {'M',   0,  40,   4, "XG #040 Tutti 1", "HardEl.P"},
    {'M',   0,  40,   5, "XG #040 Tutti 1", "DX Phase"},
    {'M',   0,  40,   6, "XG #040 Tutti 1", "+ElHarpsi"},
    {'M',   0,  40,   7, "XG #040 Tutti 1", "+CsmcClav"},
    {'M',   0,  40,  16, "XG #040 Tutti 1", "16+2\"2/3"},
    {'M',   0,  40,  19, "XG #040 Tutti 1", "NotreDam"},
    {'M',   0,  40,  20, "XG #040 Tutti 1", "Puff Org"},
    {'M',   0,  40,  24, "XG #040 Tutti 1", "+Wayside"},
    {'M',   0,  40,  25, "XG #040 Tutti 1", "Nyln&Stl"},
    {'M',   0,  40,  26, "XG #040 Tutti 1", "+OrganGtr"},
    {'M',   0,  40,  28, "XG #040 Tutti 1", "FunkGtr1"},
    {'M',   0,  40,  29, "XG #040 Tutti 1", "+Parallel"},
    {'M',   0,  40,  30, "XG #040 Tutti 1", "FeedbkGt"},
    {'M',   0,  40,  32, "XG #040 Tutti 1", "JazzRthm"},
    {'M',   0,  40,  33, "XG #040 Tutti 1", "Ba&DstEG"},
    {'M',   0,  40,  34, "XG #040 Tutti 1", "+PkB&MtGt"},
    {'M',   0,  40,  38, "XG #040 Tutti 1", "TechnoBa"},
    {'M',   0,  40,  39, "XG #040 Tutti 1", "ModulrBa"},
    {'M',   0,  40,  40, "XG #040 Tutti 1", "+Unison"},
    {'M',   0,  40,  41, "XG #040 Tutti 1", "+VlaDoubl"},
    {'M',   0,  40,  44, "XG #040 Tutti 1", "Susp.Str"},
    {'M',   0,  40,  45, "XG #040 Tutti 1", "+Sleep"},
    {'M',   0,  40,  46, "XG #040 Tutti 1", "YangChin"},
    {'M',   0,  40,  48, "XG #040 Tutti 1", "Orchestr"},
    {'M',   0,  40,  49, "XG #040 Tutti 1", "Warm Str"},
    {'M',   0,  40,  50, "XG #040 Tutti 1", "GrandPad ***"},
    {'M',   0,  40,  52, "XG #040 Tutti 1", "ChoirStr"},
    {'M',   0,  40,  54, "XG #040 Tutti 1", "SyVoice2"},
    {'M',   0,  40,  55, "XG #040 Tutti 1", "+Throne"},
    {'M',   0,  40,  56, "XG #040 Tutti 1", "+Backyard"},
    {'M',   0,  40,  61, "XG #040 Tutti 1", "BrssSec2"},
    {'M',   0,  40,  62, "XG #040 Tutti 1", "SyBrsSub ***"},
    {'M',   0,  40,  63, "XG #040 Tutti 1", "SynBrss4"},
    {'M',   0,  40,  65, "XG #040 Tutti 1", "Sax Sect"},
    {'M',   0,  40,  66, "XG #040 Tutti 1", "BrthTnSx"},
    {'M',   0,  40,  71, "XG #040 Tutti 1", "+Syn&Clr"},
    {'M',   0,  40,  73, "XG #040 Tutti 1", "+Brthy Fl"},
    {'M',   0,  40,  81, "XG #040 Tutti 1", "PulseSaw"},
    {'M',   0,  40,  82, "XG #040 Tutti 1", "+Novice"},
    {'M',   0,  40,  83, "XG #040 Tutti 1", "+SaltLead"},
    {'M',   0,  40,  89, "XG #040 Tutti 1", "+Vishnu"},
    {'M',   0,  40,  94, "XG #040 Tutti 1", "+Tiu"},
    {'M',   0,  40,  98, "XG #040 Tutti 1", "GlockChi"},
    {'M',   0,  40,  99, "XG #040 Tutti 1", "Nylon EP"},
    {'M',   0,  40, 104, "XG #040 Tutti 1", "+Bhuj"},
    {'M',   0,  41,   0, "XG #041 Tutti 2", "Dream"},
    {'M',   0,  41,   2, "XG #041 Tutti 2", "LayerCP2"},
    {'M',   0,  41,   5, "XG #041 Tutti 2", "DX+Analg"},
    {'M',   0,  41,  25, "XG #041 Tutti 2", "Stl&Body"},
    {'M',   0,  41,  26, "XG #041 Tutti 2", "+OctPlate"},
    {'M',   0,  41,  28, "XG #041 Tutti 2", "MuteStlG"},
    {'M',   0,  41,  30, "XG #041 Tutti 2", "FeedbkG2"},
    {'M',   0,  41,  32, "XG #041 Tutti 2", "+PckAcoBa"},
    {'M',   0,  41,  38, "XG #041 Tutti 2", "+Kik'n'Ba"},
    {'M',   0,  41,  39, "XG #041 Tutti 2", "DX Bass"},
    {'M',   0,  41,  48, "XG #041 Tutti 2", "Orchstr2"},
    {'M',   0,  41,  49, "XG #041 Tutti 2", "Kingdom"},
    {'M',   0,  41,  50, "XG #041 Tutti 2", "+SweepStr"},
    {'M',   0,  41,  52, "XG #041 Tutti 2", "+Dead Sea"},
    {'M',   0,  41,  54, "XG #041 Tutti 2", "Choral"},
    {'M',   0,  41,  61, "XG #041 Tutti 2", "Hi Brass"},
    {'M',   0,  41,  63, "XG #041 Tutti 2", "ChoirBrs"},
    {'M',   0,  41,  66, "XG #041 Tutti 2", "SoftTenr"},
    {'M',   0,  41,  81, "XG #041 Tutti 2", "Dr.Lead"},
    {'M',   0,  41,  98, "XG #041 Tutti 2", "ClearBel"},
    {'M',   0,  42,   5, "XG #042 Tutti 3", "DXKotoEP"},
    {'M',   0,  42,  30, "XG #042 Tutti 3", "+TwinDist"},
    {'M',   0,  42,  38, "XG #042 Tutti 3", "+NEP"},
    {'M',   0,  42,  39, "XG #042 Tutti 3", "+DXBa Brt"},
    {'M',   0,  42,  48, "XG #042 Tutti 3", "TremOrch"},
    {'M',   0,  42,  50, "XG #042 Tutti 3", "+SwpStOct"},
    {'M',   0,  42,  61, "XG #042 Tutti 3", "MelloBrs"},
    {'M',   0,  42,  63, "XG #042 Tutti 3", "+AnHrnRch"},
    {'M',   0,  42,  98, "XG #042 Tutti 3", "ChorBell"},
    {'M',   0,  43,  24, "XG #043 Velocity Switch", "VelGtHrm"},
    {'M',   0,  43,  28, "XG #043 Velocity Switch", "FunkGtr2"},
    {'M',   0,  43,  29, "XG #043 Velocity Switch", "Gt.Pinch"},
    {'M',   0,  43,  30, "XG #043 Velocity Switch", "RckRthm2 **"},
    {'M',   0,  43,  32, "XG #043 Velocity Switch", "+Blink Ba"},
    {'M',   0,  43,  33, "XG #043 Velocity Switch", "FngrSlap"},
    {'M',   0,  43,  37, "XG #043 Velocity Switch", "VeloSlap"},
    {'M',   0,  43,  47, "XG #043 Velocity Switch", "+Roll&Hit"},
    {'M',   0,  43,  65, "XG #043 Velocity Switch", "HyprAlto"},
    {'M',   0,  45,   4, "XG #045 Velocity Cross Fade", "VX El.P1"},
    {'M',   0,  45,   5, "XG #045 Velocity Cross Fade", "VX El.P2"},
    {'M',   0,  45,  11, "XG #045 Velocity Cross Fade", "HardVibe"},
    {'M',   0,  45,  28, "XG #045 Velocity Cross Fade", "Jazz Man"},
    {'M',   0,  45,  30, "XG #045 Velocity Cross Fade", "RckRthm1 **"},
    {'M',   0,  45,  32, "XG #045 Velocity Cross Fade", "VXUprght"},
    {'M',   0,  45,  33, "XG #045 Velocity Cross Fade", "FngBass2"},
    {'M',   0,  45,  48, "XG #045 Velocity Cross Fade", "Velo.Str"},
    {'M',   0,  45,  62, "XG #045 Velocity Cross Fade", "AnVelBr1"},
    {'M',   0,  45,  63, "XG #045 Velocity Cross Fade", "AnVelBr2"},
    {'M',   0,  45,  81, "XG #045 Velocity Cross Fade", "VeloLead"},
    {'M',   0,  45,  96, "XG #045 Velocity Cross Fade", "ClaviPad"},
    {'M',   0,  48,   5, "XG #048 Detune 4", "+ChoEP K"},
    {'M',   0,  52,   5, "XG #052 Tutti 4", "+DXMallet"},
    {'M',   0,  52,  48, "XG #052 Tutti 4", "+Lento"},
    {'M',   0,  52,  61, "XG #052 Tutti 4", "+Bund"},
    {'M',   0,  53,  61, "XG #053 Tutti 5", "+FakeHorn"},
    {'M',   0,  54,  61, "XG #054 Tutti 6", "+FkHrnOct"},
    {'M',   0,  64,   0, "XG #064 Other Waves 1", "+ConGrnd"},
    {'M',   0,  64,   1, "XG #064 Other Waves 1", "+BrConGrd"},
    {'M',   0,  64,   4, "XG #064 Other Waves 1", "60sEl.P1"},
    {'M',   0,  64,   5, "XG #064 Other Waves 1", "+Shrakawa"},
    {'M',   0,  64,   6, "XG #064 Other Waves 1", "+SynHrpsi"},
    {'M',   0,  64,   7, "XG #064 Other Waves 1", "PulseClv"},
    {'M',   0,  64,   8, "XG #064 Other Waves 1", "+FMCelesta"},
    {'M',   0,  64,  10, "XG #064 Other Waves 1", "Orgel"},
    {'M',   0,  64,  12, "XG #064 Other Waves 1", "SineMrmb"},
    {'M',   0,  64,  16, "XG #064 Other Waves 1", "Organ Ba"},
    {'M',   0,  64,  17, "XG #064 Other Waves 1", "JazOrgan ***"},
    {'M',   0,  64,  18, "XG #064 Other Waves 1", "RotaryOr"},
    {'M',   0,  64,  19, "XG #064 Other Waves 1", "OrgFlute"},
    {'M',   0,  64,  20, "XG #064 Other Waves 1", "+SyReedDk"},
    {'M',   0,  64,  23, "XG #064 Other Waves 1", "TngoAcd2"},
    {'M',   0,  64,  24, "XG #064 Other Waves 1", "+EsGuitar"},
    {'M',   0,  64,  25, "XG #064 Other Waves 1", "+Nashvlle"},
    {'M',   0,  64,  26, "XG #064 Other Waves 1", "+SuperJzM"},
    {'M',   0,  64,  27, "XG #064 Other Waves 1", "CleanGt2 **"},
    {'M',   0,  64,  28, "XG #064 Other Waves 1", "+Wrench"},
    {'M',   0,  64,  29, "XG #064 Other Waves 1", "+ManhttnM"},
    {'M',   0,  64,  30, "XG #064 Other Waves 1", "+Bite"},
    {'M',   0,  64,  31, "XG #064 Other Waves 1", "AcoHarmo **"},
    {'M',   0,  64,  32, "XG #064 Other Waves 1", "+Boston"},
    {'M',   0,  64,  33, "XG #064 Other Waves 1", "Jazzy Ba **"},
    {'M',   0,  64,  34, "XG #064 Other Waves 1", "+HardPick"},
    {'M',   0,  64,  35, "XG #064 Other Waves 1", "+PwrFrtls"},
    {'M',   0,  64,  36, "XG #064 Other Waves 1", "Slapper ***"},
    {'M',   0,  64,  38, "XG #064 Other Waves 1", "Orbiter"},
    {'M',   0,  64,  39, "XG #064 Other Waves 1", "X WireBa"},
    {'M',   0,  64,  40, "XG #064 Other Waves 1", "+Cadenza"},
    {'M',   0,  64,  41, "XG #064 Other Waves 1", "+Sonata"},
    {'M',   0,  64,  42, "XG #064 Other Waves 1", "++CelloSec"},
    {'M',   0,  64,  43, "XG #064 Other Waves 1", "++CB Sec"},
    {'M',   0,  64,  44, "XG #064 Other Waves 1", "+Fear"},
    {'M',   0,  64,  45, "XG #064 Other Waves 1", "++Collegno"},
    {'M',   0,  64,  46, "XG #064 Other Waves 1", "El.Harp"},
    {'M',   0,  64,  48, "XG #064 Other Waves 1", "+SprStrng"},
    {'M',   0,  64,  49, "XG #064 Other Waves 1", "70s Str"},
    {'M',   0,  64,  50, "XG #064 Other Waves 1", "Syn Str4"},
    {'M',   0,  64,  51, "XG #064 Other Waves 1", "+Hope"},
    {'M',   0,  64,  52, "XG #064 Other Waves 1", "StrngAah **"},
    {'M',   0,  64,  53, "XG #064 Other Waves 1", "VoiceDoo **"},
    {'M',   0,  64,  54, "XG #064 Other Waves 1", "AnaVoice"},
    {'M',   0,  64,  55, "XG #064 Other Waves 1", "Impact"},
    {'M',   0,  64,  56, "XG #064 Other Waves 1", "Dark Trp ***"},
    {'M',   0,  64,  57, "XG #064 Other Waves 1", "BrghtTrb ***"},
    {'M',   0,  64,  58, "XG #064 Other Waves 1", "+++Hrd Tuba"},
    {'M',   0,  64,  59, "XG #064 Other Waves 1", "MuteTrp2 **"},
    {'M',   0,  64,  60, "XG #064 Other Waves 1", "+Syn Horn"},
    {'M',   0,  64,  61, "XG #064 Other Waves 1", "+SprBrass"},
    {'M',   0,  64,  62, "XG #064 Other Waves 1", "AnaBrss1"},
    {'M',   0,  64,  63, "XG #064 Other Waves 1", "AnaBrss2"},
    {'M',   0,  64,  64, "XG #064 Other Waves 1", "+Mdtation"},
    {'M',   0,  64,  65, "XG #064 Other Waves 1", "+ASaxPwrd"},
    {'M',   0,  64,  66, "XG #064 Other Waves 1", "TnrSax 2"},
    {'M',   0,  64,  68, "XG #064 Other Waves 1", "+Heinz"},
    {'M',   0,  64,  73, "XG #064 Other Waves 1", "+Boehm"},
    {'M',   0,  64,  74, "XG #064 Other Waves 1", "+Piplith"},
    {'M',   0,  64,  75, "XG #064 Other Waves 1", "PanFlut2 **"},
    {'M',   0,  64,  76, "XG #064 Other Waves 1", "+BottlLgt"},
    {'M',   0,  64,  78, "XG #064 Other Waves 1", "+Reverie"},
    {'M',   0,  64,  79, "XG #064 Other Waves 1", "+Opalina"},
    {'M',   0,  64,  80, "XG #064 Other Waves 1", "Mellow"},
    {'M',   0,  64,  81, "XG #064 Other Waves 1", "Digger ***"},
    {'M',   0,  64,  82, "XG #064 Other Waves 1", "Vent Syn **"},
    {'M',   0,  64,  83, "XG #064 Other Waves 1", "Rubby"},
    {'M',   0,  64,  84, "XG #064 Other Waves 1", "DistLead"},
    {'M',   0,  64,  85, "XG #064 Other Waves 1", "Vox Lead"},
    {'M',   0,  64,  87, "XG #064 Other Waves 1", "Fat&Prky"},
    {'M',   0,  64,  88, "XG #064 Other Waves 1", "Fantasy"},
    {'M',   0,  64,  89, "XG #064 Other Waves 1", "Horn Pad"},
    {'M',   0,  64,  90, "XG #064 Other Waves 1", "PolyPd80"},
    {'M',   0,  64,  91, "XG #064 Other Waves 1", "Heaven"},
    {'M',   0,  64,  92, "XG #064 Other Waves 1", "Glacier"},
    {'M',   0,  64,  93, "XG #064 Other Waves 1", "Tine Pad"},
    {'M',   0,  64,  94, "XG #064 Other Waves 1", "+Aries"},
    {'M',   0,  64,  95, "XG #064 Other Waves 1", "PolarPad"},
    {'M',   0,  64,  96, "XG #064 Other Waves 1", "HrmoRain"},
    {'M',   0,  64,  97, "XG #064 Other Waves 1", "Ancestrl"},
    {'M',   0,  64,  98, "XG #064 Other Waves 1", "SynMalet"},
    {'M',   0,  64,  99, "XG #064 Other Waves 1", "NylnHarp"},
    {'M',   0,  64, 100, "XG #064 Other Waves 1", "FantaBel"},
    {'M',   0,  64, 101, "XG #064 Other Waves 1", "GobSynth"},
    {'M',   0,  64, 102, "XG #064 Other Waves 1", "EchoBell"},
    {'M',   0,  64, 103, "XG #064 Other Waves 1", "Starz"},
    {'M',   0,  64, 104, "XG #064 Other Waves 1", "+Raga Syn"},
    {'M',   0,  64, 105, "XG #064 Other Waves 1", "+El Banjo"},
    {'M',   0,  64, 107, "XG #064 Other Waves 1", "+FM Koto"},
    {'M',   0,  64, 108, "XG #064 Other Waves 1", "BigKalim **"},
    {'M',   0,  64, 109, "XG #064 Other Waves 1", "+Thistle"},
    {'M',   0,  64, 111, "XG #064 Other Waves 1", "Shanai 2"},
    {'M',   0,  64, 112, "XG #064 Other Waves 1", "+TcklBell"},
    {'M',   0,  64, 117, "XG #064 Other Waves 1", "Mel Tom2"},
    {'M',   0,  64, 118, "XG #064 Other Waves 1", "Ana Tom"},
    {'M',   0,  64, 119, "XG #064 Other Waves 1", "Rev Cym2**"},
    {'M',   0,  65,   0, "XG #065 Other Waves 2", "+ConGrndK"},
    {'M',   0,  65,   1, "XG #065 Other Waves 2", "+BrConGrK"},
    {'M',   0,  65,   4, "XG #065 Other Waves 2", "+Old EP"},
    {'M',   0,  65,   5, "XG #065 Other Waves 2", "+OldEP Tn"},
    {'M',   0,  65,   7, "XG #065 Other Waves 2", "PierceCl"},
    {'M',   0,  65,  10, "XG #065 Other Waves 2", "+SmalOrgl"},
    {'M',   0,  65,  16, "XG #065 Other Waves 2", "70sDrOr2"},
    {'M',   0,  65,  17, "XG #065 Other Waves 2", "WarmJzOr ***"},
    {'M',   0,  65,  18, "XG #065 Other Waves 2", "SloRotar"},
    {'M',   0,  65,  19, "XG #065 Other Waves 2", "TrmOrgFl"},
    {'M',   0,  65,  23, "XG #065 Other Waves 2", "+TightAcd"},
    {'M',   0,  65,  24, "XG #065 Other Waves 2", "+EsGtrHrd"},
    {'M',   0,  65,  25, "XG #065 Other Waves 2", "+NashvllR"},
    {'M',   0,  65,  26, "XG #065 Other Waves 2", "+SuperJzB"},
    {'M',   0,  65,  27, "XG #065 Other Waves 2", "MidT.Gtr ***"},
    {'M',   0,  65,  28, "XG #065 Other Waves 2", "+WrenchHv"},
    {'M',   0,  65,  29, "XG #065 Other Waves 2", "+ManhttnB"},
    {'M',   0,  65,  30, "XG #065 Other Waves 2", "+Bite Res"},
    {'M',   0,  65,  31, "XG #065 Other Waves 2", "GtFeedbk"},
    {'M',   0,  65,  32, "XG #065 Other Waves 2", "+BostnBrt"},
    {'M',   0,  65,  33, "XG #065 Other Waves 2", "Mod.Bass"},
    {'M',   0,  65,  34, "XG #065 Other Waves 2", "+HrdPikRs"},
    {'M',   0,  65,  35, "XG #065 Other Waves 2", "+PwrFrtlR"},
    {'M',   0,  65,  36, "XG #065 Other Waves 2", "Thum&Slp ***"},
    {'M',   0,  65,  38, "XG #065 Other Waves 2", "Sqr.Bass"},
    {'M',   0,  65,  39, "XG #065 Other Waves 2", "AtkPulse ***"},
    {'M',   0,  65,  40, "XG #065 Other Waves 2", "+CadenzDk"},
    {'M',   0,  65,  41, "XG #065 Other Waves 2", "++Vla Sec"},
    {'M',   0,  65,  42, "XG #065 Other Waves 2", "++Hrd Vcs"},
    {'M',   0,  65,  43, "XG #065 Other Waves 2", "++Hrd CBs"},
    {'M',   0,  65,  44, "XG #065 Other Waves 2", "+Fear Dtd"},
    {'M',   0,  65,  48, "XG #065 Other Waves 2", "+SprStrSt"},
    {'M',   0,  65,  49, "XG #065 Other Waves 2", "Strings3"},
    {'M',   0,  65,  50, "XG #065 Other Waves 2", "Syn Str5"},
    {'M',   0,  65,  51, "XG #065 Other Waves 2", "+Virgo"},
    {'M',   0,  65,  52, "XG #065 Other Waves 2", "Male Aah **"},
    {'M',   0,  65,  53, "XG #065 Other Waves 2", "+Hmn"},
    {'M',   0,  65,  54, "XG #065 Other Waves 2", "+Aspirate"},
    {'M',   0,  65,  55, "XG #065 Other Waves 2", "BrssStab **"},
    {'M',   0,  65,  56, "XG #065 Other Waves 2", "DrkTpSft ***"},
    {'M',   0,  65,  57, "XG #065 Other Waves 2", "MellowTb ***"},
    {'M',   0,  65,  58, "XG #065 Other Waves 2", "+++Slw Tuba +++"},
    {'M',   0,  65,  59, "XG #065 Other Waves 2", "+Bkstairs"},
    {'M',   0,  65,  60, "XG #065 Other Waves 2", "HornOrc2"},
    {'M',   0,  65,  61, "XG #065 Other Waves 2", "+SprBrCut"},
    {'M',   0,  65,  62, "XG #065 Other Waves 2", "+SynthThn"},
    {'M',   0,  65,  63, "XG #065 Other Waves 2", "+Soft Cut"},
    {'M',   0,  65,  64, "XG #065 Other Waves 2", "+MdtatnRs"},
    {'M',   0,  65,  65, "XG #065 Other Waves 2", "+FakeAlto"},
    {'M',   0,  65,  66, "XG #065 Other Waves 2", "+SprTenor"},
    {'M',   0,  65,  68, "XG #065 Other Waves 2", "+HeinzUni"},
    {'M',   0,  65,  73, "XG #065 Other Waves 2", "+Boehm Br"},
    {'M',   0,  65,  74, "XG #065 Other Waves 2", "+Home"},
    {'M',   0,  65,  75, "XG #065 Other Waves 2", "+Meadow"},
    {'M',   0,  65,  80, "XG #065 Other Waves 2", "SoloSine"},
    {'M',   0,  65,  81, "XG #065 Other Waves 2", "+Dunce"},
    {'M',   0,  65,  82, "XG #065 Other Waves 2", "PureLead"},
    {'M',   0,  65,  83, "XG #065 Other Waves 2", "HardSync ***"},
    {'M',   0,  65,  84, "XG #065 Other Waves 2", "WireLead"},
    {'M',   0,  65,  85, "XG #065 Other Waves 2", "Br.Layer ***"},
    {'M',   0,  65,  87, "XG #065 Other Waves 2", "Soft Wrl"},
    {'M',   0,  65,  88, "XG #065 Other Waves 2", "+Libra"},
    {'M',   0,  65,  89, "XG #065 Other Waves 2", "RotarStr"},
    {'M',   0,  65,  90, "XG #065 Other Waves 2", "ClickPad"},
    {'M',   0,  65,  91, "XG #065 Other Waves 2", "Lite Pad **"},
    {'M',   0,  65,  92, "XG #065 Other Waves 2", "GlassPad"},
    {'M',   0,  65,  93, "XG #065 Other Waves 2", "Pan Pad"},
    {'M',   0,  65,  94, "XG #065 Other Waves 2", "ChorusPd"},
    {'M',   0,  65,  95, "XG #065 Other Waves 2", "Sweepy **"},
    {'M',   0,  65,  96, "XG #065 Other Waves 2", "AfrcnWnd"},
    {'M',   0,  65,  97, "XG #065 Other Waves 2", "Rave **"},
    {'M',   0,  65,  98, "XG #065 Other Waves 2", "SftCryst"},
    {'M',   0,  65,  99, "XG #065 Other Waves 2", "Harp Vox"},
    {'M',   0,  65, 100, "XG #065 Other Waves 2", "ShinStar"},
    {'M',   0,  65, 101, "XG #065 Other Waves 2", "Creeper"},
    {'M',   0,  65, 102, "XG #065 Other Waves 2", "Big Pan"},
    {'M',   0,  65, 103, "XG #065 Other Waves 2", "Odin **"},
    {'M',   0,  65, 117, "XG #065 Other Waves 2", "Real Tom"},
    {'M',   0,  65, 118, "XG #065 Other Waves 2", "ElecPerc"},
    {'M',   0,  65, 119, "XG #065 Other Waves 2", "Rev Cym3"},
    {'M',   0,  66,   0, "XG #066 Other Waves 3", "+DblConGr"},
    {'M',   0,  66,   1, "XG #066 Other Waves 3", "+MIDIGrd3"},
    {'M',   0,  66,   4, "XG #066 Other Waves 3", "+Tribecca"},
    {'M',   0,  66,   5, "XG #066 Other Waves 3", "+Flips"},
    {'M',   0,  66,   7, "XG #066 Other Waves 3", "+ClrClavi"},
    {'M',   0,  66,  16, "XG #066 Other Waves 3", "CheezOrg"},
    {'M',   0,  66,  17, "XG #066 Other Waves 3", "ClikOrgn ***"},
    {'M',   0,  66,  18, "XG #066 Other Waves 3", "FstRotar"},
    {'M',   0,  66,  23, "XG #066 Other Waves 3", "+TghtAcdD"},
    {'M',   0,  66,  24, "XG #066 Other Waves 3", "+EsGtMllo"},
    {'M',   0,  66,  25, "XG #066 Other Waves 3", "+Nashvl12"},
    {'M',   0,  66,  26, "XG #066 Other Waves 3", "+SuperJzD"},
    {'M',   0,  66,  27, "XG #066 Other Waves 3", "MidTGtSt ***"},
    {'M',   0,  66,  28, "XG #066 Other Waves 3", "+WrnchDbl"},
    {'M',   0,  66,  29, "XG #066 Other Waves 3", "+ManhttnD"},
    {'M',   0,  66,  30, "XG #066 Other Waves 3", "+BiteDtd"},
    {'M',   0,  66,  31, "XG #066 Other Waves 3", "GtrHrmo2"},
    {'M',   0,  66,  32, "XG #066 Other Waves 3", "+Coolth"},
    {'M',   0,  66,  33, "XG #066 Other Waves 3", "+Chase"},
    {'M',   0,  66,  34, "XG #066 Other Waves 3", "+PkBass+"},
    {'M',   0,  66,  35, "XG #066 Other Waves 3", "+TalkinBa"},
    {'M',   0,  66,  36, "XG #066 Other Waves 3", "+GltzySlp"},
    {'M',   0,  66,  38, "XG #066 Other Waves 3", "RubberBa"},
    {'M',   0,  66,  39, "XG #066 Other Waves 3", "CS Light ***"},
    {'M',   0,  66,  40, "XG #066 Other Waves 3", "++Vln Sec"},
    {'M',   0,  66,  41, "XG #066 Other Waves 3", "++Hrd Vlas"},
    {'M',   0,  66,  42, "XG #066 Other Waves 3", "++Slw Vcs"},
    {'M',   0,  66,  43, "XG #066 Other Waves 3", "++Slw CBs"},
    {'M',   0,  66,  44, "XG #066 Other Waves 3", "+Apoclyps"},
    {'M',   0,  66,  48, "XG #066 Other Waves 3", "+Triste"},
    {'M',   0,  66,  50, "XG #066 Other Waves 3", "+Solitude"},
    {'M',   0,  66,  51, "XG #066 Other Waves 3", "+Platinum"},
    {'M',   0,  66,  52, "XG #066 Other Waves 3", "+Scroll"},
    {'M',   0,  66,  53, "XG #066 Other Waves 3", "+WrlChoir"},
    {'M',   0,  66,  54, "XG #066 Other Waves 3", "+AsprateD"},
    {'M',   0,  66,  55, "XG #066 Other Waves 3", "DoublHit **"},
    {'M',   0,  66,  56, "XG #066 Other Waves 3", "+Soft Trp"},
    {'M',   0,  66,  57, "XG #066 Other Waves 3", "+JJJ"},
    {'M',   0,  66,  58, "XG #066 Other Waves 3", "+++Euphon"},
    {'M',   0,  66,  60, "XG #066 Other Waves 3", "+++Brt Horn"},
    {'M',   0,  66,  61, "XG #066 Other Waves 3", "+SprBrBlw"},
    {'M',   0,  66,  62, "XG #066 Other Waves 3", "+SyncBrss"},
    {'M',   0,  66,  63, "XG #066 Other Waves 3", "+AnaHornS"},
    {'M',   0,  66,  65, "XG #066 Other Waves 3", "+FakeAlt+"},
    {'M',   0,  66,  66, "XG #066 Other Waves 3", "+SprTnr +"},
    {'M',   0,  66,  68, "XG #066 Other Waves 3", "Oboe Exp"},
    {'M',   0,  66,  73, "XG #066 Other Waves 3", "+Pastoral"},
    {'M',   0,  66,  80, "XG #066 Other Waves 3", "SineLead"},
    {'M',   0,  66,  81, "XG #066 Other Waves 3", "+BrassSaw"},
    {'M',   0,  66,  82, "XG #066 Other Waves 3", "+ElPrimtv"},
    {'M',   0,  66,  84, "XG #066 Other Waves 3", "SynPluck ***"},
    {'M',   0,  66,  85, "XG #066 Other Waves 3", "+Cypher 1"},
    {'M',   0,  66,  87, "XG #066 Other Waves 3", "+Cant"},
    {'M',   0,  66,  88, "XG #066 Other Waves 3", "Bell Pad"},
    {'M',   0,  66,  89, "XG #066 Other Waves 3", "+++LightPad"},
    {'M',   0,  66,  90, "XG #066 Other Waves 3", "Ana. Pad"},
    {'M',   0,  66,  91, "XG #066 Other Waves 3", "Itopia"},
    {'M',   0,  66,  92, "XG #066 Other Waves 3", "+SqrTwang"},
    {'M',   0,  66,  93, "XG #066 Other Waves 3", "+Queever"},
    {'M',   0,  66,  95, "XG #066 Other Waves 3", "Celstial"},
    {'M',   0,  66,  96, "XG #066 Other Waves 3", "Carib"},
    {'M',   0,  66,  97, "XG #066 Other Waves 3", "Fairy ***"},
    {'M',   0,  66,  98, "XG #066 Other Waves 3", "LoudGlok"},
    {'M',   0,  66,  99, "XG #066 Other Waves 3", "AtmosPad"},
    {'M',   0,  66, 100, "XG #066 Other Waves 3", "BritStab"},
    {'M',   0,  66, 101, "XG #066 Other Waves 3", "Ring Pad"},
    {'M',   0,  66, 102, "XG #066 Other Waves 3", "SynPiano"},
    {'M',   0,  66, 117, "XG #066 Other Waves 3", "Rock Tom"},
    {'M',   0,  66, 118, "XG #066 Other Waves 3", "Syn Perc"},
    {'M',   0,  67,   0, "XG #067 Other Waves 4", "+MIDIGrd1"},
    {'M',   0,  67,   1, "XG #067 Other Waves 4", "+MIDIGrd4"},
    {'M',   0,  67,   4, "XG #067 Other Waves 4", "+Diploid1"},
    {'M',   0,  67,   5, "XG #067 Other Waves 4", "+FlipsDtd"},
    {'M',   0,  67,   7, "XG #067 Other Waves 4", "+SwpClavi"},
    {'M',   0,  67,  16, "XG #067 Other Waves 4", "DrawOrg3"},
    {'M',   0,  67,  17, "XG #067 Other Waves 4", "+Grace"},
    {'M',   0,  67,  18, "XG #067 Other Waves 4", "+GlaclRtr"},
    {'M',   0,  67,  24, "XG #067 Other Waves 4", "+EsGtrDcy"},
    {'M',   0,  67,  25, "XG #067 Other Waves 4", "+++OldSmple"},
    {'M',   0,  67,  26, "XG #067 Other Waves 4", "+SuperJzR"},
    {'M',   0,  67,  27, "XG #067 Other Waves 4", "NasalGtr ***"},
    {'M',   0,  67,  28, "XG #067 Other Waves 4", "+Tin"},
    {'M',   0,  67,  29, "XG #067 Other Waves 4", "+ManhttnP"},
    {'M',   0,  67,  30, "XG #067 Other Waves 4", "+Bite +"},
    {'M',   0,  67,  31, "XG #067 Other Waves 4", "+Shimla"},
    {'M',   0,  67,  32, "XG #067 Other Waves 4", "+Coolth B"},
    {'M',   0,  67,  33, "XG #067 Other Waves 4", "+ChaseRes"},
    {'M',   0,  67,  34, "XG #067 Other Waves 4", "PickBa 4"},
    {'M',   0,  67,  35, "XG #067 Other Waves 4", "+NoizFrtl"},
    {'M',   0,  67,  36, "XG #067 Other Waves 4", "+FM Slap"},
    {'M',   0,  67,  38, "XG #067 Other Waves 4", "Fish ***"},
    {'M',   0,  67,  39, "XG #067 Other Waves 4", "MetlBass ***"},
    {'M',   0,  67,  40, "XG #067 Other Waves 4", "++Hrd Vlns"},
    {'M',   0,  67,  41, "XG #067 Other Waves 4", "++Slw Vlas"},
    {'M',   0,  67,  44, "XG #067 Other Waves 4", "+++Brt Trem"},
    {'M',   0,  67,  48, "XG #067 Other Waves 4", "+Basso"},
    {'M',   0,  67,  50, "XG #067 Other Waves 4", "+Fate"},
    {'M',   0,  67,  51, "XG #067 Other Waves 4", "+OctavPWM"},
    {'M',   0,  67,  52, "XG #067 Other Waves 4", "+Scroll +"},
    {'M',   0,  67,  53, "XG #067 Other Waves 4", "+++Ooh St"},
    {'M',   0,  67,  54, "XG #067 Other Waves 4", "+Facula"},
    {'M',   0,  67,  55, "XG #067 Other Waves 4", "BrStab80 **"},
    {'M',   0,  67,  56, "XG #067 Other Waves 4", "+Blow"},
    {'M',   0,  67,  57, "XG #067 Other Waves 4", "+++BrllntTb"},
    {'M',   0,  67,  60, "XG #067 Other Waves 4", "+++Hrd Horn"},
    {'M',   0,  67,  61, "XG #067 Other Waves 4", "+Pwrd Sfz"},
    {'M',   0,  67,  62, "XG #067 Other Waves 4", "+SyncBrSt"},
    {'M',   0,  67,  65, "XG #067 Other Waves 4", "+FakeAltD"},
    {'M',   0,  67,  66, "XG #067 Other Waves 4", "+SprTnrSt"},
    {'M',   0,  67,  73, "XG #067 Other Waves 4", "+Shepherd"},
    {'M',   0,  67,  80, "XG #067 Other Waves 4", "Pulse Ld ***"},
    {'M',   0,  67,  81, "XG #067 Other Waves 4", "+SawRiver"},
    {'M',   0,  67,  84, "XG #067 Other Waves 4", "+++The Sync"},
    {'M',   0,  67,  85, "XG #067 Other Waves 4", "+Cypher 2"},
    {'M',   0,  67,  87, "XG #067 Other Waves 4", "+Mogul"},
    {'M',   0,  67,  90, "XG #067 Other Waves 4", "SquarPad"},
    {'M',   0,  67,  91, "XG #067 Other Waves 4", "CC Pad"},
    {'M',   0,  67,  92, "XG #067 Other Waves 4", "SquarPd8"},
    {'M',   0,  67,  95, "XG #067 Other Waves 4", "+Monsoon"},
    {'M',   0,  67,  97, "XG #067 Other Waves 4", "+Hermit"},
    {'M',   0,  67,  98, "XG #067 Other Waves 4", "ChrstBel"},
    {'M',   0,  67,  99, "XG #067 Other Waves 4", "Planet"},
    {'M',   0,  67, 101, "XG #067 Other Waves 4", "Ritual"},
    {'M',   0,  67, 102, "XG #067 Other Waves 4", "Creation"},
    {'M',   0,  67, 117, "XG #067 Other Waves 4", "+++Tim'sSet"},
    {'M',   0,  68,   0, "XG #068 Other Waves 5", "+MIDIGrd2"},
    {'M',   0,  68,   1, "XG #068 Other Waves 5", "+OldPiano"},
    {'M',   0,  68,   4, "XG #068 Other Waves 5", "+Flops"},
    {'M',   0,  68,   5, "XG #068 Other Waves 5", "+Flicks"},
    {'M',   0,  68,   7, "XG #068 Other Waves 5", "+SynClavi"},
    {'M',   0,  68,  16, "XG #068 Other Waves 5", "StdiumOr ***"},
    {'M',   0,  68,  17, "XG #068 Other Waves 5", "+CrnGrace"},
    {'M',   0,  68,  26, "XG #068 Other Waves 5", "+Dx JzGtr"},
    {'M',   0,  68,  27, "XG #068 Other Waves 5", "NaslGtSt ***"},
    {'M',   0,  68,  28, "XG #068 Other Waves 5", "+++GrooveMt"},
    {'M',   0,  68,  30, "XG #068 Other Waves 5", "+Burnout"},
    {'M',   0,  68,  33, "XG #068 Other Waves 5", "+BlueBass"},
    {'M',   0,  68,  36, "XG #068 Other Waves 5", "+FMSlpDtd"},
    {'M',   0,  68,  38, "XG #068 Other Waves 5", "HardReso ***"},
    {'M',   0,  68,  39, "XG #068 Other Waves 5", "+FrcOscBa"},
    {'M',   0,  68,  40, "XG #068 Other Waves 5", "++Slw Vlns"},
    {'M',   0,  68,  48, "XG #068 Other Waves 5", "++Stacc H"},
    {'M',   0,  68,  50, "XG #068 Other Waves 5", "+Thulium"},
    {'M',   0,  68,  51, "XG #068 Other Waves 5", "+Taurus"},
    {'M',   0,  68,  52, "XG #068 Other Waves 5", "+++Aah St"},
    {'M',   0,  68,  55, "XG #068 Other Waves 5", "Bass Hit ***"},
    {'M',   0,  68,  56, "XG #068 Other Waves 5", "+Blow Dbl"},
    {'M',   0,  68,  57, "XG #068 Other Waves 5", "+++Hrd Tb"},
    {'M',   0,  68,  61, "XG #068 Other Waves 5", "+PwrSfzBr"},
    {'M',   0,  68,  62, "XG #068 Other Waves 5", "+AnaHorn1"},
    {'M',   0,  68,  66, "XG #068 Other Waves 5", "+Tnr&Alto"},
    {'M',   0,  68,  80, "XG #068 Other Waves 5", "SyncLead ***"},
    {'M',   0,  68,  81, "XG #068 Other Waves 5", "+BrPulsDb"},
    {'M',   0,  68,  85, "XG #068 Other Waves 5", "+Cypher 3"},
    {'M',   0,  68,  87, "XG #068 Other Waves 5", "+Distance"},
    {'M',   0,  68,  90, "XG #068 Other Waves 5", "Show Pad ***"},
    {'M',   0,  68,  91, "XG #068 Other Waves 5", "CosmicPd ***"},
    {'M',   0,  68,  95, "XG #068 Other Waves 5", "+lo"},
    {'M',   0,  68,  98, "XG #068 Other Waves 5", "VibeBell"},
    {'M',   0,  68,  99, "XG #068 Other Waves 5", "+Lyra"},
    {'M',   0,  68, 101, "XG #068 Other Waves 5", "ToHeaven"},
    {'M',   0,  68, 102, "XG #068 Other Waves 5", "StarDust"},
    {'M',   0,  69,   0, "XG #069 Other Waves 6", "+++OldestAP"},
    {'M',   0,  69,   4, "XG #069 Other Waves 6", "+Soho"},
    {'M',   0,  69,   5, "XG #069 Other Waves 6", "+FliksDtd"},
    {'M',   0,  69,   7, "XG #069 Other Waves 6", "+SprClavi"},
    {'M',   0,  69,  16, "XG #069 Other Waves 6", "StdiumO2 ***"},
    {'M',   0,  69,  17, "XG #069 Other Waves 6", "+DimClick"},
    {'M',   0,  69,  26, "XG #069 Other Waves 6", "+DXJzGt.D"},
    {'M',   0,  69,  27, "XG #069 Other Waves 6", "+Hammer M"},
    {'M',   0,  69,  30, "XG #069 Other Waves 6", "+Bombay"},
    {'M',   0,  69,  33, "XG #069 Other Waves 6", "JazzyBa2"},
    {'M',   0,  69,  38, "XG #069 Other Waves 6", "+Wah Saw"},
    {'M',   0,  69,  39, "XG #069 Other Waves 6", "+Cubit"},
    {'M',   0,  69,  48, "XG #069 Other Waves 6", "++Stacc L"},
    {'M',   0,  69,  50, "XG #069 Other Waves 6", "+Brook"},
    {'M',   0,  69,  51, "XG #069 Other Waves 6", "+Frost"},
    {'M',   0,  69,  52, "XG #069 Other Waves 6", "+++Aah Mix"},
    {'M',   0,  69,  55, "XG #069 Other Waves 6", "BassHit+ ***"},
    {'M',   0,  69,  56, "XG #069 Other Waves 6", "4th TP"},
    {'M',   0,  69,  57, "XG #069 Other Waves 6", "+++BrtBTb"},
    {'M',   0,  69,  61, "XG #069 Other Waves 6", "+Alto&Trp"},
    {'M',   0,  69,  62, "XG #069 Other Waves 6", "+AnaHorn2"},
    {'M',   0,  69,  80, "XG #069 Other Waves 6", "+ForcdOsc"},
    {'M',   0,  69,  81, "XG #069 Other Waves 6", "+Saw Trp"},
    {'M',   0,  69,  85, "XG #069 Other Waves 6", "+SprCyphr"},
    {'M',   0,  69,  87, "XG #069 Other Waves 6", "+Sync B&L"},
    {'M',   0,  69,  90, "XG #069 Other Waves 6", "+Pixie"},
    {'M',   0,  69,  91, "XG #069 Other Waves 6", "+++Aah Pad"},
    {'M',   0,  69,  98, "XG #069 Other Waves 6", "DigiBell"},
    {'M',   0,  69,  99, "XG #069 Other Waves 6", "+Akasaka"},
    {'M',   0,  69, 101, "XG #069 Other Waves 6", "MilkyWay **"},
    {'M',   0,  69, 102, "XG #069 Other Waves 6", "Reso&Pan"},
    {'M',   0,  70,   4, "XG #070 Other Waves 7", "+FlopsDtd"},
    {'M',   0,  70,   5, "XG #070 Other Waves 7", "+BrightDX"},
    {'M',   0,  70,   7, "XG #070 Other Waves 7", "+GtrClavi"},
    {'M',   0,  70,  16, "XG #070 Other Waves 7", "GospelOr ***"},
    {'M',   0,  70,  17, "XG #070 Other Waves 7", "+Dusk"},
    {'M',   0,  70,  26, "XG #070 Other Waves 7", "+PulsJazz"},
    {'M',   0,  70,  27, "XG #070 Other Waves 7", "+Hammer B"},
    {'M',   0,  70,  30, "XG #070 Other Waves 7", "+Bombay S"},
    {'M',   0,  70,  33, "XG #070 Other Waves 7", "FngBass3"},
    {'M',   0,  70,  38, "XG #070 Other Waves 7", "+Pluto"},
    {'M',   0,  70,  39, "XG #070 Other Waves 7", "+Cubit +"},
    {'M',   0,  70,  48, "XG #070 Other Waves 7", "Hall Str"},
    {'M',   0,  70,  50, "XG #070 Other Waves 7", "+Brook St"},
    {'M',   0,  70,  51, "XG #070 Other Waves 7", "+Leo"},
    {'M',   0,  70,  52, "XG #070 Other Waves 7", "+++Aah Orch"},
    {'M',   0,  70,  55, "XG #070 Other Waves 7", "6th Hit ***"},
    {'M',   0,  70,  56, "XG #070 Other Waves 7", "Synth Tp"},
    {'M',   0,  70,  57, "XG #070 Other Waves 7", "+++Hrd Btb"},
    {'M',   0,  70,  61, "XG #070 Other Waves 7", "+Tnr&Trp"},
    {'M',   0,  70,  62, "XG #070 Other Waves 7", "+AnHrnOct"},
    {'M',   0,  70,  80, "XG #070 Other Waves 7", "+Accent"},
    {'M',   0,  70,  81, "XG #070 Other Waves 7", "+Hue"},
    {'M',   0,  70,  87, "XG #070 Other Waves 7", "Ba Lead"},
    {'M',   0,  70,  90, "XG #070 Other Waves 7", "+Pisces"},
    {'M',   0,  70,  91, "XG #070 Other Waves 7", "+++Ooh Pad"},
    {'M',   0,  70,  98, "XG #070 Other Waves 7", "AirBells"},
    {'M',   0,  70,  99, "XG #070 Other Waves 7", "+DgBermda"},
    {'M',   0,  70, 101, "XG #070 Other Waves 7", "Night"},
    {'M',   0,  70, 102, "XG #070 Other Waves 7", "Ana Echo"},
    {'M',   0,  71,   4, "XG #071 Other Waves 8", "+Diploid2"},
    {'M',   0,  71,   5, "XG #071 Other Waves 8", "+BrtDXDtd"},
    {'M',   0,  71,   7, "XG #071 Other Waves 8", "+HardyPlk"},
    {'M',   0,  71,  16, "XG #071 Other Waves 8", "ClkGsplO ***"},
    {'M',   0,  71,  17, "XG #071 Other Waves 8", "+FM Click"},
    {'M',   0,  71,  26, "XG #071 Other Waves 8", "+RghcastN"},
    {'M',   0,  71,  27, "XG #071 Other Waves 8", "+Hammer D"},
    {'M',   0,  71,  30, "XG #071 Other Waves 8", "+Jaipur"},
    {'M',   0,  71,  38, "XG #071 Other Waves 8", "+Pluto +"},
    {'M',   0,  71,  39, "XG #071 Other Waves 8", "+Keel"},
    {'M',   0,  71,  48, "XG #071 Other Waves 8", "Str/FHrn"},
    {'M',   0,  71,  50, "XG #071 Other Waves 8", "OldSynSt"},
    {'M',   0,  71,  51, "XG #071 Other Waves 8", "+SolPlexs"},
    {'M',   0,  71,  55, "XG #071 Other Waves 8", "6thHit + ***"},
    {'M',   0,  71,  56, "XG #071 Other Waves 8", "+++Sweet Tp"},
    {'M',   0,  71,  61, "XG #071 Other Waves 8", "+BrssBros"},
    {'M',   0,  71,  62, "XG #071 Other Waves 8", "+SawBrPwr"},
    {'M',   0,  71,  80, "XG #071 Other Waves 8", "+Brick"},
    {'M',   0,  71,  81, "XG #071 Other Waves 8", "+StrghtSw"},
    {'M',   0,  71,  90, "XG #071 Other Waves 8", "+Spiral"},
    {'M',   0,  71,  91, "XG #071 Other Waves 8", "+++Ooh Aah"},
    {'M',   0,  71,  98, "XG #071 Other Waves 8", "BellHarp"},
    {'M',   0,  71,  99, "XG #071 Other Waves 8", "Cloud Pd"},
    {'M',   0,  71, 101, "XG #071 Other Waves 8", "Glisten"},
    {'M',   0,  72,   4, "XG #072 Other Waves 9", "+Brooklyn"},
    {'M',   0,  72,   5, "XG #072 Other Waves 9", "+Kitayama"},
    {'M',   0,  72,   7, "XG #072 Other Waves 9", "+HrdyPlk"},
    {'M',   0,  72,  16, "XG #072 Other Waves 9", "ChapelOr ***"},
    {'M',   0,  72,  17, "XG #072 Other Waves 9", "+Spoony"},
    {'M',   0,  72,  26, "XG #072 Other Waves 9", "+RghcastM"},
    {'M',   0,  72,  27, "XG #072 Other Waves 9", "+HammerSt"},
    {'M',   0,  72,  38, "XG #072 Other Waves 9", "+Stimuli"},
    {'M',   0,  72,  39, "XG #072 Other Waves 9", "+KeelPwrd"},
    {'M',   0,  72,  48, "XG #072 Other Waves 9", "+++SolidStr"},
    {'M',   0,  72,  51, "XG #072 Other Waves 9", "SunRise"},
    {'M',   0,  72,  55, "XG #072 Other Waves 9", "Euro Hit ***"},
    {'M',   0,  72,  56, "XG #072 Other Waves 9", "+++Mllw Swt"},
    {'M',   0,  72,  61, "XG #072 Other Waves 9", "+VagueBro"},
    {'M',   0,  72,  80, "XG #072 Other Waves 9", "+Alum"},
    {'M',   0,  72,  81, "XG #072 Other Waves 9", "+StrtPuls"},
    {'M',   0,  72,  90, "XG #072 Other Waves 9", "PolySPd2"},
    {'M',   0,  72,  98, "XG #072 Other Waves 9", "Gamelmba"},
    {'M',   0,  72,  99, "XG #072 Other Waves 9", "PulseKey"},
    {'M',   0,  72, 101, "XG #072 Other Waves 9", "Puffy **"},
    {'M',   0,  73,   4, "XG #073 Other Waves 10", "+Diploid3"},
    {'M',   0,  73,   5, "XG #073 Other Waves 10", "+Turnpik1"},
    {'M',   0,  73,   7, "XG #073 Other Waves 10", "+FMClavDb"},
    {'M',   0,  73,  16, "XG #073 Other Waves 10", "+DimChors"},
    {'M',   0,  73,  17, "XG #073 Other Waves 10", "+SprRotry"},
    {'M',   0,  73,  27, "XG #073 Other Waves 10", "+FMChoGtr"},
    {'M',   0,  73,  38, "XG #073 Other Waves 10", "+RunPulse"},
    {'M',   0,  73,  39, "XG #073 Other Waves 10", "+PlnPulse"},
    {'M',   0,  73,  48, "XG #073 Other Waves 10", "+++SwellStr"},
    {'M',   0,  73,  55, "XG #073 Other Waves 10", "EuroHit+ ***"},
    {'M',   0,  73,  56, "XG #073 Other Waves 10", "+++NormTps"},
    {'M',   0,  73,  61, "XG #073 Other Waves 10", "BrssSec3"},
    {'M',   0,  73,  80, "XG #073 Other Waves 10", "+Query"},
    {'M',   0,  73,  81, "XG #073 Other Waves 10", "+PWMania"},
    {'M',   0,  73,  90, "XG #073 Other Waves 10", "PolyPdKg"},
    {'M',   0,  73,  98, "XG #073 Other Waves 10", "+Bounce"},
    {'M',   0,  73,  99, "XG #073 Other Waves 10", "NoisePno"},
    {'M',   0,  73, 101, "XG #073 Other Waves 10", "+Mimicry"},
    {'M',   0,  74,   4, "XG #074 Other Waves 11", "PhunkyDX"},
    {'M',   0,  74,   5, "XG #074 Other Waves 11", "Turnpik2"},
    {'M',   0,  74,   7, "XG #074 Other Waves 11", "+++RobotClv"},
    {'M',   0,  74,  16, "XG #074 Other Waves 11", "Dawn"},
    {'M',   0,  74,  17, "XG #074 Other Waves 11", "LoFiOrgn"},
    {'M',   0,  74,  27, "XG #074 Other Waves 11", "FMChoGtS"},
    {'M',   0,  74,  38, "XG #074 Other Waves 11", "TalkPuls"},
    {'M',   0,  74,  39, "XG #074 Other Waves 11", "PwrdPuls"},
    {'M',   0,  74,  48, "XG #074 Other Waves 11", "+++Str+Brss"},
    {'M',   0,  74,  55, "XG #074 Other Waves 11", "Blowout"},
    {'M',   0,  74,  56, "XG #074 Other Waves 11", "+++BrlltTp"},
    {'M',   0,  74,  61, "XG #074 Other Waves 11", "SfrzdBr2"},
    {'M',   0,  74,  80, "XG #074 Other Waves 11", "+FMSlwSwp"},
    {'M',   0,  74,  81, "XG #074 Other Waves 11", "+Mod Saw"},
    {'M',   0,  74,  98, "XG #074 Other Waves 11", "Ana Bell"},
    {'M',   0,  74, 101, "XG #074 Other Waves 11", "+Parasite"},
    {'M',   0,  75,   4, "XG #075 Other Waves 12", "+Nasal DX"},
    {'M',   0,  75,   5, "XG #075 Other Waves 12", "+Cerritos"},
    {'M',   0,  75,  16, "XG #075 Other Waves 12", "+Mellorgn"},
    {'M',   0,  75,  17, "XG #075 Other Waves 12", "+BeepOrgn"},
    {'M',   0,  75,  27, "XG #075 Other Waves 12", "+PeskyGtr"},
    {'M',   0,  75,  38, "XG #075 Other Waves 12", "+Node"},
    {'M',   0,  75,  39, "XG #075 Other Waves 12", "+PwrPulsB"},
    {'M',   0,  75,  48, "XG #075 Other Waves 12", "+++3oct Str"},
    {'M',   0,  75,  55, "XG #075 Other Waves 12", "+++Trcrtps"},
    {'M',   0,  75,  56, "XG #075 Other Waves 12", "+++Fantare"},
    {'M',   0,  75,  61, "XG #075 Other Waves 12", "OctBrass"},
    {'M',   0,  75,  80, "XG #075 Other Waves 12", "+SyncLdDb"},
    {'M',   0,  75,  81, "XG #075 Other Waves 12", "+Toad"},
    {'M',   0,  75, 101, "XG #075 Other Waves 12", "+Cicada"},
    {'M',   0,  76,   4, "XG #076 Other Waves 13", "+NaslDxDt"},
    {'M',   0,  76,   5, "XG #076 Other Waves 13", "+Sunset"},
    {'M',   0,  76,  16, "XG #076 Other Waves 13", "+Fuzzorgn"},
    {'M',   0,  76,  17, "XG #076 Other Waves 13", "+Belief"},
    {'M',   0,  76,  27, "XG #076 Other Waves 13", "+ClaviGtr"},
    {'M',   0,  76,  38, "XG #076 Other Waves 13", "+Stainer"},
    {'M',   0,  76,  39, "XG #076 Other Waves 13", "+Pwrd Saw"},
    {'M',   0,  76,  48, "XG #076 Other Waves 13", "+++5partStr"},
    {'M',   0,  76,  61, "XG #076 Other Waves 13", "+++Alps"},
    {'M',   0,  76,  80, "XG #076 Other Waves 13", "+Curse"},
    {'M',   0,  76,  81, "XG #076 Other Waves 13", "+FatOctav"},
    {'M',   0,  76, 101, "XG #076 Other Waves 13", "+Beacon"},
    {'M',   0,  77,   4, "XG #077 Other Waves 14", "+Din"},
    {'M',   0,  77,   5, "XG #077 Other Waves 14", "+Soft DX"},
    {'M',   0,  77,  16, "XG #077 Other Waves 14", "+FMO"},
    {'M',   0,  77,  17, "XG #077 Other Waves 14", "+SnapOrgn"},
    {'M',   0,  77,  38, "XG #077 Other Waves 14", "+StainAtk"},
    {'M',   0,  77,  39, "XG #077 Other Waves 14", "+++SmoothBs"},
    {'M',   0,  77,  61, "XG #077 Other Waves 14", "+++Symphnc"},
    {'M',   0,  77,  80, "XG #077 Other Waves 14", "+OctvBeep"},
    {'M',   0,  77,  81, "XG #077 Other Waves 14", "+Overdose"},
    {'M',   0,  77, 101, "XG #077 Other Waves 14", "+++GblnTalk"},
    {'M',   0,  78,   4, "XG #078 Other Waves 15", "+++4 Way EP"},
    {'M',   0,  78,   5, "XG #078 Other Waves 15", "+Reso DX"},
    {'M',   0,  78,  16, "XG #078 Other Waves 15", "70sDrOr3"},
    {'M',   0,  78,  38, "XG #078 Other Waves 15", "+SweepSqr"},
    {'M',   0,  78,  39, "XG #078 Other Waves 15", "+++SynAttck"},
    {'M',   0,  78,  61, "XG #078 Other Waves 15", "+++Phoenix"},
    {'M',   0,  78,  80, "XG #078 Other Waves 15", "Sine Ld2"},
    {'M',   0,  78,  81, "XG #078 Other Waves 15", "+PWMDecay"},
    {'M',   0,  78, 101, "XG #078 Other Waves 15", "+++Temple"},
    {'M',   0,  79,   4, "XG #079 Other Waves 16", "+++Easy EP"},
    {'M',   0,  79,   5, "XG #079 Other Waves 16", "+PiercnDX"},
    {'M',   0,  79,  16, "XG #079 Other Waves 16", "+++MoodOrgn"},
    {'M',   0,  79,  38, "XG #079 Other Waves 16", "+SwpSqr+"},
    {'M',   0,  79,  80, "XG #079 Other Waves 16", "SquarLd3"},
    {'M',   0,  79,  81, "XG #079 Other Waves 16", "+SawDecay"},
    {'M',   0,  80,   4, "XG #080 Other Waves 17", "+++Sine EP"},
    {'M',   0,  80,   5, "XG #080 Other Waves 17", "+ShvrngDX"},
    {'M',   0,  80,  38, "XG #080 Other Waves 17", "+Stinks"},
    {'M',   0,  80,  80, "XG #080 Other Waves 17", "SquarLd4"},
    {'M',   0,  80,  81, "XG #080 Other Waves 17", "FatSawLd"},
    {'M',   0,  81,   4, "XG #081 Other Waves 18", "+++Cheap EP"},
    {'M',   0,  81,   5, "XG #081 Other Waves 18", "+ShvrnDX+"},
    {'M',   0,  81,  38, "XG #081 Other Waves 18", "+StnksRes"},
    {'M',   0,  81,  81, "XG #081 Other Waves 18", "DuckLead"},
    {'M',   0,  82,   5, "XG #082 Other Waves 19", "+RattlnDX"},
    {'M',   0,  82,  38, "XG #082 Other Waves 19", "+Reso Sqr"},
    {'M',   0,  82,  81, "XG #082 Other Waves 19", "BoostSaw"},
    {'M',   0,  83,   5, "XG #083 Other Waves 20", "+RatlnDX+"},
    {'M',   0,  83,  38, "XG #083 Other Waves 20", "+Dagger"},
    {'M',   0,  83,  81, "XG #083 Other Waves 20", "Mr.Saw"},
    {'M',   0,  84,   5, "XG #084 Other Waves 21", "+TinkerDX"},
    {'M',   0,  84,  38, "XG #084 Other Waves 21", "+Zinc"},
    {'M',   0,  84,  81, "XG #084 Other Waves 21", "ThinSwLd"},
    {'M',   0,  85,   5, "XG #085 Other Waves 22", "+TinkrDX+"},
    {'M',   0,  85,  38, "XG #085 Other Waves 22", "+SweePWM"},
    {'M',   0,  85,  81, "XG #085 Other Waves 22", "MouthSaw"},
    {'M',   0,  86,  38, "XG #086 Other Waves 23", "+SwPWM St"},
    {'M',   0,  86,  81, "XG #086 Other Waves 23", "Dr.Lead2"},
    {'M',   0,  87,  38, "XG #087 Other Waves 24", "+Slow Wah"},
    {'M',   0,  87,  81, "XG #087 Other Waves 24", "Saw Unis"},
    {'M',   0,  88,  38, "XG #088 Other Waves 25", "+Crook"},
    {'M',   0,  88,  81, "XG #088 Other Waves 25", "OctSawLd"},
    {'M',   0,  89,  38, "XG #089 Other Waves 26", "FstFltBa"},
    {'M',   0,  89,  81, "XG #089 Other Waves 26", "Seq Saw1"},
    {'M',   0,  90,  38, "XG #090 Other Waves 27", "Rubber30"},
    {'M',   0,  90,  81, "XG #090 Other Waves 27", "Seq Saw2"},
    {'M',   0,  91,  38, "XG #091 Other Waves 28", "FstResB2"},
    {'M',   0,  91,  81, "XG #091 Other Waves 28", "+++SimplSaw"},
    {'M',   0,  92,  38, "XG #092 Other Waves 29", "+++MinneaBs"},
    {'M',   0,  93,  38, "XG #093 Other Waves 30", "+++Miami Bs"},
    {'M',   0,  94,  38, "XG #094 Other Waves 31", "+++ResoTalk"},
    {'M',   0,  96,  12, "XG #096 Other Instrument 1", "Balafon ***"},
    {'M',   0,  96,  14, "XG #096 Other Instrument 1", "ChrchBel"},
    {'M',   0,  96,  15, "XG #096 Other Instrument 1", "Cimbalom"},
    {'M',   0,  96,  24, "XG #096 Other Instrument 1", "Ukulele"},
    {'M',   0,  96,  25, "XG #096 Other Instrument 1", "Mandolin"},
    {'M',   0,  96,  26, "XG #096 Other Instrument 1", "PdlSteel **"},
    {'M',   0,  96,  28, "XG #096 Other Instrument 1", "Mu.DstGt **"},
    {'M',   0,  96,  32, "XG #096 Other Instrument 1", "+WalkSyBa"},
    {'M',   0,  96,  35, "XG #096 Other Instrument 1", "SynFretl"},
    {'M',   0,  96,  38, "XG #096 Other Instrument 1", "Hammer"},
    {'M',   0,  96,  46, "XG #096 Other Instrument 1", "+Vln Harp"},
    {'M',   0,  96,  53, "XG #096 Other Instrument 1", "VoiceHmn **"},
    {'M',   0,  96,  56, "XG #096 Other Instrument 1", "FluglHrn **"},
    {'M',   0,  96,  71, "XG #096 Other Instrument 1", "BassClar **"},
    {'M',   0,  96,  72, "XG #096 Other Instrument 1", "++Bang Di"},
    {'M',   0,  96,  73, "XG #096 Other Instrument 1", "++Qu Di"},
    {'M',   0,  96,  75, "XG #096 Other Instrument 1", "Kawala **"},
    {'M',   0,  96,  81, "XG #096 Other Instrument 1", "Seq Ana."},
    {'M',   0,  96, 100, "XG #096 Other Instrument 1", "Smokey"},
    {'M',   0,  96, 101, "XG #096 Other Instrument 1", "BelChoir"},
    {'M',   0,  96, 104, "XG #096 Other Instrument 1", "Tambra"},
    {'M',   0,  96, 105, "XG #096 Other Instrument 1", "Rabab"},
    {'M',   0,  96, 106, "XG #096 Other Instrument 1", "Tsugaru **"},
    {'M',   0,  96, 107, "XG #096 Other Instrument 1", "Taisho-k"},
    {'M',   0,  96, 109, "XG #096 Other Instrument 1", "++Sheng"},
    {'M',   0,  96, 110, "XG #096 Other Instrument 1", "++Er Hu"},
    {'M',   0,  96, 111, "XG #096 Other Instrument 1", "Pungi"},
    {'M',   0,  96, 112, "XG #096 Other Instrument 1", "Bonang"},
    {'M',   0,  96, 113, "XG #096 Other Instrument 1", "Atrigane ***"},
    {'M',   0,  96, 114, "XG #096 Other Instrument 1", "Tablas **"},
    {'M',   0,  96, 115, "XG #096 Other Instrument 1", "Castanet"},
    {'M',   0,  96, 116, "XG #096 Other Instrument 1", "Gr.Cassa"},
    {'M',   0,  96, 119, "XG #096 Other Instrument 1", "RevSnar1 **"},
    {'M',   0,  97,  12, "XG #097 Other Instrument 2", "Balimba"},
    {'M',   0,  97,  14, "XG #097 Other Instrument 2", "Carillon"},
    {'M',   0,  97,  15, "XG #097 Other Instrument 2", "Santur"},
    {'M',   0,  97,  25, "XG #097 Other Instrument 2", "+MndlnEns"},
    {'M',   0,  97,  32, "XG #097 Other Instrument 2", "+Dim&Cool"},
    {'M',   0,  97,  35, "XG #097 Other Instrument 2", "SmthFrtl"},
    {'M',   0,  97,  47, "XG #097 Other Instrument 2", "+VlnHrpDt"},
    {'M',   0,  97,  56, "XG #097 Other Instrument 2", "+Cornet"},
    {'M',   0,  97, 101, "XG #097 Other Instrument 2", "Dharma ***"},
    {'M',   0,  97, 104, "XG #097 Other Instrument 2", "Tamboura"},
    {'M',   0,  97, 105, "XG #097 Other Instrument 2", "Gopichnt"},
    {'M',   0,  97, 107, "XG #097 Other Instrument 2", "Kanoon"},
    {'M',   0,  97, 110, "XG #097 Other Instrument 2", "++Ban Hu"},
    {'M',   0,  97, 111, "XG #097 Other Instrument 2", "Hichriki"},
    {'M',   0,  97, 112, "XG #097 Other Instrument 2", "Altair"},
    {'M',   0,  97, 114, "XG #097 Other Instrument 2", "GlasPerc"},
    {'M',   0,  97, 119, "XG #097 Other Instrument 2", "RevSnar2 **"},
    {'M',   0,  98,  12, "XG #098 Other Instrument 3", "Log Drum"},
    {'M',   0,  98,  15, "XG #098 Other Instrument 3", "++Yang Qin"},
    {'M',   0,  98, 105, "XG #098 Other Instrument 3", "Oud"},
    {'M',   0,  98, 107, "XG #098 Other Instrument 3", "++Zheng"},
    {'M',   0,  98, 110, "XG #098 Other Instrument 3", "++Jing Hu"},
    {'M',   0,  98, 111, "XG #098 Other Instrument 3", "++Suo Na"},
    {'M',   0,  98, 112, "XG #098 Other Instrument 3", "Gamelan"},
    {'M',   0,  98, 114, "XG #098 Other Instrument 3", "ThaiBell"},
    {'M',   0,  98, 119, "XG #098 Other Instrument 3", "RevKick1 **"},
    {'M',   0,  99, 105, "XG #099 Other Instrument 4", "++Pi Pa"},
    {'M',   0,  99, 112, "XG #099 Other Instrument 4", "S.Gamlan"},
    {'M',   0,  99, 119, "XG #099 Other Instrument 4", "RevConBD **"},
    {'M',   0, 100, 112, "XG #100 Other Instrument 5", "Rama Cym"},
    {'M',   0, 100, 119, "XG #100 Other Instrument 5", "Rev Tom1 **"},
    {'M',   0, 101, 112, "XG #101 Other Instrument 6", "AsianBel"},
    {'M',   0, 101, 119, "XG #101 Other Instrument 6", "Rev Tom2 **"},
    {'M',   0, 126,   0, "XG #126 Capital Voices on MU100 Native Map", "+GrandP #"},
    {'M',   0, 126,   1, "XG #126 Capital Voices on MU100 Native Map", "+BriteP #"},
    {'M',   0, 126,   2, "XG #126 Capital Voices on MU100 Native Map", "El.Grand"},
    {'M',   0, 126,   3, "XG #126 Capital Voices on MU100 Native Map", "HnkyTonk"},
    {'M',   0, 126,   4, "XG #126 Capital Voices on MU100 Native Map", "+EPiano1#"},
    {'M',   0, 126,   5, "XG #126 Capital Voices on MU100 Native Map", "+EPiano2#"},
    {'M',   0, 126,   6, "XG #126 Capital Voices on MU100 Native Map", "Harpsi."},
    {'M',   0, 126,   7, "XG #126 Capital Voices on MU100 Native Map", "+Clavi #"},
    {'M',   0, 126,   8, "XG #126 Capital Voices on MU100 Native Map", "Celesta"},
    {'M',   0, 126,   9, "XG #126 Capital Voices on MU100 Native Map", "Glocken"},
    {'M',   0, 126,  10, "XG #126 Capital Voices on MU100 Native Map", "MusicBox"},
    {'M',   0, 126,  11, "XG #126 Capital Voices on MU100 Native Map", "Vibes"},
    {'M',   0, 126,  12, "XG #126 Capital Voices on MU100 Native Map", "Marimba"},
    {'M',   0, 126,  13, "XG #126 Capital Voices on MU100 Native Map", "Xylophon"},
    {'M',   0, 126,  14, "XG #126 Capital Voices on MU100 Native Map", "TubulBel"},
    {'M',   0, 126,  15, "XG #126 Capital Voices on MU100 Native Map", "Dulcimer"},
    {'M',   0, 126,  16, "XG #126 Capital Voices on MU100 Native Map", "+DrawOrg#"},
    {'M',   0, 126,  17, "XG #126 Capital Voices on MU100 Native Map", "+PercOrg#"},
    {'M',   0, 126,  18, "XG #126 Capital Voices on MU100 Native Map", "+RockOrg#"},
    {'M',   0, 126,  19, "XG #126 Capital Voices on MU100 Native Map", "ChrchOrgn"},
    {'M',   0, 126,  20, "XG #126 Capital Voices on MU100 Native Map", "ReedOrgn"},
    {'M',   0, 126,  21, "XG #126 Capital Voices on MU100 Native Map", "Acordion"},
    {'M',   0, 126,  22, "XG #126 Capital Voices on MU100 Native Map", "Harmnica"},
    {'M',   0, 126,  23, "XG #126 Capital Voices on MU100 Native Map", "+TangoAc#"},
    {'M',   0, 126,  24, "XG #126 Capital Voices on MU100 Native Map", "+NylonGt#"},
    {'M',   0, 126,  25, "XG #126 Capital Voices on MU100 Native Map", "+SteelGt#"},
    {'M',   0, 126,  26, "XG #126 Capital Voices on MU100 Native Map", "+JazzGtr#"},
    {'M',   0, 126,  27, "XG #126 Capital Voices on MU100 Native Map", "CleanGtr"},
    {'M',   0, 126,  28, "XG #126 Capital Voices on MU100 Native Map", "+MuteGtr#"},
    {'M',   0, 126,  29, "XG #126 Capital Voices on MU100 Native Map", "+Ovrdriv#"},
    {'M',   0, 126,  30, "XG #126 Capital Voices on MU100 Native Map", "+DistGtr#"},
    {'M',   0, 126,  31, "XG #126 Capital Voices on MU100 Native Map", "GtrHarmo"},
    {'M',   0, 126,  32, "XG #126 Capital Voices on MU100 Native Map", "+AcoBass#"},
    {'M',   0, 126,  33, "XG #126 Capital Voices on MU100 Native Map", "+FngrBa #"},
    {'M',   0, 126,  34, "XG #126 Capital Voices on MU100 Native Map", "PickBass"},
    {'M',   0, 126,  35, "XG #126 Capital Voices on MU100 Native Map", "+Frtless#"},
    {'M',   0, 126,  36, "XG #126 Capital Voices on MU100 Native Map", "+SlapBa1#"},
    {'M',   0, 126,  37, "XG #126 Capital Voices on MU100 Native Map", "+SlapBa2#"},
    {'M',   0, 126,  38, "XG #126 Capital Voices on MU100 Native Map", "SynBass1"},
    {'M',   0, 126,  39, "XG #126 Capital Voices on MU100 Native Map", "SynBass2"},
    {'M',   0, 126,  40, "XG #126 Capital Voices on MU100 Native Map", "+Violin #"},
    {'M',   0, 126,  41, "XG #126 Capital Voices on MU100 Native Map", "Viola"},
    {'M',   0, 126,  42, "XG #126 Capital Voices on MU100 Native Map", "Cello"},
    {'M',   0, 126,  43, "XG #126 Capital Voices on MU100 Native Map", "Contrabs"},
    {'M',   0, 126,  44, "XG #126 Capital Voices on MU100 Native Map", "+TremStr#"},
    {'M',   0, 126,  45, "XG #126 Capital Voices on MU100 Native Map", "Pizz.Str"},
    {'M',   0, 126,  46, "XG #126 Capital Voices on MU100 Native Map", "Harp"},
    {'M',   0, 126,  47, "XG #126 Capital Voices on MU100 Native Map", "Timpani"},
    {'M',   0, 126,  48, "XG #126 Capital Voices on MU100 Native Map", "+Strngs1#"},
    {'M',   0, 126,  49, "XG #126 Capital Voices on MU100 Native Map", "+Strngs2#"},
    {'M',   0, 126,  50, "XG #126 Capital Voices on MU100 Native Map", "Syn Str1"},
    {'M',   0, 126,  51, "XG #126 Capital Voices on MU100 Native Map", "Syn Str2"},
    {'M',   0, 126,  52, "XG #126 Capital Voices on MU100 Native Map", "ChoirAah"},
    {'M',   0, 126,  53, "XG #126 Capital Voices on MU100 Native Map", "VoiceOoh"},
    {'M',   0, 126,  54, "XG #126 Capital Voices on MU100 Native Map", "SynVoice"},
    {'M',   0, 126,  55, "XG #126 Capital Voices on MU100 Native Map", "Orch.Hit"},
    {'M',   0, 126,  56, "XG #126 Capital Voices on MU100 Native Map", "+Trumpet#"},
    {'M',   0, 126,  57, "XG #126 Capital Voices on MU100 Native Map", "+Trmbone#"},
    {'M',   0, 126,  58, "XG #126 Capital Voices on MU100 Native Map", "Tuba"},
    {'M',   0, 126,  59, "XG #126 Capital Voices on MU100 Native Map", "+MuteTrp#"},
    {'M',   0, 126,  60, "XG #126 Capital Voices on MU100 Native Map", "Fr. Horn"},
    {'M',   0, 126,  61, "XG #126 Capital Voices on MU100 Native Map", "+BrssSec#"},
    {'M',   0, 126,  62, "XG #126 Capital Voices on MU100 Native Map", "SynBrss1"},
    {'M',   0, 126,  63, "XG #126 Capital Voices on MU100 Native Map", "SynBrss2"},
    {'M',   0, 126,  64, "XG #126 Capital Voices on MU100 Native Map", "SprnoSax"},
    {'M',   0, 126,  65, "XG #126 Capital Voices on MU100 Native Map", "+AltoSax#"},
    {'M',   0, 126,  66, "XG #126 Capital Voices on MU100 Native Map", "TenorSax"},
    {'M',   0, 126,  67, "XG #126 Capital Voices on MU100 Native Map", "Bari.Sax"},
    {'M',   0, 126,  68, "XG #126 Capital Voices on MU100 Native Map", "+Oboe #"},
    {'M',   0, 126,  69, "XG #126 Capital Voices on MU100 Native Map", "Eng.Horn"},
    {'M',   0, 126,  70, "XG #126 Capital Voices on MU100 Native Map", "Bassoon"},
    {'M',   0, 126,  71, "XG #126 Capital Voices on MU100 Native Map", "Clarinet"},
    {'M',   0, 126,  72, "XG #126 Capital Voices on MU100 Native Map", "Piccolo"},
    {'M',   0, 126,  73, "XG #126 Capital Voices on MU100 Native Map", "+Flute #"},
    {'M',   0, 126,  74, "XG #126 Capital Voices on MU100 Native Map", "Recorder"},
    {'M',   0, 126,  75, "XG #126 Capital Voices on MU100 Native Map", "+PanFlut#"},
    {'M',   0, 126,  76, "XG #126 Capital Voices on MU100 Native Map", "Bottle"},
    {'M',   0, 126,  77, "XG #126 Capital Voices on MU100 Native Map", "Shakhchi"},
    {'M',   0, 126,  78, "XG #126 Capital Voices on MU100 Native Map", "Whistle"},
    {'M',   0, 126,  79, "XG #126 Capital Voices on MU100 Native Map", "Ocarina"},
    {'M',   0, 126,  80, "XG #126 Capital Voices on MU100 Native Map", "SquareLd"},
    {'M',   0, 126,  81, "XG #126 Capital Voices on MU100 Native Map", "Saw Ld"},
    {'M',   0, 126,  82, "XG #126 Capital Voices on MU100 Native Map", "CaliopLd"},
    {'M',   0, 126,  83, "XG #126 Capital Voices on MU100 Native Map", "Chiff Ld"},
    {'M',   0, 126,  84, "XG #126 Capital Voices on MU100 Native Map", "CharanLd"},
    {'M',   0, 126,  85, "XG #126 Capital Voices on MU100 Native Map", "Voice Ld"},
    {'M',   0, 126,  86, "XG #126 Capital Voices on MU100 Native Map", "Fifth Ld"},
    {'M',   0, 126,  87, "XG #126 Capital Voices on MU100 Native Map", "Bass&Ld"},
    {'M',   0, 126,  88, "XG #126 Capital Voices on MU100 Native Map", "NewAgePad"},
    {'M',   0, 126,  89, "XG #126 Capital Voices on MU100 Native Map", "Warm Pad"},
    {'M',   0, 126,  90, "XG #126 Capital Voices on MU100 Native Map", "PolySyPd"},
    {'M',   0, 126,  91, "XG #126 Capital Voices on MU100 Native Map", "ChoirPad"},
    {'M',   0, 126,  92, "XG #126 Capital Voices on MU100 Native Map", "BowedPad"},
    {'M',   0, 126,  93, "XG #126 Capital Voices on MU100 Native Map", "MetalPad"},
    {'M',   0, 126,  94, "XG #126 Capital Voices on MU100 Native Map", "Halo Pad"},
    {'M',   0, 126,  95, "XG #126 Capital Voices on MU100 Native Map", "SweepPad"},
    {'M',   0, 126,  96, "XG #126 Capital Voices on MU100 Native Map", "Rain"},
    {'M',   0, 126,  97, "XG #126 Capital Voices on MU100 Native Map", "SoundTrk"},
    {'M',   0, 126,  98, "XG #126 Capital Voices on MU100 Native Map", "Crystal"},
    {'M',   0, 126,  99, "XG #126 Capital Voices on MU100 Native Map", "Atmosphr"},
    {'M',   0, 126, 100, "XG #126 Capital Voices on MU100 Native Map", "Bright"},
    {'M',   0, 126, 101, "XG #126 Capital Voices on MU100 Native Map", "Goblins"},
    {'M',   0, 126, 102, "XG #126 Capital Voices on MU100 Native Map", "Echoes"},
    {'M',   0, 126, 103, "XG #126 Capital Voices on MU100 Native Map", "Sci-Fi"},
    {'M',   0, 126, 104, "XG #126 Capital Voices on MU100 Native Map", "Sitar"},
    {'M',   0, 126, 105, "XG #126 Capital Voices on MU100 Native Map", "Banjo"},
    {'M',   0, 126, 106, "XG #126 Capital Voices on MU100 Native Map", "Shamisen"},
    {'M',   0, 126, 107, "XG #126 Capital Voices on MU100 Native Map", "Koto"},
    {'M',   0, 126, 108, "XG #126 Capital Voices on MU100 Native Map", "Kalimba"},
    {'M',   0, 126, 109, "XG #126 Capital Voices on MU100 Native Map", "Bagpipe"},
    {'M',   0, 126, 110, "XG #126 Capital Voices on MU100 Native Map", "Fiddle"},
    {'M',   0, 126, 111, "XG #126 Capital Voices on MU100 Native Map", "Shanai"},
    {'M',   0, 126, 112, "XG #126 Capital Voices on MU100 Native Map", "TnklBell"},
    {'M',   0, 126, 113, "XG #126 Capital Voices on MU100 Native Map", "Agogo"},
    {'M',   0, 126, 114, "XG #126 Capital Voices on MU100 Native Map", "SteelDrm"},
    {'M',   0, 126, 115, "XG #126 Capital Voices on MU100 Native Map", "Woodblok"},
    {'M',   0, 126, 116, "XG #126 Capital Voices on MU100 Native Map", "TaikoDrm"},
    {'M',   0, 126, 117, "XG #126 Capital Voices on MU100 Native Map", "MelodTom"},
    {'M',   0, 126, 118, "XG #126 Capital Voices on MU100 Native Map", "Syn Drum"},
    {'M',   0, 126, 119, "XG #126 Capital Voices on MU100 Native Map", "RevCymbl"},
    {'M',   0, 126, 120, "XG #126 Capital Voices on MU100 Native Map", "FretNoiz"},
    {'M',   0, 126, 121, "XG #126 Capital Voices on MU100 Native Map", "BrthNoiz"},
    {'M',   0, 126, 122, "XG #126 Capital Voices on MU100 Native Map", "Seashore"},
    {'M',   0, 126, 123, "XG #126 Capital Voices on MU100 Native Map", "Tweet"},
    {'M',   0, 126, 124, "XG #126 Capital Voices on MU100 Native Map", "Telphone"},
    {'M',   0, 126, 125, "XG #126 Capital Voices on MU100 Native Map", "Helicptr"},
    {'M',   0, 126, 126, "XG #126 Capital Voices on MU100 Native Map", "Applause"},
    {'M',   0, 126, 127, "XG #126 Capital Voices on MU100 Native Map", "Gunshot"},
    {'M',   0, 127,   0, "XG #127 Capital Voices on MU Basic Map", "GrandPno"},
    {'M',   0, 127,   1, "XG #127 Capital Voices on MU Basic Map", "BritePno"},
    {'M',   0, 127,   2, "XG #127 Capital Voices on MU Basic Map", "El.Grand"},
    {'M',   0, 127,   3, "XG #127 Capital Voices on MU Basic Map", "HnkyTonk"},
    {'M',   0, 127,   4, "XG #127 Capital Voices on MU Basic Map", "E.Piano1"},
    {'M',   0, 127,   5, "XG #127 Capital Voices on MU Basic Map", "E.Piano2"},
    {'M',   0, 127,   6, "XG #127 Capital Voices on MU Basic Map", "Harpsi."},
    {'M',   0, 127,   7, "XG #127 Capital Voices on MU Basic Map", "Clavi"},
    {'M',   0, 127,   8, "XG #127 Capital Voices on MU Basic Map", "Celesta"},
    {'M',   0, 127,   9, "XG #127 Capital Voices on MU Basic Map", "Glocken"},
    {'M',   0, 127,  10, "XG #127 Capital Voices on MU Basic Map", "MusicBox"},
    {'M',   0, 127,  11, "XG #127 Capital Voices on MU Basic Map", "Vibes"},
    {'M',   0, 127,  12, "XG #127 Capital Voices on MU Basic Map", "Marimba"},
    {'M',   0, 127,  13, "XG #127 Capital Voices on MU Basic Map", "Xylophon"},
    {'M',   0, 127,  14, "XG #127 Capital Voices on MU Basic Map", "TubulBel"},
    {'M',   0, 127,  15, "XG #127 Capital Voices on MU Basic Map", "Dulcimer"},
    {'M',   0, 127,  16, "XG #127 Capital Voices on MU Basic Map", "DrawOrgn"},
    {'M',   0, 127,  17, "XG #127 Capital Voices on MU Basic Map", "PercOrgn"},
    {'M',   0, 127,  18, "XG #127 Capital Voices on MU Basic Map", "RockOrgn"},
    {'M',   0, 127,  19, "XG #127 Capital Voices on MU Basic Map", "ChrchOrgn"},
    {'M',   0, 127,  20, "XG #127 Capital Voices on MU Basic Map", "ReedOrgn"},
    {'M',   0, 127,  21, "XG #127 Capital Voices on MU Basic Map", "Acordion"},
    {'M',   0, 127,  22, "XG #127 Capital Voices on MU Basic Map", "Harmnica"},
    {'M',   0, 127,  23, "XG #127 Capital Voices on MU Basic Map", "TangoAcd"},
    {'M',   0, 127,  24, "XG #127 Capital Voices on MU Basic Map", "NylonGtr"},
    {'M',   0, 127,  25, "XG #127 Capital Voices on MU Basic Map", "SteelGtr"},
    {'M',   0, 127,  26, "XG #127 Capital Voices on MU Basic Map", "Jazz Gtr"},
    {'M',   0, 127,  27, "XG #127 Capital Voices on MU Basic Map", "CleanGtr"},
    {'M',   0, 127,  28, "XG #127 Capital Voices on MU Basic Map", "Mute Gtr"},
    {'M',   0, 127,  29, "XG #127 Capital Voices on MU Basic Map", "Ovrdrive"},
    {'M',   0, 127,  30, "XG #127 Capital Voices on MU Basic Map", "Dist.Gtr"},
    {'M',   0, 127,  31, "XG #127 Capital Voices on MU Basic Map", "GtrHarmo"},
    {'M',   0, 127,  32, "XG #127 Capital Voices on MU Basic Map", "Aco.Bass"},
    {'M',   0, 127,  33, "XG #127 Capital Voices on MU Basic Map", "FngrBass"},
    {'M',   0, 127,  34, "XG #127 Capital Voices on MU Basic Map", "PickBass"},
    {'M',   0, 127,  35, "XG #127 Capital Voices on MU Basic Map", "Fretless"},
    {'M',   0, 127,  36, "XG #127 Capital Voices on MU Basic Map", "SlapBas1"},
    {'M',   0, 127,  37, "XG #127 Capital Voices on MU Basic Map", "SlapBas2"},
    {'M',   0, 127,  38, "XG #127 Capital Voices on MU Basic Map", "SynBass1"},
    {'M',   0, 127,  39, "XG #127 Capital Voices on MU Basic Map", "SynBass2"},
    {'M',   0, 127,  40, "XG #127 Capital Voices on MU Basic Map", "Violin"},
    {'M',   0, 127,  41, "XG #127 Capital Voices on MU Basic Map", "Viola"},
    {'M',   0, 127,  42, "XG #127 Capital Voices on MU Basic Map", "Cello"},
    {'M',   0, 127,  43, "XG #127 Capital Voices on MU Basic Map", "Contrabs"},
    {'M',   0, 127,  44, "XG #127 Capital Voices on MU Basic Map", "Trem.Str"},
    {'M',   0, 127,  45, "XG #127 Capital Voices on MU Basic Map", "Pizz.Str"},
    {'M',   0, 127,  46, "XG #127 Capital Voices on MU Basic Map", "Harp"},
    {'M',   0, 127,  47, "XG #127 Capital Voices on MU Basic Map", "Timpani"},
    {'M',   0, 127,  48, "XG #127 Capital Voices on MU Basic Map", "Strings1"},
    {'M',   0, 127,  49, "XG #127 Capital Voices on MU Basic Map", "Strings2"},
    {'M',   0, 127,  50, "XG #127 Capital Voices on MU Basic Map", "Syn Str1"},
    {'M',   0, 127,  51, "XG #127 Capital Voices on MU Basic Map", "Syn Str2"},
    {'M',   0, 127,  52, "XG #127 Capital Voices on MU Basic Map", "ChoirAah"},
    {'M',   0, 127,  53, "XG #127 Capital Voices on MU Basic Map", "VoiceOoh"},
    {'M',   0, 127,  54, "XG #127 Capital Voices on MU Basic Map", "SynVoice"},
    {'M',   0, 127,  55, "XG #127 Capital Voices on MU Basic Map", "Orch.Hit"},
    {'M',   0, 127,  56, "XG #127 Capital Voices on MU Basic Map", "Trumpet"},
    {'M',   0, 127,  57, "XG #127 Capital Voices on MU Basic Map", "Trombone"},
    {'M',   0, 127,  58, "XG #127 Capital Voices on MU Basic Map", "Tuba"},
    {'M',   0, 127,  59, "XG #127 Capital Voices on MU Basic Map", "Mute Trp"},
    {'M',   0, 127,  60, "XG #127 Capital Voices on MU Basic Map", "Fr. Horn"},
    {'M',   0, 127,  61, "XG #127 Capital Voices on MU Basic Map", "BrssSect"},
    {'M',   0, 127,  62, "XG #127 Capital Voices on MU Basic Map", "SynBrss1"},
    {'M',   0, 127,  63, "XG #127 Capital Voices on MU Basic Map", "SynBrss2"},
    {'M',   0, 127,  64, "XG #127 Capital Voices on MU Basic Map", "SprnoSax"},
    {'M',   0, 127,  65, "XG #127 Capital Voices on MU Basic Map", "Alto Sax"},
    {'M',   0, 127,  66, "XG #127 Capital Voices on MU Basic Map", "TenorSax"},
    {'M',   0, 127,  67, "XG #127 Capital Voices on MU Basic Map", "Bari.Sax"},
    {'M',   0, 127,  68, "XG #127 Capital Voices on MU Basic Map", "Oboe"},
    {'M',   0, 127,  69, "XG #127 Capital Voices on MU Basic Map", "Eng.Horn"},
    {'M',   0, 127,  70, "XG #127 Capital Voices on MU Basic Map", "Bassoon"},
    {'M',   0, 127,  71, "XG #127 Capital Voices on MU Basic Map", "Clarinet"},
    {'M',   0, 127,  72, "XG #127 Capital Voices on MU Basic Map", "Piccolo"},
    {'M',   0, 127,  73, "XG #127 Capital Voices on MU Basic Map", "Flute"},
    {'M',   0, 127,  74, "XG #127 Capital Voices on MU Basic Map", "Recorder"},
    {'M',   0, 127,  75, "XG #127 Capital Voices on MU Basic Map", "PanFlute"},
    {'M',   0, 127,  76, "XG #127 Capital Voices on MU Basic Map", "Bottle"},
    {'M',   0, 127,  77, "XG #127 Capital Voices on MU Basic Map", "Shakhchi"},
    {'M',   0, 127,  78, "XG #127 Capital Voices on MU Basic Map", "Whistle"},
    {'M',   0, 127,  79, "XG #127 Capital Voices on MU Basic Map", "Ocarina"},
    {'M',   0, 127,  80, "XG #127 Capital Voices on MU Basic Map", "SquareLd"},
    {'M',   0, 127,  81, "XG #127 Capital Voices on MU Basic Map", "Saw Ld"},
    {'M',   0, 127,  82, "XG #127 Capital Voices on MU Basic Map", "CaliopLd"},
    {'M',   0, 127,  83, "XG #127 Capital Voices on MU Basic Map", "Chiff Ld"},
    {'M',   0, 127,  84, "XG #127 Capital Voices on MU Basic Map", "CharanLd"},
    {'M',   0, 127,  85, "XG #127 Capital Voices on MU Basic Map", "Voice Ld"},
    {'M',   0, 127,  86, "XG #127 Capital Voices on MU Basic Map", "Fifth Ld"},
    {'M',   0, 127,  87, "XG #127 Capital Voices on MU Basic Map", "Bass&Ld"},
    {'M',   0, 127,  88, "XG #127 Capital Voices on MU Basic Map", "NewAgePad"},
    {'M',   0, 127,  89, "XG #127 Capital Voices on MU Basic Map", "Warm Pad"},
    {'M',   0, 127,  90, "XG #127 Capital Voices on MU Basic Map", "PolySyPd"},
    {'M',   0, 127,  91, "XG #127 Capital Voices on MU Basic Map", "ChoirPad"},
    {'M',   0, 127,  92, "XG #127 Capital Voices on MU Basic Map", "BowedPad"},
    {'M',   0, 127,  93, "XG #127 Capital Voices on MU Basic Map", "MetalPad"},
    {'M',   0, 127,  94, "XG #127 Capital Voices on MU Basic Map", "Halo Pad"},
    {'M',   0, 127,  95, "XG #127 Capital Voices on MU Basic Map", "SweepPad"},
    {'M',   0, 127,  96, "XG #127 Capital Voices on MU Basic Map", "Rain"},
    {'M',   0, 127,  97, "XG #127 Capital Voices on MU Basic Map", "SoundTrk"},
    {'M',   0, 127,  98, "XG #127 Capital Voices on MU Basic Map", "Crystal"},
    {'M',   0, 127,  99, "XG #127 Capital Voices on MU Basic Map", "Atmosphr"},
    {'M',   0, 127, 100, "XG #127 Capital Voices on MU Basic Map", "Bright"},
    {'M',   0, 127, 101, "XG #127 Capital Voices on MU Basic Map", "Goblins"},
    {'M',   0, 127, 102, "XG #127 Capital Voices on MU Basic Map", "Echoes"},
    {'M',   0, 127, 103, "XG #127 Capital Voices on MU Basic Map", "Sci-Fi"},
    {'M',   0, 127, 104, "XG #127 Capital Voices on MU Basic Map", "Sitar"},
    {'M',   0, 127, 105, "XG #127 Capital Voices on MU Basic Map", "Banjo"},
    {'M',   0, 127, 106, "XG #127 Capital Voices on MU Basic Map", "Shamisen"},
    {'M',   0, 127, 107, "XG #127 Capital Voices on MU Basic Map", "Koto"},
    {'M',   0, 127, 108, "XG #127 Capital Voices on MU Basic Map", "Kalimba"},
    {'M',   0, 127, 109, "XG #127 Capital Voices on MU Basic Map", "Bagpipe"},
    {'M',   0, 127, 110, "XG #127 Capital Voices on MU Basic Map", "Fiddle"},
    {'M',   0, 127, 111, "XG #127 Capital Voices on MU Basic Map", "Shanai"},
    {'M',   0, 127, 112, "XG #127 Capital Voices on MU Basic Map", "TnklBell"},
    {'M',   0, 127, 113, "XG #127 Capital Voices on MU Basic Map", "Agogo"},
    {'M',   0, 127, 114, "XG #127 Capital Voices on MU Basic Map", "SteelDrm"},
    {'M',   0, 127, 115, "XG #127 Capital Voices on MU Basic Map", "Woodblok"},
    {'M',   0, 127, 116, "XG #127 Capital Voices on MU Basic Map", "TaikoDrm"},
    {'M',   0, 127, 117, "XG #127 Capital Voices on MU Basic Map", "MelodTom"},
    {'M',   0, 127, 118, "XG #127 Capital Voices on MU Basic Map", "Syn Drum"},
    {'M',   0, 127, 119, "XG #127 Capital Voices on MU Basic Map", "RevCymbl"},
    {'M',   0, 127, 120, "XG #127 Capital Voices on MU Basic Map", "FretNoiz"},
    {'M',   0, 127, 121, "XG #127 Capital Voices on MU Basic Map", "BrthNoiz"},
    {'M',   0, 127, 122, "XG #127 Capital Voices on MU Basic Map", "Seashore"},
    {'M',   0, 127, 123, "XG #127 Capital Voices on MU Basic Map", "Tweet"},
    {'M',   0, 127, 124, "XG #127 Capital Voices on MU Basic Map", "Telphone"},
    {'M',   0, 127, 125, "XG #127 Capital Voices on MU Basic Map", "Helicptr"},
    {'M',   0, 127, 126, "XG #127 Capital Voices on MU Basic Map", "Applause"},
    {'M',   0, 127, 127, "XG #127 Capital Voices on MU Basic Map", "Gunshot"},
    {'M',  48,   0,   0, "XG MEV #000 Timbre", "01 +MtdClavi"},
    {'M',  48,   0,   1, "XG MEV #000 Timbre", "02 +Orimba"},
    {'M',  48,   8,   0, "XG MEV #008 Timbre, Poly", "+Beeline"},
    {'M',  48,   8,   1, "XG MEV #008 Timbre, Poly", "+BlineHrd"},
    {'M',  48,   8,   2, "XG MEV #008 Timbre, Poly", "+Dwarf"},
    {'M',  48,   8,   3, "XG MEV #008 Timbre, Poly", "+Byte"},
    {'M',  48,   8,   4, "XG MEV #008 Timbre, Poly", "+Ping"},
    {'M',  48,   8,   5, "XG MEV #008 Timbre, Poly", "+NastyCut"},
    {'M',  48,   8,   6, "XG MEV #008 Timbre, Poly", "+NstyCtSt"},
    {'M',  48,   8,   7, "XG MEV #008 Timbre, Poly", "+Xe"},
    {'M',  48,   8,   8, "XG MEV #008 Timbre, Poly", "+OrganHit"},
    {'M',  48,   8,   9, "XG MEV #008 Timbre, Poly", "+OrgnHit+"},
    {'M',  48,   8,  10, "XG MEV #008 Timbre, Poly", "+Soft Hit"},
    {'M',  48,   8,  11, "XG MEV #008 Timbre, Poly", "+Vein"},
    {'M',  48,   8,  12, "XG MEV #008 Timbre, Poly", "+Packet"},
    {'M',  48,   8,  13, "XG MEV #008 Timbre, Poly", "+Jolly"},
    {'M',  48,   8,  14, "XG MEV #008 Timbre, Poly", "+Jolly +"},
    {'M',  48,   8,  15, "XG MEV #008 Timbre, Poly", "+Envy"},
    {'M',  48,   8,  16, "XG MEV #008 Timbre, Poly", "+EnvyShrt"},
    {'M',  48,   8,  17, "XG MEV #008 Timbre, Poly", "+Cough"},
    {'M',  48,   8,  18, "XG MEV #008 Timbre, Poly", "+Remark"},
    {'M',  48,   8,  19, "XG MEV #008 Timbre, Poly", "+Potala"},
    {'M',  48,   8,  20, "XG MEV #008 Timbre, Poly", "+Fury"},
    {'M',  48,   8,  21, "XG MEV #008 Timbre, Poly", "+Glocken+"},
    {'M',  48,  16,   0, "XG MEV #016 Timbre, Looped", "+Rage"},
    {'M',  48,  16,   1, "XG MEV #016 Timbre, Looped", "+TnglCaos"},
    {'M',  48,  16,   2, "XG MEV #016 Timbre, Looped", "+Incontnc"},
    {'M',  48,  16,   3, "XG MEV #016 Timbre, Looped", "+IncntClk"},
    {'M',  48,  16,   4, "XG MEV #016 Timbre, Looped", "+CheapOsc"},
    {'M',  48,  16,   5, "XG MEV #016 Timbre, Looped", "+CheapOc+"},
    {'M',  48,  16,   6, "XG MEV #016 Timbre, Looped", "+NstOrSpl"},
    {'M',  48,  16,   7, "XG MEV #016 Timbre, Looped", "+SprClick"},
    {'M',  48,  16,   8, "XG MEV #016 Timbre, Looped", "+Boomout"},
    {'M',  48,  16,   9, "XG MEV #016 Timbre, Looped", "+ChoirSpl"},
    {'M',  48,  16,  10, "XG MEV #016 Timbre, Looped", "+Veld"},
    {'M',  48,  16,  11, "XG MEV #016 Timbre, Looped", "+VaporVox"},
    {'M',  48,  16,  12, "XG MEV #016 Timbre, Looped", "+VoxShoot"},
    {'M',  48,  16,  13, "XG MEV #016 Timbre, Looped", "+SemiacOr"},
    {'M',  48,  24,   0, "XG MEV #024 Timbre, Looped, Poly", "+Fuss"},
    {'M',  48,  24,   1, "XG MEV #024 Timbre, Looped, Poly", "+VanAllen"},
    {'M',  48,  24,   2, "XG MEV #024 Timbre, Looped, Poly", "+Divinity"},
    {'M',  48,  24,   3, "XG MEV #024 Timbre, Looped, Poly", "+Paranoia"},
    {'M',  48,  24,   4, "XG MEV #024 Timbre, Looped, Poly", "+Vexation"},
    {'M',  48,  24,   5, "XG MEV #024 Timbre, Looped, Poly", "+CalcOrg1"},
    {'M',  48,  24,   6, "XG MEV #024 Timbre, Looped, Poly", "+CalcOrg2"},
    {'M',  48,  24,   7, "XG MEV #024 Timbre, Looped, Poly", "+CalcOrg3"},
    {'M',  48,  24,   8, "XG MEV #024 Timbre, Looped, Poly", "+ChoCalcO"},
    {'M',  48,  24,   9, "XG MEV #024 Timbre, Looped, Poly", "+Sodium"},
    {'M',  48,  56,   0, "XG MEV #056 Phrase, Looped, Poly", "+Insanity"},
    {'M',  48,  56,   1, "XG MEV #056 Phrase, Looped, Poly", "+Habakkuk"},
    {'M',  48,  64,   0, "XG MEV #064 SFX, Timbre", "+DstnFire"},
    {'M',  48,  64,   1, "XG MEV #064 SFX, Timbre", "+BlowNoiz"},
    {'M',  48,  64,   2, "XG MEV #064 SFX, Timbre", "+Fall"},
    {'M',  48,  64,   3, "XG MEV #064 SFX, Timbre", "+Chaff"},
    {'M',  48,  72,   0, "XG MEV #072 SFX, Timbre, Poly", "+Litning"},
    {'M',  48,  72,   1, "XG MEV #072 SFX, Timbre, Poly", "+Oof"},
    {'M',  48,  72,   2, "XG MEV #072 SFX, Timbre, Poly", "+Lust"},
    {'M',  48,  72,   3, "XG MEV #072 SFX, Timbre, Poly", "+Lust St"},
    {'M',  48,  72,   4, "XG MEV #072 SFX, Timbre, Poly", "+SoftHit+"},
    {'M',  48,  72,   5, "XG MEV #072 SFX, Timbre, Poly", "+Cough +"},
    {'M',  48,  72,   6, "XG MEV #072 SFX, Timbre, Poly", "+Ruin"},
    {'M',  48,  72,   7, "XG MEV #072 SFX, Timbre, Poly", "+Ruin St"},
    {'M',  48,  72,   8, "XG MEV #072 SFX, Timbre, Poly", "+Vein +"},
    {'M',  48,  72,   9, "XG MEV #072 SFX, Timbre, Poly", "+Universe"},
    {'M',  48,  72,  10, "XG MEV #072 SFX, Timbre, Poly", "+UnvrsSwp"},
    {'M',  48,  72,  11, "XG MEV #072 SFX, Timbre, Poly", "+Xe +"},
    {'M',  48,  72,  12, "XG MEV #072 SFX, Timbre, Poly", "+Daze"},
    {'M',  48,  72,  13, "XG MEV #072 SFX, Timbre, Poly", "+Refusal"},
    {'M',  48,  72,  14, "XG MEV #072 SFX, Timbre, Poly", "+Refusal+"},
    {'M',  48,  72,  15, "XG MEV #072 SFX, Timbre, Poly", "+Lie"},
    {'M',  48,  72,  16, "XG MEV #072 SFX, Timbre, Poly", "+Malice"},
    {'M',  48,  72,  17, "XG MEV #072 SFX, Timbre, Poly", "+Doubt"},
    {'M',  48,  72,  18, "XG MEV #072 SFX, Timbre, Poly", "+DoubtDbl"},
    {'M',  48,  80,   0, "XG MEV #080 SFX, Phrase", "+Zipzap"},
    {'M',  48,  88,   0, "XG MEV #088 SFX, Phrase, Poly", "+Escape"},
    {'M',  48,  88,   1, "XG MEV #088 SFX, Phrase, Poly", "+Zigzag"},
    {'M',  48,  88,   2, "XG MEV #088 SFX, Phrase, Poly", "+Scud"},
    {'M',  48,  88,   3, "XG MEV #088 SFX, Phrase, Poly", "+Hinayana"},
    {'M',  48,  88,   4, "XG MEV #088 SFX, Phrase, Poly", "+Boot"},
    {'M',  48,  88,   5, "XG MEV #088 SFX, Phrase, Poly", "+Anathema"},
    {'M',  48,  88,   6, "XG MEV #088 SFX, Phrase, Poly", "+Germ 1"},
    {'M',  48,  88,   7, "XG MEV #088 SFX, Phrase, Poly", "+Germ 2"},
    {'M',  48,  88,   8, "XG MEV #088 SFX, Phrase, Poly", "+Malfnctn"},
    {'M',  48,  96,   0, "XG MEV #096 Rhythm, Timbre", "+TinKettl"},
    {'M',  48,  96,   1, "XG MEV #096 Rhythm, Timbre", "+K/S Anlg"},
    {'M',  48,  96,   2, "XG MEV #096 Rhythm, Timbre", "+K/S Humd"},
    {'M',  48,  96,   3, "XG MEV #096 Rhythm, Timbre", "+K/S Slap"},
    {'M',  48,  96,   4, "XG MEV #096 Rhythm, Timbre", "+Rvr/Crsh"},
    {'M',  48, 104,   0, "XG MEV #104 Rhythm, Timbre, Poly", "+Satya"},
    {'M',  48, 104,   1, "XG MEV #104 Rhythm, Timbre, Poly", "+Monastry"},
    {'M',  48, 104,   2, "XG MEV #104 Rhythm, Timbre, Poly", "+Vinaya"},
    {'M',  48, 104,   3, "XG MEV #104 Rhythm, Timbre, Poly", "+Prana"},
    {'M',  48, 104,   4, "XG MEV #104 Rhythm, Timbre, Poly", "+Delight"},
    {'M',  48, 104,   5, "XG MEV #104 Rhythm, Timbre, Poly", "+Puja"},
    {'M',  48, 104,   6, "XG MEV #104 Rhythm, Timbre, Poly", "+Bent Tom"},
    {'M',  48, 120,   0, "XG MEV #120 Rhythm, Phrase, Poly", "+Vodou"},
    {'M',  49,  20,   0, "XG MEV #048 Phrase, Looped", "+Reflex 1"},
    {'M',  49,  20,   1, "XG MEV #048 Phrase, Looped", "+Reflex 2"},
    {'M',  49,  20,   2, "XG MEV #048 Phrase, Looped", "+Prcesion"},
    {'M',  49,  20,   3, "XG MEV #048 Phrase, Looped", "+RndmWalk"},
    {'M',  49,  20,   4, "XG MEV #048 Phrase, Looped", "+RandmRun"},
    {'M',  64,   0,   0, "XG SFX #000", "CuttngNz"},
    {'M',  64,   0,   1, "XG SFX #000", "CttngNz2"},
    {'M',  64,   0,   2, "XG SFX #000", "DstCutNz **"},
    {'M',  64,   0,   3, "XG SFX #000", "Str Slap"},
    {'M',  64,   0,   4, "XG SFX #000", "B.Slide **"},
    {'M',  64,   0,   5, "XG SFX #000", "P.Scrape **"},
    {'M',  64,   0,  16, "XG SFX #000", "Fl.KClik"},
    {'M',  64,   0,  32, "XG SFX #000", "Shower"},
    {'M',  64,   0,  33, "XG SFX #000", "Thunder"},
    {'M',  64,   0,  34, "XG SFX #000", "Wind"},
    {'M',  64,   0,  35, "XG SFX #000", "Stream"},
    {'M',  64,   0,  36, "XG SFX #000", "Bubble"},
    {'M',  64,   0,  37, "XG SFX #000", "Feed **"},
    {'M',  64,   0,  38, "XG SFX #000", "+Cave"},
    {'M',  64,   0,  48, "XG SFX #000", "Dog"},
    {'M',  64,   0,  49, "XG SFX #000", "Horse"},
    {'M',  64,   0,  50, "XG SFX #000", "Tweet 2"},
    {'M',  64,   0,  51, "XG SFX #000", "Kitty **"},
    {'M',  64,   0,  52, "XG SFX #000", "Growl **"},
    {'M',  64,   0,  53, "XG SFX #000", "Haunted **"},
    {'M',  64,   0,  54, "XG SFX #000", "Ghost"},
    {'M',  64,   0,  55, "XG SFX #000", "Maou **"},
    {'M',  64,   0,  56, "XG SFX #000", "+Insects"},
    {'M',  64,   0,  57, "XG SFX #000", "+Bacteria"},
    {'M',  64,   0,  64, "XG SFX #000", "PhonCall **"},
    {'M',  64,   0,  65, "XG SFX #000", "DoorSqek"},
    {'M',  64,   0,  66, "XG SFX #000", "DoorSlam"},
    {'M',  64,   0,  67, "XG SFX #000", "ScratchC"},
    {'M',  64,   0,  68, "XG SFX #000", "ScratchS **"},
    {'M',  64,   0,  69, "XG SFX #000", "WindChim"},
    {'M',  64,   0,  70, "XG SFX #000", "Telphon2"},
    {'M',  64,   0,  71, "XG SFX #000", "+++AnthrScr"},
    {'M',  64,   0,  72, "XG SFX #000", "+++TurnTabl"},
    {'M',  64,   0,  80, "XG SFX #000", "CarElgnt"},
    {'M',  64,   0,  81, "XG SFX #000", "CarTSqel"},
    {'M',  64,   0,  82, "XG SFX #000", "Car Pass"},
    {'M',  64,   0,  83, "XG SFX #000", "CarCrash"},
    {'M',  64,   0,  84, "XG SFX #000", "Siren"},
    {'M',  64,   0,  85, "XG SFX #000", "Train"},
    {'M',  64,   0,  86, "XG SFX #000", "JetPlane"},
    {'M',  64,   0,  87, "XG SFX #000", "Starship"},
    {'M',  64,   0,  88, "XG SFX #000", "Burst"},
    {'M',  64,   0,  89, "XG SFX #000", "Coaster"},
    {'M',  64,   0,  90, "XG SFX #000", "Submarin"},
    {'M',  64,   0,  91, "XG SFX #000", "+Cnectvty"},
    {'M',  64,   0,  92, "XG SFX #000", "+Mystery"},
    {'M',  64,   0,  93, "XG SFX #000", "+++Charging"},
    {'M',  64,   0,  96, "XG SFX #000", "Laugh"},
    {'M',  64,   0,  97, "XG SFX #000", "Scream"},
    {'M',  64,   0,  98, "XG SFX #000", "Punch"},
    {'M',  64,   0,  99, "XG SFX #000", "Heart"},
    {'M',  64,   0, 100, "XG SFX #000", "Footstep"},
    {'M',  64,   0, 101, "XG SFX #000", "Applaus2 **"},
    {'M',  64,   0, 112, "XG SFX #000", "MchinGun"},
    {'M',  64,   0, 113, "XG SFX #000", "LaserGun"},
    {'M',  64,   0, 114, "XG SFX #000", "Xplosion"},
    {'M',  64,   0, 115, "XG SFX #000", "Firework"},
    {'M',  64,   0, 116, "XG SFX #000", "+Fireball"},
    {'P',   1,   0,  28, "XG #001 SFX Kit 1", "Insects ****"},
    {'P',   1,   0,  29, "XG #001 SFX Kit 1", "Bacteria ****"},
    {'P',   1,   0,  36, "XG #001 SFX Kit 1", "Cutting Noise"},
    {'P',   1,   0,  37, "XG #001 SFX Kit 1", "Cutting Noise 2"},
    {'P',   1,   0,  38, "XG #001 SFX Kit 1", "Distorted Cutting Noise **"},
    {'P',   1,   0,  39, "XG #001 SFX Kit 1", "String Slap"},
    {'P',   1,   0,  40, "XG #001 SFX Kit 1", "Bass Slide **"},
    {'P',   1,   0,  41, "XG #001 SFX Kit 1", "Pick Scrape **"},
    {'P',   1,   0,  52, "XG #001 SFX Kit 1", "Flute Key Click"},
    {'P',   1,   0,  68, "XG #001 SFX Kit 1", "Shower"},
    {'P',   1,   0,  69, "XG #001 SFX Kit 1", "Thunder"},
    {'P',   1,   0,  70, "XG #001 SFX Kit 1", "Wind"},
    {'P',   1,   0,  71, "XG #001 SFX Kit 1", "Stream"},
    {'P',   1,   0,  72, "XG #001 SFX Kit 1", "Bubble"},
    {'P',   1,   0,  73, "XG #001 SFX Kit 1", "Feed **"},
    {'P',   1,   0,  74, "XG #001 SFX Kit 1", "Cave ****"},
    {'P',   1,   0,  84, "XG #001 SFX Kit 1", "Dog"},
    {'P',   1,   0,  85, "XG #001 SFX Kit 1", "Horse"},
    {'P',   1,   0,  86, "XG #001 SFX Kit 1", "Bird Tweet 2"},
    {'P',   1,   0,  87, "XG #001 SFX Kit 1", "Kitty **"},
    {'P',   1,   0,  88, "XG #001 SFX Kit 1", "Growl **"},
    {'P',   1,   0,  89, "XG #001 SFX Kit 1", "Haunted **"},
    {'P',   1,   0,  90, "XG #001 SFX Kit 1", "Ghost"},
    {'P',   1,   0,  91, "XG #001 SFX Kit 1", "Maou **"},
    {'P',   1,   1,  36, "XG #002 SFX Kit 2", "Phone Call **"},
    {'P',   1,   1,  37, "XG #002 SFX Kit 2", "Door Squeak"},
    {'P',   1,   1,  38, "XG #002 SFX Kit 2", "Door Slam"},
    {'P',   1,   1,  39, "XG #002 SFX Kit 2", "Scratch Cut"},
    {'P',   1,   1,  40, "XG #002 SFX Kit 2", "Scratch H 3"},
    {'P',   1,   1,  41, "XG #002 SFX Kit 2", "Wind Chime"},
    {'P',   1,   1,  42, "XG #002 SFX Kit 2", "Telephone Ring 2"},
    {'P',   1,   1,  43, "XG #002 SFX Kit 2", "+++Anthr"},
    {'P',   1,   1,  44, "XG #002 SFX Kit 2", "+++Turn Table"},
    {'P',   1,   1,  52, "XG #002 SFX Kit 2", "Car Engine Ignition"},
    {'P',   1,   1,  53, "XG #002 SFX Kit 2", "Car Tires Squeal"},
    {'P',   1,   1,  54, "XG #002 SFX Kit 2", "Car Passing"},
    {'P',   1,   1,  55, "XG #002 SFX Kit 2", "Car Crash"},
    {'P',   1,   1,  56, "XG #002 SFX Kit 2", "Siren"},
    {'P',   1,   1,  57, "XG #002 SFX Kit 2", "Train"},
    {'P',   1,   1,  58, "XG #002 SFX Kit 2", "Jet Plane"},
    {'P',   1,   1,  59, "XG #002 SFX Kit 2", "Starship"},
    {'P',   1,   1,  60, "XG #002 SFX Kit 2", "Burst"},
    {'P',   1,   1,  61, "XG #002 SFX Kit 2", "Roller Coaster"},
    {'P',   1,   1,  62, "XG #002 SFX Kit 2", "Submarine"},
    {'P',   1,   1,  63, "XG #002 SFX Kit 2", "Connectivity ****"},
    {'P',   1,   1,  64, "XG #002 SFX Kit 2", "Mystery ****"},
    {'P',   1,   1,  65, "XG #002 SFX Kit 2", "+++Charging"},
    {'P',   1,   1,  68, "XG #002 SFX Kit 2", "Laugh"},
    {'P',   1,   1,  69, "XG #002 SFX Kit 2", "Scream"},
    {'P',   1,   1,  70, "XG #002 SFX Kit 2", "Punch"},
    {'P',   1,   1,  71, "XG #002 SFX Kit 2", "Heart Beat"},
    {'P',   1,   1,  72, "XG #002 SFX Kit 2", "Foot Steps"},
    {'P',   1,   1,  73, "XG #002 SFX Kit 2", "Applause 2 **"},
    {'P',   1,   1,  84, "XG #002 SFX Kit 2", "Machine Gun"},
    {'P',   1,   1,  85, "XG #002 SFX Kit 2", "Laser Gun"},
    {'P',   1,   1,  86, "XG #002 SFX Kit 2", "Explosion"},
    {'P',   1,   1,  87, "XG #002 SFX Kit 2", "Firework"},
    {'P',   1,   1,  88, "XG #002 SFX Kit 2", "Fireball ****"},
    {'P',   1,  16,  36, "XG #017 Techno Kit K/S ****", "Heavy Techno Kick 1 ****"},
    {'P',   1,  16,  37, "XG #017 Techno Kit K/S ****", "Heavy Techno Kick 2 ****"},
    {'P',   1,  16,  38, "XG #017 Techno Kit K/S ****", "Psychedelic Kick ****"},
    {'P',   1,  16,  39, "XG #017 Techno Kit K/S ****", "Gate Tekno Kick ****"},
    {'P',   1,  16,  40, "XG #017 Techno Kit K/S ****", "Rap Kick ****"},
    {'P',   1,  16,  41, "XG #017 Techno Kit K/S ****", "Heavy Techno Kick 3 ****"},
    {'P',   1,  16,  42, "XG #017 Techno Kit K/S ****", "Heavy Techno Kick 4 ****"},
    {'P',   1,  16,  43, "XG #017 Techno Kit K/S ****", "Future Kick ****"},
    {'P',   1,  16,  44, "XG #017 Techno Kit K/S ****", "Asian Kick ****"},
    {'P',   1,  16,  45, "XG #017 Techno Kit K/S ****", "Imbalance Kick ****"},
    {'P',   1,  16,  46, "XG #017 Techno Kit K/S ****", "Justice Kick ****"},
    {'P',   1,  16,  47, "XG #017 Techno Kit K/S ****", "Minimal Kick ****"},
    {'P',   1,  16,  52, "XG #017 Techno Kit K/S ****", "Radio Snare ****"},
    {'P',   1,  16,  53, "XG #017 Techno Kit K/S ****", "Cold Dry Snare 1 ****"},
    {'P',   1,  16,  54, "XG #017 Techno Kit K/S ****", "Slap Snare ****"},
    {'P',   1,  16,  55, "XG #017 Techno Kit K/S ****", "Cold Dry Snare 2 ****"},
    {'P',   1,  16,  56, "XG #017 Techno Kit K/S ****", "Cold Dry Snare 3 ****"},
    {'P',   1,  16,  57, "XG #017 Techno Kit K/S ****", "Lo-Fi Metal Snare ****"},
    {'P',   1,  16,  68, "XG #017 Techno Kit K/S ****", "Hi Pitch Slap H ****"},
    {'P',   1,  16,  69, "XG #017 Techno Kit K/S ****", "Hi Pitch Slap L ****"},
    {'P',   1,  17,  36, "XG #018 Techno Kit Hi ****", "Hyper Tom H 1 ****"},
    {'P',   1,  17,  37, "XG #018 Techno Kit Hi ****", "Asian Tom H ****"},
    {'P',   1,  17,  38, "XG #018 Techno Kit Hi ****", "Lo-Fi Tom H ****"},
    {'P',   1,  17,  39, "XG #018 Techno Kit Hi ****", "Hyper Tom H 2 ****"},
    {'P',   1,  17,  40, "XG #018 Techno Kit Hi ****", "Flanged Tom H ****"},
    {'P',   1,  17,  41, "XG #018 Techno Kit Hi ****", "Minimal Tom H ****"},
    {'P',   1,  17,  42, "XG #018 Techno Kit Hi ****", "Vox Drum H ****"},
    {'P',   1,  17,  43, "XG #018 Techno Kit Hi ****", "Android Walk 1 H ****"},
    {'P',   1,  17,  44, "XG #018 Techno Kit Hi ****", "Android Walk 2 H ****"},
    {'P',   1,  17,  45, "XG #018 Techno Kit Hi ****", "Electro Blip H ****"},
    {'P',   1,  17,  46, "XG #018 Techno Kit Hi ****", "Wood Percussions H ****"},
    {'P',   1,  17,  47, "XG #018 Techno Kit Hi ****", "Wood Door Open H ****"},
    {'P',   1,  17,  48, "XG #018 Techno Kit Hi ****", "Reso Noise Burst H ****"},
    {'P',   1,  17,  49, "XG #018 Techno Kit Hi ****", "LFO Metal Attack H ****"},
    {'P',   1,  17,  50, "XG #018 Techno Kit Hi ****", "Steel Conga H ****"},
    {'P',   1,  17,  51, "XG #018 Techno Kit Hi ****", "Rate Down Snare H ****"},
    {'P',   1,  17,  52, "XG #018 Techno Kit Hi ****", "Pop Ambient H ****"},
    {'P',   1,  17,  53, "XG #018 Techno Kit Hi ****", "Tunnel Ambient H ****"},
    {'P',   1,  17,  54, "XG #018 Techno Kit Hi ****", "Vibraslap H ****"},
    {'P',   1,  17,  55, "XG #018 Techno Kit Hi ****", "Gun Shot Slap H ****"},
    {'P',   1,  17,  56, "XG #018 Techno Kit Hi ****", "Punch Snare H ****"},
    {'P',   1,  17,  57, "XG #018 Techno Kit Hi ****", "Bomb Snare H ****"},
    {'P',   1,  17,  58, "XG #018 Techno Kit Hi ****", "Space Tambourine H ****"},
    {'P',   1,  17,  59, "XG #018 Techno Kit Hi ****", "Ambient Cow Bell H ****"},
    {'P',   1,  17,  60, "XG #018 Techno Kit Hi ****", "Chink Hat H ****"},
    {'P',   1,  17,  61, "XG #018 Techno Kit Hi ****", "Coal Mine 1 H ****"},
    {'P',   1,  17,  62, "XG #018 Techno Kit Hi ****", "Coal Mine 2 H ****"},
    {'P',   1,  17,  63, "XG #018 Techno Kit Hi ****", "Hammer Hit 1 H ****"},
    {'P',   1,  17,  64, "XG #018 Techno Kit Hi ****", "Hammer Hit 2 H ****"},
    {'P',   1,  17,  65, "XG #018 Techno Kit Hi ****", "Hammer Hit 3 H ****"},
    {'P',   1,  17,  66, "XG #018 Techno Kit Hi ****", "Insensible Hah H ****"},
    {'P',   1,  17,  67, "XG #018 Techno Kit Hi ****", "Robot 1 H ****"},
    {'P',   1,  17,  68, "XG #018 Techno Kit Hi ****", "Insensible Fuh H ****"},
    {'P',   1,  17,  69, "XG #018 Techno Kit Hi ****", "Robot 2 H ****"},
    {'P',   1,  17,  70, "XG #018 Techno Kit Hi ****", "Rude Loop Cymbal H ****"},
    {'P',   1,  17,  71, "XG #018 Techno Kit Hi ****", "Noise Burst H ****"},
    {'P',   1,  17,  72, "XG #018 Techno Kit Hi ****", "Fizzer H ****"},
    {'P',   1,  17,  73, "XG #018 Techno Kit Hi ****", "Lo-Fi Shaker H ****"},
    {'P',   1,  17,  74, "XG #018 Techno Kit Hi ****", "Temple Gong H ****"},
    {'P',   1,  18,  36, "XG #019 Techno Kit Lo ****", "Hyper Tom L 1 ****"},
    {'P',   1,  18,  37, "XG #019 Techno Kit Lo ****", "Asian Tom L ****"},
    {'P',   1,  18,  38, "XG #019 Techno Kit Lo ****", "Lo-Fi Tom L ****"},
    {'P',   1,  18,  39, "XG #019 Techno Kit Lo ****", "Hyper Tom L 2 ****"},
    {'P',   1,  18,  40, "XG #019 Techno Kit Lo ****", "Flanged Tom L ****"},
    {'P',   1,  18,  41, "XG #019 Techno Kit Lo ****", "Minimal Tom L ****"},
    {'P',   1,  18,  42, "XG #019 Techno Kit Lo ****", "Vox Drum L ****"},
    {'P',   1,  18,  43, "XG #019 Techno Kit Lo ****", "Android Walk 1 L ****"},
    {'P',   1,  18,  44, "XG #019 Techno Kit Lo ****", "Android Walk 2 L ****"},
    {'P',   1,  18,  45, "XG #019 Techno Kit Lo ****", "Electro Blip L ****"},
    {'P',   1,  18,  46, "XG #019 Techno Kit Lo ****", "Wood Percussions L ****"},
    {'P',   1,  18,  47, "XG #019 Techno Kit Lo ****", "Wood Door Open L ****"},
    {'P',   1,  18,  48, "XG #019 Techno Kit Lo ****", "Reso Noise Burst L ****"},
    {'P',   1,  18,  49, "XG #019 Techno Kit Lo ****", "LFO Metal Attack L ****"},
    {'P',   1,  18,  50, "XG #019 Techno Kit Lo ****", "Steel Conga L ****"},
    {'P',   1,  18,  51, "XG #019 Techno Kit Lo ****", "Rate Down Snare L ****"},
    {'P',   1,  18,  52, "XG #019 Techno Kit Lo ****", "Pop Ambient L ****"},
    {'P',   1,  18,  53, "XG #019 Techno Kit Lo ****", "Tunnel Ambient L ****"},
    {'P',   1,  18,  54, "XG #019 Techno Kit Lo ****", "Vibraslap L ****"},
    {'P',   1,  18,  55, "XG #019 Techno Kit Lo ****", "Gun Shot Slap L ****"},
    {'P',   1,  18,  56, "XG #019 Techno Kit Lo ****", "Punch Snare L ****"},
    {'P',   1,  18,  57, "XG #019 Techno Kit Lo ****", "Bomb Snare L ****"},
    {'P',   1,  18,  58, "XG #019 Techno Kit Lo ****", "Space Tambourine L ****"},
    {'P',   1,  18,  59, "XG #019 Techno Kit Lo ****", "Ambient Cow Bell L ****"},
    {'P',   1,  18,  60, "XG #019 Techno Kit Lo ****", "Chink Hat L ****"},
    {'P',   1,  18,  61, "XG #019 Techno Kit Lo ****", "Coal Mine 1 L ****"},
    {'P',   1,  18,  62, "XG #019 Techno Kit Lo ****", "Coal Mine 2 L ****"},
    {'P',   1,  18,  63, "XG #019 Techno Kit Lo ****", "Hammer Hit 1 L ****"},
    {'P',   1,  18,  64, "XG #019 Techno Kit Lo ****", "Hammer Hit 2 L ****"},
    {'P',   1,  18,  65, "XG #019 Techno Kit Lo ****", "Hammer Hit 3 L ****"},
    {'P',   1,  18,  66, "XG #019 Techno Kit Lo ****", "Insensible Hah L ****"},
    {'P',   1,  18,  67, "XG #019 Techno Kit Lo ****", "Robot 1 L ****"},
    {'P',   1,  18,  68, "XG #019 Techno Kit Lo ****", "Insensible Fuh L ****"},
    {'P',   1,  18,  69, "XG #019 Techno Kit Lo ****", "Robot 2 L ****"},
    {'P',   1,  18,  70, "XG #019 Techno Kit Lo ****", "Rude Loop Cymbal L ****"},
    {'P',   1,  18,  71, "XG #019 Techno Kit Lo ****", "Noise Burst L ****"},
    {'P',   1,  18,  72, "XG #019 Techno Kit Lo ****", "Fizzer L ****"},
    {'P',   1,  18,  73, "XG #019 Techno Kit Lo ****", "Lo-Fi Shaker L ****"},
    {'P',   1,  18,  74, "XG #019 Techno Kit Lo ****", "Temple Gong L ****"},
    {'P',   1,  32,  36, "XG #033 Sakura Kit ****", "Dora ****"},
    {'P',   1,  32,  37, "XG #033 Sakura Kit ****", "Tsuzumi Pon ****"},
    {'P',   1,  32,  38, "XG #033 Sakura Kit ****", "Tsuzumi Pu ****"},
    {'P',   1,  32,  39, "XG #033 Sakura Kit ****", "Tsuzumi Ta ****"},
    {'P',   1,  32,  40, "XG #033 Sakura Kit ****", "Tsuzumi Chon ****"},
    {'P',   1,  32,  41, "XG #033 Sakura Kit ****", "Tsuzumi Tsu ****"},
    {'P',   1,  32,  42, "XG #033 Sakura Kit ****", "Shimetaiko Ten ****"},
    {'P',   1,  32,  43, "XG #033 Sakura Kit ****", "Shimetaiko Tsu ****"},
    {'P',   1,  32,  44, "XG #033 Sakura Kit ****", "Yagura Open ****"},
    {'P',   1,  32,  45, "XG #033 Sakura Kit ****", "Ohdaiko Rim ****"},
    {'P',   1,  32,  46, "XG #033 Sakura Kit ****", "Oriental Tambourine ****"},
    {'P',   1,  32,  47, "XG #033 Sakura Kit ****", "Oriental Drum 1 ****"},
    {'P',   1,  32,  48, "XG #033 Sakura Kit ****", "Oriental Drum 2 ****"},
    {'P',   1,  32,  49, "XG #033 Sakura Kit ****", "Oriental Drum 3 ****"},
    {'P',   1,  32,  50, "XG #033 Sakura Kit ****", "Oriental Rim 1 ****"},
    {'P',   1,  32,  51, "XG #033 Sakura Kit ****", "Oriental Rim 2 ****"},
    {'P',   1,  32,  52, "XG #033 Sakura Kit ****", "Oriental Rim 3 ****"},
    {'P',   1,  32,  53, "XG #033 Sakura Kit ****", "Oriental Rim 4 ****"},
    {'P',   1,  32,  54, "XG #033 Sakura Kit ****", "Oriental Metal Rim 1 ****"},
    {'P',   1,  32,  55, "XG #033 Sakura Kit ****", "Oriental Metal Rim 2 ****"},
    {'P',   1,  32,  56, "XG #033 Sakura Kit ****", "Oriental Metal Rim 3 ****"},
    {'P',   1,  32,  57, "XG #033 Sakura Kit ****", "Oriental Metal Rim 4 ****"},
    {'P',   1,  32,  58, "XG #033 Sakura Kit ****", "Oriental Metal Rim 5 ****"},
    {'P',   1,  33,  36, "XG #034 Small Latin Kit ****", "Latin Cymbal"},
    {'P',   1,  33,  37, "XG #034 Small Latin Kit ****", "Claves SL 1 ****"},
    {'P',   1,  33,  38, "XG #034 Small Latin Kit ****", "Claves SL 2 ****"},
    {'P',   1,  33,  39, "XG #034 Small Latin Kit ****", "Claves SL 3 ****"},
    {'P',   1,  33,  40, "XG #034 Small Latin Kit ****", "Claves SL 4 ****"},
    {'P',   1,  33,  41, "XG #034 Small Latin Kit ****", "Claves SL 5 ****"},
    {'P',   1,  33,  42, "XG #034 Small Latin Kit ****", "Muted Percussion SL 1 ****"},
    {'P',   1,  33,  43, "XG #034 Small Latin Kit ****", "Muted Percussion SL 2 ****"},
    {'P',   1,  33,  44, "XG #034 Small Latin Kit ****", "Muted Percussion SL 3 ****"},
    {'P',   1,  33,  45, "XG #034 Small Latin Kit ****", "Muted Percussion SL 4 ****"},
    {'P',   1,  33,  46, "XG #034 Small Latin Kit ****", "Pandiero ****"},
    {'P',   1,  33,  47, "XG #034 Small Latin Kit ****", "Surdo Mute SL 1 ****"},
    {'P',   1,  33,  48, "XG #034 Small Latin Kit ****", "Surdo Mute SL 2 ****"},
    {'P',   1,  33,  49, "XG #034 Small Latin Kit ****", "Surdo Open SL ****"},
    {'P',   1,  33,  50, "XG #034 Small Latin Kit ****", "Surdo Rim SL ****"},
    {'P',   1,  33,  51, "XG #034 Small Latin Kit ****", "Tambourim Mute ****"},
    {'P',   1,  33,  52, "XG #034 Small Latin Kit ****", "Tambourim Open ****"},
    {'P',   1,  33,  53, "XG #034 Small Latin Kit ****", "TimbaleL Drum ****"},
    {'P',   1,  33,  54, "XG #034 Small Latin Kit ****", "TimbaleH Drum ****"},
    {'P',   1,  33,  55, "XG #034 Small Latin Kit ****", "TimbaleL Rim ****"},
    {'P',   1,  33,  56, "XG #034 Small Latin Kit ****", "TimbaleH Rim ****"},
    {'P',   1,  33,  57, "XG #034 Small Latin Kit ****", "Timbale Paila 1 ****"},
    {'P',   1,  33,  58, "XG #034 Small Latin Kit ****", "Timbale Paila 2 ****"},
    {'P',   1,  34,  36, "XG #035 +++China Kit", "++Bangu"},
    {'P',   1,  34,  37, "XG #035 +++China Kit", "++Dagu Mute"},
    {'P',   1,  34,  38, "XG #035 +++China Kit", "++Dagu Heavy"},
    {'P',   1,  34,  39, "XG #035 +++China Kit", "++Paigu High"},
    {'P',   1,  34,  40, "XG #035 +++China Kit", "++Paigu Middle"},
    {'P',   1,  34,  41, "XG #035 +++China Kit", "++Paigu Low"},
    {'P',   1,  34,  52, "XG #035 +++China Kit", "++Zhongcha Mute"},
    {'P',   1,  34,  53, "XG #035 +++China Kit", "++Zhongcha Open"},
    {'P',   1,  34,  54, "XG #035 +++China Kit", "++Zhongluo Mute"},
    {'P',   1,  34,  55, "XG #035 +++China Kit", "++Zhongluo Open"},
    {'P',   1,  34,  56, "XG #035 +++China Kit", "++Xiaoluo Open"},
    {'P',   1,  34,  57, "XG #035 +++China Kit", "++Xizocha Mute"},
    {'P',   1,  34,  58, "XG #035 +++China Kit", "++Ziaocha Open"},
    {'P',   1,  34,  68, "XG #035 +++China Kit", "++Banzi"},
    {'P',   1,  34,  69, "XG #035 +++China Kit", "++Muyu High"},
    {'P',   1,  34,  70, "XG #035 +++China Kit", "++Muyu Mid-High"},
    {'P',   1,  34,  71, "XG #035 +++China Kit", "++Muyu Mid"},
    {'P',   1,  34,  72, "XG #035 +++China Kit", "++Muyu Mid-Low"},
    {'P',   1,  34,  73, "XG #035 +++China Kit", "++Muyu Low"},
    {'P',   0,   0,  30, "XG #001 StandKit MU100 Native ****", "Castanet# ****"},
    {'P',   0,   0,  31, "XG #001 StandKit MU100 Native ****", "Snare Soft# ****"},
    {'P',   0,   0,  34, "XG #001 StandKit MU100 Native ****", "Open Rim Shot# ****"},
    {'P',   0,   0,  37, "XG #001 StandKit MU100 Native ****", "Side Stick# ****"},
    {'P',   0,   0,  38, "XG #001 StandKit MU100 Native ****", "Snare# ****"},
    {'P',   0,   0,  40, "XG #001 StandKit MU100 Native ****", "Snare Tight# ****"},
    {'P',   0,   0,  42, "XG #001 StandKit MU100 Native ****", "Hi-Hat Closed# ****"},
    {'P',   0,   0,  44, "XG #001 StandKit MU100 Native ****", "Hi-Hat Pedal# ****"},
    {'P',   0,   0,  46, "XG #001 StandKit MU100 Native ****", "Hi-Hat Open# ****"},
    {'P',   0,   0,  49, "XG #001 StandKit MU100 Native ****", "Crash Cymbal 1# ****"},
    {'P',   0,   0,  51, "XG #001 StandKit MU100 Native ****", "Ride Cymbal 1# ****"},
    {'P',   0,   0,  52, "XG #001 StandKit MU100 Native ****", "Chinese Cymbal# ****"},
    {'P',   0,   0,  53, "XG #001 StandKit MU100 Native ****", "Ride Cymbal Cup# ****"},
    {'P',   0,   0,  56, "XG #001 StandKit MU100 Native ****", "Cowbell# ****"},
    {'P',   0,   0,  57, "XG #001 StandKit MU100 Native ****", "Crash Cymbal 2# ****"},
    {'P',   0,   0,  59, "XG #001 StandKit MU100 Native ****", "Ride Cymbal 2# ****"},
    {'P',   0,   0,  62, "XG #001 StandKit MU100 Native ****", "Conga H Mute# ****"},
    {'P',   0,   0,  63, "XG #001 StandKit MU100 Native ****", "Conga H Open# ****"},
    {'P',   0,   0,  64, "XG #001 StandKit MU100 Native ****", "Conga L# ****"},
    {'P',   0,   1,  29, "XG #002 StndKit2", "Snare Roll 2"},
    {'P',   0,   1,  31, "XG #002 StndKit2", "Snare Soft 2"},
    {'P',   0,   1,  34, "XG #002 StndKit2", "Open Rim Shot H Short"},
    {'P',   0,   1,  35, "XG #002 StndKit2", "Kick Tight Short"},
    {'P',   0,   1,  36, "XG #002 StndKit2", "Kick Short"},
    {'P',   0,   1,  38, "XG #002 StndKit2", "Snare Short"},
    {'P',   0,   1,  40, "XG #002 StndKit2", "Snare Tight H"},
    {'P',   0,   2,  13, "XG #003 Dry Kit ***", "Surdo Mute V ***"},
    {'P',   0,   2,  14, "XG #003 Dry Kit ***", "Surdo Open V ***"},
    {'P',   0,   2,  25, "XG #003 Dry Kit ***", "Brush Tap V ***"},
    {'P',   0,   2,  26, "XG #003 Dry Kit ***", "Brush Swirl V ***"},
    {'P',   0,   2,  27, "XG #003 Dry Kit ***", "Brush Slap V ***"},
    {'P',   0,   2,  28, "XG #003 Dry Kit ***", "Brush Tap Swirl V ***"},
    {'P',   0,   2,  29, "XG #003 Dry Kit ***", "Snare Roll V ***"},
    {'P',   0,   2,  31, "XG #003 Dry Kit ***", "Snare Dry Soft ***"},
    {'P',   0,   2,  33, "XG #003 Dry Kit ***", "Kick Dry Soft ***"},
    {'P',   0,   2,  34, "XG #003 Dry Kit ***", "Open Rim Shot Dry V ***"},
    {'P',   0,   2,  35, "XG #003 Dry Kit ***", "Kick Dry Tight ***"},
    {'P',   0,   2,  36, "XG #003 Dry Kit ***", "Kick Dry Mute ***"},
    {'P',   0,   2,  37, "XG #003 Dry Kit ***", "Side Stick Dry ***"},
    {'P',   0,   2,  38, "XG #003 Dry Kit ***", "Snare Dry ***"},
    {'P',   0,   2,  40, "XG #003 Dry Kit ***", "Snare Dry Mute ***"},
    {'P',   0,   2,  41, "XG #003 Dry Kit ***", "Floor Tom L Short ***"},
    {'P',   0,   2,  43, "XG #003 Dry Kit ***", "Floor Tom H Short ***"},
    {'P',   0,   2,  45, "XG #003 Dry Kit ***", "Low Tom Short ***"},
    {'P',   0,   2,  47, "XG #003 Dry Kit ***", "Mid Tom L Short ***"},
    {'P',   0,   2,  48, "XG #003 Dry Kit ***", "Mid Tom H Short ***"},
    {'P',   0,   2,  49, "XG #003 Dry Kit ***", "Crash Cymbal 1 V ***"},
    {'P',   0,   2,  50, "XG #003 Dry Kit ***", "High Tom Short ***"},
    {'P',   0,   2,  51, "XG #003 Dry Kit ***", "Ride Cymbal 1 V ***"},
    {'P',   0,   2,  52, "XG #003 Dry Kit ***", "Chinese Cymbal V ***"},
    {'P',   0,   2,  55, "XG #003 Dry Kit ***", "Splash Cymbal V ***"},
    {'P',   0,   2,  57, "XG #003 Dry Kit ***", "Crash Cymbal 2 V ***"},
    {'P',   0,   2,  59, "XG #003 Dry Kit ***", "Ride Cymbal 2V ***"},
    {'P',   0,   2,  60, "XG #003 Dry Kit ***", "Bongo H V ***"},
    {'P',   0,   2,  61, "XG #003 Dry Kit ***", "Bongo L V ***"},
    {'P',   0,   2,  62, "XG #003 Dry Kit ***", "Conga H Mute V ***"},
    {'P',   0,   2,  63, "XG #003 Dry Kit ***", "Conga H Open V ***"},
    {'P',   0,   2,  64, "XG #003 Dry Kit ***", "Conga L V ***"},
    {'P',   0,   2,  65, "XG #003 Dry Kit ***", "Timbale H V ***"},
    {'P',   0,   2,  66, "XG #003 Dry Kit ***", "Timbale L V ***"},
    {'P',   0,   2,  67, "XG #003 Dry Kit ***", "Agogo H V ***"},
    {'P',   0,   2,  68, "XG #003 Dry Kit ***", "Agogo L V ***"},
    {'P',   0,   2,  71, "XG #003 Dry Kit ***", "Samba Whistle H V ***"},
    {'P',   0,   2,  72, "XG #003 Dry Kit ***", "Samba Whistle L V ***"},
    {'P',   0,   2,  78, "XG #003 Dry Kit ***", "Cuica Mute V ***"},
    {'P',   0,   2,  79, "XG #003 Dry Kit ***", "Cuica Open V ***"},
    {'P',   0,   3,  13, "XG #004 BriteKit ***", "Surdo Mute B ***"},
    {'P',   0,   3,  14, "XG #004 BriteKit ***", "Surdo Open B ***"},
    {'P',   0,   3,  15, "XG #004 BriteKit ***", "Hi Q B ***"},
    {'P',   0,   3,  16, "XG #004 BriteKit ***", "Whip Slap B ***"},
    {'P',   0,   3,  17, "XG #004 BriteKit ***", "Scratch H B ***"},
    {'P',   0,   3,  18, "XG #004 BriteKit ***", "Scratch L B ***"},
    {'P',   0,   3,  19, "XG #004 BriteKit ***", "Finger Snap B ***"},
    {'P',   0,   3,  20, "XG #004 BriteKit ***", "Click Noise B ***"},
    {'P',   0,   3,  21, "XG #004 BriteKit ***", "Metronome Click B ***"},
    {'P',   0,   3,  22, "XG #004 BriteKit ***", "Metronome Bell B ***"},
    {'P',   0,   3,  23, "XG #004 BriteKit ***", "Seq Click L B ***"},
    {'P',   0,   3,  24, "XG #004 BriteKit ***", "Seq Click H B ***"},
    {'P',   0,   3,  25, "XG #004 BriteKit ***", "Brush Tap B ***"},
    {'P',   0,   3,  26, "XG #004 BriteKit ***", "Brush Swirl B ***"},
    {'P',   0,   3,  27, "XG #004 BriteKit ***", "Brush Slap B ***"},
    {'P',   0,   3,  28, "XG #004 BriteKit ***", "Brush Tap Swirl B ***"},
    {'P',   0,   3,  29, "XG #004 BriteKit ***", "Snare Roll B ***"},
    {'P',   0,   3,  30, "XG #004 BriteKit ***", "Castanet B ***"},
    {'P',   0,   3,  31, "XG #004 BriteKit ***", "Snare Soft B ***"},
    {'P',   0,   3,  32, "XG #004 BriteKit ***", "Sticks B ***"},
    {'P',   0,   3,  33, "XG #004 BriteKit ***", "Kick Soft B ***"},
    {'P',   0,   3,  34, "XG #004 BriteKit ***", "Open Rim Shot B ***"},
    {'P',   0,   3,  35, "XG #004 BriteKit ***", "Kick Tight B ***"},
    {'P',   0,   3,  36, "XG #004 BriteKit ***", "Kick B ***"},
    {'P',   0,   3,  37, "XG #004 BriteKit ***", "Side Stick B ***"},
    {'P',   0,   3,  38, "XG #004 BriteKit ***", "Snare B ***"},
    {'P',   0,   3,  39, "XG #004 BriteKit ***", "Hand Clap B ***"},
    {'P',   0,   3,  40, "XG #004 BriteKit ***", "Snare Tight B ***"},
    {'P',   0,   3,  41, "XG #004 BriteKit ***", "Floor Tom L B ***"},
    {'P',   0,   3,  42, "XG #004 BriteKit ***", "Hi-Hat Closed B ***"},
    {'P',   0,   3,  43, "XG #004 BriteKit ***", "Floor Tom H B ***"},
    {'P',   0,   3,  44, "XG #004 BriteKit ***", "Hi-Hat Pedal B ***"},
    {'P',   0,   3,  45, "XG #004 BriteKit ***", "Low Tom B ***"},
    {'P',   0,   3,  46, "XG #004 BriteKit ***", "Hi-Hat Open B ***"},
    {'P',   0,   3,  47, "XG #004 BriteKit ***", "Mid Tom L B ***"},
    {'P',   0,   3,  48, "XG #004 BriteKit ***", "Mid Tom H B ***"},
    {'P',   0,   3,  49, "XG #004 BriteKit ***", "Crash Cymbal 1 B ***"},
    {'P',   0,   3,  50, "XG #004 BriteKit ***", "High Tom B ***"},
    {'P',   0,   3,  51, "XG #004 BriteKit ***", "Ride Cymbal 1 B ***"},
    {'P',   0,   3,  52, "XG #004 BriteKit ***", "Chinese Cymbal B ***"},
    {'P',   0,   3,  53, "XG #004 BriteKit ***", "Ride Cymbal Cup Short ***"},
    {'P',   0,   3,  54, "XG #004 BriteKit ***", "Tambourine B ***"},
    {'P',   0,   3,  55, "XG #004 BriteKit ***", "Splash Cymbal B ***"},
    {'P',   0,   3,  56, "XG #004 BriteKit ***", "Cowbell B ***"},
    {'P',   0,   3,  57, "XG #004 BriteKit ***", "Crash Cymbal 2 B ***"},
    {'P',   0,   3,  58, "XG #004 BriteKit ***", "Vibraslap B ***"},
    {'P',   0,   3,  59, "XG #004 BriteKit ***", "Ride Cymbal 2 B ***"},
    {'P',   0,   3,  60, "XG #004 BriteKit ***", "Bongo H B ***"},
    {'P',   0,   3,  61, "XG #004 BriteKit ***", "Bongo L B ***"},
    {'P',   0,   3,  62, "XG #004 BriteKit ***", "Conga H Mute B ***"},
    {'P',   0,   3,  63, "XG #004 BriteKit ***", "Conga H Open B ***"},
    {'P',   0,   3,  64, "XG #004 BriteKit ***", "Conga L B ***"},
    {'P',   0,   3,  65, "XG #004 BriteKit ***", "Timbale H B ***"},
    {'P',   0,   3,  66, "XG #004 BriteKit ***", "Timbale L B ***"},
    {'P',   0,   3,  67, "XG #004 BriteKit ***", "Agogo H B ***"},
    {'P',   0,   3,  68, "XG #004 BriteKit ***", "Agogo L B ***"},
    {'P',   0,   3,  69, "XG #004 BriteKit ***", "Cabasa B ***"},
    {'P',   0,   3,  70, "XG #004 BriteKit ***", "Maracas B ***"},
    {'P',   0,   3,  71, "XG #004 BriteKit ***", "Samba Whistle H B ***"},
    {'P',   0,   3,  72, "XG #004 BriteKit ***", "Samba Whistle L B ***"},
    {'P',   0,   3,  73, "XG #004 BriteKit ***", "Guiro Short B ***"},
    {'P',   0,   3,  74, "XG #004 BriteKit ***", "Guiro Long B ***"},
    {'P',   0,   3,  75, "XG #004 BriteKit ***", "Claves B ***"},
    {'P',   0,   3,  76, "XG #004 BriteKit ***", "Wood Block H B ***"},
    {'P',   0,   3,  77, "XG #004 BriteKit ***", "Wood Block L B ***"},
    {'P',   0,   3,  78, "XG #004 BriteKit ***", "Cuica Mute B ***"},
    {'P',   0,   3,  79, "XG #004 BriteKit ***", "Cuica Open B ***"},
    {'P',   0,   3,  80, "XG #004 BriteKit ***", "Triangle Mute B ***"},
    {'P',   0,   3,  81, "XG #004 BriteKit ***", "Triangle Open B ***"},
    {'P',   0,   3,  82, "XG #004 BriteKit ***", "Shaker B ***"},
    {'P',   0,   3,  83, "XG #004 BriteKit ***", "Jingle Bells B ***"},
    {'P',   0,   3,  84, "XG #004 BriteKit ***", "Bell Tree B ***"},
    {'P',   0,   4,  13, "XG #005 Skim Kit ****", "Surdo Mute V ***"},
    {'P',   0,   4,  14, "XG #005 Skim Kit ****", "Surdo Open V ***"},
    {'P',   0,   4,  25, "XG #005 Skim Kit ****", "Brush Tap V ***"},
    {'P',   0,   4,  26, "XG #005 Skim Kit ****", "Brush Swirl V ***"},
    {'P',   0,   4,  27, "XG #005 Skim Kit ****", "Brush Slap V ***"},
    {'P',   0,   4,  28, "XG #005 Skim Kit ****", "Brush Tap Swirl L ****"},
    {'P',   0,   4,  29, "XG #005 Skim Kit ****", "Snare Roll V ***"},
    {'P',   0,   4,  30, "XG #005 Skim Kit ****", "Castanet Sk ****"},
    {'P',   0,   4,  31, "XG #005 Skim Kit ****", "Snare Dry Q ****"},
    {'P',   0,   4,  33, "XG #005 Skim Kit ****", "Kick Dry Soft HPF ****"},
    {'P',   0,   4,  34, "XG #005 Skim Kit ****", "Open Rim Shot Dry HPF ****"},
    {'P',   0,   4,  35, "XG #005 Skim Kit ****", "Kick Dry Tight Q ****"},
    {'P',   0,   4,  36, "XG #005 Skim Kit ****", "Kick Dry Mute HPF ****"},
    {'P',   0,   4,  37, "XG #005 Skim Kit ****", "Side Stick Dry Q ****"},
    {'P',   0,   4,  38, "XG #005 Skim Kit ****", "Snare Dry H ****"},
    {'P',   0,   4,  39, "XG #005 Skim Kit ****", "Hand Clap Sk ****"},
    {'P',   0,   4,  40, "XG #005 Skim Kit ****", "Snare Dry Mute Q ****"},
    {'P',   0,   4,  41, "XG #005 Skim Kit ****", "Floor Tom L Short Sk ****"},
    {'P',   0,   4,  42, "XG #005 Skim Kit ****", "Hi-Hat Closed H ****"},
    {'P',   0,   4,  43, "XG #005 Skim Kit ****", "Floor Tom H Short Sk ****"},
    {'P',   0,   4,  44, "XG #005 Skim Kit ****", "Hi-Hat Pedal H ****"},
    {'P',   0,   4,  45, "XG #005 Skim Kit ****", "Low Tom Short Sk ****"},
    {'P',   0,   4,  46, "XG #005 Skim Kit ****", "Hi-Hat Open H ****"},
    {'P',   0,   4,  47, "XG #005 Skim Kit ****", "Mid Tom L Short Sk ****"},
    {'P',   0,   4,  48, "XG #005 Skim Kit ****", "Mid Tom H Short Sk ****"},
    {'P',   0,   4,  49, "XG #005 Skim Kit ****", "Crash Cymbal 1 Q ****"},
    {'P',   0,   4,  50, "XG #005 Skim Kit ****", "High Tom Short Sk ****"},
    {'P',   0,   4,  51, "XG #005 Skim Kit ****", "Ride Cymbal 1 Sk ****"},
    {'P',   0,   4,  52, "XG #005 Skim Kit ****", "Chinese Cymbal HPF ****"},
    {'P',   0,   4,  53, "XG #005 Skim Kit ****", "Ride Cup Q ****"},
    {'P',   0,   4,  54, "XG #005 Skim Kit ****", "Tambourine Q ****"},
    {'P',   0,   4,  55, "XG #005 Skim Kit ****", "Splash Cymbal H ****"},
    {'P',   0,   4,  56, "XG #005 Skim Kit ****", "Cowbell HPF ****"},
    {'P',   0,   4,  57, "XG #005 Skim Kit ****", "Crash Cymbal 2 Dark ****"},
    {'P',   0,   4,  59, "XG #005 Skim Kit ****", "Ride Cymbal 2 Q ****"},
    {'P',   0,   4,  60, "XG #005 Skim Kit ****", "Bongo H HPF ****"},
    {'P',   0,   4,  61, "XG #005 Skim Kit ****", "Bongo L HPF ****"},
    {'P',   0,   4,  62, "XG #005 Skim Kit ****", "Conga H Mute HPF ****"},
    {'P',   0,   4,  63, "XG #005 Skim Kit ****", "Conga H Open HPF ****"},
    {'P',   0,   4,  64, "XG #005 Skim Kit ****", "Conga L HPF ****"},
    {'P',   0,   4,  65, "XG #005 Skim Kit ****", "Timbale H HPF ****"},
    {'P',   0,   4,  66, "XG #005 Skim Kit ****", "Timbale L HPF ****"},
    {'P',   0,   4,  67, "XG #005 Skim Kit ****", "Agogo H HPF ****"},
    {'P',   0,   4,  68, "XG #005 Skim Kit ****", "Agogo L HPF ****"},
    {'P',   0,   4,  69, "XG #005 Skim Kit ****", "Cabasa BPF ****"},
    {'P',   0,   4,  70, "XG #005 Skim Kit ****", "Maracas L ****"},
    {'P',   0,   4,  71, "XG #005 Skim Kit ****", "Samba Whistle H BPF ****"},
    {'P',   0,   4,  72, "XG #005 Skim Kit ****", "Samba Whistle L BPF ****"},
    {'P',   0,   4,  79, "XG #005 Skim Kit ****", "Cuica Open H ****"},
    {'P',   0,   5,  13, "XG #006 Slim Kit ****", "Surdo Mute V ***"},
    {'P',   0,   5,  14, "XG #006 Slim Kit ****", "Surdo Open V ***"},
    {'P',   0,   5,  25, "XG #006 Slim Kit ****", "Brush Tap V ***"},
    {'P',   0,   5,  26, "XG #006 Slim Kit ****", "Brush Swirl V ***"},
    {'P',   0,   5,  27, "XG #006 Slim Kit ****", "Brush Slap V ***"},
    {'P',   0,   5,  28, "XG #006 Slim Kit ****", "Brush Tap Swirl L ****"},
    {'P',   0,   5,  29, "XG #006 Slim Kit ****", "Snare Roll V ***"},
    {'P',   0,   5,  30, "XG #006 Slim Kit ****", "Castanet H ****"},
    {'P',   0,   5,  31, "XG #006 Slim Kit ****", "Snare Rough Q ****"},
    {'P',   0,   5,  32, "XG #006 Slim Kit ****", "Sticks Q ****"},
    {'P',   0,   5,  33, "XG #006 Slim Kit ****", "Kick Soft Dark ****"},
    {'P',   0,   5,  34, "XG #006 Slim Kit ****", "Open Rim Shot Sl ****"},
    {'P',   0,   5,  35, "XG #006 Slim Kit ****", "Kick Tight Short L ****"},
    {'P',   0,   5,  36, "XG #006 Slim Kit ****", "Kick Short Dark ****"},
    {'P',   0,   5,  37, "XG #006 Slim Kit ****", "Side Stick Q ****"},
    {'P',   0,   5,  38, "XG #006 Slim Kit ****", "Snare Short HPF ****"},
    {'P',   0,   5,  39, "XG #006 Slim Kit ****", "Hand Clap Dark ****"},
    {'P',   0,   5,  40, "XG #006 Slim Kit ****", "Snare Tight Mute ****"},
    {'P',   0,   5,  41, "XG #006 Slim Kit ****", "Floor Tom L Tight ****"},
    {'P',   0,   5,  42, "XG #006 Slim Kit ****", "Hi-Hat Closed L Q ****"},
    {'P',   0,   5,  43, "XG #006 Slim Kit ****", "Floor Tom H Tight ****"},
    {'P',   0,   5,  44, "XG #006 Slim Kit ****", "HiHat Pedal Q ****"},
    {'P',   0,   5,  45, "XG #006 Slim Kit ****", "Low Tom Tight ****"},
    {'P',   0,   5,  46, "XG #006 Slim Kit ****", "Hi-Hat Open Q ****"},
    {'P',   0,   5,  47, "XG #006 Slim Kit ****", "Mid Tom L Tight ****"},
    {'P',   0,   5,  48, "XG #006 Slim Kit ****", "Mid Tom H Tight ****"},
    {'P',   0,   5,  49, "XG #006 Slim Kit ****", "Crash Cymbal 1 L ****"},
    {'P',   0,   5,  50, "XG #006 Slim Kit ****", "High Tom Tight ****"},
    {'P',   0,   5,  51, "XG #006 Slim Kit ****", "Ride Cymbal 1 L ****"},
    {'P',   0,   5,  52, "XG #006 Slim Kit ****", "Chinese Cymbal L ****"},
    {'P',   0,   5,  53, "XG #006 Slim Kit ****", "Ride Cymbal Cup L ****"},
    {'P',   0,   5,  55, "XG #006 Slim Kit ****", "Splash Cymbal L Short ****"},
    {'P',   0,   5,  56, "XG #006 Slim Kit ****", "Cowbell Dark ****"},
    {'P',   0,   5,  57, "XG #006 Slim Kit ****", "Crash Cymbal 2 L ****"},
    {'P',   0,   5,  59, "XG #006 Slim Kit ****", "Ride Cymbal 2 L ****"},
    {'P',   0,   5,  60, "XG #006 Slim Kit ****", "Bongo H V ***"},
    {'P',   0,   5,  61, "XG #006 Slim Kit ****", "Bongo L V ***"},
    {'P',   0,   5,  62, "XG #006 Slim Kit ****", "Conga H Mute V ***"},
    {'P',   0,   5,  63, "XG #006 Slim Kit ****", "Conga H Open V ***"},
    {'P',   0,   5,  64, "XG #006 Slim Kit ****", "Conga L H ****"},
    {'P',   0,   5,  65, "XG #006 Slim Kit ****", "Timbale H V ***"},
    {'P',   0,   5,  66, "XG #006 Slim Kit ****", "Timbale L V ***"},
    {'P',   0,   5,  67, "XG #006 Slim Kit ****", "Agogo H V ***"},
    {'P',   0,   5,  68, "XG #006 Slim Kit ****", "Agogo L V ***"},
    {'P',   0,   5,  70, "XG #006 Slim Kit ****", "Maracas Q ****"},
    {'P',   0,   5,  71, "XG #006 Slim Kit ****", "Samba Whistle H V ***"},
    {'P',   0,   5,  72, "XG #006 Slim Kit ****", "Samba Whistle L V ***"},
    {'P',   0,   5,  79, "XG #006 Slim Kit ****", "Cuica Open H ****"},
    {'P',   0,   6,  13, "XG #007 RogueKit ****", "Surdo Mute V ***"},
    {'P',   0,   6,  14, "XG #007 RogueKit ****", "Surdo Open V ***"},
    {'P',   0,   6,  25, "XG #007 RogueKit ****", "Brush Tap V ***"},
    {'P',   0,   6,  26, "XG #007 RogueKit ****", "Brush Swirl V ***"},
    {'P',   0,   6,  27, "XG #007 RogueKit ****", "Brush Slap V ***"},
    {'P',   0,   6,  28, "XG #007 RogueKit ****", "Brush Tap Swirl L ****"},
    {'P',   0,   6,  29, "XG #007 RogueKit ****", "Snare Roll V ***"},
    {'P',   0,   6,  30, "XG #007 RogueKit ****", "Castanet H ****"},
    {'P',   0,   6,  31, "XG #007 RogueKit ****", "Snare Brass Soft ****"},
    {'P',   0,   6,  33, "XG #007 RogueKit ****", "Kick Soft 2 ****"},
    {'P',   0,   6,  34, "XG #007 RogueKit ****", "Open Rim Shot 2 ****"},
    {'P',   0,   6,  35, "XG #007 RogueKit ****", "Kick Dark 2 ****"},
    {'P',   0,   6,  36, "XG #007 RogueKit ****", "Kick Tight 4 ****"},
    {'P',   0,   6,  37, "XG #007 RogueKit ****", "Side Stick Tight ****"},
    {'P',   0,   6,  38, "XG #007 RogueKit ****", "Snare Snappy Short ****"},
    {'P',   0,   6,  40, "XG #007 RogueKit ****", "Snare Brass Hard ****"},
    {'P',   0,   6,  41, "XG #007 RogueKit ****", "Floor Tom L Tight ****"},
    {'P',   0,   6,  42, "XG #007 RogueKit ****", "Hi-Hat Closed 3 ****"},
    {'P',   0,   6,  43, "XG #007 RogueKit ****", "Floor Tom H Tight ****"},
    {'P',   0,   6,  44, "XG #007 RogueKit ****", "HiHat Pedal 3 ****"},
    {'P',   0,   6,  45, "XG #007 RogueKit ****", "Low Tom Tight ****"},
    {'P',   0,   6,  46, "XG #007 RogueKit ****", "Hi-Hat Open 3 ****"},
    {'P',   0,   6,  47, "XG #007 RogueKit ****", "Mid Tom L Tight ****"},
    {'P',   0,   6,  48, "XG #007 RogueKit ****", "Mid Tom H Tight ****"},
    {'P',   0,   6,  49, "XG #007 RogueKit ****", "Crash Cymbal 3 ****"},
    {'P',   0,   6,  50, "XG #007 RogueKit ****", "High Tom Tight ****"},
    {'P',   0,   6,  51, "XG #007 RogueKit ****", "Ride Cymbal 3 ****"},
    {'P',   0,   6,  52, "XG #007 RogueKit ****", "Chinese Cymbal 2 ****"},
    {'P',   0,   6,  53, "XG #007 RogueKit ****", "Ride Cymbal Cup 2 ****"},
    {'P',   0,   6,  55, "XG #007 RogueKit ****", "Splash Cymbal V ***"},
    {'P',   0,   6,  56, "XG #007 RogueKit ****", "Cowbell Mid ****"},
    {'P',   0,   6,  57, "XG #007 RogueKit ****", "Crash Cymbal 4 ****"},
    {'P',   0,   6,  59, "XG #007 RogueKit ****", "Ride Cymbal 4 Q ****"},
    {'P',   0,   6,  60, "XG #007 RogueKit ****", "Bongo H V ***"},
    {'P',   0,   6,  61, "XG #007 RogueKit ****", "Bongo L V ***"},
    {'P',   0,   6,  62, "XG #007 RogueKit ****", "Conga H Mute 2 ****"},
    {'P',   0,   6,  63, "XG #007 RogueKit ****", "Conga H Open 2 ****"},
    {'P',   0,   6,  64, "XG #007 RogueKit ****", "Conga L 2 ****"},
    {'P',   0,   6,  65, "XG #007 RogueKit ****", "Timbale H V ***"},
    {'P',   0,   6,  66, "XG #007 RogueKit ****", "Timbale L V ***"},
    {'P',   0,   6,  67, "XG #007 RogueKit ****", "Agogo H V ***"},
    {'P',   0,   6,  68, "XG #007 RogueKit ****", "Agogo L V ***"},
    {'P',   0,   6,  70, "XG #007 RogueKit ****", "Maracas Q ****"},
    {'P',   0,   6,  71, "XG #007 RogueKit ****", "Samba Whistle H V ***"},
    {'P',   0,   6,  72, "XG #007 RogueKit ****", "Samba Whistle L V ***"},
    {'P',   0,   6,  79, "XG #007 RogueKit ****", "Cuica Open H ****"},
    {'P',   0,   7,  13, "XG #008 Hob Kit ****", "Surdo Mute V ***"},
    {'P',   0,   7,  14, "XG #008 Hob Kit ****", "Surdo Open V ***"},
    {'P',   0,   7,  25, "XG #008 Hob Kit ****", "Brush Tap V ***"},
    {'P',   0,   7,  26, "XG #008 Hob Kit ****", "Brush Swirl V ***"},
    {'P',   0,   7,  27, "XG #008 Hob Kit ****", "Brush Slap V ***"},
    {'P',   0,   7,  28, "XG #008 Hob Kit ****", "Brush Tap Swirl L ****"},
    {'P',   0,   7,  29, "XG #008 Hob Kit ****", "Snare Roll V ***"},
    {'P',   0,   7,  30, "XG #008 Hob Kit ****", "Castanet H ****"},
    {'P',   0,   7,  31, "XG #008 Hob Kit ****", "Snare Brass Soft H ****"},
    {'P',   0,   7,  33, "XG #008 Hob Kit ****", "Kick Soft 2 H ****"},
    {'P',   0,   7,  34, "XG #008 Hob Kit ****", "Open Rim Shot H Short 2 ****"},
    {'P',   0,   7,  35, "XG #008 Hob Kit ****", "Kick Dark 2 H ****"},
    {'P',   0,   7,  36, "XG #008 Hob Kit ****", "Kick Tight 4 H ****"},
    {'P',   0,   7,  37, "XG #008 Hob Kit ****", "Side Stick Tight H ****"},
    {'P',   0,   7,  38, "XG #008 Hob Kit ****", "Snare Snappy Short H ****"},
    {'P',   0,   7,  40, "XG #008 Hob Kit ****", "Snare Brass Hard H ****"},
    {'P',   0,   7,  41, "XG #008 Hob Kit ****", "Floor Tom L Tight H ****"},
    {'P',   0,   7,  42, "XG #008 Hob Kit ****", "Hi-Hat Closed 3 H ****"},
    {'P',   0,   7,  43, "XG #008 Hob Kit ****", "Floor Tom H Tight H ****"},
    {'P',   0,   7,  44, "XG #008 Hob Kit ****", "HiHat Pedal 3 H ****"},
    {'P',   0,   7,  45, "XG #008 Hob Kit ****", "Low Tom Tight H ****"},
    {'P',   0,   7,  46, "XG #008 Hob Kit ****", "Hi-Hat Open 3 H ****"},
    {'P',   0,   7,  47, "XG #008 Hob Kit ****", "Mid Tom L Tight H ****"},
    {'P',   0,   7,  48, "XG #008 Hob Kit ****", "Mid Tom H Tight H ****"},
    {'P',   0,   7,  49, "XG #008 Hob Kit ****", "Crash Cymbal 3 H ****"},
    {'P',   0,   7,  50, "XG #008 Hob Kit ****", "High Tom Tight H ****"},
    {'P',   0,   7,  51, "XG #008 Hob Kit ****", "Ride Cymbal 3 H ****"},
    {'P',   0,   7,  52, "XG #008 Hob Kit ****", "Chinese Cymbal 2 H ****"},
    {'P',   0,   7,  53, "XG #008 Hob Kit ****", "Ride Cymbal Cup 2 H ****"},
    {'P',   0,   7,  54, "XG #008 Hob Kit ****", "Tambourine H ****"},
    {'P',   0,   7,  55, "XG #008 Hob Kit ****", "Splash Cymbal H ***"},
    {'P',   0,   7,  56, "XG #008 Hob Kit ****", "Cowbell H ****"},
    {'P',   0,   7,  57, "XG #008 Hob Kit ****", "Crash Cymbal 4 H ****"},
    {'P',   0,   7,  59, "XG #008 Hob Kit ****", "Ride Cymbal 4 ****"},
    {'P',   0,   7,  60, "XG #008 Hob Kit ****", "Bongo H V ***"},
    {'P',   0,   7,  61, "XG #008 Hob Kit ****", "Bongo L V ***"},
    {'P',   0,   7,  62, "XG #008 Hob Kit ****", "Conga H Mute 2 H ****"},
    {'P',   0,   7,  63, "XG #008 Hob Kit ****", "Conga H Open 2 H ****"},
    {'P',   0,   7,  64, "XG #008 Hob Kit ****", "Conga L 2 H ****"},
    {'P',   0,   7,  65, "XG #008 Hob Kit ****", "Timbale H V ***"},
    {'P',   0,   7,  66, "XG #008 Hob Kit ****", "Timbale L V ***"},
    {'P',   0,   7,  67, "XG #008 Hob Kit ****", "Agogo H V ***"},
    {'P',   0,   7,  68, "XG #008 Hob Kit ****", "Agogo L V ***"},
    {'P',   0,   7,  70, "XG #008 Hob Kit ****", "Maracas Q ****"},
    {'P',   0,   7,  71, "XG #008 Hob Kit ****", "Samba Whistle H V ***"},
    {'P',   0,   7,  72, "XG #008 Hob Kit ****", "Samba Whistle L V ***"},
    {'P',   0,   7,  79, "XG #008 Hob Kit ****", "Cuica Open H ****"},
    {'P',   0,   8,  36, "XG #009 Room Kit", "Kick Room **"},
    {'P',   0,   8,  38, "XG #009 Room Kit", "Snare Snappy"},
    {'P',   0,   8,  40, "XG #009 Room Kit", "Snare Tight Snappy"},
    {'P',   0,   8,  41, "XG #009 Room Kit", "Tom Room 1"},
    {'P',   0,   8,  43, "XG #009 Room Kit", "Tom Room 2"},
    {'P',   0,   8,  45, "XG #009 Room Kit", "Tom Room 3"},
    {'P',   0,   8,  47, "XG #009 Room Kit", "Tom Room 4"},
    {'P',   0,   8,  48, "XG #009 Room Kit", "Tom Toom 5"},
    {'P',   0,   8,  50, "XG #009 Room Kit", "Tom Room 6"},
    {'P',   0,   9,  13, "XG #010 DarkRKit ***", "Surdo Mute V ***"},
    {'P',   0,   9,  14, "XG #010 DarkRKit ***", "Surdo Open V ***"},
    {'P',   0,   9,  25, "XG #010 DarkRKit ***", "Brush Tap V ***"},
    {'P',   0,   9,  26, "XG #010 DarkRKit ***", "Brush Swirl V ***"},
    {'P',   0,   9,  27, "XG #010 DarkRKit ***", "Brush Slap V ***"},
    {'P',   0,   9,  28, "XG #010 DarkRKit ***", "Brush Tap Swirl V ***"},
    {'P',   0,   9,  29, "XG #010 DarkRKit ***", "Snare Roll V ***"},
    {'P',   0,   9,  35, "XG #010 DarkRKit ***", "Kick Dark ***"},
    {'P',   0,   9,  36, "XG #010 DarkRKit ***", "Kick Room Gate ***"},
    {'P',   0,   9,  38, "XG #010 DarkRKit ***", "Snare Snappy 2 ***"},
    {'P',   0,   9,  40, "XG #010 DarkRKit ***", "Snare Tight Snappy"},
    {'P',   0,   9,  41, "XG #010 DarkRKit ***", "Tom Room 1 Q ***"},
    {'P',   0,   9,  42, "XG #010 DarkRKit ***", "Hi-Hat Closed Q ***"},
    {'P',   0,   9,  43, "XG #010 DarkRKit ***", "Room Tom 2 Q ***"},
    {'P',   0,   9,  44, "XG #010 DarkRKit ***", "Hi-Hat Pedal Q ***"},
    {'P',   0,   9,  45, "XG #010 DarkRKit ***", "Room Tom 3 Q ***"},
    {'P',   0,   9,  46, "XG #010 DarkRKit ***", "Hi-Hat Open L ***"},
    {'P',   0,   9,  47, "XG #010 DarkRKit ***", "Tom Room 4 Dark ***"},
    {'P',   0,   9,  48, "XG #010 DarkRKit ***", "Tom Room 5 L Dark ***"},
    {'P',   0,   9,  50, "XG #010 DarkRKit ***", "Tom Room 6 L Dark ***"},
    {'P',   0,   9,  51, "XG #010 DarkRKit ***", "Ride Cymbal 1 V ***"},
    {'P',   0,   9,  52, "XG #010 DarkRKit ***", "Chinese Cymbal V ***"},
    {'P',   0,   9,  55, "XG #010 DarkRKit ***", "Splash Cymbal V ***"},
    {'P',   0,   9,  57, "XG #010 DarkRKit ***", "Crash Cymbal 2 V ***"},
    {'P',   0,   9,  59, "XG #010 DarkRKit ***", "Ride Cymbal 2 V ***"},
    {'P',   0,   9,  60, "XG #010 DarkRKit ***", "Bongo H V ***"},
    {'P',   0,   9,  61, "XG #010 DarkRKit ***", "Bongo L V ***"},
    {'P',   0,   9,  62, "XG #010 DarkRKit ***", "Conga H Mute V ***"},
    {'P',   0,   9,  63, "XG #010 DarkRKit ***", "Conga H Open V ***"},
    {'P',   0,   9,  64, "XG #010 DarkRKit ***", "Conga L V ***"},
    {'P',   0,   9,  65, "XG #010 DarkRKit ***", "Timbale H V ***"},
    {'P',   0,   9,  66, "XG #010 DarkRKit ***", "Timbale L V ***"},
    {'P',   0,   9,  67, "XG #010 DarkRKit ***", "Agogo H V ***"},
    {'P',   0,   9,  68, "XG #010 DarkRKit ***", "Agogo L V ***"},
    {'P',   0,   9,  71, "XG #010 DarkRKit ***", "Samba Whistle H V ***"},
    {'P',   0,   9,  72, "XG #010 DarkRKit ***", "Samba Whistle L V ***"},
    {'P',   0,   9,  78, "XG #010 DarkRKit ***", "Cuica Mute V ***"},
    {'P',   0,   9,  79, "XG #010 DarkRKit ***", "Cuica Open V ***"},
    {'P',   0,  16,  31, "XG #017 Rock Kit", "Snare Noisy"},
    {'P',   0,  16,  33, "XG #017 Rock Kit", "Kick Tight 2"},
    {'P',   0,  16,  35, "XG #017 Rock Kit", "Kick 2"},
    {'P',   0,  16,  36, "XG #017 Rock Kit", "Kick Gate"},
    {'P',   0,  16,  38, "XG #017 Rock Kit", "Snare Rock"},
    {'P',   0,  16,  40, "XG #017 Rock Kit", "Snare Rock Rim"},
    {'P',   0,  16,  41, "XG #017 Rock Kit", "Tom Rock 1"},
    {'P',   0,  16,  43, "XG #017 Rock Kit", "Tom Rock 2"},
    {'P',   0,  16,  45, "XG #017 Rock Kit", "Tom Rock 3"},
    {'P',   0,  16,  47, "XG #017 Rock Kit", "Tom Rock 4"},
    {'P',   0,  16,  48, "XG #017 Rock Kit", "Tom Rock 5"},
    {'P',   0,  16,  50, "XG #017 Rock Kit", "Tom Rock 6"},
    {'P',   0,  17,  13, "XG #018 RockKit2 ***", "Surdo Mute V ***"},
    {'P',   0,  17,  14, "XG #018 RockKit2 ***", "Surdo Open V ***"},
    {'P',   0,  17,  25, "XG #018 RockKit2 ***", "Brush Tap V ***"},
    {'P',   0,  17,  26, "XG #018 RockKit2 ***", "Brush Swirl V ***"},
    {'P',   0,  17,  27, "XG #018 RockKit2 ***", "Brush Slap V ***"},
    {'P',   0,  17,  28, "XG #018 RockKit2 ***", "Brush Tap Swirl V ***"},
    {'P',   0,  17,  29, "XG #018 RockKit2 ***", "Snare Roll V ***"},
    {'P',   0,  17,  31, "XG #018 RockKit2 ***", "Snare Noisy 5 ***"},
    {'P',   0,  17,  33, "XG #018 RockKit2 ***", "Kick Tight 3 ***"},
    {'P',   0,  17,  35, "XG #018 RockKit2 ***", "Kick 4 ***"},
    {'P',   0,  17,  36, "XG #018 RockKit2 ***", "Kick Gate 2 ***"},
    {'P',   0,  17,  38, "XG #018 RockKit2 ***", "Snare Rock 2 ***"},
    {'P',   0,  17,  40, "XG #018 RockKit2 ***", "Snare Rock Rim Q ***"},
    {'P',   0,  17,  41, "XG #018 RockKit2 ***", "Tom Rock 1 H ***"},
    {'P',   0,  17,  42, "XG #018 RockKit2 ***", "Hi-Hat Closed 2 ***"},
    {'P',   0,  17,  43, "XG #018 RockKit2 ***", "Tom Rock 2 H ***"},
    {'P',   0,  17,  44, "XG #018 RockKit2 ***", "Hi-Hat Pedal 2 ***"},
    {'P',   0,  17,  45, "XG #018 RockKit2 ***", "Tom Rock 3 L Short ***"},
    {'P',   0,  17,  46, "XG #018 RockKit2 ***", "Hi-Hat Open 2 ***"},
    {'P',   0,  17,  47, "XG #018 RockKit2 ***", "Tom Rock 4 L Short ***"},
    {'P',   0,  17,  48, "XG #018 RockKit2 ***", "Tom Rock 5 L Short ***"},
    {'P',   0,  17,  50, "XG #018 RockKit2 ***", "Tom Rock 6 L Short ***"},
    {'P',   0,  17,  51, "XG #018 RockKit2 ***", "Ride Cymbal 1 V ***"},
    {'P',   0,  17,  52, "XG #018 RockKit2 ***", "Chinese Cymbal V ***"},
    {'P',   0,  17,  55, "XG #018 RockKit2 ***", "Splash Cymbal V ***"},
    {'P',   0,  17,  57, "XG #018 RockKit2 ***", "Crash Cymbal 2 V ***"},
    {'P',   0,  17,  59, "XG #018 RockKit2 ***", "Ride Cymbal 2 V ***"},
    {'P',   0,  17,  60, "XG #018 RockKit2 ***", "Bongo H V ***"},
    {'P',   0,  17,  61, "XG #018 RockKit2 ***", "Bongo L V ***"},
    {'P',   0,  17,  62, "XG #018 RockKit2 ***", "Conga H Mute V ***"},
    {'P',   0,  17,  63, "XG #018 RockKit2 ***", "Conga H Open V ***"},
    {'P',   0,  17,  64, "XG #018 RockKit2 ***", "Conga L V ***"},
    {'P',   0,  17,  65, "XG #018 RockKit2 ***", "Timbale H V ***"},
    {'P',   0,  17,  66, "XG #018 RockKit2 ***", "Timbale L V ***"},
    {'P',   0,  17,  67, "XG #018 RockKit2 ***", "Agogo H V ***"},
    {'P',   0,  17,  68, "XG #018 RockKit2 ***", "Agogo L V ***"},
    {'P',   0,  17,  71, "XG #018 RockKit2 ***", "Samba Whistle H V ***"},
    {'P',   0,  17,  72, "XG #018 RockKit2 ***", "Samba Whistle L V ***"},
    {'P',   0,  17,  78, "XG #018 RockKit2 ***", "Cuica Mute V ***"},
    {'P',   0,  17,  79, "XG #018 RockKit2 ***", "Cuica Open V ***"},
    {'P',   0,  24,  28, "XG #025 ElctrKit", "Reverse Cymbal"},
    {'P',   0,  24,  30, "XG #025 ElctrKit", "Hi Q 2"},
    {'P',   0,  24,  31, "XG #025 ElctrKit", "Snare Snappy Electro"},
    {'P',   0,  24,  33, "XG #025 ElctrKit", "Kick 3"},
    {'P',   0,  24,  35, "XG #025 ElctrKit", "Kick Gate"},
    {'P',   0,  24,  36, "XG #025 ElctrKit", "Kick Gate Heavy"},
    {'P',   0,  24,  38, "XG #025 ElctrKit", "Snare Noisy 2"},
    {'P',   0,  24,  40, "XG #025 ElctrKit", "Snare Noisy 3"},
    {'P',   0,  24,  41, "XG #025 ElctrKit", "Tom Electro 1"},
    {'P',   0,  24,  43, "XG #025 ElctrKit", "Tom Electro 2"},
    {'P',   0,  24,  45, "XG #025 ElctrKit", "Tom Electro 3"},
    {'P',   0,  24,  47, "XG #025 ElctrKit", "Tom Electro 4"},
    {'P',   0,  24,  48, "XG #025 ElctrKit", "Tom Electro 5"},
    {'P',   0,  24,  50, "XG #025 ElctrKit", "Tom Electro 6"},
    {'P',   0,  24,  78, "XG #025 ElctrKit", "Scratch H 2"},
    {'P',   0,  24,  79, "XG #025 ElctrKit", "Scratch L 2"},
    {'P',   0,  25,  28, "XG #026 AnalgKit", "Reverse Cymbal"},
    {'P',   0,  25,  30, "XG #026 AnalgKit", "Hi Q 2"},
    {'P',   0,  25,  31, "XG #026 AnalgKit", "Snare Noisy 4"},
    {'P',   0,  25,  33, "XG #026 AnalgKit", "Kick Tight 2"},
    {'P',   0,  25,  35, "XG #026 AnalgKit", "Kick Analog Short"},
    {'P',   0,  25,  36, "XG #026 AnalgKit", "Kick Analog"},
    {'P',   0,  25,  37, "XG #026 AnalgKit", "Side Stick Analog"},
    {'P',   0,  25,  38, "XG #026 AnalgKit", "Snare Analog"},
    {'P',   0,  25,  40, "XG #026 AnalgKit", "Snare Analog 2"},
    {'P',   0,  25,  41, "XG #026 AnalgKit", "Tom Analog 1"},
    {'P',   0,  25,  42, "XG #026 AnalgKit", "Hi-Hat Closed Analog"},
    {'P',   0,  25,  43, "XG #026 AnalgKit", "Tom Analog 2"},
    {'P',   0,  25,  44, "XG #026 AnalgKit", "Hi-Hat Closed Analog"},
    {'P',   0,  25,  45, "XG #026 AnalgKit", "Tom Analog 3"},
    {'P',   0,  25,  46, "XG #026 AnalgKit", "Hi-Hat Open Analog"},
    {'P',   0,  25,  47, "XG #026 AnalgKit", "Tom Analog 4"},
    {'P',   0,  25,  48, "XG #026 AnalgKit", "Tom Analog 5"},
    {'P',   0,  25,  49, "XG #026 AnalgKit", "Crash Analog"},
    {'P',   0,  25,  50, "XG #026 AnalgKit", "Tom Analog 6"},
    {'P',   0,  25,  56, "XG #026 AnalgKit", "Cowbell Analog"},
    {'P',   0,  25,  62, "XG #026 AnalgKit", "Conga Analog H"},
    {'P',   0,  25,  63, "XG #026 AnalgKit", "Conga Analog M"},
    {'P',   0,  25,  64, "XG #026 AnalgKit", "Conga Analog L"},
    {'P',   0,  25,  70, "XG #026 AnalgKit", "Maracas 2"},
    {'P',   0,  25,  75, "XG #026 AnalgKit", "Claves 2"},
    {'P',   0,  25,  78, "XG #026 AnalgKit", "Scratch H 2"},
    {'P',   0,  25,  79, "XG #026 AnalgKit", "Scratch L 3"},
    {'P',   0,  26,  13, "XG #027 AnlgKit2 ***", "Surdo Mute V ***"},
    {'P',   0,  26,  14, "XG #027 AnlgKit2 ***", "Surdo Open V ***"},
    {'P',   0,  26,  25, "XG #027 AnlgKit2 ***", "Brush Tap V ***"},
    {'P',   0,  26,  26, "XG #027 AnlgKit2 ***", "Brush Swirl V ***"},
    {'P',   0,  26,  27, "XG #027 AnlgKit2 ***", "Brush Slap V ***"},
    {'P',   0,  26,  28, "XG #027 AnlgKit2 ***", "Reverse Cymbal ***"},
    {'P',   0,  26,  29, "XG #027 AnlgKit2 ***", "Snare Roll V ***"},
    {'P',   0,  26,  30, "XG #027 AnlgKit2 ***", "Hi Q 2 ***"},
    {'P',   0,  26,  31, "XG #027 AnlgKit2 ***", "Snare Analog 3 ***"},
    {'P',   0,  26,  33, "XG #027 AnlgKit2 ***", "Kick Techno Soft ***"},
    {'P',   0,  26,  34, "XG #027 AnlgKit2 ***", "Open Rim Shot Dry V ***"},
    {'P',   0,  26,  35, "XG #027 AnlgKit2 ***", "Kick Techno Tight ***"},
    {'P',   0,  26,  36, "XG #027 AnlgKit2 ***", "Kick Techno ***"},
    {'P',   0,  26,  37, "XG #027 AnlgKit2 ***", "Side Stick Analog ***"},
    {'P',   0,  26,  38, "XG #027 AnlgKit2 ***", "Snare Techno ***"},
    {'P',   0,  26,  40, "XG #027 AnlgKit2 ***", "Snare Techno 2 ***"},
    {'P',   0,  26,  41, "XG #027 AnlgKit2 ***", "Tom Analog 1 ***"},
    {'P',   0,  26,  42, "XG #027 AnlgKit2 ***", "Hi-Hat Closed Analog ***"},
    {'P',   0,  26,  43, "XG #027 AnlgKit2 ***", "Tom Analog 2 ***"},
    {'P',   0,  26,  44, "XG #027 AnlgKit2 ***", "Hi-Hat Closed Analog 2 ***"},
    {'P',   0,  26,  45, "XG #027 AnlgKit2 ***", "Tom Analog 3 ***"},
    {'P',   0,  26,  46, "XG #027 AnlgKit2 ***", "Hi-Hat Open Analog ***"},
    {'P',   0,  26,  47, "XG #027 AnlgKit2 ***", "Tom Analog 4 ***"},
    {'P',   0,  26,  48, "XG #027 AnlgKit2 ***", "Tom Analog 5 ***"},
    {'P',   0,  26,  49, "XG #027 AnlgKit2 ***", "Crash Analog ***"},
    {'P',   0,  26,  50, "XG #027 AnlgKit2 ***", "Tom Analog 6 ***"},
    {'P',   0,  26,  51, "XG #027 AnlgKit2 ***", "Ride Cymbal 1 V ***"},
    {'P',   0,  26,  52, "XG #027 AnlgKit2 ***", "Chinese Cymbal V ***"},
    {'P',   0,  26,  55, "XG #027 AnlgKit2 ***", "Splash Cymbal V ***"},
    {'P',   0,  26,  56, "XG #027 AnlgKit2 ***", "Cowbell Analog ***"},
    {'P',   0,  26,  57, "XG #027 AnlgKit2 ***", "Crash Cymbal 2 V ***"},
    {'P',   0,  26,  59, "XG #027 AnlgKit2 ***", "Ride Cymbal 2 V ***"},
    {'P',   0,  26,  60, "XG #027 AnlgKit2 ***", "Bongo H V ***"},
    {'P',   0,  26,  61, "XG #027 AnlgKit2 ***", "Bongo L V ***"},
    {'P',   0,  26,  62, "XG #027 AnlgKit2 ***", "Conga Analog H ***"},
    {'P',   0,  26,  63, "XG #027 AnlgKit2 ***", "Conga Analog M ***"},
    {'P',   0,  26,  64, "XG #027 AnlgKit2 ***", "Conga Analog L ***"},
    {'P',   0,  26,  65, "XG #027 AnlgKit2 ***", "Timbale H V ***"},
    {'P',   0,  26,  66, "XG #027 AnlgKit2 ***", "Timbale L V ***"},
    {'P',   0,  26,  67, "XG #027 AnlgKit2 ***", "Agogo H V ***"},
    {'P',   0,  26,  68, "XG #027 AnlgKit2 ***", "Agogo L V ***"},
    {'P',   0,  26,  70, "XG #027 AnlgKit2 ***", "Maracas 2 ***"},
    {'P',   0,  26,  71, "XG #027 AnlgKit2 ***", "Samba Whistle H V ***"},
    {'P',   0,  26,  72, "XG #027 AnlgKit2 ***", "Samba Whistle L V ***"},
    {'P',   0,  26,  75, "XG #027 AnlgKit2 ***", "Claves 2 ***"},
    {'P',   0,  26,  78, "XG #027 AnlgKit2 ***", "Scratch H 2 ***"},
    {'P',   0,  26,  79, "XG #027 AnlgKit2 ***", "Scratch L 3 ***"},
    {'P',   0,  27,  28, "XG #028 DanceKit ***", "Reverse Cymbal"},
    {'P',   0,  27,  30, "XG #028 DanceKit ***", "Hi Q 2 ***"},
    {'P',   0,  27,  31, "XG #028 DanceKit ***", "Snare Techno 3 ***"},
    {'P',   0,  27,  33, "XG #028 DanceKit ***", "Kick Techno Q ***"},
    {'P',   0,  27,  34, "XG #028 DanceKit ***", "Rim Gate ***"},
    {'P',   0,  27,  35, "XG #028 DanceKit ***", "Kick Techno L ***"},
    {'P',   0,  27,  36, "XG #028 DanceKit ***", "Kick Techno 2 ***"},
    {'P',   0,  27,  37, "XG #028 DanceKit ***", "Side Stick Analog ***"},
    {'P',   0,  27,  38, "XG #028 DanceKit ***", "Snare Clap ***"},
    {'P',   0,  27,  40, "XG #028 DanceKit ***", "Snare Dry 2 ***"},
    {'P',   0,  27,  41, "XG #028 DanceKit ***", "Tom Analog 1 ***"},
    {'P',   0,  27,  42, "XG #028 DanceKit ***", "Hi-Hat Closed 3 ***"},
    {'P',   0,  27,  43, "XG #028 DanceKit ***", "Tom Analog 2 ***"},
    {'P',   0,  27,  44, "XG #028 DanceKit ***", "Hi-Hat Closed Analog 2 ***"},
    {'P',   0,  27,  45, "XG #028 DanceKit ***", "Tom Analog 3 ***"},
    {'P',   0,  27,  46, "XG #028 DanceKit ***", "Hi-Hat Open 3 ***"},
    {'P',   0,  27,  47, "XG #028 DanceKit ***", "Tom Analog 4 ***"},
    {'P',   0,  27,  48, "XG #028 DanceKit ***", "Tom Analog 5 ***"},
    {'P',   0,  27,  49, "XG #028 DanceKit ***", "Crash Analog ***"},
    {'P',   0,  27,  50, "XG #028 DanceKit ***", "Tom Analog 6 ***"},
    {'P',   0,  27,  56, "XG #028 DanceKit ***", "Cowbell Analog ***"},
    {'P',   0,  27,  62, "XG #028 DanceKit ***", "Conga Analog H ***"},
    {'P',   0,  27,  63, "XG #028 DanceKit ***", "Conga Analog M ***"},
    {'P',   0,  27,  64, "XG #028 DanceKit ***", "Conga Analog L ***"},
    {'P',   0,  27,  70, "XG #028 DanceKit ***", "Maracas 2 ***"},
    {'P',   0,  27,  75, "XG #028 DanceKit ***", "Claves 2 ***"},
    {'P',   0,  27,  78, "XG #028 DanceKit ***", "Scratch H 2 ***"},
    {'P',   0,  27,  79, "XG #028 DanceKit ***", "Scratch L 3 ***"},
    {'P',   0,  28,  13, "XG #029 HipHpKit ***", "Surdo Mute V ***"},
    {'P',   0,  28,  14, "XG #029 HipHpKit ***", "Surdo Open V ***"},
    {'P',   0,  28,  25, "XG #029 HipHpKit ***", "Brush Tap V ***"},
    {'P',   0,  28,  26, "XG #029 HipHpKit ***", "Brush Swirl V ***"},
    {'P',   0,  28,  27, "XG #029 HipHpKit ***", "Brush Slap V ***"},
    {'P',   0,  28,  28, "XG #029 HipHpKit ***", "Brush Tap Swirl V ***"},
    {'P',   0,  28,  29, "XG #029 HipHpKit ***", "Snare Roll V ***"},
    {'P',   0,  28,  31, "XG #029 HipHpKit ***", "Open Rim Shot 2 Soft ***"},
    {'P',   0,  28,  33, "XG #029 HipHpKit ***", "Kick Dry Soft 2 ***"},
    {'P',   0,  28,  34, "XG #029 HipHpKit ***", "Open Rim Shot 2 ***"},
    {'P',   0,  28,  35, "XG #029 HipHpKit ***", "Kick Dim ***"},
    {'P',   0,  28,  36, "XG #029 HipHpKit ***", "Kick Boon ***"},
    {'P',   0,  28,  37, "XG #029 HipHpKit ***", "Side Stick Dry ***"},
    {'P',   0,  28,  38, "XG #029 HipHpKit ***", "Snare Dry Mute ***"},
    {'P',   0,  28,  40, "XG #029 HipHpKit ***", "Snare White ***"},
    {'P',   0,  28,  41, "XG #029 HipHpKit ***", "Floor Tom L Short ***"},
    {'P',   0,  28,  42, "XG #029 HipHpKit ***", "Hi-Hat Closed 2 H ***"},
    {'P',   0,  28,  43, "XG #029 HipHpKit ***", "Floor Tom H Short ***"},
    {'P',   0,  28,  44, "XG #029 HipHpKit ***", "Hi-Hat Pedal 2 H ***"},
    {'P',   0,  28,  45, "XG #029 HipHpKit ***", "Low Tom Short ***"},
    {'P',   0,  28,  46, "XG #029 HipHpKit ***", "Hi-Hat Open 2 L ***"},
    {'P',   0,  28,  47, "XG #029 HipHpKit ***", "Mid Tom L Short ***"},
    {'P',   0,  28,  48, "XG #029 HipHpKit ***", "Mid Tom H Short ***"},
    {'P',   0,  28,  49, "XG #029 HipHpKit ***", "Crash Cymbal 1 V ***"},
    {'P',   0,  28,  50, "XG #029 HipHpKit ***", "High Tom Short ***"},
    {'P',   0,  28,  51, "XG #029 HipHpKit ***", "Ride Cymbal 1 V ***"},
    {'P',   0,  28,  52, "XG #029 HipHpKit ***", "Chinese Cymbal V ***"},
    {'P',   0,  28,  55, "XG #029 HipHpKit ***", "Splash Cymbal V ***"},
    {'P',   0,  28,  57, "XG #029 HipHpKit ***", "Crash Cymbal 2 V ***"},
    {'P',   0,  28,  59, "XG #029 HipHpKit ***", "Ride Cymbal 2 V ***"},
    {'P',   0,  28,  60, "XG #029 HipHpKit ***", "Bongo H V ***"},
    {'P',   0,  28,  61, "XG #029 HipHpKit ***", "Bongo L V ***"},
    {'P',   0,  28,  62, "XG #029 HipHpKit ***", "Conga H Mute V ***"},
    {'P',   0,  28,  63, "XG #029 HipHpKit ***", "Conga H Open V ***"},
    {'P',   0,  28,  64, "XG #029 HipHpKit ***", "Conga L V ***"},
    {'P',   0,  28,  65, "XG #029 HipHpKit ***", "Timbale H V ***"},
    {'P',   0,  28,  66, "XG #029 HipHpKit ***", "Timbale L V ***"},
    {'P',   0,  28,  67, "XG #029 HipHpKit ***", "Agogo H V ***"},
    {'P',   0,  28,  68, "XG #029 HipHpKit ***", "Agogo L V ***"},
    {'P',   0,  28,  71, "XG #029 HipHpKit ***", "Samba Whistle H V ***"},
    {'P',   0,  28,  72, "XG #029 HipHpKit ***", "Samba Whistle L V ***"},
    {'P',   0,  28,  78, "XG #029 HipHpKit ***", "Cuica Mute V ***"},
    {'P',   0,  28,  79, "XG #029 HipHpKit ***", "Cuica Open V ***"},
    {'P',   0,  29,  13, "XG #030 JunglKit *", "Surdo Mute V ***"},
    {'P',   0,  29,  14, "XG #030 JunglKit *", "Surdo Open V ***"},
    {'P',   0,  29,  24, "XG #030 JunglKit *", "Seq Click H ***"},
    {'P',   0,  29,  25, "XG #030 JunglKit *", "Brush Tap V ***"},
    {'P',   0,  29,  26, "XG #030 JunglKit *", "Brush Swirl V ***"},
    {'P',   0,  29,  27, "XG #030 JunglKit *", "Brush Slap V ***"},
    {'P',   0,  29,  28, "XG #030 JunglKit *", "Brush Tap Swirl V ***"},
    {'P',   0,  29,  29, "XG #030 JunglKit *", "Snare Roll V ***"},
    {'P',   0,  29,  31, "XG #030 JunglKit *", "Rim Gate 2 ***"},
    {'P',   0,  29,  33, "XG #030 JunglKit *", "Kick Cough ***"},
    {'P',   0,  29,  34, "XG #030 JunglKit *", "Rim Gate 3 ***"},
    {'P',   0,  29,  35, "XG #030 JunglKit *", "Kick Zap ***"},
    {'P',   0,  29,  36, "XG #030 JunglKit *", "Kick Dawn ***"},
    {'P',   0,  29,  37, "XG #030 JunglKit *", "Side Stick Dry ***"},
    {'P',   0,  29,  38, "XG #030 JunglKit *", "Snare Tin ***"},
    {'P',   0,  29,  40, "XG #030 JunglKit *", "Snare Can ***"},
    {'P',   0,  29,  41, "XG #030 JunglKit *", "Floor Tom L Short ***"},
    {'P',   0,  29,  42, "XG #030 JunglKit *", "Hi-Hat Closed 2 Soft ***"},
    {'P',   0,  29,  43, "XG #030 JunglKit *", "Floor Tom H Short ***"},
    {'P',   0,  29,  44, "XG #030 JunglKit *", "Hi-Hat Pedal 2 Soft ***"},
    {'P',   0,  29,  45, "XG #030 JunglKit *", "Low Tom Short ***"},
    {'P',   0,  29,  46, "XG #030 JunglKit *", "Hi-Hat Open 2 Soft ***"},
    {'P',   0,  29,  47, "XG #030 JunglKit *", "Mid Tom L Short ***"},
    {'P',   0,  29,  48, "XG #030 JunglKit *", "Mid Tom H Short ***"},
    {'P',   0,  29,  49, "XG #030 JunglKit *", "Crash Cymbal 1 V ***"},
    {'P',   0,  29,  50, "XG #030 JunglKit *", "High Tom Short ***"},
    {'P',   0,  29,  51, "XG #030 JunglKit *", "Ride Cymbal 1 V ***"},
    {'P',   0,  29,  52, "XG #030 JunglKit *", "Chinese Cymbal V ***"},
    {'P',   0,  29,  55, "XG #030 JunglKit *", "Splash Cymbal V ***"},
    {'P',   0,  29,  57, "XG #030 JunglKit *", "Crash Cymbal 2 V ***"},
    {'P',   0,  29,  59, "XG #030 JunglKit *", "Ride Cymbal 2 V ***"},
    {'P',   0,  29,  60, "XG #030 JunglKit *", "Bongo H V ***"},
    {'P',   0,  29,  61, "XG #030 JunglKit *", "Bongo L V ***"},
    {'P',   0,  29,  62, "XG #030 JunglKit *", "Conga H Mute V ***"},
    {'P',   0,  29,  63, "XG #030 JunglKit *", "Conga H Open V ***"},
    {'P',   0,  29,  64, "XG #030 JunglKit *", "Conga L V ***"},
    {'P',   0,  29,  65, "XG #030 JunglKit *", "Timbale H V ***"},
    {'P',   0,  29,  66, "XG #030 JunglKit *", "Timbale L V ***"},
    {'P',   0,  29,  67, "XG #030 JunglKit *", "Agogo H V ***"},
    {'P',   0,  29,  68, "XG #030 JunglKit *", "Agogo L V ***"},
    {'P',   0,  29,  71, "XG #030 JunglKit *", "Samba Whistle H V ***"},
    {'P',   0,  29,  72, "XG #030 JunglKit *", "Samba Whistle L V ***"},
    {'P',   0,  29,  78, "XG #030 JunglKit *", "Cuica Mute V ***"},
    {'P',   0,  29,  79, "XG #030 JunglKit *", "Cuica Open V ***"},
    {'P',   0,  30,  13, "XG #031 ApogeeKt *", "Surdo Mute V ***"},
    {'P',   0,  30,  14, "XG #031 ApogeeKt *", "Surdo Open V ***"},
    {'P',   0,  30,  25, "XG #031 ApogeeKt *", "Brush Tap V ***"},
    {'P',   0,  30,  26, "XG #031 ApogeeKt *", "Brush Swirl V ***"},
    {'P',   0,  30,  27, "XG #031 ApogeeKt *", "Brush Slap V ***"},
    {'P',   0,  30,  28, "XG #031 ApogeeKt *", "Reverse Cymbal ***"},
    {'P',   0,  30,  30, "XG #031 ApogeeKt *", "Hi Q 2 ***"},
    {'P',   0,  30,  31, "XG #031 ApogeeKt *", "Snare Analog 4 ****"},
    {'P',   0,  30,  33, "XG #031 ApogeeKt *", "Kick Techno 3 ****"},
    {'P',   0,  30,  34, "XG #031 ApogeeKt *", "Open Rim Shot L ****"},
    {'P',   0,  30,  35, "XG #031 ApogeeKt *", "Kick Techno 4 ****"},
    {'P',   0,  30,  36, "XG #031 ApogeeKt *", "Kick Techno Tight 2 ****"},
    {'P',   0,  30,  37, "XG #031 ApogeeKt *", "Side Stick Analog 2 ****"},
    {'P',   0,  30,  38, "XG #031 ApogeeKt *", "Snare Analog 5 ****"},
    {'P',   0,  30,  39, "XG #031 ApogeeKt *", "Clap Ambience ****"},
    {'P',   0,  30,  40, "XG #031 ApogeeKt *", "Snare Analog 6 ****"},
    {'P',   0,  30,  41, "XG #031 ApogeeKt *", "Tom Techno 1 ****"},
    {'P',   0,  30,  42, "XG #031 ApogeeKt *", "Hi-Hat Closed Analog 3 ****"},
    {'P',   0,  30,  43, "XG #031 ApogeeKt *", "Tom Techno 2 ****"},
    {'P',   0,  30,  44, "XG #031 ApogeeKt *", "Hi-Hat Closed Analog 4 ****"},
    {'P',   0,  30,  45, "XG #031 ApogeeKt *", "Tom Techno 3 ****"},
    {'P',   0,  30,  46, "XG #031 ApogeeKt *", "Hi-Hat Open Analog 2 ****"},
    {'P',   0,  30,  47, "XG #031 ApogeeKt *", "Tom Techno 4 ****"},
    {'P',   0,  30,  48, "XG #031 ApogeeKt *", "Tom Techno 5 ****"},
    {'P',   0,  30,  49, "XG #031 ApogeeKt *", "Crash Analog ***"},
    {'P',   0,  30,  50, "XG #031 ApogeeKt *", "Tom Techno 6 ****"},
    {'P',   0,  30,  51, "XG #031 ApogeeKt *", "Ride Cymbal Analog 1 ****"},
    {'P',   0,  30,  53, "XG #031 ApogeeKt *", "Ride Cymbal Cup Q ****"},
    {'P',   0,  30,  55, "XG #031 ApogeeKt *", "Splash Cymbal V ***"},
    {'P',   0,  30,  56, "XG #031 ApogeeKt *", "Cowbell Analog L ****"},
    {'P',   0,  30,  57, "XG #031 ApogeeKt *", "Crash Cymbal 2 V ***"},
    {'P',   0,  30,  59, "XG #031 ApogeeKt *", "Ride Cymbal Analog 2 ****"},
    {'P',   0,  30,  60, "XG #031 ApogeeKt *", "Bongo H V ***"},
    {'P',   0,  30,  61, "XG #031 ApogeeKt *", "Bongo L V ***"},
    {'P',   0,  30,  62, "XG #031 ApogeeKt *", "Conga Analog H Long ****"},
    {'P',   0,  30,  63, "XG #031 ApogeeKt *", "Conga Analog M Long ****"},
    {'P',   0,  30,  64, "XG #031 ApogeeKt *", "Conga Analog L Long ****"},
    {'P',   0,  30,  65, "XG #031 ApogeeKt *", "Timbale H V ***"},
    {'P',   0,  30,  66, "XG #031 ApogeeKt *", "Timbale L V ***"},
    {'P',   0,  30,  67, "XG #031 ApogeeKt *", "Agogo H V ***"},
    {'P',   0,  30,  68, "XG #031 ApogeeKt *", "Agogo L V ***"},
    {'P',   0,  30,  70, "XG #031 ApogeeKt *", "Maracas 2 ***"},
    {'P',   0,  30,  71, "XG #031 ApogeeKt *", "Samba Whistle H V ***"},
    {'P',   0,  30,  72, "XG #031 ApogeeKt *", "Samba Whistle L V ***"},
    {'P',   0,  30,  75, "XG #031 ApogeeKt *", "Claves 2 ***"},
    {'P',   0,  30,  78, "XG #031 ApogeeKt *", "Scratch H 2 ***"},
    {'P',   0,  30,  79, "XG #031 ApogeeKt *", "Scratch L 3 ***"},
    {'P',   0,  31,  13, "XG #032 PergeeKt", "Surdo Mute V ***"},
    {'P',   0,  31,  14, "XG #032 PergeeKt", "Surdo Open V ***"},
    {'P',   0,  31,  25, "XG #032 PergeeKt", "Brush Tap V ***"},
    {'P',   0,  31,  26, "XG #032 PergeeKt", "Brush Swirl V ***"},
    {'P',   0,  31,  27, "XG #032 PergeeKt", "Brush Slap V ***"},
    {'P',   0,  31,  28, "XG #032 PergeeKt", "Reverse Cymbal ***"},
    {'P',   0,  31,  30, "XG #032 PergeeKt", "Hi Q 2 ***"},
    {'P',   0,  31,  31, "XG #032 PergeeKt", "Snare Analog 4 H ****"},
    {'P',   0,  31,  33, "XG #032 PergeeKt", "Kick Techno 3 ****"},
    {'P',   0,  31,  34, "XG #032 PergeeKt", "Open Rim Shot L ****"},
    {'P',   0,  31,  35, "XG #032 PergeeKt", "Kick Techno 4 H ****"},
    {'P',   0,  31,  36, "XG #032 PergeeKt", "Kick Techno Tight 2 H ****"},
    {'P',   0,  31,  37, "XG #032 PergeeKt", "Side Stick Analog 2 H ****"},
    {'P',   0,  31,  38, "XG #032 PergeeKt", "Snare Analog 5 H ****"},
    {'P',   0,  31,  39, "XG #032 PergeeKt", "Clap Ambience H ****"},
    {'P',   0,  31,  40, "XG #032 PergeeKt", "Snare Analog 6 H ****"},
    {'P',   0,  31,  41, "XG #032 PergeeKt", "Tom Techno 1 H ****"},
    {'P',   0,  31,  42, "XG #032 PergeeKt", "Hi-Hat Closed Analog 3 H ****"},
    {'P',   0,  31,  43, "XG #032 PergeeKt", "Tom Techno 2 H ****"},
    {'P',   0,  31,  44, "XG #032 PergeeKt", "Hi-Hat Closed Analog 4 H ****"},
    {'P',   0,  31,  45, "XG #032 PergeeKt", "Tom Techno 3 H ****"},
    {'P',   0,  31,  46, "XG #032 PergeeKt", "Hi-Hat Open Analog 2 H ****"},
    {'P',   0,  31,  47, "XG #032 PergeeKt", "Tom Techno 4 H ****"},
    {'P',   0,  31,  48, "XG #032 PergeeKt", "Tom Techno 5 H ****"},
    {'P',   0,  31,  49, "XG #032 PergeeKt", "Crash Analog ****"},
    {'P',   0,  31,  50, "XG #032 PergeeKt", "Tom Techno 6 H ****"},
    {'P',   0,  31,  51, "XG #032 PergeeKt", "Ride Cymbal Analog 1 H ****"},
    {'P',   0,  31,  52, "XG #032 PergeeKt", "Chinese Cymbal H ****"},
    {'P',   0,  31,  53, "XG #032 PergeeKt", "Ride Cymbal Cup H ****"},
    {'P',   0,  31,  54, "XG #032 PergeeKt", "Tambourine H ****"},
    {'P',   0,  31,  55, "XG #032 PergeeKt", "Splash Cymbal H ****"},
    {'P',   0,  31,  56, "XG #032 PergeeKt", "Cowbell Analog H ****"},
    {'P',   0,  31,  57, "XG #032 PergeeKt", "Crash Cymbal 2 H ****"},
    {'P',   0,  31,  59, "XG #032 PergeeKt", "Ride Cymbal Analog 2 H ****"},
    {'P',   0,  31,  60, "XG #032 PergeeKt", "Bongo H V ***"},
    {'P',   0,  31,  61, "XG #032 PergeeKt", "Bongo L V ***"},
    {'P',   0,  31,  62, "XG #032 PergeeKt", "Conga Analog H Long H ****"},
    {'P',   0,  31,  63, "XG #032 PergeeKt", "Conga Analog M Long H ****"},
    {'P',   0,  31,  64, "XG #032 PergeeKt", "Conga Analog L Long H ****"},
    {'P',   0,  31,  65, "XG #032 PergeeKt", "Timbale H V ***"},
    {'P',   0,  31,  66, "XG #032 PergeeKt", "Timbale L V ***"},
    {'P',   0,  31,  67, "XG #032 PergeeKt", "Agogo H V ***"},
    {'P',   0,  31,  68, "XG #032 PergeeKt", "Agogo L V ***"},
    {'P',   0,  31,  70, "XG #032 PergeeKt", "Maracas 2 ***"},
    {'P',   0,  31,  71, "XG #032 PergeeKt", "Samba Whistle H V ***"},
    {'P',   0,  31,  72, "XG #032 PergeeKt", "Samba Whistle L V ***"},
    {'P',   0,  31,  75, "XG #032 PergeeKt", "Claves 2 ***"},
    {'P',   0,  31,  78, "XG #032 PergeeKt", "Scratch H 2 ***"},
    {'P',   0,  31,  79, "XG #032 PergeeKt", "Scratch L 3 ***"},
    {'P',   0,  32,  36, "XG #033 Jazz Kit", "Kick Jazz"},
    {'P',   0,  32,  41, "XG #033 Jazz Kit", "Tom Jazz 1"},
    {'P',   0,  32,  43, "XG #033 Jazz Kit", "Tom Jazz 2"},
    {'P',   0,  32,  45, "XG #033 Jazz Kit", "Tom Jazz 3"},
    {'P',   0,  32,  47, "XG #033 Jazz Kit", "Tom Jazz 4"},
    {'P',   0,  32,  48, "XG #033 Jazz Kit", "Tom Jazz 5"},
    {'P',   0,  32,  50, "XG #033 Jazz Kit", "Tom Jazz 6"},
    {'P',   0,  33,  13, "XG #034 JazzKit2", "Surdo Mute V ***"},
    {'P',   0,  33,  14, "XG #034 JazzKit2", "Surdo Open V ***"},
    {'P',   0,  33,  25, "XG #034 JazzKit2", "Brush Tap V ***"},
    {'P',   0,  33,  26, "XG #034 JazzKit2", "Brush Swirl V ***"},
    {'P',   0,  33,  27, "XG #034 JazzKit2", "Brush Slap V ***"},
    {'P',   0,  33,  28, "XG #034 JazzKit2", "Brush Tap Swirl V ***"},
    {'P',   0,  33,  29, "XG #034 JazzKit2", "Snare Roll V ***"},
    {'P',   0,  33,  36, "XG #034 JazzKit2", "Kick Jazz L ***"},
    {'P',   0,  33,  38, "XG #034 JazzKit2", "Snare H ***"},
    {'P',   0,  33,  40, "XG #034 JazzKit2", "Snare Tight L ***"},
    {'P',   0,  33,  41, "XG #034 JazzKit2", "Tom Jazz 7 ***"},
    {'P',   0,  33,  42, "XG #034 JazzKit2", "Hi-Hat Closed L ***"},
    {'P',   0,  33,  43, "XG #034 JazzKit2", "Tom Jazz 8 ***"},
    {'P',   0,  33,  44, "XG #034 JazzKit2", "Hi-Hat Pedal L ***"},
    {'P',   0,  33,  45, "XG #034 JazzKit2", "Tom Jazz 9 ***"},
    {'P',   0,  33,  46, "XG #034 JazzKit2", "Hi-Hat Open L ***"},
    {'P',   0,  33,  47, "XG #034 JazzKit2", "Tom Jazz 10 ***"},
    {'P',   0,  33,  48, "XG #034 JazzKit2", "Tom Jazz 11 ***"},
    {'P',   0,  33,  50, "XG #034 JazzKit2", "Tom Jazz 12 ***"},
    {'P',   0,  33,  51, "XG #034 JazzKit2", "Ride Cymbal 1 V ***"},
    {'P',   0,  33,  52, "XG #034 JazzKit2", "Chinese Cymbal V ***"},
    {'P',   0,  33,  53, "XG #034 JazzKit2", "Ride Cymbal Cup L ***"},
    {'P',   0,  33,  55, "XG #034 JazzKit2", "Splash Cymbal V ***"},
    {'P',   0,  33,  57, "XG #034 JazzKit2", "Crash Cymbal 2 V ***"},
    {'P',   0,  33,  59, "XG #034 JazzKit2", "Ride Cymbal 2 V ***"},
    {'P',   0,  33,  60, "XG #034 JazzKit2", "Bongo H V ***"},
    {'P',   0,  33,  61, "XG #034 JazzKit2", "Bongo L V ***"},
    {'P',   0,  33,  62, "XG #034 JazzKit2", "Conga H Mute V ***"},
    {'P',   0,  33,  63, "XG #034 JazzKit2", "Conga H Open V ***"},
    {'P',   0,  33,  64, "XG #034 JazzKit2", "Conga L V ***"},
    {'P',   0,  33,  65, "XG #034 JazzKit2", "Timbale H V ***"},
    {'P',   0,  33,  66, "XG #034 JazzKit2", "Timbale L V ***"},
    {'P',   0,  33,  67, "XG #034 JazzKit2", "Agogo H V ***"},
    {'P',   0,  33,  68, "XG #034 JazzKit2", "Agogo L V ***"},
    {'P',   0,  33,  71, "XG #034 JazzKit2", "Samba Whistle H V ***"},
    {'P',   0,  33,  72, "XG #034 JazzKit2", "Samba Whistle L V ***"},
    {'P',   0,  33,  78, "XG #034 JazzKit2", "Cuica Mute V ***"},
    {'P',   0,  33,  79, "XG #034 JazzKit2", "Cuica Open V ***"},
    {'P',   0,  40,  31, "XG #041 BrushKit", "Brush Slap 2"},
    {'P',   0,  40,  36, "XG #041 BrushKit", "Kick Small"},
    {'P',   0,  40,  38, "XG #041 BrushKit", "Brush Slap 3"},
    {'P',   0,  40,  40, "XG #041 BrushKit", "Brush Tap 2"},
    {'P',   0,  40,  41, "XG #041 BrushKit", "Tom Brush 1"},
    {'P',   0,  40,  43, "XG #041 BrushKit", "Tom Brush 2"},
    {'P',   0,  40,  45, "XG #041 BrushKit", "Tom Brush 3"},
    {'P',   0,  40,  47, "XG #041 BrushKit", "Tom Brush 4"},
    {'P',   0,  40,  48, "XG #041 BrushKit", "Tom Brush 5"},
    {'P',   0,  40,  50, "XG #041 BrushKit", "Tom Brush 6"},
    {'P',   0,  41,  28, "XG #042 BrshKit2", "Brush Tap Swirl L ****"},
    {'P',   0,  41,  29, "XG #042 BrshKit2", "Snare Roll V ***"},
    {'P',   0,  41,  30, "XG #042 BrshKit2", "Castanet H ****"},
    {'P',   0,  41,  31, "XG #042 BrshKit2", "Snare Dry Hard ****"},
    {'P',   0,  41,  33, "XG #042 BrshKit2", "Kick Release ****"},
    {'P',   0,  41,  34, "XG #042 BrshKit2", "Open Rim Shot Dry L ****"},
    {'P',   0,  41,  35, "XG #042 BrshKit2", "Kick Cloudy H ****"},
    {'P',   0,  41,  36, "XG #042 BrshKit2", "Kick Cloudy L ****"},
    {'P',   0,  41,  37, "XG #042 BrshKit2", "Side Stick B 2 ****"},
    {'P',   0,  41,  38, "XG #042 BrshKit2", "Brush Slap 3 Q ****"},
    {'P',   0,  41,  40, "XG #042 BrshKit2", "Brush Snare Loud ****"},
    {'P',   0,  41,  41, "XG #042 BrshKit2", "Tom Brush 7 ****"},
    {'P',   0,  41,  42, "XG #042 BrshKit2", "Hi-Hat Closed 4 ****"},
    {'P',   0,  41,  43, "XG #042 BrshKit2", "Tom Brush 8 ****"},
    {'P',   0,  41,  44, "XG #042 BrshKit2", "Hi-Hat Pedal 4 ****"},
    {'P',   0,  41,  45, "XG #042 BrshKit2", "Tom Brush 9 ****"},
    {'P',   0,  41,  46, "XG #042 BrshKit2", "Hi-Hat Open 4 ****"},
    {'P',   0,  41,  47, "XG #042 BrshKit2", "Tom Brush 10 ****"},
    {'P',   0,  41,  48, "XG #042 BrshKit2", "Tom Brush 11 ****"},
    {'P',   0,  41,  49, "XG #042 BrshKit2", "Crash Cymbal 4 ****"},
    {'P',   0,  41,  50, "XG #042 BrshKit2", "Tom Brush 12 ****"},
    {'P',   0,  41,  51, "XG #042 BrshKit2", "Ride Cymbal 1 H ****"},
    {'P',   0,  41,  53, "XG #042 BrshKit2", "Ride Cymbal Cup 6 ****"},
    {'P',   0,  41,  55, "XG #042 BrshKit2", "Splash Cymbal V ***"},
    {'P',   0,  41,  56, "XG #042 BrshKit2", "Cowbell 2 ****"},
    {'P',   0,  41,  59, "XG #042 BrshKit2", "Ride Cymbal 6 ****"},
    {'P',   0,  41,  62, "XG #042 BrshKit2", "Conga H Mute V ***"},
    {'P',   0,  41,  63, "XG #042 BrshKit2", "Conga H Open V ***"},
    {'P',   0,  41,  64, "XG #042 BrshKit2", "Conga L 2 ****"},
    {'P',   0,  41,  70, "XG #042 BrshKit2", "Maracas Q ****"},
    {'P',   0,  41,  79, "XG #042 BrshKit2", "Cuica Open H ****"},
    {'P',   0,  48,  33, "XG #049 SymphKit", "Kick Soft 2"},
    {'P',   0,  48,  35, "XG #049 SymphKit", "Gran Cassa"},
    {'P',   0,  48,  36, "XG #049 SymphKit", "Gran Cassa Mute"},
    {'P',   0,  48,  38, "XG #049 SymphKit", "Band Snare"},
    {'P',   0,  48,  40, "XG #049 SymphKit", "Band Snare 2"},
    {'P',   0,  48,  41, "XG #049 SymphKit", "Tom Jazz 1"},
    {'P',   0,  48,  43, "XG #049 SymphKit", "Tom Jazz 2"},
    {'P',   0,  48,  45, "XG #049 SymphKit", "Tom Jazz 3"},
    {'P',   0,  48,  47, "XG #049 SymphKit", "Tom Jazz 4"},
    {'P',   0,  48,  48, "XG #049 SymphKit", "Tom Jazz 5"},
    {'P',   0,  48,  49, "XG #049 SymphKit", "Hand Cynbal"},
    {'P',   0,  48,  50, "XG #049 SymphKit", "Tom Jazz 6"},
    {'P',   0,  48,  51, "XG #049 SymphKit", "Hand Cymbal Short"},
    {'P',   0,  48,  57, "XG #049 SymphKit", "Hand Cynbal 2"},
    {'P',   0,  48,  59, "XG #049 SymphKit", "Hand Cymbal 2 Short"},
    {'P',   0,  56,  25, "XG #057 Ntrl Kit +++", "+++Brush Tap Nt"},
    {'P',   0,  56,  26, "XG #057 Ntrl Kit +++", "+++BrshSwrlNt L"},
    {'P',   0,  56,  27, "XG #057 Ntrl Kit +++", "+++BrushSlap Nt"},
    {'P',   0,  56,  28, "XG #057 Ntrl Kit +++", "+++BrshSwrlNt H"},
    {'P',   0,  56,  29, "XG #057 Ntrl Kit +++", "+++SnareRoll Nt"},
    {'P',   0,  56,  31, "XG #057 Ntrl Kit +++", "+++Snare Nt L"},
    {'P',   0,  56,  33, "XG #057 Ntrl Kit +++", "+++Kick Soft Nt"},
    {'P',   0,  56,  34, "XG #057 Ntrl Kit +++", "+++Open Rim Nt"},
    {'P',   0,  56,  35, "XG #057 Ntrl Kit +++", "+++KickLight Nt"},
    {'P',   0,  56,  36, "XG #057 Ntrl Kit +++", "+++Kick Std Nt"},
    {'P',   0,  56,  37, "XG #057 Ntrl Kit +++", "+++SideStick Nt"},
    {'P',   0,  56,  38, "XG #057 Ntrl Kit +++", "+++Snare Nt M"},
    {'P',   0,  56,  40, "XG #057 Ntrl Kit +++", "+++Snare Nt H"},
    {'P',   0,  56,  41, "XG #057 Ntrl Kit +++", "+++FloorTomNt L"},
    {'P',   0,  56,  42, "XG #057 Ntrl Kit +++", "+++HiHatCloseNt"},
    {'P',   0,  56,  43, "XG #057 Ntrl Kit +++", "+++FloorTomNt H"},
    {'P',   0,  56,  44, "XG #057 Ntrl Kit +++", "+++HiHatPedalNt"},
    {'P',   0,  56,  45, "XG #057 Ntrl Kit +++", "+++Low Tom Nt"},
    {'P',   0,  56,  46, "XG #057 Ntrl Kit +++", "+++HiHatOpenNt"},
    {'P',   0,  56,  47, "XG #057 Ntrl Kit +++", "+++Mid Tom Nt L"},
    {'P',   0,  56,  48, "XG #057 Ntrl Kit +++", "+++Mid Tom Nt H"},
    {'P',   0,  56,  49, "XG #057 Ntrl Kit +++", "+++CrashCymNt 1"},
    {'P',   0,  56,  50, "XG #057 Ntrl Kit +++", "+++High Tom Nt"},
    {'P',   0,  56,  51, "XG #057 Ntrl Kit +++", "+++RideCym Nt 1"},
    {'P',   0,  56,  52, "XG #057 Ntrl Kit +++", "+++ChineseCymNt"},
    {'P',   0,  56,  53, "XG #057 Ntrl Kit +++", "+++RideCymCupNt"},
    {'P',   0,  56,  55, "XG #057 Ntrl Kit +++", "+++SplashCym Nt"},
    {'P',   0,  56,  57, "XG #057 Ntrl Kit +++", "+++CrashCymNt 2"},
    {'P',   0,  56,  59, "XG #057 Ntrl Kit +++", "+++RideCym Nt 2"},
    {'P',   0,  57,  25, "XG #058 NtFunkKt +++", "+++Brush Tap Nt"},
    {'P',   0,  57,  26, "XG #058 NtFunkKt +++", "+++BrshSwrlNt L"},
    {'P',   0,  57,  27, "XG #058 NtFunkKt +++", "+++BrushSlap Nt"},
    {'P',   0,  57,  28, "XG #058 NtFunkKt +++", "+++BrshSwrlNt H"},
    {'P',   0,  57,  29, "XG #058 NtFunkKt +++", "+++SnareRoll Nt"},
    {'P',   0,  57,  31, "XG #058 NtFunkKt +++", "+++Snare NtF L"},
    {'P',   0,  57,  33, "XG #058 NtFunkKt +++", "+++Kick Soft Nt"},
    {'P',   0,  57,  34, "XG #058 NtFunkKt +++", "+++Open Rim Nt"},
    {'P',   0,  57,  35, "XG #058 NtFunkKt +++", "+++Kick Std NtF"},
    {'P',   0,  57,  36, "XG #058 NtFunkKt +++", "+++Kick NtF"},
    {'P',   0,  57,  37, "XG #058 NtFunkKt +++", "+++SideStick Nt"},
    {'P',   0,  57,  38, "XG #058 NtFunkKt +++", "+++Snare NtF M"},
    {'P',   0,  57,  40, "XG #058 NtFunkKt +++", "+++Snare NtF H"},
    {'P',   0,  57,  41, "XG #058 NtFunkKt +++", "+++FloorTomNt L"},
    {'P',   0,  57,  42, "XG #058 NtFunkKt +++", "+++HiHatCloseNt"},
    {'P',   0,  57,  43, "XG #058 NtFunkKt +++", "+++FloorTomNt H"},
    {'P',   0,  57,  44, "XG #058 NtFunkKt +++", "+++HiHatPedalNt"},
    {'P',   0,  57,  45, "XG #058 NtFunkKt +++", "+++Low Tom Nt"},
    {'P',   0,  57,  46, "XG #058 NtFunkKt +++", "+++HiHatOpenNt"},
    {'P',   0,  57,  47, "XG #058 NtFunkKt +++", "+++Mid Tom Nt L"},
    {'P',   0,  57,  48, "XG #058 NtFunkKt +++", "+++Mid Tom Nt H"},
    {'P',   0,  57,  49, "XG #058 NtFunkKt +++", "+++CrashCymNt 1"},
    {'P',   0,  57,  50, "XG #058 NtFunkKt +++", "+++High Tom Nt"},
    {'P',   0,  57,  51, "XG #058 NtFunkKt +++", "+++RideCym Nt 1"},
    {'P',   0,  57,  52, "XG #058 NtFunkKt +++", "+++ChineseCymNt"},
    {'P',   0,  57,  53, "XG #058 NtFunkKt +++", "+++RideCymCupNt"},
    {'P',   0,  57,  55, "XG #058 NtFunkKt +++", "+++SplashCym Nt"},
    {'P',   0,  57,  57, "XG #058 NtFunkKt +++", "+++CrashCymNt 2"},
    {'P',   0,  57,  59, "XG #058 NtFunkKt +++", "+++RideCym Nt 2"},
    {'P',   0,  64,  25, "XG #065 TrampKit", "Brush Tap V ***"},
    {'P',   0,  64,  26, "XG #065 TrampKit", "Brush Swirl V ***"},
    {'P',   0,  64,  27, "XG #065 TrampKit", "Brush Slap V ***"},
    {'P',   0,  64,  28, "XG #065 TrampKit", "Reverse Cymbal ***"},
    {'P',   0,  64,  29, "XG #065 TrampKit", "Snare Roll V ***"},
    {'P',   0,  64,  30, "XG #065 TrampKit", "Hi Q 2 ***"},
    {'P',   0,  64,  31, "XG #065 TrampKit", "Snare Techno 3 Q ****"},
    {'P',   0,  64,  32, "XG #065 TrampKit", "Sticks Q ****"},
    {'P',   0,  64,  33, "XG #065 TrampKit", "Kick Techno Q 2 ****"},
    {'P',   0,  64,  34, "XG #065 TrampKit", "Rim Gate Lo-Fi ****"},
    {'P',   0,  64,  35, "XG #065 TrampKit", "Kick Techno L Q ****"},
    {'P',   0,  64,  36, "XG #065 TrampKit", "Kick Techno 2 Gate ****"},
    {'P',   0,  64,  37, "XG #065 TrampKit", "Side Stick Analog Q ****"},
    {'P',   0,  64,  38, "XG #065 TrampKit", "Snare Clap ***"},
    {'P',   0,  64,  39, "XG #065 TrampKit", "Hand Clap Dark ****"},
    {'P',   0,  64,  40, "XG #065 TrampKit", "Snare Dry 2 Gate ****"},
    {'P',   0,  64,  41, "XG #065 TrampKit", "Tom Analog 7 ****"},
    {'P',   0,  64,  42, "XG #065 TrampKit", "Hi-Hat Closed 3 Dark ****"},
    {'P',   0,  64,  43, "XG #065 TrampKit", "Tom Analog 8 ****"},
    {'P',   0,  64,  44, "XG #065 TrampKit", "Hi-Hat Closed Analog 2 L ****"},
    {'P',   0,  64,  45, "XG #065 TrampKit", "Tom Analog 9 ****"},
    {'P',   0,  64,  46, "XG #065 TrampKit", "Hi-Hat Open 3 Dark ****"},
    {'P',   0,  64,  47, "XG #065 TrampKit", "Tom Analog 10 ****"},
    {'P',   0,  64,  48, "XG #065 TrampKit", "Tom Analog 11 ****"},
    {'P',   0,  64,  49, "XG #065 TrampKit", "Crash Analog Dark ****"},
    {'P',   0,  64,  50, "XG #065 TrampKit", "Tom Analog 12 ****"},
    {'P',   0,  64,  51, "XG #065 TrampKit", "Ride Cymbal 1 Dark ****"},
    {'P',   0,  64,  52, "XG #065 TrampKit", "Chinese Cymbal L ****"},
    {'P',   0,  64,  53, "XG #065 TrampKit", "Ride Cymbal Cup Dark ****"},
    {'P',   0,  64,  54, "XG #065 TrampKit", "Tambourine Dark ****"},
    {'P',   0,  64,  55, "XG #065 TrampKit", "Splash Cymbal L Q ****"},
    {'P',   0,  64,  56, "XG #065 TrampKit", "Cowbell Analog Q ***"},
    {'P',   0,  64,  57, "XG #065 TrampKit", "Crash Cymbal 2 Q ****"},
    {'P',   0,  64,  59, "XG #065 TrampKit", "Ride Cymbal 2 D ****"},
    {'P',   0,  64,  60, "XG #065 TrampKit", "Bongo H V ***"},
    {'P',   0,  64,  61, "XG #065 TrampKit", "Bongo L V ***"},
    {'P',   0,  64,  62, "XG #065 TrampKit", "Conga Analog H Long ****"},
    {'P',   0,  64,  63, "XG #065 TrampKit", "Conga Analog M Long ****"},
    {'P',   0,  64,  64, "XG #065 TrampKit", "Conga Analog L Long ****"},
    {'P',   0,  64,  65, "XG #065 TrampKit", "Timbale H V ***"},
    {'P',   0,  64,  66, "XG #065 TrampKit", "Timbale L V ***"},
    {'P',   0,  64,  67, "XG #065 TrampKit", "Agogo H V ***"},
    {'P',   0,  64,  68, "XG #065 TrampKit", "Agogo L V ***"},
    {'P',   0,  64,  70, "XG #065 TrampKit", "Maracas 2 ***"},
    {'P',   0,  64,  71, "XG #065 TrampKit", "Samba Whistle H V ***"},
    {'P',   0,  64,  72, "XG #065 TrampKit", "Samba Whistle L V ***"},
    {'P',   0,  64,  78, "XG #065 TrampKit", "Scratch H 2 ***"},
    {'P',   0,  64,  79, "XG #065 TrampKit", "Scratch L 3 ***"},
    {'P',   0,  65,  13, "XG #066 AmberKit", "Surdo Mute V ***"},
    {'P',   0,  65,  14, "XG #066 AmberKit", "Surdo Open V ***"},
    {'P',   0,  65,  25, "XG #066 AmberKit", "Brush Tap V ***"},
    {'P',   0,  65,  26, "XG #066 AmberKit", "Brush Swirl V ***"},
    {'P',   0,  65,  27, "XG #066 AmberKit", "Brush Slap V ***"},
    {'P',   0,  65,  28, "XG #066 AmberKit", "Brush Tap Swirl L ***"},
    {'P',   0,  65,  29, "XG #066 AmberKit", "Snare Roll V ***"},
    {'P',   0,  65,  31, "XG #066 AmberKit", "Open Rim Shot 3 Soft ****"},
    {'P',   0,  65,  32, "XG #066 AmberKit", "Sticks Q ****"},
    {'P',   0,  65,  33, "XG #066 AmberKit", "KickDrySoft3 ****"},
    {'P',   0,  65,  34, "XG #066 AmberKit", "Open Rim Shot Dry ****"},
    {'P',   0,  65,  35, "XG #066 AmberKit", "Kick Comp 1 L ****"},
    {'P',   0,  65,  36, "XG #066 AmberKit", "Kick Comp 1 H ****"},
    {'P',   0,  65,  37, "XG #066 AmberKit", "Side Stick Dry L ****"},
    {'P',   0,  65,  38, "XG #066 AmberKit", "Snare Dry Mute L ****"},
    {'P',   0,  65,  39, "XG #066 AmberKit", "Hand Clap Dark ****"},
    {'P',   0,  65,  40, "XG #066 AmberKit", "Hip Snare Long ****"},
    {'P',   0,  65,  41, "XG #066 AmberKit", "Floor Tom Tech L ****"},
    {'P',   0,  65,  42, "XG #066 AmberKit", "Hi-Hat Closed Tech ****"},
    {'P',   0,  65,  43, "XG #066 AmberKit", "Floor Tom Tech H ****"},
    {'P',   0,  65,  44, "XG #066 AmberKit", "Hi-Hat Pedal Tech ****"},
    {'P',   0,  65,  45, "XG #066 AmberKit", "Low Tom Tech ****"},
    {'P',   0,  65,  46, "XG #066 AmberKit", "Hi-Hat Open 3 Dark ****"},
    {'P',   0,  65,  47, "XG #066 AmberKit", "Mid Tom Tech L ****"},
    {'P',   0,  65,  48, "XG #066 AmberKit", "Mid Tom Tech H ****"},
    {'P',   0,  65,  49, "XG #066 AmberKit", "Crash Cymbal Dark ****"},
    {'P',   0,  65,  50, "XG #066 AmberKit", "High Tom Tech ****"},
    {'P',   0,  65,  51, "XG #066 AmberKit", "Ride Cymbal 1 Dark ****"},
    {'P',   0,  65,  52, "XG #066 AmberKit", "Chinese Cymbal H2 ****"},
    {'P',   0,  65,  53, "XG #066 AmberKit", "Ride Cymbal Cup Dark"},
    {'P',   0,  65,  54, "XG #066 AmberKit", "Tambourine Dark ****"},
    {'P',   0,  65,  55, "XG #066 AmberKit", "Splash Cymbal L Q ****"},
    {'P',   0,  65,  56, "XG #066 AmberKit", "Cowbell Lo-Fi ****"},
    {'P',   0,  65,  57, "XG #066 AmberKit", "Crash Cymbal 2 Q ****"},
    {'P',   0,  65,  59, "XG #066 AmberKit", "Ride Cymbal 2 V ***"},
    {'P',   0,  65,  60, "XG #066 AmberKit", "Bongo H V ***"},
    {'P',   0,  65,  61, "XG #066 AmberKit", "Bongo L V ***"},
    {'P',   0,  65,  62, "XG #066 AmberKit", "Conga H Mute V ***"},
    {'P',   0,  65,  63, "XG #066 AmberKit", "Conga H Open V ***"},
    {'P',   0,  65,  64, "XG #066 AmberKit", "Conga L 2 ****"},
    {'P',   0,  65,  65, "XG #066 AmberKit", "Timbale H V ***"},
    {'P',   0,  65,  66, "XG #066 AmberKit", "Timbale L V ***"},
    {'P',   0,  65,  67, "XG #066 AmberKit", "Agogo H V ***"},
    {'P',   0,  65,  68, "XG #066 AmberKit", "Agogo L V ***"},
    {'P',   0,  65,  70, "XG #066 AmberKit", "Maracas Q ****"},
    {'P',   0,  65,  71, "XG #066 AmberKit", "Samba Whistle H V ***"},
    {'P',   0,  65,  72, "XG #066 AmberKit", "Samba Whistle L V ***"},
    {'P',   0,  65,  79, "XG #066 AmberKit", "Cuica Open H ***"},
    {'P',   0,  66,  13, "XG #067 CoffinKt", "Surdo Mute V ***"},
    {'P',   0,  66,  14, "XG #067 CoffinKt", "Surdo Open V ***"},
    {'P',   0,  66,  25, "XG #067 CoffinKt", "Brush Tap V ***"},
    {'P',   0,  66,  26, "XG #067 CoffinKt", "Brush Swirl V ***"},
    {'P',   0,  66,  27, "XG #067 CoffinKt", "Brush Slap V ***"},
    {'P',   0,  66,  28, "XG #067 CoffinKt", "Brush Tap Swirl L ***"},
    {'P',   0,  66,  29, "XG #067 CoffinKt", "Snare Roll V ***"},
    {'P',   0,  66,  31, "XG #067 CoffinKt", "Rim Gate 4 ****"},
    {'P',   0,  66,  32, "XG #067 CoffinKt", "Sticks Q ****"},
    {'P',   0,  66,  33, "XG #067 CoffinKt", "Kick Cough L ****"},
    {'P',   0,  66,  34, "XG #067 CoffinKt", "Rim Gate 5 ****"},
    {'P',   0,  66,  35, "XG #067 CoffinKt", "Kick Comp 2 L ****"},
    {'P',   0,  66,  36, "XG #067 CoffinKt", "Kick Comp 2 H ****"},
    {'P',   0,  66,  37, "XG #067 CoffinKt", "Side Stick Dry L ****"},
    {'P',   0,  66,  38, "XG #067 CoffinKt", "Snare Tin L ****"},
    {'P',   0,  66,  39, "XG #067 CoffinKt", "Hand Clap Dark ****"},
    {'P',   0,  66,  40, "XG #067 CoffinKt", "Snare Can L ****"},
    {'P',   0,  66,  41, "XG #067 CoffinKt", "Floor Tom Tech L ****"},
    {'P',   0,  66,  42, "XG #067 CoffinKt", "Hi-Hat Closed Tech ****"},
    {'P',   0,  66,  43, "XG #067 CoffinKt", "Floor Tom Tech H ****"},
    {'P',   0,  66,  44, "XG #067 CoffinKt", "Hi-Hat Pedal Tech ****"},
    {'P',   0,  66,  45, "XG #067 CoffinKt", "Low Tom Tech ****"},
    {'P',   0,  66,  46, "XG #067 CoffinKt", "Hi-Hat Open 3 Dark ****"},
    {'P',   0,  66,  47, "XG #067 CoffinKt", "Mid Tom Tech L ****"},
    {'P',   0,  66,  48, "XG #067 CoffinKt", "Mid Tom Tech H ****"},
    {'P',   0,  66,  49, "XG #067 CoffinKt", "Crash Cymbal Dark 2 ****"},
    {'P',   0,  66,  50, "XG #067 CoffinKt", "High Tom Tech ****"},
    {'P',   0,  66,  51, "XG #067 CoffinKt", "Ride Cymbal Hard 2 ****"},
    {'P',   0,  66,  52, "XG #067 CoffinKt", "Chinese Cymbal Q ****"},
    {'P',   0,  66,  53, "XG #067 CoffinKt", "Ride Cymbal Cup 5 ****"},
    {'P',   0,  66,  54, "XG #067 CoffinKt", "Tambourine Dark ****"},
    {'P',   0,  66,  55, "XG #067 CoffinKt", "Tech Splash Cymbal ****"},
    {'P',   0,  66,  56, "XG #067 CoffinKt", "Cowbell Lo-Fi ****"},
    {'P',   0,  66,  57, "XG #067 CoffinKt", "Crash Cymbal 2 Q ****"},
    {'P',   0,  66,  59, "XG #067 CoffinKt", "Ride Cymbal 5 ***"},
    {'P',   0,  66,  60, "XG #067 CoffinKt", "Bongo H V ***"},
    {'P',   0,  66,  61, "XG #067 CoffinKt", "Bongo L V ***"},
    {'P',   0,  66,  62, "XG #067 CoffinKt", "Conga H Mute V ***"},
    {'P',   0,  66,  63, "XG #067 CoffinKt", "Conga H Open V ***"},
    {'P',   0,  66,  64, "XG #067 CoffinKt", "Conga L 2 ****"},
    {'P',   0,  66,  65, "XG #067 CoffinKt", "Timbale H V ***"},
    {'P',   0,  66,  66, "XG #067 CoffinKt", "Timbale L V ***"},
    {'P',   0,  66,  67, "XG #067 CoffinKt", "Agogo H V ***"},
    {'P',   0,  66,  68, "XG #067 CoffinKt", "Agogo L V ***"},
    {'P',   0,  66,  70, "XG #067 CoffinKt", "Maracas Q ****"},
    {'P',   0,  66,  71, "XG #067 CoffinKt", "Samba Whistle H V ***"},
    {'P',   0,  66,  72, "XG #067 CoffinKt", "Samba Whistle L V ***"},
    {'P',   0,  66,  79, "XG #067 CoffinKt", "Cuica Open H ***"},
    {'P',   0, 126,  30, "XG #127 StndKit#", "Castanet# ****"},
    {'P',   0, 126,  31, "XG #127 StndKit#", "Snare Soft# ****"},
    {'P',   0, 126,  34, "XG #127 StndKit#", "Open Rim Shot# ****"},
    {'P',   0, 126,  37, "XG #127 StndKit#", "Side Stick# ****"},
    {'P',   0, 126,  38, "XG #127 StndKit#", "Snare# ****"},
    {'P',   0, 126,  40, "XG #127 StndKit#", "Snare Tight# ****"},
    {'P',   0, 126,  42, "XG #127 StndKit#", "Hi-Hat Closed# ****"},
    {'P',   0, 126,  44, "XG #127 StndKit#", "Hi-Hat Pedal# ****"},
    {'P',   0, 126,  46, "XG #127 StndKit#", "Hi-Hat Open# ****"},
    {'P',   0, 126,  49, "XG #127 StndKit#", "Crash Cymbal"},
    {'P',   0, 126,  51, "XG #127 StndKit#", "Ride Cymbal 1# ****"},
    {'P',   0, 126,  52, "XG #127 StndKit#", "Chinese Cym-"},
    {'P',   0, 126,  53, "XG #127 StndKit#", "Ride Cymbal"},
    {'P',   0, 126,  56, "XG #127 StndKit#", "Cowbell# ****"},
    {'P',   0, 126,  57, "XG #127 StndKit#", "Crash Cymbal 2# ****"},
    {'P',   0, 126,  59, "XG #127 StndKit#", "Ride Cymbal 2# ****"},
    {'P',   0, 126,  62, "XG #127 StndKit#", "Conga H Mute# ****"},
    {'P',   0, 126,  63, "XG #127 StndKit#", "Conga H Open# ****"},
    {'P',   0, 126,  64, "XG #127 StndKit#", "Conga L# ****"},
    {'P',   0, 127,  13, "XG #128 StandKit Mu Basic", "Surdo Mute"},
    {'P',   0, 127,  14, "XG #128 StandKit Mu Basic", "Surdo Open"},
    {'P',   0, 127,  15, "XG #128 StandKit Mu Basic", "Hi Q"},
    {'P',   0, 127,  16, "XG #128 StandKit Mu Basic", "Whip Slap"},
    {'P',   0, 127,  17, "XG #128 StandKit Mu Basic", "Scratch H"},
    {'P',   0, 127,  18, "XG #128 StandKit Mu Basic", "Scratch L"},
    {'P',   0, 127,  19, "XG #128 StandKit Mu Basic", "Finger Snap"},
    {'P',   0, 127,  20, "XG #128 StandKit Mu Basic", "Click Noise"},
    {'P',   0, 127,  21, "XG #128 StandKit Mu Basic", "Metronome Click"},
    {'P',   0, 127,  22, "XG #128 StandKit Mu Basic", "Metronome Bell"},
    {'P',   0, 127,  23, "XG #128 StandKit Mu Basic", "Seq Click L"},
    {'P',   0, 127,  24, "XG #128 StandKit Mu Basic", "Seq Click H"},
    {'P',   0, 127,  25, "XG #128 StandKit Mu Basic", "Brush Tap"},
    {'P',   0, 127,  26, "XG #128 StandKit Mu Basic", "Brush Swirl"},
    {'P',   0, 127,  27, "XG #128 StandKit Mu Basic", "Brush Slap"},
    {'P',   0, 127,  28, "XG #128 StandKit Mu Basic", "Brush Tap Swirl"},
    {'P',   0, 127,  29, "XG #128 StandKit Mu Basic", "Snare Roll"},
    {'P',   0, 127,  30, "XG #128 StandKit Mu Basic", "Castanet"},
    {'P',   0, 127,  31, "XG #128 StandKit Mu Basic", "Snare Soft"},
    {'P',   0, 127,  32, "XG #128 StandKit Mu Basic", "Sticks"},
    {'P',   0, 127,  33, "XG #128 StandKit Mu Basic", "Kick Soft"},
    {'P',   0, 127,  34, "XG #128 StandKit Mu Basic", "Open Rim Shot"},
    {'P',   0, 127,  35, "XG #128 StandKit Mu Basic", "Kick Tight"},
    {'P',   0, 127,  36, "XG #128 StandKit Mu Basic", "Kick"},
    {'P',   0, 127,  37, "XG #128 StandKit Mu Basic", "Side Stick"},
    {'P',   0, 127,  38, "XG #128 StandKit Mu Basic", "Snare"},
    {'P',   0, 127,  39, "XG #128 StandKit Mu Basic", "Hand Clap"},
    {'P',   0, 127,  40, "XG #128 StandKit Mu Basic", "Snare Tight"},
    {'P',   0, 127,  41, "XG #128 StandKit Mu Basic", "Floor Tom L"},
    {'P',   0, 127,  42, "XG #128 StandKit Mu Basic", "Hi-Hat Closed"},
    {'P',   0, 127,  43, "XG #128 StandKit Mu Basic", "Floor Tom H"},
    {'P',   0, 127,  44, "XG #128 StandKit Mu Basic", "Hi-Hat Pedal"},
    {'P',   0, 127,  45, "XG #128 StandKit Mu Basic", "Low Tom"},
    {'P',   0, 127,  46, "XG #128 StandKit Mu Basic", "Hi-Hat Open"},
    {'P',   0, 127,  47, "XG #128 StandKit Mu Basic", "Mid Tom L"},
    {'P',   0, 127,  48, "XG #128 StandKit Mu Basic", "Mid Tom H"},
    {'P',   0, 127,  49, "XG #128 StandKit Mu Basic", "Crash Cymbal 1"},
    {'P',   0, 127,  50, "XG #128 StandKit Mu Basic", "High Tom"},
    {'P',   0, 127,  51, "XG #128 StandKit Mu Basic", "Ride Cymbal 1"},
    {'P',   0, 127,  52, "XG #128 StandKit Mu Basic", "Chinese Cymbal"},
    {'P',   0, 127,  53, "XG #128 StandKit Mu Basic", "Ride Cymbal Cup"},
    {'P',   0, 127,  54, "XG #128 StandKit Mu Basic", "Tambourine"},
    {'P',   0, 127,  55, "XG #128 StandKit Mu Basic", "Splash Cymbal"},
    {'P',   0, 127,  56, "XG #128 StandKit Mu Basic", "Cowbell"},
    {'P',   0, 127,  57, "XG #128 StandKit Mu Basic", "Crash Cymbal 2"},
    {'P',   0, 127,  58, "XG #128 StandKit Mu Basic", "Vibraslap"},
    {'P',   0, 127,  59, "XG #128 StandKit Mu Basic", "Ride Cymbal 2"},
    {'P',   0, 127,  60, "XG #128 StandKit Mu Basic", "Bongo H"},
    {'P',   0, 127,  61, "XG #128 StandKit Mu Basic", "Bongo L"},
    {'P',   0, 127,  62, "XG #128 StandKit Mu Basic", "Conga H Mute"},
    {'P',   0, 127,  63, "XG #128 StandKit Mu Basic", "Conga H Open"},
    {'P',   0, 127,  64, "XG #128 StandKit Mu Basic", "Conga L"},
    {'P',   0, 127,  65, "XG #128 StandKit Mu Basic", "Timbale H"},
    {'P',   0, 127,  66, "XG #128 StandKit Mu Basic", "Timbale L"},
    {'P',   0, 127,  67, "XG #128 StandKit Mu Basic", "Agogo H"},
    {'P',   0, 127,  68, "XG #128 StandKit Mu Basic", "Agogo L"},
    {'P',   0, 127,  69, "XG #128 StandKit Mu Basic", "Cabasa"},
    {'P',   0, 127,  70, "XG #128 StandKit Mu Basic", "Maracas"},
    {'P',   0, 127,  71, "XG #128 StandKit Mu Basic", "Samba Whistle H"},
    {'P',   0, 127,  72, "XG #128 StandKit Mu Basic", "Samba Whistle L"},
    {'P',   0, 127,  73, "XG #128 StandKit Mu Basic", "Guiro Short"},
    {'P',   0, 127,  74, "XG #128 StandKit Mu Basic", "Guiro Long"},
    {'P',   0, 127,  75, "XG #128 StandKit Mu Basic", "Claves"},
    {'P',   0, 127,  76, "XG #128 StandKit Mu Basic", "Wood Block H"},
    {'P',   0, 127,  77, "XG #128 StandKit Mu Basic", "Wood Block L"},
    {'P',   0, 127,  78, "XG #128 StandKit Mu Basic", "Cuica Mute"},
    {'P',   0, 127,  79, "XG #128 StandKit Mu Basic", "Cuica Open"},
    {'P',   0, 127,  80, "XG #128 StandKit Mu Basic", "Triangle Mute"},
    {'P',   0, 127,  81, "XG #128 StandKit Mu Basic", "Triangle Open"},
    {'P',   0, 127,  82, "XG #128 StandKit Mu Basic", "Shaker"},
    {'P',   0, 127,  83, "XG #128 StandKit Mu Basic", "Jingle Bells"},
    {'P',   0, 127,  84, "XG #128 StandKit Mu Basic", "Bell Tree"},
};

static const MidiProgram GsSet[] =
{
    {'M',   1,   0,  38, "GS Var #01", "Synth Bass101"},
    {'M',   1,   0,  57, "GS Var #01", "Trombone 2"},
    {'M',   1,   0,  60, "GS Var #01", "French Horn2"},
    {'M',   1,   0,  80, "GS Var #01", "Square"},
    {'M',   1,   0,  81, "GS Var #01", "Saw"},
    {'M',   1,   0,  98, "GS Var #01", "Syn Mallet"},
    {'M',   1,   0, 102, "GS Var #01", "Echo Bell"},
    {'M',   1,   0, 104, "GS Var #01", "Sitar 2"},
    {'M',   1,   0, 120, "GS Var #01", "Gt.Cut Noise"},
    {'M',   1,   0, 121, "GS Var #01", "Fl.Key Click"},
    {'M',   1,   0, 122, "GS Var #01", "Rain"},
    {'M',   1,   0, 123, "GS Var #01", "Dog"},
    {'M',   1,   0, 124, "GS Var #01", "Telephone 2"},
    {'M',   1,   0, 125, "GS Var #01", "Car-Engine"},
    {'M',   1,   0, 126, "GS Var #01", "Laughing"},
    {'M',   1,   0, 127, "GS Var #01", "Machine Gun"},
    {'M',   2,   0, 102, "GS Var #02", "Echo Pan"},
    {'M',   2,   0, 120, "GS Var #02", "String Slap"},
    {'M',   2,   0, 122, "GS Var #02", "Thunder"},
    {'M',   2,   0, 123, "GS Var #02", "Horse-Gallop"},
    {'M',   2,   0, 124, "GS Var #02", "DoorCreaking"},
    {'M',   2,   0, 125, "GS Var #02", "Car-Stop"},
    {'M',   2,   0, 126, "GS Var #02", "Screaming"},
    {'M',   2,   0, 127, "GS Var #02", "Lasergun"},
    {'M',   3,   0, 122, "GS Var #03", "Wind"},
    {'M',   3,   0, 123, "GS Var #03", "Bird 2"},
    {'M',   3,   0, 124, "GS Var #03", "Door"},
    {'M',   3,   0, 125, "GS Var #03", "Car-Pass"},
    {'M',   3,   0, 126, "GS Var #03", "Punch"},
    {'M',   3,   0, 127, "GS Var #03", "Explosion"},
    {'M',   4,   0, 122, "GS Var #04", "Stream"},
    {'M',   4,   0, 124, "GS Var #04", "Scratch"},
    {'M',   4,   0, 125, "GS Var #04", "Car-Crash"},
    {'M',   4,   0, 126, "GS Var #04", "Heart Beat"},
    {'M',   5,   0, 122, "GS Var #05", "Bubble"},
    {'M',   5,   0, 124, "GS Var #05", "Windchime"},
    {'M',   5,   0, 125, "GS Var #05", "Siren"},
    {'M',   5,   0, 126, "GS Var #05", "Footsteps"},
    {'M',   6,   0, 125, "GS Var #06", "Train"},
    {'M',   7,   0, 125, "GS Var #07", "Jetplane"},
    {'M',   8,   0,   0, "GS Var #08", "Piano 1w"},
    {'M',   8,   0,   1, "GS Var #08", "Piano 2w"},
    {'M',   8,   0,   2, "GS Var #08", "Piano 3w"},
    {'M',   8,   0,   3, "GS Var #08", "Honky-tonk w"},
    {'M',   8,   0,   4, "GS Var #08", "Detuned EP 1"},
    {'M',   8,   0,   5, "GS Var #08", "Detuned EP 2"},
    {'M',   8,   0,   6, "GS Var #08", "Coupled Hps."},
    {'M',   8,   0,  11, "GS Var #08", "Vib.w"},
    {'M',   8,   0,  12, "GS Var #08", "Marimba w"},
    {'M',   8,   0,  14, "GS Var #08", "Church Bell"},
    {'M',   8,   0,  16, "GS Var #08", "Detuned Or.1"},
    {'M',   8,   0,  17, "GS Var #08", "Detuned Or.2"},
    {'M',   8,   0,  19, "GS Var #08", "Church Org.2"},
    {'M',   8,   0,  21, "GS Var #08", "Accordion It"},
    {'M',   8,   0,  24, "GS Var #08", "Ukulele"},
    {'M',   8,   0,  25, "GS Var #08", "12-str.Gt"},
    {'M',   8,   0,  26, "GS Var #08", "Hawaiian Gt."},
    {'M',   8,   0,  27, "GS Var #08", "Chorus Gt."},
    {'M',   8,   0,  28, "GS Var #08", "Funk Gt."},
    {'M',   8,   0,  30, "GS Var #08", "Feedback Gt."},
    {'M',   8,   0,  31, "GS Var #08", "Gt.Feedback"},
    {'M',   8,   0,  38, "GS Var #08", "Synth Bass 3"},
    {'M',   8,   0,  39, "GS Var #08", "Synth Bass 4"},
    {'M',   8,   0,  40, "GS Var #08", "Slow Violine"},
    {'M',   8,   0,  48, "GS Var #08", "Orchestra"},
    {'M',   8,   0,  50, "GS Var #08", "Syn.Strings3"},
    {'M',   8,   0,  61, "GS Var #08", "Brass 2"},
    {'M',   8,   0,  62, "GS Var #08", "Synth Brass3"},
    {'M',   8,   0,  63, "GS Var #08", "Synth Brass4"},
    {'M',   8,   0,  80, "GS Var #08", "Sine Wave"},
    {'M',   8,   0,  81, "GS Var #08", "Doctor Solo"},
    {'M',   8,   0, 107, "GS Var #08", "Taisho Koto"},
    {'M',   8,   0, 115, "GS Var #08", "Castanets"},
    {'M',   8,   0, 116, "GS Var #08", "Concert BD"},
    {'M',   8,   0, 117, "GS Var #08", "Melo. Tom 2"},
    {'M',   8,   0, 118, "GS Var #08", "808 Tom"},
    {'M',   8,   0, 125, "GS Var #08", "Starship"},
    {'M',   9,   0,  14, "GS Var #09", "Carillon"},
    {'M',   9,   0, 118, "GS Var #09", "Elec Perc"},
    {'M',   9,   0, 125, "GS Var #09", "Burst Noise"},
    {'M',  16,   0,   0, "GS Var #16", "Piano 1d"},
    {'M',  16,   0,   4, "GS Var #16", "E.Piano 1w"},
    {'M',  16,   0,   5, "GS Var #16", "E.Piano 2w"},
    {'M',  16,   0,   6, "GS Var #16", "Harpsi.w"},
    {'M',  16,   0,  16, "GS Var #16", "60's Organ 1"},
    {'M',  16,   0,  19, "GS Var #16", "Church Org.3"},
    {'M',  16,   0,  24, "GS Var #16", "Nylon Gt.o"},
    {'M',  16,   0,  25, "GS Var #16", "Mandolin"},
    {'M',  16,   0,  28, "GS Var #16", "Funk Gt.2"},
    {'M',  16,   0,  39, "GS Var #16", "Rubber Bass"},
    {'M',  16,   0,  62, "GS Var #16", "AnalogBrass1"},
    {'M',  16,   0,  63, "GS Var #16", "AnalogBrass2"},
    {'M',  24,   0,   4, "GS Var #24", "60's E.Piano"},
    {'M',  24,   0,   6, "GS Var #24", "Harpsi.o"},
    {'M',  32,   0,  16, "GS Var #32", "Organ 4"},
    {'M',  32,   0,  17, "GS Var #32", "Organ 5"},
    {'M',  32,   0,  24, "GS Var #32", "Nylon Gt.2"},
    {'M',  32,   0,  52, "GS Var #32", "Choir Aahs 2"},
    {'P',   0,   8,  41, "GS Room Set", "Room Low Tom 2"},
    {'P',   0,   8,  43, "GS Room Set", "Room Low Tom 1"},
    {'P',   0,   8,  45, "GS Room Set", "Room Mid Tom 2"},
    {'P',   0,   8,  47, "GS Room Set", "Room Mid Tom 1"},
    {'P',   0,   8,  48, "GS Room Set", "Room Hi Tom 2"},
    {'P',   0,   8,  50, "GS Room Set", "Room Hi Tom 1"},
    {'P',   0,  16,  36, "GS Power Set", "MONDO Kick"},
    {'P',   0,  16,  38, "GS Power Set", "Gated SD"},
    {'P',   0,  16,  41, "GS Power Set", "Room Low Tom 2"},
    {'P',   0,  16,  43, "GS Power Set", "Room Low Tom 1"},
    {'P',   0,  16,  45, "GS Power Set", "Room Mid Tom 2"},
    {'P',   0,  16,  47, "GS Power Set", "Room Mid Tom 1"},
    {'P',   0,  16,  48, "GS Power Set", "Room Hi Tom 2"},
    {'P',   0,  16,  50, "GS Power Set", "Room Hi Tom 1"},
    {'P',   0,  24,  36, "GS Electronic Set", "Elec BD"},
    {'P',   0,  24,  38, "GS Electronic Set", "Elec SD"},
    {'P',   0,  24,  40, "GS Electronic Set", "Gated SD"},
    {'P',   0,  24,  41, "GS Electronic Set", "Elec Low Tom 2"},
    {'P',   0,  24,  43, "GS Electronic Set", "Elec Low Tom 1"},
    {'P',   0,  24,  45, "GS Electronic Set", "Elec Mid Tom 2"},
    {'P',   0,  24,  47, "GS Electronic Set", "Elec Mid Tom 1"},
    {'P',   0,  24,  48, "GS Electronic Set", "Elec Hi Tom 2"},
    {'P',   0,  24,  50, "GS Electronic Set", "Elec Hi Tom 1"},
    {'P',   0,  24,  52, "GS Electronic Set", "Reverse Cymbal"},
    {'P',   0,  25,  36, "GS TR-808 Set", "808 Bass Drum"},
    {'P',   0,  25,  37, "GS TR-808 Set", "808 Rim Shot"},
    {'P',   0,  25,  38, "GS TR-808 Set", "808 Snare Drum"},
    {'P',   0,  25,  41, "GS TR-808 Set", "808 Low Tom 2"},
    {'P',   0,  25,  42, "GS TR-808 Set", "808 CHH"},
    {'P',   0,  25,  43, "GS TR-808 Set", "808 Low Tom 1"},
    {'P',   0,  25,  44, "GS TR-808 Set", "808 CHH"},
    {'P',   0,  25,  45, "GS TR-808 Set", "808 Mid Tom 2"},
    {'P',   0,  25,  46, "GS TR-808 Set", "808 OHH"},
    {'P',   0,  25,  47, "GS TR-808 Set", "808 Mid Tom 1"},
    {'P',   0,  25,  48, "GS TR-808 Set", "808 Hi Tom 2"},
    {'P',   0,  25,  49, "GS TR-808 Set", "808 Cymbal"},
    {'P',   0,  25,  50, "GS TR-808 Set", "808 Hi Tom 1"},
    {'P',   0,  25,  56, "GS TR-808 Set", "808 Cowbell"},
    {'P',   0,  25,  62, "GS TR-808 Set", "808 High Conga"},
    {'P',   0,  25,  63, "GS TR-808 Set", "808 Mid Conga"},
    {'P',   0,  25,  64, "GS TR-808 Set", "808 Low Conga"},
    {'P',   0,  25,  70, "GS TR-808 Set", "808 Maracas"},
    {'P',   0,  25,  75, "GS TR-808 Set", "808 Claves"},
    {'P',   0,  32,  35, "GS Jazz Set", "Jazz BD 2"},
    {'P',   0,  32,  36, "GS Jazz Set", "Jazz BD 1"},
    {'P',   0,  40,  35, "GS Brush Set", "Jazz BD 2"},
    {'P',   0,  40,  36, "GS Brush Set", "Jazz BD 1"},
    {'P',   0,  40,  38, "GS Brush Set", "Brush Tap"},
    {'P',   0,  40,  39, "GS Brush Set", "Brush Slap"},
    {'P',   0,  40,  40, "GS Brush Set", "Brush Swirl"},
    {'P',   0,  48,  27, "GS Orchestra Set", "Closed Hi-Hat"},
    {'P',   0,  48,  28, "GS Orchestra Set", "Pedal Hi-Hat"},
    {'P',   0,  48,  29, "GS Orchestra Set", "Open Hi-Hat"},
    {'P',   0,  48,  30, "GS Orchestra Set", "Ride Cymbal"},
    {'P',   0,  48,  35, "GS Orchestra Set", "Concert BD 2"},
    {'P',   0,  48,  36, "GS Orchestra Set", "Concert BD 1"},
    {'P',   0,  48,  38, "GS Orchestra Set", "Concert SD"},
    {'P',   0,  48,  39, "GS Orchestra Set", "Castanets"},
    {'P',   0,  48,  40, "GS Orchestra Set", "Concert SD"},
    {'P',   0,  48,  41, "GS Orchestra Set", "Timpani F"},
    {'P',   0,  48,  42, "GS Orchestra Set", "Timpani F#"},
    {'P',   0,  48,  43, "GS Orchestra Set", "Timpani G"},
    {'P',   0,  48,  44, "GS Orchestra Set", "Timpani G#"},
    {'P',   0,  48,  45, "GS Orchestra Set", "Timpani A"},
    {'P',   0,  48,  46, "GS Orchestra Set", "Timpani A#"},
    {'P',   0,  48,  47, "GS Orchestra Set", "Timpani B"},
    {'P',   0,  48,  48, "GS Orchestra Set", "Timpani c"},
    {'P',   0,  48,  49, "GS Orchestra Set", "Timpani c#"},
    {'P',   0,  48,  50, "GS Orchestra Set", "Timpani d"},
    {'P',   0,  48,  51, "GS Orchestra Set", "Timpani d#"},
    {'P',   0,  48,  52, "GS Orchestra Set", "Timpani e"},
    {'P',   0,  48,  53, "GS Orchestra Set", "Timpani f"},
    {'P',   0,  48,  57, "GS Orchestra Set", "Concert Cymbal 2"},
    {'P',   0,  48,  59, "GS Orchestra Set", "Concert Cymbal 1"},
    {'P',   0,  48,  88, "GS Orchestra Set", "Applause"},
    {'P',   0,  56,  39, "GS SFX Set", "High Q"},
    {'P',   0,  56,  40, "GS SFX Set", "Slap"},
    {'P',   0,  56,  41, "GS SFX Set", "Scratch Push"},
    {'P',   0,  56,  42, "GS SFX Set", "Scratch Pull"},
    {'P',   0,  56,  43, "GS SFX Set", "Sticks"},
    {'P',   0,  56,  44, "GS SFX Set", "Square Click"},
    {'P',   0,  56,  45, "GS SFX Set", "Metronome Click"},
    {'P',   0,  56,  46, "GS SFX Set", "Metronome Bell"},
    {'P',   0,  56,  47, "GS SFX Set", "Guitar Sliding Finger"},
    {'P',   0,  56,  48, "GS SFX Set", "Guitar Cutting Noise (Down)"},
    {'P',   0,  56,  49, "GS SFX Set", "Guitar Cutting Noise (Up)"},
    {'P',   0,  56,  50, "GS SFX Set", "String Slap of Double Bass"},
    {'P',   0,  56,  51, "GS SFX Set", "Flute Key click"},
    {'P',   0,  56,  52, "GS SFX Set", "Laughing"},
    {'P',   0,  56,  53, "GS SFX Set", "Screaming"},
    {'P',   0,  56,  54, "GS SFX Set", "Punch"},
    {'P',   0,  56,  55, "GS SFX Set", "Heart Beat"},
    {'P',   0,  56,  56, "GS SFX Set", "Footsteps 1"},
    {'P',   0,  56,  57, "GS SFX Set", "Footsteps 2"},
    {'P',   0,  56,  58, "GS SFX Set", "Applause"},
    {'P',   0,  56,  59, "GS SFX Set", "Door Creaking"},
    {'P',   0,  56,  60, "GS SFX Set", "Door"},
    {'P',   0,  56,  61, "GS SFX Set", "Scratch"},
    {'P',   0,  56,  62, "GS SFX Set", "Windchime"},
    {'P',   0,  56,  63, "GS SFX Set", "Car-Engine"},
    {'P',   0,  56,  64, "GS SFX Set", "Car-Stop"},
    {'P',   0,  56,  65, "GS SFX Set", "Car-Pass"},
    {'P',   0,  56,  66, "GS SFX Set", "Car-Crash"},
    {'P',   0,  56,  67, "GS SFX Set", "Siren"},
    {'P',   0,  56,  68, "GS SFX Set", "Train"},
    {'P',   0,  56,  69, "GS SFX Set", "Jetplane"},
    {'P',   0,  56,  70, "GS SFX Set", "Helicopter"},
    {'P',   0,  56,  71, "GS SFX Set", "Starship"},
    {'P',   0,  56,  72, "GS SFX Set", "Gun Shot"},
    {'P',   0,  56,  73, "GS SFX Set", "Machine Gun"},
    {'P',   0,  56,  74, "GS SFX Set", "Lasergun"},
    {'P',   0,  56,  75, "GS SFX Set", "Explosion"},
    {'P',   0,  56,  76, "GS SFX Set", "Dog"},
    {'P',   0,  56,  77, "GS SFX Set", "Horse-Gallop"},
    {'P',   0,  56,  78, "GS SFX Set", "Birds"},
    {'P',   0,  56,  79, "GS SFX Set", "Rain"},
    {'P',   0,  56,  80, "GS SFX Set", "Thunder"},
    {'P',   0,  56,  81, "GS SFX Set", "Wind"},
    {'P',   0,  56,  82, "GS SFX Set", "Seashore"},
    {'P',   0,  56,  83, "GS SFX Set", "Stream"},
    {'P',   0,  56,  84, "GS SFX Set", "Bubble"},
};

static const MidiProgram ScSet[] =
{
    {'M',   0,   1,   0, "SC-55 Map Capital Tones", "Piano 1"},
    {'M',   0,   1,   1, "SC-55 Map Capital Tones", "Piano 2"},
    {'M',   0,   1,   2, "SC-55 Map Capital Tones", "Piano 3"},
    {'M',   0,   1,   3, "SC-55 Map Capital Tones", "Honky-tonk"},
    {'M',   0,   1,   4, "SC-55 Map Capital Tones", "E.Piano 1"},
    {'M',   0,   1,   5, "SC-55 Map Capital Tones", "E.Piano 2"},
    {'M',   0,   1,   6, "SC-55 Map Capital Tones", "Harpsichord"},
    {'M',   0,   1,   7, "SC-55 Map Capital Tones", "Clav."},
    {'M',   0,   1,   8, "SC-55 Map Capital Tones", "Celesta"},
    {'M',   0,   1,   9, "SC-55 Map Capital Tones", "Glockenspil"},
    {'M',   0,   1,  10, "SC-55 Map Capital Tones", "Music Box"},
    {'M',   0,   1,  11, "SC-55 Map Capital Tones", "Vibraphone"},
    {'M',   0,   1,  12, "SC-55 Map Capital Tones", "Marimba"},
    {'M',   0,   1,  13, "SC-55 Map Capital Tones", "Xylophone"},
    {'M',   0,   1,  14, "SC-55 Map Capital Tones", "Tubularbell"},
    {'M',   0,   1,  15, "SC-55 Map Capital Tones", "Santur"},
    {'M',   0,   1,  16, "SC-55 Map Capital Tones", "Organ 1"},
    {'M',   0,   1,  17, "SC-55 Map Capital Tones", "Organ 2"},
    {'M',   0,   1,  18, "SC-55 Map Capital Tones", "Organ 3"},
    {'M',   0,   1,  19, "SC-55 Map Capital Tones", "Church Org1"},
    {'M',   0,   1,  20, "SC-55 Map Capital Tones", "Reed Organ"},
    {'M',   0,   1,  21, "SC-55 Map Capital Tones", "Accordion F"},
    {'M',   0,   1,  22, "SC-55 Map Capital Tones", "Harmonica"},
    {'M',   0,   1,  23, "SC-55 Map Capital Tones", "Bandoneon"},
    {'M',   0,   1,  24, "SC-55 Map Capital Tones", "Nylon Gt."},
    {'M',   0,   1,  25, "SC-55 Map Capital Tones", "Steel Gt."},
    {'M',   0,   1,  26, "SC-55 Map Capital Tones", "Jazz Gt."},
    {'M',   0,   1,  27, "SC-55 Map Capital Tones", "Clean Gt."},
    {'M',   0,   1,  28, "SC-55 Map Capital Tones", "Muted Gt."},
    {'M',   0,   1,  29, "SC-55 Map Capital Tones", "OverdriveGt"},
    {'M',   0,   1,  30, "SC-55 Map Capital Tones", "Dist.Gt."},
    {'M',   0,   1,  31, "SC-55 Map Capital Tones", "Gt.Harmonix"},
    {'M',   0,   1,  32, "SC-55 Map Capital Tones", "Acoustic Bs"},
    {'M',   0,   1,  33, "SC-55 Map Capital Tones", "Fingered Bs"},
    {'M',   0,   1,  34, "SC-55 Map Capital Tones", "Picked Bass"},
    {'M',   0,   1,  35, "SC-55 Map Capital Tones", "Fretless Bs."},
    {'M',   0,   1,  36, "SC-55 Map Capital Tones", "Slap Bass 1"},
    {'M',   0,   1,  37, "SC-55 Map Capital Tones", "Slap Bass 2"},
    {'M',   0,   1,  38, "SC-55 Map Capital Tones", "Syn.Bass 1"},
    {'M',   0,   1,  39, "SC-55 Map Capital Tones", "Syn.Bass 2"},
    {'M',   0,   1,  40, "SC-55 Map Capital Tones", "Violin"},
    {'M',   0,   1,  41, "SC-55 Map Capital Tones", "Viola"},
    {'M',   0,   1,  42, "SC-55 Map Capital Tones", "Cello"},
    {'M',   0,   1,  43, "SC-55 Map Capital Tones", "Contrabass"},
    {'M',   0,   1,  44, "SC-55 Map Capital Tones", "Tremolo Str"},
    {'M',   0,   1,  45, "SC-55 Map Capital Tones", "Pizzicato"},
    {'M',   0,   1,  46, "SC-55 Map Capital Tones", "Harp"},
    {'M',   0,   1,  47, "SC-55 Map Capital Tones", "Timpani"},
    {'M',   0,   1,  48, "SC-55 Map Capital Tones", "Strings"},
    {'M',   0,   1,  49, "SC-55 Map Capital Tones", "SlowStrings"},
    {'M',   0,   1,  50, "SC-55 Map Capital Tones", "SynStrings1"},
    {'M',   0,   1,  51, "SC-55 Map Capital Tones", "SynStrings2"},
    {'M',   0,   1,  52, "SC-55 Map Capital Tones", "Choir Aahs"},
    {'M',   0,   1,  53, "SC-55 Map Capital Tones", "Voice Oohs"},
    {'M',   0,   1,  54, "SC-55 Map Capital Tones", "SynVox"},
    {'M',   0,   1,  55, "SC-55 Map Capital Tones", "Orchest.Hit"},
    {'M',   0,   1,  56, "SC-55 Map Capital Tones", "Trumpet"},
    {'M',   0,   1,  57, "SC-55 Map Capital Tones", "Trombone"},
    {'M',   0,   1,  58, "SC-55 Map Capital Tones", "Tuba"},
    {'M',   0,   1,  59, "SC-55 Map Capital Tones", "MuteTrumpet"},
    {'M',   0,   1,  60, "SC-55 Map Capital Tones", "French Horn"},
    {'M',   0,   1,  61, "SC-55 Map Capital Tones", "Brass 1"},
    {'M',   0,   1,  62, "SC-55 Map Capital Tones", "Syn.Brass 1"},
    {'M',   0,   1,  63, "SC-55 Map Capital Tones", "Syn.Brass 2"},
    {'M',   0,   1,  64, "SC-55 Map Capital Tones", "Soprano Sax"},
    {'M',   0,   1,  65, "SC-55 Map Capital Tones", "Alto Sax"},
    {'M',   0,   1,  66, "SC-55 Map Capital Tones", "Tenor Sax"},
    {'M',   0,   1,  67, "SC-55 Map Capital Tones", "BaritoneSax"},
    {'M',   0,   1,  68, "SC-55 Map Capital Tones", "Oboe"},
    {'M',   0,   1,  69, "SC-55 Map Capital Tones", "English Horn"},
    {'M',   0,   1,  70, "SC-55 Map Capital Tones", "Bassoon"},
    {'M',   0,   1,  71, "SC-55 Map Capital Tones", "Clarinet"},
    {'M',   0,   1,  72, "SC-55 Map Capital Tones", "Piccolo"},
    {'M',   0,   1,  73, "SC-55 Map Capital Tones", "Flute"},
    {'M',   0,   1,  74, "SC-55 Map Capital Tones", "Recorder"},
    {'M',   0,   1,  75, "SC-55 Map Capital Tones", "Pan Flute"},
    {'M',   0,   1,  76, "SC-55 Map Capital Tones", "Bottle Blow"},
    {'M',   0,   1,  77, "SC-55 Map Capital Tones", "Shakuhachi"},
    {'M',   0,   1,  78, "SC-55 Map Capital Tones", "Whistle"},
    {'M',   0,   1,  79, "SC-55 Map Capital Tones", "Ocarina"},
    {'M',   0,   1,  80, "SC-55 Map Capital Tones", "Square Wave"},
    {'M',   0,   1,  81, "SC-55 Map Capital Tones", "Saw Wave"},
    {'M',   0,   1,  82, "SC-55 Map Capital Tones", "SynCalliope"},
    {'M',   0,   1,  83, "SC-55 Map Capital Tones", "Chiffer Lead"},
    {'M',   0,   1,  84, "SC-55 Map Capital Tones", "Charang"},
    {'M',   0,   1,  85, "SC-55 Map Capital Tones", "Solo Vox"},
    {'M',   0,   1,  86, "SC-55 Map Capital Tones", "5th Saw"},
    {'M',   0,   1,  87, "SC-55 Map Capital Tones", "Bass & Lead"},
    {'M',   0,   1,  88, "SC-55 Map Capital Tones", "Fantasia"},
    {'M',   0,   1,  89, "SC-55 Map Capital Tones", "Warm Pad"},
    {'M',   0,   1,  90, "SC-55 Map Capital Tones", "Polysynth"},
    {'M',   0,   1,  91, "SC-55 Map Capital Tones", "Space Voice"},
    {'M',   0,   1,  92, "SC-55 Map Capital Tones", "Bowed Glass"},
    {'M',   0,   1,  93, "SC-55 Map Capital Tones", "Metal Pad"},
    {'M',   0,   1,  94, "SC-55 Map Capital Tones", "Halo Pad"},
    {'M',   0,   1,  95, "SC-55 Map Capital Tones", "Sweep Pad"},
    {'M',   0,   1,  96, "SC-55 Map Capital Tones", "Ice Rain"},
    {'M',   0,   1,  97, "SC-55 Map Capital Tones", "Soundtrack"},
    {'M',   0,   1,  98, "SC-55 Map Capital Tones", "Crystal"},
    {'M',   0,   1,  99, "SC-55 Map Capital Tones", "Atmosphere"},
    {'M',   0,   1, 100, "SC-55 Map Capital Tones", "Brightness"},
    {'M',   0,   1, 101, "SC-55 Map Capital Tones", "Goblin"},
    {'M',   0,   1, 102, "SC-55 Map Capital Tones", "Echo Drops"},
    {'M',   0,   1, 103, "SC-55 Map Capital Tones", "Star Theme"},
    {'M',   0,   1, 104, "SC-55 Map Capital Tones", "Sitar"},
    {'M',   0,   1, 105, "SC-55 Map Capital Tones", "Banjo"},
    {'M',   0,   1, 106, "SC-55 Map Capital Tones", "Shamisen"},
    {'M',   0,   1, 107, "SC-55 Map Capital Tones", "Koto"},
    {'M',   0,   1, 108, "SC-55 Map Capital Tones", "Kalimba"},
    {'M',   0,   1, 109, "SC-55 Map Capital Tones", "Bagpipe"},
    {'M',   0,   1, 110, "SC-55 Map Capital Tones", "Fiddle"},
    {'M',   0,   1, 111, "SC-55 Map Capital Tones", "Shanai"},
    {'M',   0,   1, 112, "SC-55 Map Capital Tones", "Tinkle Bell"},
    {'M',   0,   1, 113, "SC-55 Map Capital Tones", "Agogo"},
    {'M',   0,   1, 114, "SC-55 Map Capital Tones", "Steel Drums"},
    {'M',   0,   1, 115, "SC-55 Map Capital Tones", "Woodblock"},
    {'M',   0,   1, 116, "SC-55 Map Capital Tones", "Taiko"},
    {'M',   0,   1, 117, "SC-55 Map Capital Tones", "Melo. Tom 1"},
    {'M',   0,   1, 118, "SC-55 Map Capital Tones", "Synth Drum"},
    {'M',   0,   1, 119, "SC-55 Map Capital Tones", "Reverse Cym"},
    {'M',   0,   1, 120, "SC-55 Map Capital Tones", "Gt.FretNoiz"},
    {'M',   0,   1, 121, "SC-55 Map Capital Tones", "BreathNoise"},
    {'M',   0,   1, 122, "SC-55 Map Capital Tones", "Seashore"},
    {'M',   0,   1, 123, "SC-55 Map Capital Tones", "Bird"},
    {'M',   0,   1, 124, "SC-55 Map Capital Tones", "Telephone 1"},
    {'M',   0,   1, 125, "SC-55 Map Capital Tones", "Helicopter"},
    {'M',   0,   1, 126, "SC-55 Map Capital Tones", "Applause"},
    {'M',   0,   1, 127, "SC-55 Map Capital Tones", "Gun Shot"},
    {'M',   0,   2,   0, "SC-88 Map Capital Tones", "Piano 1"},
    {'M',   0,   2,   1, "SC-88 Map Capital Tones", "Piano 2"},
    {'M',   0,   2,   2, "SC-88 Map Capital Tones", "Piano 3"},
    {'M',   0,   2,   3, "SC-88 Map Capital Tones", "Honky-tonk"},
    {'M',   0,   2,   4, "SC-88 Map Capital Tones", "E.Piano 1"},
    {'M',   0,   2,   5, "SC-88 Map Capital Tones", "E.Piano 2"},
    {'M',   0,   2,   6, "SC-88 Map Capital Tones", "Harpsichord"},
    {'M',   0,   2,   7, "SC-88 Map Capital Tones", "Clav."},
    {'M',   0,   2,   8, "SC-88 Map Capital Tones", "Celesta"},
    {'M',   0,   2,   9, "SC-88 Map Capital Tones", "Glockenspiel"},
    {'M',   0,   2,  10, "SC-88 Map Capital Tones", "Music Box"},
    {'M',   0,   2,  11, "SC-88 Map Capital Tones", "Vibraphone"},
    {'M',   0,   2,  12, "SC-88 Map Capital Tones", "Marimba"},
    {'M',   0,   2,  13, "SC-88 Map Capital Tones", "Xylophone"},
    {'M',   0,   2,  14, "SC-88 Map Capital Tones", "Tubular-bell"},
    {'M',   0,   2,  15, "SC-88 Map Capital Tones", "Santur"},
    {'M',   0,   2,  16, "SC-88 Map Capital Tones", "Organ 1"},
    {'M',   0,   2,  17, "SC-88 Map Capital Tones", "Organ 2"},
    {'M',   0,   2,  18, "SC-88 Map Capital Tones", "Organ 3"},
    {'M',   0,   2,  19, "SC-88 Map Capital Tones", "Church Org.1"},
    {'M',   0,   2,  20, "SC-88 Map Capital Tones", "Reed Organ"},
    {'M',   0,   2,  21, "SC-88 Map Capital Tones", "AccordionFr"},
    {'M',   0,   2,  22, "SC-88 Map Capital Tones", "Harmonica"},
    {'M',   0,   2,  23, "SC-88 Map Capital Tones", "Bandoneon"},
    {'M',   0,   2,  24, "SC-88 Map Capital Tones", "Nylonstr.Gt"},
    {'M',   0,   2,  25, "SC-88 Map Capital Tones", "Steelstr.Gt"},
    {'M',   0,   2,  26, "SC-88 Map Capital Tones", "Jazz Gt."},
    {'M',   0,   2,  27, "SC-88 Map Capital Tones", "Clean Gt."},
    {'M',   0,   2,  28, "SC-88 Map Capital Tones", "Muted Gt."},
    {'M',   0,   2,  29, "SC-88 Map Capital Tones", "Overdrive Gt"},
    {'M',   0,   2,  30, "SC-88 Map Capital Tones", "DistortionGt"},
    {'M',   0,   2,  31, "SC-88 Map Capital Tones", "Gt.Harmonix"},
    {'M',   0,   2,  32, "SC-88 Map Capital Tones", "AcousticBs."},
    {'M',   0,   2,  33, "SC-88 Map Capital Tones", "FingeredBs."},
    {'M',   0,   2,  34, "SC-88 Map Capital Tones", "Picked Bass"},
    {'M',   0,   2,  35, "SC-88 Map Capital Tones", "FretlessBs."},
    {'M',   0,   2,  36, "SC-88 Map Capital Tones", "Slap Bass 1"},
    {'M',   0,   2,  37, "SC-88 Map Capital Tones", "Slap Bass 2"},
    {'M',   0,   2,  38, "SC-88 Map Capital Tones", "SynthBass 1"},
    {'M',   0,   2,  39, "SC-88 Map Capital Tones", "SynthBass 2"},
    {'M',   0,   2,  40, "SC-88 Map Capital Tones", "Violin"},
    {'M',   0,   2,  41, "SC-88 Map Capital Tones", "Viola"},
    {'M',   0,   2,  42, "SC-88 Map Capital Tones", "Cello"},
    {'M',   0,   2,  43, "SC-88 Map Capital Tones", "Contrabass"},
    {'M',   0,   2,  44, "SC-88 Map Capital Tones", "Tremolo Str"},
    {'M',   0,   2,  45, "SC-88 Map Capital Tones", "Pizz. Str."},
    {'M',   0,   2,  46, "SC-88 Map Capital Tones", "Harp"},
    {'M',   0,   2,  47, "SC-88 Map Capital Tones", "Timpani"},
    {'M',   0,   2,  48, "SC-88 Map Capital Tones", "Strings"},
    {'M',   0,   2,  49, "SC-88 Map Capital Tones", "SlowStrings"},
    {'M',   0,   2,  50, "SC-88 Map Capital Tones", "SynStrings1"},
    {'M',   0,   2,  51, "SC-88 Map Capital Tones", "SynStrings2"},
    {'M',   0,   2,  52, "SC-88 Map Capital Tones", "Choir Aahs"},
    {'M',   0,   2,  53, "SC-88 Map Capital Tones", "Voice Oohs"},
    {'M',   0,   2,  54, "SC-88 Map Capital Tones", "SynVox"},
    {'M',   0,   2,  55, "SC-88 Map Capital Tones", "Orch. Hit"},
    {'M',   0,   2,  56, "SC-88 Map Capital Tones", "Trumpet"},
    {'M',   0,   2,  57, "SC-88 Map Capital Tones", "Trombone"},
    {'M',   0,   2,  58, "SC-88 Map Capital Tones", "Tuba"},
    {'M',   0,   2,  59, "SC-88 Map Capital Tones", "Muted Tp."},
    {'M',   0,   2,  60, "SC-88 Map Capital Tones", "FrenchHorn"},
    {'M',   0,   2,  61, "SC-88 Map Capital Tones", "Brass 1"},
    {'M',   0,   2,  62, "SC-88 Map Capital Tones", "SynthBrass1"},
    {'M',   0,   2,  63, "SC-88 Map Capital Tones", "Syn.Brass 2"},
    {'M',   0,   2,  64, "SC-88 Map Capital Tones", "Soprano Sax"},
    {'M',   0,   2,  65, "SC-88 Map Capital Tones", "Alto Sax"},
    {'M',   0,   2,  66, "SC-88 Map Capital Tones", "Tenor Sax"},
    {'M',   0,   2,  67, "SC-88 Map Capital Tones", "BaritoneSax"},
    {'M',   0,   2,  68, "SC-88 Map Capital Tones", "Oboe"},
    {'M',   0,   2,  69, "SC-88 Map Capital Tones", "EnglishHorn"},
    {'M',   0,   2,  70, "SC-88 Map Capital Tones", "Bassoon"},
    {'M',   0,   2,  71, "SC-88 Map Capital Tones", "Clarinet"},
    {'M',   0,   2,  72, "SC-88 Map Capital Tones", "Piccolo"},
    {'M',   0,   2,  73, "SC-88 Map Capital Tones", "Flute"},
    {'M',   0,   2,  74, "SC-88 Map Capital Tones", "Recorder"},
    {'M',   0,   2,  75, "SC-88 Map Capital Tones", "Pan Flute"},
    {'M',   0,   2,  76, "SC-88 Map Capital Tones", "Bottle Blow"},
    {'M',   0,   2,  77, "SC-88 Map Capital Tones", "Shakuhachi"},
    {'M',   0,   2,  78, "SC-88 Map Capital Tones", "Whistle"},
    {'M',   0,   2,  79, "SC-88 Map Capital Tones", "Ocarina"},
    {'M',   0,   2,  80, "SC-88 Map Capital Tones", "Square Wave"},
    {'M',   0,   2,  81, "SC-88 Map Capital Tones", "Saw Wave"},
    {'M',   0,   2,  82, "SC-88 Map Capital Tones", "SynCalliope"},
    {'M',   0,   2,  83, "SC-88 Map Capital Tones", "ChifferLead"},
    {'M',   0,   2,  84, "SC-88 Map Capital Tones", "Charang"},
    {'M',   0,   2,  85, "SC-88 Map Capital Tones", "Solo Vox"},
    {'M',   0,   2,  86, "SC-88 Map Capital Tones", "5th Saw"},
    {'M',   0,   2,  87, "SC-88 Map Capital Tones", "Bass & Lead"},
    {'M',   0,   2,  88, "SC-88 Map Capital Tones", "Fantasia"},
    {'M',   0,   2,  89, "SC-88 Map Capital Tones", "Warm Pad"},
    {'M',   0,   2,  90, "SC-88 Map Capital Tones", "Polysynth"},
    {'M',   0,   2,  91, "SC-88 Map Capital Tones", "Space Voice"},
    {'M',   0,   2,  92, "SC-88 Map Capital Tones", "Bowed Glass"},
    {'M',   0,   2,  93, "SC-88 Map Capital Tones", "Metal Pad"},
    {'M',   0,   2,  94, "SC-88 Map Capital Tones", "Halo Pad"},
    {'M',   0,   2,  95, "SC-88 Map Capital Tones", "Sweep Pad"},
    {'M',   0,   2,  96, "SC-88 Map Capital Tones", "Ice Rain"},
    {'M',   0,   2,  97, "SC-88 Map Capital Tones", "Soundtrack"},
    {'M',   0,   2,  98, "SC-88 Map Capital Tones", "Crystal"},
    {'M',   0,   2,  99, "SC-88 Map Capital Tones", "Atmosphere"},
    {'M',   0,   2, 100, "SC-88 Map Capital Tones", "Brightness"},
    {'M',   0,   2, 101, "SC-88 Map Capital Tones", "Goblin"},
    {'M',   0,   2, 102, "SC-88 Map Capital Tones", "Echo Drops"},
    {'M',   0,   2, 103, "SC-88 Map Capital Tones", "Star Theme"},
    {'M',   0,   2, 104, "SC-88 Map Capital Tones", "Sitar"},
    {'M',   0,   2, 105, "SC-88 Map Capital Tones", "Banjo"},
    {'M',   0,   2, 106, "SC-88 Map Capital Tones", "Shamisen"},
    {'M',   0,   2, 107, "SC-88 Map Capital Tones", "Koto"},
    {'M',   0,   2, 108, "SC-88 Map Capital Tones", "Kalimba"},
    {'M',   0,   2, 109, "SC-88 Map Capital Tones", "BagPipe"},
    {'M',   0,   2, 110, "SC-88 Map Capital Tones", "Fiddle"},
    {'M',   0,   2, 111, "SC-88 Map Capital Tones", "Shanai"},
    {'M',   0,   2, 112, "SC-88 Map Capital Tones", "Tinkle Bell"},
    {'M',   0,   2, 113, "SC-88 Map Capital Tones", "Agogo"},
    {'M',   0,   2, 114, "SC-88 Map Capital Tones", "Steel Drums"},
    {'M',   0,   2, 115, "SC-88 Map Capital Tones", "Woodblock"},
    {'M',   0,   2, 116, "SC-88 Map Capital Tones", "Taiko"},
    {'M',   0,   2, 117, "SC-88 Map Capital Tones", "Melo. Tom 1"},
    {'M',   0,   2, 118, "SC-88 Map Capital Tones", "Synth Drum"},
    {'M',   0,   2, 119, "SC-88 Map Capital Tones", "Reverse Cym."},
    {'M',   0,   2, 120, "SC-88 Map Capital Tones", "Gt.FretNoiz"},
    {'M',   0,   2, 121, "SC-88 Map Capital Tones", "BreathNoise"},
    {'M',   0,   2, 122, "SC-88 Map Capital Tones", "Seashore"},
    {'M',   0,   2, 123, "SC-88 Map Capital Tones", "Bird"},
    {'M',   0,   2, 124, "SC-88 Map Capital Tones", "Telephone 1"},
    {'M',   0,   2, 125, "SC-88 Map Capital Tones", "Helicopter"},
    {'M',   0,   2, 126, "SC-88 Map Capital Tones", "Applause"},
    {'M',   0,   2, 127, "SC-88 Map Capital Tones", "Gun Shot"},
    {'M',   0,   3,   0, "SC-88Pro Map Capital Tones", "Piano 1"},
    {'M',   0,   3,   1, "SC-88Pro Map Capital Tones", "Piano 2"},
    {'M',   0,   3,   2, "SC-88Pro Map Capital Tones", "Piano 3"},
    {'M',   0,   3,   3, "SC-88Pro Map Capital Tones", "Honky-tonk"},
    {'M',   0,   3,   4, "SC-88Pro Map Capital Tones", "E.Piano 1"},
    {'M',   0,   3,   5, "SC-88Pro Map Capital Tones", "E.Piano 2"},
    {'M',   0,   3,   6, "SC-88Pro Map Capital Tones", "Harpsichord"},
    {'M',   0,   3,   7, "SC-88Pro Map Capital Tones", "Clav."},
    {'M',   0,   3,   8, "SC-88Pro Map Capital Tones", "Celesta"},
    {'M',   0,   3,   9, "SC-88Pro Map Capital Tones", "Glockenspiel"},
    {'M',   0,   3,  10, "SC-88Pro Map Capital Tones", "Music Box"},
    {'M',   0,   3,  11, "SC-88Pro Map Capital Tones", "Vibraphone"},
    {'M',   0,   3,  12, "SC-88Pro Map Capital Tones", "Marimba"},
    {'M',   0,   3,  13, "SC-88Pro Map Capital Tones", "Xylophone"},
    {'M',   0,   3,  14, "SC-88Pro Map Capital Tones", "Tubular-bell"},
    {'M',   0,   3,  15, "SC-88Pro Map Capital Tones", "Santur"},
    {'M',   0,   3,  16, "SC-88Pro Map Capital Tones", "Organ 1"},
    {'M',   0,   3,  17, "SC-88Pro Map Capital Tones", "Organ 2"},
    {'M',   0,   3,  18, "SC-88Pro Map Capital Tones", "Organ 3"},
    {'M',   0,   3,  19, "SC-88Pro Map Capital Tones", "Church Org.1"},
    {'M',   0,   3,  20, "SC-88Pro Map Capital Tones", "Reed Organ"},
    {'M',   0,   3,  21, "SC-88Pro Map Capital Tones", "Accordion Fr"},
    {'M',   0,   3,  22, "SC-88Pro Map Capital Tones", "Harmonica"},
    {'M',   0,   3,  23, "SC-88Pro Map Capital Tones", "Bandoneon"},
    {'M',   0,   3,  24, "SC-88Pro Map Capital Tones", "Nylon-str.Gt"},
    {'M',   0,   3,  25, "SC-88Pro Map Capital Tones", "Steel-str.Gt"},
    {'M',   0,   3,  26, "SC-88Pro Map Capital Tones", "Jazz Gt."},
    {'M',   0,   3,  27, "SC-88Pro Map Capital Tones", "Clean Gt."},
    {'M',   0,   3,  28, "SC-88Pro Map Capital Tones", "Muted Gt."},
    {'M',   0,   3,  29, "SC-88Pro Map Capital Tones", "OverdriveGt"},
    {'M',   0,   3,  30, "SC-88Pro Map Capital Tones", "DistortionGt"},
    {'M',   0,   3,  31, "SC-88Pro Map Capital Tones", "Gt.Harmonics"},
    {'M',   0,   3,  32, "SC-88Pro Map Capital Tones", "Acoustic Bs."},
    {'M',   0,   3,  33, "SC-88Pro Map Capital Tones", "Fingered Bs."},
    {'M',   0,   3,  34, "SC-88Pro Map Capital Tones", "Picked Bass"},
    {'M',   0,   3,  35, "SC-88Pro Map Capital Tones", "Fretless Bs."},
    {'M',   0,   3,  36, "SC-88Pro Map Capital Tones", "Slap Bass 1"},
    {'M',   0,   3,  37, "SC-88Pro Map Capital Tones", "Slap Bass 2"},
    {'M',   0,   3,  38, "SC-88Pro Map Capital Tones", "Synth Bass 1"},
    {'M',   0,   3,  39, "SC-88Pro Map Capital Tones", "Synth Bass 2"},
    {'M',   0,   3,  40, "SC-88Pro Map Capital Tones", "Violin    :"},
    {'M',   0,   3,  41, "SC-88Pro Map Capital Tones", "Viola     :"},
    {'M',   0,   3,  42, "SC-88Pro Map Capital Tones", "Cello     :"},
    {'M',   0,   3,  43, "SC-88Pro Map Capital Tones", "Contrabass"},
    {'M',   0,   3,  44, "SC-88Pro Map Capital Tones", "Tremolo Str"},
    {'M',   0,   3,  45, "SC-88Pro Map Capital Tones", "PizzicatoStr"},
    {'M',   0,   3,  46, "SC-88Pro Map Capital Tones", "Harp"},
    {'M',   0,   3,  47, "SC-88Pro Map Capital Tones", "Timpani"},
    {'M',   0,   3,  48, "SC-88Pro Map Capital Tones", "Strings   :"},
    {'M',   0,   3,  49, "SC-88Pro Map Capital Tones", "SlowStrings"},
    {'M',   0,   3,  50, "SC-88Pro Map Capital Tones", "Syn.Strings1"},
    {'M',   0,   3,  51, "SC-88Pro Map Capital Tones", "Syn.Strings2"},
    {'M',   0,   3,  52, "SC-88Pro Map Capital Tones", "Choir Aahs"},
    {'M',   0,   3,  53, "SC-88Pro Map Capital Tones", "Voice Oohs"},
    {'M',   0,   3,  54, "SC-88Pro Map Capital Tones", "SynVox"},
    {'M',   0,   3,  55, "SC-88Pro Map Capital Tones", "OrchestraHit"},
    {'M',   0,   3,  56, "SC-88Pro Map Capital Tones", "Trumpet"},
    {'M',   0,   3,  57, "SC-88Pro Map Capital Tones", "Trombone"},
    {'M',   0,   3,  58, "SC-88Pro Map Capital Tones", "Tuba"},
    {'M',   0,   3,  59, "SC-88Pro Map Capital Tones", "MutedTrumpet"},
    {'M',   0,   3,  60, "SC-88Pro Map Capital Tones", "French Horns"},
    {'M',   0,   3,  61, "SC-88Pro Map Capital Tones", "Brass 1"},
    {'M',   0,   3,  62, "SC-88Pro Map Capital Tones", "Synth Brass1"},
    {'M',   0,   3,  63, "SC-88Pro Map Capital Tones", "Synth Brass2"},
    {'M',   0,   3,  64, "SC-88Pro Map Capital Tones", "Soprano Sax"},
    {'M',   0,   3,  65, "SC-88Pro Map Capital Tones", "Alto Sax"},
    {'M',   0,   3,  66, "SC-88Pro Map Capital Tones", "Tenor Sax"},
    {'M',   0,   3,  67, "SC-88Pro Map Capital Tones", "Baritone Sax"},
    {'M',   0,   3,  68, "SC-88Pro Map Capital Tones", "Oboe"},
    {'M',   0,   3,  69, "SC-88Pro Map Capital Tones", "English Horn"},
    {'M',   0,   3,  70, "SC-88Pro Map Capital Tones", "Bassoon"},
    {'M',   0,   3,  71, "SC-88Pro Map Capital Tones", "Clarinet"},
    {'M',   0,   3,  72, "SC-88Pro Map Capital Tones", "Piccolo"},
    {'M',   0,   3,  73, "SC-88Pro Map Capital Tones", "Flute"},
    {'M',   0,   3,  74, "SC-88Pro Map Capital Tones", "Recorder"},
    {'M',   0,   3,  75, "SC-88Pro Map Capital Tones", "Pan Flute"},
    {'M',   0,   3,  76, "SC-88Pro Map Capital Tones", "Bottle Blow"},
    {'M',   0,   3,  77, "SC-88Pro Map Capital Tones", "Shakuhachi"},
    {'M',   0,   3,  78, "SC-88Pro Map Capital Tones", "Whistle"},
    {'M',   0,   3,  79, "SC-88Pro Map Capital Tones", "Ocarina"},
    {'M',   0,   3,  80, "SC-88Pro Map Capital Tones", "Square Wave"},
    {'M',   0,   3,  81, "SC-88Pro Map Capital Tones", "Saw Wave"},
    {'M',   0,   3,  82, "SC-88Pro Map Capital Tones", "Syn.Calliope"},
    {'M',   0,   3,  83, "SC-88Pro Map Capital Tones", "Chiffer Lead"},
    {'M',   0,   3,  84, "SC-88Pro Map Capital Tones", "Charang"},
    {'M',   0,   3,  85, "SC-88Pro Map Capital Tones", "Solo Vox"},
    {'M',   0,   3,  86, "SC-88Pro Map Capital Tones", "5th Saw Wave"},
    {'M',   0,   3,  87, "SC-88Pro Map Capital Tones", "Bass & Lead"},
    {'M',   0,   3,  88, "SC-88Pro Map Capital Tones", "Fantasia"},
    {'M',   0,   3,  89, "SC-88Pro Map Capital Tones", "Warm Pad"},
    {'M',   0,   3,  90, "SC-88Pro Map Capital Tones", "Polysynth"},
    {'M',   0,   3,  91, "SC-88Pro Map Capital Tones", "Space Voice"},
    {'M',   0,   3,  92, "SC-88Pro Map Capital Tones", "Bowed Glass"},
    {'M',   0,   3,  93, "SC-88Pro Map Capital Tones", "Metal Pad"},
    {'M',   0,   3,  94, "SC-88Pro Map Capital Tones", "Halo Pad"},
    {'M',   0,   3,  95, "SC-88Pro Map Capital Tones", "Sweep Pad"},
    {'M',   0,   3,  96, "SC-88Pro Map Capital Tones", "Ice Rain"},
    {'M',   0,   3,  97, "SC-88Pro Map Capital Tones", "Soundtrack"},
    {'M',   0,   3,  98, "SC-88Pro Map Capital Tones", "Crystal"},
    {'M',   0,   3,  99, "SC-88Pro Map Capital Tones", "Atmosphere"},
    {'M',   0,   3, 100, "SC-88Pro Map Capital Tones", "Brightness"},
    {'M',   0,   3, 101, "SC-88Pro Map Capital Tones", "Goblin"},
    {'M',   0,   3, 102, "SC-88Pro Map Capital Tones", "Echo Drops"},
    {'M',   0,   3, 103, "SC-88Pro Map Capital Tones", "Star Theme"},
    {'M',   0,   3, 104, "SC-88Pro Map Capital Tones", "Sitar"},
    {'M',   0,   3, 105, "SC-88Pro Map Capital Tones", "Banjo"},
    {'M',   0,   3, 106, "SC-88Pro Map Capital Tones", "Shamisen"},
    {'M',   0,   3, 107, "SC-88Pro Map Capital Tones", "Koto"},
    {'M',   0,   3, 108, "SC-88Pro Map Capital Tones", "Kalimba"},
    {'M',   0,   3, 109, "SC-88Pro Map Capital Tones", "Bagpipe"},
    {'M',   0,   3, 110, "SC-88Pro Map Capital Tones", "Fiddle"},
    {'M',   0,   3, 111, "SC-88Pro Map Capital Tones", "Shanai"},
    {'M',   0,   3, 112, "SC-88Pro Map Capital Tones", "Tinkle Bell"},
    {'M',   0,   3, 113, "SC-88Pro Map Capital Tones", "Agogo"},
    {'M',   0,   3, 114, "SC-88Pro Map Capital Tones", "Steel Drums"},
    {'M',   0,   3, 115, "SC-88Pro Map Capital Tones", "Woodblock"},
    {'M',   0,   3, 116, "SC-88Pro Map Capital Tones", "Taiko"},
    {'M',   0,   3, 117, "SC-88Pro Map Capital Tones", "Melo. Tom 1"},
    {'M',   0,   3, 118, "SC-88Pro Map Capital Tones", "Synth Drum"},
    {'M',   0,   3, 119, "SC-88Pro Map Capital Tones", "Reverse Cym."},
    {'M',   0,   3, 120, "SC-88Pro Map Capital Tones", "Gt.FretNoise"},
    {'M',   0,   3, 121, "SC-88Pro Map Capital Tones", "Breath Noise"},
    {'M',   0,   3, 122, "SC-88Pro Map Capital Tones", "Seashore"},
    {'M',   0,   3, 123, "SC-88Pro Map Capital Tones", "Bird"},
    {'M',   0,   3, 124, "SC-88Pro Map Capital Tones", "Telephone 1"},
    {'M',   0,   3, 125, "SC-88Pro Map Capital Tones", "Helicopter"},
    {'M',   0,   3, 126, "SC-88Pro Map Capital Tones", "Applause"},
    {'M',   0,   3, 127, "SC-88Pro Map Capital Tones", "Gun Shot"},
    {'M',   0,   4,   0, "SC-8850 Map Capital Tones", "Piano 1"},
    {'M',   0,   4,   1, "SC-8850 Map Capital Tones", "Piano 2"},
    {'M',   0,   4,   2, "SC-8850 Map Capital Tones", "Piano 3"},
    {'M',   0,   4,   3, "SC-8850 Map Capital Tones", "Honky-tonk"},
    {'M',   0,   4,   4, "SC-8850 Map Capital Tones", "E.Piano 1"},
    {'M',   0,   4,   5, "SC-8850 Map Capital Tones", "E.Piano 2"},
    {'M',   0,   4,   6, "SC-8850 Map Capital Tones", "Harpsichord"},
    {'M',   0,   4,   7, "SC-8850 Map Capital Tones", "Clav."},
    {'M',   0,   4,   8, "SC-8850 Map Capital Tones", "Celesta"},
    {'M',   0,   4,   9, "SC-8850 Map Capital Tones", "Glockenspiel"},
    {'M',   0,   4,  10, "SC-8850 Map Capital Tones", "Music Box"},
    {'M',   0,   4,  11, "SC-8850 Map Capital Tones", "Vibraphone"},
    {'M',   0,   4,  12, "SC-8850 Map Capital Tones", "Marimba"},
    {'M',   0,   4,  13, "SC-8850 Map Capital Tones", "Xylophone"},
    {'M',   0,   4,  14, "SC-8850 Map Capital Tones", "Tubular-bell"},
    {'M',   0,   4,  15, "SC-8850 Map Capital Tones", "Santur"},
    {'M',   0,   4,  16, "SC-8850 Map Capital Tones", "Organ 1"},
    {'M',   0,   4,  17, "SC-8850 Map Capital Tones", "Organ 2"},
    {'M',   0,   4,  18, "SC-8850 Map Capital Tones", "Organ 3"},
    {'M',   0,   4,  19, "SC-8850 Map Capital Tones", "Church Org.1"},
    {'M',   0,   4,  20, "SC-8850 Map Capital Tones", "Reed Organ"},
    {'M',   0,   4,  21, "SC-8850 Map Capital Tones", "Accordion Fr"},
    {'M',   0,   4,  22, "SC-8850 Map Capital Tones", "Harmonica"},
    {'M',   0,   4,  23, "SC-8850 Map Capital Tones", "Bandoneon"},
    {'M',   0,   4,  24, "SC-8850 Map Capital Tones", "Nylon-str.Gt"},
    {'M',   0,   4,  25, "SC-8850 Map Capital Tones", "Steel-str.Gt"},
    {'M',   0,   4,  26, "SC-8850 Map Capital Tones", "Jazz Gt."},
    {'M',   0,   4,  27, "SC-8850 Map Capital Tones", "Clean Gt."},
    {'M',   0,   4,  28, "SC-8850 Map Capital Tones", "Muted Gt."},
    {'M',   0,   4,  29, "SC-8850 Map Capital Tones", "Overdrive Gt"},
    {'M',   0,   4,  30, "SC-8850 Map Capital Tones", "DistortionGt"},
    {'M',   0,   4,  31, "SC-8850 Map Capital Tones", "Gt.Harmonics"},
    {'M',   0,   4,  32, "SC-8850 Map Capital Tones", "Acoustic Bs."},
    {'M',   0,   4,  33, "SC-8850 Map Capital Tones", "Fingered Bs."},
    {'M',   0,   4,  34, "SC-8850 Map Capital Tones", "Picked Bass"},
    {'M',   0,   4,  35, "SC-8850 Map Capital Tones", "Fretless Bs."},
    {'M',   0,   4,  36, "SC-8850 Map Capital Tones", "Slap Bass 1"},
    {'M',   0,   4,  37, "SC-8850 Map Capital Tones", "Slap Bass 2"},
    {'M',   0,   4,  38, "SC-8850 Map Capital Tones", "Synth Bass 1"},
    {'M',   0,   4,  39, "SC-8850 Map Capital Tones", "Synth Bass 2"},
    {'M',   0,   4,  40, "SC-8850 Map Capital Tones", "Violin    :"},
    {'M',   0,   4,  41, "SC-8850 Map Capital Tones", "Viola     :"},
    {'M',   0,   4,  42, "SC-8850 Map Capital Tones", "Cello     :"},
    {'M',   0,   4,  43, "SC-8850 Map Capital Tones", "Contrabass"},
    {'M',   0,   4,  44, "SC-8850 Map Capital Tones", "Tremolo Str"},
    {'M',   0,   4,  45, "SC-8850 Map Capital Tones", "PizzicatoStr"},
    {'M',   0,   4,  46, "SC-8850 Map Capital Tones", "Harp"},
    {'M',   0,   4,  47, "SC-8850 Map Capital Tones", "Timpani"},
    {'M',   0,   4,  48, "SC-8850 Map Capital Tones", "Strings   :"},
    {'M',   0,   4,  49, "SC-8850 Map Capital Tones", "Slow Strings"},
    {'M',   0,   4,  50, "SC-8850 Map Capital Tones", "Syn.Strings1"},
    {'M',   0,   4,  51, "SC-8850 Map Capital Tones", "Syn.Strings2"},
    {'M',   0,   4,  52, "SC-8850 Map Capital Tones", "Choir Aahs"},
    {'M',   0,   4,  53, "SC-8850 Map Capital Tones", "Voice Oohs"},
    {'M',   0,   4,  54, "SC-8850 Map Capital Tones", "SynVox"},
    {'M',   0,   4,  55, "SC-8850 Map Capital Tones", "OrchestraHit"},
    {'M',   0,   4,  56, "SC-8850 Map Capital Tones", "Trumpet"},
    {'M',   0,   4,  57, "SC-8850 Map Capital Tones", "Trombone"},
    {'M',   0,   4,  58, "SC-8850 Map Capital Tones", "Tuba"},
    {'M',   0,   4,  59, "SC-8850 Map Capital Tones", "MutedTrumpet"},
    {'M',   0,   4,  60, "SC-8850 Map Capital Tones", "French Horns"},
    {'M',   0,   4,  61, "SC-8850 Map Capital Tones", "Brass 1"},
    {'M',   0,   4,  62, "SC-8850 Map Capital Tones", "Synth Brass1"},
    {'M',   0,   4,  63, "SC-8850 Map Capital Tones", "Synth Brass2"},
    {'M',   0,   4,  64, "SC-8850 Map Capital Tones", "Soprano Sax"},
    {'M',   0,   4,  65, "SC-8850 Map Capital Tones", "Alto Sax"},
    {'M',   0,   4,  66, "SC-8850 Map Capital Tones", "Tenor Sax"},
    {'M',   0,   4,  67, "SC-8850 Map Capital Tones", "Baritone Sax"},
    {'M',   0,   4,  68, "SC-8850 Map Capital Tones", "Oboe"},
    {'M',   0,   4,  69, "SC-8850 Map Capital Tones", "English Horn"},
    {'M',   0,   4,  70, "SC-8850 Map Capital Tones", "Bassoon"},
    {'M',   0,   4,  71, "SC-8850 Map Capital Tones", "Clarinet"},
    {'M',   0,   4,  72, "SC-8850 Map Capital Tones", "Piccolo"},
    {'M',   0,   4,  73, "SC-8850 Map Capital Tones", "Flute"},
    {'M',   0,   4,  74, "SC-8850 Map Capital Tones", "Recorder"},
    {'M',   0,   4,  75, "SC-8850 Map Capital Tones", "Pan Flute"},
    {'M',   0,   4,  76, "SC-8850 Map Capital Tones", "Bottle Blow"},
    {'M',   0,   4,  77, "SC-8850 Map Capital Tones", "Shakuhachi"},
    {'M',   0,   4,  78, "SC-8850 Map Capital Tones", "Whistle"},
    {'M',   0,   4,  79, "SC-8850 Map Capital Tones", "Ocarina"},
    {'M',   0,   4,  80, "SC-8850 Map Capital Tones", "Square Wave"},
    {'M',   0,   4,  81, "SC-8850 Map Capital Tones", "Saw Wave"},
    {'M',   0,   4,  82, "SC-8850 Map Capital Tones", "Syn.Calliope"},
    {'M',   0,   4,  83, "SC-8850 Map Capital Tones", "Chiffer Lead"},
    {'M',   0,   4,  84, "SC-8850 Map Capital Tones", "Charang"},
    {'M',   0,   4,  85, "SC-8850 Map Capital Tones", "Solo Vox"},
    {'M',   0,   4,  86, "SC-8850 Map Capital Tones", "5th Saw Wave"},
    {'M',   0,   4,  87, "SC-8850 Map Capital Tones", "Bass & Lead"},
    {'M',   0,   4,  88, "SC-8850 Map Capital Tones", "Fantasia"},
    {'M',   0,   4,  89, "SC-8850 Map Capital Tones", "Warm Pad"},
    {'M',   0,   4,  90, "SC-8850 Map Capital Tones", "Polysynth"},
    {'M',   0,   4,  91, "SC-8850 Map Capital Tones", "Space Voice"},
    {'M',   0,   4,  92, "SC-8850 Map Capital Tones", "Bowed Glass"},
    {'M',   0,   4,  93, "SC-8850 Map Capital Tones", "Metal Pad"},
    {'M',   0,   4,  94, "SC-8850 Map Capital Tones", "Halo Pad"},
    {'M',   0,   4,  95, "SC-8850 Map Capital Tones", "Sweep Pad"},
    {'M',   0,   4,  96, "SC-8850 Map Capital Tones", "Ice Rain"},
    {'M',   0,   4,  97, "SC-8850 Map Capital Tones", "Soundtrack"},
    {'M',   0,   4,  98, "SC-8850 Map Capital Tones", "Crystal"},
    {'M',   0,   4,  99, "SC-8850 Map Capital Tones", "Atmosphere"},
    {'M',   0,   4, 100, "SC-8850 Map Capital Tones", "Brightness"},
    {'M',   0,   4, 101, "SC-8850 Map Capital Tones", "Goblin"},
    {'M',   0,   4, 102, "SC-8850 Map Capital Tones", "Echo Drops"},
    {'M',   0,   4, 103, "SC-8850 Map Capital Tones", "Star Theme"},
    {'M',   0,   4, 104, "SC-8850 Map Capital Tones", "Sitar"},
    {'M',   0,   4, 105, "SC-8850 Map Capital Tones", "Banjo"},
    {'M',   0,   4, 106, "SC-8850 Map Capital Tones", "Shamisen"},
    {'M',   0,   4, 107, "SC-8850 Map Capital Tones", "Koto"},
    {'M',   0,   4, 108, "SC-8850 Map Capital Tones", "Kalimba"},
    {'M',   0,   4, 109, "SC-8850 Map Capital Tones", "Bagpipe"},
    {'M',   0,   4, 110, "SC-8850 Map Capital Tones", "Fiddle"},
    {'M',   0,   4, 111, "SC-8850 Map Capital Tones", "Shanai"},
    {'M',   0,   4, 112, "SC-8850 Map Capital Tones", "Tinkle Bell"},
    {'M',   0,   4, 113, "SC-8850 Map Capital Tones", "Agogo"},
    {'M',   0,   4, 114, "SC-8850 Map Capital Tones", "Steel Drums"},
    {'M',   0,   4, 115, "SC-8850 Map Capital Tones", "Woodblock"},
    {'M',   0,   4, 116, "SC-8850 Map Capital Tones", "Taiko"},
    {'M',   0,   4, 117, "SC-8850 Map Capital Tones", "Melo. Tom 1"},
    {'M',   0,   4, 118, "SC-8850 Map Capital Tones", "Synth Drum"},
    {'M',   0,   4, 119, "SC-8850 Map Capital Tones", "Reverse Cym."},
    {'M',   0,   4, 120, "SC-8850 Map Capital Tones", "Gt.FretNoise"},
    {'M',   0,   4, 121, "SC-8850 Map Capital Tones", "Breath Noise"},
    {'M',   0,   4, 122, "SC-8850 Map Capital Tones", "Seashore"},
    {'M',   0,   4, 123, "SC-8850 Map Capital Tones", "Bird"},
    {'M',   0,   4, 124, "SC-8850 Map Capital Tones", "Telephone 1"},
    {'M',   0,   4, 125, "SC-8850 Map Capital Tones", "Helicopter"},
    {'M',   0,   4, 126, "SC-8850 Map Capital Tones", "Applause"},
    {'M',   0,   4, 127, "SC-8850 Map Capital Tones", "Gun Shot"},
    {'M',   1,   0,   0, "SC-8850 Var #01", "UprightPiano"},
    {'M',   1,   0,   1, "SC-8850 Var #01", "Pop Piano"},
    {'M',   1,   0,   2, "SC-8850 Var #01", "EG+Rhodes 1"},
    {'M',   1,   0,   5, "SC-8850 Var #01", "E.Piano 3"},
    {'M',   1,   0,   6, "SC-8850 Var #01", "Harpsichord2"},
    {'M',   1,   0,   7, "SC-8850 Var #01", "Clav. 2"},
    {'M',   1,   0,   8, "SC-8850 Var #01", "Pop Celesta"},
    {'M',   1,   0,  10, "SC-8850 Var #01", "Music Box 2"},
    {'M',   1,   0,  11, "SC-8850 Var #01", "Pop Vibe."},
    {'M',   1,   0,  15, "SC-8850 Var #01", "Santur 2"},
    {'M',   1,   0,  16, "SC-8850 Var #01", "Organ 101"},
    {'M',   1,   0,  17, "SC-8850 Var #01", "Jazz Organ"},
    {'M',   1,   0,  22, "SC-8850 Var #01", "Harmonica 2"},
    {'M',   1,   0,  26, "SC-8850 Var #01", "Mellow sGt."},
    {'M',   1,   0,  27, "SC-8850 Var #01", "Clean Half"},
    {'M',   1,   0,  28, "SC-8850 Var #01", "Muted Dis.Gt"},
    {'M',   1,   0,  29, "SC-8850 Var #01", "Overdrive 2"},
    {'M',   1,   0,  30, "SC-8850 Var #01", "Dist. Gt2 :"},
    {'M',   1,   0,  32, "SC-8850 Var #01", "Rockabilly"},
    {'M',   1,   0,  33, "SC-8850 Var #01", "Fingered Bs2"},
    {'M',   1,   0,  34, "SC-8850 Var #01", "Picked Bass2"},
    {'M',   1,   0,  35, "SC-8850 Var #01", "Fretless Bs2"},
    {'M',   1,   0,  36, "SC-8850 Var #01", "Slap Pop"},
    {'M',   1,   0,  37, "SC-8850 Var #01", "Slap Bass 3"},
    {'M',   1,   0,  38, "SC-8850 Var #01", "SynthBass101"},
    {'M',   1,   0,  39, "SC-8850 Var #01", "SynthBass201"},
    {'M',   1,   0,  40, "SC-8850 Var #01", "Violin Atk:"},
    {'M',   1,   0,  41, "SC-8850 Var #01", "Viola Atk.:"},
    {'M',   1,   0,  42, "SC-8850 Var #01", "Cello Atk.:"},
    {'M',   1,   0,  45, "SC-8850 Var #01", "Vcs&Cbs Pizz"},
    {'M',   1,   0,  46, "SC-8850 Var #01", "Harp&Strings"},
    {'M',   1,   0,  48, "SC-8850 Var #01", "Bright Str:"},
    {'M',   1,   0,  49, "SC-8850 Var #01", "SlowStrings2"},
    {'M',   1,   0,  50, "SC-8850 Var #01", "OB Strings"},
    {'M',   1,   0,  51, "SC-8850 Var #01", "Syn.Strings5"},
    {'M',   1,   0,  53, "SC-8850 Var #01", "Chorus Oohs"},
    {'M',   1,   0,  54, "SC-8850 Var #01", "SynVox 2"},
    {'M',   1,   0,  55, "SC-8850 Var #01", "Bass Hit"},
    {'M',   1,   0,  56, "SC-8850 Var #01", "Trumpet 2"},
    {'M',   1,   0,  57, "SC-8850 Var #01", "Trombone 2"},
    {'M',   1,   0,  58, "SC-8850 Var #01", "Tuba 2"},
    {'M',   1,   0,  59, "SC-8850 Var #01", "Cup Mute Tp"},
    {'M',   1,   0,  60, "SC-8850 Var #01", "Fr.Horn 2"},
    {'M',   1,   0,  61, "SC-8850 Var #01", "Brass ff"},
    {'M',   1,   0,  62, "SC-8850 Var #01", "JUNO Brass"},
    {'M',   1,   0,  63, "SC-8850 Var #01", "Soft Brass"},
    {'M',   1,   0,  66, "SC-8850 Var #01", "Tenor Sax :"},
    {'M',   1,   0,  67, "SC-8850 Var #01", "Bari. Sax :"},
    {'M',   1,   0,  72, "SC-8850 Var #01", "Piccolo   :"},
    {'M',   1,   0,  73, "SC-8850 Var #01", "Flute 2   :"},
    {'M',   1,   0,  77, "SC-8850 Var #01", "Shakuhachi:"},
    {'M',   1,   0,  78, "SC-8850 Var #01", "Whistle 2"},
    {'M',   1,   0,  80, "SC-8850 Var #01", "MG Square"},
    {'M',   1,   0,  81, "SC-8850 Var #01", "OB2 Saw"},
    {'M',   1,   0,  82, "SC-8850 Var #01", "Vent Synth"},
    {'M',   1,   0,  83, "SC-8850 Var #01", "TB Lead"},
    {'M',   1,   0,  84, "SC-8850 Var #01", "Wire Lead"},
    {'M',   1,   0,  85, "SC-8850 Var #01", "Solo Vox 2"},
    {'M',   1,   0,  86, "SC-8850 Var #01", "Big Fives"},
    {'M',   1,   0,  87, "SC-8850 Var #01", "Big & Raw"},
    {'M',   1,   0,  88, "SC-8850 Var #01", "Fantasia 2"},
    {'M',   1,   0,  89, "SC-8850 Var #01", "Thick Matrix"},
    {'M',   1,   0,  90, "SC-8850 Var #01", "80's PolySyn"},
    {'M',   1,   0,  91, "SC-8850 Var #01", "Heaven II"},
    {'M',   1,   0,  92, "SC-8850 Var #01", "SoftBellPad"},
    {'M',   1,   0,  93, "SC-8850 Var #01", "Tine Pad"},
    {'M',   1,   0,  94, "SC-8850 Var #01", "Vox Pad"},
    {'M',   1,   0,  95, "SC-8850 Var #01", "Polar Pad"},
    {'M',   1,   0,  96, "SC-8850 Var #01", "Harmo Rain"},
    {'M',   1,   0,  97, "SC-8850 Var #01", "Ancestral"},
    {'M',   1,   0,  98, "SC-8850 Var #01", "Syn Mallet"},
    {'M',   1,   0,  99, "SC-8850 Var #01", "Warm Atmos"},
    {'M',   1,   0, 100, "SC-8850 Var #01", "Shining Star"},
    {'M',   1,   0, 101, "SC-8850 Var #01", "Goblinson"},
    {'M',   1,   0, 102, "SC-8850 Var #01", "Echo Bell"},
    {'M',   1,   0, 103, "SC-8850 Var #01", "Star Theme 2"},
    {'M',   1,   0, 104, "SC-8850 Var #01", "Sitar 2"},
    {'M',   1,   0, 105, "SC-8850 Var #01", "Muted Banjo"},
    {'M',   1,   0, 106, "SC-8850 Var #01", "Tsugaru"},
    {'M',   1,   0, 107, "SC-8850 Var #01", "Gu Zheng"},
    {'M',   1,   0, 111, "SC-8850 Var #01", "Shanai 2"},
    {'M',   1,   0, 114, "SC-8850 Var #01", "Island Mlt"},
    {'M',   1,   0, 116, "SC-8850 Var #01", "Small Taiko"},
    {'M',   1,   0, 117, "SC-8850 Var #01", "Real Tom"},
    {'M',   1,   0, 119, "SC-8850 Var #01", "Reverse Cym2"},
    {'M',   1,   0, 120, "SC-8850 Var #01", "Gt.Cut Noise"},
    {'M',   1,   0, 121, "SC-8850 Var #01", "Fl.Key Click"},
    {'M',   1,   0, 122, "SC-8850 Var #01", "Rain"},
    {'M',   1,   0, 123, "SC-8850 Var #01", "Dog"},
    {'M',   1,   0, 124, "SC-8850 Var #01", "Telephone 2"},
    {'M',   1,   0, 125, "SC-8850 Var #01", "Car-Engine"},
    {'M',   1,   0, 126, "SC-8850 Var #01", "Laughing"},
    {'M',   1,   0, 127, "SC-8850 Var #01", "Machine Gun"},
    {'M',   1,   1,  38, "SC-55 Map Var #01", "Syn.Bass101"},
    {'M',   1,   1,  57, "SC-55 Map Var #01", "Trombone 2"},
    {'M',   1,   1,  60, "SC-55 Map Var #01", "Fr.Horn2"},
    {'M',   1,   1,  80, "SC-55 Map Var #01", "Square"},
    {'M',   1,   1,  81, "SC-55 Map Var #01", "Saw"},
    {'M',   1,   1,  98, "SC-55 Map Var #01", "Syn Mallet"},
    {'M',   1,   1, 102, "SC-55 Map Var #01", "Echo Bell"},
    {'M',   1,   1, 104, "SC-55 Map Var #01", "Sitar 2"},
    {'M',   1,   1, 120, "SC-55 Map Var #01", "Gt.CutNoise"},
    {'M',   1,   1, 121, "SC-55 Map Var #01", "Fl.KeyClick"},
    {'M',   1,   1, 122, "SC-55 Map Var #01", "Rain"},
    {'M',   1,   1, 123, "SC-55 Map Var #01", "Dog"},
    {'M',   1,   1, 124, "SC-55 Map Var #01", "Telephone 2"},
    {'M',   1,   1, 125, "SC-55 Map Var #01", "Car-Engine"},
    {'M',   1,   1, 126, "SC-55 Map Var #01", "Laughing"},
    {'M',   1,   1, 127, "SC-55 Map Var #01", "Machine Gun"},
    {'M',   1,   2,   2, "SC-88 Map Var #01", "EG+Rhodes 1"},
    {'M',   1,   2,  11, "SC-88 Map Var #01", "Hard Vibe"},
    {'M',   1,   2,  15, "SC-88 Map Var #01", "Santur 2"},
    {'M',   1,   2,  16, "SC-88 Map Var #01", "Organ 101"},
    {'M',   1,   2,  17, "SC-88 Map Var #01", "Organ 201"},
    {'M',   1,   2,  22, "SC-88 Map Var #01", "Harmonica 2"},
    {'M',   1,   2,  26, "SC-88 Map Var #01", "Mellow Gt."},
    {'M',   1,   2,  28, "SC-88 Map Var #01", "MutedDis.Gt"},
    {'M',   1,   2,  30, "SC-88 Map Var #01", "Dist. Gt2"},
    {'M',   1,   2,  33, "SC-88 Map Var #01", "FingeredBs2"},
    {'M',   1,   2,  35, "SC-88 Map Var #01", "FretlessBs2"},
    {'M',   1,   2,  38, "SC-88 Map Var #01", "Syn.Bass101"},
    {'M',   1,   2,  39, "SC-88 Map Var #01", "Syn.Bass201"},
    {'M',   1,   2,  48, "SC-88 Map Var #01", "Strings 2"},
    {'M',   1,   2,  49, "SC-88 Map Var #01", "Slow Str. 2"},
    {'M',   1,   2,  50, "SC-88 Map Var #01", "OB Strings"},
    {'M',   1,   2,  56, "SC-88 Map Var #01", "Trumpet 2"},
    {'M',   1,   2,  57, "SC-88 Map Var #01", "Trombone 2"},
    {'M',   1,   2,  58, "SC-88 Map Var #01", "Tuba 2"},
    {'M',   1,   2,  60, "SC-88 Map Var #01", "Fr.Horn 2"},
    {'M',   1,   2,  62, "SC-88 Map Var #01", "Poly Brass"},
    {'M',   1,   2,  63, "SC-88 Map Var #01", "Soft Brass"},
    {'M',   1,   2,  80, "SC-88 Map Var #01", "Square"},
    {'M',   1,   2,  81, "SC-88 Map Var #01", "Saw"},
    {'M',   1,   2,  82, "SC-88 Map Var #01", "Vent Synth"},
    {'M',   1,   2,  86, "SC-88 Map Var #01", "Big Fives"},
    {'M',   1,   2,  87, "SC-88 Map Var #01", "Big & Raw"},
    {'M',   1,   2,  88, "SC-88 Map Var #01", "Fantasia 2"},
    {'M',   1,   2,  89, "SC-88 Map Var #01", "Thick Pad"},
    {'M',   1,   2,  90, "SC-88 Map Var #01", "80'sPolySyn"},
    {'M',   1,   2,  91, "SC-88 Map Var #01", "Heaven II"},
    {'M',   1,   2,  93, "SC-88 Map Var #01", "Tine Pad"},
    {'M',   1,   2,  95, "SC-88 Map Var #01", "Polar Pad"},
    {'M',   1,   2,  96, "SC-88 Map Var #01", "Harmo Rain"},
    {'M',   1,   2,  97, "SC-88 Map Var #01", "Ancestral"},
    {'M',   1,   2,  98, "SC-88 Map Var #01", "Syn Mallet"},
    {'M',   1,   2,  99, "SC-88 Map Var #01", "Warm Atmos"},
    {'M',   1,   2, 101, "SC-88 Map Var #01", "Goblinson"},
    {'M',   1,   2, 102, "SC-88 Map Var #01", "Echo Bell"},
    {'M',   1,   2, 103, "SC-88 Map Var #01", "StarTheme 2"},
    {'M',   1,   2, 104, "SC-88 Map Var #01", "Sitar 2"},
    {'M',   1,   2, 105, "SC-88 Map Var #01", "Muted Banjo"},
    {'M',   1,   2, 106, "SC-88 Map Var #01", "Tsugaru"},
    {'M',   1,   2, 111, "SC-88 Map Var #01", "Shannai 2"},
    {'M',   1,   2, 117, "SC-88 Map Var #01", "Real Tom"},
    {'M',   1,   2, 119, "SC-88 Map Var #01", "ReverseCym2"},
    {'M',   1,   2, 120, "SC-88 Map Var #01", "Gt.CutNoise"},
    {'M',   1,   2, 121, "SC-88 Map Var #01", "Fl.Key Click"},
    {'M',   1,   2, 122, "SC-88 Map Var #01", "Rain"},
    {'M',   1,   2, 123, "SC-88 Map Var #01", "Dog"},
    {'M',   1,   2, 124, "SC-88 Map Var #01", "Telephone 2"},
    {'M',   1,   2, 125, "SC-88 Map Var #01", "Car-Engine"},
    {'M',   1,   2, 126, "SC-88 Map Var #01", "Laughing"},
    {'M',   1,   2, 127, "SC-88 Map Var #01", "Machine Gun"},
    {'M',   1,   3,   2, "SC-88Pro Map Var #01", "EG+Rhodes 1"},
    {'M',   1,   3,   6, "SC-88Pro Map Var #01", "Harpsichord2"},
    {'M',   1,   3,   8, "SC-88Pro Map Var #01", "Pop Celesta"},
    {'M',   1,   3,  11, "SC-88Pro Map Var #01", "Pop Vibe."},
    {'M',   1,   3,  15, "SC-88Pro Map Var #01", "Santur 2"},
    {'M',   1,   3,  16, "SC-88Pro Map Var #01", "Organ 101"},
    {'M',   1,   3,  17, "SC-88Pro Map Var #01", "Jazz Organ"},
    {'M',   1,   3,  22, "SC-88Pro Map Var #01", "Harmonica 2"},
    {'M',   1,   3,  26, "SC-88Pro Map Var #01", "Mellow Gt."},
    {'M',   1,   3,  27, "SC-88Pro Map Var #01", "Clean Half"},
    {'M',   1,   3,  28, "SC-88Pro Map Var #01", "Muted Dis.Gt"},
    {'M',   1,   3,  29, "SC-88Pro Map Var #01", "Overdrive 2"},
    {'M',   1,   3,  30, "SC-88Pro Map Var #01", "Dist. Gt2 :"},
    {'M',   1,   3,  32, "SC-88Pro Map Var #01", "Rockabilly"},
    {'M',   1,   3,  33, "SC-88Pro Map Var #01", "Fingered Bs2"},
    {'M',   1,   3,  34, "SC-88Pro Map Var #01", "Picked Bass2"},
    {'M',   1,   3,  35, "SC-88Pro Map Var #01", "Fretless Bs2"},
    {'M',   1,   3,  36, "SC-88Pro Map Var #01", "Slap Pop"},
    {'M',   1,   3,  38, "SC-88Pro Map Var #01", "SynthBass101"},
    {'M',   1,   3,  39, "SC-88Pro Map Var #01", "SynthBass201"},
    {'M',   1,   3,  40, "SC-88Pro Map Var #01", "Violin Atk:"},
    {'M',   1,   3,  41, "SC-88Pro Map Var #01", "Viola Atk.:"},
    {'M',   1,   3,  42, "SC-88Pro Map Var #01", "Cello Atk.:"},
    {'M',   1,   3,  45, "SC-88Pro Map Var #01", "Vcs&Cbs Pizz"},
    {'M',   1,   3,  48, "SC-88Pro Map Var #01", "Bright Str:"},
    {'M',   1,   3,  49, "SC-88Pro Map Var #01", "SlowStrings2"},
    {'M',   1,   3,  50, "SC-88Pro Map Var #01", "OB Strings"},
    {'M',   1,   3,  51, "SC-88Pro Map Var #01", "Syn.Strings5"},
    {'M',   1,   3,  56, "SC-88Pro Map Var #01", "Trumpet 2"},
    {'M',   1,   3,  57, "SC-88Pro Map Var #01", "Trombone 2"},
    {'M',   1,   3,  58, "SC-88Pro Map Var #01", "Tuba 2"},
    {'M',   1,   3,  60, "SC-88Pro Map Var #01", "Fr.Horn 2"},
    {'M',   1,   3,  61, "SC-88Pro Map Var #01", "Brass ff"},
    {'M',   1,   3,  62, "SC-88Pro Map Var #01", "JUNO Brass"},
    {'M',   1,   3,  63, "SC-88Pro Map Var #01", "Soft Brass"},
    {'M',   1,   3,  66, "SC-88Pro Map Var #01", "Tenor Sax :"},
    {'M',   1,   3,  67, "SC-88Pro Map Var #01", "Bari. Sax :"},
    {'M',   1,   3,  72, "SC-88Pro Map Var #01", "Piccolo   :"},
    {'M',   1,   3,  73, "SC-88Pro Map Var #01", "Flute 2   :"},
    {'M',   1,   3,  77, "SC-88Pro Map Var #01", "Shakuhachi:"},
    {'M',   1,   3,  78, "SC-88Pro Map Var #01", "Whistle 2"},
    {'M',   1,   3,  80, "SC-88Pro Map Var #01", "MG Square"},
    {'M',   1,   3,  81, "SC-88Pro Map Var #01", "OB2 Saw"},
    {'M',   1,   3,  82, "SC-88Pro Map Var #01", "Vent Synth"},
    {'M',   1,   3,  83, "SC-88Pro Map Var #01", "TB Lead"},
    {'M',   1,   3,  86, "SC-88Pro Map Var #01", "Big Fives"},
    {'M',   1,   3,  87, "SC-88Pro Map Var #01", "Big & Raw"},
    {'M',   1,   3,  88, "SC-88Pro Map Var #01", "Fantasia 2"},
    {'M',   1,   3,  89, "SC-88Pro Map Var #01", "Thick Matrix"},
    {'M',   1,   3,  90, "SC-88Pro Map Var #01", "80's PolySyn"},
    {'M',   1,   3,  91, "SC-88Pro Map Var #01", "Heaven II"},
    {'M',   1,   3,  92, "SC-88Pro Map Var #01", "SoftBellPad"},
    {'M',   1,   3,  93, "SC-88Pro Map Var #01", "Tine Pad"},
    {'M',   1,   3,  94, "SC-88Pro Map Var #01", "Vox Pad"},
    {'M',   1,   3,  95, "SC-88Pro Map Var #01", "Polar Pad"},
    {'M',   1,   3,  96, "SC-88Pro Map Var #01", "Harmo Rain"},
    {'M',   1,   3,  97, "SC-88Pro Map Var #01", "Ancestral"},
    {'M',   1,   3,  98, "SC-88Pro Map Var #01", "Syn Mallet"},
    {'M',   1,   3,  99, "SC-88Pro Map Var #01", "Warm Atmos"},
    {'M',   1,   3, 100, "SC-88Pro Map Var #01", "Shining Star"},
    {'M',   1,   3, 101, "SC-88Pro Map Var #01", "Goblinson"},
    {'M',   1,   3, 102, "SC-88Pro Map Var #01", "Echo Bell"},
    {'M',   1,   3, 103, "SC-88Pro Map Var #01", "Star Theme 2"},
    {'M',   1,   3, 104, "SC-88Pro Map Var #01", "Sitar 2"},
    {'M',   1,   3, 105, "SC-88Pro Map Var #01", "Muted Banjo"},
    {'M',   1,   3, 106, "SC-88Pro Map Var #01", "Tsugaru"},
    {'M',   1,   3, 107, "SC-88Pro Map Var #01", "Gu Zheng"},
    {'M',   1,   3, 111, "SC-88Pro Map Var #01", "Shanai 2"},
    {'M',   1,   3, 114, "SC-88Pro Map Var #01", "Island Mlt"},
    {'M',   1,   3, 116, "SC-88Pro Map Var #01", "Small Taiko"},
    {'M',   1,   3, 117, "SC-88Pro Map Var #01", "Real Tom"},
    {'M',   1,   3, 119, "SC-88Pro Map Var #01", "Reverse Cym2"},
    {'M',   1,   3, 120, "SC-88Pro Map Var #01", "Gt.Cut Noise"},
    {'M',   1,   3, 121, "SC-88Pro Map Var #01", "Fl.Key Click"},
    {'M',   1,   3, 122, "SC-88Pro Map Var #01", "Rain"},
    {'M',   1,   3, 123, "SC-88Pro Map Var #01", "Dog"},
    {'M',   1,   3, 124, "SC-88Pro Map Var #01", "Telephone 2"},
    {'M',   1,   3, 125, "SC-88Pro Map Var #01", "Car-Engine"},
    {'M',   1,   3, 126, "SC-88Pro Map Var #01", "Laughing"},
    {'M',   1,   3, 127, "SC-88Pro Map Var #01", "Machine Gun"},
    {'M',   1,   4,   0, "SC-8850 Map Var #01", "UprightPiano"},
    {'M',   1,   4,   1, "SC-8850 Map Var #01", "Pop Piano"},
    {'M',   1,   4,   2, "SC-8850 Map Var #01", "EG+Rhodes 1"},
    {'M',   1,   4,   5, "SC-8850 Map Var #01", "E.Piano 3"},
    {'M',   1,   4,   6, "SC-8850 Map Var #01", "Harpsichord2"},
    {'M',   1,   4,   7, "SC-8850 Map Var #01", "Clav. 2"},
    {'M',   1,   4,   8, "SC-8850 Map Var #01", "Pop Celesta"},
    {'M',   1,   4,  10, "SC-8850 Map Var #01", "Music Box 2"},
    {'M',   1,   4,  11, "SC-8850 Map Var #01", "Pop Vibe."},
    {'M',   1,   4,  15, "SC-8850 Map Var #01", "Santur 2"},
    {'M',   1,   4,  16, "SC-8850 Map Var #01", "Organ 101"},
    {'M',   1,   4,  17, "SC-8850 Map Var #01", "Jazz Organ"},
    {'M',   1,   4,  22, "SC-8850 Map Var #01", "Harmonica 2"},
    {'M',   1,   4,  26, "SC-8850 Map Var #01", "Mellow Gt."},
    {'M',   1,   4,  27, "SC-8850 Map Var #01", "Clean Half"},
    {'M',   1,   4,  28, "SC-8850 Map Var #01", "Muted Dis.Gt"},
    {'M',   1,   4,  29, "SC-8850 Map Var #01", "Overdrive 2"},
    {'M',   1,   4,  30, "SC-8850 Map Var #01", "Dist. Gt2 :"},
    {'M',   1,   4,  32, "SC-8850 Map Var #01", "Rockabilly"},
    {'M',   1,   4,  33, "SC-8850 Map Var #01", "Fingered Bs2"},
    {'M',   1,   4,  34, "SC-8850 Map Var #01", "Picked Bass2"},
    {'M',   1,   4,  35, "SC-8850 Map Var #01", "Fretless Bs2"},
    {'M',   1,   4,  36, "SC-8850 Map Var #01", "Slap Pop"},
    {'M',   1,   4,  37, "SC-8850 Map Var #01", "Slap Bass 3"},
    {'M',   1,   4,  38, "SC-8850 Map Var #01", "SynthBass101"},
    {'M',   1,   4,  39, "SC-8850 Map Var #01", "SynthBass201"},
    {'M',   1,   4,  40, "SC-8850 Map Var #01", "Violin Atk:"},
    {'M',   1,   4,  41, "SC-8850 Map Var #01", "Viola Atk.:"},
    {'M',   1,   4,  42, "SC-8850 Map Var #01", "Cello Atk.:"},
    {'M',   1,   4,  45, "SC-8850 Map Var #01", "Vcs&Cbs Pizz"},
    {'M',   1,   4,  46, "SC-8850 Map Var #01", "Harp&Strings"},
    {'M',   1,   4,  48, "SC-8850 Map Var #01", "Bright Str:"},
    {'M',   1,   4,  49, "SC-8850 Map Var #01", "SlowStrings2"},
    {'M',   1,   4,  50, "SC-8850 Map Var #01", "OB Strings"},
    {'M',   1,   4,  51, "SC-8850 Map Var #01", "Syn.Strings5"},
    {'M',   1,   4,  53, "SC-8850 Map Var #01", "Chorus Oohs"},
    {'M',   1,   4,  54, "SC-8850 Map Var #01", "SynVox 2"},
    {'M',   1,   4,  55, "SC-8850 Map Var #01", "Bass Hit"},
    {'M',   1,   4,  56, "SC-8850 Map Var #01", "Trumpet 2"},
    {'M',   1,   4,  57, "SC-8850 Map Var #01", "Trombone 2"},
    {'M',   1,   4,  58, "SC-8850 Map Var #01", "Tuba 2"},
    {'M',   1,   4,  59, "SC-8850 Map Var #01", "Cup Mute Tp"},
    {'M',   1,   4,  60, "SC-8850 Map Var #01", "Fr.Horn 2"},
    {'M',   1,   4,  61, "SC-8850 Map Var #01", "Brass ff"},
    {'M',   1,   4,  62, "SC-8850 Map Var #01", "JUNO Brass"},
    {'M',   1,   4,  63, "SC-8850 Map Var #01", "Soft Brass"},
    {'M',   1,   4,  66, "SC-8850 Map Var #01", "Tenor Sax :"},
    {'M',   1,   4,  67, "SC-8850 Map Var #01", "Bari. Sax :"},
    {'M',   1,   4,  72, "SC-8850 Map Var #01", "Piccolo   :"},
    {'M',   1,   4,  73, "SC-8850 Map Var #01", "Flute 2   :"},
    {'M',   1,   4,  77, "SC-8850 Map Var #01", "Shakuhachi:"},
    {'M',   1,   4,  78, "SC-8850 Map Var #01", "Whistle 2"},
    {'M',   1,   4,  80, "SC-8850 Map Var #01", "MG Square"},
    {'M',   1,   4,  81, "SC-8850 Map Var #01", "OB2 Saw"},
    {'M',   1,   4,  82, "SC-8850 Map Var #01", "Vent Synth"},
    {'M',   1,   4,  83, "SC-8850 Map Var #01", "TB Lead"},
    {'M',   1,   4,  84, "SC-8850 Map Var #01", "Wire Lead"},
    {'M',   1,   4,  85, "SC-8850 Map Var #01", "Solo Vox 2"},
    {'M',   1,   4,  86, "SC-8850 Map Var #01", "Big Fives"},
    {'M',   1,   4,  87, "SC-8850 Map Var #01", "Big & Raw"},
    {'M',   1,   4,  88, "SC-8850 Map Var #01", "Fantasia 2"},
    {'M',   1,   4,  89, "SC-8850 Map Var #01", "Thick Matrix"},
    {'M',   1,   4,  90, "SC-8850 Map Var #01", "80's PolySyn"},
    {'M',   1,   4,  91, "SC-8850 Map Var #01", "Heaven II"},
    {'M',   1,   4,  92, "SC-8850 Map Var #01", "SoftBellPad"},
    {'M',   1,   4,  93, "SC-8850 Map Var #01", "Tine Pad"},
    {'M',   1,   4,  94, "SC-8850 Map Var #01", "Vox Pad"},
    {'M',   1,   4,  95, "SC-8850 Map Var #01", "Polar Pad"},
    {'M',   1,   4,  96, "SC-8850 Map Var #01", "Harmo Rain"},
    {'M',   1,   4,  97, "SC-8850 Map Var #01", "Ancestral"},
    {'M',   1,   4,  98, "SC-8850 Map Var #01", "Syn Mallet"},
    {'M',   1,   4,  99, "SC-8850 Map Var #01", "Warm Atmos"},
    {'M',   1,   4, 100, "SC-8850 Map Var #01", "Shining Star"},
    {'M',   1,   4, 101, "SC-8850 Map Var #01", "Goblinson"},
    {'M',   1,   4, 102, "SC-8850 Map Var #01", "Echo Bell"},
    {'M',   1,   4, 103, "SC-8850 Map Var #01", "Star Theme 2"},
    {'M',   1,   4, 104, "SC-8850 Map Var #01", "Sitar 2"},
    {'M',   1,   4, 105, "SC-8850 Map Var #01", "Muted Banjo"},
    {'M',   1,   4, 106, "SC-8850 Map Var #01", "Tsugaru"},
    {'M',   1,   4, 107, "SC-8850 Map Var #01", "Gu Zheng"},
    {'M',   1,   4, 111, "SC-8850 Map Var #01", "Shanai 2"},
    {'M',   1,   4, 114, "SC-8850 Map Var #01", "Island Mlt"},
    {'M',   1,   4, 116, "SC-8850 Map Var #01", "Small Taiko"},
    {'M',   1,   4, 117, "SC-8850 Map Var #01", "Real Tom"},
    {'M',   1,   4, 119, "SC-8850 Map Var #01", "Reverse Cym2"},
    {'M',   1,   4, 120, "SC-8850 Map Var #01", "Gt.Cut Noise"},
    {'M',   1,   4, 121, "SC-8850 Map Var #01", "Fl.Key Click"},
    {'M',   1,   4, 122, "SC-8850 Map Var #01", "Rain"},
    {'M',   1,   4, 123, "SC-8850 Map Var #01", "Dog"},
    {'M',   1,   4, 124, "SC-8850 Map Var #01", "Telephone 2"},
    {'M',   1,   4, 125, "SC-8850 Map Var #01", "Car-Engine"},
    {'M',   1,   4, 126, "SC-8850 Map Var #01", "Laughing"},
    {'M',   1,   4, 127, "SC-8850 Map Var #01", "Machine Gun"},
    {'M',   2,   0,   0, "SC-8850 Var #02", "Mild Piano"},
    {'M',   2,   0,   1, "SC-8850 Var #02", "Rock Piano"},
    {'M',   2,   0,   2, "SC-8850 Var #02", "EG+Rhodes 2"},
    {'M',   2,   0,   6, "SC-8850 Var #02", "Harpsichord3"},
    {'M',   2,   0,   7, "SC-8850 Var #02", "Atk Clav.1"},
    {'M',   2,   0,  15, "SC-8850 Var #02", "Santur 3"},
    {'M',   2,   0,  16, "SC-8850 Var #02", "Ful Organ 1"},
    {'M',   2,   0,  17, "SC-8850 Var #02", "E.Organ 16+2"},
    {'M',   2,   0,  27, "SC-8850 Var #02", "Open Hard 1"},
    {'M',   2,   0,  28, "SC-8850 Var #02", "TC Muted Gt."},
    {'M',   2,   0,  29, "SC-8850 Var #02", "Overdrive 3"},
    {'M',   2,   0,  30, "SC-8850 Var #02", "Dazed Guitar"},
    {'M',   2,   0,  33, "SC-8850 Var #02", "Jazz Bass"},
    {'M',   2,   0,  34, "SC-8850 Var #02", "Picked Bass3"},
    {'M',   2,   0,  35, "SC-8850 Var #02", "Fretless Bs3"},
    {'M',   2,   0,  38, "SC-8850 Var #02", "CS Bass"},
    {'M',   2,   0,  39, "SC-8850 Var #02", "Modular Bass"},
    {'M',   2,   0,  44, "SC-8850 Var #02", "Trem Str.St."},
    {'M',   2,   0,  45, "SC-8850 Var #02", "Chamber Pizz"},
    {'M',   2,   0,  46, "SC-8850 Var #02", "Harp St."},
    {'M',   2,   0,  48, "SC-8850 Var #02", "ChamberStr"},
    {'M',   2,   0,  49, "SC-8850 Var #02", "SlowStrings3"},
    {'M',   2,   0,  50, "SC-8850 Var #02", "StackStrings"},
    {'M',   2,   0,  51, "SC-8850 Var #02", "JUNO Strings"},
    {'M',   2,   0,  53, "SC-8850 Var #02", "Voice Oohs 2"},
    {'M',   2,   0,  54, "SC-8850 Var #02", "SynVox 3"},
    {'M',   2,   0,  55, "SC-8850 Var #02", "6th Hit"},
    {'M',   2,   0,  56, "SC-8850 Var #02", "Trumpet   :"},
    {'M',   2,   0,  57, "SC-8850 Var #02", "Twin bones"},
    {'M',   2,   0,  59, "SC-8850 Var #02", "MuteTrumpet2"},
    {'M',   2,   0,  60, "SC-8850 Var #02", "Horn + Orche"},
    {'M',   2,   0,  61, "SC-8850 Var #02", "Bones Sect."},
    {'M',   2,   0,  62, "SC-8850 Var #02", "Stack Brass"},
    {'M',   2,   0,  63, "SC-8850 Var #02", "Warm Brass"},
    {'M',   2,   0,  73, "SC-8850 Var #02", "Flute Exp."},
    {'M',   2,   0,  80, "SC-8850 Var #02", "Hollow Mini"},
    {'M',   2,   0,  81, "SC-8850 Var #02", "Pulse Saw"},
    {'M',   2,   0,  82, "SC-8850 Var #02", "Pure PanLead"},
    {'M',   2,   0,  83, "SC-8850 Var #02", "Hybrid Lead"},
    {'M',   2,   0,  84, "SC-8850 Var #02", "FB.Charang"},
    {'M',   2,   0,  86, "SC-8850 Var #02", "5th Lead"},
    {'M',   2,   0,  87, "SC-8850 Var #02", "Fat & Perky"},
    {'M',   2,   0,  88, "SC-8850 Var #02", "New Age Pad"},
    {'M',   2,   0,  89, "SC-8850 Var #02", "Horn Pad"},
    {'M',   2,   0,  90, "SC-8850 Var #02", "Polysynth 2"},
    {'M',   2,   0,  91, "SC-8850 Var #02", "SC Heaven"},
    {'M',   2,   0,  92, "SC-8850 Var #02", "JP8 Sqr Pad"},
    {'M',   2,   0,  93, "SC-8850 Var #02", "Panner Pad"},
    {'M',   2,   0,  94, "SC-8850 Var #02", "Vox Sweep"},
    {'M',   2,   0,  95, "SC-8850 Var #02", "Ambient BPF"},
    {'M',   2,   0,  96, "SC-8850 Var #02", "African wood"},
    {'M',   2,   0,  97, "SC-8850 Var #02", "Prologue"},
    {'M',   2,   0,  98, "SC-8850 Var #02", "Soft Crystal"},
    {'M',   2,   0,  99, "SC-8850 Var #02", "Nylon Harp"},
    {'M',   2,   0, 100, "SC-8850 Var #02", "OB Stab"},
    {'M',   2,   0, 101, "SC-8850 Var #02", "50's Sci-Fi"},
    {'M',   2,   0, 102, "SC-8850 Var #02", "Echo Pan"},
    {'M',   2,   0, 103, "SC-8850 Var #02", "Star Mind"},
    {'M',   2,   0, 104, "SC-8850 Var #02", "Detune Sitar"},
    {'M',   2,   0, 117, "SC-8850 Var #02", "Real Tom 2"},
    {'M',   2,   0, 119, "SC-8850 Var #02", "Reverse Cym3"},
    {'M',   2,   0, 120, "SC-8850 Var #02", "String Slap"},
    {'M',   2,   0, 121, "SC-8850 Var #02", "Brth Nz Menu"},
    {'M',   2,   0, 122, "SC-8850 Var #02", "Thunder"},
    {'M',   2,   0, 123, "SC-8850 Var #02", "Horse-Gallop"},
    {'M',   2,   0, 124, "SC-8850 Var #02", "DoorCreaking"},
    {'M',   2,   0, 125, "SC-8850 Var #02", "Car-Stop"},
    {'M',   2,   0, 126, "SC-8850 Var #02", "Screaming"},
    {'M',   2,   0, 127, "SC-8850 Var #02", "Lasergun"},
    {'M',   2,   1, 102, "SC-55 Map Var #02", "Echo Pan"},
    {'M',   2,   1, 120, "SC-55 Map Var #02", "String Slap"},
    {'M',   2,   1, 122, "SC-55 Map Var #02", "Thunder"},
    {'M',   2,   1, 123, "SC-55 Map Var #02", "HorseGallop"},
    {'M',   2,   1, 124, "SC-55 Map Var #02", "Creaking"},
    {'M',   2,   1, 125, "SC-55 Map Var #02", "Car-Stop"},
    {'M',   2,   1, 126, "SC-55 Map Var #02", "Screaming"},
    {'M',   2,   1, 127, "SC-55 Map Var #02", "Lasergun"},
    {'M',   2,   2,   2, "SC-88 Map Var #02", "EG+Rhodes 2"},
    {'M',   2,   2,  30, "SC-88 Map Var #02", "DazedGuitar"},
    {'M',   2,   2,  33, "SC-88 Map Var #02", "Jazz Bass"},
    {'M',   2,   2,  35, "SC-88 Map Var #02", "FretlessBs3"},
    {'M',   2,   2,  39, "SC-88 Map Var #02", "ModularBass"},
    {'M',   2,   2,  80, "SC-88 Map Var #02", "Hollow Mini"},
    {'M',   2,   2,  81, "SC-88 Map Var #02", "Pulse Saw"},
    {'M',   2,   2,  82, "SC-88 Map Var #02", "PurePanLead"},
    {'M',   2,   2,  87, "SC-88 Map Var #02", "Fat & Perky"},
    {'M',   2,   2,  89, "SC-88 Map Var #02", "Horn Pad"},
    {'M',   2,   2,  93, "SC-88 Map Var #02", "Panner Pad"},
    {'M',   2,   2,  96, "SC-88 Map Var #02", "AfricanWood"},
    {'M',   2,   2,  97, "SC-88 Map Var #02", "Prologue"},
    {'M',   2,   2,  98, "SC-88 Map Var #02", "SoftCrystal"},
    {'M',   2,   2,  99, "SC-88 Map Var #02", "Nylon Harp"},
    {'M',   2,   2, 101, "SC-88 Map Var #02", "50's Sci-Fi"},
    {'M',   2,   2, 102, "SC-88 Map Var #02", "Echo Pan"},
    {'M',   2,   2, 104, "SC-88 Map Var #02", "DetuneSitar"},
    {'M',   2,   2, 120, "SC-88 Map Var #02", "String Slap"},
    {'M',   2,   2, 122, "SC-88 Map Var #02", "Thunder"},
    {'M',   2,   2, 123, "SC-88 Map Var #02", "HorseGallop"},
    {'M',   2,   2, 124, "SC-88 Map Var #02", "Creaking"},
    {'M',   2,   2, 125, "SC-88 Map Var #02", "Car-Stop"},
    {'M',   2,   2, 126, "SC-88 Map Var #02", "Screaming"},
    {'M',   2,   2, 127, "SC-88 Map Var #02", "Lasergun"},
    {'M',   2,   3,   2, "SC-88Pro Map Var #02", "EG+Rhodes 2"},
    {'M',   2,   3,  17, "SC-88Pro Map Var #02", "E.Organ 16+2"},
    {'M',   2,   3,  27, "SC-88Pro Map Var #02", "Open Hard 1"},
    {'M',   2,   3,  28, "SC-88Pro Map Var #02", "TC Muted Gt."},
    {'M',   2,   3,  29, "SC-88Pro Map Var #02", "Overdrive 3"},
    {'M',   2,   3,  30, "SC-88Pro Map Var #02", "Dazed Guitar"},
    {'M',   2,   3,  33, "SC-88Pro Map Var #02", "Jazz Bass"},
    {'M',   2,   3,  34, "SC-88Pro Map Var #02", "Picked Bass3"},
    {'M',   2,   3,  35, "SC-88Pro Map Var #02", "Fretless Bs3"},
    {'M',   2,   3,  38, "SC-88Pro Map Var #02", "CS Bass"},
    {'M',   2,   3,  39, "SC-88Pro Map Var #02", "Modular Bass"},
    {'M',   2,   3,  45, "SC-88Pro Map Var #02", "Chamber Pizz"},
    {'M',   2,   3,  48, "SC-88Pro Map Var #02", "ChamberStr:"},
    {'M',   2,   3,  50, "SC-88Pro Map Var #02", "StackStrings"},
    {'M',   2,   3,  51, "SC-88Pro Map Var #02", "JUNO Strings"},
    {'M',   2,   3,  56, "SC-88Pro Map Var #02", "Trumpet   :"},
    {'M',   2,   3,  57, "SC-88Pro Map Var #02", "Twin bones"},
    {'M',   2,   3,  60, "SC-88Pro Map Var #02", "Horn + Orche"},
    {'M',   2,   3,  61, "SC-88Pro Map Var #02", "Bones Sect."},
    {'M',   2,   3,  62, "SC-88Pro Map Var #02", "StackBrass"},
    {'M',   2,   3,  63, "SC-88Pro Map Var #02", "Warm Brass"},
    {'M',   2,   3,  73, "SC-88Pro Map Var #02", "Flute Exp."},
    {'M',   2,   3,  80, "SC-88Pro Map Var #02", "Hollow Mini"},
    {'M',   2,   3,  81, "SC-88Pro Map Var #02", "Pulse Saw"},
    {'M',   2,   3,  82, "SC-88Pro Map Var #02", "Pure PanLead"},
    {'M',   2,   3,  86, "SC-88Pro Map Var #02", "5th Lead"},
    {'M',   2,   3,  87, "SC-88Pro Map Var #02", "Fat & Perky"},
    {'M',   2,   3,  88, "SC-88Pro Map Var #02", "New Age Pad"},
    {'M',   2,   3,  89, "SC-88Pro Map Var #02", "Horn Pad"},
    {'M',   2,   3,  90, "SC-88Pro Map Var #02", "Polysynth 2"},
    {'M',   2,   3,  91, "SC-88Pro Map Var #02", "SC Heaven"},
    {'M',   2,   3,  92, "SC-88Pro Map Var #02", "JP8 Sqr Pad"},
    {'M',   2,   3,  93, "SC-88Pro Map Var #02", "Panner Pad"},
    {'M',   2,   3,  94, "SC-88Pro Map Var #02", "Vox Sweep"},
    {'M',   2,   3,  96, "SC-88Pro Map Var #02", "African wood"},
    {'M',   2,   3,  97, "SC-88Pro Map Var #02", "Prologue"},
    {'M',   2,   3,  98, "SC-88Pro Map Var #02", "Soft Crystal"},
    {'M',   2,   3,  99, "SC-88Pro Map Var #02", "Nylon Harp"},
    {'M',   2,   3, 100, "SC-88Pro Map Var #02", "OB Stab"},
    {'M',   2,   3, 101, "SC-88Pro Map Var #02", "50's Sci-Fi"},
    {'M',   2,   3, 102, "SC-88Pro Map Var #02", "Echo Pan"},
    {'M',   2,   3, 104, "SC-88Pro Map Var #02", "Detune Sitar"},
    {'M',   2,   3, 119, "SC-88Pro Map Var #02", "Reverse Cym3"},
    {'M',   2,   3, 120, "SC-88Pro Map Var #02", "String Slap"},
    {'M',   2,   3, 122, "SC-88Pro Map Var #02", "Thunder"},
    {'M',   2,   3, 123, "SC-88Pro Map Var #02", "Horse-Gallop"},
    {'M',   2,   3, 124, "SC-88Pro Map Var #02", "DoorCreaking"},
    {'M',   2,   3, 125, "SC-88Pro Map Var #02", "Car-Stop"},
    {'M',   2,   3, 126, "SC-88Pro Map Var #02", "Screaming"},
    {'M',   2,   3, 127, "SC-88Pro Map Var #02", "Lasergun"},
    {'M',   2,   4,   0, "SC-8850 Map Var #02", "Mild Piano"},
    {'M',   2,   4,   1, "SC-8850 Map Var #02", "Rock Piano"},
    {'M',   2,   4,   2, "SC-8850 Map Var #02", "EG+Rhodes 2"},
    {'M',   2,   4,   6, "SC-8850 Map Var #02", "Harpsichord3"},
    {'M',   2,   4,   7, "SC-8850 Map Var #02", "Atk Clav.1"},
    {'M',   2,   4,  15, "SC-8850 Map Var #02", "Santur 3"},
    {'M',   2,   4,  16, "SC-8850 Map Var #02", "Ful Organ 1"},
    {'M',   2,   4,  17, "SC-8850 Map Var #02", "E.Organ 16+2"},
    {'M',   2,   4,  27, "SC-8850 Map Var #02", "Open Hard 1"},
    {'M',   2,   4,  28, "SC-8850 Map Var #02", "TC Muted Gt."},
    {'M',   2,   4,  29, "SC-8850 Map Var #02", "Overdrive 3"},
    {'M',   2,   4,  30, "SC-8850 Map Var #02", "Dazed Guitar"},
    {'M',   2,   4,  33, "SC-8850 Map Var #02", "Jazz Bass"},
    {'M',   2,   4,  34, "SC-8850 Map Var #02", "Picked Bass3"},
    {'M',   2,   4,  35, "SC-8850 Map Var #02", "Fretless Bs3"},
    {'M',   2,   4,  38, "SC-8850 Map Var #02", "CS Bass"},
    {'M',   2,   4,  39, "SC-8850 Map Var #02", "Modular Bass"},
    {'M',   2,   4,  44, "SC-8850 Map Var #02", "Trem Str.St."},
    {'M',   2,   4,  45, "SC-8850 Map Var #02", "Chamber Pizz"},
    {'M',   2,   4,  46, "SC-8850 Map Var #02", "Harp St."},
    {'M',   2,   4,  48, "SC-8850 Map Var #02", "ChamberStr"},
    {'M',   2,   4,  49, "SC-8850 Map Var #02", "SlowStrings3"},
    {'M',   2,   4,  50, "SC-8850 Map Var #02", "StackStrings"},
    {'M',   2,   4,  51, "SC-8850 Map Var #02", "JUNO Strings"},
    {'M',   2,   4,  53, "SC-8850 Map Var #02", "Voice Oohs 2"},
    {'M',   2,   4,  54, "SC-8850 Map Var #02", "SynVox 3"},
    {'M',   2,   4,  55, "SC-8850 Map Var #02", "6th Hit"},
    {'M',   2,   4,  56, "SC-8850 Map Var #02", "Trumpet   :"},
    {'M',   2,   4,  57, "SC-8850 Map Var #02", "Twin bones"},
    {'M',   2,   4,  59, "SC-8850 Map Var #02", "MuteTrumpet2"},
    {'M',   2,   4,  60, "SC-8850 Map Var #02", "Horn + Orche"},
    {'M',   2,   4,  61, "SC-8850 Map Var #02", "Bones Sect."},
    {'M',   2,   4,  62, "SC-8850 Map Var #02", "Stack Brass"},
    {'M',   2,   4,  63, "SC-8850 Map Var #02", "Warm Brass"},
    {'M',   2,   4,  73, "SC-8850 Map Var #02", "Flute Exp."},
    {'M',   2,   4,  80, "SC-8850 Map Var #02", "Hollow Mini"},
    {'M',   2,   4,  81, "SC-8850 Map Var #02", "Pulse Saw"},
    {'M',   2,   4,  82, "SC-8850 Map Var #02", "Pure PanLead"},
    {'M',   2,   4,  83, "SC-8850 Map Var #02", "Hybrid Lead"},
    {'M',   2,   4,  84, "SC-8850 Map Var #02", "FB.Charang"},
    {'M',   2,   4,  86, "SC-8850 Map Var #02", "5th Lead"},
    {'M',   2,   4,  87, "SC-8850 Map Var #02", "Fat & Perky"},
    {'M',   2,   4,  88, "SC-8850 Map Var #02", "New Age Pad"},
    {'M',   2,   4,  89, "SC-8850 Map Var #02", "Horn Pad"},
    {'M',   2,   4,  90, "SC-8850 Map Var #02", "Polysynth 2"},
    {'M',   2,   4,  91, "SC-8850 Map Var #02", "SC Heaven"},
    {'M',   2,   4,  92, "SC-8850 Map Var #02", "JP8 Sqr Pad"},
    {'M',   2,   4,  93, "SC-8850 Map Var #02", "Panner Pad"},
    {'M',   2,   4,  94, "SC-8850 Map Var #02", "Vox Sweep"},
    {'M',   2,   4,  95, "SC-8850 Map Var #02", "Ambient BPF"},
    {'M',   2,   4,  96, "SC-8850 Map Var #02", "African wood"},
    {'M',   2,   4,  97, "SC-8850 Map Var #02", "Prologue"},
    {'M',   2,   4,  98, "SC-8850 Map Var #02", "Soft Crystal"},
    {'M',   2,   4,  99, "SC-8850 Map Var #02", "Nylon Harp"},
    {'M',   2,   4, 100, "SC-8850 Map Var #02", "OB Stab"},
    {'M',   2,   4, 101, "SC-8850 Map Var #02", "50's Sci-Fi"},
    {'M',   2,   4, 102, "SC-8850 Map Var #02", "Echo Pan"},
    {'M',   2,   4, 103, "SC-8850 Map Var #02", "Star Mind"},
    {'M',   2,   4, 104, "SC-8850 Map Var #02", "Detune Sitar"},
    {'M',   2,   4, 117, "SC-8850 Map Var #02", "Real Tom 2"},
    {'M',   2,   4, 119, "SC-8850 Map Var #02", "Reverse Cym3"},
    {'M',   2,   4, 120, "SC-8850 Map Var #02", "String Slap"},
    {'M',   2,   4, 121, "SC-8850 Map Var #02", "Brth Nz Menu"},
    {'M',   2,   4, 122, "SC-8850 Map Var #02", "Thunder"},
    {'M',   2,   4, 123, "SC-8850 Map Var #02", "Horse-Gallop"},
    {'M',   2,   4, 124, "SC-8850 Map Var #02", "DoorCreaking"},
    {'M',   2,   4, 125, "SC-8850 Map Var #02", "Car-Stop"},
    {'M',   2,   4, 126, "SC-8850 Map Var #02", "Screaming"},
    {'M',   2,   4, 127, "SC-8850 Map Var #02", "Lasergun"},
    {'M',   3,   0,   7, "SC-8850 Var #03", "Atk Clav.2"},
    {'M',   3,   0,  16, "SC-8850 Var #03", "Ful Organ 2"},
    {'M',   3,   0,  17, "SC-8850 Var #03", "Jazz Organ 2"},
    {'M',   3,   0,  27, "SC-8850 Var #03", "Open Hard 2"},
    {'M',   3,   0,  29, "SC-8850 Var #03", "More Drive"},
    {'M',   3,   0,  30, "SC-8850 Var #03", "Distortion:"},
    {'M',   3,   0,  33, "SC-8850 Var #03", "Jazz Bass 2"},
    {'M',   3,   0,  34, "SC-8850 Var #03", "Picked Bass4"},
    {'M',   3,   0,  35, "SC-8850 Var #03", "Fretless Bs4"},
    {'M',   3,   0,  38, "SC-8850 Var #03", "JP-4 Bass"},
    {'M',   3,   0,  39, "SC-8850 Var #03", "Seq Bass"},
    {'M',   3,   0,  45, "SC-8850 Var #03", "St.Pizzicato"},
    {'M',   3,   0,  48, "SC-8850 Var #03", "Cello sect."},
    {'M',   3,   0,  50, "SC-8850 Var #03", "JP Strings"},
    {'M',   3,   0,  51, "SC-8850 Var #03", "FilteredOrch"},
    {'M',   3,   0,  53, "SC-8850 Var #03", "Chorus Oohs2"},
    {'M',   3,   0,  55, "SC-8850 Var #03", "Euro Hit"},
    {'M',   3,   0,  56, "SC-8850 Var #03", "Dark Trumpet"},
    {'M',   3,   0,  57, "SC-8850 Var #03", "Bones & Tuba"},
    {'M',   3,   0,  59, "SC-8850 Var #03", "MuteTrumpet3"},
    {'M',   3,   0,  60, "SC-8850 Var #03", "Wide FreHrns"},
    {'M',   3,   0,  61, "SC-8850 Var #03", "St. Brass ff"},
    {'M',   3,   0,  62, "SC-8850 Var #03", "SH-5 Brass"},
    {'M',   3,   0,  63, "SC-8850 Var #03", "Synth Brass3"},
    {'M',   3,   0,  73, "SC-8850 Var #03", "Flt Travelso"},
    {'M',   3,   0,  80, "SC-8850 Var #03", "Mellow FM"},
    {'M',   3,   0,  81, "SC-8850 Var #03", "Feline GR"},
    {'M',   3,   0,  83, "SC-8850 Var #03", "Unison SqrLd"},
    {'M',   3,   0,  84, "SC-8850 Var #03", "Fat GR Lead"},
    {'M',   3,   0,  86, "SC-8850 Var #03", "5th Ana.Clav"},
    {'M',   3,   0,  87, "SC-8850 Var #03", "JUNO Rave"},
    {'M',   3,   0,  88, "SC-8850 Var #03", "Bell Heaven"},
    {'M',   3,   0,  89, "SC-8850 Var #03", "Rotary Strng"},
    {'M',   3,   0,  90, "SC-8850 Var #03", "Poly King"},
    {'M',   3,   0,  91, "SC-8850 Var #03", "Itopia"},
    {'M',   3,   0,  92, "SC-8850 Var #03", "7thBelPad"},
    {'M',   3,   0,  93, "SC-8850 Var #03", "Steel Pad"},
    {'M',   3,   0,  95, "SC-8850 Var #03", "Sync Pad"},
    {'M',   3,   0,  96, "SC-8850 Var #03", "Anklung Pad"},
    {'M',   3,   0,  97, "SC-8850 Var #03", "Prologue 2"},
    {'M',   3,   0,  98, "SC-8850 Var #03", "Round Glock"},
    {'M',   3,   0,  99, "SC-8850 Var #03", "Harpvox"},
    {'M',   3,   0, 100, "SC-8850 Var #03", "Brass Star"},
    {'M',   3,   0, 101, "SC-8850 Var #03", "Abduction"},
    {'M',   3,   0, 102, "SC-8850 Var #03", "Echo Pan 2"},
    {'M',   3,   0, 103, "SC-8850 Var #03", "Star Dust"},
    {'M',   3,   0, 104, "SC-8850 Var #03", "Sitar 3"},
    {'M',   3,   0, 117, "SC-8850 Var #03", "Jazz Tom"},
    {'M',   3,   0, 119, "SC-8850 Var #03", "Reverse Cym4"},
    {'M',   3,   0, 120, "SC-8850 Var #03", "Gt.CutNoise2"},
    {'M',   3,   0, 121, "SC-8850 Var #03", "Fl.Breath 1"},
    {'M',   3,   0, 122, "SC-8850 Var #03", "Wind"},
    {'M',   3,   0, 123, "SC-8850 Var #03", "Bird 2"},
    {'M',   3,   0, 124, "SC-8850 Var #03", "Door"},
    {'M',   3,   0, 125, "SC-8850 Var #03", "Car-Pass"},
    {'M',   3,   0, 126, "SC-8850 Var #03", "Punch"},
    {'M',   3,   0, 127, "SC-8850 Var #03", "Explosion"},
    {'M',   3,   1, 122, "SC-55 Map Var #03", "Wind"},
    {'M',   3,   1, 123, "SC-55 Map Var #03", "Bird 2"},
    {'M',   3,   1, 124, "SC-55 Map Var #03", "Door"},
    {'M',   3,   1, 125, "SC-55 Map Var #03", "Car-Pass"},
    {'M',   3,   1, 126, "SC-55 Map Var #03", "Punch"},
    {'M',   3,   1, 127, "SC-55 Map Var #03", "Explosion"},
    {'M',   3,   2,  35, "SC-88 Map Var #03", "FretlessBs4"},
    {'M',   3,   2,  39, "SC-88 Map Var #03", "Seq Bass"},
    {'M',   3,   2,  80, "SC-88 Map Var #03", "Mellow FM"},
    {'M',   3,   2,  81, "SC-88 Map Var #03", "Feline GR"},
    {'M',   3,   2,  89, "SC-88 Map Var #03", "Rotary Strng"},
    {'M',   3,   2,  98, "SC-88 Map Var #03", "Round Glock"},
    {'M',   3,   2,  99, "SC-88 Map Var #03", "Harpvox"},
    {'M',   3,   2, 102, "SC-88 Map Var #03", "Echo Pan 2"},
    {'M',   3,   2, 120, "SC-88 Map Var #03", "Gt.CutNa. 2"},
    {'M',   3,   2, 122, "SC-88 Map Var #03", "Wind"},
    {'M',   3,   2, 123, "SC-88 Map Var #03", "Bird 2"},
    {'M',   3,   2, 124, "SC-88 Map Var #03", "Door"},
    {'M',   3,   2, 125, "SC-88 Map Var #03", "Car-Pass"},
    {'M',   3,   2, 126, "SC-88 Map Var #03", "Punch"},
    {'M',   3,   2, 127, "SC-88 Map Var #03", "Explosion"},
    {'M',   3,   3,  27, "SC-88Pro Map Var #03", "Open Hard 2"},
    {'M',   3,   3,  29, "SC-88Pro Map Var #03", "More Drive"},
    {'M',   3,   3,  30, "SC-88Pro Map Var #03", "Distortion:"},
    {'M',   3,   3,  33, "SC-88Pro Map Var #03", "Jazz Bass 2"},
    {'M',   3,   3,  34, "SC-88Pro Map Var #03", "Picked Bass4"},
    {'M',   3,   3,  35, "SC-88Pro Map Var #03", "Fretless Bs4"},
    {'M',   3,   3,  38, "SC-88Pro Map Var #03", "JP-4 Bass"},
    {'M',   3,   3,  39, "SC-88Pro Map Var #03", "Seq Bass"},
    {'M',   3,   3,  45, "SC-88Pro Map Var #03", "St.Pizzicato"},
    {'M',   3,   3,  48, "SC-88Pro Map Var #03", "Cello sect."},
    {'M',   3,   3,  50, "SC-88Pro Map Var #03", "JP Strings"},
    {'M',   3,   3,  60, "SC-88Pro Map Var #03", "Wide FreHrns"},
    {'M',   3,   3,  62, "SC-88Pro Map Var #03", "SH-5 Brass"},
    {'M',   3,   3,  73, "SC-88Pro Map Var #03", "Flt Travelso"},
    {'M',   3,   3,  80, "SC-88Pro Map Var #03", "Mellow FM"},
    {'M',   3,   3,  81, "SC-88Pro Map Var #03", "Feline GR"},
    {'M',   3,   3,  86, "SC-88Pro Map Var #03", "5th Ana.Clav"},
    {'M',   3,   3,  87, "SC-88Pro Map Var #03", "JUNO Rave"},
    {'M',   3,   3,  88, "SC-88Pro Map Var #03", "Bell Heaven"},
    {'M',   3,   3,  89, "SC-88Pro Map Var #03", "Rotary Strng"},
    {'M',   3,   3,  90, "SC-88Pro Map Var #03", "Poly King"},
    {'M',   3,   3,  92, "SC-88Pro Map Var #03", "7thBelPad"},
    {'M',   3,   3,  96, "SC-88Pro Map Var #03", "Anklung Pad"},
    {'M',   3,   3,  97, "SC-88Pro Map Var #03", "Prologue 2"},
    {'M',   3,   3,  98, "SC-88Pro Map Var #03", "Round Glock"},
    {'M',   3,   3,  99, "SC-88Pro Map Var #03", "Harpvox"},
    {'M',   3,   3, 101, "SC-88Pro Map Var #03", "Abduction"},
    {'M',   3,   3, 102, "SC-88Pro Map Var #03", "Echo Pan 2"},
    {'M',   3,   3, 104, "SC-88Pro Map Var #03", "Sitar 3"},
    {'M',   3,   3, 120, "SC-88Pro Map Var #03", "Gt.CutNoise2"},
    {'M',   3,   3, 122, "SC-88Pro Map Var #03", "Wind"},
    {'M',   3,   3, 123, "SC-88Pro Map Var #03", "Bird 2"},
    {'M',   3,   3, 124, "SC-88Pro Map Var #03", "Door"},
    {'M',   3,   3, 125, "SC-88Pro Map Var #03", "Car-Pass"},
    {'M',   3,   3, 126, "SC-88Pro Map Var #03", "Punch"},
    {'M',   3,   3, 127, "SC-88Pro Map Var #03", "Explosion"},
    {'M',   3,   4,   7, "SC-8850 Map Var #03", "Atk Clav.2"},
    {'M',   3,   4,  16, "SC-8850 Map Var #03", "Ful Organ 2"},
    {'M',   3,   4,  17, "SC-8850 Map Var #03", "Jazz Organ 2"},
    {'M',   3,   4,  27, "SC-8850 Map Var #03", "Open Hard 2"},
    {'M',   3,   4,  29, "SC-8850 Map Var #03", "More Drive"},
    {'M',   3,   4,  30, "SC-8850 Map Var #03", "Distortion:"},
    {'M',   3,   4,  33, "SC-8850 Map Var #03", "Jazz Bass 2"},
    {'M',   3,   4,  34, "SC-8850 Map Var #03", "Picked Bass4"},
    {'M',   3,   4,  35, "SC-8850 Map Var #03", "Fretless Bs4"},
    {'M',   3,   4,  38, "SC-8850 Map Var #03", "JP-4 Bass"},
    {'M',   3,   4,  39, "SC-8850 Map Var #03", "Seq Bass"},
    {'M',   3,   4,  45, "SC-8850 Map Var #03", "St.Pizzicato"},
    {'M',   3,   4,  48, "SC-8850 Map Var #03", "Cello sect."},
    {'M',   3,   4,  50, "SC-8850 Map Var #03", "JP Strings"},
    {'M',   3,   4,  51, "SC-8850 Map Var #03", "FilteredOrch"},
    {'M',   3,   4,  53, "SC-8850 Map Var #03", "Chorus Oohs2"},
    {'M',   3,   4,  55, "SC-8850 Map Var #03", "Euro Hit"},
    {'M',   3,   4,  56, "SC-8850 Map Var #03", "Dark Trumpet"},
    {'M',   3,   4,  57, "SC-8850 Map Var #03", "Bones & Tuba"},
    {'M',   3,   4,  59, "SC-8850 Map Var #03", "MuteTrumpet3"},
    {'M',   3,   4,  60, "SC-8850 Map Var #03", "Wide FreHrns"},
    {'M',   3,   4,  61, "SC-8850 Map Var #03", "St. Brass ff"},
    {'M',   3,   4,  62, "SC-8850 Map Var #03", "SH-5 Brass"},
    {'M',   3,   4,  63, "SC-8850 Map Var #03", "Synth Brass3"},
    {'M',   3,   4,  73, "SC-8850 Map Var #03", "Flt Travelso"},
    {'M',   3,   4,  80, "SC-8850 Map Var #03", "Mellow FM"},
    {'M',   3,   4,  81, "SC-8850 Map Var #03", "Feline GR"},
    {'M',   3,   4,  83, "SC-8850 Map Var #03", "Unison SqrLd"},
    {'M',   3,   4,  84, "SC-8850 Map Var #03", "Fat GR Lead"},
    {'M',   3,   4,  86, "SC-8850 Map Var #03", "5th Ana.Clav"},
    {'M',   3,   4,  87, "SC-8850 Map Var #03", "JUNO Rave"},
    {'M',   3,   4,  88, "SC-8850 Map Var #03", "Bell Heaven"},
    {'M',   3,   4,  89, "SC-8850 Map Var #03", "Rotary Strng"},
    {'M',   3,   4,  90, "SC-8850 Map Var #03", "Poly King"},
    {'M',   3,   4,  91, "SC-8850 Map Var #03", "Itopia"},
    {'M',   3,   4,  92, "SC-8850 Map Var #03", "7thBelPad"},
    {'M',   3,   4,  93, "SC-8850 Map Var #03", "Steel Pad"},
    {'M',   3,   4,  95, "SC-8850 Map Var #03", "Sync Pad"},
    {'M',   3,   4,  96, "SC-8850 Map Var #03", "Anklung Pad"},
    {'M',   3,   4,  97, "SC-8850 Map Var #03", "Prologue 2"},
    {'M',   3,   4,  98, "SC-8850 Map Var #03", "Round Glock"},
    {'M',   3,   4,  99, "SC-8850 Map Var #03", "Harpvox"},
    {'M',   3,   4, 100, "SC-8850 Map Var #03", "Brass Star"},
    {'M',   3,   4, 101, "SC-8850 Map Var #03", "Abduction"},
    {'M',   3,   4, 102, "SC-8850 Map Var #03", "Echo Pan 2"},
    {'M',   3,   4, 103, "SC-8850 Map Var #03", "Star Dust"},
    {'M',   3,   4, 104, "SC-8850 Map Var #03", "Sitar 3"},
    {'M',   3,   4, 117, "SC-8850 Map Var #03", "Jazz Tom"},
    {'M',   3,   4, 119, "SC-8850 Map Var #03", "Reverse Cym4"},
    {'M',   3,   4, 120, "SC-8850 Map Var #03", "Gt.CutNoise2"},
    {'M',   3,   4, 121, "SC-8850 Map Var #03", "Fl.Breath 1"},
    {'M',   3,   4, 122, "SC-8850 Map Var #03", "Wind"},
    {'M',   3,   4, 123, "SC-8850 Map Var #03", "Bird 2"},
    {'M',   3,   4, 124, "SC-8850 Map Var #03", "Door"},
    {'M',   3,   4, 125, "SC-8850 Map Var #03", "Car-Pass"},
    {'M',   3,   4, 126, "SC-8850 Map Var #03", "Punch"},
    {'M',   3,   4, 127, "SC-8850 Map Var #03", "Explosion"},
    {'M',   4,   0,  16, "SC-8850 Var #04", "Ful Organ 3"},
    {'M',   4,   0,  17, "SC-8850 Var #04", "Jazz Organ 3"},
    {'M',   4,   0,  27, "SC-8850 Var #04", "JC Clean Gt."},
    {'M',   4,   0,  29, "SC-8850 Var #04", "Guitar Pinch"},
    {'M',   4,   0,  30, "SC-8850 Var #04", "Dist.Fast :"},
    {'M',   4,   0,  33, "SC-8850 Var #04", "Rock Bass"},
    {'M',   4,   0,  34, "SC-8850 Var #04", "Double Pick"},
    {'M',   4,   0,  35, "SC-8850 Var #04", "Syn Fretless"},
    {'M',   4,   0,  38, "SC-8850 Var #04", "JP-8 Bass"},
    {'M',   4,   0,  39, "SC-8850 Var #04", "MG Bass"},
    {'M',   4,   0,  48, "SC-8850 Var #04", "Bright Str.2"},
    {'M',   4,   0,  50, "SC-8850 Var #04", "Chorus Str."},
    {'M',   4,   0,  51, "SC-8850 Var #04", "JP Saw Str."},
    {'M',   4,   0,  53, "SC-8850 Var #04", "OohsCodeMaj7"},
    {'M',   4,   0,  56, "SC-8850 Var #04", "Trumpet & Nz"},
    {'M',   4,   0,  57, "SC-8850 Var #04", "Bright Tb"},
    {'M',   4,   0,  61, "SC-8850 Var #04", "Quad Brass1"},
    {'M',   4,   0,  62, "SC-8850 Var #04", "MKS Brass"},
    {'M',   4,   0,  63, "SC-8850 Var #04", "Sync Brass"},
    {'M',   4,   0,  80, "SC-8850 Var #04", "CC Solo"},
    {'M',   4,   0,  81, "SC-8850 Var #04", "Big Lead"},
    {'M',   4,   0,  83, "SC-8850 Var #04", "FatSolo Lead"},
    {'M',   4,   0,  84, "SC-8850 Var #04", "Windy GR Ld"},
    {'M',   4,   0,  86, "SC-8850 Var #04", "5th Pulse"},
    {'M',   4,   0,  87, "SC-8850 Var #04", "JP8 BsLead 1"},
    {'M',   4,   0,  88, "SC-8850 Var #04", "Fantasia 3"},
    {'M',   4,   0,  89, "SC-8850 Var #04", "OB Soft Pad"},
    {'M',   4,   0,  90, "SC-8850 Var #04", "Super Poly"},
    {'M',   4,   0,  91, "SC-8850 Var #04", "Water Space"},
    {'M',   4,   0,  92, "SC-8850 Var #04", "Steel Glass"},
    {'M',   4,   0,  93, "SC-8850 Var #04", "Special Rave"},
    {'M',   4,   0,  95, "SC-8850 Var #04", "Warriors"},
    {'M',   4,   0,  96, "SC-8850 Var #04", "Rattle Pad"},
    {'M',   4,   0,  97, "SC-8850 Var #04", "Hols Strings"},
    {'M',   4,   0,  98, "SC-8850 Var #04", "Loud Glock"},
    {'M',   4,   0,  99, "SC-8850 Var #04", "HollowReleas"},
    {'M',   4,   0, 100, "SC-8850 Var #04", "Choir Stab"},
    {'M',   4,   0, 101, "SC-8850 Var #04", "Auhbient"},
    {'M',   4,   0, 102, "SC-8850 Var #04", "Big Panner"},
    {'M',   4,   0, 103, "SC-8850 Var #04", "Rep.Trance"},
    {'M',   4,   0, 104, "SC-8850 Var #04", "Sitar/Drone"},
    {'M',   4,   0, 117, "SC-8850 Var #04", "Brush Tom"},
    {'M',   4,   0, 120, "SC-8850 Var #04", "Dist.CutNoiz"},
    {'M',   4,   0, 121, "SC-8850 Var #04", "Fl.Breath 2"},
    {'M',   4,   0, 122, "SC-8850 Var #04", "Stream"},
    {'M',   4,   0, 123, "SC-8850 Var #04", "Kitty"},
    {'M',   4,   0, 124, "SC-8850 Var #04", "Scratch"},
    {'M',   4,   0, 125, "SC-8850 Var #04", "Car-Crash"},
    {'M',   4,   0, 126, "SC-8850 Var #04", "Heart Beat"},
    {'M',   4,   0, 127, "SC-8850 Var #04", "Eruption"},
    {'M',   4,   1, 122, "SC-55 Map Var #04", "Stream"},
    {'M',   4,   1, 124, "SC-55 Map Var #04", "Scratch"},
    {'M',   4,   1, 125, "SC-55 Map Var #04", "Car-Crash"},
    {'M',   4,   1, 126, "SC-55 Map Var #04", "Heart Beat"},
    {'M',   4,   2,  35, "SC-88 Map Var #04", "SynFretless"},
    {'M',   4,   2,  80, "SC-88 Map Var #04", "CC Solo"},
    {'M',   4,   2,  81, "SC-88 Map Var #04", "Big Lead"},
    {'M',   4,   2,  89, "SC-88 Map Var #04", "Soft Pad"},
    {'M',   4,   2,  98, "SC-88 Map Var #04", "Loud Glock"},
    {'M',   4,   2,  99, "SC-88 Map Var #04", "HollowRels"},
    {'M',   4,   2, 102, "SC-88 Map Var #04", "Big Panner"},
    {'M',   4,   2, 120, "SC-88 Map Var #04", "Dist.CutNz."},
    {'M',   4,   2, 122, "SC-88 Map Var #04", "Stream"},
    {'M',   4,   2, 123, "SC-88 Map Var #04", "Kitty"},
    {'M',   4,   2, 124, "SC-88 Map Var #04", "Scratch"},
    {'M',   4,   2, 125, "SC-88 Map Var #04", "Car-Crash"},
    {'M',   4,   2, 126, "SC-88 Map Var #04", "Heart Beat"},
    {'M',   4,   3,  27, "SC-88Pro Map Var #04", "JC Clean Gt."},
    {'M',   4,   3,  30, "SC-88Pro Map Var #04", "Dist.Fast :"},
    {'M',   4,   3,  33, "SC-88Pro Map Var #04", "Rock Bass"},
    {'M',   4,   3,  35, "SC-88Pro Map Var #04", "Syn Fretless"},
    {'M',   4,   3,  38, "SC-88Pro Map Var #04", "JP-8 Bass"},
    {'M',   4,   3,  39, "SC-88Pro Map Var #04", "MG Bass"},
    {'M',   4,   3,  62, "SC-88Pro Map Var #04", "MKS Brass"},
    {'M',   4,   3,  80, "SC-88Pro Map Var #04", "CC Solo"},
    {'M',   4,   3,  81, "SC-88Pro Map Var #04", "Big Lead"},
    {'M',   4,   3,  87, "SC-88Pro Map Var #04", "JP8 BsLead 1"},
    {'M',   4,   3,  89, "SC-88Pro Map Var #04", "OB Soft Pad"},
    {'M',   4,   3,  96, "SC-88Pro Map Var #04", "Rattle Pad"},
    {'M',   4,   3,  97, "SC-88Pro Map Var #04", "Hols Strings"},
    {'M',   4,   3,  98, "SC-88Pro Map Var #04", "Loud Glock"},
    {'M',   4,   3,  99, "SC-88Pro Map Var #04", "HollowReleas"},
    {'M',   4,   3, 101, "SC-88Pro Map Var #04", "Auhbient"},
    {'M',   4,   3, 102, "SC-88Pro Map Var #04", "Big Panner"},
    {'M',   4,   3, 120, "SC-88Pro Map Var #04", "Dist.CutNoiz"},
    {'M',   4,   3, 122, "SC-88Pro Map Var #04", "Stream"},
    {'M',   4,   3, 123, "SC-88Pro Map Var #04", "Kitty"},
    {'M',   4,   3, 124, "SC-88Pro Map Var #04", "Scratch"},
    {'M',   4,   3, 125, "SC-88Pro Map Var #04", "Car-Crash"},
    {'M',   4,   3, 126, "SC-88Pro Map Var #04", "Heart Beat"},
    {'M',   4,   3, 127, "SC-88Pro Map Var #04", "Eruption"},
    {'M',   4,   4,  16, "SC-8850 Map Var #04", "Ful Organ 3"},
    {'M',   4,   4,  17, "SC-8850 Map Var #04", "Jazz Organ 3"},
    {'M',   4,   4,  27, "SC-8850 Map Var #04", "JC Clean Gt."},
    {'M',   4,   4,  29, "SC-8850 Map Var #04", "Guitar Pinch"},
    {'M',   4,   4,  30, "SC-8850 Map Var #04", "Dist.Fast :"},
    {'M',   4,   4,  33, "SC-8850 Map Var #04", "Rock Bass"},
    {'M',   4,   4,  34, "SC-8850 Map Var #04", "Double Pick"},
    {'M',   4,   4,  35, "SC-8850 Map Var #04", "Syn Fretless"},
    {'M',   4,   4,  38, "SC-8850 Map Var #04", "JP-8 Bass"},
    {'M',   4,   4,  39, "SC-8850 Map Var #04", "MG Bass"},
    {'M',   4,   4,  48, "SC-8850 Map Var #04", "Bright Str.2"},
    {'M',   4,   4,  50, "SC-8850 Map Var #04", "Chorus Str."},
    {'M',   4,   4,  51, "SC-8850 Map Var #04", "JP Saw Str."},
    {'M',   4,   4,  53, "SC-8850 Map Var #04", "OohsCodeMaj7"},
    {'M',   4,   4,  56, "SC-8850 Map Var #04", "Trumpet & Nz"},
    {'M',   4,   4,  57, "SC-8850 Map Var #04", "Bright Tb"},
    {'M',   4,   4,  61, "SC-8850 Map Var #04", "Quad Brass1"},
    {'M',   4,   4,  62, "SC-8850 Map Var #04", "MKS Brass"},
    {'M',   4,   4,  63, "SC-8850 Map Var #04", "Sync Brass"},
    {'M',   4,   4,  80, "SC-8850 Map Var #04", "CC Solo"},
    {'M',   4,   4,  81, "SC-8850 Map Var #04", "Big Lead"},
    {'M',   4,   4,  83, "SC-8850 Map Var #04", "FatSolo Lead"},
    {'M',   4,   4,  84, "SC-8850 Map Var #04", "Windy GR Ld"},
    {'M',   4,   4,  86, "SC-8850 Map Var #04", "5th Pulse"},
    {'M',   4,   4,  87, "SC-8850 Map Var #04", "JP8 BsLead 1"},
    {'M',   4,   4,  88, "SC-8850 Map Var #04", "Fantasia 3"},
    {'M',   4,   4,  89, "SC-8850 Map Var #04", "OB Soft Pad"},
    {'M',   4,   4,  90, "SC-8850 Map Var #04", "Super Poly"},
    {'M',   4,   4,  91, "SC-8850 Map Var #04", "Water Space"},
    {'M',   4,   4,  92, "SC-8850 Map Var #04", "Steel Glass"},
    {'M',   4,   4,  93, "SC-8850 Map Var #04", "Special Rave"},
    {'M',   4,   4,  95, "SC-8850 Map Var #04", "Warriors"},
    {'M',   4,   4,  96, "SC-8850 Map Var #04", "Rattle Pad"},
    {'M',   4,   4,  97, "SC-8850 Map Var #04", "Hols Strings"},
    {'M',   4,   4,  98, "SC-8850 Map Var #04", "Loud Glock"},
    {'M',   4,   4,  99, "SC-8850 Map Var #04", "HollowReleas"},
    {'M',   4,   4, 100, "SC-8850 Map Var #04", "Choir Stab"},
    {'M',   4,   4, 101, "SC-8850 Map Var #04", "Auhbient"},
    {'M',   4,   4, 102, "SC-8850 Map Var #04", "Big Panner"},
    {'M',   4,   4, 103, "SC-8850 Map Var #04", "Rep.Trance"},
    {'M',   4,   4, 104, "SC-8850 Map Var #04", "Sitar/Drone"},
    {'M',   4,   4, 117, "SC-8850 Map Var #04", "Brush Tom"},
    {'M',   4,   4, 120, "SC-8850 Map Var #04", "Dist.CutNoiz"},
    {'M',   4,   4, 121, "SC-8850 Map Var #04", "Fl.Breath 2"},
    {'M',   4,   4, 122, "SC-8850 Map Var #04", "Stream"},
    {'M',   4,   4, 123, "SC-8850 Map Var #04", "Kitty"},
    {'M',   4,   4, 124, "SC-8850 Map Var #04", "Scratch"},
    {'M',   4,   4, 125, "SC-8850 Map Var #04", "Car-Crash"},
    {'M',   4,   4, 126, "SC-8850 Map Var #04", "Heart Beat"},
    {'M',   4,   4, 127, "SC-8850 Map Var #04", "Eruption"},
    {'M',   5,   0,  16, "SC-8850 Var #05", "Ful Organ 4"},
    {'M',   5,   0,  17, "SC-8850 Var #05", "Jazz Organ 4"},
    {'M',   5,   0,  27, "SC-8850 Var #05", "Atk CleanGt."},
    {'M',   5,   0,  29, "SC-8850 Var #05", "Attack Drive"},
    {'M',   5,   0,  30, "SC-8850 Var #05", "Attack Dist"},
    {'M',   5,   0,  33, "SC-8850 Var #05", "Heart Bass"},
    {'M',   5,   0,  35, "SC-8850 Var #05", "Mr.Smooth"},
    {'M',   5,   0,  38, "SC-8850 Var #05", "P5 Bass"},
    {'M',   5,   0,  39, "SC-8850 Var #05", "Mg Oct Bass1"},
    {'M',   5,   0,  48, "SC-8850 Var #05", "Bright Str.3"},
    {'M',   5,   0,  51, "SC-8850 Var #05", "Hybrid Str.2"},
    {'M',   5,   0,  53, "SC-8850 Var #05", "OohsCodeSus4"},
    {'M',   5,   0,  61, "SC-8850 Var #05", "Quad Brass2"},
    {'M',   5,   0,  62, "SC-8850 Var #05", "Jump Brass"},
    {'M',   5,   0,  63, "SC-8850 Var #05", "Fat SynBrass"},
    {'M',   5,   0,  80, "SC-8850 Var #05", "Shmoog"},
    {'M',   5,   0,  81, "SC-8850 Var #05", "Velo Lead"},
    {'M',   5,   0,  83, "SC-8850 Var #05", "ForcefulLead"},
    {'M',   5,   0,  84, "SC-8850 Var #05", "Mellow GR Ld"},
    {'M',   5,   0,  86, "SC-8850 Var #05", "JP 5th Saw"},
    {'M',   5,   0,  87, "SC-8850 Var #05", "JP8 BsLead 2"},
    {'M',   5,   0,  88, "SC-8850 Var #05", "Fantasia 4"},
    {'M',   5,   0,  89, "SC-8850 Var #05", "Sine Pad"},
    {'M',   5,   0,  91, "SC-8850 Var #05", "Cold Space"},
    {'M',   5,   0,  92, "SC-8850 Var #05", "Bottle Stack"},
    {'M',   5,   0,  93, "SC-8850 Var #05", "Metal Pad 2"},
    {'M',   5,   0,  96, "SC-8850 Var #05", "Saw Impulse"},
    {'M',   5,   0,  97, "SC-8850 Var #05", "HistoryWave"},
    {'M',   5,   0,  98, "SC-8850 Var #05", "GlockenChime"},
    {'M',   5,   0,  99, "SC-8850 Var #05", "Nylon+Rhodes"},
    {'M',   5,   0, 100, "SC-8850 Var #05", "D-50 Retour"},
    {'M',   5,   0, 101, "SC-8850 Var #05", "LFO Pad"},
    {'M',   5,   0, 102, "SC-8850 Var #05", "Reso Panner"},
    {'M',   5,   0, 103, "SC-8850 Var #05", "Etherality"},
    {'M',   5,   0, 104, "SC-8850 Var #05", "Sitar 4"},
    {'M',   5,   0, 120, "SC-8850 Var #05", "Bass Slide"},
    {'M',   5,   0, 121, "SC-8850 Var #05", "Fl.Breath 3"},
    {'M',   5,   0, 122, "SC-8850 Var #05", "Bubble"},
    {'M',   5,   0, 123, "SC-8850 Var #05", "Growl"},
    {'M',   5,   0, 124, "SC-8850 Var #05", "Wind Chimes"},
    {'M',   5,   0, 125, "SC-8850 Var #05", "Siren"},
    {'M',   5,   0, 126, "SC-8850 Var #05", "Footsteps"},
    {'M',   5,   0, 127, "SC-8850 Var #05", "Big Shot"},
    {'M',   5,   1, 122, "SC-55 Map Var #05", "Bubble"},
    {'M',   5,   1, 124, "SC-55 Map Var #05", "Wind Chimes"},
    {'M',   5,   1, 125, "SC-55 Map Var #05", "Siren"},
    {'M',   5,   1, 126, "SC-55 Map Var #05", "Footsteps"},
    {'M',   5,   2,  35, "SC-88 Map Var #05", "Mr.Smooth"},
    {'M',   5,   2,  80, "SC-88 Map Var #05", "Shmoog"},
    {'M',   5,   2,  81, "SC-88 Map Var #05", "Velo Lead"},
    {'M',   5,   2,  98, "SC-88 Map Var #05", "GlocknChime"},
    {'M',   5,   2,  99, "SC-88 Map Var #05", "Nylon+Rhodes"},
    {'M',   5,   2, 102, "SC-88 Map Var #05", "Reso Panner"},
    {'M',   5,   2, 120, "SC-88 Map Var #05", "Bass Slide"},
    {'M',   5,   2, 122, "SC-88 Map Var #05", "Bubble"},
    {'M',   5,   2, 123, "SC-88 Map Var #05", "Growl"},
    {'M',   5,   2, 124, "SC-88 Map Var #05", "Wind Chimes"},
    {'M',   5,   2, 125, "SC-88 Map Var #05", "Siren"},
    {'M',   5,   2, 126, "SC-88 Map Var #05", "Footsteps"},
    {'M',   5,   3,  35, "SC-88Pro Map Var #05", "Mr.Smooth"},
    {'M',   5,   3,  38, "SC-88Pro Map Var #05", "P5 Bass"},
    {'M',   5,   3,  39, "SC-88Pro Map Var #05", "Mg Oct Bass1"},
    {'M',   5,   3,  80, "SC-88Pro Map Var #05", "Shmoog"},
    {'M',   5,   3,  81, "SC-88Pro Map Var #05", "Velo Lead"},
    {'M',   5,   3,  87, "SC-88Pro Map Var #05", "JP8 BsLead 2"},
    {'M',   5,   3,  98, "SC-88Pro Map Var #05", "GlockenChime"},
    {'M',   5,   3,  99, "SC-88Pro Map Var #05", "Nylon+Rhodes"},
    {'M',   5,   3, 101, "SC-88Pro Map Var #05", "LFO Pad"},
    {'M',   5,   3, 102, "SC-88Pro Map Var #05", "Reso Panner"},
    {'M',   5,   3, 120, "SC-88Pro Map Var #05", "Bass Slide"},
    {'M',   5,   3, 122, "SC-88Pro Map Var #05", "Bubble"},
    {'M',   5,   3, 123, "SC-88Pro Map Var #05", "Growl"},
    {'M',   5,   3, 124, "SC-88Pro Map Var #05", "Wind Chimes"},
    {'M',   5,   3, 125, "SC-88Pro Map Var #05", "Siren"},
    {'M',   5,   3, 126, "SC-88Pro Map Var #05", "Footsteps"},
    {'M',   5,   3, 127, "SC-88Pro Map Var #05", "Big Shot"},
    {'M',   5,   4,  16, "SC-8850 Map Var #05", "Ful Organ 4"},
    {'M',   5,   4,  17, "SC-8850 Map Var #05", "Jazz Organ 4"},
    {'M',   5,   4,  27, "SC-8850 Map Var #05", "Atk CleanGt."},
    {'M',   5,   4,  29, "SC-8850 Map Var #05", "Attack Drive"},
    {'M',   5,   4,  30, "SC-8850 Map Var #05", "Attack Dist"},
    {'M',   5,   4,  33, "SC-8850 Map Var #05", "Heart Bass"},
    {'M',   5,   4,  35, "SC-8850 Map Var #05", "Mr.Smooth"},
    {'M',   5,   4,  38, "SC-8850 Map Var #05", "P5 Bass"},
    {'M',   5,   4,  39, "SC-8850 Map Var #05", "Mg Oct Bass1"},
    {'M',   5,   4,  48, "SC-8850 Map Var #05", "Bright Str.3"},
    {'M',   5,   4,  51, "SC-8850 Map Var #05", "Hybrid Str.2"},
    {'M',   5,   4,  53, "SC-8850 Map Var #05", "OohsCodeSus4"},
    {'M',   5,   4,  61, "SC-8850 Map Var #05", "Quad Brass2"},
    {'M',   5,   4,  62, "SC-8850 Map Var #05", "Jump Brass"},
    {'M',   5,   4,  63, "SC-8850 Map Var #05", "Fat SynBrass"},
    {'M',   5,   4,  80, "SC-8850 Map Var #05", "Shmoog"},
    {'M',   5,   4,  81, "SC-8850 Map Var #05", "Velo Lead"},
    {'M',   5,   4,  83, "SC-8850 Map Var #05", "ForcefulLead"},
    {'M',   5,   4,  84, "SC-8850 Map Var #05", "Mellow GR Ld"},
    {'M',   5,   4,  86, "SC-8850 Map Var #05", "JP 5th Saw"},
    {'M',   5,   4,  87, "SC-8850 Map Var #05", "JP8 BsLead 2"},
    {'M',   5,   4,  88, "SC-8850 Map Var #05", "Fantasia 4"},
    {'M',   5,   4,  89, "SC-8850 Map Var #05", "Sine Pad"},
    {'M',   5,   4,  91, "SC-8850 Map Var #05", "Cold Space"},
    {'M',   5,   4,  92, "SC-8850 Map Var #05", "Bottle Stack"},
    {'M',   5,   4,  93, "SC-8850 Map Var #05", "Metal Pad 2"},
    {'M',   5,   4,  96, "SC-8850 Map Var #05", "Saw Impulse"},
    {'M',   5,   4,  97, "SC-8850 Map Var #05", "HistoryWave"},
    {'M',   5,   4,  98, "SC-8850 Map Var #05", "GlockenChime"},
    {'M',   5,   4,  99, "SC-8850 Map Var #05", "Nylon+Rhodes"},
    {'M',   5,   4, 100, "SC-8850 Map Var #05", "D-50 Retour"},
    {'M',   5,   4, 101, "SC-8850 Map Var #05", "LFO Pad"},
    {'M',   5,   4, 102, "SC-8850 Map Var #05", "Reso Panner"},
    {'M',   5,   4, 103, "SC-8850 Map Var #05", "Etherality"},
    {'M',   5,   4, 104, "SC-8850 Map Var #05", "Sitar 4"},
    {'M',   5,   4, 120, "SC-8850 Map Var #05", "Bass Slide"},
    {'M',   5,   4, 121, "SC-8850 Map Var #05", "Fl.Breath 3"},
    {'M',   5,   4, 122, "SC-8850 Map Var #05", "Bubble"},
    {'M',   5,   4, 123, "SC-8850 Map Var #05", "Growl"},
    {'M',   5,   4, 124, "SC-8850 Map Var #05", "Wind Chimes"},
    {'M',   5,   4, 125, "SC-8850 Map Var #05", "Siren"},
    {'M',   5,   4, 126, "SC-8850 Map Var #05", "Footsteps"},
    {'M',   5,   4, 127, "SC-8850 Map Var #05", "Big Shot"},
    {'M',   6,   0,  16, "SC-8850 Var #06", "Ful Organ 5"},
    {'M',   6,   0,  17, "SC-8850 Var #06", "Jazz Organ 5"},
    {'M',   6,   0,  33, "SC-8850 Var #06", "AttackFinger"},
    {'M',   6,   0,  38, "SC-8850 Var #06", "JPMG Bass"},
    {'M',   6,   0,  39, "SC-8850 Var #06", "MG Oct Bass2"},
    {'M',   6,   0,  48, "SC-8850 Var #06", "Quad Strings"},
    {'M',   6,   0,  51, "SC-8850 Var #06", "DistStrings"},
    {'M',   6,   0,  53, "SC-8850 Var #06", "Jazz Scat"},
    {'M',   6,   0,  63, "SC-8850 Var #06", "DeepSynBrass"},
    {'M',   6,   0,  80, "SC-8850 Var #06", "LM Square"},
    {'M',   6,   0,  81, "SC-8850 Var #06", "GR-300"},
    {'M',   6,   0,  83, "SC-8850 Var #06", "Oct.UnisonLd"},
    {'M',   6,   0,  84, "SC-8850 Var #06", "GR & Pulse"},
    {'M',   6,   0,  86, "SC-8850 Var #06", "JP8000 5thFB"},
    {'M',   6,   0,  87, "SC-8850 Var #06", "SH-5 Bs.Lead"},
    {'M',   6,   0,  88, "SC-8850 Var #06", "After D !"},
    {'M',   6,   0,  89, "SC-8850 Var #06", "OB Soft Pad2"},
    {'M',   6,   0,  91, "SC-8850 Var #06", "Noise Peaker"},
    {'M',   6,   0,  96, "SC-8850 Var #06", "Strange Str."},
    {'M',   6,   0,  98, "SC-8850 Var #06", "Clear Bells"},
    {'M',   6,   0,  99, "SC-8850 Var #06", "Ambient Pad"},
    {'M',   6,   0, 100, "SC-8850 Var #06", "SouthernWind"},
    {'M',   6,   0, 101, "SC-8850 Var #06", "Random Str"},
    {'M',   6,   0, 102, "SC-8850 Var #06", "Water Piano"},
    {'M',   6,   0, 103, "SC-8850 Var #06", "Mystic Pad"},
    {'M',   6,   0, 120, "SC-8850 Var #06", "Pick Scrape"},
    {'M',   6,   0, 121, "SC-8850 Var #06", "Vox Breath 1"},
    {'M',   6,   0, 122, "SC-8850 Var #06", "Wind 2"},
    {'M',   6,   0, 123, "SC-8850 Var #06", "Growl 2"},
    {'M',   6,   0, 125, "SC-8850 Var #06", "Train"},
    {'M',   6,   0, 126, "SC-8850 Var #06", "Applause 2"},
    {'M',   6,   0, 127, "SC-8850 Var #06", "Explosion 2"},
    {'M',   6,   1, 125, "SC-55 Map Var #06", "Train"},
    {'M',   6,   2,  80, "SC-88 Map Var #06", "LM Square"},
    {'M',   6,   2,  81, "SC-88 Map Var #06", "GR-300"},
    {'M',   6,   2,  98, "SC-88 Map Var #06", "Clear Bells"},
    {'M',   6,   2,  99, "SC-88 Map Var #06", "Ambient Pad"},
    {'M',   6,   2, 102, "SC-88 Map Var #06", "Water Piano"},
    {'M',   6,   2, 120, "SC-88 Map Var #06", "Pick Scrape"},
    {'M',   6,   2, 125, "SC-88 Map Var #06", "Train"},
    {'M',   6,   2, 126, "SC-88 Map Var #06", "Applause 2"},
    {'M',   6,   3,  38, "SC-88Pro Map Var #06", "JPMG Bass"},
    {'M',   6,   3,  39, "SC-88Pro Map Var #06", "MG Oct Bass2"},
    {'M',   6,   3,  80, "SC-88Pro Map Var #06", "LM Square"},
    {'M',   6,   3,  81, "SC-88Pro Map Var #06", "GR-300"},
    {'M',   6,   3,  87, "SC-88Pro Map Var #06", "SH-5 Bs.Lead"},
    {'M',   6,   3,  98, "SC-88Pro Map Var #06", "Clear Bells"},
    {'M',   6,   3,  99, "SC-88Pro Map Var #06", "Ambient Pad"},
    {'M',   6,   3, 101, "SC-88Pro Map Var #06", "Random Str"},
    {'M',   6,   3, 102, "SC-88Pro Map Var #06", "Water Piano"},
    {'M',   6,   3, 120, "SC-88Pro Map Var #06", "Pick Scrape"},
    {'M',   6,   3, 122, "SC-88Pro Map Var #06", "Wind 2"},
    {'M',   6,   3, 125, "SC-88Pro Map Var #06", "Train"},
    {'M',   6,   3, 126, "SC-88Pro Map Var #06", "Applause 2"},
    {'M',   6,   4,  16, "SC-8850 Map Var #06", "Ful Organ 5"},
    {'M',   6,   4,  17, "SC-8850 Map Var #06", "Jazz Organ 5"},
    {'M',   6,   4,  33, "SC-8850 Map Var #06", "AttackFinger"},
    {'M',   6,   4,  38, "SC-8850 Map Var #06", "JPMG Bass"},
    {'M',   6,   4,  39, "SC-8850 Map Var #06", "MG Oct Bass2"},
    {'M',   6,   4,  48, "SC-8850 Map Var #06", "Quad Strings"},
    {'M',   6,   4,  51, "SC-8850 Map Var #06", "DistStrings"},
    {'M',   6,   4,  53, "SC-8850 Map Var #06", "Jazz Scat"},
    {'M',   6,   4,  63, "SC-8850 Map Var #06", "DeepSynBrass"},
    {'M',   6,   4,  80, "SC-8850 Map Var #06", "LM Square"},
    {'M',   6,   4,  81, "SC-8850 Map Var #06", "GR-300"},
    {'M',   6,   4,  83, "SC-8850 Map Var #06", "Oct.UnisonLd"},
    {'M',   6,   4,  84, "SC-8850 Map Var #06", "GR & Pulse"},
    {'M',   6,   4,  86, "SC-8850 Map Var #06", "JP8000 5thFB"},
    {'M',   6,   4,  87, "SC-8850 Map Var #06", "SH-5 Bs.Lead"},
    {'M',   6,   4,  88, "SC-8850 Map Var #06", "After D !"},
    {'M',   6,   4,  89, "SC-8850 Map Var #06", "OB Soft Pad2"},
    {'M',   6,   4,  91, "SC-8850 Map Var #06", "Noise Peaker"},
    {'M',   6,   4,  96, "SC-8850 Map Var #06", "Strange Str."},
    {'M',   6,   4,  98, "SC-8850 Map Var #06", "Clear Bells"},
    {'M',   6,   4,  99, "SC-8850 Map Var #06", "Ambient Pad"},
    {'M',   6,   4, 100, "SC-8850 Map Var #06", "SouthernWind"},
    {'M',   6,   4, 101, "SC-8850 Map Var #06", "Random Str"},
    {'M',   6,   4, 102, "SC-8850 Map Var #06", "Water Piano"},
    {'M',   6,   4, 103, "SC-8850 Map Var #06", "Mystic Pad"},
    {'M',   6,   4, 120, "SC-8850 Map Var #06", "Pick Scrape"},
    {'M',   6,   4, 121, "SC-8850 Map Var #06", "Vox Breath 1"},
    {'M',   6,   4, 122, "SC-8850 Map Var #06", "Wind 2"},
    {'M',   6,   4, 123, "SC-8850 Map Var #06", "Growl 2"},
    {'M',   6,   4, 125, "SC-8850 Map Var #06", "Train"},
    {'M',   6,   4, 126, "SC-8850 Map Var #06", "Applause 2"},
    {'M',   6,   4, 127, "SC-8850 Map Var #06", "Explosion 2"},
    {'M',   7,   0,  16, "SC-8850 Var #07", "Ful Organ 6"},
    {'M',   7,   0,  17, "SC-8850 Var #07", "Jazz Organ 6"},
    {'M',   7,   0,  33, "SC-8850 Var #07", "Finger Slap"},
    {'M',   7,   0,  39, "SC-8850 Var #07", "MG Blip Bs:"},
    {'M',   7,   0,  48, "SC-8850 Var #07", "Mild Strings"},
    {'M',   7,   0,  51, "SC-8850 Var #07", "JUNOFullStr."},
    {'M',   7,   0,  80, "SC-8850 Var #07", "JP8000 TWM"},
    {'M',   7,   0,  81, "SC-8850 Var #07", "LA Saw"},
    {'M',   7,   0,  83, "SC-8850 Var #07", "Unison SawLd"},
    {'M',   7,   0,  87, "SC-8850 Var #07", "Delayed Lead"},
    {'M',   7,   0,  88, "SC-8850 Var #07", "260HarmPad"},
    {'M',   7,   0,  91, "SC-8850 Var #07", "Bamboo Hit"},
    {'M',   7,   0,  96, "SC-8850 Var #07", "FastFWD Pad"},
    {'M',   7,   0,  98, "SC-8850 Var #07", "ChristmasBel"},
    {'M',   7,   0,  99, "SC-8850 Var #07", "Invisible"},
    {'M',   7,   0, 100, "SC-8850 Var #07", "SymbolicBell"},
    {'M',   7,   0, 101, "SC-8850 Var #07", "Random Pad"},
    {'M',   7,   0, 102, "SC-8850 Var #07", "Echo SynBass"},
    {'M',   7,   0, 121, "SC-8850 Var #07", "Vox Breath 2"},
    {'M',   7,   0, 122, "SC-8850 Var #07", "Cricket"},
    {'M',   7,   0, 123, "SC-8850 Var #07", "Fancy Animal"},
    {'M',   7,   0, 124, "SC-8850 Var #07", "Scratch 2"},
    {'M',   7,   0, 125, "SC-8850 Var #07", "Jetplane"},
    {'M',   7,   0, 126, "SC-8850 Var #07", "Small Club"},
    {'M',   7,   1, 125, "SC-55 Map Var #07", "Jetplane"},
    {'M',   7,   2,  81, "SC-88 Map Var #07", "LA Saw"},
    {'M',   7,   2,  98, "SC-88 Map Var #07", "X'mas Bel"},
    {'M',   7,   2, 124, "SC-88 Map Var #07", "Scratch 2"},
    {'M',   7,   2, 125, "SC-88 Map Var #07", "Jetplane"},
    {'M',   7,   3,  39, "SC-88Pro Map Var #07", "MG Blip Bs:"},
    {'M',   7,   3,  81, "SC-88Pro Map Var #07", "LA Saw"},
    {'M',   7,   3,  98, "SC-88Pro Map Var #07", "ChristmasBel"},
    {'M',   7,   3,  99, "SC-88Pro Map Var #07", "Invisible"},
    {'M',   7,   3, 101, "SC-88Pro Map Var #07", "Random Pad"},
    {'M',   7,   3, 124, "SC-88Pro Map Var #07", "Scratch 2"},
    {'M',   7,   3, 125, "SC-88Pro Map Var #07", "Jetplane"},
    {'M',   7,   3, 126, "SC-88Pro Map Var #07", "Small Club"},
    {'M',   7,   4,  16, "SC-8850 Map Var #07", "Ful Organ 6"},
    {'M',   7,   4,  17, "SC-8850 Map Var #07", "Jazz Organ 6"},
    {'M',   7,   4,  33, "SC-8850 Map Var #07", "Finger Slap"},
    {'M',   7,   4,  39, "SC-8850 Map Var #07", "MG Blip Bs:"},
    {'M',   7,   4,  48, "SC-8850 Map Var #07", "Mild Strings"},
    {'M',   7,   4,  51, "SC-8850 Map Var #07", "JUNOFullStr."},
    {'M',   7,   4,  80, "SC-8850 Map Var #07", "JP8000 TWM"},
    {'M',   7,   4,  81, "SC-8850 Map Var #07", "LA Saw"},
    {'M',   7,   4,  83, "SC-8850 Map Var #07", "Unison SawLd"},
    {'M',   7,   4,  87, "SC-8850 Map Var #07", "Delayed Lead"},
    {'M',   7,   4,  88, "SC-8850 Map Var #07", "260HarmPad"},
    {'M',   7,   4,  91, "SC-8850 Map Var #07", "Bamboo Hit"},
    {'M',   7,   4,  96, "SC-8850 Map Var #07", "FastFWD Pad"},
    {'M',   7,   4,  98, "SC-8850 Map Var #07", "ChristmasBel"},
    {'M',   7,   4,  99, "SC-8850 Map Var #07", "Invisible"},
    {'M',   7,   4, 100, "SC-8850 Map Var #07", "SymbolicBell"},
    {'M',   7,   4, 101, "SC-8850 Map Var #07", "Random Pad"},
    {'M',   7,   4, 102, "SC-8850 Map Var #07", "Echo SynBass"},
    {'M',   7,   4, 121, "SC-8850 Map Var #07", "Vox Breath 2"},
    {'M',   7,   4, 122, "SC-8850 Map Var #07", "Cricket"},
    {'M',   7,   4, 123, "SC-8850 Map Var #07", "Fancy Animal"},
    {'M',   7,   4, 124, "SC-8850 Map Var #07", "Scratch 2"},
    {'M',   7,   4, 125, "SC-8850 Map Var #07", "Jetplane"},
    {'M',   7,   4, 126, "SC-8850 Map Var #07", "Small Club"},
    {'M',   8,   0,   0, "SC-8850 Var #08", "Upright P w"},
    {'M',   8,   0,   1, "SC-8850 Var #08", "Pop Piano w"},
    {'M',   8,   0,   2, "SC-8850 Var #08", "Piano 3w"},
    {'M',   8,   0,   3, "SC-8850 Var #08", "Honky-tonk 2"},
    {'M',   8,   0,   4, "SC-8850 Var #08", "St.Soft EP"},
    {'M',   8,   0,   5, "SC-8850 Var #08", "Detuned EP 2"},
    {'M',   8,   0,   6, "SC-8850 Var #08", "Coupled Hps."},
    {'M',   8,   0,   7, "SC-8850 Var #08", "Comp Clav."},
    {'M',   8,   0,  10, "SC-8850 Var #08", "St.Music Box"},
    {'M',   8,   0,  11, "SC-8850 Var #08", "Vibraphone w"},
    {'M',   8,   0,  12, "SC-8850 Var #08", "Marimba w"},
    {'M',   8,   0,  13, "SC-8850 Var #08", "Xylophone w"},
    {'M',   8,   0,  14, "SC-8850 Var #08", "Church Bell"},
    {'M',   8,   0,  15, "SC-8850 Var #08", "Cimbalom"},
    {'M',   8,   0,  16, "SC-8850 Var #08", "Trem. Organ"},
    {'M',   8,   0,  17, "SC-8850 Var #08", "Chorus Or.2"},
    {'M',   8,   0,  18, "SC-8850 Var #08", "Rotary Org."},
    {'M',   8,   0,  19, "SC-8850 Var #08", "Church Org.2"},
    {'M',   8,   0,  20, "SC-8850 Var #08", "Wind Organ"},
    {'M',   8,   0,  21, "SC-8850 Var #08", "Accordion It"},
    {'M',   8,   0,  22, "SC-8850 Var #08", "B.Harp Basic"},
    {'M',   8,   0,  23, "SC-8850 Var #08", "Bandoneon 2"},
    {'M',   8,   0,  24, "SC-8850 Var #08", "Ukulele"},
    {'M',   8,   0,  25, "SC-8850 Var #08", "12-str.Gt"},
    {'M',   8,   0,  26, "SC-8850 Var #08", "Pedal Steel"},
    {'M',   8,   0,  27, "SC-8850 Var #08", "Chorus Gt."},
    {'M',   8,   0,  28, "SC-8850 Var #08", "Funk Pop"},
    {'M',   8,   0,  29, "SC-8850 Var #08", "LP OverDrvGt"},
    {'M',   8,   0,  30, "SC-8850 Var #08", "Feedback Gt."},
    {'M',   8,   0,  31, "SC-8850 Var #08", "Gt. Feedback"},
    {'M',   8,   0,  32, "SC-8850 Var #08", "Wild A.Bass"},
    {'M',   8,   0,  33, "SC-8850 Var #08", "ChorusJazzBs"},
    {'M',   8,   0,  34, "SC-8850 Var #08", "Muted PickBs"},
    {'M',   8,   0,  35, "SC-8850 Var #08", "Wood+FlessBs"},
    {'M',   8,   0,  36, "SC-8850 Var #08", "Reso Slap"},
    {'M',   8,   0,  37, "SC-8850 Var #08", "FM Slap"},
    {'M',   8,   0,  38, "SC-8850 Var #08", "Acid Bass"},
    {'M',   8,   0,  39, "SC-8850 Var #08", "Beef FM Bass"},
    {'M',   8,   0,  40, "SC-8850 Var #08", "Slow Violin"},
    {'M',   8,   0,  44, "SC-8850 Var #08", "Slow Tremolo"},
    {'M',   8,   0,  45, "SC-8850 Var #08", "Solo Pizz."},
    {'M',   8,   0,  46, "SC-8850 Var #08", "Uillean Harp"},
    {'M',   8,   0,  48, "SC-8850 Var #08", "Orchestra"},
    {'M',   8,   0,  49, "SC-8850 Var #08", "Legato Str."},
    {'M',   8,   0,  50, "SC-8850 Var #08", "Syn.Strings3"},
    {'M',   8,   0,  51, "SC-8850 Var #08", "Air Strings"},
    {'M',   8,   0,  52, "SC-8850 Var #08", "St.ChoirAahs"},
    {'M',   8,   0,  53, "SC-8850 Var #08", "Voice Dahs"},
    {'M',   8,   0,  54, "SC-8850 Var #08", "Syn.Voice"},
    {'M',   8,   0,  55, "SC-8850 Var #08", "Impact Hit"},
    {'M',   8,   0,  56, "SC-8850 Var #08", "Flugel Horn"},
    {'M',   8,   0,  57, "SC-8850 Var #08", "Bs. Trombone"},
    {'M',   8,   0,  58, "SC-8850 Var #08", "Tuba + Horn"},
    {'M',   8,   0,  59, "SC-8850 Var #08", "Muted Horns"},
    {'M',   8,   0,  60, "SC-8850 Var #08", "F.Hrn Slow:"},
    {'M',   8,   0,  61, "SC-8850 Var #08", "Brass 2"},
    {'M',   8,   0,  62, "SC-8850 Var #08", "Pro Brass"},
    {'M',   8,   0,  63, "SC-8850 Var #08", "SynBrass sfz"},
    {'M',   8,   0,  64, "SC-8850 Var #08", "Soprano Exp."},
    {'M',   8,   0,  65, "SC-8850 Var #08", "AltoSax Exp."},
    {'M',   8,   0,  66, "SC-8850 Var #08", "BreathyTn.:"},
    {'M',   8,   0,  67, "SC-8850 Var #08", "Bari & Tenor"},
    {'M',   8,   0,  68, "SC-8850 Var #08", "Oboe Exp."},
    {'M',   8,   0,  71, "SC-8850 Var #08", "Bs Clarinet"},
    {'M',   8,   0,  72, "SC-8850 Var #08", "Nay"},
    {'M',   8,   0,  73, "SC-8850 Var #08", "Flute + Vln"},
    {'M',   8,   0,  75, "SC-8850 Var #08", "Kawala"},
    {'M',   8,   0,  80, "SC-8850 Var #08", "2600 Sine"},
    {'M',   8,   0,  81, "SC-8850 Var #08", "Doctor Solo"},
    {'M',   8,   0,  82, "SC-8850 Var #08", "LM Pure Lead"},
    {'M',   8,   0,  83, "SC-8850 Var #08", "Mad Lead"},
    {'M',   8,   0,  84, "SC-8850 Var #08", "Dist.Lead"},
    {'M',   8,   0,  85, "SC-8850 Var #08", "Vox Lead"},
    {'M',   8,   0,  86, "SC-8850 Var #08", "4th Lead"},
    {'M',   8,   0,  89, "SC-8850 Var #08", "Octave Pad"},
    {'M',   8,   0,  90, "SC-8850 Var #08", "Power Stack"},
    {'M',   8,   0,  91, "SC-8850 Var #08", "Cosmic Voice"},
    {'M',   8,   0,  94, "SC-8850 Var #08", "Horror Pad"},
    {'M',   8,   0,  95, "SC-8850 Var #08", "Converge"},
    {'M',   8,   0,  96, "SC-8850 Var #08", "Clavi Pad"},
    {'M',   8,   0,  97, "SC-8850 Var #08", "Rave"},
    {'M',   8,   0,  98, "SC-8850 Var #08", "Vibra Bells"},
    {'M',   8,   0,  99, "SC-8850 Var #08", "Pulsey Key"},
    {'M',   8,   0, 100, "SC-8850 Var #08", "Org Bell"},
    {'M',   8,   0, 101, "SC-8850 Var #08", "LowBirds Pad"},
    {'M',   8,   0, 102, "SC-8850 Var #08", "Pan Sequence"},
    {'M',   8,   0, 103, "SC-8850 Var #08", "Dream Pad"},
    {'M',   8,   0, 104, "SC-8850 Var #08", "Tambra"},
    {'M',   8,   0, 105, "SC-8850 Var #08", "Rabab"},
    {'M',   8,   0, 106, "SC-8850 Var #08", "Syn Shamisen"},
    {'M',   8,   0, 107, "SC-8850 Var #08", "Taisho Koto"},
    {'M',   8,   0, 108, "SC-8850 Var #08", "Sanza"},
    {'M',   8,   0, 109, "SC-8850 Var #08", "Didgeridoo"},
    {'M',   8,   0, 110, "SC-8850 Var #08", "Er Hu"},
    {'M',   8,   0, 111, "SC-8850 Var #08", "Pungi"},
    {'M',   8,   0, 112, "SC-8850 Var #08", "Bonang"},
    {'M',   8,   0, 113, "SC-8850 Var #08", "Atarigane"},
    {'M',   8,   0, 115, "SC-8850 Var #08", "Castanets"},
    {'M',   8,   0, 116, "SC-8850 Var #08", "Concert BD"},
    {'M',   8,   0, 117, "SC-8850 Var #08", "Melo. Tom 2"},
    {'M',   8,   0, 118, "SC-8850 Var #08", "808 Tom"},
    {'M',   8,   0, 119, "SC-8850 Var #08", "Rev.Snare 1"},
    {'M',   8,   0, 120, "SC-8850 Var #08", "Gt. FX Menu"},
    {'M',   8,   0, 121, "SC-8850 Var #08", "Trombone Nz"},
    {'M',   8,   0, 123, "SC-8850 Var #08", "Seal"},
    {'M',   8,   0, 124, "SC-8850 Var #08", "ScratchKey"},
    {'M',   8,   0, 125, "SC-8850 Var #08", "Starship"},
    {'M',   8,   0, 126, "SC-8850 Var #08", "ApplauseWave"},
    {'M',   8,   1,   0, "SC-55 Map Var #08", "Piano 1w"},
    {'M',   8,   1,   1, "SC-55 Map Var #08", "Piano 2w"},
    {'M',   8,   1,   2, "SC-55 Map Var #08", "Piano 3w"},
    {'M',   8,   1,   3, "SC-55 Map Var #08", "HonkyTonk w"},
    {'M',   8,   1,   4, "SC-55 Map Var #08", "Detuned EP1"},
    {'M',   8,   1,   5, "SC-55 Map Var #08", "Detuned EP2"},
    {'M',   8,   1,   6, "SC-55 Map Var #08", "Coupled Hps"},
    {'M',   8,   1,  11, "SC-55 Map Var #08", "Vib.w"},
    {'M',   8,   1,  12, "SC-55 Map Var #08", "Marimba w"},
    {'M',   8,   1,  14, "SC-55 Map Var #08", "Church Bell"},
    {'M',   8,   1,  16, "SC-55 Map Var #08", "Detuned Or1"},
    {'M',   8,   1,  17, "SC-55 Map Var #08", "Detuned Or2"},
    {'M',   8,   1,  19, "SC-55 Map Var #08", "Church Org2"},
    {'M',   8,   1,  21, "SC-55 Map Var #08", "Accordion I"},
    {'M',   8,   1,  24, "SC-55 Map Var #08", "Ukulele"},
    {'M',   8,   1,  25, "SC-55 Map Var #08", "12-str.Gt"},
    {'M',   8,   1,  26, "SC-55 Map Var #08", "Hawaiian Gt"},
    {'M',   8,   1,  27, "SC-55 Map Var #08", "Chorus Gt."},
    {'M',   8,   1,  28, "SC-55 Map Var #08", "Funk Gt."},
    {'M',   8,   1,  30, "SC-55 Map Var #08", "Feedback Gt"},
    {'M',   8,   1,  31, "SC-55 Map Var #08", "Gt.Feedback"},
    {'M',   8,   1,  38, "SC-55 Map Var #08", "Syn.Bass 3"},
    {'M',   8,   1,  39, "SC-55 Map Var #08", "Syn.Bass 4"},
    {'M',   8,   1,  40, "SC-55 Map Var #08", "Slow Violin"},
    {'M',   8,   1,  48, "SC-55 Map Var #08", "Orchestra"},
    {'M',   8,   1,  50, "SC-55 Map Var #08", "SynStrings3"},
    {'M',   8,   1,  61, "SC-55 Map Var #08", "Brass 2"},
    {'M',   8,   1,  62, "SC-55 Map Var #08", "Syn.Brass3"},
    {'M',   8,   1,  63, "SC-55 Map Var #08", "Syn.Brass4"},
    {'M',   8,   1,  80, "SC-55 Map Var #08", "Sine Wave"},
    {'M',   8,   1,  81, "SC-55 Map Var #08", "Doctor Solo"},
    {'M',   8,   1, 107, "SC-55 Map Var #08", "Taisho Koto"},
    {'M',   8,   1, 115, "SC-55 Map Var #08", "Castanets"},
    {'M',   8,   1, 116, "SC-55 Map Var #08", "Concert BD"},
    {'M',   8,   1, 117, "SC-55 Map Var #08", "Melo. Tom 2"},
    {'M',   8,   1, 118, "SC-55 Map Var #08", "808 Tom"},
    {'M',   8,   1, 125, "SC-55 Map Var #08", "Starship"},
    {'M',   8,   2,   0, "SC-88 Map Var #08", "Piano 1w"},
    {'M',   8,   2,   1, "SC-88 Map Var #08", "Piano 2w"},
    {'M',   8,   2,   2, "SC-88 Map Var #08", "Piano 3w"},
    {'M',   8,   2,   3, "SC-88 Map Var #08", "Old Upright"},
    {'M',   8,   2,   4, "SC-88 Map Var #08", "St.Soft EP"},
    {'M',   8,   2,   5, "SC-88 Map Var #08", "Detuned EP2"},
    {'M',   8,   2,   6, "SC-88 Map Var #08", "Coupled Hps"},
    {'M',   8,   2,  11, "SC-88 Map Var #08", "Vib.w"},
    {'M',   8,   2,  12, "SC-88 Map Var #08", "Marimba w"},
    {'M',   8,   2,  14, "SC-88 Map Var #08", "Church Bell"},
    {'M',   8,   2,  15, "SC-88 Map Var #08", "Cimbalom"},
    {'M',   8,   2,  16, "SC-88 Map Var #08", "DetunedOr.1"},
    {'M',   8,   2,  17, "SC-88 Map Var #08", "DetunedOr.2"},
    {'M',   8,   2,  18, "SC-88 Map Var #08", "Rotary Org."},
    {'M',   8,   2,  19, "SC-88 Map Var #08", "ChurchOrg.2"},
    {'M',   8,   2,  21, "SC-88 Map Var #08", "AccordionIt"},
    {'M',   8,   2,  24, "SC-88 Map Var #08", "Ukulele"},
    {'M',   8,   2,  25, "SC-88 Map Var #08", "12-str.Gt"},
    {'M',   8,   2,  26, "SC-88 Map Var #08", "Pedal Steel"},
    {'M',   8,   2,  27, "SC-88 Map Var #08", "Chorus Gt."},
    {'M',   8,   2,  28, "SC-88 Map Var #08", "Funk Pop"},
    {'M',   8,   2,  30, "SC-88 Map Var #08", "FeedbackGt."},
    {'M',   8,   2,  31, "SC-88 Map Var #08", "Gt.Feedback"},
    {'M',   8,   2,  34, "SC-88 Map Var #08", "MutePickBs."},
    {'M',   8,   2,  36, "SC-88 Map Var #08", "Reso Slap"},
    {'M',   8,   2,  38, "SC-88 Map Var #08", "Acid Bass"},
    {'M',   8,   2,  39, "SC-88 Map Var #08", "Beef FMBass"},
    {'M',   8,   2,  40, "SC-88 Map Var #08", "Slow Violin"},
    {'M',   8,   2,  44, "SC-88 Map Var #08", "SlowTremolo"},
    {'M',   8,   2,  48, "SC-88 Map Var #08", "Orchestra"},
    {'M',   8,   2,  49, "SC-88 Map Var #08", "Legato Str."},
    {'M',   8,   2,  50, "SC-88 Map Var #08", "SynStrings3"},
    {'M',   8,   2,  52, "SC-88 Map Var #08", "St.Choir"},
    {'M',   8,   2,  54, "SC-88 Map Var #08", "Syn.Voice"},
    {'M',   8,   2,  55, "SC-88 Map Var #08", "Impact Hit"},
    {'M',   8,   2,  56, "SC-88 Map Var #08", "Flugel Horn"},
    {'M',   8,   2,  60, "SC-88 Map Var #08", "Fr.HornSolo"},
    {'M',   8,   2,  61, "SC-88 Map Var #08", "Brass 2"},
    {'M',   8,   2,  62, "SC-88 Map Var #08", "Syn.Brass 3"},
    {'M',   8,   2,  63, "SC-88 Map Var #08", "Syn.Brass 4"},
    {'M',   8,   2,  65, "SC-88 Map Var #08", "Hyper Alto"},
    {'M',   8,   2,  66, "SC-88 Map Var #08", "BreathyTnr."},
    {'M',   8,   2,  71, "SC-88 Map Var #08", "Bs Clarinet"},
    {'M',   8,   2,  75, "SC-88 Map Var #08", "Kawala"},
    {'M',   8,   2,  80, "SC-88 Map Var #08", "Sine Wave"},
    {'M',   8,   2,  81, "SC-88 Map Var #08", "Doctor Solo"},
    {'M',   8,   2,  84, "SC-88 Map Var #08", "Dist.Lead"},
    {'M',   8,   2,  95, "SC-88 Map Var #08", "Converge"},
    {'M',   8,   2,  96, "SC-88 Map Var #08", "Clavi Pad"},
    {'M',   8,   2,  97, "SC-88 Map Var #08", "Rave"},
    {'M',   8,   2,  98, "SC-88 Map Var #08", "Vibra Bells"},
    {'M',   8,   2, 104, "SC-88 Map Var #08", "Tambra"},
    {'M',   8,   2, 105, "SC-88 Map Var #08", "Rabab"},
    {'M',   8,   2, 107, "SC-88 Map Var #08", "Taisho Koto"},
    {'M',   8,   2, 111, "SC-88 Map Var #08", "Pungi"},
    {'M',   8,   2, 112, "SC-88 Map Var #08", "Bonang"},
    {'M',   8,   2, 113, "SC-88 Map Var #08", "Atarigane"},
    {'M',   8,   2, 115, "SC-88 Map Var #08", "Castanets"},
    {'M',   8,   2, 116, "SC-88 Map Var #08", "Concert BD"},
    {'M',   8,   2, 117, "SC-88 Map Var #08", "Melo. Tom 2"},
    {'M',   8,   2, 118, "SC-88 Map Var #08", "808 Tom"},
    {'M',   8,   2, 119, "SC-88 Map Var #08", "Rev.Snare 1"},
    {'M',   8,   2, 125, "SC-88 Map Var #08", "Starship"},
    {'M',   8,   3,   0, "SC-88Pro Map Var #08", "Piano 1w"},
    {'M',   8,   3,   1, "SC-88Pro Map Var #08", "Piano 2w"},
    {'M',   8,   3,   2, "SC-88Pro Map Var #08", "Piano 3w"},
    {'M',   8,   3,   3, "SC-88Pro Map Var #08", "Honky-tonk 2"},
    {'M',   8,   3,   4, "SC-88Pro Map Var #08", "St.Soft EP"},
    {'M',   8,   3,   5, "SC-88Pro Map Var #08", "Detuned EP 2"},
    {'M',   8,   3,   6, "SC-88Pro Map Var #08", "Coupled Hps."},
    {'M',   8,   3,   7, "SC-88Pro Map Var #08", "Comp Clav."},
    {'M',   8,   3,  11, "SC-88Pro Map Var #08", "Vibraphone w"},
    {'M',   8,   3,  12, "SC-88Pro Map Var #08", "Marimba w"},
    {'M',   8,   3,  14, "SC-88Pro Map Var #08", "Church Bell"},
    {'M',   8,   3,  15, "SC-88Pro Map Var #08", "Cimbalom"},
    {'M',   8,   3,  16, "SC-88Pro Map Var #08", "Trem. Organ"},
    {'M',   8,   3,  17, "SC-88Pro Map Var #08", "Chorus Or.2"},
    {'M',   8,   3,  18, "SC-88Pro Map Var #08", "Rotary Org."},
    {'M',   8,   3,  19, "SC-88Pro Map Var #08", "Church Org.2"},
    {'M',   8,   3,  20, "SC-88Pro Map Var #08", "Wind Organ"},
    {'M',   8,   3,  21, "SC-88Pro Map Var #08", "Accordion It"},
    {'M',   8,   3,  23, "SC-88Pro Map Var #08", "Bandoneon 2"},
    {'M',   8,   3,  24, "SC-88Pro Map Var #08", "Ukulele"},
    {'M',   8,   3,  25, "SC-88Pro Map Var #08", "12-str.Gt"},
    {'M',   8,   3,  26, "SC-88Pro Map Var #08", "Pedal Steel"},
    {'M',   8,   3,  27, "SC-88Pro Map Var #08", "Chorus Gt."},
    {'M',   8,   3,  28, "SC-88Pro Map Var #08", "Funk Pop"},
    {'M',   8,   3,  29, "SC-88Pro Map Var #08", "LP OverDrvGt"},
    {'M',   8,   3,  30, "SC-88Pro Map Var #08", "Feedback Gt."},
    {'M',   8,   3,  31, "SC-88Pro Map Var #08", "Gt. Feedback"},
    {'M',   8,   3,  32, "SC-88Pro Map Var #08", "Wild A.Bass"},
    {'M',   8,   3,  33, "SC-88Pro Map Var #08", "ChorusJazzBs"},
    {'M',   8,   3,  34, "SC-88Pro Map Var #08", "Muted PickBs"},
    {'M',   8,   3,  35, "SC-88Pro Map Var #08", "Wood+FlessBs"},
    {'M',   8,   3,  36, "SC-88Pro Map Var #08", "Reso Slap"},
    {'M',   8,   3,  37, "SC-88Pro Map Var #08", "FM Slap"},
    {'M',   8,   3,  38, "SC-88Pro Map Var #08", "Acid Bass"},
    {'M',   8,   3,  39, "SC-88Pro Map Var #08", "Beef FM Bass"},
    {'M',   8,   3,  40, "SC-88Pro Map Var #08", "Slow Violin"},
    {'M',   8,   3,  44, "SC-88Pro Map Var #08", "Slow Tremolo"},
    {'M',   8,   3,  45, "SC-88Pro Map Var #08", "Solo Pizz."},
    {'M',   8,   3,  48, "SC-88Pro Map Var #08", "Orchestra"},
    {'M',   8,   3,  49, "SC-88Pro Map Var #08", "Legato Str."},
    {'M',   8,   3,  50, "SC-88Pro Map Var #08", "Syn.Strings3"},
    {'M',   8,   3,  51, "SC-88Pro Map Var #08", "Air Strings"},
    {'M',   8,   3,  52, "SC-88Pro Map Var #08", "St.ChoirAahs"},
    {'M',   8,   3,  53, "SC-88Pro Map Var #08", "Voice Dahs"},
    {'M',   8,   3,  54, "SC-88Pro Map Var #08", "Syn.Voice"},
    {'M',   8,   3,  55, "SC-88Pro Map Var #08", "Impact Hit"},
    {'M',   8,   3,  56, "SC-88Pro Map Var #08", "Flugel Horn"},
    {'M',   8,   3,  57, "SC-88Pro Map Var #08", "Bs. Trombone"},
    {'M',   8,   3,  59, "SC-88Pro Map Var #08", "Muted Horns"},
    {'M',   8,   3,  60, "SC-88Pro Map Var #08", "F.Hrn Slow:"},
    {'M',   8,   3,  61, "SC-88Pro Map Var #08", "Brass 2"},
    {'M',   8,   3,  62, "SC-88Pro Map Var #08", "Pro Brass"},
    {'M',   8,   3,  63, "SC-88Pro Map Var #08", "SynBrass sfz"},
    {'M',   8,   3,  64, "SC-88Pro Map Var #08", "Soprano Exp."},
    {'M',   8,   3,  65, "SC-88Pro Map Var #08", "AltoSax Exp."},
    {'M',   8,   3,  66, "SC-88Pro Map Var #08", "BreathyTn.:"},
    {'M',   8,   3,  68, "SC-88Pro Map Var #08", "Oboe Exp."},
    {'M',   8,   3,  71, "SC-88Pro Map Var #08", "Bs Clarinet"},
    {'M',   8,   3,  72, "SC-88Pro Map Var #08", "Nay"},
    {'M',   8,   3,  73, "SC-88Pro Map Var #08", "Flute + Vln"},
    {'M',   8,   3,  75, "SC-88Pro Map Var #08", "Kawala"},
    {'M',   8,   3,  80, "SC-88Pro Map Var #08", "2600 Sine"},
    {'M',   8,   3,  81, "SC-88Pro Map Var #08", "Doctor Solo"},
    {'M',   8,   3,  83, "SC-88Pro Map Var #08", "Mad Lead"},
    {'M',   8,   3,  84, "SC-88Pro Map Var #08", "Dist.Lead"},
    {'M',   8,   3,  85, "SC-88Pro Map Var #08", "Vox Lead"},
    {'M',   8,   3,  86, "SC-88Pro Map Var #08", "4th Lead"},
    {'M',   8,   3,  89, "SC-88Pro Map Var #08", "Octave Pad"},
    {'M',   8,   3,  90, "SC-88Pro Map Var #08", "Power Stack"},
    {'M',   8,   3,  91, "SC-88Pro Map Var #08", "Cosmic Voice"},
    {'M',   8,   3,  94, "SC-88Pro Map Var #08", "Horror Pad"},
    {'M',   8,   3,  95, "SC-88Pro Map Var #08", "Converge"},
    {'M',   8,   3,  96, "SC-88Pro Map Var #08", "Clavi Pad"},
    {'M',   8,   3,  97, "SC-88Pro Map Var #08", "Rave"},
    {'M',   8,   3,  98, "SC-88Pro Map Var #08", "Vibra Bells"},
    {'M',   8,   3,  99, "SC-88Pro Map Var #08", "Pulsey Key"},
    {'M',   8,   3, 100, "SC-88Pro Map Var #08", "Org Bell"},
    {'M',   8,   3, 101, "SC-88Pro Map Var #08", "LowBirds Pad"},
    {'M',   8,   3, 102, "SC-88Pro Map Var #08", "Pan Sequence"},
    {'M',   8,   3, 103, "SC-88Pro Map Var #08", "Dream Pad"},
    {'M',   8,   3, 104, "SC-88Pro Map Var #08", "Tambra"},
    {'M',   8,   3, 105, "SC-88Pro Map Var #08", "Rabab"},
    {'M',   8,   3, 106, "SC-88Pro Map Var #08", "Syn Shamisen"},
    {'M',   8,   3, 107, "SC-88Pro Map Var #08", "Taisho Koto"},
    {'M',   8,   3, 108, "SC-88Pro Map Var #08", "Sanza"},
    {'M',   8,   3, 109, "SC-88Pro Map Var #08", "Didgeridoo"},
    {'M',   8,   3, 110, "SC-88Pro Map Var #08", "Er Hu"},
    {'M',   8,   3, 111, "SC-88Pro Map Var #08", "Pungi"},
    {'M',   8,   3, 112, "SC-88Pro Map Var #08", "Bonang"},
    {'M',   8,   3, 113, "SC-88Pro Map Var #08", "Atarigane"},
    {'M',   8,   3, 115, "SC-88Pro Map Var #08", "Castanets"},
    {'M',   8,   3, 116, "SC-88Pro Map Var #08", "Concert BD"},
    {'M',   8,   3, 117, "SC-88Pro Map Var #08", "Melo. Tom 2"},
    {'M',   8,   3, 118, "SC-88Pro Map Var #08", "808 Tom"},
    {'M',   8,   3, 119, "SC-88Pro Map Var #08", "Rev.Snare 1"},
    {'M',   8,   3, 120, "SC-88Pro Map Var #08", "Gt. FX Menu"},
    {'M',   8,   3, 124, "SC-88Pro Map Var #08", "ScratchKey"},
    {'M',   8,   3, 125, "SC-88Pro Map Var #08", "Starship"},
    {'M',   8,   3, 126, "SC-88Pro Map Var #08", "ApplauseWave"},
    {'M',   8,   4,   0, "SC-8850 Map Var #08", "Upright P w"},
    {'M',   8,   4,   1, "SC-8850 Map Var #08", "Pop Piano w"},
    {'M',   8,   4,   2, "SC-8850 Map Var #08", "Piano 3w"},
    {'M',   8,   4,   3, "SC-8850 Map Var #08", "Honky-tonk 2"},
    {'M',   8,   4,   4, "SC-8850 Map Var #08", "St.Soft EP"},
    {'M',   8,   4,   5, "SC-8850 Map Var #08", "Detuned EP 2"},
    {'M',   8,   4,   6, "SC-8850 Map Var #08", "Coupled Hps."},
    {'M',   8,   4,   7, "SC-8850 Map Var #08", "Comp Clav."},
    {'M',   8,   4,  10, "SC-8850 Map Var #08", "St.Music Box"},
    {'M',   8,   4,  11, "SC-8850 Map Var #08", "Vibraphone w"},
    {'M',   8,   4,  12, "SC-8850 Map Var #08", "Marimba w"},
    {'M',   8,   4,  13, "SC-8850 Map Var #08", "Xylophone w"},
    {'M',   8,   4,  14, "SC-8850 Map Var #08", "Church Bell"},
    {'M',   8,   4,  15, "SC-8850 Map Var #08", "Cimbalom"},
    {'M',   8,   4,  16, "SC-8850 Map Var #08", "Trem. Organ"},
    {'M',   8,   4,  17, "SC-8850 Map Var #08", "Chorus Or.2"},
    {'M',   8,   4,  18, "SC-8850 Map Var #08", "Rotary Org."},
    {'M',   8,   4,  19, "SC-8850 Map Var #08", "Church Org.2"},
    {'M',   8,   4,  20, "SC-8850 Map Var #08", "Wind Organ"},
    {'M',   8,   4,  21, "SC-8850 Map Var #08", "Accordion It"},
    {'M',   8,   4,  22, "SC-8850 Map Var #08", "B.Harp Basic"},
    {'M',   8,   4,  23, "SC-8850 Map Var #08", "Bandoneon 2"},
    {'M',   8,   4,  24, "SC-8850 Map Var #08", "Ukulele"},
    {'M',   8,   4,  25, "SC-8850 Map Var #08", "12-str.Gt"},
    {'M',   8,   4,  26, "SC-8850 Map Var #08", "Pedal Steel"},
    {'M',   8,   4,  27, "SC-8850 Map Var #08", "Chorus Gt."},
    {'M',   8,   4,  28, "SC-8850 Map Var #08", "Funk Pop"},
    {'M',   8,   4,  29, "SC-8850 Map Var #08", "LP OverDrvGt"},
    {'M',   8,   4,  30, "SC-8850 Map Var #08", "Feedback Gt."},
    {'M',   8,   4,  31, "SC-8850 Map Var #08", "Gt. Feedback"},
    {'M',   8,   4,  32, "SC-8850 Map Var #08", "Wild A.Bass"},
    {'M',   8,   4,  33, "SC-8850 Map Var #08", "ChorusJazzBs"},
    {'M',   8,   4,  34, "SC-8850 Map Var #08", "Muted PickBs"},
    {'M',   8,   4,  35, "SC-8850 Map Var #08", "Wood+FlessBs"},
    {'M',   8,   4,  36, "SC-8850 Map Var #08", "Reso Slap"},
    {'M',   8,   4,  37, "SC-8850 Map Var #08", "FM Slap"},
    {'M',   8,   4,  38, "SC-8850 Map Var #08", "Acid Bass"},
    {'M',   8,   4,  39, "SC-8850 Map Var #08", "Beef FM Bass"},
    {'M',   8,   4,  40, "SC-8850 Map Var #08", "Slow Violin"},
    {'M',   8,   4,  44, "SC-8850 Map Var #08", "Slow Tremolo"},
    {'M',   8,   4,  45, "SC-8850 Map Var #08", "Solo Pizz."},
    {'M',   8,   4,  46, "SC-8850 Map Var #08", "Uillean Harp"},
    {'M',   8,   4,  48, "SC-8850 Map Var #08", "Orchestra"},
    {'M',   8,   4,  49, "SC-8850 Map Var #08", "Legato Str."},
    {'M',   8,   4,  50, "SC-8850 Map Var #08", "Syn.Strings3"},
    {'M',   8,   4,  51, "SC-8850 Map Var #08", "Air Strings"},
    {'M',   8,   4,  52, "SC-8850 Map Var #08", "St.ChoirAahs"},
    {'M',   8,   4,  53, "SC-8850 Map Var #08", "Voice Dahs"},
    {'M',   8,   4,  54, "SC-8850 Map Var #08", "Syn.Voice"},
    {'M',   8,   4,  55, "SC-8850 Map Var #08", "Impact Hit"},
    {'M',   8,   4,  56, "SC-8850 Map Var #08", "Flugel Horn"},
    {'M',   8,   4,  57, "SC-8850 Map Var #08", "Bs. Trombone"},
    {'M',   8,   4,  58, "SC-8850 Map Var #08", "Tuba + Horn"},
    {'M',   8,   4,  59, "SC-8850 Map Var #08", "Muted Horns"},
    {'M',   8,   4,  60, "SC-8850 Map Var #08", "F.Hrn Slow:"},
    {'M',   8,   4,  61, "SC-8850 Map Var #08", "Brass 2"},
    {'M',   8,   4,  62, "SC-8850 Map Var #08", "Pro Brass"},
    {'M',   8,   4,  63, "SC-8850 Map Var #08", "SynBrass sfz"},
    {'M',   8,   4,  64, "SC-8850 Map Var #08", "Soprano Exp."},
    {'M',   8,   4,  65, "SC-8850 Map Var #08", "AltoSax Exp."},
    {'M',   8,   4,  66, "SC-8850 Map Var #08", "BreathyTn.:"},
    {'M',   8,   4,  67, "SC-8850 Map Var #08", "Bari & Tenor"},
    {'M',   8,   4,  68, "SC-8850 Map Var #08", "Oboe Exp."},
    {'M',   8,   4,  71, "SC-8850 Map Var #08", "Bs Clarinet"},
    {'M',   8,   4,  72, "SC-8850 Map Var #08", "Nay"},
    {'M',   8,   4,  73, "SC-8850 Map Var #08", "Flute + Vln"},
    {'M',   8,   4,  75, "SC-8850 Map Var #08", "Kawala"},
    {'M',   8,   4,  80, "SC-8850 Map Var #08", "2600 Sine"},
    {'M',   8,   4,  81, "SC-8850 Map Var #08", "Doctor Solo"},
    {'M',   8,   4,  82, "SC-8850 Map Var #08", "LM Pure Lead"},
    {'M',   8,   4,  83, "SC-8850 Map Var #08", "Mad Lead"},
    {'M',   8,   4,  84, "SC-8850 Map Var #08", "Dist.Lead"},
    {'M',   8,   4,  85, "SC-8850 Map Var #08", "Vox Lead"},
    {'M',   8,   4,  86, "SC-8850 Map Var #08", "4th Lead"},
    {'M',   8,   4,  89, "SC-8850 Map Var #08", "Octave Pad"},
    {'M',   8,   4,  90, "SC-8850 Map Var #08", "Power Stack"},
    {'M',   8,   4,  91, "SC-8850 Map Var #08", "Cosmic Voice"},
    {'M',   8,   4,  94, "SC-8850 Map Var #08", "Horror Pad"},
    {'M',   8,   4,  95, "SC-8850 Map Var #08", "Converge"},
    {'M',   8,   4,  96, "SC-8850 Map Var #08", "Clavi Pad"},
    {'M',   8,   4,  97, "SC-8850 Map Var #08", "Rave"},
    {'M',   8,   4,  98, "SC-8850 Map Var #08", "Vibra Bells"},
    {'M',   8,   4,  99, "SC-8850 Map Var #08", "Pulsey Key"},
    {'M',   8,   4, 100, "SC-8850 Map Var #08", "Org Bell"},
    {'M',   8,   4, 101, "SC-8850 Map Var #08", "LowBirds Pad"},
    {'M',   8,   4, 102, "SC-8850 Map Var #08", "Pan Sequence"},
    {'M',   8,   4, 103, "SC-8850 Map Var #08", "Dream Pad"},
    {'M',   8,   4, 104, "SC-8850 Map Var #08", "Tambra"},
    {'M',   8,   4, 105, "SC-8850 Map Var #08", "Rabab"},
    {'M',   8,   4, 106, "SC-8850 Map Var #08", "Syn Shamisen"},
    {'M',   8,   4, 107, "SC-8850 Map Var #08", "Taisho Koto"},
    {'M',   8,   4, 108, "SC-8850 Map Var #08", "Sanza"},
    {'M',   8,   4, 109, "SC-8850 Map Var #08", "Didgeridoo"},
    {'M',   8,   4, 110, "SC-8850 Map Var #08", "Er Hu"},
    {'M',   8,   4, 111, "SC-8850 Map Var #08", "Pungi"},
    {'M',   8,   4, 112, "SC-8850 Map Var #08", "Bonang"},
    {'M',   8,   4, 113, "SC-8850 Map Var #08", "Atarigane"},
    {'M',   8,   4, 115, "SC-8850 Map Var #08", "Castanets"},
    {'M',   8,   4, 116, "SC-8850 Map Var #08", "Concert BD"},
    {'M',   8,   4, 117, "SC-8850 Map Var #08", "Melo. Tom 2"},
    {'M',   8,   4, 118, "SC-8850 Map Var #08", "808 Tom"},
    {'M',   8,   4, 119, "SC-8850 Map Var #08", "Rev.Snare 1"},
    {'M',   8,   4, 120, "SC-8850 Map Var #08", "Gt. FX Menu"},
    {'M',   8,   4, 121, "SC-8850 Map Var #08", "Trombone Nz"},
    {'M',   8,   4, 123, "SC-8850 Map Var #08", "Seal"},
    {'M',   8,   4, 124, "SC-8850 Map Var #08", "ScratchKey"},
    {'M',   8,   4, 125, "SC-8850 Map Var #08", "Starship"},
    {'M',   8,   4, 126, "SC-8850 Map Var #08", "ApplauseWave"},
    {'M',   9,   0,   0, "SC-8850 Var #09", "Mild Piano w"},
    {'M',   9,   0,   1, "SC-8850 Var #09", "Rock Piano w"},
    {'M',   9,   0,   4, "SC-8850 Var #09", "Cho. E.Piano"},
    {'M',   9,   0,   5, "SC-8850 Var #09", "Detuned EP 3"},
    {'M',   9,   0,  11, "SC-8850 Var #09", "Vibraphones"},
    {'M',   9,   0,  14, "SC-8850 Var #09", "Carillon"},
    {'M',   9,   0,  16, "SC-8850 Var #09", "Organ o"},
    {'M',   9,   0,  17, "SC-8850 Var #09", "Octave Organ"},
    {'M',   9,   0,  21, "SC-8850 Var #09", "Dist. Accord"},
    {'M',   9,   0,  22, "SC-8850 Var #09", "B.Harp Suppl"},
    {'M',   9,   0,  25, "SC-8850 Var #09", "Nylon+Steel"},
    {'M',   9,   0,  27, "SC-8850 Var #09", "JC Chorus Gt"},
    {'M',   9,   0,  29, "SC-8850 Var #09", "LP OverDrv:"},
    {'M',   9,   0,  30, "SC-8850 Var #09", "Feedback Gt2"},
    {'M',   9,   0,  31, "SC-8850 Var #09", "Gt.Feedback2"},
    {'M',   9,   0,  32, "SC-8850 Var #09", "Atk A.Bass"},
    {'M',   9,   0,  36, "SC-8850 Var #09", "Unison Slap"},
    {'M',   9,   0,  38, "SC-8850 Var #09", "TB303 Bass"},
    {'M',   9,   0,  39, "SC-8850 Var #09", "Dly Bass"},
    {'M',   9,   0,  44, "SC-8850 Var #09", "Suspense Str"},
    {'M',   9,   0,  48, "SC-8850 Var #09", "Orchestra 2"},
    {'M',   9,   0,  49, "SC-8850 Var #09", "Warm Strings"},
    {'M',   9,   0,  50, "SC-8850 Var #09", "Syn.Strings4"},
    {'M',   9,   0,  51, "SC-8850 Var #09", "Atk Syn Str."},
    {'M',   9,   0,  52, "SC-8850 Var #09", "Melted Choir"},
    {'M',   9,   0,  53, "SC-8850 Var #09", "JzVoice Dat"},
    {'M',   9,   0,  54, "SC-8850 Var #09", "Silent Night"},
    {'M',   9,   0,  55, "SC-8850 Var #09", "Philly Hit"},
    {'M',   9,   0,  60, "SC-8850 Var #09", "Dual Horns"},
    {'M',   9,   0,  61, "SC-8850 Var #09", "Brass 3"},
    {'M',   9,   0,  62, "SC-8850 Var #09", "P5 Brass"},
    {'M',   9,   0,  63, "SC-8850 Var #09", "OB Brass"},
    {'M',   9,   0,  65, "SC-8850 Var #09", "Grow Sax"},
    {'M',   9,   0,  66, "SC-8850 Var #09", "St.Tenor Sax"},
    {'M',   9,   0,  72, "SC-8850 Var #09", "Nay Tremolo"},
    {'M',   9,   0,  73, "SC-8850 Var #09", "Pipe & Reed"},
    {'M',   9,   0,  80, "SC-8850 Var #09", "Sine Lead"},
    {'M',   9,   0,  81, "SC-8850 Var #09", "Fat Saw Lead"},
    {'M',   9,   0,  82, "SC-8850 Var #09", "LM Blow Lead"},
    {'M',   9,   0,  83, "SC-8850 Var #09", "CrowdingLead"},
    {'M',   9,   0,  84, "SC-8850 Var #09", "Acid Guitar1"},
    {'M',   9,   0,  85, "SC-8850 Var #09", "LFO Vox"},
    {'M',   9,   0,  89, "SC-8850 Var #09", "Stack Pad"},
    {'M',   9,   0,  90, "SC-8850 Var #09", "Octave Stack"},
    {'M',   9,   0,  91, "SC-8850 Var #09", "Auh Vox"},
    {'M',   9,   0,  94, "SC-8850 Var #09", "SynVox Pad"},
    {'M',   9,   0,  95, "SC-8850 Var #09", "Shwimmer"},
    {'M',   9,   0,  96, "SC-8850 Var #09", "EP Pad"},
    {'M',   9,   0,  98, "SC-8850 Var #09", "Digi Bells"},
    {'M',   9,   0,  99, "SC-8850 Var #09", "Noise Piano"},
    {'M',   9,   0, 101, "SC-8850 Var #09", "Falling Down"},
    {'M',   9,   0, 102, "SC-8850 Var #09", "Aqua"},
    {'M',   9,   0, 103, "SC-8850 Var #09", "Silky Pad"},
    {'M',   9,   0, 105, "SC-8850 Var #09", "San Xian"},
    {'M',   9,   0, 108, "SC-8850 Var #09", "Bodhran"},
    {'M',   9,   0, 109, "SC-8850 Var #09", "Uillean Pipe"},
    {'M',   9,   0, 110, "SC-8850 Var #09", "Gao Hu"},
    {'M',   9,   0, 112, "SC-8850 Var #09", "Gender"},
    {'M',   9,   0, 116, "SC-8850 Var #09", "ConcertBD Mt"},
    {'M',   9,   0, 117, "SC-8850 Var #09", "Rock Tom"},
    {'M',   9,   0, 118, "SC-8850 Var #09", "Elec Perc"},
    {'M',   9,   0, 119, "SC-8850 Var #09", "Rev.Snare 2"},
    {'M',   9,   0, 120, "SC-8850 Var #09", "Bartok Pizz."},
    {'M',   9,   0, 121, "SC-8850 Var #09", "Trumpet Nz"},
    {'M',   9,   0, 124, "SC-8850 Var #09", "TapeRewind"},
    {'M',   9,   0, 125, "SC-8850 Var #09", "Burst Noise"},
    {'M',   9,   0, 126, "SC-8850 Var #09", "BabyLaughing"},
    {'M',   9,   1,  14, "SC-55 Map Var #09", "Carillon"},
    {'M',   9,   1, 118, "SC-55 Map Var #09", "Elec Perc"},
    {'M',   9,   1, 125, "SC-55 Map Var #09", "Burst Noise"},
    {'M',   9,   2,  14, "SC-88 Map Var #09", "Carillon"},
    {'M',   9,   2,  16, "SC-88 Map Var #09", "Organ 109"},
    {'M',   9,   2,  25, "SC-88 Map Var #09", "Nylon+Steel"},
    {'M',   9,   2,  30, "SC-88 Map Var #09", "FeedbackGt2"},
    {'M',   9,   2,  38, "SC-88 Map Var #09", "TB303 Bass"},
    {'M',   9,   2,  39, "SC-88 Map Var #09", "X Wire Bass"},
    {'M',   9,   2,  44, "SC-88 Map Var #09", "Suspense Str"},
    {'M',   9,   2,  48, "SC-88 Map Var #09", "Orchestra 2"},
    {'M',   9,   2,  49, "SC-88 Map Var #09", "WarmStrings"},
    {'M',   9,   2,  52, "SC-88 Map Var #09", "Mello Choir"},
    {'M',   9,   2,  55, "SC-88 Map Var #09", "Philly Hit"},
    {'M',   9,   2,  62, "SC-88 Map Var #09", "Quack Brass"},
    {'M',   9,   2,  95, "SC-88 Map Var #09", "Shwimmer"},
    {'M',   9,   2,  98, "SC-88 Map Var #09", "Digi Bells"},
    {'M',   9,   2, 112, "SC-88 Map Var #09", "Gender"},
    {'M',   9,   2, 117, "SC-88 Map Var #09", "Rock Tom"},
    {'M',   9,   2, 118, "SC-88 Map Var #09", "Elec Perc"},
    {'M',   9,   2, 119, "SC-88 Map Var #09", "Rev.Snare 2"},
    {'M',   9,   2, 125, "SC-88 Map Var #09", "Burst Noise"},
    {'M',   9,   3,   4, "SC-88Pro Map Var #09", "Cho. E.Piano"},
    {'M',   9,   3,  11, "SC-88Pro Map Var #09", "Vibraphones"},
    {'M',   9,   3,  14, "SC-88Pro Map Var #09", "Carillon"},
    {'M',   9,   3,  16, "SC-88Pro Map Var #09", "Organ. o"},
    {'M',   9,   3,  17, "SC-88Pro Map Var #09", "Octave Organ"},
    {'M',   9,   3,  21, "SC-88Pro Map Var #09", "Dist. Accord"},
    {'M',   9,   3,  25, "SC-88Pro Map Var #09", "Nylon+Steel"},
    {'M',   9,   3,  27, "SC-88Pro Map Var #09", "JC Chorus Gt"},
    {'M',   9,   3,  29, "SC-88Pro Map Var #09", "LP OverDrv:"},
    {'M',   9,   3,  30, "SC-88Pro Map Var #09", "Feedback Gt2"},
    {'M',   9,   3,  31, "SC-88Pro Map Var #09", "Gt.Feedback2"},
    {'M',   9,   3,  36, "SC-88Pro Map Var #09", "Unison Slap"},
    {'M',   9,   3,  38, "SC-88Pro Map Var #09", "TB303 Bass"},
    {'M',   9,   3,  39, "SC-88Pro Map Var #09", "Dly Bass"},
    {'M',   9,   3,  44, "SC-88Pro Map Var #09", "Suspense Str"},
    {'M',   9,   3,  48, "SC-88Pro Map Var #09", "Orchestra 2"},
    {'M',   9,   3,  49, "SC-88Pro Map Var #09", "Warm Strings"},
    {'M',   9,   3,  50, "SC-88Pro Map Var #09", "Syn.Strings4"},
    {'M',   9,   3,  52, "SC-88Pro Map Var #09", "Melted Choir"},
    {'M',   9,   3,  54, "SC-88Pro Map Var #09", "Silent Night"},
    {'M',   9,   3,  55, "SC-88Pro Map Var #09", "Philly Hit"},
    {'M',   9,   3,  60, "SC-88Pro Map Var #09", "Dual Horns"},
    {'M',   9,   3,  61, "SC-88Pro Map Var #09", "Brass 3"},
    {'M',   9,   3,  62, "SC-88Pro Map Var #09", "P5 Brass"},
    {'M',   9,   3,  63, "SC-88Pro Map Var #09", "OB Brass"},
    {'M',   9,   3,  65, "SC-88Pro Map Var #09", "Grow Sax"},
    {'M',   9,   3,  66, "SC-88Pro Map Var #09", "St.Tenor Sax"},
    {'M',   9,   3,  72, "SC-88Pro Map Var #09", "Nay Tremolo"},
    {'M',   9,   3,  80, "SC-88Pro Map Var #09", "Sine Lead"},
    {'M',   9,   3,  81, "SC-88Pro Map Var #09", "Fat Saw Lead"},
    {'M',   9,   3,  84, "SC-88Pro Map Var #09", "Acid Guitar1"},
    {'M',   9,   3,  85, "SC-88Pro Map Var #09", "LFO Vox"},
    {'M',   9,   3,  89, "SC-88Pro Map Var #09", "Stack Pad"},
    {'M',   9,   3,  90, "SC-88Pro Map Var #09", "Octave Stack"},
    {'M',   9,   3,  91, "SC-88Pro Map Var #09", "Auh Vox"},
    {'M',   9,   3,  95, "SC-88Pro Map Var #09", "Shwimmer"},
    {'M',   9,   3,  98, "SC-88Pro Map Var #09", "Digi Bells"},
    {'M',   9,   3,  99, "SC-88Pro Map Var #09", "Noise Piano"},
    {'M',   9,   3, 101, "SC-88Pro Map Var #09", "Falling Down"},
    {'M',   9,   3, 102, "SC-88Pro Map Var #09", "Aqua"},
    {'M',   9,   3, 103, "SC-88Pro Map Var #09", "Silky Pad"},
    {'M',   9,   3, 105, "SC-88Pro Map Var #09", "San Xian"},
    {'M',   9,   3, 110, "SC-88Pro Map Var #09", "Gao Hu"},
    {'M',   9,   3, 112, "SC-88Pro Map Var #09", "Gender"},
    {'M',   9,   3, 117, "SC-88Pro Map Var #09", "Rock Tom"},
    {'M',   9,   3, 118, "SC-88Pro Map Var #09", "Elec Perc"},
    {'M',   9,   3, 119, "SC-88Pro Map Var #09", "Rev.Snare 2"},
    {'M',   9,   3, 120, "SC-88Pro Map Var #09", "Bartok Pizz."},
    {'M',   9,   3, 124, "SC-88Pro Map Var #09", "TapeRewind"},
    {'M',   9,   3, 125, "SC-88Pro Map Var #09", "Burst Noise"},
    {'M',   9,   4,   0, "SC-8850 Map Var #09", "Mild Piano w"},
    {'M',   9,   4,   1, "SC-8850 Map Var #09", "Rock Piano w"},
    {'M',   9,   4,   4, "SC-8850 Map Var #09", "Cho. E.Piano"},
    {'M',   9,   4,   5, "SC-8850 Map Var #09", "Detuned EP 3"},
    {'M',   9,   4,  11, "SC-8850 Map Var #09", "Vibraphones"},
    {'M',   9,   4,  14, "SC-8850 Map Var #09", "Carillon"},
    {'M',   9,   4,  16, "SC-8850 Map Var #09", "Organ o"},
    {'M',   9,   4,  17, "SC-8850 Map Var #09", "Octave Organ"},
    {'M',   9,   4,  21, "SC-8850 Map Var #09", "Dist. Accord"},
    {'M',   9,   4,  22, "SC-8850 Map Var #09", "B.Harp Suppl"},
    {'M',   9,   4,  25, "SC-8850 Map Var #09", "Nylon+Steel"},
    {'M',   9,   4,  27, "SC-8850 Map Var #09", "JC Chorus Gt"},
    {'M',   9,   4,  29, "SC-8850 Map Var #09", "LP OverDrv:"},
    {'M',   9,   4,  30, "SC-8850 Map Var #09", "Feedback Gt2"},
    {'M',   9,   4,  31, "SC-8850 Map Var #09", "Gt.Feedback2"},
    {'M',   9,   4,  32, "SC-8850 Map Var #09", "Atk A.Bass"},
    {'M',   9,   4,  36, "SC-8850 Map Var #09", "Unison Slap"},
    {'M',   9,   4,  38, "SC-8850 Map Var #09", "TB303 Bass"},
    {'M',   9,   4,  39, "SC-8850 Map Var #09", "Dly Bass"},
    {'M',   9,   4,  44, "SC-8850 Map Var #09", "Suspense Str"},
    {'M',   9,   4,  48, "SC-8850 Map Var #09", "Orchestra 2"},
    {'M',   9,   4,  49, "SC-8850 Map Var #09", "Warm Strings"},
    {'M',   9,   4,  50, "SC-8850 Map Var #09", "Syn.Strings4"},
    {'M',   9,   4,  51, "SC-8850 Map Var #09", "Atk Syn Str."},
    {'M',   9,   4,  52, "SC-8850 Map Var #09", "Melted Choir"},
    {'M',   9,   4,  53, "SC-8850 Map Var #09", "JzVoice Dat"},
    {'M',   9,   4,  54, "SC-8850 Map Var #09", "Silent Night"},
    {'M',   9,   4,  55, "SC-8850 Map Var #09", "Philly Hit"},
    {'M',   9,   4,  60, "SC-8850 Map Var #09", "Dual Horns"},
    {'M',   9,   4,  61, "SC-8850 Map Var #09", "Brass 3"},
    {'M',   9,   4,  62, "SC-8850 Map Var #09", "P5 Brass"},
    {'M',   9,   4,  63, "SC-8850 Map Var #09", "OB Brass"},
    {'M',   9,   4,  65, "SC-8850 Map Var #09", "Grow Sax"},
    {'M',   9,   4,  66, "SC-8850 Map Var #09", "St.Tenor Sax"},
    {'M',   9,   4,  72, "SC-8850 Map Var #09", "Nay Tremolo"},
    {'M',   9,   4,  73, "SC-8850 Map Var #09", "Pipe & Reed"},
    {'M',   9,   4,  80, "SC-8850 Map Var #09", "Sine Lead"},
    {'M',   9,   4,  81, "SC-8850 Map Var #09", "Fat Saw Lead"},
    {'M',   9,   4,  82, "SC-8850 Map Var #09", "LM Blow Lead"},
    {'M',   9,   4,  83, "SC-8850 Map Var #09", "CrowdingLead"},
    {'M',   9,   4,  84, "SC-8850 Map Var #09", "Acid Guitar1"},
    {'M',   9,   4,  85, "SC-8850 Map Var #09", "LFO Vox"},
    {'M',   9,   4,  89, "SC-8850 Map Var #09", "Stack Pad"},
    {'M',   9,   4,  90, "SC-8850 Map Var #09", "Octave Stack"},
    {'M',   9,   4,  91, "SC-8850 Map Var #09", "Auh Vox"},
    {'M',   9,   4,  94, "SC-8850 Map Var #09", "SynVox Pad"},
    {'M',   9,   4,  95, "SC-8850 Map Var #09", "Shwimmer"},
    {'M',   9,   4,  96, "SC-8850 Map Var #09", "EP Pad"},
    {'M',   9,   4,  98, "SC-8850 Map Var #09", "Digi Bells"},
    {'M',   9,   4,  99, "SC-8850 Map Var #09", "Noise Piano"},
    {'M',   9,   4, 101, "SC-8850 Map Var #09", "Falling Down"},
    {'M',   9,   4, 102, "SC-8850 Map Var #09", "Aqua"},
    {'M',   9,   4, 103, "SC-8850 Map Var #09", "Silky Pad"},
    {'M',   9,   4, 105, "SC-8850 Map Var #09", "San Xian"},
    {'M',   9,   4, 108, "SC-8850 Map Var #09", "Bodhran"},
    {'M',   9,   4, 109, "SC-8850 Map Var #09", "Uillean Pipe"},
    {'M',   9,   4, 110, "SC-8850 Map Var #09", "Gao Hu"},
    {'M',   9,   4, 112, "SC-8850 Map Var #09", "Gender"},
    {'M',   9,   4, 116, "SC-8850 Map Var #09", "ConcertBD Mt"},
    {'M',   9,   4, 117, "SC-8850 Map Var #09", "Rock Tom"},
    {'M',   9,   4, 118, "SC-8850 Map Var #09", "Elec Perc"},
    {'M',   9,   4, 119, "SC-8850 Map Var #09", "Rev.Snare 2"},
    {'M',   9,   4, 120, "SC-8850 Map Var #09", "Bartok Pizz."},
    {'M',   9,   4, 121, "SC-8850 Map Var #09", "Trumpet Nz"},
    {'M',   9,   4, 124, "SC-8850 Map Var #09", "TapeRewind"},
    {'M',   9,   4, 125, "SC-8850 Map Var #09", "Burst Noise"},
    {'M',   9,   4, 126, "SC-8850 Map Var #09", "BabyLaughing"},
    {'M',  10,   0,   4, "SC-8850 Var #10", "SilentRhodes"},
    {'M',  10,   0,   5, "SC-8850 Var #10", "EP Legend"},
    {'M',  10,   0,  14, "SC-8850 Var #10", "Church Bell2"},
    {'M',  10,   0,  16, "SC-8850 Var #10", "Ful Organ 7"},
    {'M',  10,   0,  25, "SC-8850 Var #10", "Atk Steel Gt"},
    {'M',  10,   0,  29, "SC-8850 Var #10", "LP Half Drv"},
    {'M',  10,   0,  38, "SC-8850 Var #10", "Tekno Bass"},
    {'M',  10,   0,  39, "SC-8850 Var #10", "X Wire Bass"},
    {'M',  10,   0,  44, "SC-8850 Var #10", "SuspenseStr2"},
    {'M',  10,   0,  48, "SC-8850 Var #10", "Tremolo Orch"},
    {'M',  10,   0,  49, "SC-8850 Var #10", "St.Slow Str."},
    {'M',  10,   0,  50, "SC-8850 Var #10", "Syn.Strings6"},
    {'M',  10,   0,  51, "SC-8850 Var #10", "StraightStr."},
    {'M',  10,   0,  52, "SC-8850 Var #10", "Church Choir"},
    {'M',  10,   0,  53, "SC-8850 Var #10", "JzVoice Bop"},
    {'M',  10,   0,  54, "SC-8850 Var #10", "Syn.Voice 2"},
    {'M',  10,   0,  55, "SC-8850 Var #10", "Double Hit"},
    {'M',  10,   0,  61, "SC-8850 Var #10", "Brass sfz"},
    {'M',  10,   0,  62, "SC-8850 Var #10", "OrchSynBrass"},
    {'M',  10,   0,  63, "SC-8850 Var #10", "Reso Brass"},
    {'M',  10,   0,  80, "SC-8850 Var #10", "KG Lead"},
    {'M',  10,   0,  81, "SC-8850 Var #10", "JP8000 Saw"},
    {'M',  10,   0,  83, "SC-8850 Var #10", "Double Sqr."},
    {'M',  10,   0,  84, "SC-8850 Var #10", "Acid Guitar2"},
    {'M',  10,   0,  85, "SC-8850 Var #10", "Vox Lead 2"},
    {'M',  10,   0,  89, "SC-8850 Var #10", "Human Pad"},
    {'M',  10,   0,  90, "SC-8850 Var #10", "Reso Stack"},
    {'M',  10,   0,  91, "SC-8850 Var #10", "AuhAuh"},
    {'M',  10,   0,  94, "SC-8850 Var #10", "SynVox Pad 2"},
    {'M',  10,   0,  95, "SC-8850 Var #10", "Celestial Pd"},
    {'M',  10,   0,  96, "SC-8850 Var #10", "Tambra Pad"},
    {'M',  10,   0,  98, "SC-8850 Var #10", "Music Bell"},
    {'M',  10,   0,  99, "SC-8850 Var #10", "Heaven Atmos"},
    {'M',  10,   0, 101, "SC-8850 Var #10", "LFO RAVE"},
    {'M',  10,   0, 102, "SC-8850 Var #10", "Panning Lead"},
    {'M',  10,   0, 103, "SC-8850 Var #10", "Dream Pad 2"},
    {'M',  10,   0, 108, "SC-8850 Var #10", "Bodhran Mute"},
    {'M',  10,   0, 109, "SC-8850 Var #10", "UillnPipe Nm"},
    {'M',  10,   0, 112, "SC-8850 Var #10", "Gamelan Gong"},
    {'M',  10,   0, 118, "SC-8850 Var #10", "Sine Perc."},
    {'M',  10,   0, 120, "SC-8850 Var #10", "Guitar Slap"},
    {'M',  10,   0, 124, "SC-8850 Var #10", "Phono Noise"},
    {'M',  10,   0, 125, "SC-8850 Var #10", "Calculating"},
    {'M',  10,   2,  38, "SC-88 Map Var #10", "Tekno Bass"},
    {'M',  10,   2,  48, "SC-88 Map Var #10", "TremoloOrch"},
    {'M',  10,   2,  49, "SC-88 Map Var #10", "St.SlowStr."},
    {'M',  10,   2,  55, "SC-88 Map Var #10", "Double Hit"},
    {'M',  10,   2,  95, "SC-88 Map Var #10", "CelestialPd"},
    {'M',  10,   2, 112, "SC-88 Map Var #10", "Gamelan Gong"},
    {'M',  10,   3,   4, "SC-88Pro Map Var #10", "SilentRhodes"},
    {'M',  10,   3,  38, "SC-88Pro Map Var #10", "Tekno Bass"},
    {'M',  10,   3,  39, "SC-88Pro Map Var #10", "X Wire Bass"},
    {'M',  10,   3,  48, "SC-88Pro Map Var #10", "Tremolo Orch"},
    {'M',  10,   3,  49, "SC-88Pro Map Var #10", "St.Slow Str."},
    {'M',  10,   3,  52, "SC-88Pro Map Var #10", "Church Choir"},
    {'M',  10,   3,  55, "SC-88Pro Map Var #10", "Double Hit"},
    {'M',  10,   3,  61, "SC-88Pro Map Var #10", "Brass sfz"},
    {'M',  10,   3,  63, "SC-88Pro Map Var #10", "Reso Brass"},
    {'M',  10,   3,  80, "SC-88Pro Map Var #10", "KG Lead"},
    {'M',  10,   3,  84, "SC-88Pro Map Var #10", "Acid Guitar2"},
    {'M',  10,   3,  90, "SC-88Pro Map Var #10", "Reso Stack"},
    {'M',  10,   3,  91, "SC-88Pro Map Var #10", "AuhAuh"},
    {'M',  10,   3,  95, "SC-88Pro Map Var #10", "Celestial Pd"},
    {'M',  10,   3,  98, "SC-88Pro Map Var #10", "Music Bell"},
    {'M',  10,   3, 101, "SC-88Pro Map Var #10", "LFO RAVE"},
    {'M',  10,   3, 112, "SC-88Pro Map Var #10", "Gamelan Gong"},
    {'M',  10,   3, 118, "SC-88Pro Map Var #10", "Sine Perc."},
    {'M',  10,   3, 120, "SC-88Pro Map Var #10", "Guitar Slap"},
    {'M',  10,   3, 124, "SC-88Pro Map Var #10", "Phono Noise"},
    {'M',  10,   3, 125, "SC-88Pro Map Var #10", "Calculating"},
    {'M',  10,   4,   4, "SC-8850 Map Var #10", "SilentRhodes"},
    {'M',  10,   4,   5, "SC-8850 Map Var #10", "EP Legend"},
    {'M',  10,   4,  14, "SC-8850 Map Var #10", "Church Bell2"},
    {'M',  10,   4,  16, "SC-8850 Map Var #10", "Ful Organ 7"},
    {'M',  10,   4,  25, "SC-8850 Map Var #10", "Atk Steel Gt"},
    {'M',  10,   4,  29, "SC-8850 Map Var #10", "LP Half Drv"},
    {'M',  10,   4,  38, "SC-8850 Map Var #10", "Tekno Bass"},
    {'M',  10,   4,  39, "SC-8850 Map Var #10", "X Wire Bass"},
    {'M',  10,   4,  44, "SC-8850 Map Var #10", "SuspenseStr2"},
    {'M',  10,   4,  48, "SC-8850 Map Var #10", "Tremolo Orch"},
    {'M',  10,   4,  49, "SC-8850 Map Var #10", "St.Slow Str."},
    {'M',  10,   4,  50, "SC-8850 Map Var #10", "Syn.Strings6"},
    {'M',  10,   4,  51, "SC-8850 Map Var #10", "StraightStr."},
    {'M',  10,   4,  52, "SC-8850 Map Var #10", "Church Choir"},
    {'M',  10,   4,  53, "SC-8850 Map Var #10", "JzVoice Bop"},
    {'M',  10,   4,  54, "SC-8850 Map Var #10", "Syn.Voice 2"},
    {'M',  10,   4,  55, "SC-8850 Map Var #10", "Double Hit"},
    {'M',  10,   4,  61, "SC-8850 Map Var #10", "Brass sfz"},
    {'M',  10,   4,  62, "SC-8850 Map Var #10", "OrchSynBrass"},
    {'M',  10,   4,  63, "SC-8850 Map Var #10", "Reso Brass"},
    {'M',  10,   4,  80, "SC-8850 Map Var #10", "KG Lead"},
    {'M',  10,   4,  81, "SC-8850 Map Var #10", "JP8000 Saw"},
    {'M',  10,   4,  83, "SC-8850 Map Var #10", "Double Sqr."},
    {'M',  10,   4,  84, "SC-8850 Map Var #10", "Acid Guitar2"},
    {'M',  10,   4,  85, "SC-8850 Map Var #10", "Vox Lead 2"},
    {'M',  10,   4,  89, "SC-8850 Map Var #10", "Human Pad"},
    {'M',  10,   4,  90, "SC-8850 Map Var #10", "Reso Stack"},
    {'M',  10,   4,  91, "SC-8850 Map Var #10", "AuhAuh"},
    {'M',  10,   4,  94, "SC-8850 Map Var #10", "SynVox Pad 2"},
    {'M',  10,   4,  95, "SC-8850 Map Var #10", "Celestial Pd"},
    {'M',  10,   4,  96, "SC-8850 Map Var #10", "Tambra Pad"},
    {'M',  10,   4,  98, "SC-8850 Map Var #10", "Music Bell"},
    {'M',  10,   4,  99, "SC-8850 Map Var #10", "Heaven Atmos"},
    {'M',  10,   4, 101, "SC-8850 Map Var #10", "LFO RAVE"},
    {'M',  10,   4, 102, "SC-8850 Map Var #10", "Panning Lead"},
    {'M',  10,   4, 103, "SC-8850 Map Var #10", "Dream Pad 2"},
    {'M',  10,   4, 108, "SC-8850 Map Var #10", "Bodhran Mute"},
    {'M',  10,   4, 109, "SC-8850 Map Var #10", "UillnPipe Nm"},
    {'M',  10,   4, 112, "SC-8850 Map Var #10", "Gamelan Gong"},
    {'M',  10,   4, 118, "SC-8850 Map Var #10", "Sine Perc."},
    {'M',  10,   4, 120, "SC-8850 Map Var #10", "Guitar Slap"},
    {'M',  10,   4, 124, "SC-8850 Map Var #10", "Phono Noise"},
    {'M',  10,   4, 125, "SC-8850 Map Var #10", "Calculating"},
    {'M',  11,   0,  16, "SC-8850 Var #11", "Ful Organ 8"},
    {'M',  11,   0,  29, "SC-8850 Var #11", "LP Half Drv2"},
    {'M',  11,   0,  38, "SC-8850 Var #11", "TB303 Bass 2"},
    {'M',  11,   0,  39, "SC-8850 Var #11", "WireStr Bass"},
    {'M',  11,   0,  48, "SC-8850 Var #11", "Choir Str."},
    {'M',  11,   0,  49, "SC-8850 Var #11", "St.Slow Str2"},
    {'M',  11,   0,  50, "SC-8850 Var #11", "Syn.Strings7"},
    {'M',  11,   0,  52, "SC-8850 Var #11", "Boys Choir 1"},
    {'M',  11,   0,  53, "SC-8850 Var #11", "JzVoice Dow"},
    {'M',  11,   0,  55, "SC-8850 Var #11", "Perc. Hit"},
    {'M',  11,   0,  63, "SC-8850 Var #11", "DistSqrBrass"},
    {'M',  11,   0,  80, "SC-8850 Var #11", "Twin Sine"},
    {'M',  11,   0,  81, "SC-8850 Var #11", "D-50 Fat Saw"},
    {'M',  11,   0,  84, "SC-8850 Var #11", "Dance Dst.Gt"},
    {'M',  11,   0,  89, "SC-8850 Var #11", "Sync Brs.Pad"},
    {'M',  11,   0,  90, "SC-8850 Var #11", "Techno Stack"},
    {'M',  11,   0,  91, "SC-8850 Var #11", "Vocorderman"},
    {'M',  11,   0,  94, "SC-8850 Var #11", "Breath&Rise"},
    {'M',  11,   0,  95, "SC-8850 Var #11", "Bag Sweep"},
    {'M',  11,   0,  96, "SC-8850 Var #11", "CP Pad"},
    {'M',  11,   0,  98, "SC-8850 Var #11", "Analog Bell"},
    {'M',  11,   0,  99, "SC-8850 Var #11", "Tambra Atmos"},
    {'M',  11,   0, 101, "SC-8850 Var #11", "LFO Horror"},
    {'M',  11,   0, 102, "SC-8850 Var #11", "PanningBrass"},
    {'M',  11,   0, 103, "SC-8850 Var #11", "Silky Pad 2"},
    {'M',  11,   0, 109, "SC-8850 Var #11", "UillnPipe Or"},
    {'M',  11,   0, 112, "SC-8850 Var #11", "St.Gamelan"},
    {'M',  11,   0, 118, "SC-8850 Var #11", "606 Tom"},
    {'M',  11,   0, 120, "SC-8850 Var #11", "Chord Stroke"},
    {'M',  11,   0, 124, "SC-8850 Var #11", "MC-500 Beep"},
    {'M',  11,   0, 125, "SC-8850 Var #11", "Perc. Bang"},
    {'M',  11,   2,  48, "SC-88 Map Var #11", "Choir Str."},
    {'M',  11,   2, 112, "SC-88 Map Var #11", "St.Gamelan"},
    {'M',  11,   3,  38, "SC-88Pro Map Var #11", "TB303 Bass 2"},
    {'M',  11,   3,  39, "SC-88Pro Map Var #11", "WireStr Bass"},
    {'M',  11,   3,  48, "SC-88Pro Map Var #11", "Choir Str."},
    {'M',  11,   3,  55, "SC-88Pro Map Var #11", "Perc. Hit"},
    {'M',  11,   3,  81, "SC-88Pro Map Var #11", "D-50 Fat Saw"},
    {'M',  11,   3,  90, "SC-88Pro Map Var #11", "Techno Stack"},
    {'M',  11,   3,  91, "SC-88Pro Map Var #11", "Vocorderman"},
    {'M',  11,   3,  95, "SC-88Pro Map Var #11", "Bag Sweep"},
    {'M',  11,   3,  98, "SC-88Pro Map Var #11", "Analog Bell"},
    {'M',  11,   3, 101, "SC-88Pro Map Var #11", "LFO Horror"},
    {'M',  11,   3, 112, "SC-88Pro Map Var #11", "St.Gamelan"},
    {'M',  11,   3, 118, "SC-88Pro Map Var #11", "606 Tom"},
    {'M',  11,   3, 120, "SC-88Pro Map Var #11", "Chord Stroke"},
    {'M',  11,   3, 124, "SC-88Pro Map Var #11", "MC-500 Beep"},
    {'M',  11,   3, 125, "SC-88Pro Map Var #11", "Perc. Bang"},
    {'M',  11,   4,  16, "SC-8850 Map Var #11", "Ful Organ 8"},
    {'M',  11,   4,  29, "SC-8850 Map Var #11", "LP Half Drv2"},
    {'M',  11,   4,  38, "SC-8850 Map Var #11", "TB303 Bass 2"},
    {'M',  11,   4,  39, "SC-8850 Map Var #11", "WireStr Bass"},
    {'M',  11,   4,  48, "SC-8850 Map Var #11", "Choir Str."},
    {'M',  11,   4,  49, "SC-8850 Map Var #11", "St.Slow Str2"},
    {'M',  11,   4,  50, "SC-8850 Map Var #11", "Syn.Strings7"},
    {'M',  11,   4,  52, "SC-8850 Map Var #11", "Boys Choir 1"},
    {'M',  11,   4,  53, "SC-8850 Map Var #11", "JzVoice Doun"},
    {'M',  11,   4,  55, "SC-8850 Map Var #11", "Perc. Hit"},
    {'M',  11,   4,  63, "SC-8850 Map Var #11", "DistSqrBrass"},
    {'M',  11,   4,  80, "SC-8850 Map Var #11", "Twin Sine"},
    {'M',  11,   4,  81, "SC-8850 Map Var #11", "D-50 Fat Saw"},
    {'M',  11,   4,  84, "SC-8850 Map Var #11", "Dance Dst.Gt"},
    {'M',  11,   4,  89, "SC-8850 Map Var #11", "Sync Brs.Pad"},
    {'M',  11,   4,  90, "SC-8850 Map Var #11", "Techno Stack"},
    {'M',  11,   4,  91, "SC-8850 Map Var #11", "Vocorderman"},
    {'M',  11,   4,  94, "SC-8850 Map Var #11", "Breath&Rise"},
    {'M',  11,   4,  95, "SC-8850 Map Var #11", "Bag Sweep"},
    {'M',  11,   4,  96, "SC-8850 Map Var #11", "CP Pad"},
    {'M',  11,   4,  98, "SC-8850 Map Var #11", "Analog Bell"},
    {'M',  11,   4,  99, "SC-8850 Map Var #11", "Tambra Atmos"},
    {'M',  11,   4, 101, "SC-8850 Map Var #11", "LFO Horror"},
    {'M',  11,   4, 102, "SC-8850 Map Var #11", "PanningBrass"},
    {'M',  11,   4, 103, "SC-8850 Map Var #11", "Silky Pad 2"},
    {'M',  11,   4, 109, "SC-8850 Map Var #11", "UillnPipe Or"},
    {'M',  11,   4, 112, "SC-8850 Map Var #11", "St.Gamelan"},
    {'M',  11,   4, 118, "SC-8850 Map Var #11", "606 Tom"},
    {'M',  11,   4, 120, "SC-8850 Map Var #11", "Chord Stroke"},
    {'M',  11,   4, 124, "SC-8850 Map Var #11", "MC-500 Beep"},
    {'M',  11,   4, 125, "SC-8850 Map Var #11", "Perc. Bang"},
    {'M',  12,   0,  16, "SC-8850 Var #12", "Ful Organ 9"},
    {'M',  12,   0,  29, "SC-8850 Var #12", "LP Chorus"},
    {'M',  12,   0,  38, "SC-8850 Var #12", "Kicked TB303"},
    {'M',  12,   0,  39, "SC-8850 Var #12", "Blip Bass :"},
    {'M',  12,   0,  48, "SC-8850 Var #12", "Strings+Horn"},
    {'M',  12,   0,  49, "SC-8850 Var #12", "S.Str+Choir"},
    {'M',  12,   0,  50, "SC-8850 Var #12", "LoFi Strings"},
    {'M',  12,   0,  52, "SC-8850 Var #12", "Boys Choir 2"},
    {'M',  12,   0,  53, "SC-8850 Var #12", "JzVoice Thum"},
    {'M',  12,   0,  55, "SC-8850 Var #12", "Shock Wave"},
    {'M',  12,   0,  61, "SC-8850 Var #12", "Brass sfz 2"},
    {'M',  12,   0,  63, "SC-8850 Var #12", "JP8000SawBrs"},
    {'M',  12,   0,  81, "SC-8850 Var #12", "OB DoubleSaw"},
    {'M',  12,   0,  84, "SC-8850 Var #12", "DanceDst.Gt2"},
    {'M',  12,   0,  89, "SC-8850 Var #12", "Oct.PWM Pad"},
    {'M',  12,   0,  90, "SC-8850 Var #12", "Pulse Stack"},
    {'M',  12,   0,  91, "SC-8850 Var #12", "Holy Voices"},
    {'M',  12,   0,  94, "SC-8850 Var #12", "Tears Voices"},
    {'M',  12,   0,  95, "SC-8850 Var #12", "Sweep Pipe"},
    {'M',  12,   0,  98, "SC-8850 Var #12", "Blow Bell"},
    {'M',  12,   0, 101, "SC-8850 Var #12", "LFO Techno"},
    {'M',  12,   0, 112, "SC-8850 Var #12", "Jang Gu"},
    {'M',  12,   0, 118, "SC-8850 Var #12", "909 Tom"},
    {'M',  12,   0, 120, "SC-8850 Var #12", "Biwa Stroke"},
    {'M',  12,   0, 124, "SC-8850 Var #12", "Scratch 3"},
    {'M',  12,   0, 125, "SC-8850 Var #12", "Burner"},
    {'M',  12,   3,  38, "SC-88Pro Map Var #12", "Kicked TB303"},
    {'M',  12,   3,  39, "SC-88Pro Map Var #12", "Blip Bass :"},
    {'M',  12,   3,  48, "SC-88Pro Map Var #12", "Strings+Horn"},
    {'M',  12,   3,  55, "SC-88Pro Map Var #12", "Shock Wave"},
    {'M',  12,   3, 101, "SC-88Pro Map Var #12", "LFO Techno"},
    {'M',  12,   3, 112, "SC-88Pro Map Var #12", "Jang-Gu"},
    {'M',  12,   3, 118, "SC-88Pro Map Var #12", "909 Tom"},
    {'M',  12,   3, 120, "SC-88Pro Map Var #12", "Biwa Stroke"},
    {'M',  12,   4,  16, "SC-8850 Map Var #12", "Ful Organ 9"},
    {'M',  12,   4,  29, "SC-8850 Map Var #12", "LP Chorus"},
    {'M',  12,   4,  38, "SC-8850 Map Var #12", "Kicked TB303"},
    {'M',  12,   4,  39, "SC-8850 Map Var #12", "Blip Bass :"},
    {'M',  12,   4,  48, "SC-8850 Map Var #12", "Strings+Horn"},
    {'M',  12,   4,  49, "SC-8850 Map Var #12", "S.Str+Choir"},
    {'M',  12,   4,  50, "SC-8850 Map Var #12", "LoFi Strings"},
    {'M',  12,   4,  52, "SC-8850 Map Var #12", "Boys Choir 2"},
    {'M',  12,   4,  53, "SC-8850 Map Var #12", "JzVoice Thum"},
    {'M',  12,   4,  55, "SC-8850 Map Var #12", "Shock Wave"},
    {'M',  12,   4,  61, "SC-8850 Map Var #12", "Brass sfz 2"},
    {'M',  12,   4,  63, "SC-8850 Map Var #12", "JP8000SawBrs"},
    {'M',  12,   4,  81, "SC-8850 Map Var #12", "OB DoubleSaw"},
    {'M',  12,   4,  84, "SC-8850 Map Var #12", "DanceDst.Gt2"},
    {'M',  12,   4,  89, "SC-8850 Map Var #12", "Oct.PWM Pad"},
    {'M',  12,   4,  90, "SC-8850 Map Var #12", "Pulse Stack"},
    {'M',  12,   4,  91, "SC-8850 Map Var #12", "Holy Voices"},
    {'M',  12,   4,  94, "SC-8850 Map Var #12", "Tears Voices"},
    {'M',  12,   4,  95, "SC-8850 Map Var #12", "Sweep Pipe"},
    {'M',  12,   4,  98, "SC-8850 Map Var #12", "Blow Bell"},
    {'M',  12,   4, 101, "SC-8850 Map Var #12", "LFO Techno"},
    {'M',  12,   4, 112, "SC-8850 Map Var #12", "Jang Gu"},
    {'M',  12,   4, 118, "SC-8850 Map Var #12", "909 Tom"},
    {'M',  12,   4, 120, "SC-8850 Map Var #12", "Biwa Stroke"},
    {'M',  12,   4, 124, "SC-8850 Map Var #12", "Scratch 3"},
    {'M',  12,   4, 125, "SC-8850 Map Var #12", "Burner"},
    {'M',  13,   0,  38, "SC-8850 Var #13", "TB303 Saw Bs"},
    {'M',  13,   0,  39, "SC-8850 Var #13", "RubberBass 1"},
    {'M',  13,   0,  48, "SC-8850 Var #13", "Str.+Flute"},
    {'M',  13,   0,  49, "SC-8850 Var #13", "S.Str+Choir2"},
    {'M',  13,   0,  52, "SC-8850 Var #13", "St.BoysChoir"},
    {'M',  13,   0,  55, "SC-8850 Var #13", "Bounce Hit"},
    {'M',  13,   0,  81, "SC-8850 Var #13", "JP DoubleSaw"},
    {'M',  13,   0,  89, "SC-8850 Var #13", "JP Soft Pad"},
    {'M',  13,   0,  90, "SC-8850 Var #13", "TwinOct.Rave"},
    {'M',  13,   0,  95, "SC-8850 Var #13", "Sweep Stack"},
    {'M',  13,   0,  98, "SC-8850 Var #13", "Hyper Bell"},
    {'M',  13,   0, 101, "SC-8850 Var #13", "Alternative"},
    {'M',  13,   0, 112, "SC-8850 Var #13", "Jegogan"},
    {'M',  13,   0, 118, "SC-8850 Var #13", "606 Dist.Tom"},
    {'M',  13,   0, 120, "SC-8850 Var #13", "Biwa Tremolo"},
    {'M',  13,   0, 124, "SC-8850 Var #13", "Scratch 4"},
    {'M',  13,   0, 125, "SC-8850 Var #13", "Glass & Glam"},
    {'M',  13,   3,  38, "SC-88Pro Map Var #13", "TB303 Saw Bs"},
    {'M',  13,   3,  39, "SC-88Pro Map Var #13", "RubberBass 1"},
    {'M',  13,   3, 101, "SC-88Pro Map Var #13", "Alternative"},
    {'M',  13,   3, 120, "SC-88Pro Map Var #13", "Biwa Tremolo"},
    {'M',  13,   4,  38, "SC-8850 Map Var #13", "TB303 Saw Bs"},
    {'M',  13,   4,  39, "SC-8850 Map Var #13", "RubberBass 1"},
    {'M',  13,   4,  48, "SC-8850 Map Var #13", "Str.+Flute"},
    {'M',  13,   4,  49, "SC-8850 Map Var #13", "S.Str+Choir2"},
    {'M',  13,   4,  52, "SC-8850 Map Var #13", "St.BoysChoir"},
    {'M',  13,   4,  55, "SC-8850 Map Var #13", "Bounce Hit"},
    {'M',  13,   4,  81, "SC-8850 Map Var #13", "JP DoubleSaw"},
    {'M',  13,   4,  89, "SC-8850 Map Var #13", "JP Soft Pad"},
    {'M',  13,   4,  90, "SC-8850 Map Var #13", "TwinOct.Rave"},
    {'M',  13,   4,  95, "SC-8850 Map Var #13", "Sweep Stack"},
    {'M',  13,   4,  98, "SC-8850 Map Var #13", "Hyper Bell"},
    {'M',  13,   4, 101, "SC-8850 Map Var #13", "Alternative"},
    {'M',  13,   4, 112, "SC-8850 Map Var #13", "Jegogan"},
    {'M',  13,   4, 118, "SC-8850 Map Var #13", "606 Dist.Tom"},
    {'M',  13,   4, 120, "SC-8850 Map Var #13", "Biwa Tremolo"},
    {'M',  13,   4, 124, "SC-8850 Map Var #13", "Scratch 4"},
    {'M',  13,   4, 125, "SC-8850 Map Var #13", "Glass & Glam"},
    {'M',  14,   0,  38, "SC-8850 Var #14", "Rubber303 Bs"},
    {'M',  14,   0,  39, "SC-8850 Var #14", "Syn Bell Bs"},
    {'M',  14,   0,  48, "SC-8850 Var #14", "Choir Str.2"},
    {'M',  14,   0,  52, "SC-8850 Var #14", "Rich Choir"},
    {'M',  14,   0,  55, "SC-8850 Var #14", "Drill Hit"},
    {'M',  14,   0,  61, "SC-8850 Var #14", "FatPop Brass"},
    {'M',  14,   0,  81, "SC-8850 Var #14", "FatSawLead 2"},
    {'M',  14,   0,  90, "SC-8850 Var #14", "Oct.Rave"},
    {'M',  14,   0,  95, "SC-8850 Var #14", "Deep Sweep"},
    {'M',  14,   0, 101, "SC-8850 Var #14", "UFO FX"},
    {'M',  14,   0, 112, "SC-8850 Var #14", "Jublag"},
    {'M',  14,   0, 124, "SC-8850 Var #14", "Scratch 5"},
    {'M',  14,   0, 125, "SC-8850 Var #14", "Ice Ring"},
    {'M',  14,   3,  38, "SC-88Pro Map Var #14", "Rubber303 Bs"},
    {'M',  14,   3, 101, "SC-88Pro Map Var #14", "UFO FX"},
    {'M',  14,   4,  38, "SC-8850 Map Var #14", "Rubber303 Bs"},
    {'M',  14,   4,  39, "SC-8850 Map Var #14", "Syn Bell Bs"},
    {'M',  14,   4,  48, "SC-8850 Map Var #14", "Choir Str."},
    {'M',  14,   4,  52, "SC-8850 Map Var #14", "Rich Choir"},
    {'M',  14,   4,  55, "SC-8850 Map Var #14", "Drill Hit"},
    {'M',  14,   4,  61, "SC-8850 Map Var #14", "FatPop Brass"},
    {'M',  14,   4,  81, "SC-8850 Map Var #14", "FatSawLead 2"},
    {'M',  14,   4,  90, "SC-8850 Map Var #14", "Oct.Rave"},
    {'M',  14,   4,  95, "SC-8850 Map Var #14", "Deep Sweep"},
    {'M',  14,   4, 101, "SC-8850 Map Var #14", "UFO FX"},
    {'M',  14,   4, 112, "SC-8850 Map Var #14", "Jublag"},
    {'M',  14,   4, 124, "SC-8850 Map Var #14", "Scratch 5"},
    {'M',  14,   4, 125, "SC-8850 Map Var #14", "Ice Ring"},
    {'M',  15,   0,  38, "SC-8850 Var #15", "Reso 303 Bs"},
    {'M',  15,   0,  39, "SC-8850 Var #15", "Odd Bass"},
    {'M',  15,   0,  48, "SC-8850 Var #15", "Choir Str.3"},
    {'M',  15,   0,  55, "SC-8850 Var #15", "Thrill Hit"},
    {'M',  15,   0,  81, "SC-8850 Var #15", "JP SuperSaw"},
    {'M',  15,   0,  90, "SC-8850 Var #15", "Happy Synth"},
    {'M',  15,   0,  95, "SC-8850 Var #15", "Stray Pad"},
    {'M',  15,   0, 101, "SC-8850 Var #15", "Gargle Man"},
    {'M',  15,   0, 112, "SC-8850 Var #15", "Pemade"},
    {'M',  15,   0, 124, "SC-8850 Var #15", "Scratch 6"},
    {'M',  15,   0, 125, "SC-8850 Var #15", "Over Blow"},
    {'M',  15,   3,  38, "SC-88Pro Map Var #15", "Reso 303 Bs"},
    {'M',  15,   3, 101, "SC-88Pro Map Var #15", "Gargle Man"},
    {'M',  15,   4,  38, "SC-8850 Map Var #15", "Reso 303 Bs"},
    {'M',  15,   4,  39, "SC-8850 Map Var #15", "Odd Bass"},
    {'M',  15,   4,  48, "SC-8850 Map Var #15", "Choir Str.2"},
    {'M',  15,   4,  55, "SC-8850 Map Var #15", "Thrill Hit"},
    {'M',  15,   4,  81, "SC-8850 Map Var #15", "JP SuperSaw"},
    {'M',  15,   4,  90, "SC-8850 Map Var #15", "Happy Synth"},
    {'M',  15,   4,  95, "SC-8850 Map Var #15", "Stray Pad"},
    {'M',  15,   4, 101, "SC-8850 Map Var #15", "Gargle Man"},
    {'M',  15,   4, 112, "SC-8850 Map Var #15", "Pemade"},
    {'M',  15,   4, 124, "SC-8850 Map Var #15", "Scratch 6"},
    {'M',  15,   4, 125, "SC-8850 Map Var #15", "Over Blow"},
    {'M',  16,   0,   0, "SC-8850 Var #16", "European Pf"},
    {'M',  16,   0,   1, "SC-8850 Var #16", "Dance Piano"},
    {'M',  16,   0,   4, "SC-8850 Var #16", "FM+SA EP"},
    {'M',  16,   0,   5, "SC-8850 Var #16", "St.FM EP"},
    {'M',  16,   0,   6, "SC-8850 Var #16", "Harpsi.w"},
    {'M',  16,   0,   7, "SC-8850 Var #16", "Reso Clav."},
    {'M',  16,   0,  12, "SC-8850 Var #16", "Barafon"},
    {'M',  16,   0,  14, "SC-8850 Var #16", "Tubularbellw"},
    {'M',  16,   0,  15, "SC-8850 Var #16", "Zither 1"},
    {'M',  16,   0,  16, "SC-8850 Var #16", "60's Organ 1"},
    {'M',  16,   0,  18, "SC-8850 Var #16", "Rotary Org.S"},
    {'M',  16,   0,  19, "SC-8850 Var #16", "Church Org.3"},
    {'M',  16,   0,  20, "SC-8850 Var #16", "Puff Organ"},
    {'M',  16,   0,  21, "SC-8850 Var #16", "Cho. Accord"},
    {'M',  16,   0,  23, "SC-8850 Var #16", "Bandoneon 3"},
    {'M',  16,   0,  24, "SC-8850 Var #16", "Nylon Gt.o"},
    {'M',  16,   0,  25, "SC-8850 Var #16", "Mandolin"},
    {'M',  16,   0,  27, "SC-8850 Var #16", "TC FrontPick"},
    {'M',  16,   0,  28, "SC-8850 Var #16", "Funk Gt.2"},
    {'M',  16,   0,  30, "SC-8850 Var #16", "Power Guitar"},
    {'M',  16,   0,  31, "SC-8850 Var #16", "Ac.Gt.Harmnx"},
    {'M',  16,   0,  32, "SC-8850 Var #16", "Bass + OHH"},
    {'M',  16,   0,  33, "SC-8850 Var #16", "F.Bass/Harm."},
    {'M',  16,   0,  34, "SC-8850 Var #16", "P.Bass/Harm."},
    {'M',  16,   0,  38, "SC-8850 Var #16", "Reso SH Bass"},
    {'M',  16,   0,  39, "SC-8850 Var #16", "RubberBass 2"},
    {'M',  16,   0,  45, "SC-8850 Var #16", "Solo Spic."},
    {'M',  16,   0,  46, "SC-8850 Var #16", "Synth Harp"},
    {'M',  16,   0,  48, "SC-8850 Var #16", "St. Strings"},
    {'M',  16,   0,  50, "SC-8850 Var #16", "High Strings"},
    {'M',  16,   0,  52, "SC-8850 Var #16", "Choir Hahs"},
    {'M',  16,   0,  53, "SC-8850 Var #16", "VoiceLah Fem"},
    {'M',  16,   0,  54, "SC-8850 Var #16", "VP330 Choir"},
    {'M',  16,   0,  55, "SC-8850 Var #16", "Lo Fi Rave"},
    {'M',  16,   0,  56, "SC-8850 Var #16", "4th Trumpets"},
    {'M',  16,   0,  57, "SC-8850 Var #16", "Euphonium"},
    {'M',  16,   0,  60, "SC-8850 Var #16", "Synth Horn"},
    {'M',  16,   0,  61, "SC-8850 Var #16", "Brass Fall"},
    {'M',  16,   0,  62, "SC-8850 Var #16", "Oct SynBrass"},
    {'M',  16,   0,  63, "SC-8850 Var #16", "Velo Brass 1"},
    {'M',  16,   0,  65, "SC-8850 Var #16", "AltoSax + Tp"},
    {'M',  16,   0,  68, "SC-8850 Var #16", "Multi Reed"},
    {'M',  16,   0,  71, "SC-8850 Var #16", "Multi Wind"},
    {'M',  16,   0,  72, "SC-8850 Var #16", "Di"},
    {'M',  16,   0,  73, "SC-8850 Var #16", "Tron Flute"},
    {'M',  16,   0,  75, "SC-8850 Var #16", "Zampona"},
    {'M',  16,   0,  80, "SC-8850 Var #16", "P5 Square"},
    {'M',  16,   0,  81, "SC-8850 Var #16", "Waspy Synth"},
    {'M',  16,   0,  84, "SC-8850 Var #16", "P5 Sync Lead"},
    {'M',  16,   0,  90, "SC-8850 Var #16", "ForwardSweep"},
    {'M',  16,   0,  98, "SC-8850 Var #16", "Choral Bells"},
    {'M',  16,   0, 101, "SC-8850 Var #16", "Sweep FX"},
    {'M',  16,   0, 103, "SC-8850 Var #16", "New Century"},
    {'M',  16,   0, 104, "SC-8850 Var #16", "Tamboura"},
    {'M',  16,   0, 105, "SC-8850 Var #16", "Gopichant"},
    {'M',  16,   0, 107, "SC-8850 Var #16", "Kanoon"},
    {'M',  16,   0, 111, "SC-8850 Var #16", "Hichiriki"},
    {'M',  16,   0, 112, "SC-8850 Var #16", "RAMA Cymbal"},
    {'M',  16,   0, 113, "SC-8850 Var #16", "Tambourine"},
    {'M',  16,   0, 115, "SC-8850 Var #16", "Angklung"},
    {'M',  16,   0, 116, "SC-8850 Var #16", "Jungle BD"},
    {'M',  16,   0, 117, "SC-8850 Var #16", "Rash SD"},
    {'M',  16,   0, 119, "SC-8850 Var #16", "Rev.Kick 1"},
    {'M',  16,   0, 120, "SC-8850 Var #16", "A.Bs.Nz Menu"},
    {'M',  16,   0, 122, "SC-8850 Var #16", "Pink Noise"},
    {'M',  16,   0, 124, "SC-8850 Var #16", "Scratch 7"},
    {'M',  16,   0, 125, "SC-8850 Var #16", "Crack Bottle"},
    {'M',  16,   0, 126, "SC-8850 Var #16", "Voice One"},
    {'M',  16,   1,   0, "SC-55 Map Var #16", "Piano 1d"},
    {'M',  16,   1,   4, "SC-55 Map Var #16", "E.Piano 1v"},
    {'M',  16,   1,   5, "SC-55 Map Var #16", "E.Piano 2v"},
    {'M',  16,   1,   6, "SC-55 Map Var #16", "Harpsi.w"},
    {'M',  16,   1,  16, "SC-55 Map Var #16", "60's Organ1"},
    {'M',  16,   1,  19, "SC-55 Map Var #16", "Church Org.3"},
    {'M',  16,   1,  24, "SC-55 Map Var #16", "Nylon Gt.o"},
    {'M',  16,   1,  25, "SC-55 Map Var #16", "Mandolin"},
    {'M',  16,   1,  28, "SC-55 Map Var #16", "Funk Gt.2"},
    {'M',  16,   1,  39, "SC-55 Map Var #16", "Rubber Bass"},
    {'M',  16,   1,  62, "SC-55 Map Var #16", "Analog Brs1"},
    {'M',  16,   1,  63, "SC-55 Map Var #16", "Analog Brs2"},
    {'M',  16,   2,   0, "SC-88 Map Var #16", "Piano 1d"},
    {'M',  16,   2,   4, "SC-88 Map Var #16", "FM+SA EP"},
    {'M',  16,   2,   5, "SC-88 Map Var #16", "St.FM EP"},
    {'M',  16,   2,   6, "SC-88 Map Var #16", "Harpsi.w"},
    {'M',  16,   2,  12, "SC-88 Map Var #16", "Barafon"},
    {'M',  16,   2,  16, "SC-88 Map Var #16", "60'sOrgan 1"},
    {'M',  16,   2,  18, "SC-88 Map Var #16", "RotaryOrg.S"},
    {'M',  16,   2,  19, "SC-88 Map Var #16", "ChurchOrg.3"},
    {'M',  16,   2,  24, "SC-88 Map Var #16", "Nylon Gt.o"},
    {'M',  16,   2,  25, "SC-88 Map Var #16", "Mandolin"},
    {'M',  16,   2,  28, "SC-88 Map Var #16", "Funk Gt.2"},
    {'M',  16,   2,  30, "SC-88 Map Var #16", "PowerGuitar"},
    {'M',  16,   2,  31, "SC-88 Map Var #16", "Ac.Gt.Harm."},
    {'M',  16,   2,  38, "SC-88 Map Var #16", "Reso SHBass"},
    {'M',  16,   2,  39, "SC-88 Map Var #16", "Rubber Bass"},
    {'M',  16,   2,  48, "SC-88 Map Var #16", "St.Strings"},
    {'M',  16,   2,  55, "SC-88 Map Var #16", "Lo Fi Rave"},
    {'M',  16,   2,  60, "SC-88 Map Var #16", "Horn Orch"},
    {'M',  16,   2,  61, "SC-88 Map Var #16", "Brass Fall"},
    {'M',  16,   2,  62, "SC-88 Map Var #16", "OctaveBrass"},
    {'M',  16,   2,  63, "SC-88 Map Var #16", "VeloBrass 1"},
    {'M',  16,   2,  81, "SC-88 Map Var #16", "Waspy Synth"},
    {'M',  16,   2,  98, "SC-88 Map Var #16", "ChoralBells"},
    {'M',  16,   2, 104, "SC-88 Map Var #16", "Tamboura"},
    {'M',  16,   2, 105, "SC-88 Map Var #16", "Gopichant"},
    {'M',  16,   2, 107, "SC-88 Map Var #16", "Kanoon"},
    {'M',  16,   2, 111, "SC-88 Map Var #16", "Hichiriki"},
    {'M',  16,   2, 112, "SC-88 Map Var #16", "RAMA Cymbal"},
    {'M',  16,   2, 119, "SC-88 Map Var #16", "Rev.Kick 1"},
    {'M',  16,   3,   0, "SC-88Pro Map Var #16", "European Pf"},
    {'M',  16,   3,   1, "SC-88Pro Map Var #16", "Dance Piano"},
    {'M',  16,   3,   4, "SC-88Pro Map Var #16", "FM+SA EP"},
    {'M',  16,   3,   5, "SC-88Pro Map Var #16", "St.FM EP"},
    {'M',  16,   3,   6, "SC-88Pro Map Var #16", "Harpsi.w"},
    {'M',  16,   3,   7, "SC-88Pro Map Var #16", "Reso Clav."},
    {'M',  16,   3,  12, "SC-88Pro Map Var #16", "Barafon"},
    {'M',  16,   3,  15, "SC-88Pro Map Var #16", "Zither 1"},
    {'M',  16,   3,  16, "SC-88Pro Map Var #16", "60's Organ 1"},
    {'M',  16,   3,  18, "SC-88Pro Map Var #16", "Rotary Org.S"},
    {'M',  16,   3,  19, "SC-88Pro Map Var #16", "Church Org.3"},
    {'M',  16,   3,  21, "SC-88Pro Map Var #16", "Cho. Accord"},
    {'M',  16,   3,  23, "SC-88Pro Map Var #16", "Bandoneon 3"},
    {'M',  16,   3,  24, "SC-88Pro Map Var #16", "Nylon Gt.o"},
    {'M',  16,   3,  25, "SC-88Pro Map Var #16", "Mandolin"},
    {'M',  16,   3,  27, "SC-88Pro Map Var #16", "TC FrontPick"},
    {'M',  16,   3,  28, "SC-88Pro Map Var #16", "Funk Gt.2"},
    {'M',  16,   3,  30, "SC-88Pro Map Var #16", "Power Guitar"},
    {'M',  16,   3,  31, "SC-88Pro Map Var #16", "Ac.Gt.Harmnx"},
    {'M',  16,   3,  32, "SC-88Pro Map Var #16", "Bass + OHH"},
    {'M',  16,   3,  33, "SC-88Pro Map Var #16", "F.Bass/Harm."},
    {'M',  16,   3,  34, "SC-88Pro Map Var #16", "P.Bass/Harm."},
    {'M',  16,   3,  38, "SC-88Pro Map Var #16", "Reso SH Bass"},
    {'M',  16,   3,  39, "SC-88Pro Map Var #16", "RubberBass 2"},
    {'M',  16,   3,  45, "SC-88Pro Map Var #16", "Solo Spic."},
    {'M',  16,   3,  46, "SC-88Pro Map Var #16", "Synth Harp"},
    {'M',  16,   3,  48, "SC-88Pro Map Var #16", "St. Strings"},
    {'M',  16,   3,  50, "SC-88Pro Map Var #16", "High Strings"},
    {'M',  16,   3,  52, "SC-88Pro Map Var #16", "Choir Hahs"},
    {'M',  16,   3,  54, "SC-88Pro Map Var #16", "VP330 Choir"},
    {'M',  16,   3,  55, "SC-88Pro Map Var #16", "Lo Fi Rave"},
    {'M',  16,   3,  56, "SC-88Pro Map Var #16", "4th Trumpets"},
    {'M',  16,   3,  60, "SC-88Pro Map Var #16", "Synth Horn"},
    {'M',  16,   3,  61, "SC-88Pro Map Var #16", "Brass Fall"},
    {'M',  16,   3,  62, "SC-88Pro Map Var #16", "Oct SynBrass"},
    {'M',  16,   3,  63, "SC-88Pro Map Var #16", "Velo Brass 1"},
    {'M',  16,   3,  65, "SC-88Pro Map Var #16", "AltoSax + Tp"},
    {'M',  16,   3,  68, "SC-88Pro Map Var #16", "Multi Reed"},
    {'M',  16,   3,  71, "SC-88Pro Map Var #16", "Multi Wind"},
    {'M',  16,   3,  72, "SC-88Pro Map Var #16", "Di"},
    {'M',  16,   3,  73, "SC-88Pro Map Var #16", "Tron Flute"},
    {'M',  16,   3,  75, "SC-88Pro Map Var #16", "Zampona"},
    {'M',  16,   3,  80, "SC-88Pro Map Var #16", "P5 Square"},
    {'M',  16,   3,  81, "SC-88Pro Map Var #16", "Waspy Synth"},
    {'M',  16,   3,  84, "SC-88Pro Map Var #16", "P5 Sync Lead"},
    {'M',  16,   3,  98, "SC-88Pro Map Var #16", "Choral Bells"},
    {'M',  16,   3, 101, "SC-88Pro Map Var #16", "Sweep FX"},
    {'M',  16,   3, 103, "SC-88Pro Map Var #16", "New Century"},
    {'M',  16,   3, 104, "SC-88Pro Map Var #16", "Tamboura"},
    {'M',  16,   3, 105, "SC-88Pro Map Var #16", "Gopichant"},
    {'M',  16,   3, 107, "SC-88Pro Map Var #16", "Kanoon"},
    {'M',  16,   3, 111, "SC-88Pro Map Var #16", "Hichiriki"},
    {'M',  16,   3, 112, "SC-88Pro Map Var #16", "RAMA Cymbal"},
    {'M',  16,   3, 113, "SC-88Pro Map Var #16", "Tambourine"},
    {'M',  16,   3, 115, "SC-88Pro Map Var #16", "Angklung"},
    {'M',  16,   3, 116, "SC-88Pro Map Var #16", "Jungle BD"},
    {'M',  16,   3, 117, "SC-88Pro Map Var #16", "Rash SD"},
    {'M',  16,   3, 119, "SC-88Pro Map Var #16", "Rev.Kick 1"},
    {'M',  16,   3, 122, "SC-88Pro Map Var #16", "Pink Noise"},
    {'M',  16,   3, 126, "SC-88Pro Map Var #16", "Voice One"},
    {'M',  16,   4,   0, "SC-8850 Map Var #16", "European Pf"},
    {'M',  16,   4,   1, "SC-8850 Map Var #16", "Dance Piano"},
    {'M',  16,   4,   4, "SC-8850 Map Var #16", "FM+SA EP"},
    {'M',  16,   4,   5, "SC-8850 Map Var #16", "St.FM EP"},
    {'M',  16,   4,   6, "SC-8850 Map Var #16", "Harpsi.w"},
    {'M',  16,   4,   7, "SC-8850 Map Var #16", "Reso Clav."},
    {'M',  16,   4,  12, "SC-8850 Map Var #16", "Barafon"},
    {'M',  16,   4,  14, "SC-8850 Map Var #16", "Tubularbellw"},
    {'M',  16,   4,  15, "SC-8850 Map Var #16", "Zither 1"},
    {'M',  16,   4,  16, "SC-8850 Map Var #16", "60's Organ 1"},
    {'M',  16,   4,  18, "SC-8850 Map Var #16", "Rotary Org.S"},
    {'M',  16,   4,  19, "SC-8850 Map Var #16", "Church Org.3"},
    {'M',  16,   4,  20, "SC-8850 Map Var #16", "Puff Organ"},
    {'M',  16,   4,  21, "SC-8850 Map Var #16", "Cho. Accord"},
    {'M',  16,   4,  23, "SC-8850 Map Var #16", "Bandoneon 3"},
    {'M',  16,   4,  24, "SC-8850 Map Var #16", "Nylon Gt.o"},
    {'M',  16,   4,  25, "SC-8850 Map Var #16", "Mandolin"},
    {'M',  16,   4,  27, "SC-8850 Map Var #16", "TC FrontPick"},
    {'M',  16,   4,  28, "SC-8850 Map Var #16", "Funk Gt.2"},
    {'M',  16,   4,  30, "SC-8850 Map Var #16", "Power Guitar"},
    {'M',  16,   4,  31, "SC-8850 Map Var #16", "Ac.Gt.Harmnx"},
    {'M',  16,   4,  32, "SC-8850 Map Var #16", "Bass + OHH"},
    {'M',  16,   4,  33, "SC-8850 Map Var #16", "F.Bass/Harm."},
    {'M',  16,   4,  34, "SC-8850 Map Var #16", "P.Bass/Harm."},
    {'M',  16,   4,  38, "SC-8850 Map Var #16", "Reso SH Bass"},
    {'M',  16,   4,  39, "SC-8850 Map Var #16", "RubberBass 2"},
    {'M',  16,   4,  45, "SC-8850 Map Var #16", "Solo Spic."},
    {'M',  16,   4,  46, "SC-8850 Map Var #16", "Synth Harp"},
    {'M',  16,   4,  48, "SC-8850 Map Var #16", "St. Strings"},
    {'M',  16,   4,  50, "SC-8850 Map Var #16", "High Strings"},
    {'M',  16,   4,  52, "SC-8850 Map Var #16", "Choir Hahs"},
    {'M',  16,   4,  53, "SC-8850 Map Var #16", "VoiceLah Fem"},
    {'M',  16,   4,  54, "SC-8850 Map Var #16", "VP330 Choir"},
    {'M',  16,   4,  55, "SC-8850 Map Var #16", "Lo Fi Rave"},
    {'M',  16,   4,  56, "SC-8850 Map Var #16", "4th Trumpets"},
    {'M',  16,   4,  57, "SC-8850 Map Var #16", "Euphonium"},
    {'M',  16,   4,  60, "SC-8850 Map Var #16", "Synth Horn"},
    {'M',  16,   4,  61, "SC-8850 Map Var #16", "Brass Fall"},
    {'M',  16,   4,  62, "SC-8850 Map Var #16", "Oct SynBrass"},
    {'M',  16,   4,  63, "SC-8850 Map Var #16", "Velo Brass 1"},
    {'M',  16,   4,  65, "SC-8850 Map Var #16", "AltoSax + Tp"},
    {'M',  16,   4,  68, "SC-8850 Map Var #16", "Multi Reed"},
    {'M',  16,   4,  71, "SC-8850 Map Var #16", "Multi Wind"},
    {'M',  16,   4,  72, "SC-8850 Map Var #16", "Di"},
    {'M',  16,   4,  73, "SC-8850 Map Var #16", "Tron Flute"},
    {'M',  16,   4,  75, "SC-8850 Map Var #16", "Zampona"},
    {'M',  16,   4,  80, "SC-8850 Map Var #16", "P5 Square"},
    {'M',  16,   4,  81, "SC-8850 Map Var #16", "Waspy Synth"},
    {'M',  16,   4,  84, "SC-8850 Map Var #16", "P5 Sync Lead"},
    {'M',  16,   4,  90, "SC-8850 Map Var #16", "ForwardSweep"},
    {'M',  16,   4,  98, "SC-8850 Map Var #16", "Choral Bells"},
    {'M',  16,   4, 101, "SC-8850 Map Var #16", "Sweep FX"},
    {'M',  16,   4, 103, "SC-8850 Map Var #16", "New Century"},
    {'M',  16,   4, 104, "SC-8850 Map Var #16", "Tamboura"},
    {'M',  16,   4, 105, "SC-8850 Map Var #16", "Gopichant"},
    {'M',  16,   4, 107, "SC-8850 Map Var #16", "Kanoon"},
    {'M',  16,   4, 111, "SC-8850 Map Var #16", "Hichiriki"},
    {'M',  16,   4, 112, "SC-8850 Map Var #16", "RAMA Cymbal"},
    {'M',  16,   4, 113, "SC-8850 Map Var #16", "Tambourine"},
    {'M',  16,   4, 115, "SC-8850 Map Var #16", "Angklung"},
    {'M',  16,   4, 116, "SC-8850 Map Var #16", "Jungle BD"},
    {'M',  16,   4, 117, "SC-8850 Map Var #16", "Rash SD"},
    {'M',  16,   4, 119, "SC-8850 Map Var #16", "Rev.Kick 1"},
    {'M',  16,   4, 120, "SC-8850 Map Var #16", "A.Bs.Nz Menu"},
    {'M',  16,   4, 122, "SC-8850 Map Var #16", "Pink Noise"},
    {'M',  16,   4, 124, "SC-8850 Map Var #16", "Scratch 7"},
    {'M',  16,   4, 125, "SC-8850 Map Var #16", "Crack Bottle"},
    {'M',  16,   4, 126, "SC-8850 Map Var #16", "Voice One"},
    {'M',  17,   0,   4, "SC-8850 Var #17", "Dist E.Piano"},
    {'M',  17,   0,   7, "SC-8850 Var #17", "Phase Clav"},
    {'M',  17,   0,  12, "SC-8850 Var #17", "Barafon 2"},
    {'M',  17,   0,  15, "SC-8850 Var #17", "Zither 2"},
    {'M',  17,   0,  16, "SC-8850 Var #17", "60's Organ 2"},
    {'M',  17,   0,  18, "SC-8850 Var #17", "Rock Organ 1"},
    {'M',  17,   0,  25, "SC-8850 Var #17", "Mandolin 2"},
    {'M',  17,   0,  27, "SC-8850 Var #17", "TC Rear Pick"},
    {'M',  17,   0,  30, "SC-8850 Var #17", "Power Gt.2"},
    {'M',  17,   0,  38, "SC-8850 Var #17", "TB303 Sqr Bs"},
    {'M',  17,   0,  39, "SC-8850 Var #17", "SH101 Bass 1"},
    {'M',  17,   0,  45, "SC-8850 Var #17", "StringsSpic."},
    {'M',  17,   0,  48, "SC-8850 Var #17", "St.Strings 2"},
    {'M',  17,   0,  50, "SC-8850 Var #17", "Hybrid Str."},
    {'M',  17,   0,  53, "SC-8850 Var #17", "ChorusLahFem"},
    {'M',  17,   0,  54, "SC-8850 Var #17", "Vinyl Choir"},
    {'M',  17,   0,  55, "SC-8850 Var #17", "Techno Hit"},
    {'M',  17,   0,  61, "SC-8850 Var #17", "Trumpet Fall"},
    {'M',  17,   0,  62, "SC-8850 Var #17", "Hybrid Brass"},
    {'M',  17,   0,  63, "SC-8850 Var #17", "Transbrass"},
    {'M',  17,   0,  65, "SC-8850 Var #17", "Sax Section"},
    {'M',  17,   0,  71, "SC-8850 Var #17", "Quad Wind"},
    {'M',  17,   0,  73, "SC-8850 Var #17", "Indian Flute"},
    {'M',  17,   0,  75, "SC-8850 Var #17", "Zampona Atk"},
    {'M',  17,   0,  80, "SC-8850 Var #17", "OB Square"},
    {'M',  17,   0,  81, "SC-8850 Var #17", "PM Lead"},
    {'M',  17,   0,  84, "SC-8850 Var #17", "Fat SyncLead"},
    {'M',  17,   0,  90, "SC-8850 Var #17", "ReverseSweep"},
    {'M',  17,   0,  98, "SC-8850 Var #17", "Air Bells"},
    {'M',  17,   0, 101, "SC-8850 Var #17", "LM Has Come"},
    {'M',  17,   0, 103, "SC-8850 Var #17", "7th Atmos."},
    {'M',  17,   0, 112, "SC-8850 Var #17", "Kajar"},
    {'M',  17,   0, 115, "SC-8850 Var #17", "Angkl Rhythm"},
    {'M',  17,   0, 116, "SC-8850 Var #17", "Techno BD"},
    {'M',  17,   0, 117, "SC-8850 Var #17", "House SD"},
    {'M',  17,   0, 119, "SC-8850 Var #17", "Rev.ConBD"},
    {'M',  17,   0, 120, "SC-8850 Var #17", "D.Gt.Nz Menu"},
    {'M',  17,   0, 122, "SC-8850 Var #17", "White Noise"},
    {'M',  17,   0, 125, "SC-8850 Var #17", "Pour Bottle"},
    {'M',  17,   0, 126, "SC-8850 Var #17", "Voice Two"},
    {'M',  17,   2,  12, "SC-88 Map Var #17", "Barafon 2"},
    {'M',  17,   2,  16, "SC-88 Map Var #17", "60'sOrgan 2"},
    {'M',  17,   2,  30, "SC-88 Map Var #17", "Power Gt.2"},
    {'M',  17,   2,  39, "SC-88 Map Var #17", "SH101Bass 1"},
    {'M',  17,   2,  63, "SC-88 Map Var #17", "VeloBrass 2"},
    {'M',  17,   2,  98, "SC-88 Map Var #17", "Air Bells"},
    {'M',  17,   2, 119, "SC-88 Map Var #17", "Rev.ConBD"},
    {'M',  17,   3,   4, "SC-88Pro Map Var #17", "Dist E.Piano"},
    {'M',  17,   3,  12, "SC-88Pro Map Var #17", "Barafon 2"},
    {'M',  17,   3,  15, "SC-88Pro Map Var #17", "Zither 2"},
    {'M',  17,   3,  16, "SC-88Pro Map Var #17", "60's Organ 2"},
    {'M',  17,   3,  18, "SC-88Pro Map Var #17", "Rock Organ 1"},
    {'M',  17,   3,  25, "SC-88Pro Map Var #17", "Mandolin 2"},
    {'M',  17,   3,  27, "SC-88Pro Map Var #17", "TC Rear Pick"},
    {'M',  17,   3,  30, "SC-88Pro Map Var #17", "Power Gt.2"},
    {'M',  17,   3,  38, "SC-88Pro Map Var #17", "TB303 Sqr Bs"},
    {'M',  17,   3,  39, "SC-88Pro Map Var #17", "SH101 Bass 1"},
    {'M',  17,   3,  50, "SC-88Pro Map Var #17", "Hybrid Str."},
    {'M',  17,   3,  54, "SC-88Pro Map Var #17", "Vinyl Choir"},
    {'M',  17,   3,  55, "SC-88Pro Map Var #17", "Techno Hit"},
    {'M',  17,   3,  61, "SC-88Pro Map Var #17", "Trumpet Fall"},
    {'M',  17,   3,  62, "SC-88Pro Map Var #17", "Hybrid Brass"},
    {'M',  17,   3,  63, "SC-88Pro Map Var #17", "Transbrass"},
    {'M',  17,   3,  75, "SC-88Pro Map Var #17", "Zampona Atk"},
    {'M',  17,   3,  80, "SC-88Pro Map Var #17", "OB Square"},
    {'M',  17,   3,  81, "SC-88Pro Map Var #17", "PM Lead"},
    {'M',  17,   3,  84, "SC-88Pro Map Var #17", "Fat SyncLead"},
    {'M',  17,   3,  98, "SC-88Pro Map Var #17", "Air Bells"},
    {'M',  17,   3, 103, "SC-88Pro Map Var #17", "7th Atmos."},
    {'M',  17,   3, 115, "SC-88Pro Map Var #17", "Angkl Rhythm"},
    {'M',  17,   3, 116, "SC-88Pro Map Var #17", "Techno BD"},
    {'M',  17,   3, 117, "SC-88Pro Map Var #17", "House SD"},
    {'M',  17,   3, 119, "SC-88Pro Map Var #17", "Rev.ConBD"},
    {'M',  17,   3, 122, "SC-88Pro Map Var #17", "White Noise"},
    {'M',  17,   3, 126, "SC-88Pro Map Var #17", "Voice Two"},
    {'M',  17,   4,   4, "SC-8850 Map Var #17", "Dist E.Piano"},
    {'M',  17,   4,   7, "SC-8850 Map Var #17", "Phase Clav"},
    {'M',  17,   4,  12, "SC-8850 Map Var #17", "Barafon 2"},
    {'M',  17,   4,  15, "SC-8850 Map Var #17", "Zither 2"},
    {'M',  17,   4,  16, "SC-8850 Map Var #17", "60's Organ 2"},
    {'M',  17,   4,  18, "SC-8850 Map Var #17", "Rock Organ 1"},
    {'M',  17,   4,  25, "SC-8850 Map Var #17", "Mandolin 2"},
    {'M',  17,   4,  27, "SC-8850 Map Var #17", "TC Rear Pick"},
    {'M',  17,   4,  30, "SC-8850 Map Var #17", "Power Gt.2"},
    {'M',  17,   4,  38, "SC-8850 Map Var #17", "TB303 Sqr Bs"},
    {'M',  17,   4,  39, "SC-8850 Map Var #17", "SH101 Bass 1"},
    {'M',  17,   4,  45, "SC-8850 Map Var #17", "StringsSpic."},
    {'M',  17,   4,  48, "SC-8850 Map Var #17", "St.Strings 2"},
    {'M',  17,   4,  50, "SC-8850 Map Var #17", "Hybrid Str."},
    {'M',  17,   4,  53, "SC-8850 Map Var #17", "ChorusLahFem"},
    {'M',  17,   4,  54, "SC-8850 Map Var #17", "Vinyl Choir"},
    {'M',  17,   4,  55, "SC-8850 Map Var #17", "Techno Hit"},
    {'M',  17,   4,  61, "SC-8850 Map Var #17", "Trumpet Fall"},
    {'M',  17,   4,  62, "SC-8850 Map Var #17", "Hybrid Brass"},
    {'M',  17,   4,  63, "SC-8850 Map Var #17", "Transbrass"},
    {'M',  17,   4,  65, "SC-8850 Map Var #17", "Sax Section"},
    {'M',  17,   4,  71, "SC-8850 Map Var #17", "Quad Wind"},
    {'M',  17,   4,  73, "SC-8850 Map Var #17", "Indian Flute"},
    {'M',  17,   4,  75, "SC-8850 Map Var #17", "Zampona Atk"},
    {'M',  17,   4,  80, "SC-8850 Map Var #17", "OB Square"},
    {'M',  17,   4,  81, "SC-8850 Map Var #17", "PM Lead"},
    {'M',  17,   4,  84, "SC-8850 Map Var #17", "Fat SyncLead"},
    {'M',  17,   4,  90, "SC-8850 Map Var #17", "ReverseSweep"},
    {'M',  17,   4,  98, "SC-8850 Map Var #17", "Air Bells"},
    {'M',  17,   4, 101, "SC-8850 Map Var #17", "LM Has Come"},
    {'M',  17,   4, 103, "SC-8850 Map Var #17", "7th Atmos."},
    {'M',  17,   4, 112, "SC-8850 Map Var #17", "Kajar"},
    {'M',  17,   4, 115, "SC-8850 Map Var #17", "Angkl Rhythm"},
    {'M',  17,   4, 116, "SC-8850 Map Var #17", "Techno BD"},
    {'M',  17,   4, 117, "SC-8850 Map Var #17", "House SD"},
    {'M',  17,   4, 119, "SC-8850 Map Var #17", "Rev.ConBD"},
    {'M',  17,   4, 120, "SC-8850 Map Var #17", "D.Gt.Nz Menu"},
    {'M',  17,   4, 122, "SC-8850 Map Var #17", "White Noise"},
    {'M',  17,   4, 125, "SC-8850 Map Var #17", "Pour Bottle"},
    {'M',  17,   4, 126, "SC-8850 Map Var #17", "Voice Two"},
    {'M',  18,   0,  16, "SC-8850 Var #18", "60's Organ 3"},
    {'M',  18,   0,  18, "SC-8850 Var #18", "Rock Organ 2"},
    {'M',  18,   0,  25, "SC-8850 Var #18", "MandolinTrem"},
    {'M',  18,   0,  27, "SC-8850 Var #18", "TC Clean ff"},
    {'M',  18,   0,  30, "SC-8850 Var #18", "5th Dist."},
    {'M',  18,   0,  38, "SC-8850 Var #18", "TB303 DistBs"},
    {'M',  18,   0,  39, "SC-8850 Var #18", "SH101 Bass 2"},
    {'M',  18,   0,  48, "SC-8850 Var #18", "St.Strings 3"},
    {'M',  18,   0,  53, "SC-8850 Var #18", "VoiceLuh Fem"},
    {'M',  18,   0,  54, "SC-8850 Var #18", "JX8P Vox"},
    {'M',  18,   0,  55, "SC-8850 Var #18", "Dist. Hit"},
    {'M',  18,   0,  62, "SC-8850 Var #18", "OctSynBrass2"},
    {'M',  18,   0,  80, "SC-8850 Var #18", "JP-8 Square"},
    {'M',  18,   0,  81, "SC-8850 Var #18", "CS Saw Lead"},
    {'M',  18,   0,  84, "SC-8850 Var #18", "Rock Lead"},
    {'M',  18,   0,  98, "SC-8850 Var #18", "Bell Harp"},
    {'M',  18,   0, 101, "SC-8850 Var #18", "FallinInsect"},
    {'M',  18,   0, 103, "SC-8850 Var #18", "Galaxy Way"},
    {'M',  18,   0, 112, "SC-8850 Var #18", "Kelontuk"},
    {'M',  18,   0, 116, "SC-8850 Var #18", "Bounce"},
    {'M',  18,   0, 117, "SC-8850 Var #18", "Jungle SD"},
    {'M',  18,   0, 120, "SC-8850 Var #18", "E.Gt.NzMenu1"},
    {'M',  18,   0, 125, "SC-8850 Var #18", "Soda"},
    {'M',  18,   0, 126, "SC-8850 Var #18", "Voice Three"},
    {'M',  18,   2,  16, "SC-88 Map Var #18", "60'sOrgan 3"},
    {'M',  18,   2,  30, "SC-88 Map Var #18", "5th Dist."},
    {'M',  18,   2,  39, "SC-88 Map Var #18", "SH101Bass 2"},
    {'M',  18,   2,  98, "SC-88 Map Var #18", "Bell Harp"},
    {'M',  18,   3,  16, "SC-88Pro Map Var #18", "60's Organ 3"},
    {'M',  18,   3,  18, "SC-88Pro Map Var #18", "Rock Organ 2"},
    {'M',  18,   3,  25, "SC-88Pro Map Var #18", "MandolinTrem"},
    {'M',  18,   3,  27, "SC-88Pro Map Var #18", "TC Clean ff"},
    {'M',  18,   3,  30, "SC-88Pro Map Var #18", "5th Dist."},
    {'M',  18,   3,  38, "SC-88Pro Map Var #18", "TB303 DistBs"},
    {'M',  18,   3,  39, "SC-88Pro Map Var #18", "SH101 Bass 2"},
    {'M',  18,   3,  55, "SC-88Pro Map Var #18", "Dist. Hit"},
    {'M',  18,   3,  80, "SC-88Pro Map Var #18", "JP-8 Square"},
    {'M',  18,   3,  81, "SC-88Pro Map Var #18", "CS Saw Lead"},
    {'M',  18,   3,  84, "SC-88Pro Map Var #18", "Rock Lead"},
    {'M',  18,   3,  98, "SC-88Pro Map Var #18", "Bell Harp"},
    {'M',  18,   3, 103, "SC-88Pro Map Var #18", "Galaxy Way"},
    {'M',  18,   3, 116, "SC-88Pro Map Var #18", "Bounce"},
    {'M',  18,   3, 117, "SC-88Pro Map Var #18", "Jungle SD"},
    {'M',  18,   3, 126, "SC-88Pro Map Var #18", "Voice Three"},
    {'M',  18,   4,  16, "SC-8850 Map Var #18", "60's Organ 3"},
    {'M',  18,   4,  18, "SC-8850 Map Var #18", "Rock Organ 2"},
    {'M',  18,   4,  25, "SC-8850 Map Var #18", "MandolinTrem"},
    {'M',  18,   4,  27, "SC-8850 Map Var #18", "TC Clean ff"},
    {'M',  18,   4,  30, "SC-8850 Map Var #18", "5th Dist."},
    {'M',  18,   4,  38, "SC-8850 Map Var #18", "TB303 DistBs"},
    {'M',  18,   4,  39, "SC-8850 Map Var #18", "SH101 Bass 2"},
    {'M',  18,   4,  48, "SC-8850 Map Var #18", "St.Strings 3"},
    {'M',  18,   4,  53, "SC-8850 Map Var #18", "VoiceLuh Fem"},
    {'M',  18,   4,  54, "SC-8850 Map Var #18", "JX8P Vox"},
    {'M',  18,   4,  55, "SC-8850 Map Var #18", "Dist. Hit"},
    {'M',  18,   4,  62, "SC-8850 Map Var #18", "OctSynBrass2"},
    {'M',  18,   4,  80, "SC-8850 Map Var #18", "JP-8 Square"},
    {'M',  18,   4,  81, "SC-8850 Map Var #18", "CS Saw Lead"},
    {'M',  18,   4,  84, "SC-8850 Map Var #18", "Rock Lead"},
    {'M',  18,   4,  98, "SC-8850 Map Var #18", "Bell Harp"},
    {'M',  18,   4, 101, "SC-8850 Map Var #18", "FallinInsect"},
    {'M',  18,   4, 103, "SC-8850 Map Var #18", "Galaxy Way"},
    {'M',  18,   4, 112, "SC-8850 Map Var #18", "Kelontuk"},
    {'M',  18,   4, 116, "SC-8850 Map Var #18", "Bounce"},
    {'M',  18,   4, 117, "SC-8850 Map Var #18", "Jungle SD"},
    {'M',  18,   4, 120, "SC-8850 Map Var #18", "E.Gt.NzMenu1"},
    {'M',  18,   4, 125, "SC-8850 Map Var #18", "Soda"},
    {'M',  18,   4, 126, "SC-8850 Map Var #18", "Voice Three"},
    {'M',  19,   0,  16, "SC-8850 Var #19", "Farf Organ"},
    {'M',  19,   0,  27, "SC-8850 Var #19", "TC Clean 2:"},
    {'M',  19,   0,  38, "SC-8850 Var #19", "Clavi Bass"},
    {'M',  19,   0,  39, "SC-8850 Var #19", "Smooth Bass"},
    {'M',  19,   0,  48, "SC-8850 Var #19", "Orchestra 3"},
    {'M',  19,   0,  53, "SC-8850 Var #19", "ChorusLuhFem"},
    {'M',  19,   0,  54, "SC-8850 Var #19", "Analog Voice"},
    {'M',  19,   0,  55, "SC-8850 Var #19", "Bam Hit"},
    {'M',  19,   0,  62, "SC-8850 Var #19", "BPF Brass"},
    {'M',  19,   0,  80, "SC-8850 Var #19", "Dist Square"},
    {'M',  19,   0,  84, "SC-8850 Var #19", "5th DecaSync"},
    {'M',  19,   0,  98, "SC-8850 Var #19", "Gamelimba"},
    {'M',  19,   0, 101, "SC-8850 Var #19", "LFO Oct.Rave"},
    {'M',  19,   0, 103, "SC-8850 Var #19", "Rising OSC."},
    {'M',  19,   0, 107, "SC-8850 Var #19", "Kanoon+Choir"},
    {'M',  19,   0, 112, "SC-8850 Var #19", "Kelontuk Mt"},
    {'M',  19,   0, 117, "SC-8850 Var #19", "909 SD"},
    {'M',  19,   0, 120, "SC-8850 Var #19", "E.Gt.NzMenu2"},
    {'M',  19,   0, 125, "SC-8850 Var #19", "Open CD Tray"},
    {'M',  19,   0, 126, "SC-8850 Var #19", "Voice Tah"},
    {'M',  19,   2,  39, "SC-88 Map Var #19", "Smooth Bass"},
    {'M',  19,   2,  98, "SC-88 Map Var #19", "Gamelimba"},
    {'M',  19,   3,  16, "SC-88Pro Map Var #19", "Farf Organ"},
    {'M',  19,   3,  27, "SC-88Pro Map Var #19", "TC Clean 2:"},
    {'M',  19,   3,  39, "SC-88Pro Map Var #19", "Smooth Bass"},
    {'M',  19,   3,  55, "SC-88Pro Map Var #19", "Bam Hit"},
    {'M',  19,   3,  84, "SC-88Pro Map Var #19", "5th DecaSync"},
    {'M',  19,   3,  98, "SC-88Pro Map Var #19", "Gamelimba"},
    {'M',  19,   3, 107, "SC-88Pro Map Var #19", "Kanoon+Choir"},
    {'M',  19,   3, 117, "SC-88Pro Map Var #19", "909 SD"},
    {'M',  19,   3, 126, "SC-88Pro Map Var #19", "Voice Tah"},
    {'M',  19,   4,  16, "SC-8850 Map Var #19", "Farf Organ"},
    {'M',  19,   4,  27, "SC-8850 Map Var #19", "TC Clean 2:"},
    {'M',  19,   4,  38, "SC-8850 Map Var #19", "Clavi Bass"},
    {'M',  19,   4,  39, "SC-8850 Map Var #19", "Smooth Bass"},
    {'M',  19,   4,  48, "SC-8850 Map Var #19", "Orchestra 3"},
    {'M',  19,   4,  53, "SC-8850 Map Var #19", "ChorusLuhFem"},
    {'M',  19,   4,  54, "SC-8850 Map Var #19", "Analog Voice"},
    {'M',  19,   4,  55, "SC-8850 Map Var #19", "Bam Hit"},
    {'M',  19,   4,  62, "SC-8850 Map Var #19", "BPF Brass"},
    {'M',  19,   4,  80, "SC-8850 Map Var #19", "Dist Square"},
    {'M',  19,   4,  84, "SC-8850 Map Var #19", "5th DecaSync"},
    {'M',  19,   4,  98, "SC-8850 Map Var #19", "Gamelimba"},
    {'M',  19,   4, 101, "SC-8850 Map Var #19", "LFO Oct.Rave"},
    {'M',  19,   4, 103, "SC-8850 Map Var #19", "Rising OSC."},
    {'M',  19,   4, 107, "SC-8850 Map Var #19", "Kanoon+Choir"},
    {'M',  19,   4, 112, "SC-8850 Map Var #19", "Kelontuk Mt"},
    {'M',  19,   4, 117, "SC-8850 Map Var #19", "909 SD"},
    {'M',  19,   4, 120, "SC-8850 Map Var #19", "E.Gt.NzMenu2"},
    {'M',  19,   4, 125, "SC-8850 Map Var #19", "Open CD Tray"},
    {'M',  19,   4, 126, "SC-8850 Map Var #19", "Voice Tah"},
    {'M',  20,   0,  27, "SC-8850 Var #20", "LP Rear Pick"},
    {'M',  20,   0,  38, "SC-8850 Var #20", "Hammer"},
    {'M',  20,   0,  39, "SC-8850 Var #20", "SH101 Bass 3"},
    {'M',  20,   0,  48, "SC-8850 Var #20", "Orchestra 4"},
    {'M',  20,   0,  53, "SC-8850 Var #20", "VoiceLan Fem"},
    {'M',  20,   0,  55, "SC-8850 Var #20", "Bit Hit"},
    {'M',  20,   0,  80, "SC-8850 Var #20", "303SquarDst1"},
    {'M',  20,   0,  84, "SC-8850 Var #20", "Dirty Sync"},
    {'M',  20,   0,  98, "SC-8850 Var #20", "JUNO Bell"},
    {'M',  20,   0, 101, "SC-8850 Var #20", "Just Before"},
    {'M',  20,   0, 112, "SC-8850 Var #20", "Kelontuk Sid"},
    {'M',  20,   0, 120, "SC-8850 Var #20", "G.StrokeMenu"},
    {'M',  20,   0, 125, "SC-8850 Var #20", "Audio Switch"},
    {'M',  20,   0, 126, "SC-8850 Var #20", "Voice Whey"},
    {'M',  20,   3,  39, "SC-88Pro Map Var #20", "SH101 Bass 3"},
    {'M',  20,   3,  55, "SC-88Pro Map Var #20", "Bit Hit"},
    {'M',  20,   3,  84, "SC-88Pro Map Var #20", "Dirty Sync"},
    {'M',  20,   3,  98, "SC-88Pro Map Var #20", "JUNO Bell"},
    {'M',  20,   3, 126, "SC-88Pro Map Var #20", "Voice Whey"},
    {'M',  20,   4,  27, "SC-8850 Map Var #20", "LP Rear Pick"},
    {'M',  20,   4,  38, "SC-8850 Map Var #20", "Hammer"},
    {'M',  20,   4,  39, "SC-8850 Map Var #20", "SH101 Bass 3"},
    {'M',  20,   4,  48, "SC-8850 Map Var #20", "Orchestra 4"},
    {'M',  20,   4,  53, "SC-8850 Map Var #20", "VoiceLan Fem"},
    {'M',  20,   4,  55, "SC-8850 Map Var #20", "Bit Hit"},
    {'M',  20,   4,  80, "SC-8850 Map Var #20", "303SquarDst1"},
    {'M',  20,   4,  84, "SC-8850 Map Var #20", "Dirty Sync"},
    {'M',  20,   4,  98, "SC-8850 Map Var #20", "JUNO Bell"},
    {'M',  20,   4, 101, "SC-8850 Map Var #20", "Just Before"},
    {'M',  20,   4, 112, "SC-8850 Map Var #20", "Kelontuk Sid"},
    {'M',  20,   4, 120, "SC-8850 Map Var #20", "G.StrokeMenu"},
    {'M',  20,   4, 125, "SC-8850 Map Var #20", "Audio Switch"},
    {'M',  20,   4, 126, "SC-8850 Map Var #20", "Voice Whey"},
    {'M',  21,   0,  27, "SC-8850 Var #21", "LP Rear 2"},
    {'M',  21,   0,  38, "SC-8850 Var #21", "Jungle Bass"},
    {'M',  21,   0,  39, "SC-8850 Var #21", "Spike Bass"},
    {'M',  21,   0,  53, "SC-8850 Var #21", "ChorusLanFem"},
    {'M',  21,   0,  55, "SC-8850 Var #21", "Bim Hit"},
    {'M',  21,   0,  80, "SC-8850 Var #21", "303SquarDst2"},
    {'M',  21,   0,  84, "SC-8850 Var #21", "DualSyncLead"},
    {'M',  21,   0,  98, "SC-8850 Var #21", "JP Bell"},
    {'M',  21,   0, 101, "SC-8850 Var #21", "RND Fl.Chord"},
    {'M',  21,   0, 112, "SC-8850 Var #21", "Kopyak Op"},
    {'M',  21,   0, 120, "SC-8850 Var #21", "Gt.SlideMenu"},
    {'M',  21,   0, 125, "SC-8850 Var #21", "Key Typing"},
    {'M',  21,   3,  39, "SC-88Pro Map Var #21", "Spike Bass"},
    {'M',  21,   3,  55, "SC-88Pro Map Var #21", "Bim Hit"},
    {'M',  21,   4,  27, "SC-8850 Map Var #21", "LP Rear 2"},
    {'M',  21,   4,  38, "SC-8850 Map Var #21", "Jungle Bass"},
    {'M',  21,   4,  39, "SC-8850 Map Var #21", "Spike Bass"},
    {'M',  21,   4,  53, "SC-8850 Map Var #21", "ChorusLanFem"},
    {'M',  21,   4,  55, "SC-8850 Map Var #21", "Bim Hit"},
    {'M',  21,   4,  80, "SC-8850 Map Var #21", "303SquarDst2"},
    {'M',  21,   4,  84, "SC-8850 Map Var #21", "DualSyncLead"},
    {'M',  21,   4,  98, "SC-8850 Map Var #21", "JP Bell"},
    {'M',  21,   4, 101, "SC-8850 Map Var #21", "RND Fl.Chord"},
    {'M',  21,   4, 112, "SC-8850 Map Var #21", "Kopyak Op"},
    {'M',  21,   4, 120, "SC-8850 Map Var #21", "Gt.SlideMenu"},
    {'M',  21,   4, 125, "SC-8850 Map Var #21", "Key Typing"},
    {'M',  22,   0,  27, "SC-8850 Var #22", "LP RearAtack"},
    {'M',  22,   0,  38, "SC-8850 Var #22", "Square Bass"},
    {'M',  22,   0,  39, "SC-8850 Var #22", "House Bass:"},
    {'M',  22,   0,  53, "SC-8850 Var #22", "VoiceAah Fem"},
    {'M',  22,   0,  55, "SC-8850 Var #22", "Technorg Hit"},
    {'M',  22,   0,  80, "SC-8850 Var #22", "303 Mix Sqr"},
    {'M',  22,   0,  84, "SC-8850 Var #22", "LA Brass Ld"},
    {'M',  22,   0,  98, "SC-8850 Var #22", "Pizz Bell"},
    {'M',  22,   0, 101, "SC-8850 Var #22", "RandomEnding"},
    {'M',  22,   0, 112, "SC-8850 Var #22", "Kopyak Mt"},
    {'M',  22,   0, 120, "SC-8850 Var #22", "A.Bs.Mute Nz"},
    {'M',  22,   0, 125, "SC-8850 Var #22", "SL 1"},
    {'M',  22,   0, 126, "SC-8850 Var #22", "Voice Kikit"},
    {'M',  22,   3,  39, "SC-88Pro Map Var #22", "House Bass:"},
    {'M',  22,   3,  55, "SC-88Pro Map Var #22", "Technorg Hit"},
    {'M',  22,   4,  27, "SC-8850 Map Var #22", "LP RearAtack"},
    {'M',  22,   4,  38, "SC-8850 Map Var #22", "Square Bass"},
    {'M',  22,   4,  39, "SC-8850 Map Var #22", "House Bass:"},
    {'M',  22,   4,  53, "SC-8850 Map Var #22", "VoiceAah Fem"},
    {'M',  22,   4,  55, "SC-8850 Map Var #22", "Technorg Hit"},
    {'M',  22,   4,  80, "SC-8850 Map Var #22", "303 Mix Sqr"},
    {'M',  22,   4,  84, "SC-8850 Map Var #22", "LA Brass Ld"},
    {'M',  22,   4,  98, "SC-8850 Map Var #22", "Pizz Bell"},
    {'M',  22,   4, 101, "SC-8850 Map Var #22", "RandomEnding"},
    {'M',  22,   4, 112, "SC-8850 Map Var #22", "Kopyak Mt"},
    {'M',  22,   4, 120, "SC-8850 Map Var #22", "A.Bs.Mute Nz"},
    {'M',  22,   4, 125, "SC-8850 Map Var #22", "SL 1"},
    {'M',  22,   4, 126, "SC-8850 Map Var #22", "Voice Kikit"},
    {'M',  23,   0,  27, "SC-8850 Var #23", "Mid Tone GTR"},
    {'M',  23,   0,  38, "SC-8850 Var #23", "Square Bass2"},
    {'M',  23,   0,  39, "SC-8850 Var #23", "KG Bass"},
    {'M',  23,   0,  53, "SC-8850 Var #23", "VoiceUuh Fem"},
    {'M',  23,   0,  55, "SC-8850 Var #23", "Rave Hit"},
    {'M',  23,   0,  80, "SC-8850 Var #23", "Dual Sqr&Saw"},
    {'M',  23,   0,  98, "SC-8850 Var #23", "Bottom Bell"},
    {'M',  23,   0, 101, "SC-8850 Var #23", "Random Sine"},
    {'M',  23,   0, 112, "SC-8850 Var #23", "Ceng Ceng"},
    {'M',  23,   0, 120, "SC-8850 Var #23", "A.Bs.TouchNz"},
    {'M',  23,   0, 125, "SC-8850 Var #23", "SL 2"},
    {'M',  23,   0, 126, "SC-8850 Var #23", "Voice ComeOn"},
    {'M',  23,   3,  39, "SC-88Pro Map Var #23", "KG Bass"},
    {'M',  23,   3,  55, "SC-88Pro Map Var #23", "Rave Hit"},
    {'M',  23,   4,  27, "SC-8850 Map Var #23", "Mid Tone GTR"},
    {'M',  23,   4,  38, "SC-8850 Map Var #23", "Square Bass2"},
    {'M',  23,   4,  39, "SC-8850 Map Var #23", "KG Bass"},
    {'M',  23,   4,  53, "SC-8850 Map Var #23", "VoiceUuh Fem"},
    {'M',  23,   4,  55, "SC-8850 Map Var #23", "Rave Hit"},
    {'M',  23,   4,  80, "SC-8850 Map Var #23", "Dual Sqr&Saw"},
    {'M',  23,   4,  98, "SC-8850 Map Var #23", "Bottom Bell"},
    {'M',  23,   4, 101, "SC-8850 Map Var #23", "Random Sine"},
    {'M',  23,   4, 112, "SC-8850 Map Var #23", "Ceng Ceng"},
    {'M',  23,   4, 120, "SC-8850 Map Var #23", "A.Bs.TouchNz"},
    {'M',  23,   4, 125, "SC-8850 Map Var #23", "SL 2"},
    {'M',  23,   4, 126, "SC-8850 Map Var #23", "Voice ComeOn"},
    {'M',  24,   0,   0, "SC-8850 Var #24", "Piano + Str."},
    {'M',  24,   0,   4, "SC-8850 Var #24", "Wurly"},
    {'M',  24,   0,   5, "SC-8850 Var #24", "Hard FM EP"},
    {'M',  24,   0,   6, "SC-8850 Var #24", "Harpsi.o"},
    {'M',  24,   0,   7, "SC-8850 Var #24", "Clav.o"},
    {'M',  24,   0,  12, "SC-8850 Var #24", "Log drum"},
    {'M',  24,   0,  15, "SC-8850 Var #24", "Dulcimer"},
    {'M',  24,   0,  16, "SC-8850 Var #24", "Cheese Organ"},
    {'M',  24,   0,  18, "SC-8850 Var #24", "Rotary Org.F"},
    {'M',  24,   0,  19, "SC-8850 Var #24", "Organ Flute"},
    {'M',  24,   0,  21, "SC-8850 Var #24", "Hard Accord"},
    {'M',  24,   0,  24, "SC-8850 Var #24", "Velo Harmnix"},
    {'M',  24,   0,  27, "SC-8850 Var #24", "Chung Ruan"},
    {'M',  24,   0,  28, "SC-8850 Var #24", "Jazz Man"},
    {'M',  24,   0,  30, "SC-8850 Var #24", "Rock Rhythm"},
    {'M',  24,   0,  31, "SC-8850 Var #24", "E.Bass Harm."},
    {'M',  24,   0,  38, "SC-8850 Var #24", "Arpeggio Bs"},
    {'M',  24,   0,  39, "SC-8850 Var #24", "Sync Bass"},
    {'M',  24,   0,  46, "SC-8850 Var #24", "Yang Qin"},
    {'M',  24,   0,  48, "SC-8850 Var #24", "Velo Strings"},
    {'M',  24,   0,  50, "SC-8850 Var #24", "Tron Strings"},
    {'M',  24,   0,  52, "SC-8850 Var #24", "Chorus Lahs"},
    {'M',  24,   0,  53, "SC-8850 Var #24", "Fem Lah&Lan"},
    {'M',  24,   0,  55, "SC-8850 Var #24", "Strings Hit"},
    {'M',  24,   0,  56, "SC-8850 Var #24", "Bright Tp."},
    {'M',  24,   0,  60, "SC-8850 Var #24", "F.Horn Rip"},
    {'M',  24,   0,  61, "SC-8850 Var #24", "Octave Brass"},
    {'M',  24,   0,  75, "SC-8850 Var #24", "Tin Whistle"},
    {'M',  24,   0,  80, "SC-8850 Var #24", "Pulse Lead"},
    {'M',  24,   0,  81, "SC-8850 Var #24", "MG Saw 1"},
    {'M',  24,   0,  84, "SC-8850 Var #24", "JUNO Sub Osc"},
    {'M',  24,   0,  90, "SC-8850 Var #24", "Minor Rave"},
    {'M',  24,   0, 101, "SC-8850 Var #24", "EatingFilter"},
    {'M',  24,   0, 105, "SC-8850 Var #24", "Oud"},
    {'M',  24,   0, 107, "SC-8850 Var #24", "Oct Harp"},
    {'M',  24,   0, 111, "SC-8850 Var #24", "Mizmar"},
    {'M',  24,   0, 112, "SC-8850 Var #24", "Reyoung"},
    {'M',  24,   0, 115, "SC-8850 Var #24", "Finger Snaps"},
    {'M',  24,   0, 116, "SC-8850 Var #24", "KendangWadon"},
    {'M',  24,   0, 117, "SC-8850 Var #24", "Jngl SD Roll"},
    {'M',  24,   0, 119, "SC-8850 Var #24", "Rev.Tom 1"},
    {'M',  24,   0, 120, "SC-8850 Var #24", "A.Bs.AtackNz"},
    {'M',  24,   0, 125, "SC-8850 Var #24", "Car Engine 2"},
    {'M',  24,   0, 126, "SC-8850 Var #24", "Voice Aou"},
    {'M',  24,   1,   4, "SC-55 Map Var #24", "60's E.Piano"},
    {'M',  24,   1,   6, "SC-55 Map Var #24", "Harpsi.o"},
    {'M',  24,   2,   4, "SC-88 Map Var #24", "60'sE.Piano"},
    {'M',  24,   2,   5, "SC-88 Map Var #24", "Hard FM EP"},
    {'M',  24,   2,   6, "SC-88 Map Var #24", "Harpsi.o"},
    {'M',  24,   2,  12, "SC-88 Map Var #24", "Log Drum"},
    {'M',  24,   2,  16, "SC-88 Map Var #24", "Cheese Organ"},
    {'M',  24,   2,  18, "SC-88 Map Var #24", "RotaryOrg.F"},
    {'M',  24,   2,  19, "SC-88 Map Var #24", "Organ Flute"},
    {'M',  24,   2,  24, "SC-88 Map Var #24", "VeloHarmnix"},
    {'M',  24,   2,  30, "SC-88 Map Var #24", "Rock Rhythm"},
    {'M',  24,   2,  48, "SC-88 Map Var #24", "VeloStrings"},
    {'M',  24,   2,  56, "SC-88 Map Var #24", "Bright Tp."},
    {'M',  24,   2, 105, "SC-88 Map Var #24", "Oud"},
    {'M',  24,   2, 119, "SC-88 Map Var #24", "Rev.Tom 1"},
    {'M',  24,   3,   0, "SC-88Pro Map Var #24", "Piano + Str."},
    {'M',  24,   3,   4, "SC-88Pro Map Var #24", "Wurly"},
    {'M',  24,   3,   5, "SC-88Pro Map Var #24", "Hard FM EP"},
    {'M',  24,   3,   6, "SC-88Pro Map Var #24", "Harpsi.o"},
    {'M',  24,   3,   7, "SC-88Pro Map Var #24", "Clav.o"},
    {'M',  24,   3,  12, "SC-88Pro Map Var #24", "Log drum"},
    {'M',  24,   3,  15, "SC-88Pro Map Var #24", "Dulcimer"},
    {'M',  24,   3,  16, "SC-88Pro Map Var #24", "Cheese Organ"},
    {'M',  24,   3,  18, "SC-88Pro Map Var #24", "Rotary Org.F"},
    {'M',  24,   3,  19, "SC-88Pro Map Var #24", "Organ Flute"},
    {'M',  24,   3,  21, "SC-88Pro Map Var #24", "Hard Accord"},
    {'M',  24,   3,  24, "SC-88Pro Map Var #24", "Velo Harmnix"},
    {'M',  24,   3,  30, "SC-88Pro Map Var #24", "Rock Rhythm"},
    {'M',  24,   3,  31, "SC-88Pro Map Var #24", "E.Bass Harm."},
    {'M',  24,   3,  38, "SC-88Pro Map Var #24", "Arpeggio Bs"},
    {'M',  24,   3,  39, "SC-88Pro Map Var #24", "Sync Bass"},
    {'M',  24,   3,  48, "SC-88Pro Map Var #24", "Velo Strings"},
    {'M',  24,   3,  50, "SC-88Pro Map Var #24", "Tron Strings"},
    {'M',  24,   3,  52, "SC-88Pro Map Var #24", "Chorus Lahs"},
    {'M',  24,   3,  55, "SC-88Pro Map Var #24", "Strings Hit"},
    {'M',  24,   3,  56, "SC-88Pro Map Var #24", "Bright Tp."},
    {'M',  24,   3,  60, "SC-88Pro Map Var #24", "F.Horn Rip"},
    {'M',  24,   3,  61, "SC-88Pro Map Var #24", "Octave Brass"},
    {'M',  24,   3,  80, "SC-88Pro Map Var #24", "Pulse Lead"},
    {'M',  24,   3,  81, "SC-88Pro Map Var #24", "MG Saw 1"},
    {'M',  24,   3,  84, "SC-88Pro Map Var #24", "JUNO Sub Osc"},
    {'M',  24,   3, 105, "SC-88Pro Map Var #24", "Oud"},
    {'M',  24,   3, 107, "SC-88Pro Map Var #24", "Oct Harp"},
    {'M',  24,   3, 111, "SC-88Pro Map Var #24", "Mizmar"},
    {'M',  24,   3, 115, "SC-88Pro Map Var #24", "Finger Snaps"},
    {'M',  24,   3, 119, "SC-88Pro Map Var #24", "Rev.Tom 1"},
    {'M',  24,   4,   0, "SC-8850 Map Var #24", "Piano + Str."},
    {'M',  24,   4,   4, "SC-8850 Map Var #24", "Wurly"},
    {'M',  24,   4,   5, "SC-8850 Map Var #24", "Hard FM EP"},
    {'M',  24,   4,   6, "SC-8850 Map Var #24", "Harpsi.o"},
    {'M',  24,   4,   7, "SC-8850 Map Var #24", "Clav.o"},
    {'M',  24,   4,  12, "SC-8850 Map Var #24", "Log drum"},
    {'M',  24,   4,  15, "SC-8850 Map Var #24", "Dulcimer"},
    {'M',  24,   4,  16, "SC-8850 Map Var #24", "Cheese Organ"},
    {'M',  24,   4,  18, "SC-8850 Map Var #24", "Rotary Org.F"},
    {'M',  24,   4,  19, "SC-8850 Map Var #24", "Organ Flute"},
    {'M',  24,   4,  21, "SC-8850 Map Var #24", "Hard Accord"},
    {'M',  24,   4,  24, "SC-8850 Map Var #24", "Velo Harmnix"},
    {'M',  24,   4,  27, "SC-8850 Map Var #24", "Chung Ruan"},
    {'M',  24,   4,  28, "SC-8850 Map Var #24", "Jazz Man"},
    {'M',  24,   4,  30, "SC-8850 Map Var #24", "Rock Rhythm"},
    {'M',  24,   4,  31, "SC-8850 Map Var #24", "E.Bass Harm."},
    {'M',  24,   4,  38, "SC-8850 Map Var #24", "Arpeggio Bs"},
    {'M',  24,   4,  39, "SC-8850 Map Var #24", "Sync Bass"},
    {'M',  24,   4,  46, "SC-8850 Map Var #24", "Yang Qin"},
    {'M',  24,   4,  48, "SC-8850 Map Var #24", "Velo Strings"},
    {'M',  24,   4,  50, "SC-8850 Map Var #24", "Tron Strings"},
    {'M',  24,   4,  52, "SC-8850 Map Var #24", "Chorus Lahs"},
    {'M',  24,   4,  53, "SC-8850 Map Var #24", "Fem Lah&Lan"},
    {'M',  24,   4,  55, "SC-8850 Map Var #24", "Strings Hit"},
    {'M',  24,   4,  56, "SC-8850 Map Var #24", "Bright Tp."},
    {'M',  24,   4,  60, "SC-8850 Map Var #24", "F.Horn Rip"},
    {'M',  24,   4,  61, "SC-8850 Map Var #24", "Octave Brass"},
    {'M',  24,   4,  75, "SC-8850 Map Var #24", "Tin Whistle"},
    {'M',  24,   4,  80, "SC-8850 Map Var #24", "Pulse Lead"},
    {'M',  24,   4,  81, "SC-8850 Map Var #24", "MG Saw 1"},
    {'M',  24,   4,  84, "SC-8850 Map Var #24", "JUNO Sub Osc"},
    {'M',  24,   4,  90, "SC-8850 Map Var #24", "Minor Rave"},
    {'M',  24,   4, 101, "SC-8850 Map Var #24", "EatingFilter"},
    {'M',  24,   4, 105, "SC-8850 Map Var #24", "Oud"},
    {'M',  24,   4, 107, "SC-8850 Map Var #24", "Oct Harp"},
    {'M',  24,   4, 111, "SC-8850 Map Var #24", "Mizmar"},
    {'M',  24,   4, 112, "SC-8850 Map Var #24", "Reyong"},
    {'M',  24,   4, 115, "SC-8850 Map Var #24", "Finger Snaps"},
    {'M',  24,   4, 116, "SC-8850 Map Var #24", "KendangWadon"},
    {'M',  24,   4, 117, "SC-8850 Map Var #24", "Jngl SD Roll"},
    {'M',  24,   4, 119, "SC-8850 Map Var #24", "Rev.Tom 1"},
    {'M',  24,   4, 120, "SC-8850 Map Var #24", "A.Bs.AtackNz"},
    {'M',  24,   4, 125, "SC-8850 Map Var #24", "Car Engine 2"},
    {'M',  24,   4, 126, "SC-8850 Map Var #24", "Voice Aou"},
    {'M',  25,   0,   0, "SC-8850 Var #25", "Piano + Str2"},
    {'M',  25,   0,   4, "SC-8850 Var #25", "Hard Rhodes"},
    {'M',  25,   0,  16, "SC-8850 Var #25", "D-50 Organ"},
    {'M',  25,   0,  21, "SC-8850 Var #25", "Soft Accord"},
    {'M',  25,   0,  27, "SC-8850 Var #25", "Chung Ruan 2"},
    {'M',  25,   0,  30, "SC-8850 Var #25", "Rock Rhythm2"},
    {'M',  25,   0,  39, "SC-8850 Var #25", "MG 5th Bass"},
    {'M',  25,   0,  46, "SC-8850 Var #25", "Yang Qin 2"},
    {'M',  25,   0,  50, "SC-8850 Var #25", "Noiz Strings"},
    {'M',  25,   0,  55, "SC-8850 Var #25", "Stack Hit"},
    {'M',  25,   0,  56, "SC-8850 Var #25", "Warm Tp."},
    {'M',  25,   0,  61, "SC-8850 Var #25", "Brass + Reed"},
    {'M',  25,   0,  75, "SC-8850 Var #25", "TinWhtsle Nm"},
    {'M',  25,   0,  80, "SC-8850 Var #25", "JP8 PulseLd1"},
    {'M',  25,   0,  81, "SC-8850 Var #25", "MG Saw 2"},
    {'M',  25,   0,  84, "SC-8850 Var #25", "2600 Sub Osc"},
    {'M',  25,   0, 101, "SC-8850 Var #25", "Noise&SawHit"},
    {'M',  25,   0, 112, "SC-8850 Var #25", "Kempur"},
    {'M',  25,   0, 116, "SC-8850 Var #25", "Bebarongan"},
    {'M',  25,   0, 119, "SC-8850 Var #25", "Rev.Tom 2"},
    {'M',  25,   0, 120, "SC-8850 Var #25", "TC Up Nz"},
    {'M',  25,   0, 125, "SC-8850 Var #25", "Car Horn"},
    {'M',  25,   0, 126, "SC-8850 Var #25", "Voice Oou"},
    {'M',  25,   2,   4, "SC-88 Map Var #25", "Hard Rhodes"},
    {'M',  25,   2,  30, "SC-88 Map Var #25", "RockRhythm2"},
    {'M',  25,   2,  56, "SC-88 Map Var #25", "Warm Tp."},
    {'M',  25,   2, 119, "SC-88 Map Var #25", "Rev.Tom 2"},
    {'M',  25,   3,   4, "SC-88Pro Map Var #25", "Hard Rhodes"},
    {'M',  25,   3,  16, "SC-88Pro Map Var #25", "D-50 Organ"},
    {'M',  25,   3,  21, "SC-88Pro Map Var #25", "Soft Accord"},
    {'M',  25,   3,  30, "SC-88Pro Map Var #25", "Rock Rhythm2"},
    {'M',  25,   3,  39, "SC-88Pro Map Var #25", "MG 5th Bass"},
    {'M',  25,   3,  50, "SC-88Pro Map Var #25", "Noiz Strings"},
    {'M',  25,   3,  55, "SC-88Pro Map Var #25", "Stack Hit"},
    {'M',  25,   3,  56, "SC-88Pro Map Var #25", "Warm Tp."},
    {'M',  25,   3,  61, "SC-88Pro Map Var #25", "Brass + Reed"},
    {'M',  25,   3,  80, "SC-88Pro Map Var #25", "JP8 PulseLd1"},
    {'M',  25,   3,  81, "SC-88Pro Map Var #25", "MG Saw 2"},
    {'M',  25,   3, 119, "SC-88Pro Map Var #25", "Rev.Tom 2"},
    {'M',  25,   4,   0, "SC-8850 Map Var #25", "Piano + Str2"},
    {'M',  25,   4,   4, "SC-8850 Map Var #25", "Hard Rhodes"},
    {'M',  25,   4,  16, "SC-8850 Map Var #25", "D-50 Organ"},
    {'M',  25,   4,  21, "SC-8850 Map Var #25", "Soft Accord"},
    {'M',  25,   4,  27, "SC-8850 Map Var #25", "Chung Ruan 2"},
    {'M',  25,   4,  30, "SC-8850 Map Var #25", "Rock Rhythm2"},
    {'M',  25,   4,  39, "SC-8850 Map Var #25", "MG 5th Bass"},
    {'M',  25,   4,  46, "SC-8850 Map Var #25", "Yang Qin 2"},
    {'M',  25,   4,  50, "SC-8850 Map Var #25", "Noiz Strings"},
    {'M',  25,   4,  55, "SC-8850 Map Var #25", "Stack Hit"},
    {'M',  25,   4,  56, "SC-8850 Map Var #25", "Warm Tp."},
    {'M',  25,   4,  61, "SC-8850 Map Var #25", "Brass + Reed"},
    {'M',  25,   4,  75, "SC-8850 Map Var #25", "TinWhtsle Nm"},
    {'M',  25,   4,  80, "SC-8850 Map Var #25", "JP8 PulseLd1"},
    {'M',  25,   4,  81, "SC-8850 Map Var #25", "MG Saw 2"},
    {'M',  25,   4,  84, "SC-8850 Map Var #25", "2600 Sub Osc"},
    {'M',  25,   4, 101, "SC-8850 Map Var #25", "Noise&SawHit"},
    {'M',  25,   4, 112, "SC-8850 Map Var #25", "Kempur"},
    {'M',  25,   4, 116, "SC-8850 Map Var #25", "Bebarongan"},
    {'M',  25,   4, 119, "SC-8850 Map Var #25", "Rev.Tom 2"},
    {'M',  25,   4, 120, "SC-8850 Map Var #25", "TC Up Nz"},
    {'M',  25,   4, 125, "SC-8850 Map Var #25", "Car-Horn"},
    {'M',  25,   4, 126, "SC-8850 Map Var #25", "Voice Oou"},
    {'M',  26,   0,   0, "SC-8850 Var #26", "Piano+Choir1"},
    {'M',  26,   0,   4, "SC-8850 Var #26", "MellowRhodes"},
    {'M',  26,   0,  16, "SC-8850 Var #26", "JUNO Organ"},
    {'M',  26,   0,  30, "SC-8850 Var #26", "Dist Rtm GTR"},
    {'M',  26,   0,  39, "SC-8850 Var #26", "RND Bass"},
    {'M',  26,   0,  46, "SC-8850 Var #26", "SynthYangQin"},
    {'M',  26,   0,  55, "SC-8850 Var #26", "Industry Hit"},
    {'M',  26,   0,  56, "SC-8850 Var #26", "Warm Tp.2"},
    {'M',  26,   0,  61, "SC-8850 Var #26", "Fat + Reed"},
    {'M',  26,   0,  75, "SC-8850 Var #26", "TinWhtsle Or"},
    {'M',  26,   0,  80, "SC-8850 Var #26", "JP8 PulseLd2"},
    {'M',  26,   0,  81, "SC-8850 Var #26", "OB Saw 1"},
    {'M',  26,   0,  84, "SC-8850 Var #26", "JP8000Fd Osc"},
    {'M',  26,   0, 101, "SC-8850 Var #26", "Pour Magic"},
    {'M',  26,   0, 116, "SC-8850 Var #26", "Pelegongan"},
    {'M',  26,   0, 119, "SC-8850 Var #26", "Rev.Tom 3"},
    {'M',  26,   0, 120, "SC-8850 Var #26", "TC DownMt.Nz"},
    {'M',  26,   0, 125, "SC-8850 Var #26", "Boeeeen"},
    {'M',  26,   0, 126, "SC-8850 Var #26", "Voice Hie"},
    {'M',  26,   2,   4, "SC-88 Map Var #26", "MellowRhodes"},
    {'M',  26,   3,   4, "SC-88Pro Map Var #26", "MellowRhodes"},
    {'M',  26,   3,  16, "SC-88Pro Map Var #26", "JUNO Organ"},
    {'M',  26,   3,  39, "SC-88Pro Map Var #26", "RND Bass"},
    {'M',  26,   3,  80, "SC-88Pro Map Var #26", "JP8 PulseLd2"},
    {'M',  26,   3,  81, "SC-88Pro Map Var #26", "OB Saw 1"},
    {'M',  26,   4,   0, "SC-8850 Map Var #26", "Piano+Choir1"},
    {'M',  26,   4,   4, "SC-8850 Map Var #26", "MellowRhodes"},
    {'M',  26,   4,  16, "SC-8850 Map Var #26", "JUNO Organ"},
    {'M',  26,   4,  30, "SC-8850 Map Var #26", "Dist Rtm GTR"},
    {'M',  26,   4,  39, "SC-8850 Map Var #26", "RND Bass"},
    {'M',  26,   4,  46, "SC-8850 Map Var #26", "SynthYangQin"},
    {'M',  26,   4,  55, "SC-8850 Map Var #26", "Industry Hit"},
    {'M',  26,   4,  56, "SC-8850 Map Var #26", "Warm Tp.2"},
    {'M',  26,   4,  61, "SC-8850 Map Var #26", "Fat + Reed"},
    {'M',  26,   4,  75, "SC-8850 Map Var #26", "TinWhtsle Or"},
    {'M',  26,   4,  80, "SC-8850 Map Var #26", "JP8 PulseLd2"},
    {'M',  26,   4,  81, "SC-8850 Map Var #26", "OB Saw 1"},
    {'M',  26,   4,  84, "SC-8850 Map Var #26", "JP8000Fd Osc"},
    {'M',  26,   4, 101, "SC-8850 Map Var #26", "Pour Magic"},
    {'M',  26,   4, 116, "SC-8850 Map Var #26", "Pelegongan"},
    {'M',  26,   4, 119, "SC-8850 Map Var #26", "Rev.Tom 3"},
    {'M',  26,   4, 120, "SC-8850 Map Var #26", "TC DownMt.Nz"},
    {'M',  26,   4, 125, "SC-8850 Map Var #26", "Boeeeen"},
    {'M',  26,   4, 126, "SC-8850 Map Var #26", "Voice Hie"},
    {'M',  27,   0,   0, "SC-8850 Var #27", "Piano+Choir2"},
    {'M',  27,   0,  16, "SC-8850 Var #27", "Hybrid Organ"},
    {'M',  27,   0,  39, "SC-8850 Var #27", "WowMG Bass"},
    {'M',  27,   0,  55, "SC-8850 Var #27", "Clap Hit"},
    {'M',  27,   0,  56, "SC-8850 Var #27", "Twin Tp."},
    {'M',  27,   0,  80, "SC-8850 Var #27", "MG Reso. Pls"},
    {'M',  27,   0,  81, "SC-8850 Var #27", "OB Saw 2"},
    {'M',  27,   0, 101, "SC-8850 Var #27", "DancingDrill"},
    {'M',  27,   0, 116, "SC-8850 Var #27", "Dholak 1"},
    {'M',  27,   0, 119, "SC-8850 Var #27", "Rev.Tom 4"},
    {'M',  27,   0, 120, "SC-8850 Var #27", "TC UpMt.Nz"},
    {'M',  27,   0, 125, "SC-8850 Var #27", "R.Crossing"},
    {'M',  27,   3,  16, "SC-88Pro Map Var #27", "Hybrid Organ"},
    {'M',  27,   3,  39, "SC-88Pro Map Var #27", "WowMG Bass"},
    {'M',  27,   3,  80, "SC-88Pro Map Var #27", "MG Reso. Pls"},
    {'M',  27,   3,  81, "SC-88Pro Map Var #27", "OB Saw 2"},
    {'M',  27,   4,   0, "SC-8850 Map Var #27", "Piano+Choir2"},
    {'M',  27,   4,  16, "SC-8850 Map Var #27", "Hybrid Organ"},
    {'M',  27,   4,  39, "SC-8850 Map Var #27", "WowMG Bass"},
    {'M',  27,   4,  55, "SC-8850 Map Var #27", "Clap Hit"},
    {'M',  27,   4,  56, "SC-8850 Map Var #27", "Twin Tp."},
    {'M',  27,   4,  80, "SC-8850 Map Var #27", "MG Reso. Pls"},
    {'M',  27,   4,  81, "SC-8850 Map Var #27", "OB Saw 2"},
    {'M',  27,   4, 101, "SC-8850 Map Var #27", "DancingDrill"},
    {'M',  27,   4, 116, "SC-8850 Map Var #27", "Dholak 1"},
    {'M',  27,   4, 119, "SC-8850 Map Var #27", "Rev.Tom 4"},
    {'M',  27,   4, 120, "SC-8850 Map Var #27", "TC UpMt.Nz"},
    {'M',  27,   4, 125, "SC-8850 Map Var #27", "R.Crossing"},
    {'M',  28,   0,  16, "SC-8850 Var #28", "VS Organ"},
    {'M',  28,   0,  39, "SC-8850 Var #28", "Bubble Bass"},
    {'M',  28,   0,  80, "SC-8850 Var #28", "JP8 PulseLd3"},
    {'M',  28,   0,  81, "SC-8850 Var #28", "D-50 Saw"},
    {'M',  28,   0, 101, "SC-8850 Var #28", "Dirty Stack"},
    {'M',  28,   0, 105, "SC-8850 Var #28", "Oud+Strings"},
    {'M',  28,   0, 116, "SC-8850 Var #28", "Dholak 2"},
    {'M',  28,   0, 120, "SC-8850 Var #28", "TC Down Nz"},
    {'M',  28,   0, 125, "SC-8850 Var #28", "Compresser"},
    {'M',  28,   3,  16, "SC-88Pro Map Var #28", "VS Organ"},
    {'M',  28,   3,  39, "SC-88Pro Map Var #28", "Bubble Bass"},
    {'M',  28,   3,  81, "SC-88Pro Map Var #28", "D-50 Saw"},
    {'M',  28,   3, 105, "SC-88Pro Map Var #28", "Oud+Strings"},
    {'M',  28,   4,  16, "SC-8850 Map Var #28", "VS Organ"},
    {'M',  28,   4,  39, "SC-8850 Map Var #28", "Bubble Bass"},
    {'M',  28,   4,  80, "SC-8850 Map Var #28", "JP8 PulseLd3"},
    {'M',  28,   4,  81, "SC-8850 Map Var #28", "D-50 Saw"},
    {'M',  28,   4, 101, "SC-8850 Map Var #28", "Dirty Stack"},
    {'M',  28,   4, 105, "SC-8850 Map Var #28", "Oud+Strings"},
    {'M',  28,   4, 116, "SC-8850 Map Var #28", "Dholak 2"},
    {'M',  28,   4, 120, "SC-8850 Map Var #28", "TC Down Nz"},
    {'M',  28,   4, 125, "SC-8850 Map Var #28", "Compresser"},
    {'M',  29,   0,  16, "SC-8850 Var #29", "Digi Church"},
    {'M',  29,   0,  39, "SC-8850 Var #29", "Attack Pulse"},
    {'M',  29,   0,  80, "SC-8850 Var #29", "260RingLead"},
    {'M',  29,   0,  81, "SC-8850 Var #29", "SH-101 Saw"},
    {'M',  29,   0, 101, "SC-8850 Var #29", "Big Blue"},
    {'M',  29,   0, 120, "SC-8850 Var #29", "DstGT.Up Nz"},
    {'M',  29,   0, 125, "SC-8850 Var #29", "Sword Boom!"},
    {'M',  29,   3,  16, "SC-88Pro Map Var #29", "Digi Church"},
    {'M',  29,   3,  81, "SC-88Pro Map Var #29", "SH-101 Saw"},
    {'M',  29,   4,  16, "SC-8850 Map Var #29", "Digi Church"},
    {'M',  29,   4,  39, "SC-8850 Map Var #29", "Attack Pulse"},
    {'M',  29,   4,  80, "SC-8850 Map Var #29", "260RingLead"},
    {'M',  29,   4,  81, "SC-8850 Map Var #29", "SH-101 Saw"},
    {'M',  29,   4, 101, "SC-8850 Map Var #29", "Big Blue"},
    {'M',  29,   4, 120, "SC-8850 Map Var #29", "DstGT.Up Nz"},
    {'M',  29,   4, 125, "SC-8850 Map Var #29", "Sword Boom!"},
    {'M',  30,   0,  16, "SC-8850 Var #30", "JX-8P Organ"},
    {'M',  30,   0,  39, "SC-8850 Var #30", "Sync Bass 2"},
    {'M',  30,   0,  80, "SC-8850 Var #30", "303DistLead"},
    {'M',  30,   0,  81, "SC-8850 Var #30", "CS Saw"},
    {'M',  30,   0, 101, "SC-8850 Var #30", "Static Hit"},
    {'M',  30,   0, 120, "SC-8850 Var #30", "DstGT.DwnNz1"},
    {'M',  30,   0, 125, "SC-8850 Var #30", "Sword Cross"},
    {'M',  30,   3,  81, "SC-88Pro Map Var #30", "CS Saw"},
    {'M',  30,   4,  16, "SC-8850 Map Var #30", "JX-8P Organ"},
    {'M',  30,   4,  39, "SC-8850 Map Var #30", "Sync Bass 2"},
    {'M',  30,   4,  80, "SC-8850 Map Var #30", "303DistLead"},
    {'M',  30,   4,  81, "SC-8850 Map Var #30", "CS Saw"},
    {'M',  30,   4, 101, "SC-8850 Map Var #30", "Static Hit"},
    {'M',  30,   4, 120, "SC-8850 Map Var #30", "DstGT.DwnNz1"},
    {'M',  30,   4, 125, "SC-8850 Map Var #30", "Sword Cross"},
    {'M',  31,   0,  16, "SC-8850 Var #31", "FM Organ"},
    {'M',  31,   0,  39, "SC-8850 Var #31", "Pulse Mix Bs"},
    {'M',  31,   0,  80, "SC-8850 Var #31", "JP8000DistLd"},
    {'M',  31,   0,  81, "SC-8850 Var #31", "MG Saw Lead"},
    {'M',  31,   0, 101, "SC-8850 Var #31", "Atl.Mod.FX"},
    {'M',  31,   0, 120, "SC-8850 Var #31", "DstGT.DwnNz2"},
    {'M',  31,   0, 125, "SC-8850 Var #31", "Stab! 1"},
    {'M',  31,   3,  81, "SC-88Pro Map Var #31", "MG Saw Lead"},
    {'M',  31,   4,  16, "SC-8850 Map Var #31", "FM Organ"},
    {'M',  31,   4,  39, "SC-8850 Map Var #31", "Pulse Mix Bs"},
    {'M',  31,   4,  80, "SC-8850 Map Var #31", "JP8000DistLd"},
    {'M',  31,   4,  81, "SC-8850 Map Var #31", "MG Saw Lead"},
    {'M',  31,   4, 101, "SC-8850 Map Var #31", "Atl.Mod.FX"},
    {'M',  31,   4, 120, "SC-8850 Map Var #31", "DstGT.DwnNz2"},
    {'M',  31,   4, 125, "SC-8850 Map Var #31", "Stab! 1"},
    {'M',  32,   0,   5, "SC-8850 Var #32", "EP Phase"},
    {'M',  32,   0,   6, "SC-8850 Var #32", "Synth Harpsi"},
    {'M',  32,   0,   7, "SC-8850 Var #32", "Analog Clav."},
    {'M',  32,   0,  16, "SC-8850 Var #32", "70's E.Organ"},
    {'M',  32,   0,  17, "SC-8850 Var #32", "Perc. Organ"},
    {'M',  32,   0,  19, "SC-8850 Var #32", "Trem.Flute"},
    {'M',  32,   0,  24, "SC-8850 Var #32", "Nylon Gt.2"},
    {'M',  32,   0,  25, "SC-8850 Var #32", "Steel Gt.2"},
    {'M',  32,   0,  38, "SC-8850 Var #32", "Hit&Saw Bass"},
    {'M',  32,   0,  39, "SC-8850 Var #32", "MG Dist Bass"},
    {'M',  32,   0,  48, "SC-8850 Var #32", "Oct Strings1"},
    {'M',  32,   0,  52, "SC-8850 Var #32", "Chorus Aahs"},
    {'M',  32,   0,  53, "SC-8850 Var #32", "VoiceWah Mal"},
    {'M',  32,   0,  56, "SC-8850 Var #32", "Syn. Trumpet"},
    {'M',  32,   0,  61, "SC-8850 Var #32", "Orch Brass"},
    {'M',  32,   0,  80, "SC-8850 Var #32", "HipHop SinLd"},
    {'M',  32,   0,  81, "SC-8850 Var #32", "OB Saw Lead"},
    {'M',  32,   0, 101, "SC-8850 Var #32", "Acid Copter"},
    {'M',  32,   0, 105, "SC-8850 Var #32", "Pi Pa"},
    {'M',  32,   0, 111, "SC-8850 Var #32", "Suona 1"},
    {'M',  32,   0, 112, "SC-8850 Var #32", "Jngl Crash"},
    {'M',  32,   0, 115, "SC-8850 Var #32", "909 HandClap"},
    {'M',  32,   0, 116, "SC-8850 Var #32", "Jngl BD Roll"},
    {'M',  32,   0, 120, "SC-8850 Var #32", "DstGT.MuteNz"},
    {'M',  32,   0, 125, "SC-8850 Var #32", "Stab! 2"},
    {'M',  32,   1,  16, "SC-55 Map Var #32", "Organ 4"},
    {'M',  32,   1,  17, "SC-55 Map Var #32", "Organ 5"},
    {'M',  32,   1,  24, "SC-55 Map Var #32", "Nylon Gt.2"},
    {'M',  32,   1,  52, "SC-55 Map Var #32", "Choir Aahs2"},
    {'M',  32,   2,  16, "SC-88 Map Var #32", "Organ 4"},
    {'M',  32,   2,  17, "SC-88 Map Var #32", "Organ 5"},
    {'M',  32,   2,  19, "SC-88 Map Var #32", "Trem.Flute"},
    {'M',  32,   2,  24, "SC-88 Map Var #32", "Nylon Gt.2"},
    {'M',  32,   2,  25, "SC-88 Map Var #32", "Steel Gt.2"},
    {'M',  32,   2,  52, "SC-88 Map Var #32", "ChoirAahs 2"},
    {'M',  32,   3,   6, "SC-88Pro Map Var #32", "Synth Harpsi"},
    {'M',  32,   3,   7, "SC-88Pro Map Var #32", "Analog Clav."},
    {'M',  32,   3,  16, "SC-88Pro Map Var #32", "70's E.Organ"},
    {'M',  32,   3,  17, "SC-88Pro Map Var #32", "Perc. Organ"},
    {'M',  32,   3,  19, "SC-88Pro Map Var #32", "Trem.Flute"},
    {'M',  32,   3,  24, "SC-88Pro Map Var #32", "Nylon Gt 2"},
    {'M',  32,   3,  25, "SC-88Pro Map Var #32", "Steel Gt.2"},
    {'M',  32,   3,  48, "SC-88Pro Map Var #32", "Oct Strings1"},
    {'M',  32,   3,  52, "SC-88Pro Map Var #32", "Chorus Aahs"},
    {'M',  32,   3,  56, "SC-88Pro Map Var #32", "Syn. Trumpet"},
    {'M',  32,   3,  81, "SC-88Pro Map Var #32", "OB Saw Lead"},
    {'M',  32,   3, 105, "SC-88Pro Map Var #32", "Pi Pa"},
    {'M',  32,   3, 111, "SC-88Pro Map Var #32", "Suona 1"},
    {'M',  32,   3, 115, "SC-88Pro Map Var #32", "909 HandClap"},
    {'M',  32,   4,   5, "SC-8850 Map Var #32", "EP Phase"},
    {'M',  32,   4,   6, "SC-8850 Map Var #32", "Synth Harpsi"},
    {'M',  32,   4,   7, "SC-8850 Map Var #32", "Analog Clav."},
    {'M',  32,   4,  16, "SC-8850 Map Var #32", "70's E.Organ"},
    {'M',  32,   4,  17, "SC-8850 Map Var #32", "Perc. Organ"},
    {'M',  32,   4,  19, "SC-8850 Map Var #32", "Trem.Flute"},
    {'M',  32,   4,  24, "SC-8850 Map Var #32", "Nylon Gt.2"},
    {'M',  32,   4,  25, "SC-8850 Map Var #32", "Steel Gt.2"},
    {'M',  32,   4,  38, "SC-8850 Map Var #32", "Hit&Saw Bass"},
    {'M',  32,   4,  39, "SC-8850 Map Var #32", "MG Dist Bass"},
    {'M',  32,   4,  48, "SC-8850 Map Var #32", "Oct Strings1"},
    {'M',  32,   4,  52, "SC-8850 Map Var #32", "Chorus Aahs"},
    {'M',  32,   4,  53, "SC-8850 Map Var #32", "VoiceWah Mal"},
    {'M',  32,   4,  56, "SC-8850 Map Var #32", "Syn. Trumpet"},
    {'M',  32,   4,  61, "SC-8850 Map Var #32", "Orch Brass"},
    {'M',  32,   4,  80, "SC-8850 Map Var #32", "HipHop SinLd"},
    {'M',  32,   4,  81, "SC-8850 Map Var #32", "OB Saw Lead"},
    {'M',  32,   4, 101, "SC-8850 Map Var #32", "Acid Copter"},
    {'M',  32,   4, 105, "SC-8850 Map Var #32", "Pi Pa"},
    {'M',  32,   4, 111, "SC-8850 Map Var #32", "Suona 1"},
    {'M',  32,   4, 112, "SC-8850 Map Var #32", "Jngl Crash"},
    {'M',  32,   4, 115, "SC-8850 Map Var #32", "909 HandClap"},
    {'M',  32,   4, 116, "SC-8850 Map Var #32", "Jngl BD Roll"},
    {'M',  32,   4, 120, "SC-8850 Map Var #32", "DstGT.MuteNz"},
    {'M',  32,   4, 125, "SC-8850 Map Var #32", "Stab! 2"},
    {'M',  33,   0,   7, "SC-8850 Var #33", "JP8 Clav. 1"},
    {'M',  33,   0,  16, "SC-8850 Var #33", "Even Bar"},
    {'M',  33,   0,  17, "SC-8850 Var #33", "Perc.Organ 2"},
    {'M',  33,   0,  19, "SC-8850 Var #33", "Theater Org."},
    {'M',  33,   0,  25, "SC-8850 Var #33", "Steel + Body"},
    {'M',  33,   0,  38, "SC-8850 Var #33", "Ring Bass"},
    {'M',  33,   0,  39, "SC-8850 Var #33", "Seq Bass 2"},
    {'M',  33,   0,  48, "SC-8850 Var #33", "Oct Strings2"},
    {'M',  33,   0,  52, "SC-8850 Var #33", "Male Aah+Str"},
    {'M',  33,   0,  53, "SC-8850 Var #33", "ChorusWahMal"},
    {'M',  33,   0,  61, "SC-8850 Var #33", "Orch Brass 2"},
    {'M',  33,   0,  80, "SC-8850 Var #33", "HipHop SqrLd"},
    {'M',  33,   0,  81, "SC-8850 Var #33", "P5 Saw Lead"},
    {'M',  33,   0, 111, "SC-8850 Var #33", "Suona 2"},
    {'M',  33,   2,  16, "SC-88 Map Var #33", "Even Bar"},
    {'M',  33,   3,   7, "SC-88Pro Map Var #33", "JP8 Clav. 1"},
    {'M',  33,   3,  16, "SC-88Pro Map Var #33", "Even Bar"},
    {'M',  33,   3,  19, "SC-88Pro Map Var #33", "Theater Org."},
    {'M',  33,   3,  48, "SC-88Pro Map Var #33", "Oct Strings2"},
    {'M',  33,   3,  52, "SC-88Pro Map Var #33", "Male Aah+Str"},
    {'M',  33,   3,  81, "SC-88Pro Map Var #33", "P5 Saw Lead"},
    {'M',  33,   3, 111, "SC-88Pro Map Var #33", "Suona 2"},
    {'M',  33,   4,   7, "SC-8850 Map Var #33", "JP8 Clav. 1"},
    {'M',  33,   4,  16, "SC-8850 Map Var #33", "Even Bar"},
    {'M',  33,   4,  17, "SC-8850 Map Var #33", "Perc.Organ 2"},
    {'M',  33,   4,  19, "SC-8850 Map Var #33", "Theater Org."},
    {'M',  33,   4,  25, "SC-8850 Map Var #33", "Steel + Body"},
    {'M',  33,   4,  38, "SC-8850 Map Var #33", "Ring Bass"},
    {'M',  33,   4,  39, "SC-8850 Map Var #33", "Seq Bass 2"},
    {'M',  33,   4,  48, "SC-8850 Map Var #33", "Oct Strings2"},
    {'M',  33,   4,  52, "SC-8850 Map Var #33", "Male Aah+Str"},
    {'M',  33,   4,  53, "SC-8850 Map Var #33", "ChorusWahMal"},
    {'M',  33,   4,  61, "SC-8850 Map Var #33", "Orch Brass 2"},
    {'M',  33,   4,  80, "SC-8850 Map Var #33", "HipHop SqrLd"},
    {'M',  33,   4,  81, "SC-8850 Map Var #33", "P5 Saw Lead"},
    {'M',  33,   4, 111, "SC-8850 Map Var #33", "Suona 2"},
    {'M',  34,   0,  17, "SC-8850 Var #34", "Perc.Organ 3"},
    {'M',  34,   0,  38, "SC-8850 Var #34", "AtkSineBass"},
    {'M',  34,   0,  39, "SC-8850 Var #34", "3rd Bass"},
    {'M',  34,   0,  48, "SC-8850 Var #34", "ContraBsSect"},
    {'M',  34,   0,  53, "SC-8850 Var #34", "VoiceWoh Mal"},
    {'M',  34,   0,  80, "SC-8850 Var #34", "HipHop PlsLd"},
    {'M',  34,   0,  81, "SC-8850 Var #34", "MG unison"},
    {'M',  34,   0, 120, "SC-8850 Var #34", "Gt.StrokeNz5"},
    {'M',  34,   3,  81, "SC-88Pro Map Var #34", "MG unison"},
    {'M',  34,   4,  17, "SC-8850 Map Var #34", "Perc.Organ 3"},
    {'M',  34,   4,  38, "SC-8850 Map Var #34", "AtkSineBass"},
    {'M',  34,   4,  39, "SC-8850 Map Var #34", "3rd Bass"},
    {'M',  34,   4,  48, "SC-8850 Map Var #34", "ContraBsSect"},
    {'M',  34,   4,  53, "SC-8850 Map Var #34", "VoiceWoh Mal"},
    {'M',  34,   4,  80, "SC-8850 Map Var #34", "HipHop PlsLd"},
    {'M',  34,   4,  81, "SC-8850 Map Var #34", "MG unison"},
    {'M',  34,   4, 120, "SC-8850 Map Var #34", "Gt.StrokeNz5"},
    {'M',  35,   0,   7, "SC-8850 Var #35", "JP8 Clav. 2"},
    {'M',  35,   0,  17, "SC-8850 Var #35", "Perc.Organ 4"},
    {'M',  35,   0,  38, "SC-8850 Var #35", "OB sine Bass"},
    {'M',  35,   0,  39, "SC-8850 Var #35", "MG Oct Bass"},
    {'M',  35,   0,  53, "SC-8850 Var #35", "ChorusWohMal"},
    {'M',  35,   0,  61, "SC-8850 Var #35", "St.FatPopBrs"},
    {'M',  35,   0,  80, "SC-8850 Var #35", "Flux Pulse"},
    {'M',  35,   0,  81, "SC-8850 Var #35", "Oct Saw Lead"},
    {'M',  35,   0, 120, "SC-8850 Var #35", "StlGt.SldNz1"},
    {'M',  35,   3,   7, "SC-88Pro Map Var #35", "JP8 Clav. 2"},
    {'M',  35,   3,  81, "SC-88Pro Map Var #35", "Oct Saw Lead"},
    {'M',  35,   4,   7, "SC-8850 Map Var #35", "JP8 Clav. 2"},
    {'M',  35,   4,  17, "SC-8850 Map Var #35", "Perc.Organ 4"},
    {'M',  35,   4,  38, "SC-8850 Map Var #35", "OB sine Bass"},
    {'M',  35,   4,  39, "SC-8850 Map Var #35", "MG Oct Bass"},
    {'M',  35,   4,  53, "SC-8850 Map Var #35", "ChorusWohMal"},
    {'M',  35,   4,  61, "SC-8850 Map Var #35", "St.FatPopBrs"},
    {'M',  35,   4,  80, "SC-8850 Map Var #35", "Flux Pulse"},
    {'M',  35,   4,  81, "SC-8850 Map Var #35", "Oct Saw Lead"},
    {'M',  35,   4, 120, "SC-8850 Map Var #35", "StlGt.SldNz1"},
    {'M',  36,   0,   7, "SC-8850 Var #36", "SynRingClav."},
    {'M',  36,   0,  38, "SC-8850 Var #36", "Auxiliary Bs"},
    {'M',  36,   0,  39, "SC-8850 Var #36", "SlowEnvBass"},
    {'M',  36,   0,  53, "SC-8850 Var #36", "VoiceAah Mal"},
    {'M',  36,   0,  61, "SC-8850 Var #36", "St.Orch Brs"},
    {'M',  36,   0,  81, "SC-8850 Var #36", "Natural Lead"},
    {'M',  36,   0, 120, "SC-8850 Var #36", "StlGt.SldNz2"},
    {'M',  36,   4,   7, "SC-8850 Map Var #36", "SynRingClav."},
    {'M',  36,   4,  38, "SC-8850 Map Var #36", "Auxiliary Bs"},
    {'M',  36,   4,  39, "SC-8850 Map Var #36", "SlowEnvBass"},
    {'M',  36,   4,  53, "SC-8850 Map Var #36", "VoiceAah Mal"},
    {'M',  36,   4,  61, "SC-8850 Map Var #36", "St.Orch Brs"},
    {'M',  36,   4,  81, "SC-8850 Map Var #36", "Natural Lead"},
    {'M',  36,   4, 120, "SC-8850 Map Var #36", "StlGt.SldNz2"},
    {'M',  37,   0,   7, "SC-8850 Var #37", "SynDistClav."},
    {'M',  37,   0,  39, "SC-8850 Var #37", "Mild Bass"},
    {'M',  37,   0,  53, "SC-8850 Var #37", "VoiceOoh Mal"},
    {'M',  37,   0,  61, "SC-8850 Var #37", "St.Orch Brs2"},
    {'M',  37,   0, 120, "SC-8850 Var #37", "StlGt.SldNz3"},
    {'M',  37,   4,   7, "SC-8850 Map Var #37", "SynDistClav."},
    {'M',  37,   4,  39, "SC-8850 Map Var #37", "Mild Bass"},
    {'M',  37,   4,  53, "SC-8850 Map Var #37", "VoiceOoh Mal"},
    {'M',  37,   4,  61, "SC-8850 Map Var #37", "St.Orch Brs2"},
    {'M',  37,   4, 120, "SC-8850 Map Var #37", "StlGt.SldNz3"},
    {'M',  38,   0,   7, "SC-8850 Var #38", "JP8000 Clav."},
    {'M',  38,   0,  39, "SC-8850 Var #38", "DistEnvBass"},
    {'M',  38,   0,  61, "SC-8850 Var #38", "St.Orch Brs3"},
    {'M',  38,   0, 120, "SC-8850 Var #38", "StlGt.SldNz4"},
    {'M',  38,   4,   7, "SC-8850 Map Var #38", "JP8000 Clav."},
    {'M',  38,   4,  39, "SC-8850 Map Var #38", "DistEnvBass"},
    {'M',  38,   4,  61, "SC-8850 Map Var #38", "St.Orch Brs3"},
    {'M',  38,   4, 120, "SC-8850 Map Var #38", "StlGt.SldNz4"},
    {'M',  39,   0,   7, "SC-8850 Var #39", "Pulse Clav"},
    {'M',  39,   0,  39, "SC-8850 Var #39", "MG LightBass"},
    {'M',  39,   0, 120, "SC-8850 Var #39", "Gt.StrokeNz1"},
    {'M',  39,   4,   7, "SC-8850 Map Var #39", "Pulse Clav"},
    {'M',  39,   4,  39, "SC-8850 Map Var #39", "MG LightBass"},
    {'M',  39,   4, 120, "SC-8850 Map Var #39", "Gt.StrokeNz1"},
    {'M',  40,   0,  16, "SC-8850 Var #40", "Organ Bass"},
    {'M',  40,   0,  24, "SC-8850 Var #40", "Lequint Gt."},
    {'M',  40,   0,  38, "SC-8850 Var #40", "303SqDistBs"},
    {'M',  40,   0,  39, "SC-8850 Var #40", "DistSynBass"},
    {'M',  40,   0,  48, "SC-8850 Var #40", "60s Strings"},
    {'M',  40,   0,  53, "SC-8850 Var #40", "Humming"},
    {'M',  40,   0,  81, "SC-8850 Var #40", "SequenceSaw1"},
    {'M',  40,   0, 112, "SC-8850 Var #40", "Crash Menu"},
    {'M',  40,   0, 115, "SC-8850 Var #40", "HandClapMenu"},
    {'M',  40,   0, 116, "SC-8850 Var #40", "Kick Menu 1"},
    {'M',  40,   0, 117, "SC-8850 Var #40", "SD Menu 1"},
    {'M',  40,   0, 119, "SC-8850 Var #40", "Rev.SD Menu1"},
    {'M',  40,   0, 120, "SC-8850 Var #40", "Gt.StrokeNz2"},
    {'M',  40,   2,  16, "SC-88 Map Var #40", "Organ Bass"},
    {'M',  40,   2,  24, "SC-88 Map Var #40", "Lequint Gt."},
    {'M',  40,   3,  16, "SC-88Pro Map Var #40", "Organ Bass"},
    {'M',  40,   3,  24, "SC-88Pro Map Var #40", "Lequint Gt."},
    {'M',  40,   3,  81, "SC-88Pro Map Var #40", "SequenceSaw1"},
    {'M',  40,   4,  16, "SC-8850 Map Var #40", "Organ Bass"},
    {'M',  40,   4,  24, "SC-8850 Map Var #40", "Lequint Gt."},
    {'M',  40,   4,  38, "SC-8850 Map Var #40", "303SqDistBs"},
    {'M',  40,   4,  39, "SC-8850 Map Var #40", "DistSynBass"},
    {'M',  40,   4,  48, "SC-8850 Map Var #40", "60s Strings"},
    {'M',  40,   4,  53, "SC-8850 Map Var #40", "Humming"},
    {'M',  40,   4,  81, "SC-8850 Map Var #40", "SequenceSaw1"},
    {'M',  40,   4, 112, "SC-8850 Map Var #40", "Crash Menu"},
    {'M',  40,   4, 115, "SC-8850 Map Var #40", "HandClapMenu"},
    {'M',  40,   4, 116, "SC-8850 Map Var #40", "Kick Menu 1"},
    {'M',  40,   4, 117, "SC-8850 Map Var #40", "SD Menu 1"},
    {'M',  40,   4, 119, "SC-8850 Map Var #40", "Rev.SD Menu1"},
    {'M',  40,   4, 120, "SC-8850 Map Var #40", "Gt.StrokeNz2"},
    {'M',  41,   0,  38, "SC-8850 Var #41", "303SqDistBs2"},
    {'M',  41,   0,  39, "SC-8850 Var #41", "Rise Bass"},
    {'M',  41,   0,  81, "SC-8850 Var #41", "SequenceSaw2"},
    {'M',  41,   0, 112, "SC-8850 Var #41", "RideCym Menu"},
    {'M',  41,   0, 116, "SC-8850 Var #41", "Kick Menu 2"},
    {'M',  41,   0, 117, "SC-8850 Var #41", "SD Menu 2"},
    {'M',  41,   0, 119, "SC-8850 Var #41", "Rev.SD Menu2"},
    {'M',  41,   0, 120, "SC-8850 Var #41", "Gt.StrokeNz3"},
    {'M',  41,   3,  81, "SC-88Pro Map Var #41", "SequenceSaw2"},
    {'M',  41,   4,  38, "SC-8850 Map Var #41", "303SqDistBs2"},
    {'M',  41,   4,  39, "SC-8850 Map Var #41", "Rise Bass"},
    {'M',  41,   4,  81, "SC-8850 Map Var #41", "SequenceSaw2"},
    {'M',  41,   4, 112, "SC-8850 Map Var #41", "RideCym Menu"},
    {'M',  41,   4, 116, "SC-8850 Map Var #41", "Kick Menu 2"},
    {'M',  41,   4, 117, "SC-8850 Map Var #41", "SD Menu 2"},
    {'M',  41,   4, 119, "SC-8850 Map Var #41", "Rev.SD Menu2"},
    {'M',  41,   4, 120, "SC-8850 Map Var #41", "Gt.StrokeNz3"},
    {'M',  42,   0,  38, "SC-8850 Var #42", "303SqDistBs3"},
    {'M',  42,   0,  39, "SC-8850 Var #42", "Cyber Bass"},
    {'M',  42,   0,  81, "SC-8850 Var #42", "Reso Saw"},
    {'M',  42,   0, 112, "SC-8850 Var #42", "RideBellMenu"},
    {'M',  42,   0, 116, "SC-8850 Var #42", "Kick Menu 3"},
    {'M',  42,   0, 117, "SC-8850 Var #42", "SD Menu 3"},
    {'M',  42,   0, 119, "SC-8850 Var #42", "Rev.SD Menu3"},
    {'M',  42,   0, 120, "SC-8850 Var #42", "Gt.StrokeNz4"},
    {'M',  42,   3,  81, "SC-88Pro Map Var #42", "Reso Saw"},
    {'M',  42,   4,  38, "SC-8850 Map Var #42", "303SqDistBs3"},
    {'M',  42,   4,  39, "SC-8850 Map Var #42", "Cyber Bass"},
    {'M',  42,   4,  81, "SC-8850 Map Var #42", "Reso Saw"},
    {'M',  42,   4, 112, "SC-8850 Map Var #42", "RideBellMenu"},
    {'M',  42,   4, 116, "SC-8850 Map Var #42", "Kick Menu 3"},
    {'M',  42,   4, 117, "SC-8850 Map Var #42", "SD Menu 3"},
    {'M',  42,   4, 119, "SC-8850 Map Var #42", "Rev.SD Menu3"},
    {'M',  42,   4, 120, "SC-8850 Map Var #42", "Gt.StrokeNz4"},
    {'M',  43,   0,  38, "SC-8850 Var #43", "303Sqr.Rev"},
    {'M',  43,   0,  81, "SC-8850 Var #43", "Cheese Saw 1"},
    {'M',  43,   0, 116, "SC-8850 Var #43", "Kick Menu 4"},
    {'M',  43,   0, 117, "SC-8850 Var #43", "SD Menu 4"},
    {'M',  43,   0, 119, "SC-8850 Var #43", "Rev.BD Menu1"},
    {'M',  43,   3,  81, "SC-88Pro Map Var #43", "Cheese Saw 1"},
    {'M',  43,   4,  38, "SC-8850 Map Var #43", "303Sqr.Rev"},
    {'M',  43,   4,  81, "SC-8850 Map Var #43", "Cheese Saw 1"},
    {'M',  43,   4, 116, "SC-8850 Map Var #43", "Kick Menu 4"},
    {'M',  43,   4, 117, "SC-8850 Map Var #43", "SD Menu 4"},
    {'M',  43,   4, 119, "SC-8850 Map Var #43", "Rev.BD Menu1"},
    {'M',  44,   0,  38, "SC-8850 Var #44", "TeeBee"},
    {'M',  44,   0,  81, "SC-8850 Var #44", "Cheese Saw 2"},
    {'M',  44,   0, 117, "SC-8850 Var #44", "SD Menu 5"},
    {'M',  44,   0, 119, "SC-8850 Var #44", "Rev.BD Menu2"},
    {'M',  44,   3,  81, "SC-88Pro Map Var #44", "Cheese Saw 2"},
    {'M',  44,   4,  38, "SC-8850 Map Var #44", "TeeBee"},
    {'M',  44,   4,  81, "SC-8850 Map Var #44", "Cheese Saw 2"},
    {'M',  44,   4, 117, "SC-8850 Map Var #44", "SD Menu 5"},
    {'M',  44,   4, 119, "SC-8850 Map Var #44", "Rev.BD Menu2"},
    {'M',  45,   0,  81, "SC-8850 Var #45", "Rhythmic Saw"},
    {'M',  45,   0, 119, "SC-8850 Var #45", "Rev.BD Menu3"},
    {'M',  45,   3,  81, "SC-88Pro Map Var #45", "Rhythmic Saw"},
    {'M',  45,   4,  81, "SC-8850 Map Var #45", "Rhythmic Saw"},
    {'M',  45,   4, 119, "SC-8850 Map Var #45", "Rev.BD Menu3"},
    {'M',  46,   0,  81, "SC-8850 Var #46", "SequencedSaw"},
    {'M',  46,   0, 119, "SC-8850 Var #46", "Rev.ClapMenu"},
    {'M',  46,   4,  81, "SC-8850 Map Var #46", "SequencedSaw"},
    {'M',  46,   4, 119, "SC-8850 Map Var #46", "Rev.ClapMenu"},
    {'M',  47,   0,  81, "SC-8850 Var #47", "Techno Saw"},
    {'M',  47,   4,  81, "SC-8850 Map Var #47", "Techno Saw"},
    {'M',  48,   0,  16, "SC-8850 Var #48", "5th Organ"},
    {'M',  48,   3,  16, "SC-88Pro Map Var #48", "5th Organ"},
    {'M',  48,   4,  16, "SC-8850 Map Var #48", "5th Organ"},
    {'M', 126,   1,   0, "SC-55 Map CM-64(PCM)", "Piano 2"},
    {'M', 126,   1,   1, "SC-55 Map CM-64(PCM)", "Piano 2"},
    {'M', 126,   1,   2, "SC-55 Map CM-64(PCM)", "Piano 2"},
    {'M', 126,   1,   3, "SC-55 Map CM-64(PCM)", "Honky-tonk"},
    {'M', 126,   1,   4, "SC-55 Map CM-64(PCM)", "Piano 1"},
    {'M', 126,   1,   5, "SC-55 Map CM-64(PCM)", "Piano 2"},
    {'M', 126,   1,   6, "SC-55 Map CM-64(PCM)", "Piano 2"},
    {'M', 126,   1,   7, "SC-55 Map CM-64(PCM)", "E.Piano 1"},
    {'M', 126,   1,   8, "SC-55 Map CM-64(PCM)", "Detuned EP1"},
    {'M', 126,   1,   9, "SC-55 Map CM-64(PCM)", "E.Piano 2"},
    {'M', 126,   1,  10, "SC-55 Map CM-64(PCM)", "Steel Gt."},
    {'M', 126,   1,  11, "SC-55 Map CM-64(PCM)", "Steel Gt."},
    {'M', 126,   1,  12, "SC-55 Map CM-64(PCM)", "12-str.Gt"},
    {'M', 126,   1,  13, "SC-55 Map CM-64(PCM)", "Funk Gt."},
    {'M', 126,   1,  14, "SC-55 Map CM-64(PCM)", "Muted Gt."},
    {'M', 126,   1,  15, "SC-55 Map CM-64(PCM)", "Slap Bass 1"},
    {'M', 126,   1,  16, "SC-55 Map CM-64(PCM)", "Slap Bass 1"},
    {'M', 126,   1,  17, "SC-55 Map CM-64(PCM)", "Slap Bass 1"},
    {'M', 126,   1,  18, "SC-55 Map CM-64(PCM)", "Slap Bass 1"},
    {'M', 126,   1,  19, "SC-55 Map CM-64(PCM)", "Slap Bass 2"},
    {'M', 126,   1,  20, "SC-55 Map CM-64(PCM)", "Slap Bass 2"},
    {'M', 126,   1,  21, "SC-55 Map CM-64(PCM)", "Slap Bass 2"},
    {'M', 126,   1,  22, "SC-55 Map CM-64(PCM)", "Slap Bass 2"},
    {'M', 126,   1,  23, "SC-55 Map CM-64(PCM)", "Fingerd Bs"},
    {'M', 126,   1,  24, "SC-55 Map CM-64(PCM)", "Fingerd Bs"},
    {'M', 126,   1,  25, "SC-55 Map CM-64(PCM)", "Picked Bs"},
    {'M', 126,   1,  26, "SC-55 Map CM-64(PCM)", "Picked Bs"},
    {'M', 126,   1,  27, "SC-55 Map CM-64(PCM)", "Fretless Bs"},
    {'M', 126,   1,  28, "SC-55 Map CM-64(PCM)", "Acoustic Bs"},
    {'M', 126,   1,  29, "SC-55 Map CM-64(PCM)", "Choir Aahs"},
    {'M', 126,   1,  30, "SC-55 Map CM-64(PCM)", "Choir Aahs"},
    {'M', 126,   1,  31, "SC-55 Map CM-64(PCM)", "Choir Aahs"},
    {'M', 126,   1,  32, "SC-55 Map CM-64(PCM)", "Choir Aahs"},
    {'M', 126,   1,  33, "SC-55 Map CM-64(PCM)", "Slow Strings"},
    {'M', 126,   1,  34, "SC-55 Map CM-64(PCM)", "Strings"},
    {'M', 126,   1,  35, "SC-55 Map CM-64(PCM)", "SynStrings3"},
    {'M', 126,   1,  36, "SC-55 Map CM-64(PCM)", "SynStrings3"},
    {'M', 126,   1,  37, "SC-55 Map CM-64(PCM)", "Organ 1"},
    {'M', 126,   1,  38, "SC-55 Map CM-64(PCM)", "Organ 1"},
    {'M', 126,   1,  39, "SC-55 Map CM-64(PCM)", "Organ 1"},
    {'M', 126,   1,  40, "SC-55 Map CM-64(PCM)", "Organ 2"},
    {'M', 126,   1,  41, "SC-55 Map CM-64(PCM)", "Organ 1"},
    {'M', 126,   1,  42, "SC-55 Map CM-64(PCM)", "Organ 1"},
    {'M', 126,   1,  43, "SC-55 Map CM-64(PCM)", "Organ 2"},
    {'M', 126,   1,  44, "SC-55 Map CM-64(PCM)", "Organ 2"},
    {'M', 126,   1,  45, "SC-55 Map CM-64(PCM)", "Organ 2"},
    {'M', 126,   1,  46, "SC-55 Map CM-64(PCM)", "Trumpet"},
    {'M', 126,   1,  47, "SC-55 Map CM-64(PCM)", "Trumpet"},
    {'M', 126,   1,  48, "SC-55 Map CM-64(PCM)", "Trombone"},
    {'M', 126,   1,  49, "SC-55 Map CM-64(PCM)", "Trombone"},
    {'M', 126,   1,  50, "SC-55 Map CM-64(PCM)", "Trombone"},
    {'M', 126,   1,  51, "SC-55 Map CM-64(PCM)", "Trombone"},
    {'M', 126,   1,  52, "SC-55 Map CM-64(PCM)", "Trombone"},
    {'M', 126,   1,  53, "SC-55 Map CM-64(PCM)", "Trombone"},
    {'M', 126,   1,  54, "SC-55 Map CM-64(PCM)", "Alto Sax"},
    {'M', 126,   1,  55, "SC-55 Map CM-64(PCM)", "Tenor Sax"},
    {'M', 126,   1,  56, "SC-55 Map CM-64(PCM)", "BaritoneSax"},
    {'M', 126,   1,  57, "SC-55 Map CM-64(PCM)", "Alto Sax"},
    {'M', 126,   1,  58, "SC-55 Map CM-64(PCM)", "Brass 1"},
    {'M', 126,   1,  59, "SC-55 Map CM-64(PCM)", "Brass 1"},
    {'M', 126,   1,  60, "SC-55 Map CM-64(PCM)", "Brass 2"},
    {'M', 126,   1,  61, "SC-55 Map CM-64(PCM)", "Brass 2"},
    {'M', 126,   1,  62, "SC-55 Map CM-64(PCM)", "Brass 1"},
    {'M', 126,   1,  63, "SC-55 Map CM-64(PCM)", "Orchest.Hit"},
    {'M', 127,   1,   0, "SC-55 Map CM-64(LA)", "Acou Piano 1"},
    {'M', 127,   1,   1, "SC-55 Map CM-64(LA)", "Acou Piano 2"},
    {'M', 127,   1,   2, "SC-55 Map CM-64(LA)", "Acou Piano 3"},
    {'M', 127,   1,   3, "SC-55 Map CM-64(LA)", "Elec Piano 1"},
    {'M', 127,   1,   4, "SC-55 Map CM-64(LA)", "Elec Piano 2"},
    {'M', 127,   1,   5, "SC-55 Map CM-64(LA)", "Elec Piano 3"},
    {'M', 127,   1,   6, "SC-55 Map CM-64(LA)", "Elec Piano 4"},
    {'M', 127,   1,   7, "SC-55 Map CM-64(LA)", "Honkytonk"},
    {'M', 127,   1,   8, "SC-55 Map CM-64(LA)", "Elec Org 1"},
    {'M', 127,   1,   9, "SC-55 Map CM-64(LA)", "Elec Org 2"},
    {'M', 127,   1,  10, "SC-55 Map CM-64(LA)", "Elec Org 3"},
    {'M', 127,   1,  11, "SC-55 Map CM-64(LA)", "Elec Org 4"},
    {'M', 127,   1,  12, "SC-55 Map CM-64(LA)", "Pipe Org 1"},
    {'M', 127,   1,  13, "SC-55 Map CM-64(LA)", "Pipe Org 2"},
    {'M', 127,   1,  14, "SC-55 Map CM-64(LA)", "Pipe Org 3"},
    {'M', 127,   1,  15, "SC-55 Map CM-64(LA)", "Accordion"},
    {'M', 127,   1,  16, "SC-55 Map CM-64(LA)", "Harpsi 1"},
    {'M', 127,   1,  17, "SC-55 Map CM-64(LA)", "Harpsi 2"},
    {'M', 127,   1,  18, "SC-55 Map CM-64(LA)", "Harpsi 3"},
    {'M', 127,   1,  19, "SC-55 Map CM-64(LA)", "Clavi 1"},
    {'M', 127,   1,  20, "SC-55 Map CM-64(LA)", "Clavi 2"},
    {'M', 127,   1,  21, "SC-55 Map CM-64(LA)", "Clavi 3"},
    {'M', 127,   1,  22, "SC-55 Map CM-64(LA)", "Celesta 1"},
    {'M', 127,   1,  23, "SC-55 Map CM-64(LA)", "Celesta 2"},
    {'M', 127,   1,  24, "SC-55 Map CM-64(LA)", "Syn Brass 1"},
    {'M', 127,   1,  25, "SC-55 Map CM-64(LA)", "Syn Brass 2"},
    {'M', 127,   1,  26, "SC-55 Map CM-64(LA)", "Syn Brass 3"},
    {'M', 127,   1,  27, "SC-55 Map CM-64(LA)", "Syn Brass 4"},
    {'M', 127,   1,  28, "SC-55 Map CM-64(LA)", "Syn Bass 1"},
    {'M', 127,   1,  29, "SC-55 Map CM-64(LA)", "Syn Bass 2"},
    {'M', 127,   1,  30, "SC-55 Map CM-64(LA)", "Syn Bass 3"},
    {'M', 127,   1,  31, "SC-55 Map CM-64(LA)", "Syn Bass 4"},
    {'M', 127,   1,  32, "SC-55 Map CM-64(LA)", "Fantasy"},
    {'M', 127,   1,  33, "SC-55 Map CM-64(LA)", "Harmo Pan"},
    {'M', 127,   1,  34, "SC-55 Map CM-64(LA)", "Chorale"},
    {'M', 127,   1,  35, "SC-55 Map CM-64(LA)", "Glasses"},
    {'M', 127,   1,  36, "SC-55 Map CM-64(LA)", "Soundtrack"},
    {'M', 127,   1,  37, "SC-55 Map CM-64(LA)", "Atmosphere"},
    {'M', 127,   1,  38, "SC-55 Map CM-64(LA)", "Warm Bell"},
    {'M', 127,   1,  39, "SC-55 Map CM-64(LA)", "Funny Vox"},
    {'M', 127,   1,  40, "SC-55 Map CM-64(LA)", "Echo Bell"},
    {'M', 127,   1,  41, "SC-55 Map CM-64(LA)", "Ice Rain"},
    {'M', 127,   1,  42, "SC-55 Map CM-64(LA)", "Oboe 2001"},
    {'M', 127,   1,  43, "SC-55 Map CM-64(LA)", "Echo Pan"},
    {'M', 127,   1,  44, "SC-55 Map CM-64(LA)", "Doctor Solo"},
    {'M', 127,   1,  45, "SC-55 Map CM-64(LA)", "School Daze"},
    {'M', 127,   1,  46, "SC-55 Map CM-64(LA)", "Bellsinger"},
    {'M', 127,   1,  47, "SC-55 Map CM-64(LA)", "Square Wave"},
    {'M', 127,   1,  48, "SC-55 Map CM-64(LA)", "Str Sect 1"},
    {'M', 127,   1,  49, "SC-55 Map CM-64(LA)", "Str Sect 2"},
    {'M', 127,   1,  50, "SC-55 Map CM-64(LA)", "Str Sect 3"},
    {'M', 127,   1,  51, "SC-55 Map CM-64(LA)", "Pizzicato"},
    {'M', 127,   1,  52, "SC-55 Map CM-64(LA)", "Violin 1"},
    {'M', 127,   1,  53, "SC-55 Map CM-64(LA)", "Violin 2"},
    {'M', 127,   1,  54, "SC-55 Map CM-64(LA)", "Cello 1"},
    {'M', 127,   1,  55, "SC-55 Map CM-64(LA)", "Cello 2"},
    {'M', 127,   1,  56, "SC-55 Map CM-64(LA)", "Contrabass"},
    {'M', 127,   1,  57, "SC-55 Map CM-64(LA)", "Harp 1"},
    {'M', 127,   1,  58, "SC-55 Map CM-64(LA)", "Harp 2"},
    {'M', 127,   1,  59, "SC-55 Map CM-64(LA)", "Guitar 1"},
    {'M', 127,   1,  60, "SC-55 Map CM-64(LA)", "Guitar 2"},
    {'M', 127,   1,  61, "SC-55 Map CM-64(LA)", "Elec Gtr 1"},
    {'M', 127,   1,  62, "SC-55 Map CM-64(LA)", "Elec Gtr 2"},
    {'M', 127,   1,  63, "SC-55 Map CM-64(LA)", "Sitar"},
    {'M', 127,   1,  64, "SC-55 Map CM-64(LA)", "Acou Bass 1"},
    {'M', 127,   1,  65, "SC-55 Map CM-64(LA)", "Acou Bass 2"},
    {'M', 127,   1,  66, "SC-55 Map CM-64(LA)", "Elec Bass 1"},
    {'M', 127,   1,  67, "SC-55 Map CM-64(LA)", "Elec Bass 2"},
    {'M', 127,   1,  68, "SC-55 Map CM-64(LA)", "Slap Bass 1"},
    {'M', 127,   1,  69, "SC-55 Map CM-64(LA)", "Slap Bass 2"},
    {'M', 127,   1,  70, "SC-55 Map CM-64(LA)", "Fretless 1"},
    {'M', 127,   1,  71, "SC-55 Map CM-64(LA)", "Fretless 2"},
    {'M', 127,   1,  72, "SC-55 Map CM-64(LA)", "Flute 1"},
    {'M', 127,   1,  73, "SC-55 Map CM-64(LA)", "Flute 2"},
    {'M', 127,   1,  74, "SC-55 Map CM-64(LA)", "Piccolo 1"},
    {'M', 127,   1,  75, "SC-55 Map CM-64(LA)", "Piccolo 2"},
    {'M', 127,   1,  76, "SC-55 Map CM-64(LA)", "Recorder"},
    {'M', 127,   1,  77, "SC-55 Map CM-64(LA)", "Pan Pipes"},
    {'M', 127,   1,  78, "SC-55 Map CM-64(LA)", "Sax 1"},
    {'M', 127,   1,  79, "SC-55 Map CM-64(LA)", "Sax 2"},
    {'M', 127,   1,  80, "SC-55 Map CM-64(LA)", "Sax 3"},
    {'M', 127,   1,  81, "SC-55 Map CM-64(LA)", "Sax 4"},
    {'M', 127,   1,  82, "SC-55 Map CM-64(LA)", "Clarinet 1"},
    {'M', 127,   1,  83, "SC-55 Map CM-64(LA)", "Clarinet 2"},
    {'M', 127,   1,  84, "SC-55 Map CM-64(LA)", "Oboe"},
    {'M', 127,   1,  85, "SC-55 Map CM-64(LA)", "Engl Horn"},
    {'M', 127,   1,  86, "SC-55 Map CM-64(LA)", "Bassoon"},
    {'M', 127,   1,  87, "SC-55 Map CM-64(LA)", "Harmonica"},
    {'M', 127,   1,  88, "SC-55 Map CM-64(LA)", "Trumpet 1"},
    {'M', 127,   1,  89, "SC-55 Map CM-64(LA)", "Trumpet 2"},
    {'M', 127,   1,  90, "SC-55 Map CM-64(LA)", "Trombone 1"},
    {'M', 127,   1,  91, "SC-55 Map CM-64(LA)", "Trombone 2"},
    {'M', 127,   1,  92, "SC-55 Map CM-64(LA)", "Fr Horn 1"},
    {'M', 127,   1,  93, "SC-55 Map CM-64(LA)", "Fr Horn 2"},
    {'M', 127,   1,  94, "SC-55 Map CM-64(LA)", "Tuba"},
    {'M', 127,   1,  95, "SC-55 Map CM-64(LA)", "Brs Sect 1"},
    {'M', 127,   1,  96, "SC-55 Map CM-64(LA)", "Brs Sect 2"},
    {'M', 127,   1,  97, "SC-55 Map CM-64(LA)", "Vibe 1"},
    {'M', 127,   1,  98, "SC-55 Map CM-64(LA)", "Vibe 2"},
    {'M', 127,   1,  99, "SC-55 Map CM-64(LA)", "Syn Mallet"},
    {'M', 127,   1, 100, "SC-55 Map CM-64(LA)", "Widbell"},
    {'M', 127,   1, 101, "SC-55 Map CM-64(LA)", "Glock"},
    {'M', 127,   1, 102, "SC-55 Map CM-64(LA)", "Tue Bell"},
    {'M', 127,   1, 103, "SC-55 Map CM-64(LA)", "Xylophone"},
    {'M', 127,   1, 104, "SC-55 Map CM-64(LA)", "Marimba"},
    {'M', 127,   1, 105, "SC-55 Map CM-64(LA)", "Koto"},
    {'M', 127,   1, 106, "SC-55 Map CM-64(LA)", "Sho"},
    {'M', 127,   1, 107, "SC-55 Map CM-64(LA)", "Shakuhachi"},
    {'M', 127,   1, 108, "SC-55 Map CM-64(LA)", "Whistle 1"},
    {'M', 127,   1, 109, "SC-55 Map CM-64(LA)", "Whistle 2"},
    {'M', 127,   1, 110, "SC-55 Map CM-64(LA)", "Bottleblow"},
    {'M', 127,   1, 111, "SC-55 Map CM-64(LA)", "Breathpipe"},
    {'M', 127,   1, 112, "SC-55 Map CM-64(LA)", "Timpani"},
    {'M', 127,   1, 113, "SC-55 Map CM-64(LA)", "Melodic Tom"},
    {'M', 127,   1, 114, "SC-55 Map CM-64(LA)", "Deep Snare"},
    {'M', 127,   1, 115, "SC-55 Map CM-64(LA)", "Elec Perc 1"},
    {'M', 127,   1, 116, "SC-55 Map CM-64(LA)", "Elec Perc 2"},
    {'M', 127,   1, 117, "SC-55 Map CM-64(LA)", "Taiko"},
    {'M', 127,   1, 118, "SC-55 Map CM-64(LA)", "Taiko Rim"},
    {'M', 127,   1, 119, "SC-55 Map CM-64(LA)", "Cymbal"},
    {'M', 127,   1, 120, "SC-55 Map CM-64(LA)", "Castanets"},
    {'M', 127,   1, 121, "SC-55 Map CM-64(LA)", "Triangle"},
    {'M', 127,   1, 122, "SC-55 Map CM-64(LA)", "Orche Hit"},
    {'M', 127,   1, 123, "SC-55 Map CM-64(LA)", "Telephone"},
    {'M', 127,   1, 124, "SC-55 Map CM-64(LA)", "Bird Tweet"},
    {'M', 127,   1, 125, "SC-55 Map CM-64(LA)", "OneNote Jam"},
    {'M', 127,   1, 126, "SC-55 Map CM-64(LA)", "Water Bell"},
    {'M', 127,   1, 127, "SC-55 Map CM-64(LA)", "Jungle Tune"},
    {'P',   0,   1,   0, "SC-8850 STANDARD 2 Set", "[88] Standard 1 Kick 1"},
    {'P',   0,   1,   1, "SC-8850 STANDARD 2 Set", "[88] Standard 1 Kick 2"},
    {'P',   0,   1,   2, "SC-8850 STANDARD 2 Set", "[88] Standard 2 Kick 1"},
    {'P',   0,   1,   3, "SC-8850 STANDARD 2 Set", "[88] Standard 2 Kick 2"},
    {'P',   0,   1,   4, "SC-8850 STANDARD 2 Set", "[55] Kick Drum 1"},
    {'P',   0,   1,   5, "SC-8850 STANDARD 2 Set", "[55] Kick Drum 2"},
    {'P',   0,   1,   6, "SC-8850 STANDARD 2 Set", "[88] Jazz Kick 1"},
    {'P',   0,   1,   7, "SC-8850 STANDARD 2 Set", "[88] Jazz Kick 2"},
    {'P',   0,   1,   8, "SC-8850 STANDARD 2 Set", "[88] Room Kick 1"},
    {'P',   0,   1,   9, "SC-8850 STANDARD 2 Set", "[88] Room Kick 2"},
    {'P',   0,   1,  10, "SC-8850 STANDARD 2 Set", "[88] Power Kick 1"},
    {'P',   0,   1,  11, "SC-8850 STANDARD 2 Set", "[88] Power Kick 2"},
    {'P',   0,   1,  12, "SC-8850 STANDARD 2 Set", "[88] Electric Kick 2"},
    {'P',   0,   1,  13, "SC-8850 STANDARD 2 Set", "[88] Electric Kick 1"},
    {'P',   0,   1,  14, "SC-8850 STANDARD 2 Set", "[88] TR-808 Kick"},
    {'P',   0,   1,  15, "SC-8850 STANDARD 2 Set", "[88] TR-909 Kick"},
    {'P',   0,   1,  16, "SC-8850 STANDARD 2 Set", "[88] Dance Kick"},
    {'P',   0,   1,  17, "SC-8850 STANDARD 2 Set", "[Pro] Voice One"},
    {'P',   0,   1,  18, "SC-8850 STANDARD 2 Set", "[Pro] Voice Two"},
    {'P',   0,   1,  19, "SC-8850 STANDARD 2 Set", "[Pro] Voice Three"},
    {'P',   0,   1,  22, "SC-8850 STANDARD 2 Set", "MC-500 Beep 1"},
    {'P',   0,   1,  23, "SC-8850 STANDARD 2 Set", "MC-500 Beep 2"},
    {'P',   0,   1,  24, "SC-8850 STANDARD 2 Set", "Concert SD"},
    {'P',   0,   1,  25, "SC-8850 STANDARD 2 Set", "Snare Roll"},
    {'P',   0,   1,  26, "SC-8850 STANDARD 2 Set", "Finger Snap"},
    {'P',   0,   1,  27, "SC-8850 STANDARD 2 Set", "High Q"},
    {'P',   0,   1,  28, "SC-8850 STANDARD 2 Set", "Slap"},
    {'P',   0,   1,  29, "SC-8850 STANDARD 2 Set", "Scratch Push"},
    {'P',   0,   1,  30, "SC-8850 STANDARD 2 Set", "Scratch Pull"},
    {'P',   0,   1,  31, "SC-8850 STANDARD 2 Set", "Sticks"},
    {'P',   0,   1,  32, "SC-8850 STANDARD 2 Set", "Square Click"},
    {'P',   0,   1,  33, "SC-8850 STANDARD 2 Set", "Metronome Click"},
    {'P',   0,   1,  34, "SC-8850 STANDARD 2 Set", "Metronome Bell"},
    {'P',   0,   1,  35, "SC-8850 STANDARD 2 Set", "Standard 2 Kick 2"},
    {'P',   0,   1,  36, "SC-8850 STANDARD 2 Set", "Standard 2 Kick 1"},
    {'P',   0,   1,  37, "SC-8850 STANDARD 2 Set", "Side Stick"},
    {'P',   0,   1,  38, "SC-8850 STANDARD 2 Set", "Standard 2 Snare 1"},
    {'P',   0,   1,  39, "SC-8850 STANDARD 2 Set", "Hand Clap"},
    {'P',   0,   1,  40, "SC-8850 STANDARD 2 Set", "Standard 2 Snare 2"},
    {'P',   0,   1,  41, "SC-8850 STANDARD 2 Set", "Low Tom 2"},
    {'P',   0,   1,  42, "SC-8850 STANDARD 2 Set", "Closed Hi-Hat"},
    {'P',   0,   1,  43, "SC-8850 STANDARD 2 Set", "Low Tom 1"},
    {'P',   0,   1,  44, "SC-8850 STANDARD 2 Set", "Pedal Hi-Hat"},
    {'P',   0,   1,  45, "SC-8850 STANDARD 2 Set", "Mid Tom 2"},
    {'P',   0,   1,  46, "SC-8850 STANDARD 2 Set", "Open Hi-Hat"},
    {'P',   0,   1,  47, "SC-8850 STANDARD 2 Set", "Mid Tom 1"},
    {'P',   0,   1,  48, "SC-8850 STANDARD 2 Set", "High Tom 2"},
    {'P',   0,   1,  49, "SC-8850 STANDARD 2 Set", "Crash Cymbal1"},
    {'P',   0,   1,  50, "SC-8850 STANDARD 2 Set", "High Tom 1"},
    {'P',   0,   1,  51, "SC-8850 STANDARD 2 Set", "Ride Cymbal 1"},
    {'P',   0,   1,  52, "SC-8850 STANDARD 2 Set", "Chinese Cymbal"},
    {'P',   0,   1,  53, "SC-8850 STANDARD 2 Set", "Ride Bell"},
    {'P',   0,   1,  54, "SC-8850 STANDARD 2 Set", "Tambourine"},
    {'P',   0,   1,  55, "SC-8850 STANDARD 2 Set", "Splash Cymbal"},
    {'P',   0,   1,  56, "SC-8850 STANDARD 2 Set", "Cowbell"},
    {'P',   0,   1,  57, "SC-8850 STANDARD 2 Set", "Crash Cymbal 2"},
    {'P',   0,   1,  58, "SC-8850 STANDARD 2 Set", "Vibra-slap"},
    {'P',   0,   1,  59, "SC-8850 STANDARD 2 Set", "Ride Cymbal 2"},
    {'P',   0,   1,  60, "SC-8850 STANDARD 2 Set", "High Bongo"},
    {'P',   0,   1,  61, "SC-8850 STANDARD 2 Set", "Low Bongo"},
    {'P',   0,   1,  62, "SC-8850 STANDARD 2 Set", "Mute High Conga"},
    {'P',   0,   1,  63, "SC-8850 STANDARD 2 Set", "Open High Conga"},
    {'P',   0,   1,  64, "SC-8850 STANDARD 2 Set", "Low Conga"},
    {'P',   0,   1,  65, "SC-8850 STANDARD 2 Set", "High Timbale"},
    {'P',   0,   1,  66, "SC-8850 STANDARD 2 Set", "Low Timbale"},
    {'P',   0,   1,  67, "SC-8850 STANDARD 2 Set", "High Agogo"},
    {'P',   0,   1,  68, "SC-8850 STANDARD 2 Set", "Low Agogo"},
    {'P',   0,   1,  69, "SC-8850 STANDARD 2 Set", "Cabasa"},
    {'P',   0,   1,  70, "SC-8850 STANDARD 2 Set", "Maracas"},
    {'P',   0,   1,  71, "SC-8850 STANDARD 2 Set", "Short High Whistle"},
    {'P',   0,   1,  72, "SC-8850 STANDARD 2 Set", "Long Low Whistle"},
    {'P',   0,   1,  73, "SC-8850 STANDARD 2 Set", "Short Guiro"},
    {'P',   0,   1,  74, "SC-8850 STANDARD 2 Set", "Long Guiro"},
    {'P',   0,   1,  75, "SC-8850 STANDARD 2 Set", "Claves"},
    {'P',   0,   1,  76, "SC-8850 STANDARD 2 Set", "High Wood Block"},
    {'P',   0,   1,  77, "SC-8850 STANDARD 2 Set", "Low Wood Block"},
    {'P',   0,   1,  78, "SC-8850 STANDARD 2 Set", "Mute Cuica"},
    {'P',   0,   1,  79, "SC-8850 STANDARD 2 Set", "Open Cuica"},
    {'P',   0,   1,  80, "SC-8850 STANDARD 2 Set", "Mute Triangle"},
    {'P',   0,   1,  81, "SC-8850 STANDARD 2 Set", "Open Triangle"},
    {'P',   0,   1,  82, "SC-8850 STANDARD 2 Set", "Shaker"},
    {'P',   0,   1,  83, "SC-8850 STANDARD 2 Set", "Jingle Bell"},
    {'P',   0,   1,  84, "SC-8850 STANDARD 2 Set", "Bar Chimes"},
    {'P',   0,   1,  85, "SC-8850 STANDARD 2 Set", "Castanets"},
    {'P',   0,   1,  86, "SC-8850 STANDARD 2 Set", "Mute Surdo"},
    {'P',   0,   1,  87, "SC-8850 STANDARD 2 Set", "Open Surdo"},
    {'P',   0,   1,  88, "SC-8850 STANDARD 2 Set", "Applause 2"},
    {'P',   0,   1,  97, "SC-8850 STANDARD 2 Set", "[88] Standard 1 Snare1"},
    {'P',   0,   1,  98, "SC-8850 STANDARD 2 Set", "[88] Standard 1 Snare 2"},
    {'P',   0,   1,  99, "SC-8850 STANDARD 2 Set", "[88] Standard 2 Snare 1"},
    {'P',   0,   1, 100, "SC-8850 STANDARD 2 Set", "[88] Standard 2 Snare 2"},
    {'P',   0,   1, 101, "SC-8850 STANDARD 2 Set", "[55] Snare Drum 2"},
    {'P',   0,   1, 102, "SC-8850 STANDARD 2 Set", "[Pro] Standard 1 Snare 1"},
    {'P',   0,   1, 103, "SC-8850 STANDARD 2 Set", "[Pro] Standard 1 Snare 2"},
    {'P',   0,   1, 104, "SC-8850 STANDARD 2 Set", "[Pro] Standard 1 Snare 3"},
    {'P',   0,   1, 105, "SC-8850 STANDARD 2 Set", "[88] Jazz Snare 1"},
    {'P',   0,   1, 106, "SC-8850 STANDARD 2 Set", "[88] Jazz Snare 2"},
    {'P',   0,   1, 107, "SC-8850 STANDARD 2 Set", "[88] Room Snare 1"},
    {'P',   0,   1, 108, "SC-8850 STANDARD 2 Set", "[88] Room Snare 2"},
    {'P',   0,   1, 109, "SC-8850 STANDARD 2 Set", "[88] Power Snare 1"},
    {'P',   0,   1, 110, "SC-8850 STANDARD 2 Set", "[88] Power Snare 2"},
    {'P',   0,   1, 111, "SC-8850 STANDARD 2 Set", "[55] Gated Snare"},
    {'P',   0,   1, 112, "SC-8850 STANDARD 2 Set", "[88] Dance Snare 1"},
    {'P',   0,   1, 113, "SC-8850 STANDARD 2 Set", "[88] Dance Snare 2"},
    {'P',   0,   1, 114, "SC-8850 STANDARD 2 Set", "[88] Disco Snare"},
    {'P',   0,   1, 115, "SC-8850 STANDARD 2 Set", "[88] Electric Snare 2"},
    {'P',   0,   1, 116, "SC-8850 STANDARD 2 Set", "[55] Electric Snare"},
    {'P',   0,   1, 117, "SC-8850 STANDARD 2 Set", "[88] Electric Snare 3"},
    {'P',   0,   1, 118, "SC-8850 STANDARD 2 Set", "[Pro] TR-707 Snare 1"},
    {'P',   0,   1, 119, "SC-8850 STANDARD 2 Set", "[88] TR-808 Snare 1"},
    {'P',   0,   1, 120, "SC-8850 STANDARD 2 Set", "[88] TR-808 Snare 2"},
    {'P',   0,   1, 121, "SC-8850 STANDARD 2 Set", "[88] TR-909 Snare 1"},
    {'P',   0,   1, 122, "SC-8850 STANDARD 2 Set", "[88] TR-909 Snare 2"},
    {'P',   0,   1, 123, "SC-8850 STANDARD 2 Set", "[Pro] Rap Snare"},
    {'P',   0,   1, 124, "SC-8850 STANDARD 2 Set", "[Pro] Jungle Snare 1"},
    {'P',   0,   1, 125, "SC-8850 STANDARD 2 Set", "[Pro] House Snare 1"},
    {'P',   0,   1, 126, "SC-8850 STANDARD 2 Set", "[88] House Snare"},
    {'P',   0,   1, 127, "SC-8850 STANDARD 2 Set", "[Pro] House Snare 2"},
    {'P',   0,   2,  17, "SC-8850 STANDARD L/R Set", "[Pro] Voice One"},
    {'P',   0,   2,  18, "SC-8850 STANDARD L/R Set", "[Pro] Voice Two"},
    {'P',   0,   2,  19, "SC-8850 STANDARD L/R Set", "[Pro] Voice Three"},
    {'P',   0,   2,  22, "SC-8850 STANDARD L/R Set", "MC-500 Beep 1"},
    {'P',   0,   2,  23, "SC-8850 STANDARD L/R Set", "MC-500 Beep 2"},
    {'P',   0,   2,  24, "SC-8850 STANDARD L/R Set", "Concert SD"},
    {'P',   0,   2,  25, "SC-8850 STANDARD L/R Set", "Snare Roll"},
    {'P',   0,   2,  26, "SC-8850 STANDARD L/R Set", "Finger Snap 2"},
    {'P',   0,   2,  27, "SC-8850 STANDARD L/R Set", "High Q"},
    {'P',   0,   2,  28, "SC-8850 STANDARD L/R Set", "Slap"},
    {'P',   0,   2,  29, "SC-8850 STANDARD L/R Set", "Scratch Push"},
    {'P',   0,   2,  30, "SC-8850 STANDARD L/R Set", "Scratch Pull"},
    {'P',   0,   2,  31, "SC-8850 STANDARD L/R Set", "Sticks"},
    {'P',   0,   2,  32, "SC-8850 STANDARD L/R Set", "Square Click"},
    {'P',   0,   2,  33, "SC-8850 STANDARD L/R Set", "Metronome Click"},
    {'P',   0,   2,  34, "SC-8850 STANDARD L/R Set", "Metronome Bell"},
    {'P',   0,   2,  35, "SC-8850 STANDARD L/R Set", "[RND] Standard Kick 2"},
    {'P',   0,   2,  36, "SC-8850 STANDARD L/R Set", "[RND] Standard Kick 1"},
    {'P',   0,   2,  37, "SC-8850 STANDARD L/R Set", "Side Stick"},
    {'P',   0,   2,  38, "SC-8850 STANDARD L/R Set", "[RND] Standard Snare 1"},
    {'P',   0,   2,  39, "SC-8850 STANDARD L/R Set", "Hand Clap"},
    {'P',   0,   2,  40, "SC-8850 STANDARD L/R Set", "[RND] Standard Snare 2"},
    {'P',   0,   2,  41, "SC-8850 STANDARD L/R Set", "[RND] Low Tom 2"},
    {'P',   0,   2,  42, "SC-8850 STANDARD L/R Set", "[RND] Closed Hi-Hat"},
    {'P',   0,   2,  43, "SC-8850 STANDARD L/R Set", "[RND] Low Tom 1"},
    {'P',   0,   2,  44, "SC-8850 STANDARD L/R Set", "Pedal Hi-Hat"},
    {'P',   0,   2,  45, "SC-8850 STANDARD L/R Set", "[RND] Mid Tom 2"},
    {'P',   0,   2,  46, "SC-8850 STANDARD L/R Set", "[RND] Open Hi-Hat"},
    {'P',   0,   2,  47, "SC-8850 STANDARD L/R Set", "[RND] Mid Tom 1"},
    {'P',   0,   2,  48, "SC-8850 STANDARD L/R Set", "[RND] High Tom 2"},
    {'P',   0,   2,  49, "SC-8850 STANDARD L/R Set", "[RND] Crash Cymbal"},
    {'P',   0,   2,  50, "SC-8850 STANDARD L/R Set", "[RND] High Tom 1"},
    {'P',   0,   2,  51, "SC-8850 STANDARD L/R Set", "[RND] Ride Cymbal 1"},
    {'P',   0,   2,  52, "SC-8850 STANDARD L/R Set", "Chinese Cymbal"},
    {'P',   0,   2,  53, "SC-8850 STANDARD L/R Set", "[RND] Ride Bell 1"},
    {'P',   0,   2,  54, "SC-8850 STANDARD L/R Set", "Tambourine"},
    {'P',   0,   2,  55, "SC-8850 STANDARD L/R Set", "Splash Cymbal"},
    {'P',   0,   2,  56, "SC-8850 STANDARD L/R Set", "Cowbell"},
    {'P',   0,   2,  57, "SC-8850 STANDARD L/R Set", "Crash Cymbal 2"},
    {'P',   0,   2,  58, "SC-8850 STANDARD L/R Set", "Vibra-slap"},
    {'P',   0,   2,  59, "SC-8850 STANDARD L/R Set", "Ride Cymbal 2"},
    {'P',   0,   2,  60, "SC-8850 STANDARD L/R Set", "High Bongo"},
    {'P',   0,   2,  61, "SC-8850 STANDARD L/R Set", "Low Bongo"},
    {'P',   0,   2,  62, "SC-8850 STANDARD L/R Set", "Mute High Conga"},
    {'P',   0,   2,  63, "SC-8850 STANDARD L/R Set", "Open High Conga"},
    {'P',   0,   2,  64, "SC-8850 STANDARD L/R Set", "Low Conga"},
    {'P',   0,   2,  65, "SC-8850 STANDARD L/R Set", "High Timbale"},
    {'P',   0,   2,  66, "SC-8850 STANDARD L/R Set", "Low Timbale"},
    {'P',   0,   2,  67, "SC-8850 STANDARD L/R Set", "High Agogo"},
    {'P',   0,   2,  68, "SC-8850 STANDARD L/R Set", "Low Agogo"},
    {'P',   0,   2,  69, "SC-8850 STANDARD L/R Set", "Cabasa"},
    {'P',   0,   2,  70, "SC-8850 STANDARD L/R Set", "Maracas"},
    {'P',   0,   2,  71, "SC-8850 STANDARD L/R Set", "Short High Whistle"},
    {'P',   0,   2,  72, "SC-8850 STANDARD L/R Set", "Long Low Whistle"},
    {'P',   0,   2,  73, "SC-8850 STANDARD L/R Set", "Short Guiro"},
    {'P',   0,   2,  74, "SC-8850 STANDARD L/R Set", "Long Guiro"},
    {'P',   0,   2,  75, "SC-8850 STANDARD L/R Set", "Claves"},
    {'P',   0,   2,  76, "SC-8850 STANDARD L/R Set", "High Wood Block"},
    {'P',   0,   2,  77, "SC-8850 STANDARD L/R Set", "Low Wood Block"},
    {'P',   0,   2,  78, "SC-8850 STANDARD L/R Set", "Mute Cuica"},
    {'P',   0,   2,  79, "SC-8850 STANDARD L/R Set", "Open Cuica"},
    {'P',   0,   2,  80, "SC-8850 STANDARD L/R Set", "Mute Triangle"},
    {'P',   0,   2,  81, "SC-8850 STANDARD L/R Set", "Open Triangle"},
    {'P',   0,   2,  82, "SC-8850 STANDARD L/R Set", "Shaker"},
    {'P',   0,   2,  83, "SC-8850 STANDARD L/R Set", "Jingle Bell"},
    {'P',   0,   2,  84, "SC-8850 STANDARD L/R Set", "Bell Tree"},
    {'P',   0,   2,  85, "SC-8850 STANDARD L/R Set", "Castanets"},
    {'P',   0,   2,  86, "SC-8850 STANDARD L/R Set", "Mute Surdo"},
    {'P',   0,   2,  87, "SC-8850 STANDARD L/R Set", "Open Surdo"},
    {'P',   0,   2,  88, "SC-8850 STANDARD L/R Set", "Applause 2"},
    {'P',   0,   2,  95, "SC-8850 STANDARD L/R Set", "[L] Standard Kick 2"},
    {'P',   0,   2,  96, "SC-8850 STANDARD L/R Set", "[L] Standard Kick 1"},
    {'P',   0,   2,  97, "SC-8850 STANDARD L/R Set", "[L] Standard Crash Cymbal"},
    {'P',   0,   2,  98, "SC-8850 STANDARD L/R Set", "[L] Standard Snare 1"},
    {'P',   0,   2,  99, "SC-8850 STANDARD L/R Set", "[L] Standard Ride Cymbal"},
    {'P',   0,   2, 100, "SC-8850 STANDARD L/R Set", "[L] Standard Snare 2"},
    {'P',   0,   2, 101, "SC-8850 STANDARD L/R Set", "[L] Standard Low Tom"},
    {'P',   0,   2, 102, "SC-8850 STANDARD L/R Set", "[L] Standard Closed Hi-Hat"},
    {'P',   0,   2, 103, "SC-8850 STANDARD L/R Set", "[L] Standard Mid Tom"},
    {'P',   0,   2, 104, "SC-8850 STANDARD L/R Set", "[L] Standard Ride Bell"},
    {'P',   0,   2, 105, "SC-8850 STANDARD L/R Set", "[L] Standard High Tom"},
    {'P',   0,   2, 106, "SC-8850 STANDARD L/R Set", "[L] Standard Open Hi-Hat"},
    {'P',   0,   2, 107, "SC-8850 STANDARD L/R Set", "[R] Standard Kick 2"},
    {'P',   0,   2, 108, "SC-8850 STANDARD L/R Set", "[R] Standard Kick 1"},
    {'P',   0,   2, 109, "SC-8850 STANDARD L/R Set", "[R] Standard Crash Cymbal"},
    {'P',   0,   2, 110, "SC-8850 STANDARD L/R Set", "[R] Standard Snare 1"},
    {'P',   0,   2, 111, "SC-8850 STANDARD L/R Set", "[R] Standard Ride Cymbal"},
    {'P',   0,   2, 112, "SC-8850 STANDARD L/R Set", "[R] Standard Snare 2"},
    {'P',   0,   2, 113, "SC-8850 STANDARD L/R Set", "[R] Standard Low Tom"},
    {'P',   0,   2, 114, "SC-8850 STANDARD L/R Set", "[R] Standard Closed Hi-Hat"},
    {'P',   0,   2, 115, "SC-8850 STANDARD L/R Set", "[R] Standard Mid Tom"},
    {'P',   0,   2, 116, "SC-8850 STANDARD L/R Set", "[R] Standard Ride Bell"},
    {'P',   0,   2, 117, "SC-8850 STANDARD L/R Set", "[R] Standard High Tom"},
    {'P',   0,   2, 118, "SC-8850 STANDARD L/R Set", "[R] Standard Open Hi-Hat"},
    {'P',   0,   8,  20, "SC-8850 ROOM Set", "Standard Kick 2"},
    {'P',   0,   8,  21, "SC-8850 ROOM Set", "Standard Kick 1"},
    {'P',   0,   8,  26, "SC-8850 ROOM Set", "Finger Snap"},
    {'P',   0,   8,  29, "SC-8850 ROOM Set", "Scratch Push"},
    {'P',   0,   8,  30, "SC-8850 ROOM Set", "Scratch Pull"},
    {'P',   0,   8,  35, "SC-8850 ROOM Set", "Room Kick 2"},
    {'P',   0,   8,  36, "SC-8850 ROOM Set", "Room Kick 1"},
    {'P',   0,   8,  37, "SC-8850 ROOM Set", "Side Stick"},
    {'P',   0,   8,  38, "SC-8850 ROOM Set", "Room Snare 1"},
    {'P',   0,   8,  39, "SC-8850 ROOM Set", "Hand Clap"},
    {'P',   0,   8,  40, "SC-8850 ROOM Set", "Room Snare 2"},
    {'P',   0,   8,  41, "SC-8850 ROOM Set", "Room Low Tom 2"},
    {'P',   0,   8,  42, "SC-8850 ROOM Set", "Room Closed Hi-Hat 3"},
    {'P',   0,   8,  43, "SC-8850 ROOM Set", "Room Low Tom 1"},
    {'P',   0,   8,  44, "SC-8850 ROOM Set", "Pedal Hi-Hat"},
    {'P',   0,   8,  45, "SC-8850 ROOM Set", "Room Mid Tom 2"},
    {'P',   0,   8,  46, "SC-8850 ROOM Set", "Room Open Hi-Hat 3"},
    {'P',   0,   8,  47, "SC-8850 ROOM Set", "Room Mid Tom 1"},
    {'P',   0,   8,  48, "SC-8850 ROOM Set", "Room High Tom 2"},
    {'P',   0,   8,  49, "SC-8850 ROOM Set", "Room Crash Cymbal"},
    {'P',   0,   8,  50, "SC-8850 ROOM Set", "Room High Tom 1"},
    {'P',   0,   8,  51, "SC-8850 ROOM Set", "Room Ride Cymbal"},
    {'P',   0,   8,  52, "SC-8850 ROOM Set", "Chinese Cymbal"},
    {'P',   0,   8,  53, "SC-8850 ROOM Set", "Room Ride Bell"},
    {'P',   0,   8,  95, "SC-8850 ROOM Set", "Standard 1 Snare 1"},
    {'P',   0,   8,  96, "SC-8850 ROOM Set", "Standard 1 Snare 2"},
    {'P',   0,   9,   0, "SC-8850 HIP HOP Set", "[88] Electric Kick 2"},
    {'P',   0,   9,   1, "SC-8850 HIP HOP Set", "[88] Electric Kick 1"},
    {'P',   0,   9,   2, "SC-8850 HIP HOP Set", "[Pro] CR-78 Kick 1"},
    {'P',   0,   9,   3, "SC-8850 HIP HOP Set", "[Pro] CR-78 Kick 2"},
    {'P',   0,   9,   4, "SC-8850 HIP HOP Set", "[Pro] TR-606 Kick1"},
    {'P',   0,   9,   5, "SC-8850 HIP HOP Set", "[Pro] TR-707 Kick 1"},
    {'P',   0,   9,   6, "SC-8850 HIP HOP Set", "[55] TR-808 Kick"},
    {'P',   0,   9,   7, "SC-8850 HIP HOP Set", "[88] TR-808 Kick"},
    {'P',   0,   9,   8, "SC-8850 HIP HOP Set", "[Pro] TR-808 Kick 2"},
    {'P',   0,   9,   9, "SC-8850 HIP HOP Set", "[88] TR-909 Kick"},
    {'P',   0,   9,  10, "SC-8850 HIP HOP Set", "[88] Dance Kick"},
    {'P',   0,   9,  11, "SC-8850 HIP HOP Set", "[Pro] Hip-Hop Kick 2"},
    {'P',   0,   9,  12, "SC-8850 HIP HOP Set", "[Pro] TR-909 Kick 1"},
    {'P',   0,   9,  13, "SC-8850 HIP HOP Set", "[Pro] Hip-Hop Kick 3"},
    {'P',   0,   9,  14, "SC-8850 HIP HOP Set", "[Pro] Jungle Kick 1"},
    {'P',   0,   9,  15, "SC-8850 HIP HOP Set", "[Pro] Techno Kick 1"},
    {'P',   0,   9,  16, "SC-8850 HIP HOP Set", "[Pro] Bounce Kick"},
    {'P',   0,   9,  17, "SC-8850 HIP HOP Set", "[Pro] Voice One"},
    {'P',   0,   9,  18, "SC-8850 HIP HOP Set", "[Pro] Voice Two"},
    {'P',   0,   9,  19, "SC-8850 HIP HOP Set", "[Pro] Voice Three"},
    {'P',   0,   9,  20, "SC-8850 HIP HOP Set", "Jungle Kick 2"},
    {'P',   0,   9,  21, "SC-8850 HIP HOP Set", "Jungle Kick 1"},
    {'P',   0,   9,  22, "SC-8850 HIP HOP Set", "MC-500 Beep 1"},
    {'P',   0,   9,  23, "SC-8850 HIP HOP Set", "MC-500 Beep 2"},
    {'P',   0,   9,  24, "SC-8850 HIP HOP Set", "Concert SD"},
    {'P',   0,   9,  25, "SC-8850 HIP HOP Set", "Snare Roll"},
    {'P',   0,   9,  26, "SC-8850 HIP HOP Set", "Finger Snap 2"},
    {'P',   0,   9,  27, "SC-8850 HIP HOP Set", "High Q"},
    {'P',   0,   9,  28, "SC-8850 HIP HOP Set", "Slap"},
    {'P',   0,   9,  29, "SC-8850 HIP HOP Set", "Scratch Push 2"},
    {'P',   0,   9,  30, "SC-8850 HIP HOP Set", "Scratch Pull 2"},
    {'P',   0,   9,  31, "SC-8850 HIP HOP Set", "Sticks"},
    {'P',   0,   9,  32, "SC-8850 HIP HOP Set", "Square Click"},
    {'P',   0,   9,  33, "SC-8850 HIP HOP Set", "Metronome Click"},
    {'P',   0,   9,  34, "SC-8850 HIP HOP Set", "Metronome Bell"},
    {'P',   0,   9,  35, "SC-8850 HIP HOP Set", "Hip-Hop Kick 2"},
    {'P',   0,   9,  36, "SC-8850 HIP HOP Set", "Hip-Hop Kick 1"},
    {'P',   0,   9,  37, "SC-8850 HIP HOP Set", "TR-808 Rim Shot"},
    {'P',   0,   9,  38, "SC-8850 HIP HOP Set", "LoFi Snare 1"},
    {'P',   0,   9,  39, "SC-8850 HIP HOP Set", "TR-707 Claps"},
    {'P',   0,   9,  40, "SC-8850 HIP HOP Set", "LoFi Snare 2"},
    {'P',   0,   9,  41, "SC-8850 HIP HOP Set", "Jazz Low Tom 2"},
    {'P',   0,   9,  42, "SC-8850 HIP HOP Set", "Room Closed Hi-Hat"},
    {'P',   0,   9,  43, "SC-8850 HIP HOP Set", "Jazz Low Tom 1"},
    {'P',   0,   9,  44, "SC-8850 HIP HOP Set", "Pedal Hi-Hat"},
    {'P',   0,   9,  45, "SC-8850 HIP HOP Set", "Jazz Mid Tom 2"},
    {'P',   0,   9,  46, "SC-8850 HIP HOP Set", "Room Open Hi-Hat"},
    {'P',   0,   9,  47, "SC-8850 HIP HOP Set", "Jazz Mid Tom 1"},
    {'P',   0,   9,  48, "SC-8850 HIP HOP Set", "Jazz High Tom 2"},
    {'P',   0,   9,  49, "SC-8850 HIP HOP Set", "TR-909 Crash Cymbal"},
    {'P',   0,   9,  50, "SC-8850 HIP HOP Set", "Jazz High Tom 1"},
    {'P',   0,   9,  51, "SC-8850 HIP HOP Set", "Ride Cymbal1"},
    {'P',   0,   9,  52, "SC-8850 HIP HOP Set", "Reverse Cymbal"},
    {'P',   0,   9,  53, "SC-8850 HIP HOP Set", "Ride Bell"},
    {'P',   0,   9,  54, "SC-8850 HIP HOP Set", "Shake Tambourine"},
    {'P',   0,   9,  55, "SC-8850 HIP HOP Set", "Splash Cymbal"},
    {'P',   0,   9,  56, "SC-8850 HIP HOP Set", "TR-808 Cowbell"},
    {'P',   0,   9,  57, "SC-8850 HIP HOP Set", "Crash Cymbal 2"},
    {'P',   0,   9,  58, "SC-8850 HIP HOP Set", "Vibra-slap"},
    {'P',   0,   9,  59, "SC-8850 HIP HOP Set", "Ride Cymbal 2"},
    {'P',   0,   9,  60, "SC-8850 HIP HOP Set", "High Bongo"},
    {'P',   0,   9,  61, "SC-8850 HIP HOP Set", "Low Bongo"},
    {'P',   0,   9,  62, "SC-8850 HIP HOP Set", "Mute High Conga"},
    {'P',   0,   9,  63, "SC-8850 HIP HOP Set", "Open High Conga"},
    {'P',   0,   9,  64, "SC-8850 HIP HOP Set", "Low Conga"},
    {'P',   0,   9,  65, "SC-8850 HIP HOP Set", "High Timbale"},
    {'P',   0,   9,  66, "SC-8850 HIP HOP Set", "Low Timbale"},
    {'P',   0,   9,  67, "SC-8850 HIP HOP Set", "High Agogo"},
    {'P',   0,   9,  68, "SC-8850 HIP HOP Set", "Low Agogo"},
    {'P',   0,   9,  69, "SC-8850 HIP HOP Set", "Cabasa"},
    {'P',   0,   9,  70, "SC-8850 HIP HOP Set", "TR-808 Maracas"},
    {'P',   0,   9,  71, "SC-8850 HIP HOP Set", "Short High Whistle"},
    {'P',   0,   9,  72, "SC-8850 HIP HOP Set", "Long Low Whistle"},
    {'P',   0,   9,  73, "SC-8850 HIP HOP Set", "Short Guiro"},
    {'P',   0,   9,  74, "SC-8850 HIP HOP Set", "CR-78 Guiro"},
    {'P',   0,   9,  75, "SC-8850 HIP HOP Set", "TR-808 Claves"},
    {'P',   0,   9,  76, "SC-8850 HIP HOP Set", "High Wood Block"},
    {'P',   0,   9,  77, "SC-8850 HIP HOP Set", "Low Wood Block"},
    {'P',   0,   9,  78, "SC-8850 HIP HOP Set", "High Hoo"},
    {'P',   0,   9,  79, "SC-8850 HIP HOP Set", "Low Hoo"},
    {'P',   0,   9,  80, "SC-8850 HIP HOP Set", "Electric Mute Triangle"},
    {'P',   0,   9,  81, "SC-8850 HIP HOP Set", "Electric Open Triangle"},
    {'P',   0,   9,  82, "SC-8850 HIP HOP Set", "Shaker 2"},
    {'P',   0,   9,  83, "SC-8850 HIP HOP Set", "Jingle Bell"},
    {'P',   0,   9,  84, "SC-8850 HIP HOP Set", "Bell Tree"},
    {'P',   0,   9,  85, "SC-8850 HIP HOP Set", "Castanets"},
    {'P',   0,   9,  86, "SC-8850 HIP HOP Set", "Mute Surdo"},
    {'P',   0,   9,  87, "SC-8850 HIP HOP Set", "Open Surdo"},
    {'P',   0,   9,  88, "SC-8850 HIP HOP Set", "Small Club 1"},
    {'P',   0,   9,  89, "SC-8850 HIP HOP Set", "Hip-Hop Snare 2"},
    {'P',   0,   9,  90, "SC-8850 HIP HOP Set", "LoFi Snare Rim"},
    {'P',   0,   9,  91, "SC-8850 HIP HOP Set", "Hip-Hop Claps"},
    {'P',   0,   9,  92, "SC-8850 HIP HOP Set", "Stantard 1 Snare 1"},
    {'P',   0,   9,  93, "SC-8850 HIP HOP Set", "Standard 1 Snare 2"},
    {'P',   0,   9,  94, "SC-8850 HIP HOP Set", "Room Snare 1"},
    {'P',   0,   9,  95, "SC-8850 HIP HOP Set", "Room Snare 2"},
    {'P',   0,   9,  96, "SC-8850 HIP HOP Set", "Dance Snare"},
    {'P',   0,   9,  97, "SC-8850 HIP HOP Set", "[Pro] Techno Hit"},
    {'P',   0,   9,  98, "SC-8850 HIP HOP Set", "[Pro] Philly Hit"},
    {'P',   0,   9,  99, "SC-8850 HIP HOP Set", "[Pro] Impact Hit"},
    {'P',   0,   9, 100, "SC-8850 HIP HOP Set", "[Pro] Lo-Fi Rave"},
    {'P',   0,   9, 101, "SC-8850 HIP HOP Set", "[Pro] Bam Hit"},
    {'P',   0,   9, 102, "SC-8850 HIP HOP Set", "[Pro] Bim Hit"},
    {'P',   0,   9, 103, "SC-8850 HIP HOP Set", "[Pro] Tape Rewind"},
    {'P',   0,   9, 104, "SC-8850 HIP HOP Set", "[Pro] Phonograph Noise"},
    {'P',   0,   9, 105, "SC-8850 HIP HOP Set", "[88] Power Snare 1"},
    {'P',   0,   9, 106, "SC-8850 HIP HOP Set", "[88] Dance Snare 1"},
    {'P',   0,   9, 107, "SC-8850 HIP HOP Set", "[88] Dance Snare 2"},
    {'P',   0,   9, 108, "SC-8850 HIP HOP Set", "[88] Disco Snare"},
    {'P',   0,   9, 109, "SC-8850 HIP HOP Set", "[88] Electric Snare 2"},
    {'P',   0,   9, 110, "SC-8850 HIP HOP Set", "[55] Electric Snare"},
    {'P',   0,   9, 111, "SC-8850 HIP HOP Set", "[88] Electric Snare 3"},
    {'P',   0,   9, 112, "SC-8850 HIP HOP Set", "[Pro] TR-606 Snare 2"},
    {'P',   0,   9, 113, "SC-8850 HIP HOP Set", "[Pro] TR-707 Snare 1"},
    {'P',   0,   9, 114, "SC-8850 HIP HOP Set", "[88] TR-808 Snare 2"},
    {'P',   0,   9, 115, "SC-8850 HIP HOP Set", "[88] TR-808 Snare 1"},
    {'P',   0,   9, 116, "SC-8850 HIP HOP Set", "[Pro] TR-808 Snare 2"},
    {'P',   0,   9, 117, "SC-8850 HIP HOP Set", "[88] TR-909 Snare 1"},
    {'P',   0,   9, 118, "SC-8850 HIP HOP Set", "[88] TR-909 Snare 2"},
    {'P',   0,   9, 119, "SC-8850 HIP HOP Set", "[Pro] TR-909 Snare 1"},
    {'P',   0,   9, 120, "SC-8850 HIP HOP Set", "[Pro] TR-909 Snare 2"},
    {'P',   0,   9, 121, "SC-8850 HIP HOP Set", "[Pro] Rap Snare"},
    {'P',   0,   9, 122, "SC-8850 HIP HOP Set", "[Pro] Jungle Snare"},
    {'P',   0,   9, 123, "SC-8850 HIP HOP Set", "[Pro] House Snare 1"},
    {'P',   0,   9, 124, "SC-8850 HIP HOP Set", "[88] House Snare"},
    {'P',   0,   9, 125, "SC-8850 HIP HOP Set", "[Pro] House Snare 2"},
    {'P',   0,   9, 126, "SC-8850 HIP HOP Set", "[Pro] Voice Tah"},
    {'P',   0,   9, 127, "SC-8850 HIP HOP Set", "[88] Slappy"},
    {'P',   0,  10,  20, "SC-8850 JUNGLE Set", "HipHop Kick 2"},
    {'P',   0,  10,  21, "SC-8850 JUNGLE Set", "HipHop Kick 1"},
    {'P',   0,  10,  26, "SC-8850 JUNGLE Set", "Finger Snap 2"},
    {'P',   0,  10,  29, "SC-8850 JUNGLE Set", "Scratch Push 2"},
    {'P',   0,  10,  30, "SC-8850 JUNGLE Set", "Scratch Pull 2"},
    {'P',   0,  10,  35, "SC-8850 JUNGLE Set", "Jungle Kick 2"},
    {'P',   0,  10,  36, "SC-8850 JUNGLE Set", "Jungle Kick 1"},
    {'P',   0,  10,  37, "SC-8850 JUNGLE Set", "Jungle Snare Rim"},
    {'P',   0,  10,  38, "SC-8850 JUNGLE Set", "HipHop Snare 1"},
    {'P',   0,  10,  39, "SC-8850 JUNGLE Set", "R&B Claps"},
    {'P',   0,  10,  40, "SC-8850 JUNGLE Set", "Jungle Snare"},
    {'P',   0,  10,  41, "SC-8850 JUNGLE Set", "TR-909 Low Tom 2"},
    {'P',   0,  10,  42, "SC-8850 JUNGLE Set", "TR-606 Closed Hi-Hat"},
    {'P',   0,  10,  43, "SC-8850 JUNGLE Set", "TR-909 Low Tom 1"},
    {'P',   0,  10,  44, "SC-8850 JUNGLE Set", "Jungle Hi-Hat"},
    {'P',   0,  10,  45, "SC-8850 JUNGLE Set", "TR-909 Mid Tom 2"},
    {'P',   0,  10,  46, "SC-8850 JUNGLE Set", "TR-606 Open Hi-Hat"},
    {'P',   0,  10,  47, "SC-8850 JUNGLE Set", "TR-909 Mid Tom 1"},
    {'P',   0,  10,  48, "SC-8850 JUNGLE Set", "TR-909 High Tom 2"},
    {'P',   0,  10,  49, "SC-8850 JUNGLE Set", "Jungle Crash"},
    {'P',   0,  10,  50, "SC-8850 JUNGLE Set", "TR-909 High Tom 1"},
    {'P',   0,  10,  51, "SC-8850 JUNGLE Set", "Ride Cymbal 1"},
    {'P',   0,  10,  52, "SC-8850 JUNGLE Set", "Reverse Cymabal"},
    {'P',   0,  10,  53, "SC-8850 JUNGLE Set", "Ride Bell"},
    {'P',   0,  10,  54, "SC-8850 JUNGLE Set", "Shake Tambourine"},
    {'P',   0,  10,  55, "SC-8850 JUNGLE Set", "Splash Cymbal"},
    {'P',   0,  10,  56, "SC-8850 JUNGLE Set", "TR808 Cowbell"},
    {'P',   0,  10,  74, "SC-8850 JUNGLE Set", "CR-78 Guiro"},
    {'P',   0,  10,  75, "SC-8850 JUNGLE Set", "TR-808 Claves"},
    {'P',   0,  10,  78, "SC-8850 JUNGLE Set", "High Hoo"},
    {'P',   0,  10,  79, "SC-8850 JUNGLE Set", "Low Hoo"},
    {'P',   0,  10,  80, "SC-8850 JUNGLE Set", "Electric Mute Triangle"},
    {'P',   0,  10,  81, "SC-8850 JUNGLE Set", "Electric Open Triangle"},
    {'P',   0,  10,  82, "SC-8850 JUNGLE Set", "Jungle Shaker"},
    {'P',   0,  10,  88, "SC-8850 JUNGLE Set", "Small Club 1"},
    {'P',   0,  10,  89, "SC-8850 JUNGLE Set", "Jungle Kick Roll"},
    {'P',   0,  10,  90, "SC-8850 JUNGLE Set", "Jungle Snare Roll"},
    {'P',   0,  10,  91, "SC-8850 JUNGLE Set", "TR-606 Snare 2"},
    {'P',   0,  10,  92, "SC-8850 JUNGLE Set", "Dance Snare"},
    {'P',   0,  10,  93, "SC-8850 JUNGLE Set", "Techno Snare"},
    {'P',   0,  10,  94, "SC-8850 JUNGLE Set", "House Snare"},
    {'P',   0,  10,  95, "SC-8850 JUNGLE Set", "Rock Snare Dry"},
    {'P',   0,  10,  96, "SC-8850 JUNGLE Set", "LoFi Snare 1"},
    {'P',   0,  11,  20, "SC-8850 TECHNO Set", "TR-909 Kick 2"},
    {'P',   0,  11,  21, "SC-8850 TECHNO Set", "Fat Kick"},
    {'P',   0,  11,  26, "SC-8850 TECHNO Set", "Finger Snap 2"},
    {'P',   0,  11,  29, "SC-8850 TECHNO Set", "Scratch Push 2"},
    {'P',   0,  11,  30, "SC-8850 TECHNO Set", "Scratch Pull 2"},
    {'P',   0,  11,  35, "SC-8850 TECHNO Set", "TR-808 Kick"},
    {'P',   0,  11,  36, "SC-8850 TECHNO Set", "TR-909 Kick 1"},
    {'P',   0,  11,  37, "SC-8850 TECHNO Set", "TR-909 Snare Rim"},
    {'P',   0,  11,  38, "SC-8850 TECHNO Set", "TR-606 Snare 2"},
    {'P',   0,  11,  39, "SC-8850 TECHNO Set", "TR-909 Claps"},
    {'P',   0,  11,  40, "SC-8850 TECHNO Set", "Techno Snare"},
    {'P',   0,  11,  41, "SC-8850 TECHNO Set", "TR-606 Low Tom 2"},
    {'P',   0,  11,  42, "SC-8850 TECHNO Set", "TR-707 Closed Hi-Hat"},
    {'P',   0,  11,  43, "SC-8850 TECHNO Set", "TR-606 Low Tom 1"},
    {'P',   0,  11,  44, "SC-8850 TECHNO Set", "CR-78 Closed Hi-Hat"},
    {'P',   0,  11,  45, "SC-8850 TECHNO Set", "TR-606 Mid Tom 2"},
    {'P',   0,  11,  46, "SC-8850 TECHNO Set", "TR-909 Open Hi-Hat"},
    {'P',   0,  11,  47, "SC-8850 TECHNO Set", "TR-606 Mid Tom 1"},
    {'P',   0,  11,  48, "SC-8850 TECHNO Set", "TR-606 High Tom 2"},
    {'P',   0,  11,  49, "SC-8850 TECHNO Set", "TR-909 Crash Cymbal"},
    {'P',   0,  11,  50, "SC-8850 TECHNO Set", "TR-606 High Tom 1"},
    {'P',   0,  11,  51, "SC-8850 TECHNO Set", "Ride Cymbal 1"},
    {'P',   0,  11,  52, "SC-8850 TECHNO Set", "Reverse Cymbal"},
    {'P',   0,  11,  53, "SC-8850 TECHNO Set", "Ride Bell"},
    {'P',   0,  11,  54, "SC-8850 TECHNO Set", "Shake Tambourine"},
    {'P',   0,  11,  55, "SC-8850 TECHNO Set", "Splash Cymbal"},
    {'P',   0,  11,  56, "SC-8850 TECHNO Set", "TR-808Cowbell"},
    {'P',   0,  11,  57, "SC-8850 TECHNO Set", "TR-909 Crash Cymbal"},
    {'P',   0,  11,  60, "SC-8850 TECHNO Set", "CR-78 High Bongo"},
    {'P',   0,  11,  61, "SC-8850 TECHNO Set", "CR-78 Low Bongo"},
    {'P',   0,  11,  62, "SC-8850 TECHNO Set", "TR-808 High Conga"},
    {'P',   0,  11,  63, "SC-8850 TECHNO Set", "TR-808 Mute Conga"},
    {'P',   0,  11,  64, "SC-8850 TECHNO Set", "TR-808 Low Conga"},
    {'P',   0,  11,  70, "SC-8850 TECHNO Set", "TR-808 Maracas"},
    {'P',   0,  11,  74, "SC-8850 TECHNO Set", "CR-78 Guiro"},
    {'P',   0,  11,  75, "SC-8850 TECHNO Set", "TR-808 Claves"},
    {'P',   0,  11,  78, "SC-8850 TECHNO Set", "High Hoo"},
    {'P',   0,  11,  79, "SC-8850 TECHNO Set", "Low Hoo"},
    {'P',   0,  11,  80, "SC-8850 TECHNO Set", "Electric Mute Triangle"},
    {'P',   0,  11,  81, "SC-8850 TECHNO Set", "Electric Open Triangle"},
    {'P',   0,  11,  82, "SC-8850 TECHNO Set", "TR-626 Shaker"},
    {'P',   0,  11,  88, "SC-8850 TECHNO Set", "Applause 2"},
    {'P',   0,  11,  89, "SC-8850 TECHNO Set", "Dance Snare"},
    {'P',   0,  11,  90, "SC-8850 TECHNO Set", "House Snare"},
    {'P',   0,  11,  91, "SC-8850 TECHNO Set", "Rock Snare Dry"},
    {'P',   0,  11,  92, "SC-8850 TECHNO Set", "Jungle Snare"},
    {'P',   0,  11,  93, "SC-8850 TECHNO Set", "LoFi Snare 1"},
    {'P',   0,  11,  94, "SC-8850 TECHNO Set", "LoFi Snare 2"},
    {'P',   0,  11,  95, "SC-8850 TECHNO Set", "HipHop Snare 1"},
    {'P',   0,  11,  96, "SC-8850 TECHNO Set", "HipHop Snare 2"},
    {'P',   0,  12,  26, "SC-8850 ROOM L/R Set", "Finger Snap"},
    {'P',   0,  12,  29, "SC-8850 ROOM L/R Set", "Scratch Push"},
    {'P',   0,  12,  30, "SC-8850 ROOM L/R Set", "Scratch Pull"},
    {'P',   0,  12,  35, "SC-8850 ROOM L/R Set", "[RND] Room Kick 2"},
    {'P',   0,  12,  36, "SC-8850 ROOM L/R Set", "[RND] Room Kick 1"},
    {'P',   0,  12,  37, "SC-8850 ROOM L/R Set", "Side Stick"},
    {'P',   0,  12,  38, "SC-8850 ROOM L/R Set", "[RND] Room Snare 1"},
    {'P',   0,  12,  39, "SC-8850 ROOM L/R Set", "Hand Clap"},
    {'P',   0,  12,  40, "SC-8850 ROOM L/R Set", "[RND] Room Snare 2"},
    {'P',   0,  12,  41, "SC-8850 ROOM L/R Set", "Room Low Tom 2"},
    {'P',   0,  12,  42, "SC-8850 ROOM L/R Set", "[RND] Room Closed Hi-Hat"},
    {'P',   0,  12,  43, "SC-8850 ROOM L/R Set", "Room Low Tom 1"},
    {'P',   0,  12,  44, "SC-8850 ROOM L/R Set", "Pedal Hi-Hat"},
    {'P',   0,  12,  45, "SC-8850 ROOM L/R Set", "Room Mid Tom 2"},
    {'P',   0,  12,  46, "SC-8850 ROOM L/R Set", "[RND] Room Open Hi-Hat"},
    {'P',   0,  12,  47, "SC-8850 ROOM L/R Set", "Room Mid Tom 1"},
    {'P',   0,  12,  48, "SC-8850 ROOM L/R Set", "Room High Tom 2"},
    {'P',   0,  12,  49, "SC-8850 ROOM L/R Set", "[RND] Room Crash Cymbal"},
    {'P',   0,  12,  50, "SC-8850 ROOM L/R Set", "Room High Tom 1"},
    {'P',   0,  12,  51, "SC-8850 ROOM L/R Set", "[RND] Room Ride Cymbal"},
    {'P',   0,  12,  52, "SC-8850 ROOM L/R Set", "Chinese Cymbal"},
    {'P',   0,  12,  53, "SC-8850 ROOM L/R Set", "[RND] Room Ride Bell"},
    {'P',   0,  12,  54, "SC-8850 ROOM L/R Set", "Tambourine"},
    {'P',   0,  12,  55, "SC-8850 ROOM L/R Set", "Splash Cymbal"},
    {'P',   0,  12,  56, "SC-8850 ROOM L/R Set", "Cowbell"},
    {'P',   0,  12,  88, "SC-8850 ROOM L/R Set", "Applause 2"},
    {'P',   0,  12,  95, "SC-8850 ROOM L/R Set", "[L] Room Kick 2"},
    {'P',   0,  12,  96, "SC-8850 ROOM L/R Set", "[L] Room Kick 1"},
    {'P',   0,  12,  97, "SC-8850 ROOM L/R Set", "[L] Room Crash Cymbal"},
    {'P',   0,  12,  98, "SC-8850 ROOM L/R Set", "[L] Room Snare 1"},
    {'P',   0,  12,  99, "SC-8850 ROOM L/R Set", "[L] Room Ride Cymbal"},
    {'P',   0,  12, 100, "SC-8850 ROOM L/R Set", "[L] Room Snare 2"},
    {'P',   0,  12, 101, "SC-8850 ROOM L/R Set", "[L] Room Low Tom"},
    {'P',   0,  12, 102, "SC-8850 ROOM L/R Set", "[L] Room Closed Hi-Hat"},
    {'P',   0,  12, 103, "SC-8850 ROOM L/R Set", "[L] Room Mid Tom"},
    {'P',   0,  12, 104, "SC-8850 ROOM L/R Set", "[L] Room Ride Bell"},
    {'P',   0,  12, 105, "SC-8850 ROOM L/R Set", "[L] Room High Tom"},
    {'P',   0,  12, 106, "SC-8850 ROOM L/R Set", "[L] Room Open Hi-Hat"},
    {'P',   0,  12, 107, "SC-8850 ROOM L/R Set", "[R] Room Kick 2"},
    {'P',   0,  12, 108, "SC-8850 ROOM L/R Set", "[R] Room Kick 1"},
    {'P',   0,  12, 109, "SC-8850 ROOM L/R Set", "[R] Room Crash Cymbal"},
    {'P',   0,  12, 110, "SC-8850 ROOM L/R Set", "[R] Room Snare 1"},
    {'P',   0,  12, 111, "SC-8850 ROOM L/R Set", "[R] Room Ride Cymbal"},
    {'P',   0,  12, 112, "SC-8850 ROOM L/R Set", "[R] Room Snare 2"},
    {'P',   0,  12, 113, "SC-8850 ROOM L/R Set", "[R] Room Low Tom"},
    {'P',   0,  12, 114, "SC-8850 ROOM L/R Set", "[R] Room Closed Hi-Hat"},
    {'P',   0,  12, 115, "SC-8850 ROOM L/R Set", "[R] Room Mid Tom"},
    {'P',   0,  12, 116, "SC-8850 ROOM L/R Set", "[R] Room Ride Bell"},
    {'P',   0,  12, 117, "SC-8850 ROOM L/R Set", "[R] Room High Tom"},
    {'P',   0,  12, 118, "SC-8850 ROOM L/R Set", "[R] Room Open Hi-Hat"},
    {'P',   0,  13,  20, "SC-8850 HOUSE Set", "Fat Kick"},
    {'P',   0,  13,  21, "SC-8850 HOUSE Set", "Dance Kick"},
    {'P',   0,  13,  26, "SC-8850 HOUSE Set", "Finger Snap 2"},
    {'P',   0,  13,  29, "SC-8850 HOUSE Set", "Scratch Push 2"},
    {'P',   0,  13,  30, "SC-8850 HOUSE Set", "Scratch Pull 2"},
    {'P',   0,  13,  35, "SC-8850 HOUSE Set", "TR-909 Kick 2"},
    {'P',   0,  13,  36, "SC-8850 HOUSE Set", "TR-909 Kick 1"},
    {'P',   0,  13,  37, "SC-8850 HOUSE Set", "House Snare Rim"},
    {'P',   0,  13,  38, "SC-8850 HOUSE Set", "House Snare 1"},
    {'P',   0,  13,  39, "SC-8850 HOUSE Set", "TR-909 Claps"},
    {'P',   0,  13,  40, "SC-8850 HOUSE Set", "House Snare 2"},
    {'P',   0,  13,  41, "SC-8850 HOUSE Set", "TR-909 Low Tom 2"},
    {'P',   0,  13,  42, "SC-8850 HOUSE Set", "Room Closed Hi-Hat"},
    {'P',   0,  13,  43, "SC-8850 HOUSE Set", "TR-909 Low Tom 1"},
    {'P',   0,  13,  44, "SC-8850 HOUSE Set", "Pedal Hi-Hat"},
    {'P',   0,  13,  45, "SC-8850 HOUSE Set", "TR-909 Mid Tom 2"},
    {'P',   0,  13,  46, "SC-8850 HOUSE Set", "Room Open Hi-Hat"},
    {'P',   0,  13,  47, "SC-8850 HOUSE Set", "TR-909 Mid Tom 1"},
    {'P',   0,  13,  48, "SC-8850 HOUSE Set", "TR-909 High Tom 2"},
    {'P',   0,  13,  49, "SC-8850 HOUSE Set", "TR-909 Crash Cymbal"},
    {'P',   0,  13,  50, "SC-8850 HOUSE Set", "TR-909 High Tom 1"},
    {'P',   0,  13,  51, "SC-8850 HOUSE Set", "TR-909 Ride Cymbal"},
    {'P',   0,  13,  52, "SC-8850 HOUSE Set", "Reverse Cymbal"},
    {'P',   0,  13,  53, "SC-8850 HOUSE Set", "Ride Bell"},
    {'P',   0,  13,  54, "SC-8850 HOUSE Set", "Shake Tambourine"},
    {'P',   0,  13,  55, "SC-8850 HOUSE Set", "Splash Cymbal"},
    {'P',   0,  13,  56, "SC-8850 HOUSE Set", "TR-808 Cowbell"},
    {'P',   0,  13,  57, "SC-8850 HOUSE Set", "TR-909 Crash Cymbal"},
    {'P',   0,  13,  60, "SC-8850 HOUSE Set", "CR-78 High Bongo"},
    {'P',   0,  13,  61, "SC-8850 HOUSE Set", "CR-78 Low Bongo"},
    {'P',   0,  13,  62, "SC-8850 HOUSE Set", "TR-808 High Conga"},
    {'P',   0,  13,  63, "SC-8850 HOUSE Set", "TR-808 Mute Conga"},
    {'P',   0,  13,  64, "SC-8850 HOUSE Set", "TR-808 Low Conga"},
    {'P',   0,  13,  70, "SC-8850 HOUSE Set", "TR-808 Maracas"},
    {'P',   0,  13,  74, "SC-8850 HOUSE Set", "CR-78 Guiro"},
    {'P',   0,  13,  75, "SC-8850 HOUSE Set", "TR-808 Claves"},
    {'P',   0,  13,  78, "SC-8850 HOUSE Set", "High Hoo"},
    {'P',   0,  13,  79, "SC-8850 HOUSE Set", "Low Hoo"},
    {'P',   0,  13,  80, "SC-8850 HOUSE Set", "Electric Mute Triangle"},
    {'P',   0,  13,  81, "SC-8850 HOUSE Set", "Electric Open Triangle"},
    {'P',   0,  13,  82, "SC-8850 HOUSE Set", "TR-626 Shaker"},
    {'P',   0,  13,  88, "SC-8850 HOUSE Set", "Small Club 1"},
    {'P',   0,  13,  89, "SC-8850 HOUSE Set", "TR-606 Snare 2"},
    {'P',   0,  13,  90, "SC-8850 HOUSE Set", "Dance Snare"},
    {'P',   0,  13,  91, "SC-8850 HOUSE Set", "Techno Snare"},
    {'P',   0,  13,  92, "SC-8850 HOUSE Set", "Rock Snare Dry"},
    {'P',   0,  13,  93, "SC-8850 HOUSE Set", "Hip-Hop Snare 1"},
    {'P',   0,  13,  94, "SC-8850 HOUSE Set", "LoFi Snare 1"},
    {'P',   0,  13,  95, "SC-8850 HOUSE Set", "LoFi Snare 2"},
    {'P',   0,  13,  96, "SC-8850 HOUSE Set", "Jungle Snare"},
    {'P',   0,  16,  26, "SC-8850 POWER Set", "Finger Snap 2"},
    {'P',   0,  16,  29, "SC-8850 POWER Set", "Scratch Push"},
    {'P',   0,  16,  30, "SC-8850 POWER Set", "Scratch Pull"},
    {'P',   0,  16,  35, "SC-8850 POWER Set", "Power Kick 2"},
    {'P',   0,  16,  36, "SC-8850 POWER Set", "Power Kick 1"},
    {'P',   0,  16,  37, "SC-8850 POWER Set", "Side Stick"},
    {'P',   0,  16,  38, "SC-8850 POWER Set", "Power Snare 1"},
    {'P',   0,  16,  39, "SC-8850 POWER Set", "Hand Clap"},
    {'P',   0,  16,  40, "SC-8850 POWER Set", "Power Snare 2"},
    {'P',   0,  16,  41, "SC-8850 POWER Set", "Power Low Tom 2"},
    {'P',   0,  16,  42, "SC-8850 POWER Set", "Closed Hi-Hat 1"},
    {'P',   0,  16,  43, "SC-8850 POWER Set", "Power Low Tom 1"},
    {'P',   0,  16,  44, "SC-8850 POWER Set", "Pedal Hi-Hat"},
    {'P',   0,  16,  45, "SC-8850 POWER Set", "Power Mid Tom 2"},
    {'P',   0,  16,  46, "SC-8850 POWER Set", "Open Hi-Hat 1"},
    {'P',   0,  16,  47, "SC-8850 POWER Set", "Power Mid Tom 1"},
    {'P',   0,  16,  48, "SC-8850 POWER Set", "Power High Tom 2"},
    {'P',   0,  16,  49, "SC-8850 POWER Set", "Crash Cymbal 1"},
    {'P',   0,  16,  50, "SC-8850 POWER Set", "Power High Tom 1"},
    {'P',   0,  16,  51, "SC-8850 POWER Set", "Ride Cymbal 1"},
    {'P',   0,  16,  52, "SC-8850 POWER Set", "Chinese Cymbal"},
    {'P',   0,  16,  53, "SC-8850 POWER Set", "Ride Bell"},
    {'P',   0,  16,  54, "SC-8850 POWER Set", "Tambourine"},
    {'P',   0,  16,  55, "SC-8850 POWER Set", "Splash Cymbal"},
    {'P',   0,  16,  56, "SC-8850 POWER Set", "Cowbell"},
    {'P',   0,  16,  84, "SC-8850 POWER Set", "Bell Tree"},
    {'P',   0,  16,  88, "SC-8850 POWER Set", "Applause2"},
    {'P',   0,  24,   0, "SC-8850 ELECTRONIC Set", "[88] Electric Kick 2"},
    {'P',   0,  24,   1, "SC-8850 ELECTRONIC Set", "[88] Electric Kick 1"},
    {'P',   0,  24,   2, "SC-8850 ELECTRONIC Set", "CR-78 Kick 1"},
    {'P',   0,  24,   3, "SC-8850 ELECTRONIC Set", "CR-78 Kick 2"},
    {'P',   0,  24,   4, "SC-8850 ELECTRONIC Set", "TR-606 Kick1"},
    {'P',   0,  24,   5, "SC-8850 ELECTRONIC Set", "TR-707 Kick 1"},
    {'P',   0,  24,   6, "SC-8850 ELECTRONIC Set", "[55] TR-808 Kick"},
    {'P',   0,  24,   7, "SC-8850 ELECTRONIC Set", "[88] TR-808 Kick"},
    {'P',   0,  24,   8, "SC-8850 ELECTRONIC Set", "TR-808 Kick 2"},
    {'P',   0,  24,   9, "SC-8850 ELECTRONIC Set", "[88] TR-909 Kick"},
    {'P',   0,  24,  10, "SC-8850 ELECTRONIC Set", "[88] Dance Kick"},
    {'P',   0,  24,  11, "SC-8850 ELECTRONIC Set", "Hip-Hop Kick 2"},
    {'P',   0,  24,  12, "SC-8850 ELECTRONIC Set", "TR-909 Kick 1"},
    {'P',   0,  24,  13, "SC-8850 ELECTRONIC Set", "Hip-Hop Kick 3"},
    {'P',   0,  24,  14, "SC-8850 ELECTRONIC Set", "Jungle Kick 1"},
    {'P',   0,  24,  15, "SC-8850 ELECTRONIC Set", "Techno Kick 1"},
    {'P',   0,  24,  16, "SC-8850 ELECTRONIC Set", "Bounce Kick"},
    {'P',   0,  24,  26, "SC-8850 ELECTRONIC Set", "Finger Snap 2"},
    {'P',   0,  24,  29, "SC-8850 ELECTRONIC Set", "Scratch Push 2"},
    {'P',   0,  24,  30, "SC-8850 ELECTRONIC Set", "Scratch Pull 2"},
    {'P',   0,  24,  35, "SC-8850 ELECTRONIC Set", "Electric Kick 2"},
    {'P',   0,  24,  36, "SC-8850 ELECTRONIC Set", "Electric Kick 1"},
    {'P',   0,  24,  37, "SC-8850 ELECTRONIC Set", "Side Stick"},
    {'P',   0,  24,  38, "SC-8850 ELECTRONIC Set", "Electric Snare 1"},
    {'P',   0,  24,  39, "SC-8850 ELECTRONIC Set", "Hand Clap"},
    {'P',   0,  24,  40, "SC-8850 ELECTRONIC Set", "Electric Snare 2"},
    {'P',   0,  24,  41, "SC-8850 ELECTRONIC Set", "Electric Low Tom 2"},
    {'P',   0,  24,  42, "SC-8850 ELECTRONIC Set", "Closed Hi-Hat 2"},
    {'P',   0,  24,  43, "SC-8850 ELECTRONIC Set", "Electric Low Tom 1"},
    {'P',   0,  24,  44, "SC-8850 ELECTRONIC Set", "Pedal Hi-Hat"},
    {'P',   0,  24,  45, "SC-8850 ELECTRONIC Set", "Electric Mid Tom 2"},
    {'P',   0,  24,  46, "SC-8850 ELECTRONIC Set", "Open Hi-Hat 2"},
    {'P',   0,  24,  47, "SC-8850 ELECTRONIC Set", "Electric Mid Tom 1"},
    {'P',   0,  24,  48, "SC-8850 ELECTRONIC Set", "Electric High Tom 2"},
    {'P',   0,  24,  49, "SC-8850 ELECTRONIC Set", "Crash Cymbal 1"},
    {'P',   0,  24,  50, "SC-8850 ELECTRONIC Set", "Electric High Tom 1"},
    {'P',   0,  24,  51, "SC-8850 ELECTRONIC Set", "Ride Cymbal 1"},
    {'P',   0,  24,  52, "SC-8850 ELECTRONIC Set", "Reverse Cymbal"},
    {'P',   0,  24,  88, "SC-8850 ELECTRONIC Set", "Small Club 1"},
    {'P',   0,  24,  97, "SC-8850 ELECTRONIC Set", "Techno Hit"},
    {'P',   0,  24,  98, "SC-8850 ELECTRONIC Set", "Philly Hit"},
    {'P',   0,  24,  99, "SC-8850 ELECTRONIC Set", "Impact Hit"},
    {'P',   0,  24, 100, "SC-8850 ELECTRONIC Set", "Lo-Fi Rave"},
    {'P',   0,  24, 101, "SC-8850 ELECTRONIC Set", "Bam Hit"},
    {'P',   0,  24, 102, "SC-8850 ELECTRONIC Set", "Bim Hit"},
    {'P',   0,  24, 103, "SC-8850 ELECTRONIC Set", "Tape Rewind"},
    {'P',   0,  24, 104, "SC-8850 ELECTRONIC Set", "Phonograph Noise"},
    {'P',   0,  24, 105, "SC-8850 ELECTRONIC Set", "[88] Power Snare 1"},
    {'P',   0,  24, 106, "SC-8850 ELECTRONIC Set", "[88] Dance Snare 1"},
    {'P',   0,  24, 107, "SC-8850 ELECTRONIC Set", "[88] Dance Snare 2"},
    {'P',   0,  24, 108, "SC-8850 ELECTRONIC Set", "[88] Disco Snare"},
    {'P',   0,  24, 109, "SC-8850 ELECTRONIC Set", "[88] Electric Snare 2"},
    {'P',   0,  24, 110, "SC-8850 ELECTRONIC Set", "[55] Electric Snare"},
    {'P',   0,  24, 111, "SC-8850 ELECTRONIC Set", "[88] Electric Snare 3"},
    {'P',   0,  24, 112, "SC-8850 ELECTRONIC Set", "TR-606 Snare 2"},
    {'P',   0,  24, 113, "SC-8850 ELECTRONIC Set", "TR-707 Snare 1"},
    {'P',   0,  24, 114, "SC-8850 ELECTRONIC Set", "[88] TR-808 Snare 2"},
    {'P',   0,  24, 115, "SC-8850 ELECTRONIC Set", "[88] TR-808 Snare 1"},
    {'P',   0,  24, 116, "SC-8850 ELECTRONIC Set", "TR-808 Snare 2"},
    {'P',   0,  24, 117, "SC-8850 ELECTRONIC Set", "[88] TR-909 Snare 1"},
    {'P',   0,  24, 118, "SC-8850 ELECTRONIC Set", "[88] TR-909 Snare 2"},
    {'P',   0,  24, 119, "SC-8850 ELECTRONIC Set", "TR-909 Snare 1"},
    {'P',   0,  24, 120, "SC-8850 ELECTRONIC Set", "TR-909 Snare 2"},
    {'P',   0,  24, 121, "SC-8850 ELECTRONIC Set", "Rap Snare"},
    {'P',   0,  24, 122, "SC-8850 ELECTRONIC Set", "Jungle Snare"},
    {'P',   0,  24, 123, "SC-8850 ELECTRONIC Set", "House Snare 1"},
    {'P',   0,  24, 124, "SC-8850 ELECTRONIC Set", "[88] House Snare"},
    {'P',   0,  24, 125, "SC-8850 ELECTRONIC Set", "House Snare 2"},
    {'P',   0,  24, 126, "SC-8850 ELECTRONIC Set", "Voice Tah"},
    {'P',   0,  24, 127, "SC-8850 ELECTRONIC Set", "[88] Slappy"},
    {'P',   0,  25,  26, "SC-8850 TR-808 Set", "Finger Snap 2"},
    {'P',   0,  25,  29, "SC-8850 TR-808 Set", "Scratch Push 2"},
    {'P',   0,  25,  30, "SC-8850 TR-808 Set", "Scratch Pull 2"},
    {'P',   0,  25,  35, "SC-8850 TR-808 Set", "TR-808 Kick 2"},
    {'P',   0,  25,  36, "SC-8850 TR-808 Set", "TR-808 Kick 1"},
    {'P',   0,  25,  37, "SC-8850 TR-808 Set", "TR-808 Rim Shot"},
    {'P',   0,  25,  38, "SC-8850 TR-808 Set", "TR-808 Snare 1"},
    {'P',   0,  25,  39, "SC-8850 TR-808 Set", "Hand Clap"},
    {'P',   0,  25,  40, "SC-8850 TR-808 Set", "TR-808 Snare 2"},
    {'P',   0,  25,  41, "SC-8850 TR-808 Set", "TR-808 Low Tom 2"},
    {'P',   0,  25,  42, "SC-8850 TR-808 Set", "TR-808 Closed Hi-Hat 2"},
    {'P',   0,  25,  43, "SC-8850 TR-808 Set", "TR-808 Low Tom 1"},
    {'P',   0,  25,  44, "SC-8850 TR-808 Set", "TR-808 Closed Hi-Hat"},
    {'P',   0,  25,  45, "SC-8850 TR-808 Set", "TR-808 Mid Tom 2"},
    {'P',   0,  25,  46, "SC-8850 TR-808 Set", "TR-808 Open Hi-Hat"},
    {'P',   0,  25,  47, "SC-8850 TR-808 Set", "TR-808 Mid Tom 1"},
    {'P',   0,  25,  48, "SC-8850 TR-808 Set", "TR-808 High Tom 2"},
    {'P',   0,  25,  49, "SC-8850 TR-808 Set", "TR-808 Crash Cymbal"},
    {'P',   0,  25,  50, "SC-8850 TR-808 Set", "TR-808 High Tom 1"},
    {'P',   0,  25,  51, "SC-8850 TR-808 Set", "TR-606 Ride Cymbal"},
    {'P',   0,  25,  52, "SC-8850 TR-808 Set", "Chinese Cymbal"},
    {'P',   0,  25,  53, "SC-8850 TR-808 Set", "Ride Bell"},
    {'P',   0,  25,  54, "SC-8850 TR-808 Set", "CR-78 Tambourine"},
    {'P',   0,  25,  55, "SC-8850 TR-808 Set", "Splash Cymbal"},
    {'P',   0,  25,  56, "SC-8850 TR-808 Set", "TR-808 Cowbell"},
    {'P',   0,  25,  57, "SC-8850 TR-808 Set", "TR-909 Crash Cymbal"},
    {'P',   0,  25,  59, "SC-8850 TR-808 Set", "Ride Cymbal 2"},
    {'P',   0,  25,  60, "SC-8850 TR-808 Set", "CR-78 High Bongo"},
    {'P',   0,  25,  61, "SC-8850 TR-808 Set", "CR-78 Low Bongo"},
    {'P',   0,  25,  62, "SC-8850 TR-808 Set", "TR-808 High Conga"},
    {'P',   0,  25,  63, "SC-8850 TR-808 Set", "TR-808 Mute Conga"},
    {'P',   0,  25,  64, "SC-8850 TR-808 Set", "TR-808 Low Conga"},
    {'P',   0,  25,  70, "SC-8850 TR-808 Set", "TR-808 Maracas"},
    {'P',   0,  25,  74, "SC-8850 TR-808 Set", "CR-78 Guiro"},
    {'P',   0,  25,  75, "SC-8850 TR-808 Set", "TR-808 Claves"},
    {'P',   0,  25,  78, "SC-8850 TR-808 Set", "High Hoo"},
    {'P',   0,  25,  79, "SC-8850 TR-808 Set", "Low Hoo"},
    {'P',   0,  25,  80, "SC-8850 TR-808 Set", "Electric Mute Triangle"},
    {'P',   0,  25,  81, "SC-8850 TR-808 Set", "Electric Open Triangle"},
    {'P',   0,  25,  82, "SC-8850 TR-808 Set", "TR-626 Shaker"},
    {'P',   0,  25,  88, "SC-8850 TR-808 Set", "Small Club 1"},
    {'P',   0,  26,  20, "SC-8850 DANCE Set", "TR-909 Kick 2"},
    {'P',   0,  26,  21, "SC-8850 DANCE Set", "TR-909 Kick 1"},
    {'P',   0,  26,  26, "SC-8850 DANCE Set", "Finger Snap 2"},
    {'P',   0,  26,  29, "SC-8850 DANCE Set", "Scratch Push 2"},
    {'P',   0,  26,  30, "SC-8850 DANCE Set", "Scratch Pull 2"},
    {'P',   0,  26,  35, "SC-8850 DANCE Set", "Fat Kick"},
    {'P',   0,  26,  36, "SC-8850 DANCE Set", "Dance Kick"},
    {'P',   0,  26,  37, "SC-8850 DANCE Set", "Dance Rim Shot"},
    {'P',   0,  26,  38, "SC-8850 DANCE Set", "Dance Snare"},
    {'P',   0,  26,  39, "SC-8850 DANCE Set", "Comp Claps 2"},
    {'P',   0,  26,  40, "SC-8850 DANCE Set", "Rock SD Dry"},
    {'P',   0,  26,  41, "SC-8850 DANCE Set", "Electric Low Tom 2"},
    {'P',   0,  26,  42, "SC-8850 DANCE Set", "CR-78 Closed Hi-Hat"},
    {'P',   0,  26,  43, "SC-8850 DANCE Set", "Electric Low Tom 1"},
    {'P',   0,  26,  44, "SC-8850 DANCE Set", "TR-808 Closed Hi-Hat 2"},
    {'P',   0,  26,  45, "SC-8850 DANCE Set", "Electric Mid Tom 2"},
    {'P',   0,  26,  46, "SC-8850 DANCE Set", "CR-78 Open Hi-Hat"},
    {'P',   0,  26,  47, "SC-8850 DANCE Set", "Electric Mid Tom 1"},
    {'P',   0,  26,  48, "SC-8850 DANCE Set", "Electric High Tom 2"},
    {'P',   0,  26,  49, "SC-8850 DANCE Set", "TR-808 Crash Cymbal"},
    {'P',   0,  26,  50, "SC-8850 DANCE Set", "Electric High Tom 1"},
    {'P',   0,  26,  51, "SC-8850 DANCE Set", "TR-606 Ride Cymbal"},
    {'P',   0,  26,  52, "SC-8850 DANCE Set", "Reverse Cymbal"},
    {'P',   0,  26,  53, "SC-8850 DANCE Set", "Ride Bell"},
    {'P',   0,  26,  54, "SC-8850 DANCE Set", "Shake Tambourine"},
    {'P',   0,  26,  55, "SC-8850 DANCE Set", "Splash Cymbal"},
    {'P',   0,  26,  56, "SC-8850 DANCE Set", "TR-808 Cowbell"},
    {'P',   0,  26,  70, "SC-8850 DANCE Set", "Maracas"},
    {'P',   0,  26,  74, "SC-8850 DANCE Set", "Long Guiro"},
    {'P',   0,  26,  75, "SC-8850 DANCE Set", "Claves"},
    {'P',   0,  26,  78, "SC-8850 DANCE Set", "High Hoo"},
    {'P',   0,  26,  79, "SC-8850 DANCE Set", "Low Hoo"},
    {'P',   0,  26,  80, "SC-8850 DANCE Set", "Electric Mute Triangle"},
    {'P',   0,  26,  81, "SC-8850 DANCE Set", "Electric Open Triangle"},
    {'P',   0,  26,  82, "SC-8850 DANCE Set", "TR-626 Shaker"},
    {'P',   0,  26,  88, "SC-8850 DANCE Set", "Small Club 1"},
    {'P',   0,  26,  89, "SC-8850 DANCE Set", "TR-606 Snare 2"},
    {'P',   0,  26,  90, "SC-8850 DANCE Set", "Techno Snare"},
    {'P',   0,  26,  91, "SC-8850 DANCE Set", "House Snare"},
    {'P',   0,  26,  92, "SC-8850 DANCE Set", "Jungle Snare"},
    {'P',   0,  26,  93, "SC-8850 DANCE Set", "LoFi Snare 1"},
    {'P',   0,  26,  94, "SC-8850 DANCE Set", "LoFi Snare 2"},
    {'P',   0,  26,  95, "SC-8850 DANCE Set", "Hip-Hop Snare 1"},
    {'P',   0,  26,  96, "SC-8850 DANCE Set", "Hip-Hop Snare 2"},
    {'P',   0,  27,  26, "SC-8850 CR-78 Set", "Finger Snap 2"},
    {'P',   0,  27,  29, "SC-8850 CR-78 Set", "Scratch Push 2"},
    {'P',   0,  27,  30, "SC-8850 CR-78 Set", "Scratch Pull 2"},
    {'P',   0,  27,  35, "SC-8850 CR-78 Set", "CR-78 Kick 2"},
    {'P',   0,  27,  36, "SC-8850 CR-78 Set", "CR-78 Kick 1"},
    {'P',   0,  27,  37, "SC-8850 CR-78 Set", "CR-78 Rim Shot"},
    {'P',   0,  27,  38, "SC-8850 CR-78 Set", "CR-78 Snare 1"},
    {'P',   0,  27,  39, "SC-8850 CR-78 Set", "TR-707 Hand Clap"},
    {'P',   0,  27,  40, "SC-8850 CR-78 Set", "CR-78 Snare 2"},
    {'P',   0,  27,  41, "SC-8850 CR-78 Set", "CR-78 Low Tom 2"},
    {'P',   0,  27,  42, "SC-8850 CR-78 Set", "CR-78 Closed Hi-Hat"},
    {'P',   0,  27,  43, "SC-8850 CR-78 Set", "CR-78 Low Tom 1"},
    {'P',   0,  27,  44, "SC-8850 CR-78 Set", "TR-606 Closed Hi-Hat"},
    {'P',   0,  27,  45, "SC-8850 CR-78 Set", "CR-78 Mid Tom 2"},
    {'P',   0,  27,  46, "SC-8850 CR-78 Set", "CR-78 Open Hi-Hat"},
    {'P',   0,  27,  47, "SC-8850 CR-78 Set", "CR-78 Mid Tom 1"},
    {'P',   0,  27,  48, "SC-8850 CR-78 Set", "CR-78 High Tom 2"},
    {'P',   0,  27,  49, "SC-8850 CR-78 Set", "TR-808 Crash Cymbal"},
    {'P',   0,  27,  50, "SC-8850 CR-78 Set", "CR-78 High Tom 1"},
    {'P',   0,  27,  51, "SC-8850 CR-78 Set", "TR-606 Ride Cymbal"},
    {'P',   0,  27,  52, "SC-8850 CR-78 Set", "Chinese Cymbal"},
    {'P',   0,  27,  53, "SC-8850 CR-78 Set", "Ride Bell"},
    {'P',   0,  27,  54, "SC-8850 CR-78 Set", "CR-78 Tambourine"},
    {'P',   0,  27,  55, "SC-8850 CR-78 Set", "Splash Cymbal"},
    {'P',   0,  27,  56, "SC-8850 CR-78 Set", "CR-78 Cowbell"},
    {'P',   0,  27,  57, "SC-8850 CR-78 Set", "TR-909 Crash Cymbal"},
    {'P',   0,  27,  59, "SC-8850 CR-78 Set", "Ride Cymbal Edge"},
    {'P',   0,  27,  60, "SC-8850 CR-78 Set", "CR-78 High Bongo"},
    {'P',   0,  27,  61, "SC-8850 CR-78 Set", "CR-78 Low Bongo"},
    {'P',   0,  27,  62, "SC-8850 CR-78 Set", "TR-808 High Conga"},
    {'P',   0,  27,  63, "SC-8850 CR-78 Set", "TR-808 Mute Conga"},
    {'P',   0,  27,  64, "SC-8850 CR-78 Set", "TR-808 Low Conga"},
    {'P',   0,  27,  70, "SC-8850 CR-78 Set", "CR-78 Maracas"},
    {'P',   0,  27,  74, "SC-8850 CR-78 Set", "CR-78 Guiro"},
    {'P',   0,  27,  75, "SC-8850 CR-78 Set", "CR-78 Claves"},
    {'P',   0,  27,  78, "SC-8850 CR-78 Set", "High Hoo"},
    {'P',   0,  27,  79, "SC-8850 CR-78 Set", "Low Hoo"},
    {'P',   0,  27,  80, "SC-8850 CR-78 Set", "CR-78 Metalic Beat 1"},
    {'P',   0,  27,  81, "SC-8850 CR-78 Set", "CR-78 Metalic Beat 2"},
    {'P',   0,  27,  82, "SC-8850 CR-78 Set", "TR-626 Shaker"},
    {'P',   0,  27,  88, "SC-8850 CR-78 Set", "Small Club 1"},
    {'P',   0,  28,  26, "SC-8850 TR-606 Set", "Finger Snap 2"},
    {'P',   0,  28,  29, "SC-8850 TR-606 Set", "Scratch Push 2"},
    {'P',   0,  28,  30, "SC-8850 TR-606 Set", "Scratch Pull 2"},
    {'P',   0,  28,  35, "SC-8850 TR-606 Set", "CR-78 Kink 2"},
    {'P',   0,  28,  36, "SC-8850 TR-606 Set", "TR-606 Kick 1"},
    {'P',   0,  28,  37, "SC-8850 TR-606 Set", "CR-78 Rim Shot"},
    {'P',   0,  28,  38, "SC-8850 TR-606 Set", "TR-606 Snare 1"},
    {'P',   0,  28,  39, "SC-8850 TR-606 Set", "TR-707 Hand Clap"},
    {'P',   0,  28,  40, "SC-8850 TR-606 Set", "TR-606 Snare 2"},
    {'P',   0,  28,  41, "SC-8850 TR-606 Set", "TR-606 Low Tom 2"},
    {'P',   0,  28,  42, "SC-8850 TR-606 Set", "TR-606 Closed Hi-Hat"},
    {'P',   0,  28,  43, "SC-8850 TR-606 Set", "TR-606 Low Tom 1"},
    {'P',   0,  28,  44, "SC-8850 TR-606 Set", "TR-606 Closed Hi-Hat"},
    {'P',   0,  28,  45, "SC-8850 TR-606 Set", "TR-606 Mid Tom 2"},
    {'P',   0,  28,  46, "SC-8850 TR-606 Set", "TR-606 Open Hi-Hat"},
    {'P',   0,  28,  47, "SC-8850 TR-606 Set", "TR-606 Mid Tom 1"},
    {'P',   0,  28,  48, "SC-8850 TR-606 Set", "TR-606 High Tom 2"},
    {'P',   0,  28,  49, "SC-8850 TR-606 Set", "TR-808 Crash Cymbal"},
    {'P',   0,  28,  50, "SC-8850 TR-606 Set", "TR-606 High Tom 1"},
    {'P',   0,  28,  51, "SC-8850 TR-606 Set", "TR-606 Ride Cymbal"},
    {'P',   0,  28,  52, "SC-8850 TR-606 Set", "Chinese Cymbal"},
    {'P',   0,  28,  53, "SC-8850 TR-606 Set", "Ride Bell"},
    {'P',   0,  28,  54, "SC-8850 TR-606 Set", "CR-78 Tambourine"},
    {'P',   0,  28,  55, "SC-8850 TR-606 Set", "Splash Cymbal"},
    {'P',   0,  28,  56, "SC-8850 TR-606 Set", "CR-78 Cowbell"},
    {'P',   0,  28,  57, "SC-8850 TR-606 Set", "TR-909 Crash Cymbal"},
    {'P',   0,  28,  59, "SC-8850 TR-606 Set", "Ride Cymbal Edge"},
    {'P',   0,  28,  60, "SC-8850 TR-606 Set", "CR-78 High Bongo"},
    {'P',   0,  28,  61, "SC-8850 TR-606 Set", "CR-78 Low Bongo"},
    {'P',   0,  28,  62, "SC-8850 TR-606 Set", "TR-808 High Conga"},
    {'P',   0,  28,  63, "SC-8850 TR-606 Set", "TR-808 Mute Conga"},
    {'P',   0,  28,  64, "SC-8850 TR-606 Set", "TR-808 Low Conga"},
    {'P',   0,  28,  70, "SC-8850 TR-606 Set", "CR-78 Maracas"},
    {'P',   0,  28,  74, "SC-8850 TR-606 Set", "CR-78 Guiro"},
    {'P',   0,  28,  75, "SC-8850 TR-606 Set", "CR-78 Claves"},
    {'P',   0,  28,  78, "SC-8850 TR-606 Set", "High Hoo"},
    {'P',   0,  28,  79, "SC-8850 TR-606 Set", "Low Hoo"},
    {'P',   0,  28,  80, "SC-8850 TR-606 Set", "CR-78 Metalic Beat 1"},
    {'P',   0,  28,  81, "SC-8850 TR-606 Set", "CR-78 Metalic Beat 2"},
    {'P',   0,  28,  82, "SC-8850 TR-606 Set", "TR-626 Shaker"},
    {'P',   0,  28,  88, "SC-8850 TR-606 Set", "Small Club 1"},
    {'P',   0,  29,  26, "SC-8850 TR-707 Set", "Finger Snap 2"},
    {'P',   0,  29,  29, "SC-8850 TR-707 Set", "Scratch Push 2"},
    {'P',   0,  29,  30, "SC-8850 TR-707 Set", "Scratch Pull 2"},
    {'P',   0,  29,  35, "SC-8850 TR-707 Set", "TR-707 Kick 2"},
    {'P',   0,  29,  36, "SC-8850 TR-707 Set", "TR-707 Kick 1"},
    {'P',   0,  29,  37, "SC-8850 TR-707 Set", "TR-707 Rim Shot"},
    {'P',   0,  29,  38, "SC-8850 TR-707 Set", "TR-707 Snare 1"},
    {'P',   0,  29,  39, "SC-8850 TR-707 Set", "TR-707 Hand Clap"},
    {'P',   0,  29,  40, "SC-8850 TR-707 Set", "TR-707 Snare 2"},
    {'P',   0,  29,  41, "SC-8850 TR-707 Set", "TR-707 Low Tom 2"},
    {'P',   0,  29,  42, "SC-8850 TR-707 Set", "TR-707 Closed Hi-Hat"},
    {'P',   0,  29,  43, "SC-8850 TR-707 Set", "TR-707 Low Tom 1"},
    {'P',   0,  29,  44, "SC-8850 TR-707 Set", "TR-707 Closed Hi-Hat"},
    {'P',   0,  29,  45, "SC-8850 TR-707 Set", "TR-707 Mid Tom 2"},
    {'P',   0,  29,  46, "SC-8850 TR-707 Set", "TR-707 Open Hi-Hat"},
    {'P',   0,  29,  47, "SC-8850 TR-707 Set", "TR-707 Mid Tom 1"},
    {'P',   0,  29,  48, "SC-8850 TR-707 Set", "TR-707 High Tom 2"},
    {'P',   0,  29,  49, "SC-8850 TR-707 Set", "TR-909 Crash Cymbal"},
    {'P',   0,  29,  50, "SC-8850 TR-707 Set", "TR-707 High Tom 1"},
    {'P',   0,  29,  51, "SC-8850 TR-707 Set", "TR-909 Ride Cymbal"},
    {'P',   0,  29,  52, "SC-8850 TR-707 Set", "Chinese Cymbal"},
    {'P',   0,  29,  53, "SC-8850 TR-707 Set", "Ride Bell"},
    {'P',   0,  29,  54, "SC-8850 TR-707 Set", "Tambourine 2"},
    {'P',   0,  29,  55, "SC-8850 TR-707 Set", "Splash Cymbal"},
    {'P',   0,  29,  56, "SC-8850 TR-707 Set", "TR-808 Cowbell"},
    {'P',   0,  29,  57, "SC-8850 TR-707 Set", "Crash Cymbal 2"},
    {'P',   0,  29,  59, "SC-8850 TR-707 Set", "Ride Cymbal Edge"},
    {'P',   0,  29,  60, "SC-8850 TR-707 Set", "High Bongo"},
    {'P',   0,  29,  61, "SC-8850 TR-707 Set", "Low Bongo"},
    {'P',   0,  29,  62, "SC-8850 TR-707 Set", "Mute High Conga"},
    {'P',   0,  29,  63, "SC-8850 TR-707 Set", "Open High Conga"},
    {'P',   0,  29,  64, "SC-8850 TR-707 Set", "Low Conga"},
    {'P',   0,  29,  70, "SC-8850 TR-707 Set", "TR-808 Maracas"},
    {'P',   0,  29,  74, "SC-8850 TR-707 Set", "Long Guiro"},
    {'P',   0,  29,  75, "SC-8850 TR-707 Set", "Claves"},
    {'P',   0,  29,  78, "SC-8850 TR-707 Set", "High Hoo"},
    {'P',   0,  29,  79, "SC-8850 TR-707 Set", "Low Hoo"},
    {'P',   0,  29,  80, "SC-8850 TR-707 Set", "Electric Mute Triangle"},
    {'P',   0,  29,  81, "SC-8850 TR-707 Set", "Electric Open Triangle"},
    {'P',   0,  29,  82, "SC-8850 TR-707 Set", "TR-626 Shaker"},
    {'P',   0,  29,  88, "SC-8850 TR-707 Set", "Small Club 1"},
    {'P',   0,  30,  26, "SC-8850 TR-909 Set", "Finger Snap 2"},
    {'P',   0,  30,  29, "SC-8850 TR-909 Set", "Scratch Push 2"},
    {'P',   0,  30,  30, "SC-8850 TR-909 Set", "Scratch Pull 2"},
    {'P',   0,  30,  35, "SC-8850 TR-909 Set", "TR-909 Kick 2"},
    {'P',   0,  30,  36, "SC-8850 TR-909 Set", "TR-909 Kick 1"},
    {'P',   0,  30,  37, "SC-8850 TR-909 Set", "TR-909 Rim"},
    {'P',   0,  30,  38, "SC-8850 TR-909 Set", "TR-909 Snare 1"},
    {'P',   0,  30,  39, "SC-8850 TR-909 Set", "TR-909 Hand Clap"},
    {'P',   0,  30,  40, "SC-8850 TR-909 Set", "TR-909 Snare2"},
    {'P',   0,  30,  41, "SC-8850 TR-909 Set", "TR-909 Low Tom 2"},
    {'P',   0,  30,  42, "SC-8850 TR-909 Set", "TR-707 Closed Hi-Hat"},
    {'P',   0,  30,  43, "SC-8850 TR-909 Set", "TR-909 Low Tom 1"},
    {'P',   0,  30,  44, "SC-8850 TR-909 Set", "TR-707 Closed Hi-Hat"},
    {'P',   0,  30,  45, "SC-8850 TR-909 Set", "TR-909 Mid Tom 2"},
    {'P',   0,  30,  46, "SC-8850 TR-909 Set", "TR-909 Open Hi-Hat"},
    {'P',   0,  30,  47, "SC-8850 TR-909 Set", "TR-909 Mid Tom 1"},
    {'P',   0,  30,  48, "SC-8850 TR-909 Set", "TR-909 High Tom 2"},
    {'P',   0,  30,  49, "SC-8850 TR-909 Set", "TR-909 Crash Cymbal"},
    {'P',   0,  30,  50, "SC-8850 TR-909 Set", "TR-909 High Tom 1"},
    {'P',   0,  30,  51, "SC-8850 TR-909 Set", "TR-909 Ride Cymbal"},
    {'P',   0,  30,  52, "SC-8850 TR-909 Set", "Chinese Cymbal"},
    {'P',   0,  30,  53, "SC-8850 TR-909 Set", "Ride Bell"},
    {'P',   0,  30,  54, "SC-8850 TR-909 Set", "Tambourine 2"},
    {'P',   0,  30,  55, "SC-8850 TR-909 Set", "Splash Cymbal"},
    {'P',   0,  30,  56, "SC-8850 TR-909 Set", "TR-808 Cowbell"},
    {'P',   0,  30,  59, "SC-8850 TR-909 Set", "Ride Cymbal Edge"},
    {'P',   0,  30,  70, "SC-8850 TR-909 Set", "TR-808 Maracas"},
    {'P',   0,  30,  74, "SC-8850 TR-909 Set", "CR-78 Guiro"},
    {'P',   0,  30,  75, "SC-8850 TR-909 Set", "TR-808 Claves"},
    {'P',   0,  30,  78, "SC-8850 TR-909 Set", "High Hoo"},
    {'P',   0,  30,  79, "SC-8850 TR-909 Set", "Low Hoo"},
    {'P',   0,  30,  80, "SC-8850 TR-909 Set", "Electric Mute Triangle"},
    {'P',   0,  30,  81, "SC-8850 TR-909 Set", "Electric Open Triangle"},
    {'P',   0,  30,  82, "SC-8850 TR-909 Set", "TR-626 Shaker"},
    {'P',   0,  30,  88, "SC-8850 TR-909 Set", "Applause 2"},
    {'P',   0,  32,   0, "SC-8850 JAZZ Set", "[88] Standard 1 Kick 1"},
    {'P',   0,  32,   1, "SC-8850 JAZZ Set", "[88] Standard 1 Kick 2"},
    {'P',   0,  32,   2, "SC-8850 JAZZ Set", "[88] Standard 2 Kick 1"},
    {'P',   0,  32,   3, "SC-8850 JAZZ Set", "[88] Standard 2 Kick 2"},
    {'P',   0,  32,   4, "SC-8850 JAZZ Set", "[55] Kick Drum 1"},
    {'P',   0,  32,   5, "SC-8850 JAZZ Set", "[55] Kick Drum 2"},
    {'P',   0,  32,   6, "SC-8850 JAZZ Set", "[88] Jazz Kick 1"},
    {'P',   0,  32,   7, "SC-8850 JAZZ Set", "[88] Jazz Kick 2"},
    {'P',   0,  32,   8, "SC-8850 JAZZ Set", "[88] Room Kick 1"},
    {'P',   0,  32,   9, "SC-8850 JAZZ Set", "[88] Room Kick 2"},
    {'P',   0,  32,  10, "SC-8850 JAZZ Set", "[88] Power Kick 1"},
    {'P',   0,  32,  11, "SC-8850 JAZZ Set", "[88] Power Kick 2"},
    {'P',   0,  32,  12, "SC-8850 JAZZ Set", "[88] Electric Kick 2"},
    {'P',   0,  32,  13, "SC-8850 JAZZ Set", "[88] Electric Kick 1"},
    {'P',   0,  32,  14, "SC-8850 JAZZ Set", "[88] TR-808 Kick"},
    {'P',   0,  32,  15, "SC-8850 JAZZ Set", "[88] TR-909 Kick"},
    {'P',   0,  32,  16, "SC-8850 JAZZ Set", "[88] Dance Kick"},
    {'P',   0,  32,  17, "SC-8850 JAZZ Set", "[Pro] Voice One"},
    {'P',   0,  32,  18, "SC-8850 JAZZ Set", "[Pro] Voice Two"},
    {'P',   0,  32,  19, "SC-8850 JAZZ Set", "[Pro] Voice Three"},
    {'P',   0,  32,  20, "SC-8850 JAZZ Set", "Brush Kick 2"},
    {'P',   0,  32,  21, "SC-8850 JAZZ Set", "Brush Kick 1"},
    {'P',   0,  32,  22, "SC-8850 JAZZ Set", "MC-500 Beep 1"},
    {'P',   0,  32,  23, "SC-8850 JAZZ Set", "MC-500 Beep 2"},
    {'P',   0,  32,  24, "SC-8850 JAZZ Set", "Concert SD"},
    {'P',   0,  32,  25, "SC-8850 JAZZ Set", "Snare Roll"},
    {'P',   0,  32,  26, "SC-8850 JAZZ Set", "Finger Snap 2"},
    {'P',   0,  32,  27, "SC-8850 JAZZ Set", "High Q"},
    {'P',   0,  32,  28, "SC-8850 JAZZ Set", "Slap"},
    {'P',   0,  32,  29, "SC-8850 JAZZ Set", "Scratch Push"},
    {'P',   0,  32,  30, "SC-8850 JAZZ Set", "Scratch Pull"},
    {'P',   0,  32,  31, "SC-8850 JAZZ Set", "Sticks"},
    {'P',   0,  32,  32, "SC-8850 JAZZ Set", "Square Click"},
    {'P',   0,  32,  33, "SC-8850 JAZZ Set", "Metronome Click"},
    {'P',   0,  32,  34, "SC-8850 JAZZ Set", "Metronome Bell"},
    {'P',   0,  32,  35, "SC-8850 JAZZ Set", "Jazz Kick 2"},
    {'P',   0,  32,  36, "SC-8850 JAZZ Set", "Jazz Kick 1"},
    {'P',   0,  32,  37, "SC-8850 JAZZ Set", "Side Stick"},
    {'P',   0,  32,  38, "SC-8850 JAZZ Set", "Jazz Snare 1"},
    {'P',   0,  32,  39, "SC-8850 JAZZ Set", "Hand Clap 2"},
    {'P',   0,  32,  40, "SC-8850 JAZZ Set", "Jazz Snare 2"},
    {'P',   0,  32,  41, "SC-8850 JAZZ Set", "Jazz Low Tom 2"},
    {'P',   0,  32,  42, "SC-8850 JAZZ Set", "Closed Hi-Hat 2"},
    {'P',   0,  32,  43, "SC-8850 JAZZ Set", "Jazz Low Tom 1"},
    {'P',   0,  32,  44, "SC-8850 JAZZ Set", "Pedal Hi-Hat"},
    {'P',   0,  32,  45, "SC-8850 JAZZ Set", "Jazz Mid Tom 2"},
    {'P',   0,  32,  46, "SC-8850 JAZZ Set", "Open Hi-Hat 2"},
    {'P',   0,  32,  47, "SC-8850 JAZZ Set", "Jazz Mid Tom 1"},
    {'P',   0,  32,  48, "SC-8850 JAZZ Set", "Jazz High Tom 2"},
    {'P',   0,  32,  49, "SC-8850 JAZZ Set", "Jazz Crash Cymbal"},
    {'P',   0,  32,  50, "SC-8850 JAZZ Set", "Jazz High Tom 1"},
    {'P',   0,  32,  51, "SC-8850 JAZZ Set", "Jazz Ride Cymbal"},
    {'P',   0,  32,  52, "SC-8850 JAZZ Set", "Chinese Cymbal"},
    {'P',   0,  32,  53, "SC-8850 JAZZ Set", "Jazz Ride Bell"},
    {'P',   0,  32,  54, "SC-8850 JAZZ Set", "Tambourine"},
    {'P',   0,  32,  55, "SC-8850 JAZZ Set", "Splash Cymbal"},
    {'P',   0,  32,  56, "SC-8850 JAZZ Set", "Cowbell"},
    {'P',   0,  32,  57, "SC-8850 JAZZ Set", "Crash Cymbal 2"},
    {'P',   0,  32,  58, "SC-8850 JAZZ Set", "Vibra-slap"},
    {'P',   0,  32,  59, "SC-8850 JAZZ Set", "Ride Cymbal Edge"},
    {'P',   0,  32,  60, "SC-8850 JAZZ Set", "High Bongo"},
    {'P',   0,  32,  61, "SC-8850 JAZZ Set", "Low Bongo"},
    {'P',   0,  32,  62, "SC-8850 JAZZ Set", "Mute High Conga"},
    {'P',   0,  32,  63, "SC-8850 JAZZ Set", "Open High Conga"},
    {'P',   0,  32,  64, "SC-8850 JAZZ Set", "Low Conga"},
    {'P',   0,  32,  65, "SC-8850 JAZZ Set", "High Timbale"},
    {'P',   0,  32,  66, "SC-8850 JAZZ Set", "Low Timbale"},
    {'P',   0,  32,  67, "SC-8850 JAZZ Set", "High Agogo"},
    {'P',   0,  32,  68, "SC-8850 JAZZ Set", "Low Agogo"},
    {'P',   0,  32,  69, "SC-8850 JAZZ Set", "Cabasa"},
    {'P',   0,  32,  70, "SC-8850 JAZZ Set", "Maracas"},
    {'P',   0,  32,  71, "SC-8850 JAZZ Set", "Short High Whistle"},
    {'P',   0,  32,  72, "SC-8850 JAZZ Set", "Long Low Whistle"},
    {'P',   0,  32,  73, "SC-8850 JAZZ Set", "Short Guiro"},
    {'P',   0,  32,  74, "SC-8850 JAZZ Set", "Long Guiro"},
    {'P',   0,  32,  75, "SC-8850 JAZZ Set", "Claves"},
    {'P',   0,  32,  76, "SC-8850 JAZZ Set", "High Wood Block"},
    {'P',   0,  32,  77, "SC-8850 JAZZ Set", "Low Wood Block"},
    {'P',   0,  32,  78, "SC-8850 JAZZ Set", "Mute Cuica"},
    {'P',   0,  32,  79, "SC-8850 JAZZ Set", "Open Cuica"},
    {'P',   0,  32,  80, "SC-8850 JAZZ Set", "Mute Triangle"},
    {'P',   0,  32,  81, "SC-8850 JAZZ Set", "Open Triangle"},
    {'P',   0,  32,  82, "SC-8850 JAZZ Set", "Shaker"},
    {'P',   0,  32,  83, "SC-8850 JAZZ Set", "Jingle Bell"},
    {'P',   0,  32,  84, "SC-8850 JAZZ Set", "Bell Tree"},
    {'P',   0,  32,  85, "SC-8850 JAZZ Set", "Castanets"},
    {'P',   0,  32,  86, "SC-8850 JAZZ Set", "Mute Surdo"},
    {'P',   0,  32,  87, "SC-8850 JAZZ Set", "Open Surdo"},
    {'P',   0,  32,  88, "SC-8850 JAZZ Set", "Applause"},
    {'P',   0,  32,  98, "SC-8850 JAZZ Set", "Brush Tap 2"},
    {'P',   0,  32,  99, "SC-8850 JAZZ Set", "Brush Slap 2"},
    {'P',   0,  32, 100, "SC-8850 JAZZ Set", "[88] Brush Tap 1"},
    {'P',   0,  32, 101, "SC-8850 JAZZ Set", "[88] Brush Tap 2"},
    {'P',   0,  32, 102, "SC-8850 JAZZ Set", "[88] Brush Slap 1"},
    {'P',   0,  32, 103, "SC-8850 JAZZ Set", "[88] Brush Slap 2"},
    {'P',   0,  32, 104, "SC-8850 JAZZ Set", "[88] Brush Slap 3"},
    {'P',   0,  32, 105, "SC-8850 JAZZ Set", "[88] Brush Swirl 1"},
    {'P',   0,  32, 106, "SC-8850 JAZZ Set", "[88] Brush Swirl 2"},
    {'P',   0,  32, 107, "SC-8850 JAZZ Set", "[88] Brush Long Swirl"},
    {'P',   0,  32, 108, "SC-8850 JAZZ Set", "[88] Jazz Snare 1"},
    {'P',   0,  32, 109, "SC-8850 JAZZ Set", "[88] Jazz Snare 2"},
    {'P',   0,  32, 110, "SC-8850 JAZZ Set", "[88] Standard 1 Snare1"},
    {'P',   0,  32, 111, "SC-8850 JAZZ Set", "[88] Standard 1 Snare2"},
    {'P',   0,  32, 112, "SC-8850 JAZZ Set", "[88] Standard 2 Snare1"},
    {'P',   0,  32, 113, "SC-8850 JAZZ Set", "[88] Standard 2 Snare2"},
    {'P',   0,  32, 114, "SC-8850 JAZZ Set", "[55] Snare Drum 2"},
    {'P',   0,  32, 115, "SC-8850 JAZZ Set", "[Pro] Standard 1 Snare 1"},
    {'P',   0,  32, 116, "SC-8850 JAZZ Set", "[Pro] Standard 1 Snare 2"},
    {'P',   0,  32, 117, "SC-8850 JAZZ Set", "[Pro] Standard 1 Snare 3"},
    {'P',   0,  32, 118, "SC-8850 JAZZ Set", "[88] Room Snare 1"},
    {'P',   0,  32, 119, "SC-8850 JAZZ Set", "[88] Room Snare 2"},
    {'P',   0,  32, 120, "SC-8850 JAZZ Set", "[88] Power Snare 1"},
    {'P',   0,  32, 121, "SC-8850 JAZZ Set", "[88] Power Snare 2"},
    {'P',   0,  32, 122, "SC-8850 JAZZ Set", "[88] Gated Snare"},
    {'P',   0,  32, 123, "SC-8850 JAZZ Set", "[88] Dance Snare 1"},
    {'P',   0,  32, 124, "SC-8850 JAZZ Set", "[88] Dance Snare 2"},
    {'P',   0,  32, 125, "SC-8850 JAZZ Set", "[88] Disco Snare"},
    {'P',   0,  32, 126, "SC-8850 JAZZ Set", "[88] Electric Snare 2"},
    {'P',   0,  32, 127, "SC-8850 JAZZ Set", "[88] Electric Snare 3"},
    {'P',   0,  33,  26, "SC-8850 JAZZ L/R Set", "Finger Snap 2"},
    {'P',   0,  33,  29, "SC-8850 JAZZ L/R Set", "Scratch Push"},
    {'P',   0,  33,  30, "SC-8850 JAZZ L/R Set", "Scratch Pull"},
    {'P',   0,  33,  35, "SC-8850 JAZZ L/R Set", "[RND] Jazz Kick 2"},
    {'P',   0,  33,  36, "SC-8850 JAZZ L/R Set", "[RND] Jazz Kick 1"},
    {'P',   0,  33,  37, "SC-8850 JAZZ L/R Set", "Side Stick"},
    {'P',   0,  33,  38, "SC-8850 JAZZ L/R Set", "[RND] Jazz Snare 1"},
    {'P',   0,  33,  39, "SC-8850 JAZZ L/R Set", "Hand Clap 2"},
    {'P',   0,  33,  40, "SC-8850 JAZZ L/R Set", "[RND] Jazz Snare 2"},
    {'P',   0,  33,  41, "SC-8850 JAZZ L/R Set", "[RND] Jazz Low Tom 2"},
    {'P',   0,  33,  42, "SC-8850 JAZZ L/R Set", "[RND] Jazz Closed Hi-Hat"},
    {'P',   0,  33,  43, "SC-8850 JAZZ L/R Set", "[RND] Jazz Low Tom 1"},
    {'P',   0,  33,  44, "SC-8850 JAZZ L/R Set", "Pedal Hi-Hat"},
    {'P',   0,  33,  45, "SC-8850 JAZZ L/R Set", "[RND] Jazz Mid Tom 2"},
    {'P',   0,  33,  46, "SC-8850 JAZZ L/R Set", "[RND] Jazz Open Hi-Hat"},
    {'P',   0,  33,  47, "SC-8850 JAZZ L/R Set", "[RND] Jazz Mid Tom 1"},
    {'P',   0,  33,  48, "SC-8850 JAZZ L/R Set", "[RND] Jazz High Tom 2"},
    {'P',   0,  33,  49, "SC-8850 JAZZ L/R Set", "[RND] Jazz Crash Cymbal"},
    {'P',   0,  33,  50, "SC-8850 JAZZ L/R Set", "[RND] Jazz High Tom 1"},
    {'P',   0,  33,  51, "SC-8850 JAZZ L/R Set", "[RND] Jazz Ride Cymbal"},
    {'P',   0,  33,  52, "SC-8850 JAZZ L/R Set", "Chinese Cymbal"},
    {'P',   0,  33,  53, "SC-8850 JAZZ L/R Set", "[RND] Jazz Ride Bell"},
    {'P',   0,  33,  54, "SC-8850 JAZZ L/R Set", "Tambourine"},
    {'P',   0,  33,  55, "SC-8850 JAZZ L/R Set", "Splash Cymbal"},
    {'P',   0,  33,  56, "SC-8850 JAZZ L/R Set", "Cowbell"},
    {'P',   0,  33,  59, "SC-8850 JAZZ L/R Set", "Ride Cymbal Edge"},
    {'P',   0,  33,  88, "SC-8850 JAZZ L/R Set", "Applause"},
    {'P',   0,  33,  95, "SC-8850 JAZZ L/R Set", "[L] Jazz Kick 2"},
    {'P',   0,  33,  96, "SC-8850 JAZZ L/R Set", "[L] Jazz Kick 1"},
    {'P',   0,  33,  97, "SC-8850 JAZZ L/R Set", "[L] Jazz Crash Cymbal"},
    {'P',   0,  33,  98, "SC-8850 JAZZ L/R Set", "[L] Jazz Snare 1"},
    {'P',   0,  33,  99, "SC-8850 JAZZ L/R Set", "[L] Jazz Ride Cymbal"},
    {'P',   0,  33, 100, "SC-8850 JAZZ L/R Set", "[L] Jazz Snare 2"},
    {'P',   0,  33, 101, "SC-8850 JAZZ L/R Set", "[L] Jazz Low Tom"},
    {'P',   0,  33, 102, "SC-8850 JAZZ L/R Set", "[L] Jazz Closed Hi-Hat"},
    {'P',   0,  33, 103, "SC-8850 JAZZ L/R Set", "[L] Jazz Mid Tom"},
    {'P',   0,  33, 104, "SC-8850 JAZZ L/R Set", "[L] Jazz Ride Bell"},
    {'P',   0,  33, 105, "SC-8850 JAZZ L/R Set", "[L] Jazz High Tom"},
    {'P',   0,  33, 106, "SC-8850 JAZZ L/R Set", "[L] Jazz Open Hi-Hat"},
    {'P',   0,  33, 107, "SC-8850 JAZZ L/R Set", "[R] Jazz Kick 2"},
    {'P',   0,  33, 108, "SC-8850 JAZZ L/R Set", "[R] Jazz Kick 1"},
    {'P',   0,  33, 109, "SC-8850 JAZZ L/R Set", "[R] Jazz Crash Cymbal"},
    {'P',   0,  33, 110, "SC-8850 JAZZ L/R Set", "[R] Jazz Snare 1"},
    {'P',   0,  33, 111, "SC-8850 JAZZ L/R Set", "[R] Jazz Ride Cymbal"},
    {'P',   0,  33, 112, "SC-8850 JAZZ L/R Set", "[R] Jazz Snare 2"},
    {'P',   0,  33, 113, "SC-8850 JAZZ L/R Set", "[R] Jazz Low Tom"},
    {'P',   0,  33, 114, "SC-8850 JAZZ L/R Set", "[R] Jazz Closed Hi-Hat"},
    {'P',   0,  33, 115, "SC-8850 JAZZ L/R Set", "[R] Jazz Mid Tom"},
    {'P',   0,  33, 116, "SC-8850 JAZZ L/R Set", "[R] Jazz Ride Bell"},
    {'P',   0,  33, 117, "SC-8850 JAZZ L/R Set", "[R] Jazz High Tom"},
    {'P',   0,  33, 118, "SC-8850 JAZZ L/R Set", "[R] Jazz Open Hi-Hat"},
    {'P',   0,  40,   0, "SC-8850 BRUSH Set", "[88] Standard 1 Kick 1"},
    {'P',   0,  40,   1, "SC-8850 BRUSH Set", "[88] Standard 1 Kick 2"},
    {'P',   0,  40,   2, "SC-8850 BRUSH Set", "[88] Standard 2 Kick 1"},
    {'P',   0,  40,   3, "SC-8850 BRUSH Set", "[88] Standard 2 Kick 2"},
    {'P',   0,  40,   4, "SC-8850 BRUSH Set", "[55] Kick Drum 1"},
    {'P',   0,  40,   5, "SC-8850 BRUSH Set", "[55] Kick Drum 2"},
    {'P',   0,  40,   6, "SC-8850 BRUSH Set", "[88] Jazz Kick 1"},
    {'P',   0,  40,   7, "SC-8850 BRUSH Set", "[88] Jazz Kick 2"},
    {'P',   0,  40,   8, "SC-8850 BRUSH Set", "[88] Room Kick 1"},
    {'P',   0,  40,   9, "SC-8850 BRUSH Set", "[88] Room Kick 2"},
    {'P',   0,  40,  10, "SC-8850 BRUSH Set", "[88] Power Kick 1"},
    {'P',   0,  40,  11, "SC-8850 BRUSH Set", "[88] Power Kick 2"},
    {'P',   0,  40,  12, "SC-8850 BRUSH Set", "[88] Electric Kick 2"},
    {'P',   0,  40,  13, "SC-8850 BRUSH Set", "[88] Electric Kick 1"},
    {'P',   0,  40,  14, "SC-8850 BRUSH Set", "[88] TR-808 Kick"},
    {'P',   0,  40,  15, "SC-8850 BRUSH Set", "[88] TR-909 Kick"},
    {'P',   0,  40,  16, "SC-8850 BRUSH Set", "[88] Dance Kick"},
    {'P',   0,  40,  17, "SC-8850 BRUSH Set", "[Pro] Voice One"},
    {'P',   0,  40,  18, "SC-8850 BRUSH Set", "[Pro] Voice Two"},
    {'P',   0,  40,  19, "SC-8850 BRUSH Set", "[Pro] Voice Three"},
    {'P',   0,  40,  22, "SC-8850 BRUSH Set", "MC-500 Beep 1"},
    {'P',   0,  40,  23, "SC-8850 BRUSH Set", "MC-500 Beep 2"},
    {'P',   0,  40,  24, "SC-8850 BRUSH Set", "Concert SD"},
    {'P',   0,  40,  25, "SC-8850 BRUSH Set", "Snare Roll"},
    {'P',   0,  40,  26, "SC-8850 BRUSH Set", "Finger Snap 2"},
    {'P',   0,  40,  27, "SC-8850 BRUSH Set", "High Q"},
    {'P',   0,  40,  28, "SC-8850 BRUSH Set", "Slap"},
    {'P',   0,  40,  29, "SC-8850 BRUSH Set", "Scratch Push"},
    {'P',   0,  40,  30, "SC-8850 BRUSH Set", "Scratch Pull"},
    {'P',   0,  40,  31, "SC-8850 BRUSH Set", "Sticks"},
    {'P',   0,  40,  32, "SC-8850 BRUSH Set", "Square Click"},
    {'P',   0,  40,  33, "SC-8850 BRUSH Set", "Metronome Click"},
    {'P',   0,  40,  34, "SC-8850 BRUSH Set", "Metronome Bell"},
    {'P',   0,  40,  35, "SC-8850 BRUSH Set", "Jazz Kick 2"},
    {'P',   0,  40,  36, "SC-8850 BRUSH Set", "Jazz Kick 1"},
    {'P',   0,  40,  37, "SC-8850 BRUSH Set", "Side Stick"},
    {'P',   0,  40,  38, "SC-8850 BRUSH Set", "Brush Tap 1"},
    {'P',   0,  40,  39, "SC-8850 BRUSH Set", "Brush Slap 1"},
    {'P',   0,  40,  40, "SC-8850 BRUSH Set", "Brush Swirl 1"},
    {'P',   0,  40,  41, "SC-8850 BRUSH Set", "Brush Low Tom 2"},
    {'P',   0,  40,  42, "SC-8850 BRUSH Set", "Brush Closed Hi-Hat"},
    {'P',   0,  40,  43, "SC-8850 BRUSH Set", "Brush Low Tom 1"},
    {'P',   0,  40,  44, "SC-8850 BRUSH Set", "Pedal Hi-Hat"},
    {'P',   0,  40,  45, "SC-8850 BRUSH Set", "Brush Mid Tom 2"},
    {'P',   0,  40,  46, "SC-8850 BRUSH Set", "Brush Open Hi-Hat"},
    {'P',   0,  40,  47, "SC-8850 BRUSH Set", "Brush Mid Tom 1"},
    {'P',   0,  40,  48, "SC-8850 BRUSH Set", "Brush High Tom 2"},
    {'P',   0,  40,  49, "SC-8850 BRUSH Set", "Brush Crash Cymbal"},
    {'P',   0,  40,  50, "SC-8850 BRUSH Set", "Brush High Tom 1"},
    {'P',   0,  40,  51, "SC-8850 BRUSH Set", "Ride Cymbal Inner"},
    {'P',   0,  40,  52, "SC-8850 BRUSH Set", "Chinese Cymbal"},
    {'P',   0,  40,  53, "SC-8850 BRUSH Set", "Brush Ride Bell"},
    {'P',   0,  40,  54, "SC-8850 BRUSH Set", "Tambourine"},
    {'P',   0,  40,  55, "SC-8850 BRUSH Set", "Splash Cymbal"},
    {'P',   0,  40,  56, "SC-8850 BRUSH Set", "Cowbell"},
    {'P',   0,  40,  57, "SC-8850 BRUSH Set", "Crash Cymbal 2"},
    {'P',   0,  40,  58, "SC-8850 BRUSH Set", "Vibra-slap"},
    {'P',   0,  40,  59, "SC-8850 BRUSH Set", "Ride Cymbal Edge"},
    {'P',   0,  40,  60, "SC-8850 BRUSH Set", "High Bongo"},
    {'P',   0,  40,  61, "SC-8850 BRUSH Set", "Low Bongo"},
    {'P',   0,  40,  62, "SC-8850 BRUSH Set", "Mute High Conga"},
    {'P',   0,  40,  63, "SC-8850 BRUSH Set", "Open High Conga"},
    {'P',   0,  40,  64, "SC-8850 BRUSH Set", "Low Conga"},
    {'P',   0,  40,  65, "SC-8850 BRUSH Set", "High Timbale"},
    {'P',   0,  40,  66, "SC-8850 BRUSH Set", "Low Timbale"},
    {'P',   0,  40,  67, "SC-8850 BRUSH Set", "High Agogo"},
    {'P',   0,  40,  68, "SC-8850 BRUSH Set", "Low Agogo"},
    {'P',   0,  40,  69, "SC-8850 BRUSH Set", "Cabasa"},
    {'P',   0,  40,  70, "SC-8850 BRUSH Set", "Maracas"},
    {'P',   0,  40,  71, "SC-8850 BRUSH Set", "Short High Whistle"},
    {'P',   0,  40,  72, "SC-8850 BRUSH Set", "Long Low Whistle"},
    {'P',   0,  40,  73, "SC-8850 BRUSH Set", "Short Guiro"},
    {'P',   0,  40,  74, "SC-8850 BRUSH Set", "Long Guiro"},
    {'P',   0,  40,  75, "SC-8850 BRUSH Set", "Claves"},
    {'P',   0,  40,  76, "SC-8850 BRUSH Set", "High Wood Block"},
    {'P',   0,  40,  77, "SC-8850 BRUSH Set", "Low Wood Block"},
    {'P',   0,  40,  78, "SC-8850 BRUSH Set", "Mute Cuica"},
    {'P',   0,  40,  79, "SC-8850 BRUSH Set", "Open Cuica"},
    {'P',   0,  40,  80, "SC-8850 BRUSH Set", "Mute Triangle"},
    {'P',   0,  40,  81, "SC-8850 BRUSH Set", "Open Triangle"},
    {'P',   0,  40,  82, "SC-8850 BRUSH Set", "Shaker"},
    {'P',   0,  40,  83, "SC-8850 BRUSH Set", "Jingle Bell"},
    {'P',   0,  40,  84, "SC-8850 BRUSH Set", "Bell Tree"},
    {'P',   0,  40,  85, "SC-8850 BRUSH Set", "Castanets"},
    {'P',   0,  40,  86, "SC-8850 BRUSH Set", "Mute Surdo"},
    {'P',   0,  40,  87, "SC-8850 BRUSH Set", "Open Surdo"},
    {'P',   0,  40,  88, "SC-8850 BRUSH Set", "Applause"},
    {'P',   0,  40, 100, "SC-8850 BRUSH Set", "[88] Brush Tap 1"},
    {'P',   0,  40, 101, "SC-8850 BRUSH Set", "[88] Brush Tap 2"},
    {'P',   0,  40, 102, "SC-8850 BRUSH Set", "[88] Brush Slap 1"},
    {'P',   0,  40, 103, "SC-8850 BRUSH Set", "[88] Brush Slap 2"},
    {'P',   0,  40, 104, "SC-8850 BRUSH Set", "[88] Brush Slap 3"},
    {'P',   0,  40, 105, "SC-8850 BRUSH Set", "[88] Brush Swirl 1"},
    {'P',   0,  40, 106, "SC-8850 BRUSH Set", "[88] Brush Swirl 2"},
    {'P',   0,  40, 107, "SC-8850 BRUSH Set", "[88] Brush Long Swirl"},
    {'P',   0,  40, 108, "SC-8850 BRUSH Set", "[88] Jazz Snare 1"},
    {'P',   0,  40, 109, "SC-8850 BRUSH Set", "[88] Jazz Snare 2"},
    {'P',   0,  40, 110, "SC-8850 BRUSH Set", "[88] Standard 1 Snare1"},
    {'P',   0,  40, 111, "SC-8850 BRUSH Set", "[88] Standard 1 Snare2"},
    {'P',   0,  40, 112, "SC-8850 BRUSH Set", "[88] Standard 2 Snare1"},
    {'P',   0,  40, 113, "SC-8850 BRUSH Set", "[88] Standard 2 Snare2"},
    {'P',   0,  40, 114, "SC-8850 BRUSH Set", "[55] Snare Drum 2"},
    {'P',   0,  40, 115, "SC-8850 BRUSH Set", "Standard 1 Snare 1"},
    {'P',   0,  40, 116, "SC-8850 BRUSH Set", "Standard 1 Snare 2"},
    {'P',   0,  40, 117, "SC-8850 BRUSH Set", "Standard 1 Snare 3"},
    {'P',   0,  40, 118, "SC-8850 BRUSH Set", "[88] Room Snare 1"},
    {'P',   0,  40, 119, "SC-8850 BRUSH Set", "[88] Room Snare 2"},
    {'P',   0,  40, 120, "SC-8850 BRUSH Set", "[88] Power Snare 1"},
    {'P',   0,  40, 121, "SC-8850 BRUSH Set", "[88] Power Snare 2"},
    {'P',   0,  40, 122, "SC-8850 BRUSH Set", "[88] Gated Snare"},
    {'P',   0,  40, 123, "SC-8850 BRUSH Set", "[88] Dance Snare 1"},
    {'P',   0,  40, 124, "SC-8850 BRUSH Set", "[88] Dance Snare 2"},
    {'P',   0,  40, 125, "SC-8850 BRUSH Set", "[88] Disco Snare"},
    {'P',   0,  40, 126, "SC-8850 BRUSH Set", "[88] Electric Snare 2"},
    {'P',   0,  40, 127, "SC-8850 BRUSH Set", "[88] Electric Snare 3"},
    {'P',   0,  41,  20, "SC-8850 BRUSH 2 Set", "Jazz Kick 2"},
    {'P',   0,  41,  21, "SC-8850 BRUSH 2 Set", "Jazz Kick 1"},
    {'P',   0,  41,  26, "SC-8850 BRUSH 2 Set", "Finger Snap 2"},
    {'P',   0,  41,  29, "SC-8850 BRUSH 2 Set", "Scratch Push"},
    {'P',   0,  41,  30, "SC-8850 BRUSH 2 Set", "Scratch Pull"},
    {'P',   0,  41,  35, "SC-8850 BRUSH 2 Set", "Brush Kick 2"},
    {'P',   0,  41,  36, "SC-8850 BRUSH 2 Set", "Brush Kick 1"},
    {'P',   0,  41,  37, "SC-8850 BRUSH 2 Set", "Side Stick"},
    {'P',   0,  41,  38, "SC-8850 BRUSH 2 Set", "Brush Tap 2"},
    {'P',   0,  41,  39, "SC-8850 BRUSH 2 Set", "Brush Slap 2"},
    {'P',   0,  41,  40, "SC-8850 BRUSH 2 Set", "Brush Swirl 1"},
    {'P',   0,  41,  41, "SC-8850 BRUSH 2 Set", "Brush Low Tom 2"},
    {'P',   0,  41,  42, "SC-8850 BRUSH 2 Set", "Brush Closed Hi-Hat"},
    {'P',   0,  41,  43, "SC-8850 BRUSH 2 Set", "Brush Low Tom 1"},
    {'P',   0,  41,  44, "SC-8850 BRUSH 2 Set", "Pedal Hi-Hat"},
    {'P',   0,  41,  45, "SC-8850 BRUSH 2 Set", "Brush Mid Tom 2"},
    {'P',   0,  41,  46, "SC-8850 BRUSH 2 Set", "Brush Open Hi-Hat"},
    {'P',   0,  41,  47, "SC-8850 BRUSH 2 Set", "Brush Mid Tom 1"},
    {'P',   0,  41,  48, "SC-8850 BRUSH 2 Set", "Brush High Tom 2"},
    {'P',   0,  41,  49, "SC-8850 BRUSH 2 Set", "Brush Crash Cymbal"},
    {'P',   0,  41,  50, "SC-8850 BRUSH 2 Set", "Brush High Tom 1"},
    {'P',   0,  41,  51, "SC-8850 BRUSH 2 Set", "Brush Ride Cymbal"},
    {'P',   0,  41,  52, "SC-8850 BRUSH 2 Set", "Chinese Cymbal"},
    {'P',   0,  41,  53, "SC-8850 BRUSH 2 Set", "Brush Ride Bell"},
    {'P',   0,  41,  54, "SC-8850 BRUSH 2 Set", "Tambourine"},
    {'P',   0,  41,  55, "SC-8850 BRUSH 2 Set", "Splash Cymbal"},
    {'P',   0,  41,  56, "SC-8850 BRUSH 2 Set", "Cowbell"},
    {'P',   0,  41,  59, "SC-8850 BRUSH 2 Set", "Ride Cymbal Edge"},
    {'P',   0,  41,  88, "SC-8850 BRUSH 2 Set", "Applause"},
    {'P',   0,  41,  98, "SC-8850 BRUSH 2 Set", "Jazz Snare 1"},
    {'P',   0,  41,  99, "SC-8850 BRUSH 2 Set", "Jazz Snare 2"},
    {'P',   0,  42,  26, "SC-8850 BRUSH 2 L/R Set", "Finger Snap 2"},
    {'P',   0,  42,  29, "SC-8850 BRUSH 2 L/R Set", "Scratch Push"},
    {'P',   0,  42,  30, "SC-8850 BRUSH 2 L/R Set", "Scratch Pull"},
    {'P',   0,  42,  35, "SC-8850 BRUSH 2 L/R Set", "[RND] Brush Kick 2"},
    {'P',   0,  42,  36, "SC-8850 BRUSH 2 L/R Set", "[RND] Brush Kick 1"},
    {'P',   0,  42,  38, "SC-8850 BRUSH 2 L/R Set", "[RND] Brush Tap 2"},
    {'P',   0,  42,  39, "SC-8850 BRUSH 2 L/R Set", "[RND] Brush Slap 2"},
    {'P',   0,  42,  40, "SC-8850 BRUSH 2 L/R Set", "Brush Swirl 1"},
    {'P',   0,  42,  41, "SC-8850 BRUSH 2 L/R Set", "[RND] Brush Low Tom 2"},
    {'P',   0,  42,  42, "SC-8850 BRUSH 2 L/R Set", "[RND] Brush Closed Hi-Hat"},
    {'P',   0,  42,  43, "SC-8850 BRUSH 2 L/R Set", "[RND] Brush Low Tom 1"},
    {'P',   0,  42,  44, "SC-8850 BRUSH 2 L/R Set", "Pedal Hi-Hat"},
    {'P',   0,  42,  45, "SC-8850 BRUSH 2 L/R Set", "[RND] Brush Mid Tom 2"},
    {'P',   0,  42,  46, "SC-8850 BRUSH 2 L/R Set", "[RND] Brush Open Hi-Hat"},
    {'P',   0,  42,  47, "SC-8850 BRUSH 2 L/R Set", "[RND] Brush Mid Tom 1"},
    {'P',   0,  42,  48, "SC-8850 BRUSH 2 L/R Set", "[RND] Brush High Tom 2"},
    {'P',   0,  42,  49, "SC-8850 BRUSH 2 L/R Set", "[RND] Brush Crash Cymbal"},
    {'P',   0,  42,  50, "SC-8850 BRUSH 2 L/R Set", "[RND] Brush High Tom 1"},
    {'P',   0,  42,  51, "SC-8850 BRUSH 2 L/R Set", "[RND] Brush Ride Cymbal"},
    {'P',   0,  42,  52, "SC-8850 BRUSH 2 L/R Set", "Chinese Cymbal"},
    {'P',   0,  42,  53, "SC-8850 BRUSH 2 L/R Set", "[RND] Brush Ride Bell"},
    {'P',   0,  42,  54, "SC-8850 BRUSH 2 L/R Set", "Tambourine"},
    {'P',   0,  42,  55, "SC-8850 BRUSH 2 L/R Set", "Splash Cymbal"},
    {'P',   0,  42,  56, "SC-8850 BRUSH 2 L/R Set", "Cowbell"},
    {'P',   0,  42,  59, "SC-8850 BRUSH 2 L/R Set", "Ride Cymbal Edge"},
    {'P',   0,  42,  88, "SC-8850 BRUSH 2 L/R Set", "Applause"},
    {'P',   0,  42,  95, "SC-8850 BRUSH 2 L/R Set", "[L] Brush Kick 2"},
    {'P',   0,  42,  96, "SC-8850 BRUSH 2 L/R Set", "[L] Brush Kick 1"},
    {'P',   0,  42,  97, "SC-8850 BRUSH 2 L/R Set", "[L] Brush Crash Cymbal"},
    {'P',   0,  42,  98, "SC-8850 BRUSH 2 L/R Set", "[L] Brush Tap 2"},
    {'P',   0,  42,  99, "SC-8850 BRUSH 2 L/R Set", "[L] Brush Ride Cymbal"},
    {'P',   0,  42, 100, "SC-8850 BRUSH 2 L/R Set", "[L] Brush Slap 2"},
    {'P',   0,  42, 101, "SC-8850 BRUSH 2 L/R Set", "[L] Brush Low Tom"},
    {'P',   0,  42, 102, "SC-8850 BRUSH 2 L/R Set", "[L] Brush Closed Hi-Hat"},
    {'P',   0,  42, 103, "SC-8850 BRUSH 2 L/R Set", "[L] Brush Mid Tom"},
    {'P',   0,  42, 104, "SC-8850 BRUSH 2 L/R Set", "[L] Brush Ride Bell"},
    {'P',   0,  42, 105, "SC-8850 BRUSH 2 L/R Set", "[L] Brush High Tom"},
    {'P',   0,  42, 106, "SC-8850 BRUSH 2 L/R Set", "[L] Brush Open Hi-Hat"},
    {'P',   0,  42, 107, "SC-8850 BRUSH 2 L/R Set", "[R] Brush Kick 2"},
    {'P',   0,  42, 108, "SC-8850 BRUSH 2 L/R Set", "[R] Brush Kick 1"},
    {'P',   0,  42, 109, "SC-8850 BRUSH 2 L/R Set", "[R] Brush Crash Cymbal"},
    {'P',   0,  42, 110, "SC-8850 BRUSH 2 L/R Set", "[R] Brush Tap 2"},
    {'P',   0,  42, 111, "SC-8850 BRUSH 2 L/R Set", "[R] Brush Ride Cymbal"},
    {'P',   0,  42, 112, "SC-8850 BRUSH 2 L/R Set", "[R] Brush Slap 2"},
    {'P',   0,  42, 113, "SC-8850 BRUSH 2 L/R Set", "[R] Brush Low Tom"},
    {'P',   0,  42, 114, "SC-8850 BRUSH 2 L/R Set", "[R] Brush Closed Hi-Hat"},
    {'P',   0,  42, 115, "SC-8850 BRUSH 2 L/R Set", "[R] Brush Mid Tom"},
    {'P',   0,  42, 116, "SC-8850 BRUSH 2 L/R Set", "[R] Brush Ride Bell"},
    {'P',   0,  42, 117, "SC-8850 BRUSH 2 L/R Set", "[R] Brush High Tom"},
    {'P',   0,  42, 118, "SC-8850 BRUSH 2 L/R Set", "[R] Brush Open Hi-Hat"},
    {'P',   0,  48,   0, "SC-8850 ORCHESTRA Set", "[88] Standard 1 Kick 1"},
    {'P',   0,  48,   1, "SC-8850 ORCHESTRA Set", "[88] Standard 1 Kick 2"},
    {'P',   0,  48,   2, "SC-8850 ORCHESTRA Set", "[88] Standard 2 Kick 1"},
    {'P',   0,  48,   3, "SC-8850 ORCHESTRA Set", "[88] Standard 2 Kick 2"},
    {'P',   0,  48,   4, "SC-8850 ORCHESTRA Set", "[55] Kick Drum 1"},
    {'P',   0,  48,   5, "SC-8850 ORCHESTRA Set", "[55] Kick Drum 2"},
    {'P',   0,  48,   6, "SC-8850 ORCHESTRA Set", "[88] Jazz Kick 1"},
    {'P',   0,  48,   7, "SC-8850 ORCHESTRA Set", "[88] Jazz Kick 2"},
    {'P',   0,  48,   8, "SC-8850 ORCHESTRA Set", "[88] Room Kick 1"},
    {'P',   0,  48,   9, "SC-8850 ORCHESTRA Set", "[88] Room Kick 2"},
    {'P',   0,  48,  10, "SC-8850 ORCHESTRA Set", "[88] Power Kick 1"},
    {'P',   0,  48,  11, "SC-8850 ORCHESTRA Set", "[88] Power Kick 2"},
    {'P',   0,  48,  12, "SC-8850 ORCHESTRA Set", "[88] Electric Kick 2"},
    {'P',   0,  48,  13, "SC-8850 ORCHESTRA Set", "[88] Electric Kick 1"},
    {'P',   0,  48,  14, "SC-8850 ORCHESTRA Set", "[88] TR-808 Kick"},
    {'P',   0,  48,  15, "SC-8850 ORCHESTRA Set", "[88] TR-909 Kick"},
    {'P',   0,  48,  16, "SC-8850 ORCHESTRA Set", "[88] Dance Kick"},
    {'P',   0,  48,  17, "SC-8850 ORCHESTRA Set", "[Pro] Voice One"},
    {'P',   0,  48,  18, "SC-8850 ORCHESTRA Set", "[Pro] Voice Two"},
    {'P',   0,  48,  19, "SC-8850 ORCHESTRA Set", "[Pro] Voice Three"},
    {'P',   0,  48,  22, "SC-8850 ORCHESTRA Set", "MC-500 Beep 1"},
    {'P',   0,  48,  23, "SC-8850 ORCHESTRA Set", "MC-500 Beep 2"},
    {'P',   0,  48,  24, "SC-8850 ORCHESTRA Set", "Concert SD"},
    {'P',   0,  48,  25, "SC-8850 ORCHESTRA Set", "Snare Roll"},
    {'P',   0,  48,  26, "SC-8850 ORCHESTRA Set", "Finger Snap"},
    {'P',   0,  48,  27, "SC-8850 ORCHESTRA Set", "Closed Hi-Hat 2"},
    {'P',   0,  48,  28, "SC-8850 ORCHESTRA Set", "Pedal Hi-Hat"},
    {'P',   0,  48,  29, "SC-8850 ORCHESTRA Set", "Open Hi-Hat 2"},
    {'P',   0,  48,  30, "SC-8850 ORCHESTRA Set", "Ride Cymbal 1"},
    {'P',   0,  48,  31, "SC-8850 ORCHESTRA Set", "Sticks"},
    {'P',   0,  48,  32, "SC-8850 ORCHESTRA Set", "Square Click"},
    {'P',   0,  48,  33, "SC-8850 ORCHESTRA Set", "Metronome Click"},
    {'P',   0,  48,  34, "SC-8850 ORCHESTRA Set", "Metronome Bell"},
    {'P',   0,  48,  35, "SC-8850 ORCHESTRA Set", "Jazz Kick 1"},
    {'P',   0,  48,  36, "SC-8850 ORCHESTRA Set", "Concert BD 1"},
    {'P',   0,  48,  37, "SC-8850 ORCHESTRA Set", "Side Stick"},
    {'P',   0,  48,  38, "SC-8850 ORCHESTRA Set", "Concert SD"},
    {'P',   0,  48,  39, "SC-8850 ORCHESTRA Set", "Castanets"},
    {'P',   0,  48,  40, "SC-8850 ORCHESTRA Set", "Concert SD"},
    {'P',   0,  48,  41, "SC-8850 ORCHESTRA Set", "Timpani F"},
    {'P',   0,  48,  42, "SC-8850 ORCHESTRA Set", "Timpani F#"},
    {'P',   0,  48,  43, "SC-8850 ORCHESTRA Set", "Timpani G"},
    {'P',   0,  48,  44, "SC-8850 ORCHESTRA Set", "Timpani G#"},
    {'P',   0,  48,  45, "SC-8850 ORCHESTRA Set", "Timpani A"},
    {'P',   0,  48,  46, "SC-8850 ORCHESTRA Set", "Timpani A#"},
    {'P',   0,  48,  47, "SC-8850 ORCHESTRA Set", "Timpani B"},
    {'P',   0,  48,  48, "SC-8850 ORCHESTRA Set", "Timpani c"},
    {'P',   0,  48,  49, "SC-8850 ORCHESTRA Set", "Timpani c#"},
    {'P',   0,  48,  50, "SC-8850 ORCHESTRA Set", "Timpani d"},
    {'P',   0,  48,  51, "SC-8850 ORCHESTRA Set", "Timpani d#"},
    {'P',   0,  48,  52, "SC-8850 ORCHESTRA Set", "Timpani e"},
    {'P',   0,  48,  53, "SC-8850 ORCHESTRA Set", "Timpani f"},
    {'P',   0,  48,  54, "SC-8850 ORCHESTRA Set", "Tambourine"},
    {'P',   0,  48,  55, "SC-8850 ORCHESTRA Set", "Splash Cymbal"},
    {'P',   0,  48,  56, "SC-8850 ORCHESTRA Set", "Cowbell"},
    {'P',   0,  48,  57, "SC-8850 ORCHESTRA Set", "Concert Cymbal 2"},
    {'P',   0,  48,  58, "SC-8850 ORCHESTRA Set", "Vibra-slap"},
    {'P',   0,  48,  59, "SC-8850 ORCHESTRA Set", "Concert Cymbal 1"},
    {'P',   0,  48,  60, "SC-8850 ORCHESTRA Set", "High Bongo"},
    {'P',   0,  48,  61, "SC-8850 ORCHESTRA Set", "Low Bongo"},
    {'P',   0,  48,  62, "SC-8850 ORCHESTRA Set", "Mute High Conga"},
    {'P',   0,  48,  63, "SC-8850 ORCHESTRA Set", "Open High Conga"},
    {'P',   0,  48,  64, "SC-8850 ORCHESTRA Set", "Low Conga"},
    {'P',   0,  48,  65, "SC-8850 ORCHESTRA Set", "High Timbale"},
    {'P',   0,  48,  66, "SC-8850 ORCHESTRA Set", "Low Timbale"},
    {'P',   0,  48,  67, "SC-8850 ORCHESTRA Set", "High Agogo"},
    {'P',   0,  48,  68, "SC-8850 ORCHESTRA Set", "Low Agogo"},
    {'P',   0,  48,  69, "SC-8850 ORCHESTRA Set", "Cabasa"},
    {'P',   0,  48,  70, "SC-8850 ORCHESTRA Set", "Maracas"},
    {'P',   0,  48,  71, "SC-8850 ORCHESTRA Set", "Short High Whistle"},
    {'P',   0,  48,  72, "SC-8850 ORCHESTRA Set", "Long Low Whistle"},
    {'P',   0,  48,  73, "SC-8850 ORCHESTRA Set", "Short Guiro"},
    {'P',   0,  48,  74, "SC-8850 ORCHESTRA Set", "Long Guiro"},
    {'P',   0,  48,  75, "SC-8850 ORCHESTRA Set", "Claves"},
    {'P',   0,  48,  76, "SC-8850 ORCHESTRA Set", "High Wood Block"},
    {'P',   0,  48,  77, "SC-8850 ORCHESTRA Set", "Low Wood Block"},
    {'P',   0,  48,  78, "SC-8850 ORCHESTRA Set", "Mute Cuica"},
    {'P',   0,  48,  79, "SC-8850 ORCHESTRA Set", "Open Cuica"},
    {'P',   0,  48,  80, "SC-8850 ORCHESTRA Set", "Mute Triangle"},
    {'P',   0,  48,  81, "SC-8850 ORCHESTRA Set", "Open Triangle"},
    {'P',   0,  48,  82, "SC-8850 ORCHESTRA Set", "Shaker"},
    {'P',   0,  48,  83, "SC-8850 ORCHESTRA Set", "Jingle Bell"},
    {'P',   0,  48,  84, "SC-8850 ORCHESTRA Set", "Bell Tree"},
    {'P',   0,  48,  85, "SC-8850 ORCHESTRA Set", "Castanets"},
    {'P',   0,  48,  86, "SC-8850 ORCHESTRA Set", "Mute Surdo"},
    {'P',   0,  48,  87, "SC-8850 ORCHESTRA Set", "Open Surdo"},
    {'P',   0,  48,  88, "SC-8850 ORCHESTRA Set", "Applause"},
    {'P',   0,  48,  97, "SC-8850 ORCHESTRA Set", "Applause 2"},
    {'P',   0,  48,  98, "SC-8850 ORCHESTRA Set", "Small Club 1"},
    {'P',   0,  48,  99, "SC-8850 ORCHESTRA Set", "[55] Timpani D#"},
    {'P',   0,  48, 100, "SC-8850 ORCHESTRA Set", "[55] Timpani E"},
    {'P',   0,  48, 101, "SC-8850 ORCHESTRA Set", "[55] Timpani F"},
    {'P',   0,  48, 102, "SC-8850 ORCHESTRA Set", "[55] Timpani F#"},
    {'P',   0,  48, 103, "SC-8850 ORCHESTRA Set", "[55] Timpani G"},
    {'P',   0,  48, 104, "SC-8850 ORCHESTRA Set", "[55] Timpani G#"},
    {'P',   0,  48, 105, "SC-8850 ORCHESTRA Set", "[55] Timpani A"},
    {'P',   0,  48, 106, "SC-8850 ORCHESTRA Set", "[55] Timpani A#"},
    {'P',   0,  48, 107, "SC-8850 ORCHESTRA Set", "[55] Timpani B"},
    {'P',   0,  48, 108, "SC-8850 ORCHESTRA Set", "[55] Timpani c"},
    {'P',   0,  48, 109, "SC-8850 ORCHESTRA Set", "[55] Timpani c#"},
    {'P',   0,  48, 110, "SC-8850 ORCHESTRA Set", "[55] Timpani d"},
    {'P',   0,  48, 111, "SC-8850 ORCHESTRA Set", "[55] Timpani d#"},
    {'P',   0,  48, 112, "SC-8850 ORCHESTRA Set", "[55] Timpani e"},
    {'P',   0,  48, 113, "SC-8850 ORCHESTRA Set", "[55] Timpani f"},
    {'P',   0,  49,  25, "SC-8850 ETHNIC Set", "Finger Snap"},
    {'P',   0,  49,  26, "SC-8850 ETHNIC Set", "Tambourine"},
    {'P',   0,  49,  27, "SC-8850 ETHNIC Set", "Castanets"},
    {'P',   0,  49,  28, "SC-8850 ETHNIC Set", "Crash Cymbal 1"},
    {'P',   0,  49,  29, "SC-8850 ETHNIC Set", "Snare Roll"},
    {'P',   0,  49,  30, "SC-8850 ETHNIC Set", "Concert SD"},
    {'P',   0,  49,  31, "SC-8850 ETHNIC Set", "Concert Cymbal"},
    {'P',   0,  49,  32, "SC-8850 ETHNIC Set", "Concert BD 1"},
    {'P',   0,  49,  33, "SC-8850 ETHNIC Set", "Jingle Bell"},
    {'P',   0,  49,  34, "SC-8850 ETHNIC Set", "Bell Tree"},
    {'P',   0,  49,  35, "SC-8850 ETHNIC Set", "Bar Chimes"},
    {'P',   0,  49,  36, "SC-8850 ETHNIC Set", "Wadaiko"},
    {'P',   0,  49,  37, "SC-8850 ETHNIC Set", "Wadaiko Rim"},
    {'P',   0,  49,  38, "SC-8850 ETHNIC Set", "Shime Taiko"},
    {'P',   0,  49,  39, "SC-8850 ETHNIC Set", "Atarigane"},
    {'P',   0,  49,  40, "SC-8850 ETHNIC Set", "Hyoushigi"},
    {'P',   0,  49,  41, "SC-8850 ETHNIC Set", "Ohkawa"},
    {'P',   0,  49,  42, "SC-8850 ETHNIC Set", "High Kotsuzumi"},
    {'P',   0,  49,  43, "SC-8850 ETHNIC Set", "Low Kotsuzumi"},
    {'P',   0,  49,  44, "SC-8850 ETHNIC Set", "Ban Gu"},
    {'P',   0,  49,  45, "SC-8850 ETHNIC Set", "Big Gong"},
    {'P',   0,  49,  46, "SC-8850 ETHNIC Set", "Small Gong"},
    {'P',   0,  49,  47, "SC-8850 ETHNIC Set", "Bend Gong"},
    {'P',   0,  49,  48, "SC-8850 ETHNIC Set", "Thai Gong"},
    {'P',   0,  49,  49, "SC-8850 ETHNIC Set", "Rama Cymbal"},
    {'P',   0,  49,  50, "SC-8850 ETHNIC Set", "Gamelan Gong"},
    {'P',   0,  49,  51, "SC-8850 ETHNIC Set", "Udo Short"},
    {'P',   0,  49,  52, "SC-8850 ETHNIC Set", "Udo Long"},
    {'P',   0,  49,  53, "SC-8850 ETHNIC Set", "Udo Slap"},
    {'P',   0,  49,  54, "SC-8850 ETHNIC Set", "Bendir"},
    {'P',   0,  49,  55, "SC-8850 ETHNIC Set", "Req Dum"},
    {'P',   0,  49,  56, "SC-8850 ETHNIC Set", "Req Tik"},
    {'P',   0,  49,  57, "SC-8850 ETHNIC Set", "Tabla Te"},
    {'P',   0,  49,  58, "SC-8850 ETHNIC Set", "Tabla Na"},
    {'P',   0,  49,  59, "SC-8850 ETHNIC Set", "Tabla Tun"},
    {'P',   0,  49,  60, "SC-8850 ETHNIC Set", "Tabla Ge"},
    {'P',   0,  49,  61, "SC-8850 ETHNIC Set", "Tabla Ge Hi"},
    {'P',   0,  49,  62, "SC-8850 ETHNIC Set", "Talking Drum"},
    {'P',   0,  49,  63, "SC-8850 ETHNIC Set", "Bend Talking Drum"},
    {'P',   0,  49,  64, "SC-8850 ETHNIC Set", "Caxixi"},
    {'P',   0,  49,  65, "SC-8850 ETHNIC Set", "Djembe"},
    {'P',   0,  49,  66, "SC-8850 ETHNIC Set", "Djembe Rim"},
    {'P',   0,  49,  67, "SC-8850 ETHNIC Set", "Timbales Low"},
    {'P',   0,  49,  68, "SC-8850 ETHNIC Set", "Timbales Paila"},
    {'P',   0,  49,  69, "SC-8850 ETHNIC Set", "Timbales High"},
    {'P',   0,  49,  70, "SC-8850 ETHNIC Set", "Cowbell"},
    {'P',   0,  49,  71, "SC-8850 ETHNIC Set", "High Bongo"},
    {'P',   0,  49,  72, "SC-8850 ETHNIC Set", "Low Bongo"},
    {'P',   0,  49,  73, "SC-8850 ETHNIC Set", "Mute High Conga"},
    {'P',   0,  49,  74, "SC-8850 ETHNIC Set", "Open High Conga"},
    {'P',   0,  49,  75, "SC-8850 ETHNIC Set", "Mute Low Conga"},
    {'P',   0,  49,  76, "SC-8850 ETHNIC Set", "Conga Slap"},
    {'P',   0,  49,  77, "SC-8850 ETHNIC Set", "Open Low Conga"},
    {'P',   0,  49,  78, "SC-8850 ETHNIC Set", "Conga Slide"},
    {'P',   0,  49,  79, "SC-8850 ETHNIC Set", "Mute Pandiero"},
    {'P',   0,  49,  80, "SC-8850 ETHNIC Set", "Open Pandiero"},
    {'P',   0,  49,  81, "SC-8850 ETHNIC Set", "Open Surdo"},
    {'P',   0,  49,  82, "SC-8850 ETHNIC Set", "Mute Surdo"},
    {'P',   0,  49,  83, "SC-8850 ETHNIC Set", "Tamborim"},
    {'P',   0,  49,  84, "SC-8850 ETHNIC Set", "High Agogo"},
    {'P',   0,  49,  85, "SC-8850 ETHNIC Set", "Low Agogo"},
    {'P',   0,  49,  86, "SC-8850 ETHNIC Set", "Shaker"},
    {'P',   0,  49,  87, "SC-8850 ETHNIC Set", "High Whistle"},
    {'P',   0,  49,  88, "SC-8850 ETHNIC Set", "Low Whistle"},
    {'P',   0,  49,  89, "SC-8850 ETHNIC Set", "Mute Cuica"},
    {'P',   0,  49,  90, "SC-8850 ETHNIC Set", "Open Cuica"},
    {'P',   0,  49,  91, "SC-8850 ETHNIC Set", "Mute Triangle"},
    {'P',   0,  49,  92, "SC-8850 ETHNIC Set", "Open Triangle"},
    {'P',   0,  49,  93, "SC-8850 ETHNIC Set", "Short Guiro"},
    {'P',   0,  49,  94, "SC-8850 ETHNIC Set", "Long Guiro"},
    {'P',   0,  49,  95, "SC-8850 ETHNIC Set", "Cabasa Up"},
    {'P',   0,  49,  96, "SC-8850 ETHNIC Set", "Cabasa Down"},
    {'P',   0,  49,  97, "SC-8850 ETHNIC Set", "Claves"},
    {'P',   0,  49,  98, "SC-8850 ETHNIC Set", "High Wood Block"},
    {'P',   0,  49,  99, "SC-8850 ETHNIC Set", "Low Wood Block"},
    {'P',   0,  50,  25, "SC-8850 KICK & SNARE Set", "CR-78 Kick 1"},
    {'P',   0,  50,  26, "SC-8850 KICK & SNARE Set", "CR-78 Kick 2"},
    {'P',   0,  50,  27, "SC-8850 KICK & SNARE Set", "TR-606 Kick"},
    {'P',   0,  50,  28, "SC-8850 KICK & SNARE Set", "TR-707 Kick"},
    {'P',   0,  50,  29, "SC-8850 KICK & SNARE Set", "TR-808 Kick 1"},
    {'P',   0,  50,  30, "SC-8850 KICK & SNARE Set", "TR-909 Kick 1"},
    {'P',   0,  50,  31, "SC-8850 KICK & SNARE Set", "TR-909 Kick 2"},
    {'P',   0,  50,  32, "SC-8850 KICK & SNARE Set", "Hip-Hop Kick 2"},
    {'P',   0,  50,  33, "SC-8850 KICK & SNARE Set", "Hip-Hop Kick 1"},
    {'P',   0,  50,  34, "SC-8850 KICK & SNARE Set", "Jungle Kick 2"},
    {'P',   0,  50,  35, "SC-8850 KICK & SNARE Set", "Jungle Kick 1"},
    {'P',   0,  50,  36, "SC-8850 KICK & SNARE Set", "Techno Kick 2"},
    {'P',   0,  50,  37, "SC-8850 KICK & SNARE Set", "Techno Kick 1"},
    {'P',   0,  50,  38, "SC-8850 KICK & SNARE Set", "Standard 1 Kick 2"},
    {'P',   0,  50,  39, "SC-8850 KICK & SNARE Set", "Standard 1 Kick 1"},
    {'P',   0,  50,  40, "SC-8850 KICK & SNARE Set", "[88] Standard 1 Kick 1"},
    {'P',   0,  50,  41, "SC-8850 KICK & SNARE Set", "[88] Standard 1 Kick 2"},
    {'P',   0,  50,  42, "SC-8850 KICK & SNARE Set", "[88] Standard 2 Kick 1"},
    {'P',   0,  50,  43, "SC-8850 KICK & SNARE Set", "[88] Standard 2 Kick 2"},
    {'P',   0,  50,  44, "SC-8850 KICK & SNARE Set", "[55] Kick Drum1"},
    {'P',   0,  50,  45, "SC-8850 KICK & SNARE Set", "[55] Kick Drum 2"},
    {'P',   0,  50,  46, "SC-8850 KICK & SNARE Set", "[88] Soft Kick"},
    {'P',   0,  50,  47, "SC-8850 KICK & SNARE Set", "[88] Jazz Kick 1"},
    {'P',   0,  50,  48, "SC-8850 KICK & SNARE Set", "[88] Jazz Kick 2"},
    {'P',   0,  50,  49, "SC-8850 KICK & SNARE Set", "[55] Concert BD 1"},
    {'P',   0,  50,  50, "SC-8850 KICK & SNARE Set", "[88] Room Kick 1"},
    {'P',   0,  50,  51, "SC-8850 KICK & SNARE Set", "[88] Room Kick 2"},
    {'P',   0,  50,  52, "SC-8850 KICK & SNARE Set", "[88] Power Kick1"},
    {'P',   0,  50,  53, "SC-8850 KICK & SNARE Set", "[88] Power Kick2"},
    {'P',   0,  50,  54, "SC-8850 KICK & SNARE Set", "[88] Electric Kick 2"},
    {'P',   0,  50,  55, "SC-8850 KICK & SNARE Set", "[88] Electric Kick 1"},
    {'P',   0,  50,  56, "SC-8850 KICK & SNARE Set", "[55] Electric Kick"},
    {'P',   0,  50,  57, "SC-8850 KICK & SNARE Set", "[88] TR-808 Kick"},
    {'P',   0,  50,  58, "SC-8850 KICK & SNARE Set", "[88] TR-909 Kick"},
    {'P',   0,  50,  59, "SC-8850 KICK & SNARE Set", "[88] Dance Kick"},
    {'P',   0,  50,  60, "SC-8850 KICK & SNARE Set", "[88] Standard 1 Snare 1"},
    {'P',   0,  50,  61, "SC-8850 KICK & SNARE Set", "[88] Standard 1 Snare 2"},
    {'P',   0,  50,  62, "SC-8850 KICK & SNARE Set", "[88] Standard 2 Snare 1"},
    {'P',   0,  50,  63, "SC-8850 KICK & SNARE Set", "[88] Standard 2 Snare 2"},
    {'P',   0,  50,  64, "SC-8850 KICK & SNARE Set", "[55] Tight Snare"},
    {'P',   0,  50,  65, "SC-8850 KICK & SNARE Set", "[55] Concert Snare"},
    {'P',   0,  50,  66, "SC-8850 KICK & SNARE Set", "[88] Jazz Snare 1"},
    {'P',   0,  50,  67, "SC-8850 KICK & SNARE Set", "[88] Jazz Snare 2"},
    {'P',   0,  50,  68, "SC-8850 KICK & SNARE Set", "[88] Room Snare 1"},
    {'P',   0,  50,  69, "SC-8850 KICK & SNARE Set", "[88] Room Snare 2"},
    {'P',   0,  50,  70, "SC-8850 KICK & SNARE Set", "[88] Power Snare 1"},
    {'P',   0,  50,  71, "SC-8850 KICK & SNARE Set", "[88] Power Snare 2"},
    {'P',   0,  50,  72, "SC-8850 KICK & SNARE Set", "[55] Gated Snare"},
    {'P',   0,  50,  73, "SC-8850 KICK & SNARE Set", "[88] Dance Snare 1"},
    {'P',   0,  50,  74, "SC-8850 KICK & SNARE Set", "[88] Dance Snare 2"},
    {'P',   0,  50,  75, "SC-8850 KICK & SNARE Set", "[88] Disco Snare"},
    {'P',   0,  50,  76, "SC-8850 KICK & SNARE Set", "[88] Electric Snare 2"},
    {'P',   0,  50,  77, "SC-8850 KICK & SNARE Set", "[88] House Snare"},
    {'P',   0,  50,  78, "SC-8850 KICK & SNARE Set", "[55] Electric Snare 1"},
    {'P',   0,  50,  79, "SC-8850 KICK & SNARE Set", "[88] Electric Snare 3"},
    {'P',   0,  50,  80, "SC-8850 KICK & SNARE Set", "[88] TR-808 Snare 1"},
    {'P',   0,  50,  81, "SC-8850 KICK & SNARE Set", "[88] TR-808 Snare 2"},
    {'P',   0,  50,  82, "SC-8850 KICK & SNARE Set", "[88] TR-909 Snare 1"},
    {'P',   0,  50,  83, "SC-8850 KICK & SNARE Set", "[88] TR-909 Snare 2"},
    {'P',   0,  50,  84, "SC-8850 KICK & SNARE Set", "[88] Brush Tap 1"},
    {'P',   0,  50,  85, "SC-8850 KICK & SNARE Set", "[88] Brush Tap 2"},
    {'P',   0,  50,  86, "SC-8850 KICK & SNARE Set", "[88] Brush Slap 1"},
    {'P',   0,  50,  87, "SC-8850 KICK & SNARE Set", "[88] Brush Slap 2"},
    {'P',   0,  50,  88, "SC-8850 KICK & SNARE Set", "[88] Brush Slap 3"},
    {'P',   0,  50,  89, "SC-8850 KICK & SNARE Set", "[88] Brush Swirl 1"},
    {'P',   0,  50,  90, "SC-8850 KICK & SNARE Set", "[88] Brush Swirl 2"},
    {'P',   0,  50,  91, "SC-8850 KICK & SNARE Set", "[88] Brush Long Swirl"},
    {'P',   0,  50,  92, "SC-8850 KICK & SNARE Set", "Standard 1 Snare 1"},
    {'P',   0,  50,  93, "SC-8850 KICK & SNARE Set", "Standard 1 Snare 2"},
    {'P',   0,  50,  94, "SC-8850 KICK & SNARE Set", "Standard 1 Snare 3"},
    {'P',   0,  50,  95, "SC-8850 KICK & SNARE Set", "Rap Snare"},
    {'P',   0,  50,  96, "SC-8850 KICK & SNARE Set", "Hip-Hop Snare 2"},
    {'P',   0,  50,  97, "SC-8850 KICK & SNARE Set", "Jungle Snare 1"},
    {'P',   0,  50,  98, "SC-8850 KICK & SNARE Set", "Jungle Snare 2"},
    {'P',   0,  50,  99, "SC-8850 KICK & SNARE Set", "Techno Snare 1"},
    {'P',   0,  50, 100, "SC-8850 KICK & SNARE Set", "Techno Snare 2"},
    {'P',   0,  50, 101, "SC-8850 KICK & SNARE Set", "House Snare 2"},
    {'P',   0,  50, 102, "SC-8850 KICK & SNARE Set", "CR-78 Snare 1"},
    {'P',   0,  50, 103, "SC-8850 KICK & SNARE Set", "CR-78 Snare 2"},
    {'P',   0,  50, 104, "SC-8850 KICK & SNARE Set", "TR-606 Snare 1"},
    {'P',   0,  50, 105, "SC-8850 KICK & SNARE Set", "TR-606 Snare 2"},
    {'P',   0,  50, 106, "SC-8850 KICK & SNARE Set", "TR-707 Snare 1"},
    {'P',   0,  50, 107, "SC-8850 KICK & SNARE Set", "TR-707 Snare 2"},
    {'P',   0,  50, 108, "SC-8850 KICK & SNARE Set", "Standard 3 Snare 2"},
    {'P',   0,  50, 109, "SC-8850 KICK & SNARE Set", "TR-808 Snare 2"},
    {'P',   0,  50, 110, "SC-8850 KICK & SNARE Set", "TR-909 Snare 1"},
    {'P',   0,  50, 111, "SC-8850 KICK & SNARE Set", "TR-909 Snare 2"},
    {'P',   0,  51,  25, "SC-8850 KICK & SNARE 2 Set", "[Pro] CR-78 Kick 1"},
    {'P',   0,  51,  26, "SC-8850 KICK & SNARE 2 Set", "[Pro] CR-78 Kick 2"},
    {'P',   0,  51,  27, "SC-8850 KICK & SNARE 2 Set", "[Pro] TR-606 Kick"},
    {'P',   0,  51,  28, "SC-8850 KICK & SNARE 2 Set", "[Pro] TR-707 Kick"},
    {'P',   0,  51,  29, "SC-8850 KICK & SNARE 2 Set", "[Pro] TR-808 Kick 1"},
    {'P',   0,  51,  30, "SC-8850 KICK & SNARE 2 Set", "[Pro] TR-909 Kick 1"},
    {'P',   0,  51,  31, "SC-8850 KICK & SNARE 2 Set", "[Pro] TR-909 Kick 2"},
    {'P',   0,  51,  32, "SC-8850 KICK & SNARE 2 Set", "[Pro] Hip-Hop Kick 2"},
    {'P',   0,  51,  33, "SC-8850 KICK & SNARE 2 Set", "[Pro] Hip-Hop Kick 1"},
    {'P',   0,  51,  34, "SC-8850 KICK & SNARE 2 Set", "[Pro] Jungle Kick 2"},
    {'P',   0,  51,  35, "SC-8850 KICK & SNARE 2 Set", "[Pro] Jungle Kick 1"},
    {'P',   0,  51,  36, "SC-8850 KICK & SNARE 2 Set", "[Pro] Techno Kick 2"},
    {'P',   0,  51,  37, "SC-8850 KICK & SNARE 2 Set", "[Pro] Techno Kick 1"},
    {'P',   0,  51,  38, "SC-8850 KICK & SNARE 2 Set", "[Pro] Standard 1 Kick 2"},
    {'P',   0,  51,  39, "SC-8850 KICK & SNARE 2 Set", "[Pro] Standard 1 Kick 1"},
    {'P',   0,  51,  40, "SC-8850 KICK & SNARE 2 Set", "Standard 1 Kick 2"},
    {'P',   0,  51,  41, "SC-8850 KICK & SNARE 2 Set", "Standard 1 Kick 1"},
    {'P',   0,  51,  42, "SC-8850 KICK & SNARE 2 Set", "Brush Kick 2"},
    {'P',   0,  51,  43, "SC-8850 KICK & SNARE 2 Set", "Brush Kick 1"},
    {'P',   0,  51,  44, "SC-8850 KICK & SNARE 2 Set", "Jazz Kick 2"},
    {'P',   0,  51,  45, "SC-8850 KICK & SNARE 2 Set", "Jazz Kick 1"},
    {'P',   0,  51,  46, "SC-8850 KICK & SNARE 2 Set", "Hip-Hop Kick 2"},
    {'P',   0,  51,  47, "SC-8850 KICK & SNARE 2 Set", "Hip-Hop Kick 1"},
    {'P',   0,  51,  48, "SC-8850 KICK & SNARE 2 Set", "Concert BD 1 Mute"},
    {'P',   0,  51,  49, "SC-8850 KICK & SNARE 2 Set", "[55] Concert BD 1"},
    {'P',   0,  51,  50, "SC-8850 KICK & SNARE 2 Set", "Room Kick 2"},
    {'P',   0,  51,  51, "SC-8850 KICK & SNARE 2 Set", "Room Kick 1"},
    {'P',   0,  51,  52, "SC-8850 KICK & SNARE 2 Set", "Jungle Kick 2"},
    {'P',   0,  51,  53, "SC-8850 KICK & SNARE 2 Set", "Jungle Kick 1"},
    {'P',   0,  51,  54, "SC-8850 KICK & SNARE 2 Set", "Jungle Kick Roll"},
    {'P',   0,  51,  55, "SC-8850 KICK & SNARE 2 Set", "Fat Kick"},
    {'P',   0,  51,  56, "SC-8850 KICK & SNARE 2 Set", "Dance Kick"},
    {'P',   0,  51,  57, "SC-8850 KICK & SNARE 2 Set", "TR-808 Kick"},
    {'P',   0,  51,  58, "SC-8850 KICK & SNARE 2 Set", "TR-909 Kick 2"},
    {'P',   0,  51,  59, "SC-8850 KICK & SNARE 2 Set", "TR-909 Kick 1"},
    {'P',   0,  51,  60, "SC-8850 KICK & SNARE 2 Set", "Standard 1 Snare 1"},
    {'P',   0,  51,  61, "SC-8850 KICK & SNARE 2 Set", "Standard 1 Snare 2"},
    {'P',   0,  51,  62, "SC-8850 KICK & SNARE 2 Set", "[88] Standard 2 Snare 1"},
    {'P',   0,  51,  63, "SC-8850 KICK & SNARE 2 Set", "[88] Standard 2 Snare 2"},
    {'P',   0,  51,  64, "SC-8850 KICK & SNARE 2 Set", "[55] Tight Snare"},
    {'P',   0,  51,  65, "SC-8850 KICK & SNARE 2 Set", "[55] Concert Snare"},
    {'P',   0,  51,  66, "SC-8850 KICK & SNARE 2 Set", "Jazz Snare 1"},
    {'P',   0,  51,  67, "SC-8850 KICK & SNARE 2 Set", "Jazz Snare 2"},
    {'P',   0,  51,  68, "SC-8850 KICK & SNARE 2 Set", "Room Snare 1"},
    {'P',   0,  51,  69, "SC-8850 KICK & SNARE 2 Set", "Room Snare 2"},
    {'P',   0,  51,  70, "SC-8850 KICK & SNARE 2 Set", "LoFi Snare 1"},
    {'P',   0,  51,  71, "SC-8850 KICK & SNARE 2 Set", "LoFi Snare 2"},
    {'P',   0,  51,  72, "SC-8850 KICK & SNARE 2 Set", "[55] Gated Snare"},
    {'P',   0,  51,  73, "SC-8850 KICK & SNARE 2 Set", "LoFi Snare Rim"},
    {'P',   0,  51,  74, "SC-8850 KICK & SNARE 2 Set", "[88] Dance Snare 2"},
    {'P',   0,  51,  75, "SC-8850 KICK & SNARE 2 Set", "HipHop Snare 1"},
    {'P',   0,  51,  76, "SC-8850 KICK & SNARE 2 Set", "HipHop Snare 2"},
    {'P',   0,  51,  77, "SC-8850 KICK & SNARE 2 Set", "Dance Snare"},
    {'P',   0,  51,  78, "SC-8850 KICK & SNARE 2 Set", "TR-606 Snare 2"},
    {'P',   0,  51,  79, "SC-8850 KICK & SNARE 2 Set", "Techno Snare"},
    {'P',   0,  51,  80, "SC-8850 KICK & SNARE 2 Set", "House Snare"},
    {'P',   0,  51,  81, "SC-8850 KICK & SNARE 2 Set", "Rock Snare Dry"},
    {'P',   0,  51,  82, "SC-8850 KICK & SNARE 2 Set", "Jungle Snare"},
    {'P',   0,  51,  83, "SC-8850 KICK & SNARE 2 Set", "Jungle Snare Roll"},
    {'P',   0,  51,  84, "SC-8850 KICK & SNARE 2 Set", "[88] Brush Tap 1"},
    {'P',   0,  51,  85, "SC-8850 KICK & SNARE 2 Set", "[88] Brush Tap 2"},
    {'P',   0,  51,  86, "SC-8850 KICK & SNARE 2 Set", "Brush Tap 2"},
    {'P',   0,  51,  87, "SC-8850 KICK & SNARE 2 Set", "Brush Slap 2"},
    {'P',   0,  51,  88, "SC-8850 KICK & SNARE 2 Set", "[88] Brush Slap 3"},
    {'P',   0,  51,  89, "SC-8850 KICK & SNARE 2 Set", "[88] Brush Swirl 1"},
    {'P',   0,  51,  90, "SC-8850 KICK & SNARE 2 Set", "[88] Brush Swirl 2"},
    {'P',   0,  51,  91, "SC-8850 KICK & SNARE 2 Set", "[88] Brush Long Swirl"},
    {'P',   0,  51,  92, "SC-8850 KICK & SNARE 2 Set", "[Pro] Standard 1 Snare 1"},
    {'P',   0,  51,  93, "SC-8850 KICK & SNARE 2 Set", "[Pro] Standard 1 Snare 2"},
    {'P',   0,  51,  94, "SC-8850 KICK & SNARE 2 Set", "[Pro] Standard 1 Snare 3"},
    {'P',   0,  51,  95, "SC-8850 KICK & SNARE 2 Set", "[Pro] Rap Snare"},
    {'P',   0,  51,  96, "SC-8850 KICK & SNARE 2 Set", "[Pro] Hip-Hop Snare 2"},
    {'P',   0,  51,  97, "SC-8850 KICK & SNARE 2 Set", "[Pro] Jungle Snare 1"},
    {'P',   0,  51,  98, "SC-8850 KICK & SNARE 2 Set", "[Pro] Jungle Snare 2"},
    {'P',   0,  51,  99, "SC-8850 KICK & SNARE 2 Set", "[Pro] Techno Snare 1"},
    {'P',   0,  51, 100, "SC-8850 KICK & SNARE 2 Set", "[Pro] Techno Snare 2"},
    {'P',   0,  51, 101, "SC-8850 KICK & SNARE 2 Set", "[Pro] House Snare 2"},
    {'P',   0,  51, 102, "SC-8850 KICK & SNARE 2 Set", "[Pro] CR-78 Snare 1"},
    {'P',   0,  51, 103, "SC-8850 KICK & SNARE 2 Set", "[Pro] CR-78 Snare 2"},
    {'P',   0,  51, 104, "SC-8850 KICK & SNARE 2 Set", "[Pro] TR-606 Snare 1"},
    {'P',   0,  51, 105, "SC-8850 KICK & SNARE 2 Set", "[Pro] TR-606 Snare 2"},
    {'P',   0,  51, 106, "SC-8850 KICK & SNARE 2 Set", "[Pro] TR-707 Snare 1"},
    {'P',   0,  51, 107, "SC-8850 KICK & SNARE 2 Set", "[Pro] TR-707 Snare 2"},
    {'P',   0,  51, 108, "SC-8850 KICK & SNARE 2 Set", "[Pro] Standard 3 Snare 2"},
    {'P',   0,  51, 109, "SC-8850 KICK & SNARE 2 Set", "[Pro] TR-808 Snare 2"},
    {'P',   0,  51, 110, "SC-8850 KICK & SNARE 2 Set", "[Pro] TR-909 Snare 1"},
    {'P',   0,  51, 111, "SC-8850 KICK & SNARE 2 Set", "[Pro] TR-909 Snare 2"},
    {'P',   0,  52,  25, "SC-8850 ASIA Set", "Gamelan Gong 1"},
    {'P',   0,  52,  26, "SC-8850 ASIA Set", "Gamelan Gong 2"},
    {'P',   0,  52,  27, "SC-8850 ASIA Set", "Gamelan Gong 3"},
    {'P',   0,  52,  28, "SC-8850 ASIA Set", "Gamelan Gong 4"},
    {'P',   0,  52,  29, "SC-8850 ASIA Set", "Gamelan Gong 5"},
    {'P',   0,  52,  30, "SC-8850 ASIA Set", "Gamelan Gong 6"},
    {'P',   0,  52,  31, "SC-8850 ASIA Set", "Gamelan Gong 7"},
    {'P',   0,  52,  32, "SC-8850 ASIA Set", "Gamelan Gong 8"},
    {'P',   0,  52,  33, "SC-8850 ASIA Set", "Gamelan Gong 9"},
    {'P',   0,  52,  34, "SC-8850 ASIA Set", "Gamelan Gong 10"},
    {'P',   0,  52,  35, "SC-8850 ASIA Set", "Gender 1"},
    {'P',   0,  52,  36, "SC-8850 ASIA Set", "Gender 2"},
    {'P',   0,  52,  37, "SC-8850 ASIA Set", "Gender 3"},
    {'P',   0,  52,  38, "SC-8850 ASIA Set", "Gender 4"},
    {'P',   0,  52,  39, "SC-8850 ASIA Set", "Gender 5"},
    {'P',   0,  52,  40, "SC-8850 ASIA Set", "Bonang 1"},
    {'P',   0,  52,  41, "SC-8850 ASIA Set", "Bonang 2"},
    {'P',   0,  52,  42, "SC-8850 ASIA Set", "Bonang 3"},
    {'P',   0,  52,  43, "SC-8850 ASIA Set", "Bonang 4"},
    {'P',   0,  52,  44, "SC-8850 ASIA Set", "Bonang 5"},
    {'P',   0,  52,  45, "SC-8850 ASIA Set", "Rama Cymbal Low"},
    {'P',   0,  52,  46, "SC-8850 ASIA Set", "Rama Cymbal High"},
    {'P',   0,  52,  47, "SC-8850 ASIA Set", "Sagat Open"},
    {'P',   0,  52,  48, "SC-8850 ASIA Set", "Sagat Closed"},
    {'P',   0,  52,  49, "SC-8850 ASIA Set", "Jaws Harp"},
    {'P',   0,  52,  50, "SC-8850 ASIA Set", "Wadaiko"},
    {'P',   0,  52,  51, "SC-8850 ASIA Set", "Wadaiko Rim"},
    {'P',   0,  52,  52, "SC-8850 ASIA Set", "Small Taiko"},
    {'P',   0,  52,  53, "SC-8850 ASIA Set", "Shimetaiko"},
    {'P',   0,  52,  54, "SC-8850 ASIA Set", "Atarigane"},
    {'P',   0,  52,  55, "SC-8850 ASIA Set", "Hyoushigi"},
    {'P',   0,  52,  56, "SC-8850 ASIA Set", "Ohkawa"},
    {'P',   0,  52,  57, "SC-8850 ASIA Set", "High Kotsuzumi"},
    {'P',   0,  52,  58, "SC-8850 ASIA Set", "Low Kotsuzumi"},
    {'P',   0,  52,  59, "SC-8850 ASIA Set", "Yyoo Dude"},
    {'P',   0,  52,  60, "SC-8850 ASIA Set", "Buk"},
    {'P',   0,  52,  61, "SC-8850 ASIA Set", "Buk Rim"},
    {'P',   0,  52,  62, "SC-8850 ASIA Set", "Gengari p"},
    {'P',   0,  52,  63, "SC-8850 ASIA Set", "Gengari Mute Low"},
    {'P',   0,  52,  64, "SC-8850 ASIA Set", "Gengari f"},
    {'P',   0,  52,  65, "SC-8850 ASIA Set", "Gengari Mute High"},
    {'P',   0,  52,  66, "SC-8850 ASIA Set", "Gengari Samll"},
    {'P',   0,  52,  67, "SC-8850 ASIA Set", "Jang-Gu Che"},
    {'P',   0,  52,  68, "SC-8850 ASIA Set", "Jang-Gu Kun"},
    {'P',   0,  52,  69, "SC-8850 ASIA Set", "Jang-Gu Rim"},
    {'P',   0,  52,  70, "SC-8850 ASIA Set", "Jing p"},
    {'P',   0,  52,  71, "SC-8850 ASIA Set", "Jing f"},
    {'P',   0,  52,  72, "SC-8850 ASIA Set", "Jing Mute"},
    {'P',   0,  52,  73, "SC-8850 ASIA Set", "Asian Gong"},
    {'P',   0,  52,  74, "SC-8850 ASIA Set", "Big Gong"},
    {'P',   0,  52,  75, "SC-8850 ASIA Set", "Small Gong"},
    {'P',   0,  52,  76, "SC-8850 ASIA Set", "Pai Ban"},
    {'P',   0,  52,  77, "SC-8850 ASIA Set", "Ban Gu"},
    {'P',   0,  52,  78, "SC-8850 ASIA Set", "Tang Gu"},
    {'P',   0,  52,  79, "SC-8850 ASIA Set", "Tang Gu Mute"},
    {'P',   0,  52,  80, "SC-8850 ASIA Set", "Shou Luo"},
    {'P',   0,  52,  81, "SC-8850 ASIA Set", "Bend Gong"},
    {'P',   0,  52,  82, "SC-8850 ASIA Set", "Hu Yin Luo Low"},
    {'P',   0,  52,  83, "SC-8850 ASIA Set", "Hu Yin Luo Mid"},
    {'P',   0,  52,  84, "SC-8850 ASIA Set", "Hu Yin Luo Mid 2"},
    {'P',   0,  52,  85, "SC-8850 ASIA Set", "Hu Yin Luo High"},
    {'P',   0,  52,  86, "SC-8850 ASIA Set", "Hu Yin Luo High 2"},
    {'P',   0,  52,  87, "SC-8850 ASIA Set", "Nao Bo"},
    {'P',   0,  52,  88, "SC-8850 ASIA Set", "Xiao Bo"},
    {'P',   0,  52,  89, "SC-8850 ASIA Set", "Dholak 1"},
    {'P',   0,  52,  90, "SC-8850 ASIA Set", "Dholak 2"},
    {'P',   0,  53,  32, "SC-8850 CYMBAL&CLAPS Set", "Reverse Open Hi-Hat"},
    {'P',   0,  53,  33, "SC-8850 CYMBAL&CLAPS Set", "Reverse Closed Hi-Hat 1"},
    {'P',   0,  53,  34, "SC-8850 CYMBAL&CLAPS Set", "Reverse Closed Hi-Hat 2"},
    {'P',   0,  53,  35, "SC-8850 CYMBAL&CLAPS Set", "Jungle Hi-Hat"},
    {'P',   0,  53,  36, "SC-8850 CYMBAL&CLAPS Set", "[55] Closed Hi-Hat"},
    {'P',   0,  53,  37, "SC-8850 CYMBAL&CLAPS Set", "[88] Closed Hi-Hat 2"},
    {'P',   0,  53,  38, "SC-8850 CYMBAL&CLAPS Set", "[88] Closed Hi-Hat 3"},
    {'P',   0,  53,  39, "SC-8850 CYMBAL&CLAPS Set", "Closed Hi-Hat 4"},
    {'P',   0,  53,  40, "SC-8850 CYMBAL&CLAPS Set", "Closed Hi-Hat"},
    {'P',   0,  53,  41, "SC-8850 CYMBAL&CLAPS Set", "TR-707 Closed Hi-Hat"},
    {'P',   0,  53,  42, "SC-8850 CYMBAL&CLAPS Set", "TR-606 Closed Hi-Hat"},
    {'P',   0,  53,  43, "SC-8850 CYMBAL&CLAPS Set", "[88] TR-808 Closed Hi-Hat"},
    {'P',   0,  53,  44, "SC-8850 CYMBAL&CLAPS Set", "TR-808 Closed Hi-Hat"},
    {'P',   0,  53,  45, "SC-8850 CYMBAL&CLAPS Set", "CR-78 Closed Hi-Hat"},
    {'P',   0,  53,  46, "SC-8850 CYMBAL&CLAPS Set", "[55] Pedal Hi-Hat"},
    {'P',   0,  53,  47, "SC-8850 CYMBAL&CLAPS Set", "[88] Pedal Hi-Hat"},
    {'P',   0,  53,  48, "SC-8850 CYMBAL&CLAPS Set", "Pedal Hi-Hat"},
    {'P',   0,  53,  49, "SC-8850 CYMBAL&CLAPS Set", "Half-Open Hi-Hat 1"},
    {'P',   0,  53,  50, "SC-8850 CYMBAL&CLAPS Set", "Half-Open Hi-Hat 2"},
    {'P',   0,  53,  51, "SC-8850 CYMBAL&CLAPS Set", "[55] Open Hi-Hat"},
    {'P',   0,  53,  52, "SC-8850 CYMBAL&CLAPS Set", "[88] Open Hi-Hat 2"},
    {'P',   0,  53,  53, "SC-8850 CYMBAL&CLAPS Set", "[88] Open Hi-Hat 3"},
    {'P',   0,  53,  54, "SC-8850 CYMBAL&CLAPS Set", "Open Hi-Hat"},
    {'P',   0,  53,  55, "SC-8850 CYMBAL&CLAPS Set", "TR-909 Open Hi-Hat"},
    {'P',   0,  53,  56, "SC-8850 CYMBAL&CLAPS Set", "TR-707 Open Hi-Hat"},
    {'P',   0,  53,  57, "SC-8850 CYMBAL&CLAPS Set", "TR-606 Open Hi-Hat"},
    {'P',   0,  53,  58, "SC-8850 CYMBAL&CLAPS Set", "[88] TR-808 Open Hi-Hat"},
    {'P',   0,  53,  59, "SC-8850 CYMBAL&CLAPS Set", "TR-808 Open Hi-Hat"},
    {'P',   0,  53,  60, "SC-8850 CYMBAL&CLAPS Set", "CR-78 Open Hi-Hat"},
    {'P',   0,  53,  61, "SC-8850 CYMBAL&CLAPS Set", "Crash Cymbal 1"},
    {'P',   0,  53,  62, "SC-8850 CYMBAL&CLAPS Set", "Crash Cymbal 2"},
    {'P',   0,  53,  63, "SC-8850 CYMBAL&CLAPS Set", "Crash Cymbal 3"},
    {'P',   0,  53,  64, "SC-8850 CYMBAL&CLAPS Set", "Brush Crash Cymbal"},
    {'P',   0,  53,  65, "SC-8850 CYMBAL&CLAPS Set", "Hard Crash Cymbal"},
    {'P',   0,  53,  66, "SC-8850 CYMBAL&CLAPS Set", "TR-909 Crash Cymbal"},
    {'P',   0,  53,  67, "SC-8850 CYMBAL&CLAPS Set", "TR-808 Crash Cymbal"},
    {'P',   0,  53,  68, "SC-8850 CYMBAL&CLAPS Set", "Mute Crash Cymbal 1"},
    {'P',   0,  53,  69, "SC-8850 CYMBAL&CLAPS Set", "Mute Crash Cymbal 2"},
    {'P',   0,  53,  70, "SC-8850 CYMBAL&CLAPS Set", "Reverse Crash Cymbal 1"},
    {'P',   0,  53,  71, "SC-8850 CYMBAL&CLAPS Set", "Reverse Crash Cymbal 2"},
    {'P',   0,  53,  72, "SC-8850 CYMBAL&CLAPS Set", "Reverse Crash Cymbal 3"},
    {'P',   0,  53,  73, "SC-8850 CYMBAL&CLAPS Set", "Reverse TR-909 Crash Cymbal"},
    {'P',   0,  53,  74, "SC-8850 CYMBAL&CLAPS Set", "[55] Splash Cymbal"},
    {'P',   0,  53,  75, "SC-8850 CYMBAL&CLAPS Set", "Splash Cymbal"},
    {'P',   0,  53,  76, "SC-8850 CYMBAL&CLAPS Set", "[88] Ride Bell"},
    {'P',   0,  53,  77, "SC-8850 CYMBAL&CLAPS Set", "[88] Brush Ride Bell"},
    {'P',   0,  53,  78, "SC-8850 CYMBAL&CLAPS Set", "[88] Ride Cymbal 1"},
    {'P',   0,  53,  79, "SC-8850 CYMBAL&CLAPS Set", "[88] Ride Cymbal 2"},
    {'P',   0,  53,  80, "SC-8850 CYMBAL&CLAPS Set", "[88] Brush Ride Cymbal"},
    {'P',   0,  53,  81, "SC-8850 CYMBAL&CLAPS Set", "Ride Cymbal Low Inner"},
    {'P',   0,  53,  82, "SC-8850 CYMBAL&CLAPS Set", "Ride Cymbal Mid Inner"},
    {'P',   0,  53,  83, "SC-8850 CYMBAL&CLAPS Set", "Ride Cymbal High Inner"},
    {'P',   0,  53,  84, "SC-8850 CYMBAL&CLAPS Set", "Ride Cymbal Low Edge"},
    {'P',   0,  53,  85, "SC-8850 CYMBAL&CLAPS Set", "Ride Cymbal Mid Edge"},
    {'P',   0,  53,  86, "SC-8850 CYMBAL&CLAPS Set", "Ride Cymbal High Edge"},
    {'P',   0,  53,  87, "SC-8850 CYMBAL&CLAPS Set", "TR-606 Ride Cymbal"},
    {'P',   0,  53,  88, "SC-8850 CYMBAL&CLAPS Set", "TR-808 Ride Cymbal"},
    {'P',   0,  53,  89, "SC-8850 CYMBAL&CLAPS Set", "Chinese Cymbal"},
    {'P',   0,  53,  90, "SC-8850 CYMBAL&CLAPS Set", "Chinese Cymbal 2"},
    {'P',   0,  53,  91, "SC-8850 CYMBAL&CLAPS Set", "[55] Hand Clap"},
    {'P',   0,  53,  92, "SC-8850 CYMBAL&CLAPS Set", "[88] Hand Clap 2"},
    {'P',   0,  53,  93, "SC-8850 CYMBAL&CLAPS Set", "[88] Hand Clap"},
    {'P',   0,  53,  94, "SC-8850 CYMBAL&CLAPS Set", "Hand Clap"},
    {'P',   0,  53,  95, "SC-8850 CYMBAL&CLAPS Set", "Hand Clap 2"},
    {'P',   0,  53,  96, "SC-8850 CYMBAL&CLAPS Set", "TR-707 Hand Clap"},
    {'P',   0,  54,  36, "SC-8850 GAMELAN 1 Set", "Kendang Wadon"},
    {'P',   0,  54,  37, "SC-8850 GAMELAN 1 Set", "Kendang Lanang"},
    {'P',   0,  54,  38, "SC-8850 GAMELAN 1 Set", "Bebarongan"},
    {'P',   0,  54,  39, "SC-8850 GAMELAN 1 Set", "Pelegongan"},
    {'P',   0,  54,  40, "SC-8850 GAMELAN 1 Set", "Kelontuk"},
    {'P',   0,  54,  41, "SC-8850 GAMELAN 1 Set", "Kelontuk Mute"},
    {'P',   0,  54,  42, "SC-8850 GAMELAN 1 Set", "Kelontuk Side"},
    {'P',   0,  54,  43, "SC-8850 GAMELAN 1 Set", "Gamelan Gong Wadon"},
    {'P',   0,  54,  44, "SC-8850 GAMELAN 1 Set", "Gamelan Gong Lanang"},
    {'P',   0,  54,  45, "SC-8850 GAMELAN 1 Set", "Ceng-Ceng"},
    {'P',   0,  54,  46, "SC-8850 GAMELAN 1 Set", "Kopyak Open"},
    {'P',   0,  54,  47, "SC-8850 GAMELAN 1 Set", "Kopyak Mute"},
    {'P',   0,  54,  48, "SC-8850 GAMELAN 1 Set", "Kajar"},
    {'P',   0,  54,  49, "SC-8850 GAMELAN 1 Set", "Kempur"},
    {'P',   0,  54,  50, "SC-8850 GAMELAN 1 Set", "Jegogan"},
    {'P',   0,  54,  51, "SC-8850 GAMELAN 1 Set", "Jegogan"},
    {'P',   0,  54,  52, "SC-8850 GAMELAN 1 Set", "Jegogan"},
    {'P',   0,  54,  53, "SC-8850 GAMELAN 1 Set", "Jegogan"},
    {'P',   0,  54,  54, "SC-8850 GAMELAN 1 Set", "Jegogan"},
    {'P',   0,  54,  55, "SC-8850 GAMELAN 1 Set", "Jublag"},
    {'P',   0,  54,  56, "SC-8850 GAMELAN 1 Set", "Jublag"},
    {'P',   0,  54,  57, "SC-8850 GAMELAN 1 Set", "Jublag"},
    {'P',   0,  54,  58, "SC-8850 GAMELAN 1 Set", "Jublag"},
    {'P',   0,  54,  59, "SC-8850 GAMELAN 1 Set", "Jublag"},
    {'P',   0,  54,  60, "SC-8850 GAMELAN 1 Set", "Penyacah"},
    {'P',   0,  54,  61, "SC-8850 GAMELAN 1 Set", "Penyacah"},
    {'P',   0,  54,  62, "SC-8850 GAMELAN 1 Set", "Penyacah"},
    {'P',   0,  54,  63, "SC-8850 GAMELAN 1 Set", "Penyacah"},
    {'P',   0,  54,  64, "SC-8850 GAMELAN 1 Set", "Penyacah"},
    {'P',   0,  54,  65, "SC-8850 GAMELAN 1 Set", "Penyacah"},
    {'P',   0,  54,  66, "SC-8850 GAMELAN 1 Set", "Penyacah"},
    {'P',   0,  54,  67, "SC-8850 GAMELAN 1 Set", "Pemade"},
    {'P',   0,  54,  68, "SC-8850 GAMELAN 1 Set", "Pemade"},
    {'P',   0,  54,  69, "SC-8850 GAMELAN 1 Set", "Pemade"},
    {'P',   0,  54,  70, "SC-8850 GAMELAN 1 Set", "Pemade"},
    {'P',   0,  54,  71, "SC-8850 GAMELAN 1 Set", "Pemade"},
    {'P',   0,  54,  72, "SC-8850 GAMELAN 1 Set", "Pemade"},
    {'P',   0,  54,  73, "SC-8850 GAMELAN 1 Set", "Pemade"},
    {'P',   0,  54,  74, "SC-8850 GAMELAN 1 Set", "Pemade"},
    {'P',   0,  54,  75, "SC-8850 GAMELAN 1 Set", "Pemade"},
    {'P',   0,  54,  76, "SC-8850 GAMELAN 1 Set", "Pemade"},
    {'P',   0,  54,  77, "SC-8850 GAMELAN 1 Set", "Reyong"},
    {'P',   0,  54,  78, "SC-8850 GAMELAN 1 Set", "Reyong"},
    {'P',   0,  54,  79, "SC-8850 GAMELAN 1 Set", "Reyong"},
    {'P',   0,  54,  80, "SC-8850 GAMELAN 1 Set", "Reyong"},
    {'P',   0,  54,  81, "SC-8850 GAMELAN 1 Set", "Reyong"},
    {'P',   0,  54,  82, "SC-8850 GAMELAN 1 Set", "Reyong"},
    {'P',   0,  54,  83, "SC-8850 GAMELAN 1 Set", "Reyong"},
    {'P',   0,  54,  84, "SC-8850 GAMELAN 1 Set", "Reyong"},
    {'P',   0,  54,  85, "SC-8850 GAMELAN 1 Set", "Reyong"},
    {'P',   0,  54,  86, "SC-8850 GAMELAN 1 Set", "Reyong"},
    {'P',   0,  54,  87, "SC-8850 GAMELAN 1 Set", "Reyong"},
    {'P',   0,  54,  88, "SC-8850 GAMELAN 1 Set", "Reyong"},
    {'P',   0,  56,  21, "SC-8850 SFX Set", "MC-500 Beep 1"},
    {'P',   0,  56,  22, "SC-8850 SFX Set", "MC-500 Beep 2"},
    {'P',   0,  56,  23, "SC-8850 SFX Set", "Guitar Slide"},
    {'P',   0,  56,  24, "SC-8850 SFX Set", "Guitar Wah"},
    {'P',   0,  56,  25, "SC-8850 SFX Set", "Guitar Slap"},
    {'P',   0,  56,  26, "SC-8850 SFX Set", "Chord Stroke Down"},
    {'P',   0,  56,  27, "SC-8850 SFX Set", "Chord Stroke Up"},
    {'P',   0,  56,  28, "SC-8850 SFX Set", "Biwa FX"},
    {'P',   0,  56,  29, "SC-8850 SFX Set", "Phonograph Noise"},
    {'P',   0,  56,  30, "SC-8850 SFX Set", "Tape Rewind"},
    {'P',   0,  56,  31, "SC-8850 SFX Set", "Scratch Push 2"},
    {'P',   0,  56,  32, "SC-8850 SFX Set", "Scratch Pull 2"},
    {'P',   0,  56,  33, "SC-8850 SFX Set", "Cutting Noise 2 Up"},
    {'P',   0,  56,  34, "SC-8850 SFX Set", "Cutting Noise 2 Down"},
    {'P',   0,  56,  35, "SC-8850 SFX Set", "Distortion Guitar Cutting Noise Up"},
    {'P',   0,  56,  36, "SC-8850 SFX Set", "Distortion Guitar Cutting Noise Down"},
    {'P',   0,  56,  37, "SC-8850 SFX Set", "Bass Slide"},
    {'P',   0,  56,  38, "SC-8850 SFX Set", "Pick Scrape"},
    {'P',   0,  56,  39, "SC-8850 SFX Set", "High Q"},
    {'P',   0,  56,  40, "SC-8850 SFX Set", "Slap"},
    {'P',   0,  56,  41, "SC-8850 SFX Set", "Scratch Push"},
    {'P',   0,  56,  42, "SC-8850 SFX Set", "Scratch Pull"},
    {'P',   0,  56,  43, "SC-8850 SFX Set", "Sticks"},
    {'P',   0,  56,  44, "SC-8850 SFX Set", "Square Click"},
    {'P',   0,  56,  45, "SC-8850 SFX Set", "Metronome Click"},
    {'P',   0,  56,  46, "SC-8850 SFX Set", "Metronome Bell"},
    {'P',   0,  56,  47, "SC-8850 SFX Set", "Guitar Fret Noise"},
    {'P',   0,  56,  48, "SC-8850 SFX Set", "Guitar Cutting Noise Up"},
    {'P',   0,  56,  49, "SC-8850 SFX Set", "Guitar Cutting Noise Down"},
    {'P',   0,  56,  50, "SC-8850 SFX Set", "String Slap of Double Bass"},
    {'P',   0,  56,  51, "SC-8850 SFX Set", "Flute Key Click Noise"},
    {'P',   0,  56,  52, "SC-8850 SFX Set", "Laughing"},
    {'P',   0,  56,  53, "SC-8850 SFX Set", "Screaming"},
    {'P',   0,  56,  54, "SC-8850 SFX Set", "Punch"},
    {'P',   0,  56,  55, "SC-8850 SFX Set", "Heart Beat"},
    {'P',   0,  56,  56, "SC-8850 SFX Set", "Footsteps 1"},
    {'P',   0,  56,  57, "SC-8850 SFX Set", "Footsteps 2"},
    {'P',   0,  56,  58, "SC-8850 SFX Set", "Applause"},
    {'P',   0,  56,  59, "SC-8850 SFX Set", "Door Creaking"},
    {'P',   0,  56,  60, "SC-8850 SFX Set", "Door"},
    {'P',   0,  56,  61, "SC-8850 SFX Set", "Scratch"},
    {'P',   0,  56,  62, "SC-8850 SFX Set", "Wind Chimes"},
    {'P',   0,  56,  63, "SC-8850 SFX Set", "Car - Engine"},
    {'P',   0,  56,  64, "SC-8850 SFX Set", "Car - Stop"},
    {'P',   0,  56,  65, "SC-8850 SFX Set", "Car - Passing"},
    {'P',   0,  56,  66, "SC-8850 SFX Set", "Car - Crash"},
    {'P',   0,  56,  67, "SC-8850 SFX Set", "Siren"},
    {'P',   0,  56,  68, "SC-8850 SFX Set", "Train"},
    {'P',   0,  56,  69, "SC-8850 SFX Set", "Jetplane"},
    {'P',   0,  56,  70, "SC-8850 SFX Set", "Helicopter"},
    {'P',   0,  56,  71, "SC-8850 SFX Set", "Starship"},
    {'P',   0,  56,  72, "SC-8850 SFX Set", "Gun Shot"},
    {'P',   0,  56,  73, "SC-8850 SFX Set", "Machine Gun"},
    {'P',   0,  56,  74, "SC-8850 SFX Set", "Laser Gun"},
    {'P',   0,  56,  75, "SC-8850 SFX Set", "Explosion"},
    {'P',   0,  56,  76, "SC-8850 SFX Set", "Dog"},
    {'P',   0,  56,  77, "SC-8850 SFX Set", "Horse-Gallop"},
    {'P',   0,  56,  78, "SC-8850 SFX Set", "Birds"},
    {'P',   0,  56,  79, "SC-8850 SFX Set", "Rain"},
    {'P',   0,  56,  80, "SC-8850 SFX Set", "Thunder"},
    {'P',   0,  56,  81, "SC-8850 SFX Set", "Wind"},
    {'P',   0,  56,  82, "SC-8850 SFX Set", "Seashore"},
    {'P',   0,  56,  83, "SC-8850 SFX Set", "Stream"},
    {'P',   0,  56,  84, "SC-8850 SFX Set", "Bubble"},
    {'P',   0,  56,  85, "SC-8850 SFX Set", "Kitty"},
    {'P',   0,  56,  86, "SC-8850 SFX Set", "Bird 2"},
    {'P',   0,  56,  87, "SC-8850 SFX Set", "Growl"},
    {'P',   0,  56,  88, "SC-8850 SFX Set", "Applause 2"},
    {'P',   0,  56,  89, "SC-8850 SFX Set", "Telephone 1"},
    {'P',   0,  56,  90, "SC-8850 SFX Set", "Telephone 2"},
    {'P',   0,  56,  91, "SC-8850 SFX Set", "Small Club 1"},
    {'P',   0,  56,  92, "SC-8850 SFX Set", "Small Club 2"},
    {'P',   0,  56,  93, "SC-8850 SFX Set", "Applause Wave"},
    {'P',   0,  56,  94, "SC-8850 SFX Set", "Eruption"},
    {'P',   0,  56,  95, "SC-8850 SFX Set", "Big Shot"},
    {'P',   0,  56,  96, "SC-8850 SFX Set", "Percussion Bang"},
    {'P',   0,  57,  36, "SC-8850 RHYTHM FX Set", "Reverse Kick 1"},
    {'P',   0,  57,  37, "SC-8850 RHYTHM FX Set", "Reverse Concert Bass Drum"},
    {'P',   0,  57,  38, "SC-8850 RHYTHM FX Set", "Reverse Power Kick1"},
    {'P',   0,  57,  39, "SC-8850 RHYTHM FX Set", "Reverse Electric Kick 1"},
    {'P',   0,  57,  40, "SC-8850 RHYTHM FX Set", "Reverse Snare 1"},
    {'P',   0,  57,  41, "SC-8850 RHYTHM FX Set", "Reverse Snare 2"},
    {'P',   0,  57,  42, "SC-8850 RHYTHM FX Set", "Reverse Standard 1 Snare 1"},
    {'P',   0,  57,  43, "SC-8850 RHYTHM FX Set", "Reverse Tight Snare"},
    {'P',   0,  57,  44, "SC-8850 RHYTHM FX Set", "Reverse Dance Snare"},
    {'P',   0,  57,  45, "SC-8850 RHYTHM FX Set", "Reverse 808 Snare"},
    {'P',   0,  57,  46, "SC-8850 RHYTHM FX Set", "Reverse Tom 1"},
    {'P',   0,  57,  47, "SC-8850 RHYTHM FX Set", "Reverse Tom 2"},
    {'P',   0,  57,  48, "SC-8850 RHYTHM FX Set", "Reverse Sticks"},
    {'P',   0,  57,  49, "SC-8850 RHYTHM FX Set", "Reverse Slap"},
    {'P',   0,  57,  50, "SC-8850 RHYTHM FX Set", "Reverse Cymbal 1"},
    {'P',   0,  57,  51, "SC-8850 RHYTHM FX Set", "Reverse Cymbal 2"},
    {'P',   0,  57,  52, "SC-8850 RHYTHM FX Set", "Reverse Open Hi-Hat"},
    {'P',   0,  57,  53, "SC-8850 RHYTHM FX Set", "Reverse Ride Cymbal"},
    {'P',   0,  57,  54, "SC-8850 RHYTHM FX Set", "Reverse CR-78 Open Hi-Hat"},
    {'P',   0,  57,  55, "SC-8850 RHYTHM FX Set", "Reverse Closed Hi-Hat"},
    {'P',   0,  57,  56, "SC-8850 RHYTHM FX Set", "Reverse Gong"},
    {'P',   0,  57,  57, "SC-8850 RHYTHM FX Set", "Reverse Bell Tree"},
    {'P',   0,  57,  58, "SC-8850 RHYTHM FX Set", "Reverse Guiro"},
    {'P',   0,  57,  59, "SC-8850 RHYTHM FX Set", "Reverse Bendir"},
    {'P',   0,  57,  60, "SC-8850 RHYTHM FX Set", "Reverse Gun Shot"},
    {'P',   0,  57,  61, "SC-8850 RHYTHM FX Set", "Reverse Scratch"},
    {'P',   0,  57,  62, "SC-8850 RHYTHM FX Set", "Reverse Laser Gun"},
    {'P',   0,  57,  63, "SC-8850 RHYTHM FX Set", "Key Click"},
    {'P',   0,  57,  64, "SC-8850 RHYTHM FX Set", "Techno Thip"},
    {'P',   0,  57,  65, "SC-8850 RHYTHM FX Set", "Pop Drop"},
    {'P',   0,  57,  66, "SC-8850 RHYTHM FX Set", "Woody Slap"},
    {'P',   0,  57,  67, "SC-8850 RHYTHM FX Set", "Distortion Kick"},
    {'P',   0,  57,  68, "SC-8850 RHYTHM FX Set", "Syn. Drops"},
    {'P',   0,  57,  69, "SC-8850 RHYTHM FX Set", "Reverse Hi Q"},
    {'P',   0,  57,  70, "SC-8850 RHYTHM FX Set", "Pipe"},
    {'P',   0,  57,  71, "SC-8850 RHYTHM FX Set", "Ice Block"},
    {'P',   0,  57,  72, "SC-8850 RHYTHM FX Set", "Digital Tambourine"},
    {'P',   0,  57,  73, "SC-8850 RHYTHM FX Set", "Alias"},
    {'P',   0,  57,  74, "SC-8850 RHYTHM FX Set", "Modulated Bell"},
    {'P',   0,  57,  75, "SC-8850 RHYTHM FX Set", "Spark"},
    {'P',   0,  57,  76, "SC-8850 RHYTHM FX Set", "Metallic Percussion"},
    {'P',   0,  57,  77, "SC-8850 RHYTHM FX Set", "Velocity Noise FX"},
    {'P',   0,  57,  78, "SC-8850 RHYTHM FX Set", "Stereo Noise Clap"},
    {'P',   0,  57,  79, "SC-8850 RHYTHM FX Set", "Swish"},
    {'P',   0,  57,  80, "SC-8850 RHYTHM FX Set", "Slappy"},
    {'P',   0,  57,  81, "SC-8850 RHYTHM FX Set", "Voice Ou"},
    {'P',   0,  57,  82, "SC-8850 RHYTHM FX Set", "Voice Au"},
    {'P',   0,  57,  83, "SC-8850 RHYTHM FX Set", "Hoo"},
    {'P',   0,  57,  84, "SC-8850 RHYTHM FX Set", "Tape Stop 1"},
    {'P',   0,  57,  85, "SC-8850 RHYTHM FX Set", "Tape Stop 2"},
    {'P',   0,  57,  86, "SC-8850 RHYTHM FX Set", "Missile"},
    {'P',   0,  57,  87, "SC-8850 RHYTHM FX Set", "Space Birds"},
    {'P',   0,  57,  88, "SC-8850 RHYTHM FX Set", "Flying Monster"},
    {'P',   0,  58,  36, "SC-8850 RHYTHM FX 2 Set", "Reverse TR-707 Kick 1"},
    {'P',   0,  58,  37, "SC-8850 RHYTHM FX 2 Set", "Reverse TR-909 Kick 1"},
    {'P',   0,  58,  38, "SC-8850 RHYTHM FX 2 Set", "Reverse Hip-Hop Kick 1"},
    {'P',   0,  58,  39, "SC-8850 RHYTHM FX 2 Set", "Reverse Jungle Kick 2"},
    {'P',   0,  58,  40, "SC-8850 RHYTHM FX 2 Set", "Reverse Techno Kick 2"},
    {'P',   0,  58,  41, "SC-8850 RHYTHM FX 2 Set", "Reverse TR-606 Snare 2"},
    {'P',   0,  58,  42, "SC-8850 RHYTHM FX 2 Set", "Reverse CR-78 Snare 1"},
    {'P',   0,  58,  43, "SC-8850 RHYTHM FX 2 Set", "Reverse CR-78 Snare 2"},
    {'P',   0,  58,  44, "SC-8850 RHYTHM FX 2 Set", "Reverse Jungle Snare 2"},
    {'P',   0,  58,  45, "SC-8850 RHYTHM FX 2 Set", "Reverse Techno Snare 2"},
    {'P',   0,  58,  46, "SC-8850 RHYTHM FX 2 Set", "Reverse TR-707 Snare"},
    {'P',   0,  58,  47, "SC-8850 RHYTHM FX 2 Set", "Reverse TR-606 Snare 1"},
    {'P',   0,  58,  48, "SC-8850 RHYTHM FX 2 Set", "Reverse TR-909 Snare 1"},
    {'P',   0,  58,  49, "SC-8850 RHYTHM FX 2 Set", "Reverse Hip-Hop Snare 2"},
    {'P',   0,  58,  50, "SC-8850 RHYTHM FX 2 Set", "Reverse Jungle Snare 1"},
    {'P',   0,  58,  51, "SC-8850 RHYTHM FX 2 Set", "Reverse House Snare"},
    {'P',   0,  58,  52, "SC-8850 RHYTHM FX 2 Set", "Reverse Closed Hi-Hat"},
    {'P',   0,  58,  53, "SC-8850 RHYTHM FX 2 Set", "Reverse TR-606 Closed Hi-Hat"},
    {'P',   0,  58,  54, "SC-8850 RHYTHM FX 2 Set", "Reverse TR-707 Closed Hi-Hat"},
    {'P',   0,  58,  55, "SC-8850 RHYTHM FX 2 Set", "Reverse TR-808 Closed Hi-Hat"},
    {'P',   0,  58,  56, "SC-8850 RHYTHM FX 2 Set", "Reverse Jungle Hi-Hat"},
    {'P',   0,  58,  57, "SC-8850 RHYTHM FX 2 Set", "Reverse Tambourine 2"},
    {'P',   0,  58,  58, "SC-8850 RHYTHM FX 2 Set", "Reverse Shake Tambourine"},
    {'P',   0,  58,  59, "SC-8850 RHYTHM FX 2 Set", "Reverse TR-808 Open Hi-Hat"},
    {'P',   0,  58,  60, "SC-8850 RHYTHM FX 2 Set", "Reverse TR-707 Open Hi-Hat"},
    {'P',   0,  58,  61, "SC-8850 RHYTHM FX 2 Set", "Reverse Open Hi-Hat"},
    {'P',   0,  58,  62, "SC-8850 RHYTHM FX 2 Set", "Reverse TR-606 Open Hi-Hat"},
    {'P',   0,  58,  63, "SC-8850 RHYTHM FX 2 Set", "Reverse  Hu Yin Luo"},
    {'P',   0,  58,  64, "SC-8850 RHYTHM FX 2 Set", "Reverse TR-707 Crash Cymbal"},
    {'P',   0,  58,  65, "SC-8850 RHYTHM FX 2 Set", "Voice One"},
    {'P',   0,  58,  66, "SC-8850 RHYTHM FX 2 Set", "Reverse Voice One"},
    {'P',   0,  58,  67, "SC-8850 RHYTHM FX 2 Set", "Voice Two"},
    {'P',   0,  58,  68, "SC-8850 RHYTHM FX 2 Set", "Reverse Voice Two"},
    {'P',   0,  58,  69, "SC-8850 RHYTHM FX 2 Set", "Voice Three"},
    {'P',   0,  58,  70, "SC-8850 RHYTHM FX 2 Set", "Reverse Voice Three"},
    {'P',   0,  58,  71, "SC-8850 RHYTHM FX 2 Set", "Voice Tah"},
    {'P',   0,  58,  72, "SC-8850 RHYTHM FX 2 Set", "Reverse Voice Tah"},
    {'P',   0,  58,  73, "SC-8850 RHYTHM FX 2 Set", "Voice Ou"},
    {'P',   0,  58,  74, "SC-8850 RHYTHM FX 2 Set", "Voice Au"},
    {'P',   0,  58,  75, "SC-8850 RHYTHM FX 2 Set", "Voice Whey"},
    {'P',   0,  58,  76, "SC-8850 RHYTHM FX 2 Set", "Frog Vpoce"},
    {'P',   0,  58,  77, "SC-8850 RHYTHM FX 2 Set", "Reverse Yyoo Dude"},
    {'P',   0,  58,  78, "SC-8850 RHYTHM FX 2 Set", "Douby"},
    {'P',   0,  58,  79, "SC-8850 RHYTHM FX 2 Set", "Reverse Douby"},
    {'P',   0,  58,  80, "SC-8850 RHYTHM FX 2 Set", "Baert High"},
    {'P',   0,  58,  81, "SC-8850 RHYTHM FX 2 Set", "Baert Low"},
    {'P',   0,  58,  82, "SC-8850 RHYTHM FX 2 Set", "Bounce"},
    {'P',   0,  58,  83, "SC-8850 RHYTHM FX 2 Set", "Reverse bounce"},
    {'P',   0,  58,  84, "SC-8850 RHYTHM FX 2 Set", "Distortion Knock"},
    {'P',   0,  58,  85, "SC-8850 RHYTHM FX 2 Set", "Guitar Slide"},
    {'P',   0,  58,  86, "SC-8850 RHYTHM FX 2 Set", "Sub Marine"},
    {'P',   0,  58,  87, "SC-8850 RHYTHM FX 2 Set", "Noise Attack"},
    {'P',   0,  58,  88, "SC-8850 RHYTHM FX 2 Set", "Space Worms"},
    {'P',   0,  58,  89, "SC-8850 RHYTHM FX 2 Set", "Emergency !"},
    {'P',   0,  58,  90, "SC-8850 RHYTHM FX 2 Set", "Calculating..."},
    {'P',   0,  58,  91, "SC-8850 RHYTHM FX 2 Set", "Saw LFO Saw"},
    {'P',   0,  59,  17, "SC-8850 RHYTHM FX 3 Set", "Reverse Bass Mute Noise"},
    {'P',   0,  59,  18, "SC-8850 RHYTHM FX 3 Set", "Reverse Bass Touch Noise"},
    {'P',   0,  59,  19, "SC-8850 RHYTHM FX 3 Set", "Reverse Bass Attack Noise"},
    {'P',   0,  59,  20, "SC-8850 RHYTHM FX 3 Set", "Reverse Clean Guitar Cut Noise Up"},
    {'P',   0,  59,  21, "SC-8850 RHYTHM FX 3 Set", "Reverse Clean Guitar Cut Noise Down"},
    {'P',   0,  59,  22, "SC-8850 RHYTHM FX 3 Set", "Reverse Clean Guitar Mute Up"},
    {'P',   0,  59,  23, "SC-8850 RHYTHM FX 3 Set", "Reverse Clean Guitar Mute Down"},
    {'P',   0,  59,  24, "SC-8850 RHYTHM FX 3 Set", "Reverse Distortion Guitar Cut Noise Up"},
    {'P',   0,  59,  25, "SC-8850 RHYTHM FX 3 Set", "Reverse Distortion Guitar Cut Noise Down"},
    {'P',   0,  59,  26, "SC-8850 RHYTHM FX 3 Set", "Reverse Distortion Guitar Stroke Noise"},
    {'P',   0,  59,  27, "SC-8850 RHYTHM FX 3 Set", "Reverse Distortion Guitar Mute Noise"},
    {'P',   0,  59,  28, "SC-8850 RHYTHM FX 3 Set", "Reverse Steel Guitar Slide Noise 1"},
    {'P',   0,  59,  29, "SC-8850 RHYTHM FX 3 Set", "Reverse Steel Guitar Slide Noise 2"},
    {'P',   0,  59,  30, "SC-8850 RHYTHM FX 3 Set", "Reverse Steel Guitar Slide Noise 3"},
    {'P',   0,  59,  31, "SC-8850 RHYTHM FX 3 Set", "Reverse Steel Guitar Slide Noise 4"},
    {'P',   0,  59,  32, "SC-8850 RHYTHM FX 3 Set", "Reverse Steel Guitar Stroke Noise"},
    {'P',   0,  59,  33, "SC-8850 RHYTHM FX 3 Set", "Reverse Steel Guitar Stroke Noise Up 1"},
    {'P',   0,  59,  34, "SC-8850 RHYTHM FX 3 Set", "Reverse Steel Guitar Stroke Noise Down 1"},
    {'P',   0,  59,  35, "SC-8850 RHYTHM FX 3 Set", "Reverse Steel Guitar Stroke Noise Up 2"},
    {'P',   0,  59,  36, "SC-8850 RHYTHM FX 3 Set", "Reverse Steel Guitar Stroke Noise Down 2"},
    {'P',   0,  59,  37, "SC-8850 RHYTHM FX 3 Set", "Reverse Trombone Noise"},
    {'P',   0,  59,  38, "SC-8850 RHYTHM FX 3 Set", "Reverse Trumpet Noise"},
    {'P',   0,  59,  39, "SC-8850 RHYTHM FX 3 Set", "Reverse Standard Kick 2"},
    {'P',   0,  59,  40, "SC-8850 RHYTHM FX 3 Set", "Reverse Standard Kick 1"},
    {'P',   0,  59,  41, "SC-8850 RHYTHM FX 3 Set", "Reverse Room Kick 2"},
    {'P',   0,  59,  42, "SC-8850 RHYTHM FX 3 Set", "Reverse Room Kick 1"},
    {'P',   0,  59,  43, "SC-8850 RHYTHM FX 3 Set", "Reverse Jazz Kick 2"},
    {'P',   0,  59,  44, "SC-8850 RHYTHM FX 3 Set", "Reverse Jazz Kick 1"},
    {'P',   0,  59,  45, "SC-8850 RHYTHM FX 3 Set", "Reverse Brush Kick 2"},
    {'P',   0,  59,  46, "SC-8850 RHYTHM FX 3 Set", "Reverse Brush Kick 1"},
    {'P',   0,  59,  47, "SC-8850 RHYTHM FX 3 Set", "Reverse HipHop Kick 2"},
    {'P',   0,  59,  48, "SC-8850 RHYTHM FX 3 Set", "Reverse HipHop Kick 1"},
    {'P',   0,  59,  49, "SC-8850 RHYTHM FX 3 Set", "Reverse Jungle Kick 2"},
    {'P',   0,  59,  50, "SC-8850 RHYTHM FX 3 Set", "Reverse Jungle Kick 1"},
    {'P',   0,  59,  51, "SC-8850 RHYTHM FX 3 Set", "Reverse TR-808 Kick"},
    {'P',   0,  59,  52, "SC-8850 RHYTHM FX 3 Set", "Reverse TR-909 Kick 2"},
    {'P',   0,  59,  53, "SC-8850 RHYTHM FX 3 Set", "Reverse TR-909 Kick 1"},
    {'P',   0,  59,  54, "SC-8850 RHYTHM FX 3 Set", "Reverse Fat Kick"},
    {'P',   0,  59,  55, "SC-8850 RHYTHM FX 3 Set", "Reverse Dance Kick"},
    {'P',   0,  59,  56, "SC-8850 RHYTHM FX 3 Set", "Reverse Standard Snare 1"},
    {'P',   0,  59,  57, "SC-8850 RHYTHM FX 3 Set", "Reverse Standard Snare 2"},
    {'P',   0,  59,  58, "SC-8850 RHYTHM FX 3 Set", "Reverse Room Snare 1"},
    {'P',   0,  59,  59, "SC-8850 RHYTHM FX 3 Set", "Reverse Room Snare 2"},
    {'P',   0,  59,  60, "SC-8850 RHYTHM FX 3 Set", "Reverse Jazz Snare 1"},
    {'P',   0,  59,  61, "SC-8850 RHYTHM FX 3 Set", "Reverse Jazz Snare 2"},
    {'P',   0,  59,  62, "SC-8850 RHYTHM FX 3 Set", "Reverse Brush Snare 1"},
    {'P',   0,  59,  63, "SC-8850 RHYTHM FX 3 Set", "Reverse Brush Snare 2"},
    {'P',   0,  59,  64, "SC-8850 RHYTHM FX 3 Set", "Reverse Lo-Fi Snare 1"},
    {'P',   0,  59,  65, "SC-8850 RHYTHM FX 3 Set", "Reverse Lo-Fi Snare 2"},
    {'P',   0,  59,  66, "SC-8850 RHYTHM FX 3 Set", "Reverse HipHop Snare 1"},
    {'P',   0,  59,  67, "SC-8850 RHYTHM FX 3 Set", "Reverse HipHop Snare 2"},
    {'P',   0,  59,  68, "SC-8850 RHYTHM FX 3 Set", "Reverse House Snare 1"},
    {'P',   0,  59,  69, "SC-8850 RHYTHM FX 3 Set", "Reverse Jungle Snare"},
    {'P',   0,  59,  70, "SC-8850 RHYTHM FX 3 Set", "Reverse 606 Snare 2"},
    {'P',   0,  59,  71, "SC-8850 RHYTHM FX 3 Set", "Reverse Techno Snare"},
    {'P',   0,  59,  72, "SC-8850 RHYTHM FX 3 Set", "Reverse Dance Snare"},
    {'P',   0,  59,  73, "SC-8850 RHYTHM FX 3 Set", "Reverse Rock Snare Dry"},
    {'P',   0,  59,  74, "SC-8850 RHYTHM FX 3 Set", "Reverse Lo-Fi Snare Rim"},
    {'P',   0,  59,  75, "SC-8850 RHYTHM FX 3 Set", "Reverse 909 Snare Rim"},
    {'P',   0,  59,  76, "SC-8850 RHYTHM FX 3 Set", "Reverse Jungle Snare Rim"},
    {'P',   0,  59,  77, "SC-8850 RHYTHM FX 3 Set", "Reverse Dance Snare Rim"},
    {'P',   0,  59,  78, "SC-8850 RHYTHM FX 3 Set", "Reverse House Snare Rim"},
    {'P',   0,  59,  79, "SC-8850 RHYTHM FX 3 Set", "Reverse Brush Tom 1"},
    {'P',   0,  59,  80, "SC-8850 RHYTHM FX 3 Set", "Reverse Brush Tom 2"},
    {'P',   0,  59,  81, "SC-8850 RHYTHM FX 3 Set", "Reverse Brush Tom 3"},
    {'P',   0,  59,  82, "SC-8850 RHYTHM FX 3 Set", "Reverse 606 Tom"},
    {'P',   0,  59,  83, "SC-8850 RHYTHM FX 3 Set", "Reverse Jungle Crash Cymbal"},
    {'P',   0,  59,  84, "SC-8850 RHYTHM FX 3 Set", "Reverse Standard Closed Hi-Hat"},
    {'P',   0,  59,  85, "SC-8850 RHYTHM FX 3 Set", "Reverse Room Closed Hi-Hat"},
    {'P',   0,  59,  86, "SC-8850 RHYTHM FX 3 Set", "Reverse Jazz Closed Hi-Hat"},
    {'P',   0,  59,  87, "SC-8850 RHYTHM FX 3 Set", "Reverse Brush Closed Hi-Hat"},
    {'P',   0,  59,  88, "SC-8850 RHYTHM FX 3 Set", "Reverse 707 Claps"},
    {'P',   0,  59,  89, "SC-8850 RHYTHM FX 3 Set", "Reverse 909 Claps"},
    {'P',   0,  59,  90, "SC-8850 RHYTHM FX 3 Set", "Reverse R&B Claps 1"},
    {'P',   0,  59,  91, "SC-8850 RHYTHM FX 3 Set", "Reverse HipHop Claps"},
    {'P',   0,  59,  92, "SC-8850 RHYTHM FX 3 Set", "Reverse Comp Claps 2"},
    {'P',   0,  59,  93, "SC-8850 RHYTHM FX 3 Set", "Reverse Shaker 2"},
    {'P',   0,  59,  94, "SC-8850 RHYTHM FX 3 Set", "Reverse Jungle Shaker"},
    {'P',   0,  59,  95, "SC-8850 RHYTHM FX 3 Set", "Reverse Clap Hit"},
    {'P',   0,  59,  96, "SC-8850 RHYTHM FX 3 Set", "Reverse Boeeeen"},
    {'P',   0,  59,  97, "SC-8850 RHYTHM FX 3 Set", "Reverse Bounce"},
    {'P',   0,  59,  98, "SC-8850 RHYTHM FX 3 Set", "Reverse CD Tray"},
    {'P',   0,  59,  99, "SC-8850 RHYTHM FX 3 Set", "Reverse Drill"},
    {'P',   0,  59, 100, "SC-8850 RHYTHM FX 3 Set", "Reverse Glass Stir"},
    {'P',   0,  59, 101, "SC-8850 RHYTHM FX 3 Set", "Reverse Ice Ring"},
    {'P',   0,  59, 102, "SC-8850 RHYTHM FX 3 Set", "Reverse Industry Hit"},
    {'P',   0,  59, 103, "SC-8850 RHYTHM FX 3 Set", "Reverse Scratch 4"},
    {'P',   0,  59, 104, "SC-8850 RHYTHM FX 3 Set", "Reverse Scratch 5"},
    {'P',   0,  59, 105, "SC-8850 RHYTHM FX 3 Set", "Reverse Scratch 6"},
    {'P',   0,  59, 106, "SC-8850 RHYTHM FX 3 Set", "Reverse Scratch 7"},
    {'P',   0,  59, 107, "SC-8850 RHYTHM FX 3 Set", "Reverse Seal"},
    {'P',   0,  59, 108, "SC-8850 RHYTHM FX 3 Set", "Reverse Stab! 1"},
    {'P',   0,  59, 109, "SC-8850 RHYTHM FX 3 Set", "Reverse Stab! 2"},
    {'P',   0,  59, 110, "SC-8850 RHYTHM FX 3 Set", "Reverse Sword Boom"},
    {'P',   0,  59, 111, "SC-8850 RHYTHM FX 3 Set", "Reverse Sword Cross"},
    {'P',   0,  59, 112, "SC-8850 RHYTHM FX 3 Set", "Reverse Thrill Hit"},
    {'P',   0,  59, 113, "SC-8850 RHYTHM FX 3 Set", "Reverse Audio Switch"},
    {'P',   0,  59, 114, "SC-8850 RHYTHM FX 3 Set", "Reverse Keyboard Typing 1"},
    {'P',   0,  59, 115, "SC-8850 RHYTHM FX 3 Set", "Reverse Keyboard Typing 2"},
    {'P',   0,  59, 116, "SC-8850 RHYTHM FX 3 Set", "Reverse Keyboard Typing 3"},
    {'P',   0,  59, 117, "SC-8850 RHYTHM FX 3 Set", "Reverse Keyboard Typing 4"},
    {'P',   0,  59, 118, "SC-8850 RHYTHM FX 3 Set", "Reverse Keyboard Typing 5"},
    {'P',   0,  59, 119, "SC-8850 RHYTHM FX 3 Set", "Reverse Keyboard Typing 6"},
    {'P',   0,  60,  31, "SC-8850 SFX 2 Set", "Acoustic Bass Mute Noise"},
    {'P',   0,  60,  32, "SC-8850 SFX 2 Set", "Acoustic Bass Touch Noise"},
    {'P',   0,  60,  33, "SC-8850 SFX 2 Set", "Acoustic Bass Attack Noise"},
    {'P',   0,  60,  34, "SC-8850 SFX 2 Set", "Distortion Guitar Mute Noise"},
    {'P',   0,  60,  35, "SC-8850 SFX 2 Set", "Steel Guitar Slide Noise 1"},
    {'P',   0,  60,  36, "SC-8850 SFX 2 Set", "Steel Guitar Slide Noise 2"},
    {'P',   0,  60,  37, "SC-8850 SFX 2 Set", "Steel Guitar Slide Noise 3"},
    {'P',   0,  60,  38, "SC-8850 SFX 2 Set", "Steel Guitar Slide Noise 4"},
    {'P',   0,  60,  39, "SC-8850 SFX 2 Set", "Guitar Stroke Noise 1"},
    {'P',   0,  60,  40, "SC-8850 SFX 2 Set", "Guitar Stroke Noise 2"},
    {'P',   0,  60,  41, "SC-8850 SFX 2 Set", "Guitar Stroke Noise 3"},
    {'P',   0,  60,  42, "SC-8850 SFX 2 Set", "Guitar Stroke Noise 4"},
    {'P',   0,  60,  43, "SC-8850 SFX 2 Set", "Guitar Stroke Noise 5"},
    {'P',   0,  60,  44, "SC-8850 SFX 2 Set", "Open CD Tray"},
    {'P',   0,  60,  45, "SC-8850 SFX 2 Set", "Audio Switch"},
    {'P',   0,  60,  46, "SC-8850 SFX 2 Set", "Keyboard Typing 1"},
    {'P',   0,  60,  47, "SC-8850 SFX 2 Set", "Keyboard Typing 2"},
    {'P',   0,  60,  48, "SC-8850 SFX 2 Set", "Keyboard Typing 3"},
    {'P',   0,  60,  49, "SC-8850 SFX 2 Set", "Keyboard Typing 4"},
    {'P',   0,  60,  50, "SC-8850 SFX 2 Set", "Keyboard Typing 5"},
    {'P',   0,  60,  51, "SC-8850 SFX 2 Set", "Keyboard Typing 6"},
    {'P',   0,  60,  52, "SC-8850 SFX 2 Set", "Baby Laughing"},
    {'P',   0,  60,  53, "SC-8850 SFX 2 Set", "Clap Hit"},
    {'P',   0,  60,  54, "SC-8850 SFX 2 Set", "Stab! 1"},
    {'P',   0,  60,  55, "SC-8850 SFX 2 Set", "Stab! 2"},
    {'P',   0,  60,  56, "SC-8850 SFX 2 Set", "Bounce Hit"},
    {'P',   0,  60,  57, "SC-8850 SFX 2 Set", "Boeeeen"},
    {'P',   0,  60,  58, "SC-8850 SFX 2 Set", "Glass Stir"},
    {'P',   0,  60,  59, "SC-8850 SFX 2 Set", "Ice Ring"},
    {'P',   0,  60,  60, "SC-8850 SFX 2 Set", "Crack Bottle"},
    {'P',   0,  60,  61, "SC-8850 SFX 2 Set", "Pour Bottle"},
    {'P',   0,  60,  62, "SC-8850 SFX 2 Set", "Soda"},
    {'P',   0,  60,  63, "SC-8850 SFX 2 Set", "Car Engine 2"},
    {'P',   0,  60,  64, "SC-8850 SFX 2 Set", "Car - Horn"},
    {'P',   0,  60,  65, "SC-8850 SFX 2 Set", "Railroad Crossing"},
    {'P',   0,  60,  66, "SC-8850 SFX 2 Set", "SL 1"},
    {'P',   0,  60,  67, "SC-8850 SFX 2 Set", "SL 2"},
    {'P',   0,  60,  68, "SC-8850 SFX 2 Set", "Over Blow"},
    {'P',   0,  60,  69, "SC-8850 SFX 2 Set", "Sword Boom!"},
    {'P',   0,  60,  70, "SC-8850 SFX 2 Set", "Sword Cross"},
    {'P',   0,  60,  71, "SC-8850 SFX 2 Set", "Industry Hit"},
    {'P',   0,  60,  72, "SC-8850 SFX 2 Set", "Drill Hit"},
    {'P',   0,  60,  73, "SC-8850 SFX 2 Set", "Compresser"},
    {'P',   0,  60,  74, "SC-8850 SFX 2 Set", "Thrill Hit"},
    {'P',   0,  60,  75, "SC-8850 SFX 2 Set", "Explosion 2"},
    {'P',   0,  60,  76, "SC-8850 SFX 2 Set", "Seal"},
    {'P',   0,  60,  77, "SC-8850 SFX 2 Set", "Fancy Animal"},
    {'P',   0,  60,  78, "SC-8850 SFX 2 Set", "Cricket"},
    {'P',   0,  60,  79, "SC-8850 SFX 2 Set", "Bear"},
    {'P',   0,  60,  80, "SC-8850 SFX 2 Set", "Frog Vpoce"},
    {'P',   0,  60,  81, "SC-8850 SFX 2 Set", "Wind 2"},
    {'P',   0,  60,  82, "SC-8850 SFX 2 Set", "Scratch 3"},
    {'P',   0,  60,  83, "SC-8850 SFX 2 Set", "Scratch 4"},
    {'P',   0,  60,  84, "SC-8850 SFX 2 Set", "Scratch 5"},
    {'P',   0,  60,  85, "SC-8850 SFX 2 Set", "Scratch 6"},
    {'P',   0,  60,  86, "SC-8850 SFX 2 Set", "Scratch 7"},
    {'P',   0,  60,  87, "SC-8850 SFX 2 Set", "Noise Attack"},
    {'P',   0,  60,  88, "SC-8850 SFX 2 Set", "Bounce"},
    {'P',   0,  60,  89, "SC-8850 SFX 2 Set", "Dist Knock"},
    {'P',   0,  60,  90, "SC-8850 SFX 2 Set", "Bound"},
    {'P',   0,  61,  36, "SC-8850 VOICE Set", "Reverse Breath Slow"},
    {'P',   0,  61,  37, "SC-8850 VOICE Set", "Reverse Breath Short"},
    {'P',   0,  61,  38, "SC-8850 VOICE Set", "Reverse Breath Strong"},
    {'P',   0,  61,  39, "SC-8850 VOICE Set", "Reverse Woman's Breath"},
    {'P',   0,  61,  40, "SC-8850 VOICE Set", "Reverse Man's Breath"},
    {'P',   0,  61,  41, "SC-8850 VOICE Set", "Reverse Voice One"},
    {'P',   0,  61,  42, "SC-8850 VOICE Set", "Reverse Voice Two"},
    {'P',   0,  61,  43, "SC-8850 VOICE Set", "Reverse Voice Three"},
    {'P',   0,  61,  44, "SC-8850 VOICE Set", "Reverse Voice Tha"},
    {'P',   0,  61,  45, "SC-8850 VOICE Set", "Reverse Voice Come On"},
    {'P',   0,  61,  46, "SC-8850 VOICE Set", "Reverse Voice Kikit"},
    {'P',   0,  61,  47, "SC-8850 VOICE Set", "Reverse Voice Aou"},
    {'P',   0,  61,  48, "SC-8850 VOICE Set", "Reverse Voice Ooh"},
    {'P',   0,  61,  49, "SC-8850 VOICE Set", "Reverse Voice Hie 2"},
    {'P',   0,  61,  50, "SC-8850 VOICE Set", "Reverse Baby Laughing"},
    {'P',   0,  61,  51, "SC-8850 VOICE Set", "Reverse Yyooh"},
    {'P',   0,  61,  52, "SC-8850 VOICE Set", "Reverse Japanese Female Voice Lan"},
    {'P',   0,  61,  53, "SC-8850 VOICE Set", "Reverse Ooue!"},
    {'P',   0,  61,  54, "SC-8850 VOICE Set", "Flute Breath 1"},
    {'P',   0,  61,  55, "SC-8850 VOICE Set", "Flute Breath 2"},
    {'P',   0,  61,  56, "SC-8850 VOICE Set", "Flute Breath 3"},
    {'P',   0,  61,  57, "SC-8850 VOICE Set", "Voice Breath 1"},
    {'P',   0,  61,  58, "SC-8850 VOICE Set", "Voice Breath 2"},
    {'P',   0,  61,  59, "SC-8850 VOICE Set", "Voice One"},
    {'P',   0,  61,  60, "SC-8850 VOICE Set", "Voice Two"},
    {'P',   0,  61,  61, "SC-8850 VOICE Set", "Voice Three"},
    {'P',   0,  61,  62, "SC-8850 VOICE Set", "Voice Tah"},
    {'P',   0,  61,  63, "SC-8850 VOICE Set", "Voice ComeOn"},
    {'P',   0,  61,  64, "SC-8850 VOICE Set", "Voice Kikit"},
    {'P',   0,  61,  65, "SC-8850 VOICE Set", "Voice Aou"},
    {'P',   0,  61,  66, "SC-8850 VOICE Set", "Voice Oou"},
    {'P',   0,  61,  67, "SC-8850 VOICE Set", "Punch"},
    {'P',   0,  61,  68, "SC-8850 VOICE Set", "Screaming"},
    {'P',   0,  61,  69, "SC-8850 VOICE Set", "Laughing"},
    {'P',   0,  61,  70, "SC-8850 VOICE Set", "Voice Hie"},
    {'P',   0,  61,  71, "SC-8850 VOICE Set", "Baby Laughing"},
    {'P',   0,  61,  72, "SC-8850 VOICE Set", "Frog Vpoce"},
    {'P',   0,  61,  73, "SC-8850 VOICE Set", "Yyooh Dude"},
    {'P',   0,  61,  74, "SC-8850 VOICE Set", "Voice Ou"},
    {'P',   0,  61,  75, "SC-8850 VOICE Set", "Voice Au"},
    {'P',   0,  61,  76, "SC-8850 VOICE Set", "Jazz Voice Thum"},
    {'P',   0,  61,  77, "SC-8850 VOICE Set", "Jazz Voice Bap"},
    {'P',   0,  61,  78, "SC-8850 VOICE Set", "Jazz Voice Dat"},
    {'P',   0,  61,  79, "SC-8850 VOICE Set", "Jazz Voice Dow"},
    {'P',   0,  61,  80, "SC-8850 VOICE Set", "Voice Oohs 2"},
    {'P',   0,  61,  81, "SC-8850 VOICE Set", "Voice Oohs Chord Maj7 A"},
    {'P',   0,  61,  82, "SC-8850 VOICE Set", "Voice Oohs Chord Maj7 B"},
    {'P',   0,  61,  83, "SC-8850 VOICE Set", "Voice Oohs Chord Sus4 A"},
    {'P',   0,  61,  84, "SC-8850 VOICE Set", "Voice Oohs Chord Sus4 B"},
    {'P',   0,  61,  85, "SC-8850 VOICE Set", "Japanese Female Voice Lah"},
    {'P',   0,  61,  86, "SC-8850 VOICE Set", "Japanese Female Voice Lan"},
    {'P',   0,  61,  87, "SC-8850 VOICE Set", "Japanese Male Voice Wah"},
    {'P',   0,  61,  88, "SC-8850 VOICE Set", "Japanese Male Voice Woh"},
    {'P',   0,  62,  32, "SC-8850 CYM&CLAPS 2 Set", "Reverse Standard Closed Hi-Hat"},
    {'P',   0,  62,  33, "SC-8850 CYM&CLAPS 2 Set", "Reverse Room Closed Hi-Hat"},
    {'P',   0,  62,  34, "SC-8850 CYM&CLAPS 2 Set", "Reverse Jazz Closed Hi-Hat"},
    {'P',   0,  62,  35, "SC-8850 CYM&CLAPS 2 Set", "Reverse Brush Closed Hi-Hat"},
    {'P',   0,  62,  36, "SC-8850 CYM&CLAPS 2 Set", "Standard 1 Closed Hi-Hat"},
    {'P',   0,  62,  37, "SC-8850 CYM&CLAPS 2 Set", "Room Closed Hi-Hat"},
    {'P',   0,  62,  38, "SC-8850 CYM&CLAPS 2 Set", "Jazz Closed Hi-Hat"},
    {'P',   0,  62,  39, "SC-8850 CYM&CLAPS 2 Set", "Brush Closed Hi-Hat"},
    {'P',   0,  62,  40, "SC-8850 CYM&CLAPS 2 Set", "TR-707 Closed Hi-Hat"},
    {'P',   0,  62,  41, "SC-8850 CYM&CLAPS 2 Set", "TR-606 Closed Hi-Hat"},
    {'P',   0,  62,  42, "SC-8850 CYM&CLAPS 2 Set", "TR-808 Closed Hi-Hat"},
    {'P',   0,  62,  43, "SC-8850 CYM&CLAPS 2 Set", "CR-78 Closed Hi-Hat"},
    {'P',   0,  62,  44, "SC-8850 CYM&CLAPS 2 Set", "Pedal Hi-Hat"},
    {'P',   0,  62,  45, "SC-8850 CYM&CLAPS 2 Set", "Pedal Hi-Hat"},
    {'P',   0,  62,  46, "SC-8850 CYM&CLAPS 2 Set", "Pedal Hi-Hat"},
    {'P',   0,  62,  47, "SC-8850 CYM&CLAPS 2 Set", "Half-Open Hi-Hat 1"},
    {'P',   0,  62,  48, "SC-8850 CYM&CLAPS 2 Set", "Half-Open Hi-Hat 2"},
    {'P',   0,  62,  49, "SC-8850 CYM&CLAPS 2 Set", "Standard 1 Open Hi-Hat"},
    {'P',   0,  62,  50, "SC-8850 CYM&CLAPS 2 Set", "Room Open Hi-Hat"},
    {'P',   0,  62,  51, "SC-8850 CYM&CLAPS 2 Set", "Jazz Open Hi-Hat"},
    {'P',   0,  62,  52, "SC-8850 CYM&CLAPS 2 Set", "Brush Open Hi-Hat"},
    {'P',   0,  62,  53, "SC-8850 CYM&CLAPS 2 Set", "TR-909 Open Hi-Hat"},
    {'P',   0,  62,  54, "SC-8850 CYM&CLAPS 2 Set", "TR-707 Open Hi-Hat"},
    {'P',   0,  62,  55, "SC-8850 CYM&CLAPS 2 Set", "TR-606 Open Hi-Hat"},
    {'P',   0,  62,  56, "SC-8850 CYM&CLAPS 2 Set", "TR-808 Open Hi-Hat"},
    {'P',   0,  62,  57, "SC-8850 CYM&CLAPS 2 Set", "CR-78 Open Hi-Hat"},
    {'P',   0,  62,  58, "SC-8850 CYM&CLAPS 2 Set", "Standard 1 Crash Cymbal"},
    {'P',   0,  62,  59, "SC-8850 CYM&CLAPS 2 Set", "Room Crash Cymbal"},
    {'P',   0,  62,  60, "SC-8850 CYM&CLAPS 2 Set", "Jazz Crash Cymbal"},
    {'P',   0,  62,  61, "SC-8850 CYM&CLAPS 2 Set", "Brush Crash Cymbal"},
    {'P',   0,  62,  62, "SC-8850 CYM&CLAPS 2 Set", "Hard Crash Cymbal"},
    {'P',   0,  62,  63, "SC-8850 CYM&CLAPS 2 Set", "TR-909 Crash Cymbal"},
    {'P',   0,  62,  64, "SC-8850 CYM&CLAPS 2 Set", "Jungle Crash Cymbal"},
    {'P',   0,  62,  65, "SC-8850 CYM&CLAPS 2 Set", "TR-808 Crash Cymbal"},
    {'P',   0,  62,  66, "SC-8850 CYM&CLAPS 2 Set", "Standard 1 Mute Crash Cymbal"},
    {'P',   0,  62,  67, "SC-8850 CYM&CLAPS 2 Set", "Room Mute Crash Cymbal"},
    {'P',   0,  62,  68, "SC-8850 CYM&CLAPS 2 Set", "Jazz Mute Crash Cymbal"},
    {'P',   0,  62,  69, "SC-8850 CYM&CLAPS 2 Set", "Brush Mute Crash Cymbal"},
    {'P',   0,  62,  70, "SC-8850 CYM&CLAPS 2 Set", "Mute Crash Cymbal 1"},
    {'P',   0,  62,  71, "SC-8850 CYM&CLAPS 2 Set", "Mute Crash Cymbal 2"},
    {'P',   0,  62,  72, "SC-8850 CYM&CLAPS 2 Set", "Reverse Standard 1 Crash Cymbal"},
    {'P',   0,  62,  73, "SC-8850 CYM&CLAPS 2 Set", "Reverse Room Crash Cymbal"},
    {'P',   0,  62,  74, "SC-8850 CYM&CLAPS 2 Set", "Reverse Jazz Crash Cymbal"},
    {'P',   0,  62,  75, "SC-8850 CYM&CLAPS 2 Set", "Reverse Brush Crash Cymbal"},
    {'P',   0,  62,  76, "SC-8850 CYM&CLAPS 2 Set", "Splash Cymbal"},
    {'P',   0,  62,  77, "SC-8850 CYM&CLAPS 2 Set", "Standard Ride Bell"},
    {'P',   0,  62,  78, "SC-8850 CYM&CLAPS 2 Set", "Room Ride Bell"},
    {'P',   0,  62,  79, "SC-8850 CYM&CLAPS 2 Set", "Jazz Ride Bell"},
    {'P',   0,  62,  80, "SC-8850 CYM&CLAPS 2 Set", "Brush Ride Bell"},
    {'P',   0,  62,  81, "SC-8850 CYM&CLAPS 2 Set", "Standard Ride Cymbal"},
    {'P',   0,  62,  82, "SC-8850 CYM&CLAPS 2 Set", "Room Ride Cymbal"},
    {'P',   0,  62,  83, "SC-8850 CYM&CLAPS 2 Set", "Jazz Ride Cymbal"},
    {'P',   0,  62,  84, "SC-8850 CYM&CLAPS 2 Set", "Brush Ride Cymbal"},
    {'P',   0,  62,  85, "SC-8850 CYM&CLAPS 2 Set", "TR-606 Ride Cymbal"},
    {'P',   0,  62,  86, "SC-8850 CYM&CLAPS 2 Set", "TR-808 Ride Cymbal"},
    {'P',   0,  62,  87, "SC-8850 CYM&CLAPS 2 Set", "Chinese Cymbal"},
    {'P',   0,  62,  88, "SC-8850 CYM&CLAPS 2 Set", "Chinese Cymbal 2"},
    {'P',   0,  62,  89, "SC-8850 CYM&CLAPS 2 Set", "TR-707 Claps"},
    {'P',   0,  62,  90, "SC-8850 CYM&CLAPS 2 Set", "Hip-Hop Claps"},
    {'P',   0,  62,  91, "SC-8850 CYM&CLAPS 2 Set", "R&B Claps"},
    {'P',   0,  62,  92, "SC-8850 CYM&CLAPS 2 Set", "TR-909 Claps"},
    {'P',   0,  62,  93, "SC-8850 CYM&CLAPS 2 Set", "Comp Claps 2"},
    {'P',   0,  62,  94, "SC-8850 CYM&CLAPS 2 Set", "Hand Clap"},
    {'P',   0,  62,  95, "SC-8850 CYM&CLAPS 2 Set", "[Pro] Hand Clap 2"},
    {'P',   0,  62,  96, "SC-8850 CYM&CLAPS 2 Set", "[Pro] TR-707 Hand Clap"},
    {'P',   1,   0,  27, "SC-55 STANDARD Set", "High Q"},
    {'P',   1,   0,  28, "SC-55 STANDARD Set", "Slap"},
    {'P',   1,   0,  29, "SC-55 STANDARD Set", "Scratch Push"},
    {'P',   1,   0,  30, "SC-55 STANDARD Set", "Scratch Pull"},
    {'P',   1,   0,  31, "SC-55 STANDARD Set", "Sticks"},
    {'P',   1,   0,  32, "SC-55 STANDARD Set", "Square Click"},
    {'P',   1,   0,  33, "SC-55 STANDARD Set", "Metronome Click"},
    {'P',   1,   0,  34, "SC-55 STANDARD Set", "Metronome Bell"},
    {'P',   1,   0,  35, "SC-55 STANDARD Set", "Kick Drum 2"},
    {'P',   1,   0,  36, "SC-55 STANDARD Set", "Kick Drum 1"},
    {'P',   1,   0,  37, "SC-55 STANDARD Set", "Side Stick"},
    {'P',   1,   0,  38, "SC-55 STANDARD Set", "Snare Drum 1"},
    {'P',   1,   0,  39, "SC-55 STANDARD Set", "Hand Clap"},
    {'P',   1,   0,  40, "SC-55 STANDARD Set", "Snare Drum 2"},
    {'P',   1,   0,  41, "SC-55 STANDARD Set", "Low Tom 2"},
    {'P',   1,   0,  42, "SC-55 STANDARD Set", "Closed Hi-Hat"},
    {'P',   1,   0,  43, "SC-55 STANDARD Set", "Low Tom 1"},
    {'P',   1,   0,  44, "SC-55 STANDARD Set", "Pedal Hi-Hat"},
    {'P',   1,   0,  45, "SC-55 STANDARD Set", "Mid Tom 2"},
    {'P',   1,   0,  46, "SC-55 STANDARD Set", "Open Hi-Hat"},
    {'P',   1,   0,  47, "SC-55 STANDARD Set", "Mid Tom 1"},
    {'P',   1,   0,  48, "SC-55 STANDARD Set", "High Tom 2"},
    {'P',   1,   0,  49, "SC-55 STANDARD Set", "Crash Cymbal 1"},
    {'P',   1,   0,  50, "SC-55 STANDARD Set", "High Tom 1"},
    {'P',   1,   0,  51, "SC-55 STANDARD Set", "Ride Cymbal 1"},
    {'P',   1,   0,  52, "SC-55 STANDARD Set", "Chinese Cymbal"},
    {'P',   1,   0,  53, "SC-55 STANDARD Set", "Ride Bell"},
    {'P',   1,   0,  54, "SC-55 STANDARD Set", "Tambourine"},
    {'P',   1,   0,  55, "SC-55 STANDARD Set", "Splash Cymbal"},
    {'P',   1,   0,  56, "SC-55 STANDARD Set", "Cowbell"},
    {'P',   1,   0,  57, "SC-55 STANDARD Set", "Crash Cymbal 2"},
    {'P',   1,   0,  58, "SC-55 STANDARD Set", "Vibra-slap"},
    {'P',   1,   0,  59, "SC-55 STANDARD Set", "Ride Cymbal 2"},
    {'P',   1,   0,  60, "SC-55 STANDARD Set", "High Bongo"},
    {'P',   1,   0,  61, "SC-55 STANDARD Set", "Low Bongo"},
    {'P',   1,   0,  62, "SC-55 STANDARD Set", "Mute High Conga"},
    {'P',   1,   0,  63, "SC-55 STANDARD Set", "Open High Conga"},
    {'P',   1,   0,  64, "SC-55 STANDARD Set", "Low Conga"},
    {'P',   1,   0,  65, "SC-55 STANDARD Set", "High Timbale"},
    {'P',   1,   0,  66, "SC-55 STANDARD Set", "Low Timbale"},
    {'P',   1,   0,  67, "SC-55 STANDARD Set", "High Agogo"},
    {'P',   1,   0,  68, "SC-55 STANDARD Set", "Low Agogo"},
    {'P',   1,   0,  69, "SC-55 STANDARD Set", "Cabasa"},
    {'P',   1,   0,  70, "SC-55 STANDARD Set", "Maracas"},
    {'P',   1,   0,  71, "SC-55 STANDARD Set", "Short Hi Whistle"},
    {'P',   1,   0,  72, "SC-55 STANDARD Set", "Long Low Whistle"},
    {'P',   1,   0,  73, "SC-55 STANDARD Set", "Short Guiro"},
    {'P',   1,   0,  74, "SC-55 STANDARD Set", "Long Guiro"},
    {'P',   1,   0,  75, "SC-55 STANDARD Set", "Claves"},
    {'P',   1,   0,  76, "SC-55 STANDARD Set", "High Wood Block"},
    {'P',   1,   0,  77, "SC-55 STANDARD Set", "Low Wood Block"},
    {'P',   1,   0,  78, "SC-55 STANDARD Set", "Mute Cuica"},
    {'P',   1,   0,  79, "SC-55 STANDARD Set", "Open Cuica"},
    {'P',   1,   0,  80, "SC-55 STANDARD Set", "Mute Triangle"},
    {'P',   1,   0,  81, "SC-55 STANDARD Set", "Open Triangle"},
    {'P',   1,   0,  82, "SC-55 STANDARD Set", "Shaker"},
    {'P',   1,   0,  83, "SC-55 STANDARD Set", "Jingle Bell"},
    {'P',   1,   0,  84, "SC-55 STANDARD Set", "Belltree"},
    {'P',   1,   0,  85, "SC-55 STANDARD Set", "Castanets"},
    {'P',   1,   0,  86, "SC-55 STANDARD Set", "Mute Surdo"},
    {'P',   1,   0,  87, "SC-55 STANDARD Set", "Open Surdo"},
    {'P',   1,   8,  41, "SC-55 ROOM Set", "Room Low Tom 2"},
    {'P',   1,   8,  43, "SC-55 ROOM Set", "Room Low Tom 1"},
    {'P',   1,   8,  45, "SC-55 ROOM Set", "Room Mid Tom 2"},
    {'P',   1,   8,  47, "SC-55 ROOM Set", "Room Mid Tom 1"},
    {'P',   1,   8,  48, "SC-55 ROOM Set", "Room Hi Tom 2"},
    {'P',   1,   8,  50, "SC-55 ROOM Set", "Room Hi Tom 1"},
    {'P',   1,  16,  36, "SC-55 POWER Set", "MONDO Kick"},
    {'P',   1,  16,  38, "SC-55 POWER Set", "Gated SD"},
    {'P',   1,  16,  41, "SC-55 POWER Set", "Room Low Tom 2"},
    {'P',   1,  16,  43, "SC-55 POWER Set", "Room Low Tom 1"},
    {'P',   1,  16,  45, "SC-55 POWER Set", "Room Mid Tom 2"},
    {'P',   1,  16,  47, "SC-55 POWER Set", "Room Mid Tom 1"},
    {'P',   1,  16,  48, "SC-55 POWER Set", "Room Hi Tom 2"},
    {'P',   1,  16,  50, "SC-55 POWER Set", "Room Hi Tom 1"},
    {'P',   1,  24,  36, "SC-55 ELECTRONIC Set", "Elec BD"},
    {'P',   1,  24,  38, "SC-55 ELECTRONIC Set", "Elec SD"},
    {'P',   1,  24,  40, "SC-55 ELECTRONIC Set", "Gated SD"},
    {'P',   1,  24,  41, "SC-55 ELECTRONIC Set", "Elec Low Tom 2"},
    {'P',   1,  24,  43, "SC-55 ELECTRONIC Set", "Elec Low Tom 1"},
    {'P',   1,  24,  45, "SC-55 ELECTRONIC Set", "Elec Mid Tom 2"},
    {'P',   1,  24,  47, "SC-55 ELECTRONIC Set", "Elec Mid Tom 1"},
    {'P',   1,  24,  48, "SC-55 ELECTRONIC Set", "Elec Hi Tom 2"},
    {'P',   1,  24,  50, "SC-55 ELECTRONIC Set", "Elec Hi Tom 1"},
    {'P',   1,  24,  52, "SC-55 ELECTRONIC Set", "Reverse Cymbal"},
    {'P',   1,  25,  36, "SC-55 TR-808 Set", "808 Bass Drum"},
    {'P',   1,  25,  37, "SC-55 TR-808 Set", "808 Rim Shot"},
    {'P',   1,  25,  38, "SC-55 TR-808 Set", "808 Snare Drum"},
    {'P',   1,  25,  41, "SC-55 TR-808 Set", "808 Low Tom 2"},
    {'P',   1,  25,  42, "SC-55 TR-808 Set", "808 CHH"},
    {'P',   1,  25,  43, "SC-55 TR-808 Set", "808 Low Tom 1"},
    {'P',   1,  25,  44, "SC-55 TR-808 Set", "808 CHH"},
    {'P',   1,  25,  45, "SC-55 TR-808 Set", "808 Mid Tom 2"},
    {'P',   1,  25,  46, "SC-55 TR-808 Set", "808 OHH"},
    {'P',   1,  25,  47, "SC-55 TR-808 Set", "808 Mid Tom 1"},
    {'P',   1,  25,  48, "SC-55 TR-808 Set", "808 Hi Tom 2"},
    {'P',   1,  25,  49, "SC-55 TR-808 Set", "808 Cymbal"},
    {'P',   1,  25,  50, "SC-55 TR-808 Set", "808 Hi Tom 1"},
    {'P',   1,  25,  56, "SC-55 TR-808 Set", "808 Cowbell"},
    {'P',   1,  25,  62, "SC-55 TR-808 Set", "808 High Conga"},
    {'P',   1,  25,  63, "SC-55 TR-808 Set", "808 Mid Conga"},
    {'P',   1,  25,  64, "SC-55 TR-808 Set", "808 Low Conga"},
    {'P',   1,  25,  70, "SC-55 TR-808 Set", "808 Maracas"},
    {'P',   1,  25,  75, "SC-55 TR-808 Set", "808 Claves"},
    {'P',   1,  32,  35, "SC-55 JAZZ Set", "Jazz BD 2"},
    {'P',   1,  32,  36, "SC-55 JAZZ Set", "Jazz BD 1"},
    {'P',   1,  40,  35, "SC-55 BRUSH Set", "Jazz BD 2"},
    {'P',   1,  40,  36, "SC-55 BRUSH Set", "Jazz BD 1"},
    {'P',   1,  40,  38, "SC-55 BRUSH Set", "Brush Tap"},
    {'P',   1,  40,  39, "SC-55 BRUSH Set", "Brush Slap"},
    {'P',   1,  40,  40, "SC-55 BRUSH Set", "Brush Swirl"},
    {'P',   1,  48,  27, "SC-55 ORCHESTRA Set", "Closed Hi-Hat"},
    {'P',   1,  48,  28, "SC-55 ORCHESTRA Set", "Pedal Hi-Hat"},
    {'P',   1,  48,  29, "SC-55 ORCHESTRA Set", "Open Hi-Hat"},
    {'P',   1,  48,  30, "SC-55 ORCHESTRA Set", "Ride Cymbal"},
    {'P',   1,  48,  35, "SC-55 ORCHESTRA Set", "Concert BD 2"},
    {'P',   1,  48,  36, "SC-55 ORCHESTRA Set", "Concert BD 1"},
    {'P',   1,  48,  38, "SC-55 ORCHESTRA Set", "Concert SD"},
    {'P',   1,  48,  39, "SC-55 ORCHESTRA Set", "Castanets"},
    {'P',   1,  48,  40, "SC-55 ORCHESTRA Set", "Concert SD"},
    {'P',   1,  48,  41, "SC-55 ORCHESTRA Set", "Timpani F"},
    {'P',   1,  48,  42, "SC-55 ORCHESTRA Set", "Timpani F#"},
    {'P',   1,  48,  43, "SC-55 ORCHESTRA Set", "Timpani G"},
    {'P',   1,  48,  44, "SC-55 ORCHESTRA Set", "Timpani G#"},
    {'P',   1,  48,  45, "SC-55 ORCHESTRA Set", "Timpani A"},
    {'P',   1,  48,  46, "SC-55 ORCHESTRA Set", "Timpani A#"},
    {'P',   1,  48,  47, "SC-55 ORCHESTRA Set", "Timpani B"},
    {'P',   1,  48,  48, "SC-55 ORCHESTRA Set", "Timpani c"},
    {'P',   1,  48,  49, "SC-55 ORCHESTRA Set", "Timpani c#"},
    {'P',   1,  48,  50, "SC-55 ORCHESTRA Set", "Timpani d"},
    {'P',   1,  48,  51, "SC-55 ORCHESTRA Set", "Timpani d#"},
    {'P',   1,  48,  52, "SC-55 ORCHESTRA Set", "Timpani e"},
    {'P',   1,  48,  53, "SC-55 ORCHESTRA Set", "Timpani f"},
    {'P',   1,  48,  57, "SC-55 ORCHESTRA Set", "Concert Cymbal 2"},
    {'P',   1,  48,  59, "SC-55 ORCHESTRA Set", "Concert Cymbal 1"},
    {'P',   1,  48,  88, "SC-55 ORCHESTRA Set", "Applause"},
    {'P',   1,  56,  39, "SC-55 SFX Set", "High Q"},
    {'P',   1,  56,  40, "SC-55 SFX Set", "Slap"},
    {'P',   1,  56,  41, "SC-55 SFX Set", "Scratch Push"},
    {'P',   1,  56,  42, "SC-55 SFX Set", "Scratch Pull"},
    {'P',   1,  56,  43, "SC-55 SFX Set", "Sticks"},
    {'P',   1,  56,  44, "SC-55 SFX Set", "Square Click"},
    {'P',   1,  56,  45, "SC-55 SFX Set", "Metronome Click"},
    {'P',   1,  56,  46, "SC-55 SFX Set", "Metronome Bell"},
    {'P',   1,  56,  47, "SC-55 SFX Set", "Guitar Fret Noise"},
    {'P',   1,  56,  48, "SC-55 SFX Set", "Guitar cutting noise / up"},
    {'P',   1,  56,  49, "SC-55 SFX Set", "Guitar cutting noise /down"},
    {'P',   1,  56,  50, "SC-55 SFX Set", "String slap of double Bass"},
    {'P',   1,  56,  51, "SC-55 SFX Set", "Fl.Key click"},
    {'P',   1,  56,  52, "SC-55 SFX Set", "Laughing"},
    {'P',   1,  56,  53, "SC-55 SFX Set", "Screaming"},
    {'P',   1,  56,  54, "SC-55 SFX Set", "Punch"},
    {'P',   1,  56,  55, "SC-55 SFX Set", "Heart Beat"},
    {'P',   1,  56,  56, "SC-55 SFX Set", "Footsteps 1"},
    {'P',   1,  56,  57, "SC-55 SFX Set", "Footsteps 2"},
    {'P',   1,  56,  58, "SC-55 SFX Set", "Applause"},
    {'P',   1,  56,  59, "SC-55 SFX Set", "Door Creaking"},
    {'P',   1,  56,  60, "SC-55 SFX Set", "Door"},
    {'P',   1,  56,  61, "SC-55 SFX Set", "Scratch"},
    {'P',   1,  56,  62, "SC-55 SFX Set", "Windchimes"},
    {'P',   1,  56,  63, "SC-55 SFX Set", "Car-Engine"},
    {'P',   1,  56,  64, "SC-55 SFX Set", "Car-Stop"},
    {'P',   1,  56,  65, "SC-55 SFX Set", "Car-Pass"},
    {'P',   1,  56,  66, "SC-55 SFX Set", "Car-Crash"},
    {'P',   1,  56,  67, "SC-55 SFX Set", "Siren"},
    {'P',   1,  56,  68, "SC-55 SFX Set", "Train"},
    {'P',   1,  56,  69, "SC-55 SFX Set", "Jetplane"},
    {'P',   1,  56,  70, "SC-55 SFX Set", "Helicopter"},
    {'P',   1,  56,  71, "SC-55 SFX Set", "Starship"},
    {'P',   1,  56,  72, "SC-55 SFX Set", "Gun Shot"},
    {'P',   1,  56,  73, "SC-55 SFX Set", "Machine Gun"},
    {'P',   1,  56,  74, "SC-55 SFX Set", "Lasergun"},
    {'P',   1,  56,  75, "SC-55 SFX Set", "Explosion"},
    {'P',   1,  56,  76, "SC-55 SFX Set", "Dog"},
    {'P',   1,  56,  77, "SC-55 SFX Set", "Horse-Gallop"},
    {'P',   1,  56,  78, "SC-55 SFX Set", "Birds"},
    {'P',   1,  56,  79, "SC-55 SFX Set", "Rain"},
    {'P',   1,  56,  80, "SC-55 SFX Set", "Thunder"},
    {'P',   1,  56,  81, "SC-55 SFX Set", "Wind"},
    {'P',   1,  56,  82, "SC-55 SFX Set", "Seashore"},
    {'P',   1,  56,  83, "SC-55 SFX Set", "Stream"},
    {'P',   1,  56,  84, "SC-55 SFX Set", "Bubble"},
    {'P',   1, 127,  35, "SC-55 CM-64/32L Set", "CM Kick Bass Drum"},
    {'P',   1, 127,  36, "SC-55 CM-64/32L Set", "CM Kick Bass Drum"},
    {'P',   1, 127,  37, "SC-55 CM-64/32L Set", "CM Rim Shot"},
    {'P',   1, 127,  38, "SC-55 CM-64/32L Set", "CM Snare Drum"},
    {'P',   1, 127,  39, "SC-55 CM-64/32L Set", "CM Hand Clap"},
    {'P',   1, 127,  40, "SC-55 CM-64/32L Set", "CM Electronic Snare Drum"},
    {'P',   1, 127,  41, "SC-55 CM-64/32L Set", "CM Acoustic Low Tom"},
    {'P',   1, 127,  42, "SC-55 CM-64/32L Set", "CM Closed High Hat"},
    {'P',   1, 127,  43, "SC-55 CM-64/32L Set", "CM Acoustic Low Tom"},
    {'P',   1, 127,  44, "SC-55 CM-64/32L Set", "CM Open High Hat 2"},
    {'P',   1, 127,  45, "SC-55 CM-64/32L Set", "CM M.Acoustic Middle Tom"},
    {'P',   1, 127,  46, "SC-55 CM-64/32L Set", "CM Open High Hat 1"},
    {'P',   1, 127,  47, "SC-55 CM-64/32L Set", "CM M.TomAcoustic Middle Tom"},
    {'P',   1, 127,  48, "SC-55 CM-64/32L Set", "CM Acoustic High Tom"},
    {'P',   1, 127,  49, "SC-55 CM-64/32L Set", "CM Crash Cymbal"},
    {'P',   1, 127,  50, "SC-55 CM-64/32L Set", "CM Acoustic High Tom"},
    {'P',   1, 127,  51, "SC-55 CM-64/32L Set", "CM Ride Cymbal"},
    {'P',   1, 127,  54, "SC-55 CM-64/32L Set", "CM Tambourine"},
    {'P',   1, 127,  56, "SC-55 CM-64/32L Set", "CM Cowbell"},
    {'P',   1, 127,  60, "SC-55 CM-64/32L Set", "CM High Bongo"},
    {'P',   1, 127,  61, "SC-55 CM-64/32L Set", "CM Low Bongo"},
    {'P',   1, 127,  62, "SC-55 CM-64/32L Set", "CM Mute High Conga"},
    {'P',   1, 127,  63, "SC-55 CM-64/32L Set", "CM High Conga"},
    {'P',   1, 127,  64, "SC-55 CM-64/32L Set", "CM Low Conga"},
    {'P',   1, 127,  65, "SC-55 CM-64/32L Set", "CM High Timbale"},
    {'P',   1, 127,  66, "SC-55 CM-64/32L Set", "CM Low Timbale"},
    {'P',   1, 127,  67, "SC-55 CM-64/32L Set", "CM High Agogo"},
    {'P',   1, 127,  68, "SC-55 CM-64/32L Set", "CM Low Agogo"},
    {'P',   1, 127,  69, "SC-55 CM-64/32L Set", "CM Cabasa"},
    {'P',   1, 127,  70, "SC-55 CM-64/32L Set", "CM Maracas"},
    {'P',   1, 127,  71, "SC-55 CM-64/32L Set", "CM Short Whistle"},
    {'P',   1, 127,  72, "SC-55 CM-64/32L Set", "CM Long Whistle"},
    {'P',   1, 127,  73, "SC-55 CM-64/32L Set", "CM Vibrato Slap"},
    {'P',   1, 127,  75, "SC-55 CM-64/32L Set", "CM Claves"},
    {'P',   1, 127,  76, "SC-55 CM-64/32L Set", "Laughing"},
    {'P',   1, 127,  77, "SC-55 CM-64/32L Set", "Screaming"},
    {'P',   1, 127,  78, "SC-55 CM-64/32L Set", "Punch"},
    {'P',   1, 127,  79, "SC-55 CM-64/32L Set", "Heartbeat"},
    {'P',   1, 127,  80, "SC-55 CM-64/32L Set", "Footsteps 1"},
    {'P',   1, 127,  81, "SC-55 CM-64/32L Set", "Footsteps 2"},
    {'P',   1, 127,  82, "SC-55 CM-64/32L Set", "Applause"},
    {'P',   1, 127,  83, "SC-55 CM-64/32L Set", "Creaking"},
    {'P',   1, 127,  84, "SC-55 CM-64/32L Set", "Door"},
    {'P',   1, 127,  85, "SC-55 CM-64/32L Set", "Scratch"},
    {'P',   1, 127,  86, "SC-55 CM-64/32L Set", "Wind Chimes"},
    {'P',   1, 127,  87, "SC-55 CM-64/32L Set", "Car-Engine"},
    {'P',   1, 127,  88, "SC-55 CM-64/32L Set", "Car-Stop"},
    {'P',   1, 127,  89, "SC-55 CM-64/32L Set", "Car-Pass"},
    {'P',   1, 127,  90, "SC-55 CM-64/32L Set", "Car-Crash"},
    {'P',   1, 127,  91, "SC-55 CM-64/32L Set", "Siren"},
    {'P',   1, 127,  92, "SC-55 CM-64/32L Set", "Train"},
    {'P',   1, 127,  93, "SC-55 CM-64/32L Set", "Jetplane"},
    {'P',   1, 127,  94, "SC-55 CM-64/32L Set", "Helicopter"},
    {'P',   1, 127,  95, "SC-55 CM-64/32L Set", "Starship"},
    {'P',   1, 127,  96, "SC-55 CM-64/32L Set", "Gun Shot"},
    {'P',   1, 127,  97, "SC-55 CM-64/32L Set", "Machine Gun"},
    {'P',   1, 127,  98, "SC-55 CM-64/32L Set", "Lasergun"},
    {'P',   1, 127,  99, "SC-55 CM-64/32L Set", "Explosion"},
    {'P',   1, 127, 100, "SC-55 CM-64/32L Set", "Dog"},
    {'P',   1, 127, 101, "SC-55 CM-64/32L Set", "Horse-Gallop"},
    {'P',   1, 127, 102, "SC-55 CM-64/32L Set", "Birds"},
    {'P',   1, 127, 103, "SC-55 CM-64/32L Set", "Rain"},
    {'P',   1, 127, 104, "SC-55 CM-64/32L Set", "Thunder"},
    {'P',   1, 127, 105, "SC-55 CM-64/32L Set", "Wind"},
    {'P',   1, 127, 106, "SC-55 CM-64/32L Set", "Waves"},
    {'P',   1, 127, 107, "SC-55 CM-64/32L Set", "Stream"},
    {'P',   1, 127, 108, "SC-55 CM-64/32L Set", "Bubble"},
    {'P',   2,   0,  25, "SC-88 STANDARD 1 Set", "Snare Roll"},
    {'P',   2,   0,  26, "SC-88 STANDARD 1 Set", "Finger Snap"},
    {'P',   2,   0,  27, "SC-88 STANDARD 1 Set", "High Q"},
    {'P',   2,   0,  28, "SC-88 STANDARD 1 Set", "Slap"},
    {'P',   2,   0,  29, "SC-88 STANDARD 1 Set", "Scratch Push"},
    {'P',   2,   0,  30, "SC-88 STANDARD 1 Set", "Scratch Pull"},
    {'P',   2,   0,  31, "SC-88 STANDARD 1 Set", "Sticks"},
    {'P',   2,   0,  32, "SC-88 STANDARD 1 Set", "Square Click"},
    {'P',   2,   0,  33, "SC-88 STANDARD 1 Set", "Metronome Click"},
    {'P',   2,   0,  34, "SC-88 STANDARD 1 Set", "Metronome Bell"},
    {'P',   2,   0,  35, "SC-88 STANDARD 1 Set", "Standard 1 Kick 2"},
    {'P',   2,   0,  36, "SC-88 STANDARD 1 Set", "Standard 1 Kick 1"},
    {'P',   2,   0,  37, "SC-88 STANDARD 1 Set", "Side Stick"},
    {'P',   2,   0,  38, "SC-88 STANDARD 1 Set", "Standard 1 Snare 1"},
    {'P',   2,   0,  39, "SC-88 STANDARD 1 Set", "Hand Clap"},
    {'P',   2,   0,  40, "SC-88 STANDARD 1 Set", "Standard 1 Snare 2"},
    {'P',   2,   0,  41, "SC-88 STANDARD 1 Set", "Low Tom 2"},
    {'P',   2,   0,  42, "SC-88 STANDARD 1 Set", "Closed Hi-hat 1"},
    {'P',   2,   0,  43, "SC-88 STANDARD 1 Set", "Low Tom 1"},
    {'P',   2,   0,  44, "SC-88 STANDARD 1 Set", "Pedal Hi-hat"},
    {'P',   2,   0,  45, "SC-88 STANDARD 1 Set", "Mid Tom 2"},
    {'P',   2,   0,  46, "SC-88 STANDARD 1 Set", "Open Hi-hat"},
    {'P',   2,   0,  47, "SC-88 STANDARD 1 Set", "Mid Tom 1"},
    {'P',   2,   0,  48, "SC-88 STANDARD 1 Set", "High Tom 2"},
    {'P',   2,   0,  49, "SC-88 STANDARD 1 Set", "Crash Cymbal 1"},
    {'P',   2,   0,  50, "SC-88 STANDARD 1 Set", "High Tom 1"},
    {'P',   2,   0,  51, "SC-88 STANDARD 1 Set", "Ride Cymbal 1"},
    {'P',   2,   0,  52, "SC-88 STANDARD 1 Set", "Chinese Cymbal"},
    {'P',   2,   0,  53, "SC-88 STANDARD 1 Set", "Ride Bell"},
    {'P',   2,   0,  54, "SC-88 STANDARD 1 Set", "Tambourine"},
    {'P',   2,   0,  55, "SC-88 STANDARD 1 Set", "Splash Cymbal"},
    {'P',   2,   0,  56, "SC-88 STANDARD 1 Set", "Cowbell"},
    {'P',   2,   0,  57, "SC-88 STANDARD 1 Set", "Crash Cymbal 2"},
    {'P',   2,   0,  58, "SC-88 STANDARD 1 Set", "Vibra-slap"},
    {'P',   2,   0,  59, "SC-88 STANDARD 1 Set", "Ride Cymbal 2"},
    {'P',   2,   0,  60, "SC-88 STANDARD 1 Set", "High Bongo"},
    {'P',   2,   0,  61, "SC-88 STANDARD 1 Set", "Low Bongo"},
    {'P',   2,   0,  62, "SC-88 STANDARD 1 Set", "Mute High Conga"},
    {'P',   2,   0,  63, "SC-88 STANDARD 1 Set", "Open High Conga"},
    {'P',   2,   0,  64, "SC-88 STANDARD 1 Set", "Low Conga"},
    {'P',   2,   0,  65, "SC-88 STANDARD 1 Set", "High Timbale"},
    {'P',   2,   0,  66, "SC-88 STANDARD 1 Set", "Low Timbale"},
    {'P',   2,   0,  67, "SC-88 STANDARD 1 Set", "High Agogo"},
    {'P',   2,   0,  68, "SC-88 STANDARD 1 Set", "Low Agogo"},
    {'P',   2,   0,  69, "SC-88 STANDARD 1 Set", "Cabasa"},
    {'P',   2,   0,  70, "SC-88 STANDARD 1 Set", "Maracas"},
    {'P',   2,   0,  71, "SC-88 STANDARD 1 Set", "Short Hi Whistle"},
    {'P',   2,   0,  72, "SC-88 STANDARD 1 Set", "Long Low Whistle"},
    {'P',   2,   0,  73, "SC-88 STANDARD 1 Set", "Short Guiro"},
    {'P',   2,   0,  74, "SC-88 STANDARD 1 Set", "Long Guiro"},
    {'P',   2,   0,  75, "SC-88 STANDARD 1 Set", "Claves"},
    {'P',   2,   0,  76, "SC-88 STANDARD 1 Set", "High Wood Block"},
    {'P',   2,   0,  77, "SC-88 STANDARD 1 Set", "Low Wood Block"},
    {'P',   2,   0,  78, "SC-88 STANDARD 1 Set", "Mute Cuica"},
    {'P',   2,   0,  79, "SC-88 STANDARD 1 Set", "Open Cuica"},
    {'P',   2,   0,  80, "SC-88 STANDARD 1 Set", "Mute Triangle"},
    {'P',   2,   0,  81, "SC-88 STANDARD 1 Set", "Open Triangle"},
    {'P',   2,   0,  82, "SC-88 STANDARD 1 Set", "Shaker"},
    {'P',   2,   0,  83, "SC-88 STANDARD 1 Set", "Jingle Bell"},
    {'P',   2,   0,  84, "SC-88 STANDARD 1 Set", "Bell Tree"},
    {'P',   2,   0,  85, "SC-88 STANDARD 1 Set", "Castanets"},
    {'P',   2,   0,  86, "SC-88 STANDARD 1 Set", "Mute Surdo"},
    {'P',   2,   0,  87, "SC-88 STANDARD 1 Set", "Open Surdo"},
    {'P',   2,   1,  35, "SC-88 STANDARD 2 Set", "Standard 2 Kick 2"},
    {'P',   2,   1,  36, "SC-88 STANDARD 2 Set", "Standard 2 Kick 1"},
    {'P',   2,   1,  38, "SC-88 STANDARD 2 Set", "Standard 2 Snare 1"},
    {'P',   2,   1,  40, "SC-88 STANDARD 2 Set", "Standard 2 Snare 2"},
    {'P',   2,   1,  42, "SC-88 STANDARD 2 Set", "Closed Hi-hat 2"},
    {'P',   2,   1,  46, "SC-88 STANDARD 2 Set", "Open Hi-hat 2"},
    {'P',   2,   1,  84, "SC-88 STANDARD 2 Set", "Bar Chimes"},
    {'P',   2,   8,  35, "SC-88 ROOM Set", "Room Kick 2"},
    {'P',   2,   8,  36, "SC-88 ROOM Set", "Room Kick 1"},
    {'P',   2,   8,  38, "SC-88 ROOM Set", "Room Snare 1"},
    {'P',   2,   8,  40, "SC-88 ROOM Set", "Room Snare 2"},
    {'P',   2,   8,  41, "SC-88 ROOM Set", "Room Low Tom2"},
    {'P',   2,   8,  42, "SC-88 ROOM Set", "Closed Hi-hat 3"},
    {'P',   2,   8,  43, "SC-88 ROOM Set", "Room Low Tom 1"},
    {'P',   2,   8,  45, "SC-88 ROOM Set", "Room Mid Tom 2"},
    {'P',   2,   8,  46, "SC-88 ROOM Set", "Open Hi-Hat 3"},
    {'P',   2,   8,  47, "SC-88 ROOM Set", "Room Mid Tom 1"},
    {'P',   2,   8,  48, "SC-88 ROOM Set", "Room Hi Tom 2"},
    {'P',   2,   8,  50, "SC-88 ROOM Set", "Room Hi Tom 1"},
    {'P',   2,  16,  35, "SC-88 POWER Set", "Power Kick 2"},
    {'P',   2,  16,  36, "SC-88 POWER Set", "Power Kick 1"},
    {'P',   2,  16,  38, "SC-88 POWER Set", "Power Snare 1"},
    {'P',   2,  16,  40, "SC-88 POWER Set", "Power Snare 2"},
    {'P',   2,  16,  41, "SC-88 POWER Set", "Power Low Tom 2"},
    {'P',   2,  16,  42, "SC-88 POWER Set", "Closed Hi-hat 3"},
    {'P',   2,  16,  43, "SC-88 POWER Set", "Power Low Tom 1"},
    {'P',   2,  16,  45, "SC-88 POWER Set", "Power Mid Tom 2"},
    {'P',   2,  16,  46, "SC-88 POWER Set", "Open Hi-hat 3"},
    {'P',   2,  16,  47, "SC-88 POWER Set", "Power Mid Tom 1"},
    {'P',   2,  16,  48, "SC-88 POWER Set", "Power Hi Tom2"},
    {'P',   2,  16,  50, "SC-88 POWER Set", "Power Hi Tom 1"},
    {'P',   2,  24,  29, "SC-88 ELECTRONIC Set", "Scratch Push 2"},
    {'P',   2,  24,  30, "SC-88 ELECTRONIC Set", "Scratch Pull 2"},
    {'P',   2,  24,  35, "SC-88 ELECTRONIC Set", "Electric Kick 2"},
    {'P',   2,  24,  36, "SC-88 ELECTRONIC Set", "Electric Kick 1"},
    {'P',   2,  24,  38, "SC-88 ELECTRONIC Set", "Electric Snare 1"},
    {'P',   2,  24,  40, "SC-88 ELECTRONIC Set", "Electric Snare 2"},
    {'P',   2,  24,  41, "SC-88 ELECTRONIC Set", "Electric Low Tom 2"},
    {'P',   2,  24,  42, "SC-88 ELECTRONIC Set", "Closed Hi-hat 2"},
    {'P',   2,  24,  43, "SC-88 ELECTRONIC Set", "Electric Low Tom 1"},
    {'P',   2,  24,  45, "SC-88 ELECTRONIC Set", "Electric Mid Tom 2"},
    {'P',   2,  24,  46, "SC-88 ELECTRONIC Set", "Open Hi-hat 2"},
    {'P',   2,  24,  47, "SC-88 ELECTRONIC Set", "Electric Mid Tom 1"},
    {'P',   2,  24,  48, "SC-88 ELECTRONIC Set", "Electric Hi Tom 2"},
    {'P',   2,  24,  50, "SC-88 ELECTRONIC Set", "Electric Hi Tom 1"},
    {'P',   2,  24,  52, "SC-88 ELECTRONIC Set", "Reverse Cymbal"},
    {'P',   2,  25,  29, "SC-88 TR-808/909 Set", "Scratch Push 2"},
    {'P',   2,  25,  30, "SC-88 TR-808/909 Set", "Scratch Pull 2"},
    {'P',   2,  25,  35, "SC-88 TR-808/909 Set", "909 Bass Drum"},
    {'P',   2,  25,  36, "SC-88 TR-808/909 Set", "808 Bass Drum"},
    {'P',   2,  25,  37, "SC-88 TR-808/909 Set", "808 Rim Shot"},
    {'P',   2,  25,  38, "SC-88 TR-808/909 Set", "808 Snare 1"},
    {'P',   2,  25,  40, "SC-88 TR-808/909 Set", "909 Snare 1"},
    {'P',   2,  25,  41, "SC-88 TR-808/909 Set", "808 Low Tom 2"},
    {'P',   2,  25,  42, "SC-88 TR-808/909 Set", "808 CHH"},
    {'P',   2,  25,  43, "SC-88 TR-808/909 Set", "808 Low Tom 1"},
    {'P',   2,  25,  44, "SC-88 TR-808/909 Set", "808 CHH"},
    {'P',   2,  25,  45, "SC-88 TR-808/909 Set", "808 Mid Tom 2"},
    {'P',   2,  25,  46, "SC-88 TR-808/909 Set", "808 OHH"},
    {'P',   2,  25,  47, "SC-88 TR-808/909 Set", "808 Mid Tom 1"},
    {'P',   2,  25,  48, "SC-88 TR-808/909 Set", "808 Hi Tom 2"},
    {'P',   2,  25,  49, "SC-88 TR-808/909 Set", "808 Cymbal"},
    {'P',   2,  25,  50, "SC-88 TR-808/909 Set", "808 Hi Tom 1"},
    {'P',   2,  25,  56, "SC-88 TR-808/909 Set", "808 Cowbell"},
    {'P',   2,  25,  62, "SC-88 TR-808/909 Set", "808 High Conga"},
    {'P',   2,  25,  63, "SC-88 TR-808/909 Set", "808 Mid Conga"},
    {'P',   2,  25,  64, "SC-88 TR-808/909 Set", "808 Low Conga"},
    {'P',   2,  25,  70, "SC-88 TR-808/909 Set", "808 Maracas"},
    {'P',   2,  25,  75, "SC-88 TR-808/909 Set", "808 Claves"},
    {'P',   2,  26,  29, "SC-88 DANCE Set", "Scratch Push 2"},
    {'P',   2,  26,  30, "SC-88 DANCE Set", "Scratch Pull 2"},
    {'P',   2,  26,  35, "SC-88 DANCE Set", "Dance Kick"},
    {'P',   2,  26,  36, "SC-88 DANCE Set", "Electric Kick 2"},
    {'P',   2,  26,  38, "SC-88 DANCE Set", "Dance Snare 1"},
    {'P',   2,  26,  40, "SC-88 DANCE Set", "Dance Snare 2"},
    {'P',   2,  26,  41, "SC-88 DANCE Set", "Electric Low Tom 2"},
    {'P',   2,  26,  42, "SC-88 DANCE Set", "CR-78 CHH"},
    {'P',   2,  26,  43, "SC-88 DANCE Set", "Electric Low Tom 1"},
    {'P',   2,  26,  44, "SC-88 DANCE Set", "808 CHH"},
    {'P',   2,  26,  45, "SC-88 DANCE Set", "Electric Mid Tom 2"},
    {'P',   2,  26,  46, "SC-88 DANCE Set", "CR-78 OHH"},
    {'P',   2,  26,  47, "SC-88 DANCE Set", "Electric Mid Tom 1"},
    {'P',   2,  26,  48, "SC-88 DANCE Set", "Electric High Tom 2"},
    {'P',   2,  26,  50, "SC-88 DANCE Set", "Electric High Tom 1"},
    {'P',   2,  26,  52, "SC-88 DANCE Set", "Reverse Cymbal"},
    {'P',   2,  26,  78, "SC-88 DANCE Set", "High Hoo"},
    {'P',   2,  26,  79, "SC-88 DANCE Set", "Low Hoo"},
    {'P',   2,  26,  80, "SC-88 DANCE Set", "Electric Mute Triangle"},
    {'P',   2,  26,  81, "SC-88 DANCE Set", "Electric Open Triangle"},
    {'P',   2,  32,  35, "SC-88 JAZZ Set", "Jazz Kick 2"},
    {'P',   2,  32,  36, "SC-88 JAZZ Set", "Jazz Kick 1"},
    {'P',   2,  32,  38, "SC-88 JAZZ Set", "Jazz Snare 1"},
    {'P',   2,  32,  39, "SC-88 JAZZ Set", "Hand Clap2"},
    {'P',   2,  32,  40, "SC-88 JAZZ Set", "Jazz Snare 2"},
    {'P',   2,  32,  42, "SC-88 JAZZ Set", "Closed Hi-hat 2"},
    {'P',   2,  32,  46, "SC-88 JAZZ Set", "Open Hi-hat 2"},
    {'P',   2,  40,  35, "SC-88 BRUSH Set", "Jazz Kick 2"},
    {'P',   2,  40,  36, "SC-88 BRUSH Set", "Jazz Kick 1"},
    {'P',   2,  40,  38, "SC-88 BRUSH Set", "Brush Tap 1"},
    {'P',   2,  40,  39, "SC-88 BRUSH Set", "Brush Slap 1"},
    {'P',   2,  40,  40, "SC-88 BRUSH Set", "Brush Swirl 1"},
    {'P',   2,  40,  41, "SC-88 BRUSH Set", "Brush Low Tom 2"},
    {'P',   2,  40,  42, "SC-88 BRUSH Set", "Brush Closed Hi-hat"},
    {'P',   2,  40,  43, "SC-88 BRUSH Set", "Brush Low Tom 1"},
    {'P',   2,  40,  45, "SC-88 BRUSH Set", "Brush Mid Tom 2"},
    {'P',   2,  40,  46, "SC-88 BRUSH Set", "Brush Open Hi-hat"},
    {'P',   2,  40,  47, "SC-88 BRUSH Set", "Brush Mid Tom 1"},
    {'P',   2,  40,  48, "SC-88 BRUSH Set", "Brush Hi Tom 2"},
    {'P',   2,  40,  49, "SC-88 BRUSH Set", "Brush Crash Cymbal"},
    {'P',   2,  40,  50, "SC-88 BRUSH Set", "Brush Hi Tom 1"},
    {'P',   2,  40,  51, "SC-88 BRUSH Set", "Brush Ride Cymbal"},
    {'P',   2,  40,  53, "SC-88 BRUSH Set", "Brush Ride Bell"},
    {'P',   2,  48,  27, "SC-88 ORCHESTRA Set", "Closed Hi Hat 2"},
    {'P',   2,  48,  28, "SC-88 ORCHESTRA Set", "Pedal Hi Hat"},
    {'P',   2,  48,  29, "SC-88 ORCHESTRA Set", "Open Hi Hat 2"},
    {'P',   2,  48,  30, "SC-88 ORCHESTRA Set", "Ride Cymbal 1"},
    {'P',   2,  48,  35, "SC-88 ORCHESTRA Set", "Jazz Kick 1"},
    {'P',   2,  48,  36, "SC-88 ORCHESTRA Set", "Concert BD 1"},
    {'P',   2,  48,  38, "SC-88 ORCHESTRA Set", "Concert SD"},
    {'P',   2,  48,  39, "SC-88 ORCHESTRA Set", "Castanets"},
    {'P',   2,  48,  40, "SC-88 ORCHESTRA Set", "Concert SD"},
    {'P',   2,  48,  41, "SC-88 ORCHESTRA Set", "Timpani F"},
    {'P',   2,  48,  42, "SC-88 ORCHESTRA Set", "Timpani F#"},
    {'P',   2,  48,  43, "SC-88 ORCHESTRA Set", "Timpani G"},
    {'P',   2,  48,  44, "SC-88 ORCHESTRA Set", "Timpani G#"},
    {'P',   2,  48,  45, "SC-88 ORCHESTRA Set", "Timpani A"},
    {'P',   2,  48,  46, "SC-88 ORCHESTRA Set", "Timpani A#"},
    {'P',   2,  48,  47, "SC-88 ORCHESTRA Set", "Timpani B"},
    {'P',   2,  48,  48, "SC-88 ORCHESTRA Set", "Timpani c"},
    {'P',   2,  48,  49, "SC-88 ORCHESTRA Set", "Timpani c#"},
    {'P',   2,  48,  50, "SC-88 ORCHESTRA Set", "Timpani d"},
    {'P',   2,  48,  51, "SC-88 ORCHESTRA Set", "Timpani d#"},
    {'P',   2,  48,  52, "SC-88 ORCHESTRA Set", "Timpani e"},
    {'P',   2,  48,  53, "SC-88 ORCHESTRA Set", "Timpani f"},
    {'P',   2,  48,  57, "SC-88 ORCHESTRA Set", "Concert Cymbal 2"},
    {'P',   2,  48,  59, "SC-88 ORCHESTRA Set", "Concert Cymbal 1"},
    {'P',   2,  48,  88, "SC-88 ORCHESTRA Set", "Applause"},
    {'P',   2,  49,  25, "SC-88 ETHNIC Set", "Finger Snap"},
    {'P',   2,  49,  26, "SC-88 ETHNIC Set", "Tambourine"},
    {'P',   2,  49,  27, "SC-88 ETHNIC Set", "Castanets"},
    {'P',   2,  49,  28, "SC-88 ETHNIC Set", "Crash Cymbal 1"},
    {'P',   2,  49,  29, "SC-88 ETHNIC Set", "Snare Roll"},
    {'P',   2,  49,  30, "SC-88 ETHNIC Set", "Concert Snare Drum"},
    {'P',   2,  49,  31, "SC-88 ETHNIC Set", "Concert Cymbal"},
    {'P',   2,  49,  32, "SC-88 ETHNIC Set", "Concert BD 1"},
    {'P',   2,  49,  33, "SC-88 ETHNIC Set", "Jingle Bell"},
    {'P',   2,  49,  34, "SC-88 ETHNIC Set", "Bell Tree"},
    {'P',   2,  49,  35, "SC-88 ETHNIC Set", "Bar Chimes"},
    {'P',   2,  49,  36, "SC-88 ETHNIC Set", "Wadaiko"},
    {'P',   2,  49,  37, "SC-88 ETHNIC Set", "Wadaiko Rim"},
    {'P',   2,  49,  38, "SC-88 ETHNIC Set", "Shime Taiko"},
    {'P',   2,  49,  39, "SC-88 ETHNIC Set", "Atarigane"},
    {'P',   2,  49,  40, "SC-88 ETHNIC Set", "Hyoushigi"},
    {'P',   2,  49,  41, "SC-88 ETHNIC Set", "Ohkawa"},
    {'P',   2,  49,  42, "SC-88 ETHNIC Set", "High Kotsuzumi"},
    {'P',   2,  49,  43, "SC-88 ETHNIC Set", "Low Kotsuzumi"},
    {'P',   2,  49,  44, "SC-88 ETHNIC Set", "Ban Gu"},
    {'P',   2,  49,  45, "SC-88 ETHNIC Set", "Big Gong"},
    {'P',   2,  49,  46, "SC-88 ETHNIC Set", "Small Gong"},
    {'P',   2,  49,  47, "SC-88 ETHNIC Set", "Bend Gong"},
    {'P',   2,  49,  48, "SC-88 ETHNIC Set", "Thai Gong"},
    {'P',   2,  49,  49, "SC-88 ETHNIC Set", "Rama Cymbal"},
    {'P',   2,  49,  50, "SC-88 ETHNIC Set", "Gamelan Gong"},
    {'P',   2,  49,  51, "SC-88 ETHNIC Set", "Udo Short"},
    {'P',   2,  49,  52, "SC-88 ETHNIC Set", "Udo Long"},
    {'P',   2,  49,  53, "SC-88 ETHNIC Set", "Udo Slap"},
    {'P',   2,  49,  54, "SC-88 ETHNIC Set", "Bendir"},
    {'P',   2,  49,  55, "SC-88 ETHNIC Set", "Req Dum"},
    {'P',   2,  49,  56, "SC-88 ETHNIC Set", "Req Tik"},
    {'P',   2,  49,  57, "SC-88 ETHNIC Set", "Tabla Te"},
    {'P',   2,  49,  58, "SC-88 ETHNIC Set", "Tabla Na"},
    {'P',   2,  49,  59, "SC-88 ETHNIC Set", "Tabla Tun"},
    {'P',   2,  49,  60, "SC-88 ETHNIC Set", "Tabla Ge"},
    {'P',   2,  49,  61, "SC-88 ETHNIC Set", "Tabla Ge Hi"},
    {'P',   2,  49,  62, "SC-88 ETHNIC Set", "Talking Drum"},
    {'P',   2,  49,  63, "SC-88 ETHNIC Set", "Bend Talking Drum"},
    {'P',   2,  49,  64, "SC-88 ETHNIC Set", "Caxixi"},
    {'P',   2,  49,  65, "SC-88 ETHNIC Set", "Djembe"},
    {'P',   2,  49,  66, "SC-88 ETHNIC Set", "Djembe Rim"},
    {'P',   2,  49,  67, "SC-88 ETHNIC Set", "Timbales Low"},
    {'P',   2,  49,  68, "SC-88 ETHNIC Set", "Timbales Paila"},
    {'P',   2,  49,  69, "SC-88 ETHNIC Set", "Timbales High"},
    {'P',   2,  49,  70, "SC-88 ETHNIC Set", "Cowbell"},
    {'P',   2,  49,  71, "SC-88 ETHNIC Set", "Hi Bongo"},
    {'P',   2,  49,  72, "SC-88 ETHNIC Set", "Low Bongo"},
    {'P',   2,  49,  73, "SC-88 ETHNIC Set", "Mute Hi Conga"},
    {'P',   2,  49,  74, "SC-88 ETHNIC Set", "Open Hi Conga"},
    {'P',   2,  49,  75, "SC-88 ETHNIC Set", "Mute Low Conga"},
    {'P',   2,  49,  76, "SC-88 ETHNIC Set", "Conga Slap"},
    {'P',   2,  49,  77, "SC-88 ETHNIC Set", "Open Low Conga"},
    {'P',   2,  49,  78, "SC-88 ETHNIC Set", "Conga Slide"},
    {'P',   2,  49,  79, "SC-88 ETHNIC Set", "Mute Pandiero"},
    {'P',   2,  49,  80, "SC-88 ETHNIC Set", "Open Pandiero"},
    {'P',   2,  49,  81, "SC-88 ETHNIC Set", "Open Surdo"},
    {'P',   2,  49,  82, "SC-88 ETHNIC Set", "Mute Surdo"},
    {'P',   2,  49,  83, "SC-88 ETHNIC Set", "Tamborim"},
    {'P',   2,  49,  84, "SC-88 ETHNIC Set", "High Agogo"},
    {'P',   2,  49,  85, "SC-88 ETHNIC Set", "Low Agogo"},
    {'P',   2,  49,  86, "SC-88 ETHNIC Set", "Shaker"},
    {'P',   2,  49,  87, "SC-88 ETHNIC Set", "High Whistle"},
    {'P',   2,  49,  88, "SC-88 ETHNIC Set", "Low Whistle"},
    {'P',   2,  49,  89, "SC-88 ETHNIC Set", "Mute Cuica"},
    {'P',   2,  49,  90, "SC-88 ETHNIC Set", "Open Cuica"},
    {'P',   2,  49,  91, "SC-88 ETHNIC Set", "Mute Triangle"},
    {'P',   2,  49,  92, "SC-88 ETHNIC Set", "Open Triangle"},
    {'P',   2,  49,  93, "SC-88 ETHNIC Set", "Short Guiro"},
    {'P',   2,  49,  94, "SC-88 ETHNIC Set", "Long Guiro"},
    {'P',   2,  49,  95, "SC-88 ETHNIC Set", "Cabasa Up"},
    {'P',   2,  49,  96, "SC-88 ETHNIC Set", "Cabasa Down"},
    {'P',   2,  49,  97, "SC-88 ETHNIC Set", "Claves"},
    {'P',   2,  49,  98, "SC-88 ETHNIC Set", "High Wood Block"},
    {'P',   2,  49,  99, "SC-88 ETHNIC Set", "Low Wood Block"},
    {'P',   2,  50,  40, "SC-88 KICK & SNARE Set", "Standard 1 Kick 1"},
    {'P',   2,  50,  41, "SC-88 KICK & SNARE Set", "Standard 1 Kick 2"},
    {'P',   2,  50,  42, "SC-88 KICK & SNARE Set", "Standard 2 Kick 1"},
    {'P',   2,  50,  43, "SC-88 KICK & SNARE Set", "Standard 2 Kick 2"},
    {'P',   2,  50,  44, "SC-88 KICK & SNARE Set", "Kick 1"},
    {'P',   2,  50,  45, "SC-88 KICK & SNARE Set", "Kick 2"},
    {'P',   2,  50,  46, "SC-88 KICK & SNARE Set", "Soft Kick"},
    {'P',   2,  50,  47, "SC-88 KICK & SNARE Set", "Jazz Kick 1"},
    {'P',   2,  50,  48, "SC-88 KICK & SNARE Set", "Jazz Kick 2"},
    {'P',   2,  50,  49, "SC-88 KICK & SNARE Set", "Concert BD"},
    {'P',   2,  50,  50, "SC-88 KICK & SNARE Set", "Room Kick 1"},
    {'P',   2,  50,  51, "SC-88 KICK & SNARE Set", "Room Kick 2"},
    {'P',   2,  50,  52, "SC-88 KICK & SNARE Set", "Power Kick 1"},
    {'P',   2,  50,  53, "SC-88 KICK & SNARE Set", "Power Kick 2"},
    {'P',   2,  50,  54, "SC-88 KICK & SNARE Set", "Electric Kick 2"},
    {'P',   2,  50,  55, "SC-88 KICK & SNARE Set", "Electric Kick 1"},
    {'P',   2,  50,  56, "SC-88 KICK & SNARE Set", "Electric Kick"},
    {'P',   2,  50,  57, "SC-88 KICK & SNARE Set", "808 Bass Drum"},
    {'P',   2,  50,  58, "SC-88 KICK & SNARE Set", "909 Bass Drum"},
    {'P',   2,  50,  59, "SC-88 KICK & SNARE Set", "Dance Kick"},
    {'P',   2,  50,  60, "SC-88 KICK & SNARE Set", "Standard 1 Snare 1"},
    {'P',   2,  50,  61, "SC-88 KICK & SNARE Set", "Standard 1 Snare 2"},
    {'P',   2,  50,  62, "SC-88 KICK & SNARE Set", "Standard 2 Snare 1"},
    {'P',   2,  50,  63, "SC-88 KICK & SNARE Set", "Standard 2 Snare 2"},
    {'P',   2,  50,  64, "SC-88 KICK & SNARE Set", "Tight Snare"},
    {'P',   2,  50,  65, "SC-88 KICK & SNARE Set", "Concert Snare"},
    {'P',   2,  50,  66, "SC-88 KICK & SNARE Set", "Jazz Snare 1"},
    {'P',   2,  50,  67, "SC-88 KICK & SNARE Set", "Jazz Snare 2"},
    {'P',   2,  50,  68, "SC-88 KICK & SNARE Set", "Room Snare 1"},
    {'P',   2,  50,  69, "SC-88 KICK & SNARE Set", "Room Snare 2"},
    {'P',   2,  50,  70, "SC-88 KICK & SNARE Set", "Power Snare 1"},
    {'P',   2,  50,  71, "SC-88 KICK & SNARE Set", "Power Snare 2"},
    {'P',   2,  50,  72, "SC-88 KICK & SNARE Set", "Gated Snare"},
    {'P',   2,  50,  73, "SC-88 KICK & SNARE Set", "Dance Snare 1"},
    {'P',   2,  50,  74, "SC-88 KICK & SNARE Set", "Dance Snare 2"},
    {'P',   2,  50,  75, "SC-88 KICK & SNARE Set", "Disco Snare"},
    {'P',   2,  50,  76, "SC-88 KICK & SNARE Set", "Electric Snare 2"},
    {'P',   2,  50,  77, "SC-88 KICK & SNARE Set", "House Snare"},
    {'P',   2,  50,  78, "SC-88 KICK & SNARE Set", "Electric Snare 1"},
    {'P',   2,  50,  79, "SC-88 KICK & SNARE Set", "Electric Snare 3"},
    {'P',   2,  50,  80, "SC-88 KICK & SNARE Set", "808 Snare 1"},
    {'P',   2,  50,  81, "SC-88 KICK & SNARE Set", "808 Snare 2"},
    {'P',   2,  50,  82, "SC-88 KICK & SNARE Set", "909 Snare 1"},
    {'P',   2,  50,  83, "SC-88 KICK & SNARE Set", "909 Snare 2"},
    {'P',   2,  50,  84, "SC-88 KICK & SNARE Set", "Brush Tap 1"},
    {'P',   2,  50,  85, "SC-88 KICK & SNARE Set", "Brush Tap 2"},
    {'P',   2,  50,  86, "SC-88 KICK & SNARE Set", "Brush Slap 1"},
    {'P',   2,  50,  87, "SC-88 KICK & SNARE Set", "Brush Slap 2"},
    {'P',   2,  50,  88, "SC-88 KICK & SNARE Set", "Brush Slap 3"},
    {'P',   2,  50,  89, "SC-88 KICK & SNARE Set", "Brush Swirl 1"},
    {'P',   2,  50,  90, "SC-88 KICK & SNARE Set", "Brush Swirl 2"},
    {'P',   2,  50,  91, "SC-88 KICK & SNARE Set", "Brush Long Swirl"},
    {'P',   2,  56,  31, "SC-88 SFX Set", "Scratch Push 2"},
    {'P',   2,  56,  32, "SC-88 SFX Set", "Scratch Pull 2"},
    {'P',   2,  56,  33, "SC-88 SFX Set", "Cutting Noise 2 Up"},
    {'P',   2,  56,  34, "SC-88 SFX Set", "Cutting Noise 2 Down"},
    {'P',   2,  56,  35, "SC-88 SFX Set", "Distortion Guitar Cutting Noise Up"},
    {'P',   2,  56,  36, "SC-88 SFX Set", "Distortion Guitar Cutting Noise Down"},
    {'P',   2,  56,  37, "SC-88 SFX Set", "Bass Slide"},
    {'P',   2,  56,  38, "SC-88 SFX Set", "Pick Scrape"},
    {'P',   2,  56,  39, "SC-88 SFX Set", "High Q"},
    {'P',   2,  56,  40, "SC-88 SFX Set", "Slap"},
    {'P',   2,  56,  41, "SC-88 SFX Set", "Scratch Push"},
    {'P',   2,  56,  42, "SC-88 SFX Set", "Scratch Pull"},
    {'P',   2,  56,  43, "SC-88 SFX Set", "Sticks"},
    {'P',   2,  56,  44, "SC-88 SFX Set", "Square Click"},
    {'P',   2,  56,  45, "SC-88 SFX Set", "Metronome Click"},
    {'P',   2,  56,  46, "SC-88 SFX Set", "Metronome Bell"},
    {'P',   2,  56,  47, "SC-88 SFX Set", "Guitar Fret Noise"},
    {'P',   2,  56,  48, "SC-88 SFX Set", "Guitar Cutting Noise Up"},
    {'P',   2,  56,  49, "SC-88 SFX Set", "Guitar Cutting Noise Down"},
    {'P',   2,  56,  50, "SC-88 SFX Set", "String Slap of Double Bass"},
    {'P',   2,  56,  51, "SC-88 SFX Set", "Fl.Key Click"},
    {'P',   2,  56,  52, "SC-88 SFX Set", "Laughing"},
    {'P',   2,  56,  53, "SC-88 SFX Set", "Scream"},
    {'P',   2,  56,  54, "SC-88 SFX Set", "Punch"},
    {'P',   2,  56,  55, "SC-88 SFX Set", "Heart Beat"},
    {'P',   2,  56,  56, "SC-88 SFX Set", "Footsteps 1"},
    {'P',   2,  56,  57, "SC-88 SFX Set", "Footsteps 2"},
    {'P',   2,  56,  58, "SC-88 SFX Set", "Applause"},
    {'P',   2,  56,  59, "SC-88 SFX Set", "Door Creaking"},
    {'P',   2,  56,  60, "SC-88 SFX Set", "Door"},
    {'P',   2,  56,  61, "SC-88 SFX Set", "Scratch"},
    {'P',   2,  56,  62, "SC-88 SFX Set", "Wind Chimes"},
    {'P',   2,  56,  63, "SC-88 SFX Set", "Car-Engine"},
    {'P',   2,  56,  64, "SC-88 SFX Set", "Car-Stop"},
    {'P',   2,  56,  65, "SC-88 SFX Set", "Car-Pass"},
    {'P',   2,  56,  66, "SC-88 SFX Set", "Car-Crash"},
    {'P',   2,  56,  67, "SC-88 SFX Set", "Siren"},
    {'P',   2,  56,  68, "SC-88 SFX Set", "Train"},
    {'P',   2,  56,  69, "SC-88 SFX Set", "Jetplane"},
    {'P',   2,  56,  70, "SC-88 SFX Set", "Helicopter"},
    {'P',   2,  56,  71, "SC-88 SFX Set", "Starship"},
    {'P',   2,  56,  72, "SC-88 SFX Set", "Gun Shot"},
    {'P',   2,  56,  73, "SC-88 SFX Set", "Machine Gun"},
    {'P',   2,  56,  74, "SC-88 SFX Set", "Lasergun"},
    {'P',   2,  56,  75, "SC-88 SFX Set", "Explosion"},
    {'P',   2,  56,  76, "SC-88 SFX Set", "Dog"},
    {'P',   2,  56,  77, "SC-88 SFX Set", "Horse-Gallop"},
    {'P',   2,  56,  78, "SC-88 SFX Set", "Birds"},
    {'P',   2,  56,  79, "SC-88 SFX Set", "Rain"},
    {'P',   2,  56,  80, "SC-88 SFX Set", "Thunder"},
    {'P',   2,  56,  81, "SC-88 SFX Set", "Wind"},
    {'P',   2,  56,  82, "SC-88 SFX Set", "Seashore"},
    {'P',   2,  56,  83, "SC-88 SFX Set", "Stream"},
    {'P',   2,  56,  84, "SC-88 SFX Set", "Bubble"},
    {'P',   2,  56,  85, "SC-88 SFX Set", "Kitty"},
    {'P',   2,  56,  86, "SC-88 SFX Set", "Bird 2"},
    {'P',   2,  56,  87, "SC-88 SFX Set", "Growl"},
    {'P',   2,  56,  88, "SC-88 SFX Set", "Applause 2"},
    {'P',   2,  56,  89, "SC-88 SFX Set", "Telephone 1"},
    {'P',   2,  56,  90, "SC-88 SFX Set", "Telephone 2"},
    {'P',   2,  57,  36, "SC-88 RHYTHM FX Set", "Reverse Kick 1"},
    {'P',   2,  57,  37, "SC-88 RHYTHM FX Set", "Reverse Concert BD 1"},
    {'P',   2,  57,  38, "SC-88 RHYTHM FX Set", "Reverse Power Kick 1"},
    {'P',   2,  57,  39, "SC-88 RHYTHM FX Set", "Reverse Electric Kick 1"},
    {'P',   2,  57,  40, "SC-88 RHYTHM FX Set", "Reverse Snare 1"},
    {'P',   2,  57,  41, "SC-88 RHYTHM FX Set", "Reverse Snare 2"},
    {'P',   2,  57,  42, "SC-88 RHYTHM FX Set", "Reverse Standard Set 1 Snare 1"},
    {'P',   2,  57,  43, "SC-88 RHYTHM FX Set", "Reverse Tight Snare"},
    {'P',   2,  57,  44, "SC-88 RHYTHM FX Set", "Reverse Dance Snare"},
    {'P',   2,  57,  45, "SC-88 RHYTHM FX Set", "Reverse 808 Snare"},
    {'P',   2,  57,  46, "SC-88 RHYTHM FX Set", "Reverse Tom 1"},
    {'P',   2,  57,  47, "SC-88 RHYTHM FX Set", "Reverse Tom 2"},
    {'P',   2,  57,  48, "SC-88 RHYTHM FX Set", "Reverse Sticks"},
    {'P',   2,  57,  49, "SC-88 RHYTHM FX Set", "Reverse Slap"},
    {'P',   2,  57,  50, "SC-88 RHYTHM FX Set", "Reverse Cymbal 1"},
    {'P',   2,  57,  51, "SC-88 RHYTHM FX Set", "Reverse Cymbal 2"},
    {'P',   2,  57,  52, "SC-88 RHYTHM FX Set", "Reverse Open Hi-hat"},
    {'P',   2,  57,  53, "SC-88 RHYTHM FX Set", "Reverse Ride Cymbal"},
    {'P',   2,  57,  54, "SC-88 RHYTHM FX Set", "Reverse CR-78 OHH"},
    {'P',   2,  57,  55, "SC-88 RHYTHM FX Set", "Reverse Closed Hi-hat"},
    {'P',   2,  57,  56, "SC-88 RHYTHM FX Set", "Reverse Gong"},
    {'P',   2,  57,  57, "SC-88 RHYTHM FX Set", "Reverse Bell Tree"},
    {'P',   2,  57,  58, "SC-88 RHYTHM FX Set", "Reverse Guiro"},
    {'P',   2,  57,  59, "SC-88 RHYTHM FX Set", "Reverse Bendir"},
    {'P',   2,  57,  60, "SC-88 RHYTHM FX Set", "Reverse Gun Shot"},
    {'P',   2,  57,  61, "SC-88 RHYTHM FX Set", "Reverse Scratch"},
    {'P',   2,  57,  62, "SC-88 RHYTHM FX Set", "Reverse Laser"},
    {'P',   2,  57,  63, "SC-88 RHYTHM FX Set", "Key Click"},
    {'P',   2,  57,  64, "SC-88 RHYTHM FX Set", "Tekno Trip"},
    {'P',   2,  57,  65, "SC-88 RHYTHM FX Set", "Pop Drop"},
    {'P',   2,  57,  66, "SC-88 RHYTHM FX Set", "Woody Slap"},
    {'P',   2,  57,  67, "SC-88 RHYTHM FX Set", "Distortion Kick"},
    {'P',   2,  57,  68, "SC-88 RHYTHM FX Set", "Syn.Drop"},
    {'P',   2,  57,  69, "SC-88 RHYTHM FX Set", "Reverse High Q"},
    {'P',   2,  57,  70, "SC-88 RHYTHM FX Set", "Pipe"},
    {'P',   2,  57,  71, "SC-88 RHYTHM FX Set", "Ice Block"},
    {'P',   2,  57,  72, "SC-88 RHYTHM FX Set", "Digital Tambourine"},
    {'P',   2,  57,  73, "SC-88 RHYTHM FX Set", "Alias"},
    {'P',   2,  57,  74, "SC-88 RHYTHM FX Set", "Modulated Bell"},
    {'P',   2,  57,  75, "SC-88 RHYTHM FX Set", "Spark"},
    {'P',   2,  57,  76, "SC-88 RHYTHM FX Set", "Metalic Percussion"},
    {'P',   2,  57,  77, "SC-88 RHYTHM FX Set", "Velocity Noise FX"},
    {'P',   2,  57,  78, "SC-88 RHYTHM FX Set", "Stereo Noise Clap"},
    {'P',   2,  57,  79, "SC-88 RHYTHM FX Set", "Swish"},
    {'P',   2,  57,  80, "SC-88 RHYTHM FX Set", "Slappy"},
    {'P',   2,  57,  81, "SC-88 RHYTHM FX Set", "Voice Ou"},
    {'P',   2,  57,  82, "SC-88 RHYTHM FX Set", "Voice Au"},
    {'P',   2,  57,  83, "SC-88 RHYTHM FX Set", "Hoo"},
    {'P',   2,  57,  84, "SC-88 RHYTHM FX Set", "Tape Stop 1"},
    {'P',   2,  57,  85, "SC-88 RHYTHM FX Set", "Tape Stop 2"},
    {'P',   2,  57,  86, "SC-88 RHYTHM FX Set", "Missile"},
    {'P',   2,  57,  87, "SC-88 RHYTHM FX Set", "Space Bird"},
    {'P',   2,  57,  88, "SC-88 RHYTHM FX Set", "Flying Monster"},
    {'P',   3,   0,   0, "SC-88Pro STANDARD 1 Set", "[88] Standard 1 Kick 1"},
    {'P',   3,   0,   1, "SC-88Pro STANDARD 1 Set", "[88] Standard 1 Kick 2"},
    {'P',   3,   0,   2, "SC-88Pro STANDARD 1 Set", "[88] Standard 2 Kick 1"},
    {'P',   3,   0,   3, "SC-88Pro STANDARD 1 Set", "[88] Standard 2 Kick 2"},
    {'P',   3,   0,   4, "SC-88Pro STANDARD 1 Set", "[55] Kick Drum 1"},
    {'P',   3,   0,   5, "SC-88Pro STANDARD 1 Set", "[55] Kick Drum 2"},
    {'P',   3,   0,   6, "SC-88Pro STANDARD 1 Set", "[88] Jazz Kick 1"},
    {'P',   3,   0,   7, "SC-88Pro STANDARD 1 Set", "[88] Jazz Kick 2"},
    {'P',   3,   0,   8, "SC-88Pro STANDARD 1 Set", "[88] Room Kick 1"},
    {'P',   3,   0,   9, "SC-88Pro STANDARD 1 Set", "[88] Room Kick 2"},
    {'P',   3,   0,  10, "SC-88Pro STANDARD 1 Set", "[88] Power Kick 1"},
    {'P',   3,   0,  11, "SC-88Pro STANDARD 1 Set", "[88] Power Kick 2"},
    {'P',   3,   0,  12, "SC-88Pro STANDARD 1 Set", "[88] Electric Kick 2"},
    {'P',   3,   0,  13, "SC-88Pro STANDARD 1 Set", "[88] Electric Kick 1"},
    {'P',   3,   0,  14, "SC-88Pro STANDARD 1 Set", "[88] TR-808 Kick"},
    {'P',   3,   0,  15, "SC-88Pro STANDARD 1 Set", "[88] TR-909 Kick"},
    {'P',   3,   0,  16, "SC-88Pro STANDARD 1 Set", "[88] Dance Kick"},
    {'P',   3,   0,  17, "SC-88Pro STANDARD 1 Set", "Voice One"},
    {'P',   3,   0,  18, "SC-88Pro STANDARD 1 Set", "Voice Two"},
    {'P',   3,   0,  19, "SC-88Pro STANDARD 1 Set", "Voice Three"},
    {'P',   3,   0,  22, "SC-88Pro STANDARD 1 Set", "MC-500 Beep 1"},
    {'P',   3,   0,  23, "SC-88Pro STANDARD 1 Set", "MC-500 Beep 2"},
    {'P',   3,   0,  24, "SC-88Pro STANDARD 1 Set", "Concert SD"},
    {'P',   3,   0,  25, "SC-88Pro STANDARD 1 Set", "Snare Roll"},
    {'P',   3,   0,  26, "SC-88Pro STANDARD 1 Set", "Finger Snap 2"},
    {'P',   3,   0,  27, "SC-88Pro STANDARD 1 Set", "High Q"},
    {'P',   3,   0,  28, "SC-88Pro STANDARD 1 Set", "Slap"},
    {'P',   3,   0,  29, "SC-88Pro STANDARD 1 Set", "Scratch Push"},
    {'P',   3,   0,  30, "SC-88Pro STANDARD 1 Set", "Scratch Pull"},
    {'P',   3,   0,  31, "SC-88Pro STANDARD 1 Set", "Sticks"},
    {'P',   3,   0,  32, "SC-88Pro STANDARD 1 Set", "Square Click"},
    {'P',   3,   0,  33, "SC-88Pro STANDARD 1 Set", "Metronome Click"},
    {'P',   3,   0,  34, "SC-88Pro STANDARD 1 Set", "Metronome  Bell"},
    {'P',   3,   0,  35, "SC-88Pro STANDARD 1 Set", "Standard 1 Kick 2"},
    {'P',   3,   0,  36, "SC-88Pro STANDARD 1 Set", "Standard 1 Kick 1"},
    {'P',   3,   0,  37, "SC-88Pro STANDARD 1 Set", "Side Stick"},
    {'P',   3,   0,  38, "SC-88Pro STANDARD 1 Set", "Standard1 Snare 1"},
    {'P',   3,   0,  39, "SC-88Pro STANDARD 1 Set", "TR-909 Hand Clap"},
    {'P',   3,   0,  40, "SC-88Pro STANDARD 1 Set", "Standard1 Snare 2"},
    {'P',   3,   0,  41, "SC-88Pro STANDARD 1 Set", "Low Tom 2"},
    {'P',   3,   0,  42, "SC-88Pro STANDARD 1 Set", "Closed Hi-Hat"},
    {'P',   3,   0,  43, "SC-88Pro STANDARD 1 Set", "Low Tom 1"},
    {'P',   3,   0,  44, "SC-88Pro STANDARD 1 Set", "Pedal Hi-Hat"},
    {'P',   3,   0,  45, "SC-88Pro STANDARD 1 Set", "Mid Tom 2"},
    {'P',   3,   0,  46, "SC-88Pro STANDARD 1 Set", "Open Hi-Hat"},
    {'P',   3,   0,  47, "SC-88Pro STANDARD 1 Set", "Mid Tom 1"},
    {'P',   3,   0,  48, "SC-88Pro STANDARD 1 Set", "High Tom 2"},
    {'P',   3,   0,  49, "SC-88Pro STANDARD 1 Set", "Crash Cymbal 1"},
    {'P',   3,   0,  50, "SC-88Pro STANDARD 1 Set", "High Tom 1"},
    {'P',   3,   0,  51, "SC-88Pro STANDARD 1 Set", "Ride Cymbal 1"},
    {'P',   3,   0,  52, "SC-88Pro STANDARD 1 Set", "Chinese Cymbal"},
    {'P',   3,   0,  53, "SC-88Pro STANDARD 1 Set", "Ride Bell"},
    {'P',   3,   0,  54, "SC-88Pro STANDARD 1 Set", "Tambourine"},
    {'P',   3,   0,  55, "SC-88Pro STANDARD 1 Set", "Splash Cymbal"},
    {'P',   3,   0,  56, "SC-88Pro STANDARD 1 Set", "Cowbell"},
    {'P',   3,   0,  57, "SC-88Pro STANDARD 1 Set", "Crash Cymbal 2"},
    {'P',   3,   0,  58, "SC-88Pro STANDARD 1 Set", "Vibra-slap"},
    {'P',   3,   0,  59, "SC-88Pro STANDARD 1 Set", "Ride Cymbal 2"},
    {'P',   3,   0,  60, "SC-88Pro STANDARD 1 Set", "High Bongo"},
    {'P',   3,   0,  61, "SC-88Pro STANDARD 1 Set", "Low Bongo"},
    {'P',   3,   0,  62, "SC-88Pro STANDARD 1 Set", "Mute High Conga"},
    {'P',   3,   0,  63, "SC-88Pro STANDARD 1 Set", "Open High Conga"},
    {'P',   3,   0,  64, "SC-88Pro STANDARD 1 Set", "Open Low Conga"},
    {'P',   3,   0,  65, "SC-88Pro STANDARD 1 Set", "High Timbale"},
    {'P',   3,   0,  66, "SC-88Pro STANDARD 1 Set", "Low Timbale"},
    {'P',   3,   0,  67, "SC-88Pro STANDARD 1 Set", "High Agogo"},
    {'P',   3,   0,  68, "SC-88Pro STANDARD 1 Set", "Low Agogo"},
    {'P',   3,   0,  69, "SC-88Pro STANDARD 1 Set", "Cabasa"},
    {'P',   3,   0,  70, "SC-88Pro STANDARD 1 Set", "Maracas"},
    {'P',   3,   0,  71, "SC-88Pro STANDARD 1 Set", "Short High Whisle"},
    {'P',   3,   0,  72, "SC-88Pro STANDARD 1 Set", "Long Low Whisle"},
    {'P',   3,   0,  73, "SC-88Pro STANDARD 1 Set", "Short Guiro"},
    {'P',   3,   0,  74, "SC-88Pro STANDARD 1 Set", "Long Guiro"},
    {'P',   3,   0,  75, "SC-88Pro STANDARD 1 Set", "Claves"},
    {'P',   3,   0,  76, "SC-88Pro STANDARD 1 Set", "High Wood Block"},
    {'P',   3,   0,  77, "SC-88Pro STANDARD 1 Set", "Low Wood Block"},
    {'P',   3,   0,  78, "SC-88Pro STANDARD 1 Set", "Mute Cuica"},
    {'P',   3,   0,  79, "SC-88Pro STANDARD 1 Set", "Open Cuica"},
    {'P',   3,   0,  80, "SC-88Pro STANDARD 1 Set", "Mute Triangle"},
    {'P',   3,   0,  81, "SC-88Pro STANDARD 1 Set", "Open Triangle"},
    {'P',   3,   0,  82, "SC-88Pro STANDARD 1 Set", "Shaker"},
    {'P',   3,   0,  83, "SC-88Pro STANDARD 1 Set", "Jingle Bell"},
    {'P',   3,   0,  84, "SC-88Pro STANDARD 1 Set", "Bell Tree"},
    {'P',   3,   0,  85, "SC-88Pro STANDARD 1 Set", "Castanets"},
    {'P',   3,   0,  86, "SC-88Pro STANDARD 1 Set", "Mute Surdo"},
    {'P',   3,   0,  87, "SC-88Pro STANDARD 1 Set", "Open Surdo"},
    {'P',   3,   0,  88, "SC-88Pro STANDARD 1 Set", "Applause 2"},
    {'P',   3,   0,  97, "SC-88Pro STANDARD 1 Set", "[88] Standard 1 Snare 1"},
    {'P',   3,   0,  98, "SC-88Pro STANDARD 1 Set", "[88] Standard 1 Snare 2"},
    {'P',   3,   0,  99, "SC-88Pro STANDARD 1 Set", "[88] Standard 2 Snare 1"},
    {'P',   3,   0, 100, "SC-88Pro STANDARD 1 Set", "[88] Standard 2 Snare 2"},
    {'P',   3,   0, 101, "SC-88Pro STANDARD 1 Set", "[55] Snare Drum 2"},
    {'P',   3,   0, 102, "SC-88Pro STANDARD 1 Set", "Standard 1 Snare 1"},
    {'P',   3,   0, 103, "SC-88Pro STANDARD 1 Set", "Standard 1 Snare 2"},
    {'P',   3,   0, 104, "SC-88Pro STANDARD 1 Set", "Standard Snare 3"},
    {'P',   3,   0, 105, "SC-88Pro STANDARD 1 Set", "[88] Jazz Snare 1"},
    {'P',   3,   0, 106, "SC-88Pro STANDARD 1 Set", "[88] Jazz Snare 2"},
    {'P',   3,   0, 107, "SC-88Pro STANDARD 1 Set", "[88] Room Snare 1"},
    {'P',   3,   0, 108, "SC-88Pro STANDARD 1 Set", "[88] Room Snare 2"},
    {'P',   3,   0, 109, "SC-88Pro STANDARD 1 Set", "[88] Power Snare 1"},
    {'P',   3,   0, 110, "SC-88Pro STANDARD 1 Set", "[88] Power Snare 2"},
    {'P',   3,   0, 111, "SC-88Pro STANDARD 1 Set", "[55] Gated Snare"},
    {'P',   3,   0, 112, "SC-88Pro STANDARD 1 Set", "[88] Dance Snare 1"},
    {'P',   3,   0, 113, "SC-88Pro STANDARD 1 Set", "[88] Dance Snare 2"},
    {'P',   3,   0, 114, "SC-88Pro STANDARD 1 Set", "[88] Disco Snare"},
    {'P',   3,   0, 115, "SC-88Pro STANDARD 1 Set", "[88] Electric Snare 2"},
    {'P',   3,   0, 116, "SC-88Pro STANDARD 1 Set", "[55] Electric Snare"},
    {'P',   3,   0, 117, "SC-88Pro STANDARD 1 Set", "[88] Electric Snare 3"},
    {'P',   3,   0, 118, "SC-88Pro STANDARD 1 Set", "TR-707 Snare"},
    {'P',   3,   0, 119, "SC-88Pro STANDARD 1 Set", "[88] TR-808 Snare 1"},
    {'P',   3,   0, 120, "SC-88Pro STANDARD 1 Set", "[88] TR-808 Snare 2"},
    {'P',   3,   0, 121, "SC-88Pro STANDARD 1 Set", "[88] TR-909 Snare 1"},
    {'P',   3,   0, 122, "SC-88Pro STANDARD 1 Set", "[88] TR-909 Snare 2"},
    {'P',   3,   0, 123, "SC-88Pro STANDARD 1 Set", "Rap Snare"},
    {'P',   3,   0, 124, "SC-88Pro STANDARD 1 Set", "Jungle Snare 1"},
    {'P',   3,   0, 125, "SC-88Pro STANDARD 1 Set", "House Snare 1"},
    {'P',   3,   0, 126, "SC-88Pro STANDARD 1 Set", "[88] House Snare"},
    {'P',   3,   0, 127, "SC-88Pro STANDARD 1 Set", "House Snare 2"},
    {'P',   3,   1,  26, "SC-88Pro STANDARD 2 Set", "Finger Snap"},
    {'P',   3,   1,  29, "SC-88Pro STANDARD 2 Set", "Scratch Push"},
    {'P',   3,   1,  30, "SC-88Pro STANDARD 2 Set", "Scratch Pull"},
    {'P',   3,   1,  35, "SC-88Pro STANDARD 2 Set", "Standard 2 Kick 2"},
    {'P',   3,   1,  36, "SC-88Pro STANDARD 2 Set", "Standard 2 Kick 1"},
    {'P',   3,   1,  37, "SC-88Pro STANDARD 2 Set", "Side Stick"},
    {'P',   3,   1,  38, "SC-88Pro STANDARD 2 Set", "Standard 2 Snare 1"},
    {'P',   3,   1,  39, "SC-88Pro STANDARD 2 Set", "Hand Clap"},
    {'P',   3,   1,  40, "SC-88Pro STANDARD 2 Set", "Standard 2 Snare 2"},
    {'P',   3,   1,  41, "SC-88Pro STANDARD 2 Set", "Low Tom 2"},
    {'P',   3,   1,  42, "SC-88Pro STANDARD 2 Set", "Closed Hi-Hat"},
    {'P',   3,   1,  43, "SC-88Pro STANDARD 2 Set", "Low Tom 1"},
    {'P',   3,   1,  44, "SC-88Pro STANDARD 2 Set", "Pedal Hi-Hat"},
    {'P',   3,   1,  45, "SC-88Pro STANDARD 2 Set", "Mid Tom 2"},
    {'P',   3,   1,  46, "SC-88Pro STANDARD 2 Set", "Open Hi-Hat"},
    {'P',   3,   1,  47, "SC-88Pro STANDARD 2 Set", "Mid Tom 1"},
    {'P',   3,   1,  48, "SC-88Pro STANDARD 2 Set", "High Tom 2"},
    {'P',   3,   1,  49, "SC-88Pro STANDARD 2 Set", "Crash Cymbal 1"},
    {'P',   3,   1,  50, "SC-88Pro STANDARD 2 Set", "High Tom 1"},
    {'P',   3,   1,  51, "SC-88Pro STANDARD 2 Set", "Ride Cymbal"},
    {'P',   3,   1,  52, "SC-88Pro STANDARD 2 Set", "Chinese Cymbal"},
    {'P',   3,   1,  53, "SC-88Pro STANDARD 2 Set", "Ride Bell"},
    {'P',   3,   1,  54, "SC-88Pro STANDARD 2 Set", "Tambourine"},
    {'P',   3,   1,  55, "SC-88Pro STANDARD 2 Set", "Splash Cymbal"},
    {'P',   3,   1,  56, "SC-88Pro STANDARD 2 Set", "Cowbell"},
    {'P',   3,   1,  84, "SC-88Pro STANDARD 2 Set", "Bar Chimes"},
    {'P',   3,   1,  88, "SC-88Pro STANDARD 2 Set", "Applause 2"},
    {'P',   3,   2,  26, "SC-88Pro STANDARD 3 Set", "Finger Snap 2"},
    {'P',   3,   2,  29, "SC-88Pro STANDARD 3 Set", "Scratch Push"},
    {'P',   3,   2,  30, "SC-88Pro STANDARD 3 Set", "Scratch Pull"},
    {'P',   3,   2,  35, "SC-88Pro STANDARD 3 Set", "Standard 3 Kick 2"},
    {'P',   3,   2,  36, "SC-88Pro STANDARD 3 Set", "[RND] Kick"},
    {'P',   3,   2,  37, "SC-88Pro STANDARD 3 Set", "Side Stick"},
    {'P',   3,   2,  38, "SC-88Pro STANDARD 3 Set", "[RND] Snare"},
    {'P',   3,   2,  39, "SC-88Pro STANDARD 3 Set", "[RND] Hand Clap"},
    {'P',   3,   2,  40, "SC-88Pro STANDARD 3 Set", "Standard 3 Snare 2"},
    {'P',   3,   2,  41, "SC-88Pro STANDARD 3 Set", "Low Tom 2"},
    {'P',   3,   2,  42, "SC-88Pro STANDARD 3 Set", "[RND] Closed Hi-Hat"},
    {'P',   3,   2,  43, "SC-88Pro STANDARD 3 Set", "Low Tom 1"},
    {'P',   3,   2,  44, "SC-88Pro STANDARD 3 Set", "[RND] Pedal Hi-Hat"},
    {'P',   3,   2,  45, "SC-88Pro STANDARD 3 Set", "Mid Tom 2"},
    {'P',   3,   2,  46, "SC-88Pro STANDARD 3 Set", "[RND] Open Hi-Hat"},
    {'P',   3,   2,  47, "SC-88Pro STANDARD 3 Set", "Mid Tom 1"},
    {'P',   3,   2,  48, "SC-88Pro STANDARD 3 Set", "High Tom 2"},
    {'P',   3,   2,  49, "SC-88Pro STANDARD 3 Set", "[RND] Crash Cymbal"},
    {'P',   3,   2,  50, "SC-88Pro STANDARD 3 Set", "High Tom 1"},
    {'P',   3,   2,  51, "SC-88Pro STANDARD 3 Set", "[RND] Ride Cymbal 1"},
    {'P',   3,   2,  52, "SC-88Pro STANDARD 3 Set", "Chinese Cymbal"},
    {'P',   3,   2,  53, "SC-88Pro STANDARD 3 Set", "[RND] Ride Bell 1"},
    {'P',   3,   2,  54, "SC-88Pro STANDARD 3 Set", "Tambourine"},
    {'P',   3,   2,  55, "SC-88Pro STANDARD 3 Set", "Splash Cymbal"},
    {'P',   3,   2,  56, "SC-88Pro STANDARD 3 Set", "Cowbell"},
    {'P',   3,   2,  59, "SC-88Pro STANDARD 3 Set", "[RND] Ride Cymbal 2"},
    {'P',   3,   2,  88, "SC-88Pro STANDARD 3 Set", "Applause 2"},
    {'P',   3,   8,  26, "SC-88Pro ROOM Set", "Finger Snap"},
    {'P',   3,   8,  29, "SC-88Pro ROOM Set", "Scratch Push"},
    {'P',   3,   8,  30, "SC-88Pro ROOM Set", "Scratch Pull"},
    {'P',   3,   8,  35, "SC-88Pro ROOM Set", "Room Kick 2"},
    {'P',   3,   8,  36, "SC-88Pro ROOM Set", "Room Kick 1"},
    {'P',   3,   8,  37, "SC-88Pro ROOM Set", "Side Stick"},
    {'P',   3,   8,  38, "SC-88Pro ROOM Set", "Room Snare 1"},
    {'P',   3,   8,  39, "SC-88Pro ROOM Set", "Hand Clap"},
    {'P',   3,   8,  40, "SC-88Pro ROOM Set", "Room Snare 2"},
    {'P',   3,   8,  41, "SC-88Pro ROOM Set", "Room Low Tom 2"},
    {'P',   3,   8,  42, "SC-88Pro ROOM Set", "Closed Hi-Hat 3"},
    {'P',   3,   8,  43, "SC-88Pro ROOM Set", "Room Low Tom 1"},
    {'P',   3,   8,  44, "SC-88Pro ROOM Set", "Pedal Hi-Hat"},
    {'P',   3,   8,  45, "SC-88Pro ROOM Set", "Room Mid Tom 2"},
    {'P',   3,   8,  46, "SC-88Pro ROOM Set", "Open Hi-Hat 3"},
    {'P',   3,   8,  47, "SC-88Pro ROOM Set", "Room Mid Tom 1"},
    {'P',   3,   8,  48, "SC-88Pro ROOM Set", "Room High Tom 2"},
    {'P',   3,   8,  49, "SC-88Pro ROOM Set", "Crash Cymbal 1"},
    {'P',   3,   8,  50, "SC-88Pro ROOM Set", "Room High Tom 1"},
    {'P',   3,   8,  51, "SC-88Pro ROOM Set", "Ride Cymbal 1"},
    {'P',   3,   8,  52, "SC-88Pro ROOM Set", "Chinese Cymbal"},
    {'P',   3,   8,  53, "SC-88Pro ROOM Set", "Ride Bell"},
    {'P',   3,   8,  54, "SC-88Pro ROOM Set", "Tambourine"},
    {'P',   3,   8,  55, "SC-88Pro ROOM Set", "Splash Cymbal"},
    {'P',   3,   8,  56, "SC-88Pro ROOM Set", "Cowbell"},
    {'P',   3,   8,  88, "SC-88Pro ROOM Set", "Applause 2"},
    {'P',   3,   9,   0, "SC-88Pro HIP HOP Set", "[88] Electric Kick 2"},
    {'P',   3,   9,   1, "SC-88Pro HIP HOP Set", "[88] Electric Kick 1"},
    {'P',   3,   9,   2, "SC-88Pro HIP HOP Set", "CR-78 Kick 1"},
    {'P',   3,   9,   3, "SC-88Pro HIP HOP Set", "CR-78 Kick 2"},
    {'P',   3,   9,   4, "SC-88Pro HIP HOP Set", "TR-606 Kick 1"},
    {'P',   3,   9,   5, "SC-88Pro HIP HOP Set", "TR-707 Kick 1"},
    {'P',   3,   9,   6, "SC-88Pro HIP HOP Set", "[55] TR-808 Kick"},
    {'P',   3,   9,   7, "SC-88Pro HIP HOP Set", "[88] TR-808 Kick"},
    {'P',   3,   9,   8, "SC-88Pro HIP HOP Set", "TR-808 Kick 2"},
    {'P',   3,   9,   9, "SC-88Pro HIP HOP Set", "[88] TR-909 Kick"},
    {'P',   3,   9,  10, "SC-88Pro HIP HOP Set", "[88] Dance Kick"},
    {'P',   3,   9,  11, "SC-88Pro HIP HOP Set", "Hip-Hop Kick 2"},
    {'P',   3,   9,  12, "SC-88Pro HIP HOP Set", "TR-909 Kick 1"},
    {'P',   3,   9,  13, "SC-88Pro HIP HOP Set", "Hip-Hop Kick 3"},
    {'P',   3,   9,  14, "SC-88Pro HIP HOP Set", "Jungle Kick 1"},
    {'P',   3,   9,  15, "SC-88Pro HIP HOP Set", "Techno Kick 1"},
    {'P',   3,   9,  16, "SC-88Pro HIP HOP Set", "Bounce Kick"},
    {'P',   3,   9,  17, "SC-88Pro HIP HOP Set", "Voice One"},
    {'P',   3,   9,  18, "SC-88Pro HIP HOP Set", "Voice Two"},
    {'P',   3,   9,  19, "SC-88Pro HIP HOP Set", "Voice Three"},
    {'P',   3,   9,  22, "SC-88Pro HIP HOP Set", "MC-500 Beep 1"},
    {'P',   3,   9,  23, "SC-88Pro HIP HOP Set", "MC-500 Beep 2"},
    {'P',   3,   9,  24, "SC-88Pro HIP HOP Set", "Concert SD"},
    {'P',   3,   9,  25, "SC-88Pro HIP HOP Set", "Snare Roll"},
    {'P',   3,   9,  26, "SC-88Pro HIP HOP Set", "Finger Snap 2"},
    {'P',   3,   9,  27, "SC-88Pro HIP HOP Set", "High Q"},
    {'P',   3,   9,  28, "SC-88Pro HIP HOP Set", "Slap"},
    {'P',   3,   9,  29, "SC-88Pro HIP HOP Set", "Scratch Push 2"},
    {'P',   3,   9,  30, "SC-88Pro HIP HOP Set", "Scratch Pull 2"},
    {'P',   3,   9,  31, "SC-88Pro HIP HOP Set", "Sticks"},
    {'P',   3,   9,  32, "SC-88Pro HIP HOP Set", "Square Click"},
    {'P',   3,   9,  33, "SC-88Pro HIP HOP Set", "Metronome Click"},
    {'P',   3,   9,  34, "SC-88Pro HIP HOP Set", "Metronome Bell"},
    {'P',   3,   9,  35, "SC-88Pro HIP HOP Set", "Hip-Hop Kick 2"},
    {'P',   3,   9,  36, "SC-88Pro HIP HOP Set", "Hip-Hop Kick 1"},
    {'P',   3,   9,  37, "SC-88Pro HIP HOP Set", "TR-808 Rim Shot"},
    {'P',   3,   9,  38, "SC-88Pro HIP HOP Set", "Rap Snare"},
    {'P',   3,   9,  39, "SC-88Pro HIP HOP Set", "TR-909 Hand Clap"},
    {'P',   3,   9,  40, "SC-88Pro HIP HOP Set", "Hip-Hop Snare 2"},
    {'P',   3,   9,  41, "SC-88Pro HIP HOP Set", "TR-909 Low Tom 2"},
    {'P',   3,   9,  42, "SC-88Pro HIP HOP Set", "Room Closed Hi-Hat"},
    {'P',   3,   9,  43, "SC-88Pro HIP HOP Set", "TR-909 Low Tom 1"},
    {'P',   3,   9,  44, "SC-88Pro HIP HOP Set", "Pedal Hi-Hat"},
    {'P',   3,   9,  45, "SC-88Pro HIP HOP Set", "TR-909 Mid Tom 2"},
    {'P',   3,   9,  46, "SC-88Pro HIP HOP Set", "Room Open Hi-Hat"},
    {'P',   3,   9,  47, "SC-88Pro HIP HOP Set", "TR-909 Mid Tom 1"},
    {'P',   3,   9,  48, "SC-88Pro HIP HOP Set", "TR-909 High Tom 2"},
    {'P',   3,   9,  49, "SC-88Pro HIP HOP Set", "TR-909 Crash Cymbal"},
    {'P',   3,   9,  50, "SC-88Pro HIP HOP Set", "TR-909 High Tom 1"},
    {'P',   3,   9,  51, "SC-88Pro HIP HOP Set", "Ride Cymbal 1"},
    {'P',   3,   9,  52, "SC-88Pro HIP HOP Set", "Reverse Cymbal"},
    {'P',   3,   9,  53, "SC-88Pro HIP HOP Set", "Ride Bell"},
    {'P',   3,   9,  54, "SC-88Pro HIP HOP Set", "Shake Tambourine"},
    {'P',   3,   9,  55, "SC-88Pro HIP HOP Set", "Splash Cymbal"},
    {'P',   3,   9,  56, "SC-88Pro HIP HOP Set", "TR-808 Cowbell"},
    {'P',   3,   9,  57, "SC-88Pro HIP HOP Set", "Crash Cymbal 2"},
    {'P',   3,   9,  58, "SC-88Pro HIP HOP Set", "Vibra-slap"},
    {'P',   3,   9,  59, "SC-88Pro HIP HOP Set", "Ride Cymbal 2"},
    {'P',   3,   9,  60, "SC-88Pro HIP HOP Set", "High Bongo"},
    {'P',   3,   9,  61, "SC-88Pro HIP HOP Set", "Low Bongo"},
    {'P',   3,   9,  62, "SC-88Pro HIP HOP Set", "Mute High Conga"},
    {'P',   3,   9,  63, "SC-88Pro HIP HOP Set", "Open High Conga"},
    {'P',   3,   9,  64, "SC-88Pro HIP HOP Set", "Low Conga"},
    {'P',   3,   9,  65, "SC-88Pro HIP HOP Set", "High Timbale"},
    {'P',   3,   9,  66, "SC-88Pro HIP HOP Set", "Low Timbale"},
    {'P',   3,   9,  67, "SC-88Pro HIP HOP Set", "High Agogo"},
    {'P',   3,   9,  68, "SC-88Pro HIP HOP Set", "Low Agogo"},
    {'P',   3,   9,  69, "SC-88Pro HIP HOP Set", "Cabasa"},
    {'P',   3,   9,  70, "SC-88Pro HIP HOP Set", "TR-808 Maracas"},
    {'P',   3,   9,  71, "SC-88Pro HIP HOP Set", "Short High Whisle"},
    {'P',   3,   9,  72, "SC-88Pro HIP HOP Set", "Long Low Whisle"},
    {'P',   3,   9,  73, "SC-88Pro HIP HOP Set", "Short Guiro"},
    {'P',   3,   9,  74, "SC-88Pro HIP HOP Set", "CR-78 Guiro"},
    {'P',   3,   9,  75, "SC-88Pro HIP HOP Set", "TR-808 Claves"},
    {'P',   3,   9,  76, "SC-88Pro HIP HOP Set", "High Wood Block"},
    {'P',   3,   9,  77, "SC-88Pro HIP HOP Set", "Low Wood Block"},
    {'P',   3,   9,  78, "SC-88Pro HIP HOP Set", "High Hoo"},
    {'P',   3,   9,  79, "SC-88Pro HIP HOP Set", "Low Hoo"},
    {'P',   3,   9,  80, "SC-88Pro HIP HOP Set", "Mute Triangle"},
    {'P',   3,   9,  81, "SC-88Pro HIP HOP Set", "Open Triangle"},
    {'P',   3,   9,  82, "SC-88Pro HIP HOP Set", "TR-626 Shaker"},
    {'P',   3,   9,  83, "SC-88Pro HIP HOP Set", "Jingle Bell"},
    {'P',   3,   9,  84, "SC-88Pro HIP HOP Set", "Bell Tree"},
    {'P',   3,   9,  85, "SC-88Pro HIP HOP Set", "Castanets"},
    {'P',   3,   9,  86, "SC-88Pro HIP HOP Set", "Mute Surdo"},
    {'P',   3,   9,  87, "SC-88Pro HIP HOP Set", "Open Surdo"},
    {'P',   3,   9,  88, "SC-88Pro HIP HOP Set", "Small Club 1"},
    {'P',   3,   9,  97, "SC-88Pro HIP HOP Set", "Techno Hit"},
    {'P',   3,   9,  98, "SC-88Pro HIP HOP Set", "Philly Hit"},
    {'P',   3,   9,  99, "SC-88Pro HIP HOP Set", "Impact Hit"},
    {'P',   3,   9, 100, "SC-88Pro HIP HOP Set", "Lo-Fi Rave"},
    {'P',   3,   9, 101, "SC-88Pro HIP HOP Set", "Bam Hit"},
    {'P',   3,   9, 102, "SC-88Pro HIP HOP Set", "Bim Hit"},
    {'P',   3,   9, 103, "SC-88Pro HIP HOP Set", "Tape Rewind"},
    {'P',   3,   9, 104, "SC-88Pro HIP HOP Set", "Phonograph Noise"},
    {'P',   3,   9, 105, "SC-88Pro HIP HOP Set", "[88] Power Snare 1"},
    {'P',   3,   9, 106, "SC-88Pro HIP HOP Set", "[88] Dance Snare 1"},
    {'P',   3,   9, 107, "SC-88Pro HIP HOP Set", "[88] Dance Snare 2"},
    {'P',   3,   9, 108, "SC-88Pro HIP HOP Set", "[88] Disco Snare"},
    {'P',   3,   9, 109, "SC-88Pro HIP HOP Set", "[88] Electric Snare 2"},
    {'P',   3,   9, 110, "SC-88Pro HIP HOP Set", "[55] Electric Snare"},
    {'P',   3,   9, 111, "SC-88Pro HIP HOP Set", "[88] Electric Snare 3"},
    {'P',   3,   9, 112, "SC-88Pro HIP HOP Set", "TR-606 Snare 2"},
    {'P',   3,   9, 113, "SC-88Pro HIP HOP Set", "TR-707 Snare 1"},
    {'P',   3,   9, 114, "SC-88Pro HIP HOP Set", "[88] TR-808 Snare 2"},
    {'P',   3,   9, 115, "SC-88Pro HIP HOP Set", "[88] TR-808 Snare 1"},
    {'P',   3,   9, 116, "SC-88Pro HIP HOP Set", "TR-808 Snare 2"},
    {'P',   3,   9, 117, "SC-88Pro HIP HOP Set", "[88] TR-909 Snare 1"},
    {'P',   3,   9, 118, "SC-88Pro HIP HOP Set", "[88] TR-909 Snare 2"},
    {'P',   3,   9, 119, "SC-88Pro HIP HOP Set", "TR-909 Snare 1"},
    {'P',   3,   9, 120, "SC-88Pro HIP HOP Set", "TR-909 Snare 2"},
    {'P',   3,   9, 121, "SC-88Pro HIP HOP Set", "Rap Snare"},
    {'P',   3,   9, 122, "SC-88Pro HIP HOP Set", "Jungle Snare"},
    {'P',   3,   9, 123, "SC-88Pro HIP HOP Set", "House Snare 1"},
    {'P',   3,   9, 124, "SC-88Pro HIP HOP Set", "[88] House Snare"},
    {'P',   3,   9, 125, "SC-88Pro HIP HOP Set", "House Snare 2"},
    {'P',   3,   9, 126, "SC-88Pro HIP HOP Set", "Voice Tah"},
    {'P',   3,   9, 127, "SC-88Pro HIP HOP Set", "[88] Slappy"},
    {'P',   3,  10,  26, "SC-88Pro JUNGLE Set", "Finger Snap 2"},
    {'P',   3,  10,  29, "SC-88Pro JUNGLE Set", "Scratch Push 2"},
    {'P',   3,  10,  30, "SC-88Pro JUNGLE Set", "Scratch Pull 2"},
    {'P',   3,  10,  35, "SC-88Pro JUNGLE Set", "Jungle Kick 2"},
    {'P',   3,  10,  36, "SC-88Pro JUNGLE Set", "Jungle Kick 1"},
    {'P',   3,  10,  37, "SC-88Pro JUNGLE Set", "TR-808 Rim Shot"},
    {'P',   3,  10,  38, "SC-88Pro JUNGLE Set", "Jungle Snare 1"},
    {'P',   3,  10,  39, "SC-88Pro JUNGLE Set", "Hand Clap 2"},
    {'P',   3,  10,  40, "SC-88Pro JUNGLE Set", "Jungle Snare 2"},
    {'P',   3,  10,  41, "SC-88Pro JUNGLE Set", "TR-909 Low Tom 2"},
    {'P',   3,  10,  42, "SC-88Pro JUNGLE Set", "TR-606 Closed Hi-Hat"},
    {'P',   3,  10,  43, "SC-88Pro JUNGLE Set", "TR-909 Low Tom 1"},
    {'P',   3,  10,  44, "SC-88Pro JUNGLE Set", "Jungle Hi-Hat"},
    {'P',   3,  10,  45, "SC-88Pro JUNGLE Set", "TR-909 Mid Tom 2"},
    {'P',   3,  10,  46, "SC-88Pro JUNGLE Set", "TR-606 Open Hi-Hat"},
    {'P',   3,  10,  47, "SC-88Pro JUNGLE Set", "TR-909 Mid Tom 1"},
    {'P',   3,  10,  48, "SC-88Pro JUNGLE Set", "TR-909 High Tom 2"},
    {'P',   3,  10,  49, "SC-88Pro JUNGLE Set", "TR-808 Crash Cymbal"},
    {'P',   3,  10,  50, "SC-88Pro JUNGLE Set", "TR-909 High Tom1"},
    {'P',   3,  10,  51, "SC-88Pro JUNGLE Set", "Ride Cymbal 1"},
    {'P',   3,  10,  52, "SC-88Pro JUNGLE Set", "Reverse Cymbal"},
    {'P',   3,  10,  53, "SC-88Pro JUNGLE Set", "Ride Bell"},
    {'P',   3,  10,  54, "SC-88Pro JUNGLE Set", "Shake Tambourine"},
    {'P',   3,  10,  55, "SC-88Pro JUNGLE Set", "Splash Cymbal"},
    {'P',   3,  10,  56, "SC-88Pro JUNGLE Set", "TR-808 Cowbell"},
    {'P',   3,  10,  70, "SC-88Pro JUNGLE Set", "TR-808 Maracas"},
    {'P',   3,  10,  74, "SC-88Pro JUNGLE Set", "CR-78 Guiro"},
    {'P',   3,  10,  75, "SC-88Pro JUNGLE Set", "TR-808 Claves"},
    {'P',   3,  10,  78, "SC-88Pro JUNGLE Set", "High Hoo"},
    {'P',   3,  10,  79, "SC-88Pro JUNGLE Set", "Low Hoo"},
    {'P',   3,  10,  80, "SC-88Pro JUNGLE Set", "Mute Triangle"},
    {'P',   3,  10,  81, "SC-88Pro JUNGLE Set", "Open Triangle"},
    {'P',   3,  10,  82, "SC-88Pro JUNGLE Set", "TR-626 Shaker"},
    {'P',   3,  10,  88, "SC-88Pro JUNGLE Set", "Small Club1"},
    {'P',   3,  11,  26, "SC-88Pro TECHNO Set", "Finger Snap 2"},
    {'P',   3,  11,  29, "SC-88Pro TECHNO Set", "Scratch Push 2"},
    {'P',   3,  11,  30, "SC-88Pro TECHNO Set", "Scratch Pull 2"},
    {'P',   3,  11,  35, "SC-88Pro TECHNO Set", "Techno Kick 2"},
    {'P',   3,  11,  36, "SC-88Pro TECHNO Set", "Techno Kick 1"},
    {'P',   3,  11,  37, "SC-88Pro TECHNO Set", "TR-808 Rim Shot"},
    {'P',   3,  11,  38, "SC-88Pro TECHNO Set", "Techno Snare 1"},
    {'P',   3,  11,  39, "SC-88Pro TECHNO Set", "TR-707 Hand Clap"},
    {'P',   3,  11,  40, "SC-88Pro TECHNO Set", "Techno Snare 2"},
    {'P',   3,  11,  41, "SC-88Pro TECHNO Set", "TR-808 Low Tom 2"},
    {'P',   3,  11,  42, "SC-88Pro TECHNO Set", "TR-707 Closed Hi-Hat"},
    {'P',   3,  11,  43, "SC-88Pro TECHNO Set", "TR-808 Low Tom 1"},
    {'P',   3,  11,  44, "SC-88Pro TECHNO Set", "CR-78 Closed Hi-Hat"},
    {'P',   3,  11,  45, "SC-88Pro TECHNO Set", "TR-808 Mid Tom 2"},
    {'P',   3,  11,  46, "SC-88Pro TECHNO Set", "TR-909 Open Hi-Hat"},
    {'P',   3,  11,  47, "SC-88Pro TECHNO Set", "TR-808 Mid Tom 1"},
    {'P',   3,  11,  48, "SC-88Pro TECHNO Set", "TR-808 High Tom 2"},
    {'P',   3,  11,  49, "SC-88Pro TECHNO Set", "TR-909 Crash Cymbal"},
    {'P',   3,  11,  50, "SC-88Pro TECHNO Set", "TR-808 High Tom 1"},
    {'P',   3,  11,  51, "SC-88Pro TECHNO Set", "Ride Cymbal 1"},
    {'P',   3,  11,  52, "SC-88Pro TECHNO Set", "Reverse Cymbal"},
    {'P',   3,  11,  53, "SC-88Pro TECHNO Set", "Ride Bell"},
    {'P',   3,  11,  54, "SC-88Pro TECHNO Set", "Shake Tambourine"},
    {'P',   3,  11,  55, "SC-88Pro TECHNO Set", "Splash Cymbal"},
    {'P',   3,  11,  56, "SC-88Pro TECHNO Set", "TR-808 Cowbell"},
    {'P',   3,  11,  57, "SC-88Pro TECHNO Set", "TR-909 Crash Cymbal"},
    {'P',   3,  11,  60, "SC-88Pro TECHNO Set", "CR-78 High Bongo"},
    {'P',   3,  11,  61, "SC-88Pro TECHNO Set", "CR-78 Low Bongo"},
    {'P',   3,  11,  62, "SC-88Pro TECHNO Set", "TR-808 High Conga"},
    {'P',   3,  11,  63, "SC-88Pro TECHNO Set", "TR-808 Mute Conga"},
    {'P',   3,  11,  64, "SC-88Pro TECHNO Set", "TR-808 Low Conga"},
    {'P',   3,  11,  70, "SC-88Pro TECHNO Set", "TR-808 Maracas"},
    {'P',   3,  11,  74, "SC-88Pro TECHNO Set", "CR-78 Guiro"},
    {'P',   3,  11,  75, "SC-88Pro TECHNO Set", "TR-808 Claves"},
    {'P',   3,  11,  78, "SC-88Pro TECHNO Set", "High Hoo"},
    {'P',   3,  11,  79, "SC-88Pro TECHNO Set", "Low Hoo"},
    {'P',   3,  11,  80, "SC-88Pro TECHNO Set", "Mute Triangle"},
    {'P',   3,  11,  81, "SC-88Pro TECHNO Set", "Open Triangle"},
    {'P',   3,  11,  82, "SC-88Pro TECHNO Set", "TR-626 Shaker"},
    {'P',   3,  11,  88, "SC-88Pro TECHNO Set", "Applause 2"},
    {'P',   3,  16,  26, "SC-88Pro POWER Set", "Finger Snap 2"},
    {'P',   3,  16,  29, "SC-88Pro POWER Set", "Scratch Push 2"},
    {'P',   3,  16,  30, "SC-88Pro POWER Set", "Scratch Pull 2"},
    {'P',   3,  16,  35, "SC-88Pro POWER Set", "Power Kick 2"},
    {'P',   3,  16,  36, "SC-88Pro POWER Set", "Power Kick 1"},
    {'P',   3,  16,  37, "SC-88Pro POWER Set", "Side Stick"},
    {'P',   3,  16,  38, "SC-88Pro POWER Set", "Power Snare 1"},
    {'P',   3,  16,  39, "SC-88Pro POWER Set", "Hand Clap"},
    {'P',   3,  16,  40, "SC-88Pro POWER Set", "Power Snare 2"},
    {'P',   3,  16,  41, "SC-88Pro POWER Set", "Power Low Tom 2"},
    {'P',   3,  16,  42, "SC-88Pro POWER Set", "Closed Hi-Hat"},
    {'P',   3,  16,  43, "SC-88Pro POWER Set", "Power Low Tom 1"},
    {'P',   3,  16,  44, "SC-88Pro POWER Set", "Pedal Hi-Hat"},
    {'P',   3,  16,  45, "SC-88Pro POWER Set", "Power Mid Tom 2"},
    {'P',   3,  16,  46, "SC-88Pro POWER Set", "Open Hi-Hat"},
    {'P',   3,  16,  47, "SC-88Pro POWER Set", "Power Mid Tom 1"},
    {'P',   3,  16,  48, "SC-88Pro POWER Set", "Power High Tom 2"},
    {'P',   3,  16,  49, "SC-88Pro POWER Set", "Crash Cymbal 1"},
    {'P',   3,  16,  50, "SC-88Pro POWER Set", "Power High Tom 1"},
    {'P',   3,  16,  51, "SC-88Pro POWER Set", "Ride Cymbal 1"},
    {'P',   3,  16,  52, "SC-88Pro POWER Set", "Chinese Cymbal"},
    {'P',   3,  16,  53, "SC-88Pro POWER Set", "Ride Bell"},
    {'P',   3,  16,  54, "SC-88Pro POWER Set", "Tambourine"},
    {'P',   3,  16,  55, "SC-88Pro POWER Set", "Splash Cymbal"},
    {'P',   3,  16,  56, "SC-88Pro POWER Set", "Cowbell"},
    {'P',   3,  16,  88, "SC-88Pro POWER Set", "Applause 2"},
    {'P',   3,  24,  26, "SC-88Pro ELECTRONIC Set", "Finger Snap 2"},
    {'P',   3,  24,  29, "SC-88Pro ELECTRONIC Set", "Scratch Push 2"},
    {'P',   3,  24,  30, "SC-88Pro ELECTRONIC Set", "Scratch Pull 2"},
    {'P',   3,  24,  35, "SC-88Pro ELECTRONIC Set", "Electric Kick 2"},
    {'P',   3,  24,  36, "SC-88Pro ELECTRONIC Set", "Electric Kick 1"},
    {'P',   3,  24,  37, "SC-88Pro ELECTRONIC Set", "Side Stick"},
    {'P',   3,  24,  38, "SC-88Pro ELECTRONIC Set", "Electric Snare 1"},
    {'P',   3,  24,  39, "SC-88Pro ELECTRONIC Set", "Hand Clap"},
    {'P',   3,  24,  40, "SC-88Pro ELECTRONIC Set", "Electric Snare 2"},
    {'P',   3,  24,  41, "SC-88Pro ELECTRONIC Set", "Electric Low Tom 2"},
    {'P',   3,  24,  42, "SC-88Pro ELECTRONIC Set", "Closed Hi-Hat 2"},
    {'P',   3,  24,  43, "SC-88Pro ELECTRONIC Set", "Electric Low Tom 1"},
    {'P',   3,  24,  44, "SC-88Pro ELECTRONIC Set", "Pedal Hi-Hat"},
    {'P',   3,  24,  45, "SC-88Pro ELECTRONIC Set", "Electric Mid Tom 2"},
    {'P',   3,  24,  46, "SC-88Pro ELECTRONIC Set", "Open Hi-Hat 2"},
    {'P',   3,  24,  47, "SC-88Pro ELECTRONIC Set", "Electric Mid Tom 1"},
    {'P',   3,  24,  48, "SC-88Pro ELECTRONIC Set", "Electric High Tom 2"},
    {'P',   3,  24,  49, "SC-88Pro ELECTRONIC Set", "Crash Cymbal 1"},
    {'P',   3,  24,  50, "SC-88Pro ELECTRONIC Set", "Electric High Tom 1"},
    {'P',   3,  24,  51, "SC-88Pro ELECTRONIC Set", "Ride Cymbal 1"},
    {'P',   3,  24,  52, "SC-88Pro ELECTRONIC Set", "Reverse Cymbal"},
    {'P',   3,  24,  53, "SC-88Pro ELECTRONIC Set", "Ride Bell"},
    {'P',   3,  24,  54, "SC-88Pro ELECTRONIC Set", "Tambourine"},
    {'P',   3,  24,  55, "SC-88Pro ELECTRONIC Set", "Splash Cymbal"},
    {'P',   3,  24,  56, "SC-88Pro ELECTRONIC Set", "Cowbell"},
    {'P',   3,  24,  57, "SC-88Pro ELECTRONIC Set", "Crash Cymbal 2"},
    {'P',   3,  24,  60, "SC-88Pro ELECTRONIC Set", "High Bongo"},
    {'P',   3,  24,  61, "SC-88Pro ELECTRONIC Set", "Low Bongo"},
    {'P',   3,  24,  62, "SC-88Pro ELECTRONIC Set", "Mute High Conga"},
    {'P',   3,  24,  63, "SC-88Pro ELECTRONIC Set", "Open High Conga"},
    {'P',   3,  24,  64, "SC-88Pro ELECTRONIC Set", "Low Conga"},
    {'P',   3,  24,  70, "SC-88Pro ELECTRONIC Set", "Maracas"},
    {'P',   3,  24,  74, "SC-88Pro ELECTRONIC Set", "Long Guiro"},
    {'P',   3,  24,  75, "SC-88Pro ELECTRONIC Set", "Claves"},
    {'P',   3,  24,  78, "SC-88Pro ELECTRONIC Set", "Mute Cuica"},
    {'P',   3,  24,  79, "SC-88Pro ELECTRONIC Set", "Open Cuica"},
    {'P',   3,  24,  80, "SC-88Pro ELECTRONIC Set", "Mute Triangle"},
    {'P',   3,  24,  81, "SC-88Pro ELECTRONIC Set", "Open Triangle"},
    {'P',   3,  24,  82, "SC-88Pro ELECTRONIC Set", "Shaker"},
    {'P',   3,  24,  88, "SC-88Pro ELECTRONIC Set", "Small Club 1"},
    {'P',   3,  25,  26, "SC-88Pro TR-808 Set", "Finger Snap 2"},
    {'P',   3,  25,  29, "SC-88Pro TR-808 Set", "Scratch Push 2"},
    {'P',   3,  25,  30, "SC-88Pro TR-808 Set", "Scratch Pull 2"},
    {'P',   3,  25,  35, "SC-88Pro TR-808 Set", "TR-808 Kick 2"},
    {'P',   3,  25,  36, "SC-88Pro TR-808 Set", "TR-808 Kick 1"},
    {'P',   3,  25,  37, "SC-88Pro TR-808 Set", "TR-808 Rim Shot"},
    {'P',   3,  25,  38, "SC-88Pro TR-808 Set", "TR-808 Snare 1"},
    {'P',   3,  25,  39, "SC-88Pro TR-808 Set", "Hand Clap"},
    {'P',   3,  25,  40, "SC-88Pro TR-808 Set", "TR-808 Snare 2"},
    {'P',   3,  25,  41, "SC-88Pro TR-808 Set", "TR-808 Low Tom 2"},
    {'P',   3,  25,  42, "SC-88Pro TR-808 Set", "TR-808 Closed Hi-Hat 2"},
    {'P',   3,  25,  43, "SC-88Pro TR-808 Set", "TR-808 Low Tom 1"},
    {'P',   3,  25,  44, "SC-88Pro TR-808 Set", "TR-808 Closed Hi-Hat"},
    {'P',   3,  25,  45, "SC-88Pro TR-808 Set", "TR-808 Mid Tom 2"},
    {'P',   3,  25,  46, "SC-88Pro TR-808 Set", "TR-808 Open Hi-Hat"},
    {'P',   3,  25,  47, "SC-88Pro TR-808 Set", "TR-808 Mid Tom1"},
    {'P',   3,  25,  48, "SC-88Pro TR-808 Set", "TR-808 High Tom 2"},
    {'P',   3,  25,  49, "SC-88Pro TR-808 Set", "TR-808 Crash Cymbal"},
    {'P',   3,  25,  50, "SC-88Pro TR-808 Set", "TR-808 High Tom 1"},
    {'P',   3,  25,  51, "SC-88Pro TR-808 Set", "TR-606 Ride Cymbal"},
    {'P',   3,  25,  52, "SC-88Pro TR-808 Set", "Chinese Cymbal"},
    {'P',   3,  25,  53, "SC-88Pro TR-808 Set", "Ride Bell"},
    {'P',   3,  25,  54, "SC-88Pro TR-808 Set", "CR-78 Tambourine"},
    {'P',   3,  25,  55, "SC-88Pro TR-808 Set", "Splash Cymbal"},
    {'P',   3,  25,  56, "SC-88Pro TR-808 Set", "TR-808 Cowbell"},
    {'P',   3,  25,  57, "SC-88Pro TR-808 Set", "TR-909 Crash Cymbal"},
    {'P',   3,  25,  59, "SC-88Pro TR-808 Set", "Ride Cymbal 2"},
    {'P',   3,  25,  60, "SC-88Pro TR-808 Set", "CR-78 High Bongo"},
    {'P',   3,  25,  61, "SC-88Pro TR-808 Set", "CR-78 Low Bongo"},
    {'P',   3,  25,  62, "SC-88Pro TR-808 Set", "TR-808 Mute High Conga"},
    {'P',   3,  25,  63, "SC-88Pro TR-808 Set", "TR-808 Open High Conga"},
    {'P',   3,  25,  64, "SC-88Pro TR-808 Set", "TR-808 Open Low Conga"},
    {'P',   3,  25,  70, "SC-88Pro TR-808 Set", "TR-808 Maracas"},
    {'P',   3,  25,  74, "SC-88Pro TR-808 Set", "CR-78 Guiro"},
    {'P',   3,  25,  75, "SC-88Pro TR-808 Set", "TR-808 Claves"},
    {'P',   3,  25,  78, "SC-88Pro TR-808 Set", "High Hoo"},
    {'P',   3,  25,  79, "SC-88Pro TR-808 Set", "Low Hoo"},
    {'P',   3,  25,  80, "SC-88Pro TR-808 Set", "Mute Triangle"},
    {'P',   3,  25,  81, "SC-88Pro TR-808 Set", "Open Triangle"},
    {'P',   3,  25,  82, "SC-88Pro TR-808 Set", "TR-626 Shaker"},
    {'P',   3,  25,  88, "SC-88Pro TR-808 Set", "Small Club1"},
    {'P',   3,  26,  26, "SC-88Pro DANCE Set", "Finger Snap 2"},
    {'P',   3,  26,  29, "SC-88Pro DANCE Set", "Scratch Push 2"},
    {'P',   3,  26,  30, "SC-88Pro DANCE Set", "Scratch Pull 2"},
    {'P',   3,  26,  35, "SC-88Pro DANCE Set", "TR-909 Comp Kick1"},
    {'P',   3,  26,  36, "SC-88Pro DANCE Set", "Electric Kick 2"},
    {'P',   3,  26,  37, "SC-88Pro DANCE Set", "Side Stick"},
    {'P',   3,  26,  38, "SC-88Pro DANCE Set", "House Snare"},
    {'P',   3,  26,  39, "SC-88Pro DANCE Set", "TR-909 Hand Clap"},
    {'P',   3,  26,  40, "SC-88Pro DANCE Set", "Dance Snare 2"},
    {'P',   3,  26,  41, "SC-88Pro DANCE Set", "Electric Low Tom 2"},
    {'P',   3,  26,  42, "SC-88Pro DANCE Set", "CR-78 Closed Hi-Hat"},
    {'P',   3,  26,  43, "SC-88Pro DANCE Set", "Electric Low Tom 1"},
    {'P',   3,  26,  44, "SC-88Pro DANCE Set", "TR-808 Closed Hi-Hat 2"},
    {'P',   3,  26,  45, "SC-88Pro DANCE Set", "Electric Mid Tom 2"},
    {'P',   3,  26,  46, "SC-88Pro DANCE Set", "CR-78 Open Hi-Hat"},
    {'P',   3,  26,  47, "SC-88Pro DANCE Set", "Electric Mid Tom 1"},
    {'P',   3,  26,  48, "SC-88Pro DANCE Set", "Electric High Tom 2"},
    {'P',   3,  26,  49, "SC-88Pro DANCE Set", "TR-808 Crash Cymbal"},
    {'P',   3,  26,  50, "SC-88Pro DANCE Set", "Electric High Tom 1"},
    {'P',   3,  26,  51, "SC-88Pro DANCE Set", "TR-606 Ride Cymbal"},
    {'P',   3,  26,  52, "SC-88Pro DANCE Set", "Reverse Cymbal"},
    {'P',   3,  26,  53, "SC-88Pro DANCE Set", "Ride Bell"},
    {'P',   3,  26,  54, "SC-88Pro DANCE Set", "Shake Tambourine"},
    {'P',   3,  26,  55, "SC-88Pro DANCE Set", "Splash Cymbal"},
    {'P',   3,  26,  56, "SC-88Pro DANCE Set", "TR-808 Cowbell"},
    {'P',   3,  26,  57, "SC-88Pro DANCE Set", "Crash Cymbal 2"},
    {'P',   3,  26,  59, "SC-88Pro DANCE Set", "Ride Cymbal 2"},
    {'P',   3,  26,  60, "SC-88Pro DANCE Set", "High Bongo"},
    {'P',   3,  26,  61, "SC-88Pro DANCE Set", "Low Bongo"},
    {'P',   3,  26,  62, "SC-88Pro DANCE Set", "Mute High Conga"},
    {'P',   3,  26,  63, "SC-88Pro DANCE Set", "Open High Conga"},
    {'P',   3,  26,  64, "SC-88Pro DANCE Set", "Low Conga"},
    {'P',   3,  26,  70, "SC-88Pro DANCE Set", "Maracas"},
    {'P',   3,  26,  74, "SC-88Pro DANCE Set", "Long Guiro"},
    {'P',   3,  26,  75, "SC-88Pro DANCE Set", "Claves"},
    {'P',   3,  26,  78, "SC-88Pro DANCE Set", "High Hoo"},
    {'P',   3,  26,  79, "SC-88Pro DANCE Set", "Low Hoo"},
    {'P',   3,  26,  80, "SC-88Pro DANCE Set", "Mute Triangle"},
    {'P',   3,  26,  81, "SC-88Pro DANCE Set", "Open Triangle"},
    {'P',   3,  26,  82, "SC-88Pro DANCE Set", "TR-626 Shaker"},
    {'P',   3,  26,  88, "SC-88Pro DANCE Set", "Small Club1"},
    {'P',   3,  27,  26, "SC-88Pro CR-78 Set", "Finger Snap 2"},
    {'P',   3,  27,  29, "SC-88Pro CR-78 Set", "Scratch Push 2"},
    {'P',   3,  27,  30, "SC-88Pro CR-78 Set", "Scratch Pull 2"},
    {'P',   3,  27,  35, "SC-88Pro CR-78 Set", "CR-78 Kick 2"},
    {'P',   3,  27,  36, "SC-88Pro CR-78 Set", "CR-78 Kick 1"},
    {'P',   3,  27,  37, "SC-88Pro CR-78 Set", "CR-78 Rim Shot"},
    {'P',   3,  27,  38, "SC-88Pro CR-78 Set", "CR-78 Snare 1"},
    {'P',   3,  27,  39, "SC-88Pro CR-78 Set", "TR-707 Hand Clap"},
    {'P',   3,  27,  40, "SC-88Pro CR-78 Set", "CR-78 Snare 2"},
    {'P',   3,  27,  41, "SC-88Pro CR-78 Set", "CR-78 Low Tom 2"},
    {'P',   3,  27,  42, "SC-88Pro CR-78 Set", "CR-78 Closed Hi-Hat"},
    {'P',   3,  27,  43, "SC-88Pro CR-78 Set", "CR-78 Low Tom 1"},
    {'P',   3,  27,  44, "SC-88Pro CR-78 Set", "TR-606 Closed Hi-Hat"},
    {'P',   3,  27,  45, "SC-88Pro CR-78 Set", "CR-78 Mid Tom 2"},
    {'P',   3,  27,  46, "SC-88Pro CR-78 Set", "CR-78 Open Hi-Hat"},
    {'P',   3,  27,  47, "SC-88Pro CR-78 Set", "CR-78 Mid Tom 1"},
    {'P',   3,  27,  48, "SC-88Pro CR-78 Set", "CR-78 High Tom 2"},
    {'P',   3,  27,  49, "SC-88Pro CR-78 Set", "TR-808 Crash Cymbal"},
    {'P',   3,  27,  50, "SC-88Pro CR-78 Set", "CR-78 High Tom 1"},
    {'P',   3,  27,  51, "SC-88Pro CR-78 Set", "TR-606 Ride Cymbal"},
    {'P',   3,  27,  52, "SC-88Pro CR-78 Set", "Chinese Cymbal"},
    {'P',   3,  27,  53, "SC-88Pro CR-78 Set", "Ride Bell"},
    {'P',   3,  27,  54, "SC-88Pro CR-78 Set", "CR-78 Tambourine"},
    {'P',   3,  27,  55, "SC-88Pro CR-78 Set", "Splash Cymbal"},
    {'P',   3,  27,  56, "SC-88Pro CR-78 Set", "CR-78 Cowbell"},
    {'P',   3,  27,  57, "SC-88Pro CR-78 Set", "TR-909 Crash Cymbal"},
    {'P',   3,  27,  59, "SC-88Pro CR-78 Set", "Ride Cymbal Edge"},
    {'P',   3,  27,  60, "SC-88Pro CR-78 Set", "CR-78 High Bongo"},
    {'P',   3,  27,  61, "SC-88Pro CR-78 Set", "CR-78 Low Bongo"},
    {'P',   3,  27,  62, "SC-88Pro CR-78 Set", "TR-808 High Conga"},
    {'P',   3,  27,  63, "SC-88Pro CR-78 Set", "TR-808 Mute Conga"},
    {'P',   3,  27,  64, "SC-88Pro CR-78 Set", "TR-808 Low Conga"},
    {'P',   3,  27,  70, "SC-88Pro CR-78 Set", "CR-78 Maracas"},
    {'P',   3,  27,  74, "SC-88Pro CR-78 Set", "CR-78 Guiro"},
    {'P',   3,  27,  75, "SC-88Pro CR-78 Set", "CR-78 Claves"},
    {'P',   3,  27,  78, "SC-88Pro CR-78 Set", "High Hoo"},
    {'P',   3,  27,  79, "SC-88Pro CR-78 Set", "Low Hoo"},
    {'P',   3,  27,  80, "SC-88Pro CR-78 Set", "CR-78 Metalic Beat 1"},
    {'P',   3,  27,  81, "SC-88Pro CR-78 Set", "CR-78 Metalic Beat 2"},
    {'P',   3,  27,  82, "SC-88Pro CR-78 Set", "TR-626 Shaker"},
    {'P',   3,  27,  88, "SC-88Pro CR-78 Set", "Small Club 1"},
    {'P',   3,  28,  26, "SC-88Pro TR-606 Set", "Finger Snap 2"},
    {'P',   3,  28,  29, "SC-88Pro TR-606 Set", "Scratch Push 2"},
    {'P',   3,  28,  30, "SC-88Pro TR-606 Set", "Scratch Pull 2"},
    {'P',   3,  28,  35, "SC-88Pro TR-606 Set", "CR-78 Kick 2"},
    {'P',   3,  28,  36, "SC-88Pro TR-606 Set", "TR-606 Kick 1"},
    {'P',   3,  28,  37, "SC-88Pro TR-606 Set", "CR-78 Rim Shot"},
    {'P',   3,  28,  38, "SC-88Pro TR-606 Set", "TR-606 Snare 1"},
    {'P',   3,  28,  39, "SC-88Pro TR-606 Set", "TR-707 Hand Clap"},
    {'P',   3,  28,  40, "SC-88Pro TR-606 Set", "TR-606 Snare 2"},
    {'P',   3,  28,  41, "SC-88Pro TR-606 Set", "TR-606 Low Tom 2"},
    {'P',   3,  28,  42, "SC-88Pro TR-606 Set", "TR-606 Closed Hi-Hat"},
    {'P',   3,  28,  43, "SC-88Pro TR-606 Set", "TR-606 Low Tom 1"},
    {'P',   3,  28,  44, "SC-88Pro TR-606 Set", "TR-606 Closed Hi-Hat"},
    {'P',   3,  28,  45, "SC-88Pro TR-606 Set", "TR-606 Mid Tom 2"},
    {'P',   3,  28,  46, "SC-88Pro TR-606 Set", "TR-606 Open Hi-Hat"},
    {'P',   3,  28,  47, "SC-88Pro TR-606 Set", "TR-606 Mid Tom 1"},
    {'P',   3,  28,  48, "SC-88Pro TR-606 Set", "TR-606 High Tom 2"},
    {'P',   3,  28,  49, "SC-88Pro TR-606 Set", "TR-808 Crash Cymbal"},
    {'P',   3,  28,  50, "SC-88Pro TR-606 Set", "TR-606 High Tom 1"},
    {'P',   3,  28,  51, "SC-88Pro TR-606 Set", "TR-606 Ride Cymbal"},
    {'P',   3,  28,  52, "SC-88Pro TR-606 Set", "Chinese Cymbal"},
    {'P',   3,  28,  53, "SC-88Pro TR-606 Set", "Ride Bell"},
    {'P',   3,  28,  54, "SC-88Pro TR-606 Set", "CR-78 Tambourine"},
    {'P',   3,  28,  55, "SC-88Pro TR-606 Set", "Splash Cymbal"},
    {'P',   3,  28,  56, "SC-88Pro TR-606 Set", "CR-78 Cowbell"},
    {'P',   3,  28,  57, "SC-88Pro TR-606 Set", "TR-909 Crash Cymbal"},
    {'P',   3,  28,  59, "SC-88Pro TR-606 Set", "Ride Cymbal Edge"},
    {'P',   3,  28,  60, "SC-88Pro TR-606 Set", "CR-78 High Bongo"},
    {'P',   3,  28,  61, "SC-88Pro TR-606 Set", "CR-78 Low Bongo"},
    {'P',   3,  28,  62, "SC-88Pro TR-606 Set", "TR-808 High Conga"},
    {'P',   3,  28,  63, "SC-88Pro TR-606 Set", "TR-808 Mute Conga"},
    {'P',   3,  28,  64, "SC-88Pro TR-606 Set", "TR-808 Low Conga"},
    {'P',   3,  28,  70, "SC-88Pro TR-606 Set", "CR-78 Maracas"},
    {'P',   3,  28,  74, "SC-88Pro TR-606 Set", "CR-78 Guiro"},
    {'P',   3,  28,  75, "SC-88Pro TR-606 Set", "CR-78 Claves"},
    {'P',   3,  28,  78, "SC-88Pro TR-606 Set", "High Hoo"},
    {'P',   3,  28,  79, "SC-88Pro TR-606 Set", "Low Hoo"},
    {'P',   3,  28,  80, "SC-88Pro TR-606 Set", "CR-78 Metalic Beat"},
    {'P',   3,  28,  81, "SC-88Pro TR-606 Set", "CR-78 Metalic Beat"},
    {'P',   3,  28,  82, "SC-88Pro TR-606 Set", "TR-626 Shaker"},
    {'P',   3,  28,  88, "SC-88Pro TR-606 Set", "Small Club 1"},
    {'P',   3,  29,  26, "SC-88Pro TR-707 Set", "Finger Snap 2"},
    {'P',   3,  29,  29, "SC-88Pro TR-707 Set", "Scratch Push 2"},
    {'P',   3,  29,  30, "SC-88Pro TR-707 Set", "Scratch Pull 2"},
    {'P',   3,  29,  35, "SC-88Pro TR-707 Set", "TR-707 Kick 2"},
    {'P',   3,  29,  36, "SC-88Pro TR-707 Set", "TR-707 Kick 1"},
    {'P',   3,  29,  37, "SC-88Pro TR-707 Set", "TR-707 Rim Shot"},
    {'P',   3,  29,  38, "SC-88Pro TR-707 Set", "TR-707 Snare 1"},
    {'P',   3,  29,  39, "SC-88Pro TR-707 Set", "TR-707 Hand Clap"},
    {'P',   3,  29,  40, "SC-88Pro TR-707 Set", "TR-707 Snare 2"},
    {'P',   3,  29,  41, "SC-88Pro TR-707 Set", "TR-707 Low Tom 2"},
    {'P',   3,  29,  42, "SC-88Pro TR-707 Set", "TR-707 Closed Hi-Hat"},
    {'P',   3,  29,  43, "SC-88Pro TR-707 Set", "TR-707 Low Tom 1"},
    {'P',   3,  29,  44, "SC-88Pro TR-707 Set", "TR-707 Closed Hi-Hat"},
    {'P',   3,  29,  45, "SC-88Pro TR-707 Set", "TR-707 Mid Tom 2"},
    {'P',   3,  29,  46, "SC-88Pro TR-707 Set", "TR-707 Open Hi-Hat"},
    {'P',   3,  29,  47, "SC-88Pro TR-707 Set", "TR-707 Mid Tom 1"},
    {'P',   3,  29,  48, "SC-88Pro TR-707 Set", "TR-707 High Tom 2"},
    {'P',   3,  29,  49, "SC-88Pro TR-707 Set", "TR-909 Crash Cymbal"},
    {'P',   3,  29,  50, "SC-88Pro TR-707 Set", "TR-707 High Tom 1"},
    {'P',   3,  29,  51, "SC-88Pro TR-707 Set", "TR-909 Ride Cymbal"},
    {'P',   3,  29,  52, "SC-88Pro TR-707 Set", "Chinese Cymbal"},
    {'P',   3,  29,  53, "SC-88Pro TR-707 Set", "Ride Bell"},
    {'P',   3,  29,  54, "SC-88Pro TR-707 Set", "Tambourine 2"},
    {'P',   3,  29,  55, "SC-88Pro TR-707 Set", "Splash Cymbal"},
    {'P',   3,  29,  56, "SC-88Pro TR-707 Set", "TR-808 Cowbell"},
    {'P',   3,  29,  57, "SC-88Pro TR-707 Set", "Crash Cymbal 2"},
    {'P',   3,  29,  59, "SC-88Pro TR-707 Set", "Ride Cymbal Edge"},
    {'P',   3,  29,  60, "SC-88Pro TR-707 Set", "High Bongo"},
    {'P',   3,  29,  61, "SC-88Pro TR-707 Set", "Low Bongo"},
    {'P',   3,  29,  62, "SC-88Pro TR-707 Set", "Mute High Conga"},
    {'P',   3,  29,  63, "SC-88Pro TR-707 Set", "Open High Conga"},
    {'P',   3,  29,  64, "SC-88Pro TR-707 Set", "Low Conga"},
    {'P',   3,  29,  70, "SC-88Pro TR-707 Set", "TR-808 Maracas"},
    {'P',   3,  29,  74, "SC-88Pro TR-707 Set", "Long Guiro"},
    {'P',   3,  29,  75, "SC-88Pro TR-707 Set", "Claves"},
    {'P',   3,  29,  78, "SC-88Pro TR-707 Set", "High Hoo"},
    {'P',   3,  29,  79, "SC-88Pro TR-707 Set", "Low Hoo"},
    {'P',   3,  29,  80, "SC-88Pro TR-707 Set", "Mute Triangle"},
    {'P',   3,  29,  81, "SC-88Pro TR-707 Set", "Open Triangle"},
    {'P',   3,  29,  82, "SC-88Pro TR-707 Set", "TR-626 Shaker"},
    {'P',   3,  29,  88, "SC-88Pro TR-707 Set", "Small Club 1"},
    {'P',   3,  30,  26, "SC-88Pro TR-909 Set", "Finger Snap 2"},
    {'P',   3,  30,  29, "SC-88Pro TR-909 Set", "Scratch Push 2"},
    {'P',   3,  30,  30, "SC-88Pro TR-909 Set", "Scratch Pull 2"},
    {'P',   3,  30,  35, "SC-88Pro TR-909 Set", "TR-909 Kick 2"},
    {'P',   3,  30,  36, "SC-88Pro TR-909 Set", "TR-909 Kick 1"},
    {'P',   3,  30,  37, "SC-88Pro TR-909 Set", "TR-909 Rim"},
    {'P',   3,  30,  38, "SC-88Pro TR-909 Set", "TR-909 Snare 1"},
    {'P',   3,  30,  39, "SC-88Pro TR-909 Set", "TR-909 Hand Clap"},
    {'P',   3,  30,  40, "SC-88Pro TR-909 Set", "TR-909 Snare 2"},
    {'P',   3,  30,  41, "SC-88Pro TR-909 Set", "TR-909 Low Tom 2"},
    {'P',   3,  30,  42, "SC-88Pro TR-909 Set", "TR-707 Closed Hi-Hat"},
    {'P',   3,  30,  43, "SC-88Pro TR-909 Set", "TR-909 Low Tom 1"},
    {'P',   3,  30,  44, "SC-88Pro TR-909 Set", "TR-707 Closed Hi-Hat"},
    {'P',   3,  30,  45, "SC-88Pro TR-909 Set", "TR-909 Mid Tom 2"},
    {'P',   3,  30,  46, "SC-88Pro TR-909 Set", "TR-909 Open Hi-Hat"},
    {'P',   3,  30,  47, "SC-88Pro TR-909 Set", "TR-909 Mid Tom 1"},
    {'P',   3,  30,  48, "SC-88Pro TR-909 Set", "TR-909 High Tom 2"},
    {'P',   3,  30,  49, "SC-88Pro TR-909 Set", "TR-909 Crash Cymbal"},
    {'P',   3,  30,  50, "SC-88Pro TR-909 Set", "TR-909 High Tom 1"},
    {'P',   3,  30,  51, "SC-88Pro TR-909 Set", "TR-909 Ride Cymbal"},
    {'P',   3,  30,  52, "SC-88Pro TR-909 Set", "Reverse Cymbal"},
    {'P',   3,  30,  53, "SC-88Pro TR-909 Set", "Ride Bell"},
    {'P',   3,  30,  54, "SC-88Pro TR-909 Set", "Tambourine 2"},
    {'P',   3,  30,  55, "SC-88Pro TR-909 Set", "Splash Cymbal"},
    {'P',   3,  30,  56, "SC-88Pro TR-909 Set", "TR-808 Cowbell"},
    {'P',   3,  30,  57, "SC-88Pro TR-909 Set", "Crash Cymbal 2"},
    {'P',   3,  30,  59, "SC-88Pro TR-909 Set", "Ride Cymbal Edge"},
    {'P',   3,  30,  60, "SC-88Pro TR-909 Set", "High Bongo"},
    {'P',   3,  30,  61, "SC-88Pro TR-909 Set", "Low Bongo"},
    {'P',   3,  30,  62, "SC-88Pro TR-909 Set", "Mute High Conga"},
    {'P',   3,  30,  63, "SC-88Pro TR-909 Set", "Open High Conga"},
    {'P',   3,  30,  64, "SC-88Pro TR-909 Set", "Low Conga"},
    {'P',   3,  30,  70, "SC-88Pro TR-909 Set", "TR-808 Maracas"},
    {'P',   3,  30,  74, "SC-88Pro TR-909 Set", "CR-78 Guiro"},
    {'P',   3,  30,  75, "SC-88Pro TR-909 Set", "TR-808 Claves"},
    {'P',   3,  30,  78, "SC-88Pro TR-909 Set", "High Hoo"},
    {'P',   3,  30,  79, "SC-88Pro TR-909 Set", "Low Hoo"},
    {'P',   3,  30,  80, "SC-88Pro TR-909 Set", "Mute Triangle"},
    {'P',   3,  30,  81, "SC-88Pro TR-909 Set", "Open Triangle"},
    {'P',   3,  30,  82, "SC-88Pro TR-909 Set", "TR-626 Shaker"},
    {'P',   3,  30,  88, "SC-88Pro TR-909 Set", "Applause 2"},
    {'P',   3,  32,   0, "SC-88Pro JAZZ Set", "[88] Standard 1 Kick 1"},
    {'P',   3,  32,   1, "SC-88Pro JAZZ Set", "[88] Standard 1 Kick 2"},
    {'P',   3,  32,   2, "SC-88Pro JAZZ Set", "[88] Standard 2 Kick 1"},
    {'P',   3,  32,   3, "SC-88Pro JAZZ Set", "[88] Standard 2 Kick 2"},
    {'P',   3,  32,   4, "SC-88Pro JAZZ Set", "[55] Kick Drum 1"},
    {'P',   3,  32,   5, "SC-88Pro JAZZ Set", "[55] Kick Drum 2"},
    {'P',   3,  32,   6, "SC-88Pro JAZZ Set", "[88] Jazz Kick 1"},
    {'P',   3,  32,   7, "SC-88Pro JAZZ Set", "[88] Jazz Kick 2"},
    {'P',   3,  32,   8, "SC-88Pro JAZZ Set", "[88] Room Kick 1"},
    {'P',   3,  32,   9, "SC-88Pro JAZZ Set", "[88] Room Kick 2"},
    {'P',   3,  32,  10, "SC-88Pro JAZZ Set", "[88] Power Kick 1"},
    {'P',   3,  32,  11, "SC-88Pro JAZZ Set", "[88] Power Kick 2"},
    {'P',   3,  32,  12, "SC-88Pro JAZZ Set", "[88] Electric Kick 2"},
    {'P',   3,  32,  13, "SC-88Pro JAZZ Set", "[88] Electric Kick 1"},
    {'P',   3,  32,  14, "SC-88Pro JAZZ Set", "[88] TR-808 Kick"},
    {'P',   3,  32,  15, "SC-88Pro JAZZ Set", "[88] TR-909 Kick"},
    {'P',   3,  32,  16, "SC-88Pro JAZZ Set", "[88] Dance Kick"},
    {'P',   3,  32,  17, "SC-88Pro JAZZ Set", "Voice One"},
    {'P',   3,  32,  18, "SC-88Pro JAZZ Set", "Voice Two"},
    {'P',   3,  32,  19, "SC-88Pro JAZZ Set", "Voice Three"},
    {'P',   3,  32,  22, "SC-88Pro JAZZ Set", "MC-500 Beep 1"},
    {'P',   3,  32,  23, "SC-88Pro JAZZ Set", "MC-500 Beep 2"},
    {'P',   3,  32,  24, "SC-88Pro JAZZ Set", "Concert SD"},
    {'P',   3,  32,  25, "SC-88Pro JAZZ Set", "Snare Roll"},
    {'P',   3,  32,  26, "SC-88Pro JAZZ Set", "Finger Snap 2"},
    {'P',   3,  32,  27, "SC-88Pro JAZZ Set", "High Q"},
    {'P',   3,  32,  28, "SC-88Pro JAZZ Set", "Slap"},
    {'P',   3,  32,  29, "SC-88Pro JAZZ Set", "Scratch Push"},
    {'P',   3,  32,  30, "SC-88Pro JAZZ Set", "Scratch Pull"},
    {'P',   3,  32,  31, "SC-88Pro JAZZ Set", "Sticks"},
    {'P',   3,  32,  32, "SC-88Pro JAZZ Set", "Square Click"},
    {'P',   3,  32,  33, "SC-88Pro JAZZ Set", "Metronome Click"},
    {'P',   3,  32,  34, "SC-88Pro JAZZ Set", "Metronome Bell"},
    {'P',   3,  32,  35, "SC-88Pro JAZZ Set", "Jazz Kick 2"},
    {'P',   3,  32,  36, "SC-88Pro JAZZ Set", "Jazz Kick 1"},
    {'P',   3,  32,  37, "SC-88Pro JAZZ Set", "Sticks"},
    {'P',   3,  32,  38, "SC-88Pro JAZZ Set", "Jazz Snare 1"},
    {'P',   3,  32,  39, "SC-88Pro JAZZ Set", "Hand Clap 2"},
    {'P',   3,  32,  40, "SC-88Pro JAZZ Set", "Jazz Snare 2"},
    {'P',   3,  32,  41, "SC-88Pro JAZZ Set", "Low Tom 2"},
    {'P',   3,  32,  42, "SC-88Pro JAZZ Set", "Closed Hi-Hat 2"},
    {'P',   3,  32,  43, "SC-88Pro JAZZ Set", "Low Tom 1"},
    {'P',   3,  32,  44, "SC-88Pro JAZZ Set", "Pedal Hi-Hat"},
    {'P',   3,  32,  45, "SC-88Pro JAZZ Set", "Mid Tom 2"},
    {'P',   3,  32,  46, "SC-88Pro JAZZ Set", "Open Hi-Hat 2"},
    {'P',   3,  32,  47, "SC-88Pro JAZZ Set", "Mid Tom 1"},
    {'P',   3,  32,  48, "SC-88Pro JAZZ Set", "High Tom 2"},
    {'P',   3,  32,  49, "SC-88Pro JAZZ Set", "Crash Cymbal 1"},
    {'P',   3,  32,  50, "SC-88Pro JAZZ Set", "High Tom 1"},
    {'P',   3,  32,  51, "SC-88Pro JAZZ Set", "Ride Cymbal Inner"},
    {'P',   3,  32,  52, "SC-88Pro JAZZ Set", "Chinese Cymbal"},
    {'P',   3,  32,  53, "SC-88Pro JAZZ Set", "Ride Bell"},
    {'P',   3,  32,  54, "SC-88Pro JAZZ Set", "Tambourine"},
    {'P',   3,  32,  55, "SC-88Pro JAZZ Set", "Splash Cymbal"},
    {'P',   3,  32,  56, "SC-88Pro JAZZ Set", "Cowbell"},
    {'P',   3,  32,  57, "SC-88Pro JAZZ Set", "Crash Cymbal 2"},
    {'P',   3,  32,  58, "SC-88Pro JAZZ Set", "Vibra-slap"},
    {'P',   3,  32,  59, "SC-88Pro JAZZ Set", "Ride Cymbal Edge"},
    {'P',   3,  32,  60, "SC-88Pro JAZZ Set", "High Bongo"},
    {'P',   3,  32,  61, "SC-88Pro JAZZ Set", "Low Bongo"},
    {'P',   3,  32,  62, "SC-88Pro JAZZ Set", "Mute High Conga"},
    {'P',   3,  32,  63, "SC-88Pro JAZZ Set", "Open High Conga"},
    {'P',   3,  32,  64, "SC-88Pro JAZZ Set", "Low Conga"},
    {'P',   3,  32,  65, "SC-88Pro JAZZ Set", "High Timbale"},
    {'P',   3,  32,  66, "SC-88Pro JAZZ Set", "Low Timbale"},
    {'P',   3,  32,  67, "SC-88Pro JAZZ Set", "High Agogo"},
    {'P',   3,  32,  68, "SC-88Pro JAZZ Set", "Low Agogo"},
    {'P',   3,  32,  69, "SC-88Pro JAZZ Set", "Cabasa"},
    {'P',   3,  32,  70, "SC-88Pro JAZZ Set", "Maracas"},
    {'P',   3,  32,  71, "SC-88Pro JAZZ Set", "Short High Whisle"},
    {'P',   3,  32,  72, "SC-88Pro JAZZ Set", "Long Low Whisle"},
    {'P',   3,  32,  73, "SC-88Pro JAZZ Set", "Short Guiro"},
    {'P',   3,  32,  74, "SC-88Pro JAZZ Set", "Long Guiro"},
    {'P',   3,  32,  75, "SC-88Pro JAZZ Set", "Claves"},
    {'P',   3,  32,  76, "SC-88Pro JAZZ Set", "High Wood Block"},
    {'P',   3,  32,  77, "SC-88Pro JAZZ Set", "Low Wood Block"},
    {'P',   3,  32,  78, "SC-88Pro JAZZ Set", "Mute Cuica"},
    {'P',   3,  32,  79, "SC-88Pro JAZZ Set", "Open Cuica"},
    {'P',   3,  32,  80, "SC-88Pro JAZZ Set", "Mute Triangle"},
    {'P',   3,  32,  81, "SC-88Pro JAZZ Set", "Open Triangle"},
    {'P',   3,  32,  82, "SC-88Pro JAZZ Set", "Shaker"},
    {'P',   3,  32,  83, "SC-88Pro JAZZ Set", "Jingle Bell"},
    {'P',   3,  32,  84, "SC-88Pro JAZZ Set", "Bell Tree"},
    {'P',   3,  32,  85, "SC-88Pro JAZZ Set", "Castanets"},
    {'P',   3,  32,  86, "SC-88Pro JAZZ Set", "Mute Surdo"},
    {'P',   3,  32,  87, "SC-88Pro JAZZ Set", "Open Surdo"},
    {'P',   3,  32,  88, "SC-88Pro JAZZ Set", "Applause"},
    {'P',   3,  32, 100, "SC-88Pro JAZZ Set", "[88] Brush Tap 1"},
    {'P',   3,  32, 101, "SC-88Pro JAZZ Set", "[88] Brush Tap 2"},
    {'P',   3,  32, 102, "SC-88Pro JAZZ Set", "[88] Brush Slap 1"},
    {'P',   3,  32, 103, "SC-88Pro JAZZ Set", "[88] Brush Slap 2"},
    {'P',   3,  32, 104, "SC-88Pro JAZZ Set", "[88] Brush Slap 3"},
    {'P',   3,  32, 105, "SC-88Pro JAZZ Set", "[88] Brush Swirl 1"},
    {'P',   3,  32, 106, "SC-88Pro JAZZ Set", "[88] Brush Swirl 2"},
    {'P',   3,  32, 107, "SC-88Pro JAZZ Set", "[88] Brush Long Swirl"},
    {'P',   3,  32, 108, "SC-88Pro JAZZ Set", "[88] Jazz Snare 1"},
    {'P',   3,  32, 109, "SC-88Pro JAZZ Set", "[88] Jazz Snare 2"},
    {'P',   3,  32, 110, "SC-88Pro JAZZ Set", "[88] Standard 1 Snare 1"},
    {'P',   3,  32, 111, "SC-88Pro JAZZ Set", "[88] Standard 1 Snare 2"},
    {'P',   3,  32, 112, "SC-88Pro JAZZ Set", "[88] Standard 2 Snare 1"},
    {'P',   3,  32, 113, "SC-88Pro JAZZ Set", "[88] Standard 2 Snare 2"},
    {'P',   3,  32, 114, "SC-88Pro JAZZ Set", "[55] Snare Drum 2"},
    {'P',   3,  32, 115, "SC-88Pro JAZZ Set", "Standard 1 Snare 1"},
    {'P',   3,  32, 116, "SC-88Pro JAZZ Set", "Standard 1 Snare 2"},
    {'P',   3,  32, 117, "SC-88Pro JAZZ Set", "Standard 1 Snare 3"},
    {'P',   3,  32, 118, "SC-88Pro JAZZ Set", "[88] Room Snare 1"},
    {'P',   3,  32, 119, "SC-88Pro JAZZ Set", "[88] Room Snare 2"},
    {'P',   3,  32, 120, "SC-88Pro JAZZ Set", "[88] Power Snare 1"},
    {'P',   3,  32, 121, "SC-88Pro JAZZ Set", "[88] Power Snare 2"},
    {'P',   3,  32, 122, "SC-88Pro JAZZ Set", "[88] Gated Snare"},
    {'P',   3,  32, 123, "SC-88Pro JAZZ Set", "[88] Dance Snare 1"},
    {'P',   3,  32, 124, "SC-88Pro JAZZ Set", "[88] Dance Snare 2"},
    {'P',   3,  32, 125, "SC-88Pro JAZZ Set", "[88] Disco Snare"},
    {'P',   3,  32, 126, "SC-88Pro JAZZ Set", "[88] Electric Snare 2"},
    {'P',   3,  32, 127, "SC-88Pro JAZZ Set", "[88] Electric Snare 3"},
    {'P',   3,  40,  26, "SC-88Pro BRUSH Set", "Finger Snap 2"},
    {'P',   3,  40,  29, "SC-88Pro BRUSH Set", "Scratch Push"},
    {'P',   3,  40,  30, "SC-88Pro BRUSH Set", "Scratch Pull"},
    {'P',   3,  40,  35, "SC-88Pro BRUSH Set", "Jazz Kick 2"},
    {'P',   3,  40,  36, "SC-88Pro BRUSH Set", "Jazz Kick 1"},
    {'P',   3,  40,  37, "SC-88Pro BRUSH Set", "Sticks"},
    {'P',   3,  40,  38, "SC-88Pro BRUSH Set", "Brush Tap 1"},
    {'P',   3,  40,  39, "SC-88Pro BRUSH Set", "Brush Slap 1"},
    {'P',   3,  40,  40, "SC-88Pro BRUSH Set", "Brush Swirl 1"},
    {'P',   3,  40,  41, "SC-88Pro BRUSH Set", "Brush Low Tom 2"},
    {'P',   3,  40,  42, "SC-88Pro BRUSH Set", "Brush Closed Hi-Hat"},
    {'P',   3,  40,  43, "SC-88Pro BRUSH Set", "Brush Low Tom 1"},
    {'P',   3,  40,  44, "SC-88Pro BRUSH Set", "Pedal Hi-Hat"},
    {'P',   3,  40,  45, "SC-88Pro BRUSH Set", "Brush Mid Tom 2"},
    {'P',   3,  40,  46, "SC-88Pro BRUSH Set", "Brush Open Hi-Hat"},
    {'P',   3,  40,  47, "SC-88Pro BRUSH Set", "Brush Mid Tom 1"},
    {'P',   3,  40,  48, "SC-88Pro BRUSH Set", "Brush High Tom 2"},
    {'P',   3,  40,  49, "SC-88Pro BRUSH Set", "Brush Crash Cymbal"},
    {'P',   3,  40,  50, "SC-88Pro BRUSH Set", "Brush High Tom 1"},
    {'P',   3,  40,  51, "SC-88Pro BRUSH Set", "Ride Cymbal Inner"},
    {'P',   3,  40,  52, "SC-88Pro BRUSH Set", "Chinese Cymbal"},
    {'P',   3,  40,  53, "SC-88Pro BRUSH Set", "Brush Ride Bell"},
    {'P',   3,  40,  54, "SC-88Pro BRUSH Set", "Tambourine"},
    {'P',   3,  40,  55, "SC-88Pro BRUSH Set", "Splash Cymbal"},
    {'P',   3,  40,  56, "SC-88Pro BRUSH Set", "Cowbell"},
    {'P',   3,  40,  59, "SC-88Pro BRUSH Set", "Ride Cymbal Edge"},
    {'P',   3,  40,  88, "SC-88Pro BRUSH Set", "Applause"},
    {'P',   3,  48,   0, "SC-88Pro ORCHESTRA Set", "[88] Standard 1 Kick 1"},
    {'P',   3,  48,   1, "SC-88Pro ORCHESTRA Set", "[88] Standard 1 Kick 2"},
    {'P',   3,  48,   2, "SC-88Pro ORCHESTRA Set", "[88] Standard 2 Kick 1"},
    {'P',   3,  48,   3, "SC-88Pro ORCHESTRA Set", "[88] Standard 2 Kick 2"},
    {'P',   3,  48,   4, "SC-88Pro ORCHESTRA Set", "[55] Kick Drum 1"},
    {'P',   3,  48,   5, "SC-88Pro ORCHESTRA Set", "[55] Kick Drum 2"},
    {'P',   3,  48,   6, "SC-88Pro ORCHESTRA Set", "[88] Jazz Kick 1"},
    {'P',   3,  48,   7, "SC-88Pro ORCHESTRA Set", "[88] Jazz Kick 2"},
    {'P',   3,  48,   8, "SC-88Pro ORCHESTRA Set", "[88] Room Kick 1"},
    {'P',   3,  48,   9, "SC-88Pro ORCHESTRA Set", "[88] Room Kick 2"},
    {'P',   3,  48,  10, "SC-88Pro ORCHESTRA Set", "[88] Power Kick 1"},
    {'P',   3,  48,  11, "SC-88Pro ORCHESTRA Set", "[88] Power Kick 2"},
    {'P',   3,  48,  12, "SC-88Pro ORCHESTRA Set", "[88] Electric Kick 2"},
    {'P',   3,  48,  13, "SC-88Pro ORCHESTRA Set", "[88] Electric Kick 1"},
    {'P',   3,  48,  14, "SC-88Pro ORCHESTRA Set", "[88] TR-808 Kick"},
    {'P',   3,  48,  15, "SC-88Pro ORCHESTRA Set", "[88] TR-909 Kick"},
    {'P',   3,  48,  16, "SC-88Pro ORCHESTRA Set", "[88] Dance Kick"},
    {'P',   3,  48,  17, "SC-88Pro ORCHESTRA Set", "Voice One"},
    {'P',   3,  48,  18, "SC-88Pro ORCHESTRA Set", "Voice Two"},
    {'P',   3,  48,  19, "SC-88Pro ORCHESTRA Set", "Voice Three"},
    {'P',   3,  48,  22, "SC-88Pro ORCHESTRA Set", "MC-500 Beep 1"},
    {'P',   3,  48,  23, "SC-88Pro ORCHESTRA Set", "MC-500 Beep 2"},
    {'P',   3,  48,  24, "SC-88Pro ORCHESTRA Set", "Concert SD"},
    {'P',   3,  48,  25, "SC-88Pro ORCHESTRA Set", "Snare Roll"},
    {'P',   3,  48,  26, "SC-88Pro ORCHESTRA Set", "Finger Snap"},
    {'P',   3,  48,  27, "SC-88Pro ORCHESTRA Set", "Closed Hi-Hat 2"},
    {'P',   3,  48,  28, "SC-88Pro ORCHESTRA Set", "Pedal Hi-Hat"},
    {'P',   3,  48,  29, "SC-88Pro ORCHESTRA Set", "Open Hi-Hat 2"},
    {'P',   3,  48,  30, "SC-88Pro ORCHESTRA Set", "Ride Cymbal1"},
    {'P',   3,  48,  31, "SC-88Pro ORCHESTRA Set", "Sticks"},
    {'P',   3,  48,  32, "SC-88Pro ORCHESTRA Set", "Square Click"},
    {'P',   3,  48,  33, "SC-88Pro ORCHESTRA Set", "Metronome Click"},
    {'P',   3,  48,  34, "SC-88Pro ORCHESTRA Set", "Metronome Bell"},
    {'P',   3,  48,  35, "SC-88Pro ORCHESTRA Set", "Jazz Kick 1"},
    {'P',   3,  48,  36, "SC-88Pro ORCHESTRA Set", "Concert BD 1"},
    {'P',   3,  48,  37, "SC-88Pro ORCHESTRA Set", "Side Stick"},
    {'P',   3,  48,  38, "SC-88Pro ORCHESTRA Set", "Concert SD"},
    {'P',   3,  48,  39, "SC-88Pro ORCHESTRA Set", "Castanets"},
    {'P',   3,  48,  40, "SC-88Pro ORCHESTRA Set", "Concert SD"},
    {'P',   3,  48,  41, "SC-88Pro ORCHESTRA Set", "Timpani F"},
    {'P',   3,  48,  42, "SC-88Pro ORCHESTRA Set", "Timpani F#"},
    {'P',   3,  48,  43, "SC-88Pro ORCHESTRA Set", "Timpani G"},
    {'P',   3,  48,  44, "SC-88Pro ORCHESTRA Set", "Timpani G#"},
    {'P',   3,  48,  45, "SC-88Pro ORCHESTRA Set", "Timpani A"},
    {'P',   3,  48,  46, "SC-88Pro ORCHESTRA Set", "Timpani A#"},
    {'P',   3,  48,  47, "SC-88Pro ORCHESTRA Set", "Timpani B"},
    {'P',   3,  48,  48, "SC-88Pro ORCHESTRA Set", "Timpani c"},
    {'P',   3,  48,  49, "SC-88Pro ORCHESTRA Set", "Timpani c#"},
    {'P',   3,  48,  50, "SC-88Pro ORCHESTRA Set", "Timpani d"},
    {'P',   3,  48,  51, "SC-88Pro ORCHESTRA Set", "Timpani d#"},
    {'P',   3,  48,  52, "SC-88Pro ORCHESTRA Set", "Timpani e"},
    {'P',   3,  48,  53, "SC-88Pro ORCHESTRA Set", "Timpani f"},
    {'P',   3,  48,  54, "SC-88Pro ORCHESTRA Set", "Tambourine"},
    {'P',   3,  48,  55, "SC-88Pro ORCHESTRA Set", "Splash Cymbal"},
    {'P',   3,  48,  56, "SC-88Pro ORCHESTRA Set", "Cowbell"},
    {'P',   3,  48,  57, "SC-88Pro ORCHESTRA Set", "Concert Cymbal 2"},
    {'P',   3,  48,  58, "SC-88Pro ORCHESTRA Set", "Vibra-slap"},
    {'P',   3,  48,  59, "SC-88Pro ORCHESTRA Set", "Concert Cymbal 1"},
    {'P',   3,  48,  60, "SC-88Pro ORCHESTRA Set", "High Bongo"},
    {'P',   3,  48,  61, "SC-88Pro ORCHESTRA Set", "Low Bongo"},
    {'P',   3,  48,  62, "SC-88Pro ORCHESTRA Set", "Mute High Conga"},
    {'P',   3,  48,  63, "SC-88Pro ORCHESTRA Set", "Open High Conga"},
    {'P',   3,  48,  64, "SC-88Pro ORCHESTRA Set", "Low Conga"},
    {'P',   3,  48,  65, "SC-88Pro ORCHESTRA Set", "High Timbale"},
    {'P',   3,  48,  66, "SC-88Pro ORCHESTRA Set", "Low Timbale"},
    {'P',   3,  48,  67, "SC-88Pro ORCHESTRA Set", "High Agogo"},
    {'P',   3,  48,  68, "SC-88Pro ORCHESTRA Set", "Low Agogo"},
    {'P',   3,  48,  69, "SC-88Pro ORCHESTRA Set", "Cabasa"},
    {'P',   3,  48,  70, "SC-88Pro ORCHESTRA Set", "Maracas"},
    {'P',   3,  48,  71, "SC-88Pro ORCHESTRA Set", "Short High Whisle"},
    {'P',   3,  48,  72, "SC-88Pro ORCHESTRA Set", "Long Low Whisle"},
    {'P',   3,  48,  73, "SC-88Pro ORCHESTRA Set", "Short Guiro"},
    {'P',   3,  48,  74, "SC-88Pro ORCHESTRA Set", "Long Guiro"},
    {'P',   3,  48,  75, "SC-88Pro ORCHESTRA Set", "Claves"},
    {'P',   3,  48,  76, "SC-88Pro ORCHESTRA Set", "High Wood Block"},
    {'P',   3,  48,  77, "SC-88Pro ORCHESTRA Set", "Low Wood Block"},
    {'P',   3,  48,  78, "SC-88Pro ORCHESTRA Set", "Mute Cuica"},
    {'P',   3,  48,  79, "SC-88Pro ORCHESTRA Set", "Open Cuica"},
    {'P',   3,  48,  80, "SC-88Pro ORCHESTRA Set", "Mute Triangle"},
    {'P',   3,  48,  81, "SC-88Pro ORCHESTRA Set", "Open Triangle"},
    {'P',   3,  48,  82, "SC-88Pro ORCHESTRA Set", "Shaker"},
    {'P',   3,  48,  83, "SC-88Pro ORCHESTRA Set", "Jingle Bell"},
    {'P',   3,  48,  84, "SC-88Pro ORCHESTRA Set", "Bell Tree"},
    {'P',   3,  48,  85, "SC-88Pro ORCHESTRA Set", "Castanets"},
    {'P',   3,  48,  86, "SC-88Pro ORCHESTRA Set", "Mute Surdo"},
    {'P',   3,  48,  87, "SC-88Pro ORCHESTRA Set", "Open Surdo"},
    {'P',   3,  48,  88, "SC-88Pro ORCHESTRA Set", "Applause"},
    {'P',   3,  48,  97, "SC-88Pro ORCHESTRA Set", "Applause 2"},
    {'P',   3,  48,  98, "SC-88Pro ORCHESTRA Set", "Small Club 1"},
    {'P',   3,  48,  99, "SC-88Pro ORCHESTRA Set", "[55] Timpani D#"},
    {'P',   3,  48, 100, "SC-88Pro ORCHESTRA Set", "[55] Timpani E"},
    {'P',   3,  48, 101, "SC-88Pro ORCHESTRA Set", "[55] Timpani F"},
    {'P',   3,  48, 102, "SC-88Pro ORCHESTRA Set", "[55] Timpani F#"},
    {'P',   3,  48, 103, "SC-88Pro ORCHESTRA Set", "[55] Timpani G"},
    {'P',   3,  48, 104, "SC-88Pro ORCHESTRA Set", "[55] Timpani G#"},
    {'P',   3,  48, 105, "SC-88Pro ORCHESTRA Set", "[55] Timpani A"},
    {'P',   3,  48, 106, "SC-88Pro ORCHESTRA Set", "[55] Timpani A#"},
    {'P',   3,  48, 107, "SC-88Pro ORCHESTRA Set", "[55] Timpani B"},
    {'P',   3,  48, 108, "SC-88Pro ORCHESTRA Set", "[55] Timpani c"},
    {'P',   3,  48, 109, "SC-88Pro ORCHESTRA Set", "[55] Timpani c#"},
    {'P',   3,  48, 110, "SC-88Pro ORCHESTRA Set", "[55] Timpani d"},
    {'P',   3,  48, 111, "SC-88Pro ORCHESTRA Set", "[55] Timpani d#"},
    {'P',   3,  48, 112, "SC-88Pro ORCHESTRA Set", "[55] Timpani e"},
    {'P',   3,  48, 113, "SC-88Pro ORCHESTRA Set", "[55] Timpani f"},
    {'P',   3,  49,  25, "SC-88Pro ETHNIC Set", "Finger Snap"},
    {'P',   3,  49,  26, "SC-88Pro ETHNIC Set", "Tambourine"},
    {'P',   3,  49,  27, "SC-88Pro ETHNIC Set", "Castanets"},
    {'P',   3,  49,  28, "SC-88Pro ETHNIC Set", "Crash Cymbal 1"},
    {'P',   3,  49,  29, "SC-88Pro ETHNIC Set", "Snare Roll"},
    {'P',   3,  49,  30, "SC-88Pro ETHNIC Set", "Concert SD"},
    {'P',   3,  49,  31, "SC-88Pro ETHNIC Set", "Concert Cymbal"},
    {'P',   3,  49,  32, "SC-88Pro ETHNIC Set", "Concert BD 1"},
    {'P',   3,  49,  33, "SC-88Pro ETHNIC Set", "Jingle Bell"},
    {'P',   3,  49,  34, "SC-88Pro ETHNIC Set", "Bell Tree"},
    {'P',   3,  49,  35, "SC-88Pro ETHNIC Set", "Bar Chimes"},
    {'P',   3,  49,  36, "SC-88Pro ETHNIC Set", "Wadaiko"},
    {'P',   3,  49,  37, "SC-88Pro ETHNIC Set", "Wadaiko Rim"},
    {'P',   3,  49,  38, "SC-88Pro ETHNIC Set", "Shime Taiko"},
    {'P',   3,  49,  39, "SC-88Pro ETHNIC Set", "Atarigane"},
    {'P',   3,  49,  40, "SC-88Pro ETHNIC Set", "Hyoushigi"},
    {'P',   3,  49,  41, "SC-88Pro ETHNIC Set", "Ohkawa"},
    {'P',   3,  49,  42, "SC-88Pro ETHNIC Set", "High Kotsuzumi"},
    {'P',   3,  49,  43, "SC-88Pro ETHNIC Set", "Low Kotsuzumi"},
    {'P',   3,  49,  44, "SC-88Pro ETHNIC Set", "Ban Gu"},
    {'P',   3,  49,  45, "SC-88Pro ETHNIC Set", "Big Gong"},
    {'P',   3,  49,  46, "SC-88Pro ETHNIC Set", "Small Gong"},
    {'P',   3,  49,  47, "SC-88Pro ETHNIC Set", "Bend Gong"},
    {'P',   3,  49,  48, "SC-88Pro ETHNIC Set", "Thai Gong"},
    {'P',   3,  49,  49, "SC-88Pro ETHNIC Set", "Rama Cymbal"},
    {'P',   3,  49,  50, "SC-88Pro ETHNIC Set", "Gamelan Gong"},
    {'P',   3,  49,  51, "SC-88Pro ETHNIC Set", "Udo Short"},
    {'P',   3,  49,  52, "SC-88Pro ETHNIC Set", "Udo Long"},
    {'P',   3,  49,  53, "SC-88Pro ETHNIC Set", "Udo Slap"},
    {'P',   3,  49,  54, "SC-88Pro ETHNIC Set", "Bendir"},
    {'P',   3,  49,  55, "SC-88Pro ETHNIC Set", "Req Dum"},
    {'P',   3,  49,  56, "SC-88Pro ETHNIC Set", "Req Tik"},
    {'P',   3,  49,  57, "SC-88Pro ETHNIC Set", "Tabla Te"},
    {'P',   3,  49,  58, "SC-88Pro ETHNIC Set", "Tabla Na"},
    {'P',   3,  49,  59, "SC-88Pro ETHNIC Set", "Tabla Tun"},
    {'P',   3,  49,  60, "SC-88Pro ETHNIC Set", "Tabla Ge"},
    {'P',   3,  49,  61, "SC-88Pro ETHNIC Set", "Tabla Ge Hi"},
    {'P',   3,  49,  62, "SC-88Pro ETHNIC Set", "Talking Drum"},
    {'P',   3,  49,  63, "SC-88Pro ETHNIC Set", "Bend Talking Drum"},
    {'P',   3,  49,  64, "SC-88Pro ETHNIC Set", "Caxixi"},
    {'P',   3,  49,  65, "SC-88Pro ETHNIC Set", "Djembe"},
    {'P',   3,  49,  66, "SC-88Pro ETHNIC Set", "Djembe Rim"},
    {'P',   3,  49,  67, "SC-88Pro ETHNIC Set", "Timbales Low"},
    {'P',   3,  49,  68, "SC-88Pro ETHNIC Set", "Timbales Paila"},
    {'P',   3,  49,  69, "SC-88Pro ETHNIC Set", "Timbales High"},
    {'P',   3,  49,  70, "SC-88Pro ETHNIC Set", "Cowbell"},
    {'P',   3,  49,  71, "SC-88Pro ETHNIC Set", "High Bongo"},
    {'P',   3,  49,  72, "SC-88Pro ETHNIC Set", "Low Bongo"},
    {'P',   3,  49,  73, "SC-88Pro ETHNIC Set", "Mute High Conga"},
    {'P',   3,  49,  74, "SC-88Pro ETHNIC Set", "Open High Conga"},
    {'P',   3,  49,  75, "SC-88Pro ETHNIC Set", "Mute Low Conga"},
    {'P',   3,  49,  76, "SC-88Pro ETHNIC Set", "Conga Slap"},
    {'P',   3,  49,  77, "SC-88Pro ETHNIC Set", "Open Low Conga"},
    {'P',   3,  49,  78, "SC-88Pro ETHNIC Set", "Conga Slide"},
    {'P',   3,  49,  79, "SC-88Pro ETHNIC Set", "Mute Pandiero"},
    {'P',   3,  49,  80, "SC-88Pro ETHNIC Set", "Open Pandiero"},
    {'P',   3,  49,  81, "SC-88Pro ETHNIC Set", "Open Surdo"},
    {'P',   3,  49,  82, "SC-88Pro ETHNIC Set", "Mute Surdo"},
    {'P',   3,  49,  83, "SC-88Pro ETHNIC Set", "Tamborim"},
    {'P',   3,  49,  84, "SC-88Pro ETHNIC Set", "High Agogo"},
    {'P',   3,  49,  85, "SC-88Pro ETHNIC Set", "Low Agogo"},
    {'P',   3,  49,  86, "SC-88Pro ETHNIC Set", "Shaker"},
    {'P',   3,  49,  87, "SC-88Pro ETHNIC Set", "High Whistle"},
    {'P',   3,  49,  88, "SC-88Pro ETHNIC Set", "Low Whistle"},
    {'P',   3,  49,  89, "SC-88Pro ETHNIC Set", "Mute Cuica"},
    {'P',   3,  49,  90, "SC-88Pro ETHNIC Set", "Open Cuica"},
    {'P',   3,  49,  91, "SC-88Pro ETHNIC Set", "Mute Triangle"},
    {'P',   3,  49,  92, "SC-88Pro ETHNIC Set", "Open Triangle"},
    {'P',   3,  49,  93, "SC-88Pro ETHNIC Set", "Short Guiro"},
    {'P',   3,  49,  94, "SC-88Pro ETHNIC Set", "Long Guiro"},
    {'P',   3,  49,  95, "SC-88Pro ETHNIC Set", "Cabasa Up"},
    {'P',   3,  49,  96, "SC-88Pro ETHNIC Set", "Cabasa Down"},
    {'P',   3,  49,  97, "SC-88Pro ETHNIC Set", "Claves"},
    {'P',   3,  49,  98, "SC-88Pro ETHNIC Set", "High Wood Block"},
    {'P',   3,  49,  99, "SC-88Pro ETHNIC Set", "Low Wood Block"},
    {'P',   3,  50,  25, "SC-88Pro KICK & SNARE Set", "CR-78 Kick 1"},
    {'P',   3,  50,  26, "SC-88Pro KICK & SNARE Set", "CR-78 Kick 2"},
    {'P',   3,  50,  27, "SC-88Pro KICK & SNARE Set", "TR-606 Kick 1"},
    {'P',   3,  50,  28, "SC-88Pro KICK & SNARE Set", "TR-707 Kick"},
    {'P',   3,  50,  29, "SC-88Pro KICK & SNARE Set", "TR-808 Kick"},
    {'P',   3,  50,  30, "SC-88Pro KICK & SNARE Set", "TR-909 Kick 1"},
    {'P',   3,  50,  31, "SC-88Pro KICK & SNARE Set", "TR-909 Kick 2"},
    {'P',   3,  50,  32, "SC-88Pro KICK & SNARE Set", "Hip-Hop Kick 2"},
    {'P',   3,  50,  33, "SC-88Pro KICK & SNARE Set", "Hip-Hop Kick 1"},
    {'P',   3,  50,  34, "SC-88Pro KICK & SNARE Set", "Jungle Kick 2"},
    {'P',   3,  50,  35, "SC-88Pro KICK & SNARE Set", "Jungle Kick 1"},
    {'P',   3,  50,  36, "SC-88Pro KICK & SNARE Set", "Techno Kick 2"},
    {'P',   3,  50,  37, "SC-88Pro KICK & SNARE Set", "Techno Kick 1"},
    {'P',   3,  50,  38, "SC-88Pro KICK & SNARE Set", "Standard 1 Kick 2"},
    {'P',   3,  50,  39, "SC-88Pro KICK & SNARE Set", "Standard 1 Kick 1"},
    {'P',   3,  50,  40, "SC-88Pro KICK & SNARE Set", "[88] Standard 1 Kick 1"},
    {'P',   3,  50,  41, "SC-88Pro KICK & SNARE Set", "[88] Standard 1 Kick 2"},
    {'P',   3,  50,  42, "SC-88Pro KICK & SNARE Set", "[88] Standard 2 Kick 1"},
    {'P',   3,  50,  43, "SC-88Pro KICK & SNARE Set", "[88] Standard 2 Kick 2"},
    {'P',   3,  50,  44, "SC-88Pro KICK & SNARE Set", "[55] Kick Drum 1"},
    {'P',   3,  50,  45, "SC-88Pro KICK & SNARE Set", "[55] Kick Drum 2"},
    {'P',   3,  50,  46, "SC-88Pro KICK & SNARE Set", "[88] Soft Kick"},
    {'P',   3,  50,  47, "SC-88Pro KICK & SNARE Set", "[88] Jazz Kick 1"},
    {'P',   3,  50,  48, "SC-88Pro KICK & SNARE Set", "[88] Jazz Kick 2"},
    {'P',   3,  50,  49, "SC-88Pro KICK & SNARE Set", "[55] Concert BD 1"},
    {'P',   3,  50,  50, "SC-88Pro KICK & SNARE Set", "[88] Room Kick 1"},
    {'P',   3,  50,  51, "SC-88Pro KICK & SNARE Set", "[88] Room Kick 2"},
    {'P',   3,  50,  52, "SC-88Pro KICK & SNARE Set", "[88] Power Kick 1"},
    {'P',   3,  50,  53, "SC-88Pro KICK & SNARE Set", "[88] Power Kick 2"},
    {'P',   3,  50,  54, "SC-88Pro KICK & SNARE Set", "[88] Electric Kick 2"},
    {'P',   3,  50,  55, "SC-88Pro KICK & SNARE Set", "[88] Electric Kick 1"},
    {'P',   3,  50,  56, "SC-88Pro KICK & SNARE Set", "[55] Electric Kick"},
    {'P',   3,  50,  57, "SC-88Pro KICK & SNARE Set", "[88] TR-808 Kick"},
    {'P',   3,  50,  58, "SC-88Pro KICK & SNARE Set", "[88] TR-909 Kick"},
    {'P',   3,  50,  59, "SC-88Pro KICK & SNARE Set", "[88] Dance Kick"},
    {'P',   3,  50,  60, "SC-88Pro KICK & SNARE Set", "[88] Standard 1 Snare 1"},
    {'P',   3,  50,  61, "SC-88Pro KICK & SNARE Set", "[88] Standard 1 Snare 2"},
    {'P',   3,  50,  62, "SC-88Pro KICK & SNARE Set", "[88] Standard 2 Snare 1"},
    {'P',   3,  50,  63, "SC-88Pro KICK & SNARE Set", "[88] Standard 2 Snare 2"},
    {'P',   3,  50,  64, "SC-88Pro KICK & SNARE Set", "[55] Tight Snare"},
    {'P',   3,  50,  65, "SC-88Pro KICK & SNARE Set", "[55] Concert Snare"},
    {'P',   3,  50,  66, "SC-88Pro KICK & SNARE Set", "[88] Jazz Snare 1"},
    {'P',   3,  50,  67, "SC-88Pro KICK & SNARE Set", "[88] Jazz Snare 2"},
    {'P',   3,  50,  68, "SC-88Pro KICK & SNARE Set", "[88] Room Snare 1"},
    {'P',   3,  50,  69, "SC-88Pro KICK & SNARE Set", "[88] Room Snare 2"},
    {'P',   3,  50,  70, "SC-88Pro KICK & SNARE Set", "[88] Power Snare 1"},
    {'P',   3,  50,  71, "SC-88Pro KICK & SNARE Set", "[88] Power Snare 2"},
    {'P',   3,  50,  72, "SC-88Pro KICK & SNARE Set", "[55] Gated Snare"},
    {'P',   3,  50,  73, "SC-88Pro KICK & SNARE Set", "[88] Dance Snare 1"},
    {'P',   3,  50,  74, "SC-88Pro KICK & SNARE Set", "[88] Dance Snare 2"},
    {'P',   3,  50,  75, "SC-88Pro KICK & SNARE Set", "[88] Disco Snare"},
    {'P',   3,  50,  76, "SC-88Pro KICK & SNARE Set", "[88] Electric Snare 2"},
    {'P',   3,  50,  77, "SC-88Pro KICK & SNARE Set", "[88] House Snare"},
    {'P',   3,  50,  78, "SC-88Pro KICK & SNARE Set", "[55] Electric Snare 1"},
    {'P',   3,  50,  79, "SC-88Pro KICK & SNARE Set", "[88] Electric Snare 3"},
    {'P',   3,  50,  80, "SC-88Pro KICK & SNARE Set", "[88] TR-808 Snare 1"},
    {'P',   3,  50,  81, "SC-88Pro KICK & SNARE Set", "[88] TR-808 Snare 2"},
    {'P',   3,  50,  82, "SC-88Pro KICK & SNARE Set", "[88] TR-909 Snare 1"},
    {'P',   3,  50,  83, "SC-88Pro KICK & SNARE Set", "[88] TR-909 Snare 2"},
    {'P',   3,  50,  84, "SC-88Pro KICK & SNARE Set", "[88] Brush Tap 1"},
    {'P',   3,  50,  85, "SC-88Pro KICK & SNARE Set", "[88] Brush Tap 2"},
    {'P',   3,  50,  86, "SC-88Pro KICK & SNARE Set", "[88] Brush Slap 1"},
    {'P',   3,  50,  87, "SC-88Pro KICK & SNARE Set", "[88] Brush Slap 2"},
    {'P',   3,  50,  88, "SC-88Pro KICK & SNARE Set", "[88] Brush Slap 3"},
    {'P',   3,  50,  89, "SC-88Pro KICK & SNARE Set", "[88] Brush Swirl 1"},
    {'P',   3,  50,  90, "SC-88Pro KICK & SNARE Set", "[88] Brush Swirl 2"},
    {'P',   3,  50,  91, "SC-88Pro KICK & SNARE Set", "[88] Brush Long Swirl"},
    {'P',   3,  50,  92, "SC-88Pro KICK & SNARE Set", "Standard1 Snare 1"},
    {'P',   3,  50,  93, "SC-88Pro KICK & SNARE Set", "Standard1 Snare 2"},
    {'P',   3,  50,  94, "SC-88Pro KICK & SNARE Set", "Standard Snare 3"},
    {'P',   3,  50,  95, "SC-88Pro KICK & SNARE Set", "Rap Snare"},
    {'P',   3,  50,  96, "SC-88Pro KICK & SNARE Set", "Hip-Hop Snare 2"},
    {'P',   3,  50,  97, "SC-88Pro KICK & SNARE Set", "Jungle Snare 1"},
    {'P',   3,  50,  98, "SC-88Pro KICK & SNARE Set", "Jungle Snare 2"},
    {'P',   3,  50,  99, "SC-88Pro KICK & SNARE Set", "Techno Snare 1"},
    {'P',   3,  50, 100, "SC-88Pro KICK & SNARE Set", "Techno Snare 2"},
    {'P',   3,  50, 101, "SC-88Pro KICK & SNARE Set", "House Snare 2"},
    {'P',   3,  50, 102, "SC-88Pro KICK & SNARE Set", "CR-78 Snare 1"},
    {'P',   3,  50, 103, "SC-88Pro KICK & SNARE Set", "CR-78 Snare 2"},
    {'P',   3,  50, 104, "SC-88Pro KICK & SNARE Set", "TR-606 Snare 1"},
    {'P',   3,  50, 105, "SC-88Pro KICK & SNARE Set", "TR-606 Snare 2"},
    {'P',   3,  50, 106, "SC-88Pro KICK & SNARE Set", "TR-707 Snare 1"},
    {'P',   3,  50, 107, "SC-88Pro KICK & SNARE Set", "TR-707 Snare 2"},
    {'P',   3,  50, 108, "SC-88Pro KICK & SNARE Set", "Standard3 Snare 2"},
    {'P',   3,  50, 109, "SC-88Pro KICK & SNARE Set", "TR-808 Snare 2"},
    {'P',   3,  50, 110, "SC-88Pro KICK & SNARE Set", "TR-909 Snare 1"},
    {'P',   3,  50, 111, "SC-88Pro KICK & SNARE Set", "TR-909 Snare 2"},
    {'P',   3,  52,  25, "SC-88Pro ASIA Set", "Gamelan Gong 1"},
    {'P',   3,  52,  26, "SC-88Pro ASIA Set", "Gamelan Gong 2"},
    {'P',   3,  52,  27, "SC-88Pro ASIA Set", "Gamelan Gong 3"},
    {'P',   3,  52,  28, "SC-88Pro ASIA Set", "Gamelan Gong 4"},
    {'P',   3,  52,  29, "SC-88Pro ASIA Set", "Gamelan Gong 5"},
    {'P',   3,  52,  30, "SC-88Pro ASIA Set", "Gamelan Gong 6"},
    {'P',   3,  52,  31, "SC-88Pro ASIA Set", "Gamelan Gong 7"},
    {'P',   3,  52,  32, "SC-88Pro ASIA Set", "Gamelan Gong 8"},
    {'P',   3,  52,  33, "SC-88Pro ASIA Set", "Gamelan Gong 9"},
    {'P',   3,  52,  34, "SC-88Pro ASIA Set", "Gamelan Gong 20"},
    {'P',   3,  52,  35, "SC-88Pro ASIA Set", "Gender 1"},
    {'P',   3,  52,  36, "SC-88Pro ASIA Set", "Gender 2"},
    {'P',   3,  52,  37, "SC-88Pro ASIA Set", "Gender 3"},
    {'P',   3,  52,  38, "SC-88Pro ASIA Set", "Gender 4"},
    {'P',   3,  52,  39, "SC-88Pro ASIA Set", "Gender 5"},
    {'P',   3,  52,  40, "SC-88Pro ASIA Set", "Bonang 1"},
    {'P',   3,  52,  41, "SC-88Pro ASIA Set", "Bonang 2"},
    {'P',   3,  52,  42, "SC-88Pro ASIA Set", "Bonang 3"},
    {'P',   3,  52,  43, "SC-88Pro ASIA Set", "Bonang 4"},
    {'P',   3,  52,  44, "SC-88Pro ASIA Set", "Bonang 5"},
    {'P',   3,  52,  45, "SC-88Pro ASIA Set", "Rama Cymbal Low"},
    {'P',   3,  52,  46, "SC-88Pro ASIA Set", "Rama Cymbal High"},
    {'P',   3,  52,  47, "SC-88Pro ASIA Set", "Sagat Open"},
    {'P',   3,  52,  48, "SC-88Pro ASIA Set", "Sagat Closed"},
    {'P',   3,  52,  49, "SC-88Pro ASIA Set", "Jaws Harp"},
    {'P',   3,  52,  50, "SC-88Pro ASIA Set", "Wadaiko"},
    {'P',   3,  52,  51, "SC-88Pro ASIA Set", "Wadaiko Rim"},
    {'P',   3,  52,  52, "SC-88Pro ASIA Set", "Small Taiko"},
    {'P',   3,  52,  53, "SC-88Pro ASIA Set", "Shimedaiko"},
    {'P',   3,  52,  54, "SC-88Pro ASIA Set", "Atarigane"},
    {'P',   3,  52,  55, "SC-88Pro ASIA Set", "Hyousigi"},
    {'P',   3,  52,  56, "SC-88Pro ASIA Set", "Ohkawa"},
    {'P',   3,  52,  57, "SC-88Pro ASIA Set", "High Kotsuzumi"},
    {'P',   3,  52,  58, "SC-88Pro ASIA Set", "Low Kotsuzumi"},
    {'P',   3,  52,  59, "SC-88Pro ASIA Set", "Yyoo Dude"},
    {'P',   3,  52,  60, "SC-88Pro ASIA Set", "Buk"},
    {'P',   3,  52,  61, "SC-88Pro ASIA Set", "Buk Rim"},
    {'P',   3,  52,  62, "SC-88Pro ASIA Set", "Gengari p"},
    {'P',   3,  52,  63, "SC-88Pro ASIA Set", "Gengari Mute Low"},
    {'P',   3,  52,  64, "SC-88Pro ASIA Set", "Gengari f"},
    {'P',   3,  52,  65, "SC-88Pro ASIA Set", "Gengari Mute High"},
    {'P',   3,  52,  66, "SC-88Pro ASIA Set", "Gengari Samll"},
    {'P',   3,  52,  67, "SC-88Pro ASIA Set", "Jang-Gu Che"},
    {'P',   3,  52,  68, "SC-88Pro ASIA Set", "Jang-Gu Kun"},
    {'P',   3,  52,  69, "SC-88Pro ASIA Set", "Jang-Gu Rim"},
    {'P',   3,  52,  70, "SC-88Pro ASIA Set", "Jing p"},
    {'P',   3,  52,  71, "SC-88Pro ASIA Set", "Jing f"},
    {'P',   3,  52,  72, "SC-88Pro ASIA Set", "Jing Mute"},
    {'P',   3,  52,  73, "SC-88Pro ASIA Set", "Asian Gong1"},
    {'P',   3,  52,  74, "SC-88Pro ASIA Set", "Big Gong"},
    {'P',   3,  52,  75, "SC-88Pro ASIA Set", "Small Gong"},
    {'P',   3,  52,  76, "SC-88Pro ASIA Set", "Pai Ban"},
    {'P',   3,  52,  77, "SC-88Pro ASIA Set", "Ban Gu"},
    {'P',   3,  52,  78, "SC-88Pro ASIA Set", "Tang Gu"},
    {'P',   3,  52,  79, "SC-88Pro ASIA Set", "Tnag Gu Mute"},
    {'P',   3,  52,  80, "SC-88Pro ASIA Set", "Shou Luo"},
    {'P',   3,  52,  81, "SC-88Pro ASIA Set", "Bend Gong"},
    {'P',   3,  52,  82, "SC-88Pro ASIA Set", "Hu Yin Luo Low"},
    {'P',   3,  52,  83, "SC-88Pro ASIA Set", "Hu Yin Luo Mid"},
    {'P',   3,  52,  84, "SC-88Pro ASIA Set", "Hu Yin Luo Mid 2"},
    {'P',   3,  52,  85, "SC-88Pro ASIA Set", "Hu Yin Luo High"},
    {'P',   3,  52,  86, "SC-88Pro ASIA Set", "Hu Yin Luo High 2"},
    {'P',   3,  52,  87, "SC-88Pro ASIA Set", "Nao Bo"},
    {'P',   3,  52,  88, "SC-88Pro ASIA Set", "Xiao Bo"},
    {'P',   3,  53,  32, "SC-88Pro CYMBAL&CLAPS Set", "Reverse Open Hi-Hat"},
    {'P',   3,  53,  33, "SC-88Pro CYMBAL&CLAPS Set", "Reverse Closed Hi-Hat 1"},
    {'P',   3,  53,  34, "SC-88Pro CYMBAL&CLAPS Set", "Reverse Closed Hi-Hat 2"},
    {'P',   3,  53,  35, "SC-88Pro CYMBAL&CLAPS Set", "Jungle Hi-Hat"},
    {'P',   3,  53,  36, "SC-88Pro CYMBAL&CLAPS Set", "[55] Closed Hi-Hat"},
    {'P',   3,  53,  37, "SC-88Pro CYMBAL&CLAPS Set", "[88] Closed Hi-Hat 2"},
    {'P',   3,  53,  38, "SC-88Pro CYMBAL&CLAPS Set", "[88] Closed Hi-Hat 3"},
    {'P',   3,  53,  39, "SC-88Pro CYMBAL&CLAPS Set", "Closed Hi-Hat 4"},
    {'P',   3,  53,  40, "SC-88Pro CYMBAL&CLAPS Set", "Closed Hi-Hat"},
    {'P',   3,  53,  41, "SC-88Pro CYMBAL&CLAPS Set", "TR-707 Closed Hi-Hat"},
    {'P',   3,  53,  42, "SC-88Pro CYMBAL&CLAPS Set", "TR-606 Closed Hi-Hat"},
    {'P',   3,  53,  43, "SC-88Pro CYMBAL&CLAPS Set", "[88] TR-808 Closed Hi-Hat"},
    {'P',   3,  53,  44, "SC-88Pro CYMBAL&CLAPS Set", "TR-808 Closed Hi-Hat"},
    {'P',   3,  53,  45, "SC-88Pro CYMBAL&CLAPS Set", "CR-78 Closed Hi-Hat"},
    {'P',   3,  53,  46, "SC-88Pro CYMBAL&CLAPS Set", "[55] Pedal Hi-Hat"},
    {'P',   3,  53,  47, "SC-88Pro CYMBAL&CLAPS Set", "[88] Pedal Hi-Hat"},
    {'P',   3,  53,  48, "SC-88Pro CYMBAL&CLAPS Set", "Pedal Hi-Hat"},
    {'P',   3,  53,  49, "SC-88Pro CYMBAL&CLAPS Set", "Half-Open Hi-Hat 1"},
    {'P',   3,  53,  50, "SC-88Pro CYMBAL&CLAPS Set", "Half-Open Hi-Hat 2"},
    {'P',   3,  53,  51, "SC-88Pro CYMBAL&CLAPS Set", "[55] Open Hi-Hat"},
    {'P',   3,  53,  52, "SC-88Pro CYMBAL&CLAPS Set", "[88] Open Hi-Hat 2"},
    {'P',   3,  53,  53, "SC-88Pro CYMBAL&CLAPS Set", "[88] Open Hi-Hat 3"},
    {'P',   3,  53,  54, "SC-88Pro CYMBAL&CLAPS Set", "Open Hi-Hat 2"},
    {'P',   3,  53,  55, "SC-88Pro CYMBAL&CLAPS Set", "TR-909 Open Hi-Hat"},
    {'P',   3,  53,  56, "SC-88Pro CYMBAL&CLAPS Set", "TR-707 Open Hi-Hat"},
    {'P',   3,  53,  57, "SC-88Pro CYMBAL&CLAPS Set", "TR-606 Open Hi-Hat"},
    {'P',   3,  53,  58, "SC-88Pro CYMBAL&CLAPS Set", "[88] TR-808 Open Hi-Hat"},
    {'P',   3,  53,  59, "SC-88Pro CYMBAL&CLAPS Set", "TR-808 Open Hi-Hat"},
    {'P',   3,  53,  60, "SC-88Pro CYMBAL&CLAPS Set", "CR-78 Open Hi-Hat"},
    {'P',   3,  53,  61, "SC-88Pro CYMBAL&CLAPS Set", "Crash Cymbal1"},
    {'P',   3,  53,  62, "SC-88Pro CYMBAL&CLAPS Set", "Crash Cymbal2"},
    {'P',   3,  53,  63, "SC-88Pro CYMBAL&CLAPS Set", "Crash Cymbal3"},
    {'P',   3,  53,  64, "SC-88Pro CYMBAL&CLAPS Set", "Brush Crash Cymbal"},
    {'P',   3,  53,  65, "SC-88Pro CYMBAL&CLAPS Set", "Hard Crash Cymbal"},
    {'P',   3,  53,  66, "SC-88Pro CYMBAL&CLAPS Set", "TR-909 Crash Cymbal"},
    {'P',   3,  53,  67, "SC-88Pro CYMBAL&CLAPS Set", "TR-808 Crash Cymbal"},
    {'P',   3,  53,  68, "SC-88Pro CYMBAL&CLAPS Set", "Mute Crash Cymbal 1"},
    {'P',   3,  53,  69, "SC-88Pro CYMBAL&CLAPS Set", "Mute Crash Cymbal 2"},
    {'P',   3,  53,  70, "SC-88Pro CYMBAL&CLAPS Set", "Reverse Crash Cymbal 1"},
    {'P',   3,  53,  71, "SC-88Pro CYMBAL&CLAPS Set", "Reverse Crash Cymbal 2"},
    {'P',   3,  53,  72, "SC-88Pro CYMBAL&CLAPS Set", "Reverse Crash Cymbal 3"},
    {'P',   3,  53,  73, "SC-88Pro CYMBAL&CLAPS Set", "Reverse TR-909 Crash Cymbal"},
    {'P',   3,  53,  74, "SC-88Pro CYMBAL&CLAPS Set", "[55] Splash Cymbal"},
    {'P',   3,  53,  75, "SC-88Pro CYMBAL&CLAPS Set", "Splash Cymbal"},
    {'P',   3,  53,  76, "SC-88Pro CYMBAL&CLAPS Set", "[88] Ride Bell"},
    {'P',   3,  53,  77, "SC-88Pro CYMBAL&CLAPS Set", "[88] Brush Ride Bell"},
    {'P',   3,  53,  78, "SC-88Pro CYMBAL&CLAPS Set", "[88] Ride Cymbal1"},
    {'P',   3,  53,  79, "SC-88Pro CYMBAL&CLAPS Set", "[88] Ride Cymbal2"},
    {'P',   3,  53,  80, "SC-88Pro CYMBAL&CLAPS Set", "[88] Brush Ride Cymbal"},
    {'P',   3,  53,  81, "SC-88Pro CYMBAL&CLAPS Set", "Ride Cymbal Low Inner"},
    {'P',   3,  53,  82, "SC-88Pro CYMBAL&CLAPS Set", "Ride Cymbal Mid Inner"},
    {'P',   3,  53,  83, "SC-88Pro CYMBAL&CLAPS Set", "Ride Cymbal High Inner"},
    {'P',   3,  53,  84, "SC-88Pro CYMBAL&CLAPS Set", "Ride Cymbal Low Edge"},
    {'P',   3,  53,  85, "SC-88Pro CYMBAL&CLAPS Set", "Ride Cymbal Mid Edge"},
    {'P',   3,  53,  86, "SC-88Pro CYMBAL&CLAPS Set", "Ride Cymbal High Edge"},
    {'P',   3,  53,  87, "SC-88Pro CYMBAL&CLAPS Set", "TR-606 Ride Cymbal"},
    {'P',   3,  53,  88, "SC-88Pro CYMBAL&CLAPS Set", "TR-808 Ride Cymbal"},
    {'P',   3,  53,  89, "SC-88Pro CYMBAL&CLAPS Set", "Chinese Cymbal"},
    {'P',   3,  53,  90, "SC-88Pro CYMBAL&CLAPS Set", "Chinese Cymbal 2"},
    {'P',   3,  53,  91, "SC-88Pro CYMBAL&CLAPS Set", "[55] Hand Clap"},
    {'P',   3,  53,  92, "SC-88Pro CYMBAL&CLAPS Set", "[88] Hand Clap 2"},
    {'P',   3,  53,  93, "SC-88Pro CYMBAL&CLAPS Set", "[88] Hand Clap"},
    {'P',   3,  53,  94, "SC-88Pro CYMBAL&CLAPS Set", "Hand Clap"},
    {'P',   3,  53,  95, "SC-88Pro CYMBAL&CLAPS Set", "Hand Clap2"},
    {'P',   3,  53,  96, "SC-88Pro CYMBAL&CLAPS Set", "TR-707 Hand Clap"},
    {'P',   3,  56,  21, "SC-88Pro SFX Set", "MC-500 Beep 1"},
    {'P',   3,  56,  22, "SC-88Pro SFX Set", "MC-500 Beep 2"},
    {'P',   3,  56,  23, "SC-88Pro SFX Set", "Guitar Slide"},
    {'P',   3,  56,  24, "SC-88Pro SFX Set", "Guitar Wah"},
    {'P',   3,  56,  25, "SC-88Pro SFX Set", "Guitar Slap"},
    {'P',   3,  56,  26, "SC-88Pro SFX Set", "Chord Stroke Down"},
    {'P',   3,  56,  27, "SC-88Pro SFX Set", "Chord Stroke Up"},
    {'P',   3,  56,  28, "SC-88Pro SFX Set", "Biwa FX"},
    {'P',   3,  56,  29, "SC-88Pro SFX Set", "Phonograph Noise"},
    {'P',   3,  56,  30, "SC-88Pro SFX Set", "Tape Rewind"},
    {'P',   3,  56,  31, "SC-88Pro SFX Set", "Scratch Push 2"},
    {'P',   3,  56,  32, "SC-88Pro SFX Set", "Scratch Pull 2"},
    {'P',   3,  56,  33, "SC-88Pro SFX Set", "Cutting Noise 2 Up"},
    {'P',   3,  56,  34, "SC-88Pro SFX Set", "Cutting Noise 2 Down"},
    {'P',   3,  56,  35, "SC-88Pro SFX Set", "Distortion Guitar Cutting Noise Up"},
    {'P',   3,  56,  36, "SC-88Pro SFX Set", "Distortion Guitar Cutting Noise Down"},
    {'P',   3,  56,  37, "SC-88Pro SFX Set", "Bass Slide"},
    {'P',   3,  56,  38, "SC-88Pro SFX Set", "Pick Scrape"},
    {'P',   3,  56,  39, "SC-88Pro SFX Set", "High Q"},
    {'P',   3,  56,  40, "SC-88Pro SFX Set", "Slap"},
    {'P',   3,  56,  41, "SC-88Pro SFX Set", "Scratch Push"},
    {'P',   3,  56,  42, "SC-88Pro SFX Set", "Scratch Pull"},
    {'P',   3,  56,  43, "SC-88Pro SFX Set", "Sticks"},
    {'P',   3,  56,  44, "SC-88Pro SFX Set", "Square Click"},
    {'P',   3,  56,  45, "SC-88Pro SFX Set", "Metronome Click"},
    {'P',   3,  56,  46, "SC-88Pro SFX Set", "Metronome Bell"},
    {'P',   3,  56,  47, "SC-88Pro SFX Set", "Guitar Fret Noise"},
    {'P',   3,  56,  48, "SC-88Pro SFX Set", "Guitar Cutting Noise Up"},
    {'P',   3,  56,  49, "SC-88Pro SFX Set", "Guitar Cutting Noise Down"},
    {'P',   3,  56,  50, "SC-88Pro SFX Set", "String Slap of Double Bass"},
    {'P',   3,  56,  51, "SC-88Pro SFX Set", "Flute Key Click Noise"},
    {'P',   3,  56,  52, "SC-88Pro SFX Set", "Laughing"},
    {'P',   3,  56,  53, "SC-88Pro SFX Set", "Screaming"},
    {'P',   3,  56,  54, "SC-88Pro SFX Set", "Punch"},
    {'P',   3,  56,  55, "SC-88Pro SFX Set", "Heart Beat"},
    {'P',   3,  56,  56, "SC-88Pro SFX Set", "Footsteps 1"},
    {'P',   3,  56,  57, "SC-88Pro SFX Set", "Footsteps 2"},
    {'P',   3,  56,  58, "SC-88Pro SFX Set", "Applause"},
    {'P',   3,  56,  59, "SC-88Pro SFX Set", "Door Creaking"},
    {'P',   3,  56,  60, "SC-88Pro SFX Set", "Door"},
    {'P',   3,  56,  61, "SC-88Pro SFX Set", "Scratch"},
    {'P',   3,  56,  62, "SC-88Pro SFX Set", "Wind Chimes"},
    {'P',   3,  56,  63, "SC-88Pro SFX Set", "Car - Engine"},
    {'P',   3,  56,  64, "SC-88Pro SFX Set", "Car - Stop"},
    {'P',   3,  56,  65, "SC-88Pro SFX Set", "Car - Passing"},
    {'P',   3,  56,  66, "SC-88Pro SFX Set", "Car - Crash"},
    {'P',   3,  56,  67, "SC-88Pro SFX Set", "Siren"},
    {'P',   3,  56,  68, "SC-88Pro SFX Set", "Train"},
    {'P',   3,  56,  69, "SC-88Pro SFX Set", "Jetplane"},
    {'P',   3,  56,  70, "SC-88Pro SFX Set", "Helicopter"},
    {'P',   3,  56,  71, "SC-88Pro SFX Set", "Starship"},
    {'P',   3,  56,  72, "SC-88Pro SFX Set", "Gun Shot"},
    {'P',   3,  56,  73, "SC-88Pro SFX Set", "Machine Gun"},
    {'P',   3,  56,  74, "SC-88Pro SFX Set", "Laser Gun"},
    {'P',   3,  56,  75, "SC-88Pro SFX Set", "Explosion"},
    {'P',   3,  56,  76, "SC-88Pro SFX Set", "Dog"},
    {'P',   3,  56,  77, "SC-88Pro SFX Set", "Horse-Gallop"},
    {'P',   3,  56,  78, "SC-88Pro SFX Set", "Birds"},
    {'P',   3,  56,  79, "SC-88Pro SFX Set", "Rain"},
    {'P',   3,  56,  80, "SC-88Pro SFX Set", "Thunder"},
    {'P',   3,  56,  81, "SC-88Pro SFX Set", "Wind"},
    {'P',   3,  56,  82, "SC-88Pro SFX Set", "Seashore"},
    {'P',   3,  56,  83, "SC-88Pro SFX Set", "Stream"},
    {'P',   3,  56,  84, "SC-88Pro SFX Set", "Bubble"},
    {'P',   3,  56,  85, "SC-88Pro SFX Set", "Kitty"},
    {'P',   3,  56,  86, "SC-88Pro SFX Set", "Bird 2"},
    {'P',   3,  56,  87, "SC-88Pro SFX Set", "Growl"},
    {'P',   3,  56,  88, "SC-88Pro SFX Set", "Applause 2"},
    {'P',   3,  56,  89, "SC-88Pro SFX Set", "Telephone 1"},
    {'P',   3,  56,  90, "SC-88Pro SFX Set", "Telephone 2"},
    {'P',   3,  56,  91, "SC-88Pro SFX Set", "Small Club 1"},
    {'P',   3,  56,  92, "SC-88Pro SFX Set", "Small Club 2"},
    {'P',   3,  56,  93, "SC-88Pro SFX Set", "Applause Wave"},
    {'P',   3,  56,  94, "SC-88Pro SFX Set", "Eruption"},
    {'P',   3,  56,  95, "SC-88Pro SFX Set", "Big Shot"},
    {'P',   3,  56,  96, "SC-88Pro SFX Set", "Percussion Bang"},
    {'P',   3,  57,  36, "SC-88Pro RHYTHM FX Set", "Reverse Kick 1"},
    {'P',   3,  57,  37, "SC-88Pro RHYTHM FX Set", "Reverse Concert Bass Drum"},
    {'P',   3,  57,  38, "SC-88Pro RHYTHM FX Set", "Reverse Power Kick 1"},
    {'P',   3,  57,  39, "SC-88Pro RHYTHM FX Set", "Reverse Electric Kick 1"},
    {'P',   3,  57,  40, "SC-88Pro RHYTHM FX Set", "Reverse Snare 1"},
    {'P',   3,  57,  41, "SC-88Pro RHYTHM FX Set", "Reverse Snare 2"},
    {'P',   3,  57,  42, "SC-88Pro RHYTHM FX Set", "Reverse Standard 1 Snare 1"},
    {'P',   3,  57,  43, "SC-88Pro RHYTHM FX Set", "Reverse Tight Snare"},
    {'P',   3,  57,  44, "SC-88Pro RHYTHM FX Set", "Reverse Dance Snare"},
    {'P',   3,  57,  45, "SC-88Pro RHYTHM FX Set", "Reverse 808 Snare"},
    {'P',   3,  57,  46, "SC-88Pro RHYTHM FX Set", "Reverse Tom 1"},
    {'P',   3,  57,  47, "SC-88Pro RHYTHM FX Set", "Reverse Tom 2"},
    {'P',   3,  57,  48, "SC-88Pro RHYTHM FX Set", "Reverse Sticks"},
    {'P',   3,  57,  49, "SC-88Pro RHYTHM FX Set", "Reverse Slap"},
    {'P',   3,  57,  50, "SC-88Pro RHYTHM FX Set", "Reverse Cymbal 1"},
    {'P',   3,  57,  51, "SC-88Pro RHYTHM FX Set", "Reverse Cymbal 2"},
    {'P',   3,  57,  52, "SC-88Pro RHYTHM FX Set", "Reverse Open Hi-Hat"},
    {'P',   3,  57,  53, "SC-88Pro RHYTHM FX Set", "Reverse Ride Cymbal"},
    {'P',   3,  57,  54, "SC-88Pro RHYTHM FX Set", "Reverse CR-78 Open Hi-Hat"},
    {'P',   3,  57,  55, "SC-88Pro RHYTHM FX Set", "Reverse Closed Hi-Hat"},
    {'P',   3,  57,  56, "SC-88Pro RHYTHM FX Set", "Reverse Gong"},
    {'P',   3,  57,  57, "SC-88Pro RHYTHM FX Set", "Reverse Bell Tree"},
    {'P',   3,  57,  58, "SC-88Pro RHYTHM FX Set", "Reverse Guiro"},
    {'P',   3,  57,  59, "SC-88Pro RHYTHM FX Set", "Reverse Bendir"},
    {'P',   3,  57,  60, "SC-88Pro RHYTHM FX Set", "Reverse Gun Shot"},
    {'P',   3,  57,  61, "SC-88Pro RHYTHM FX Set", "Reverse Scratch"},
    {'P',   3,  57,  62, "SC-88Pro RHYTHM FX Set", "Reverse Laser Gun"},
    {'P',   3,  57,  63, "SC-88Pro RHYTHM FX Set", "Key Click"},
    {'P',   3,  57,  64, "SC-88Pro RHYTHM FX Set", "Techno Thip"},
    {'P',   3,  57,  65, "SC-88Pro RHYTHM FX Set", "Pop Drop"},
    {'P',   3,  57,  66, "SC-88Pro RHYTHM FX Set", "Woody Slap"},
    {'P',   3,  57,  67, "SC-88Pro RHYTHM FX Set", "Distortion Kick"},
    {'P',   3,  57,  68, "SC-88Pro RHYTHM FX Set", "Syn.Drops"},
    {'P',   3,  57,  69, "SC-88Pro RHYTHM FX Set", "Reverse Hi Q"},
    {'P',   3,  57,  70, "SC-88Pro RHYTHM FX Set", "Pipe"},
    {'P',   3,  57,  71, "SC-88Pro RHYTHM FX Set", "Ice Block"},
    {'P',   3,  57,  72, "SC-88Pro RHYTHM FX Set", "Digital Tambourine"},
    {'P',   3,  57,  73, "SC-88Pro RHYTHM FX Set", "Alias"},
    {'P',   3,  57,  74, "SC-88Pro RHYTHM FX Set", "Modulated Bell"},
    {'P',   3,  57,  75, "SC-88Pro RHYTHM FX Set", "Spark"},
    {'P',   3,  57,  76, "SC-88Pro RHYTHM FX Set", "Metalic Percussion"},
    {'P',   3,  57,  77, "SC-88Pro RHYTHM FX Set", "Velocity Noise FX"},
    {'P',   3,  57,  78, "SC-88Pro RHYTHM FX Set", "Stereo Noise Clap"},
    {'P',   3,  57,  79, "SC-88Pro RHYTHM FX Set", "Swish"},
    {'P',   3,  57,  80, "SC-88Pro RHYTHM FX Set", "Slappy"},
    {'P',   3,  57,  81, "SC-88Pro RHYTHM FX Set", "Voice Ou"},
    {'P',   3,  57,  82, "SC-88Pro RHYTHM FX Set", "Voice Au"},
    {'P',   3,  57,  83, "SC-88Pro RHYTHM FX Set", "Hoo"},
    {'P',   3,  57,  84, "SC-88Pro RHYTHM FX Set", "Tape Stop 1"},
    {'P',   3,  57,  85, "SC-88Pro RHYTHM FX Set", "Tape Stop 2"},
    {'P',   3,  57,  86, "SC-88Pro RHYTHM FX Set", "Missile"},
    {'P',   3,  57,  87, "SC-88Pro RHYTHM FX Set", "Space Birds"},
    {'P',   3,  57,  88, "SC-88Pro RHYTHM FX Set", "Flying Monster"},
    {'P',   3,  58,  36, "SC-88Pro RHYTHM FX 2 Set", "Reverse TR-707 Kick 1"},
    {'P',   3,  58,  37, "SC-88Pro RHYTHM FX 2 Set", "Reverse TR-909 Kick 1"},
    {'P',   3,  58,  38, "SC-88Pro RHYTHM FX 2 Set", "Reverse Hip-Hop Kick 1"},
    {'P',   3,  58,  39, "SC-88Pro RHYTHM FX 2 Set", "Reverse Jungle Kick 2"},
    {'P',   3,  58,  40, "SC-88Pro RHYTHM FX 2 Set", "Reverse Techno Kick 2"},
    {'P',   3,  58,  41, "SC-88Pro RHYTHM FX 2 Set", "Reverse TR-606 Snare 2"},
    {'P',   3,  58,  42, "SC-88Pro RHYTHM FX 2 Set", "Reverse CR-78 Snare 1"},
    {'P',   3,  58,  43, "SC-88Pro RHYTHM FX 2 Set", "Reverse CR-78 Snare 2"},
    {'P',   3,  58,  44, "SC-88Pro RHYTHM FX 2 Set", "Reverse Jungle Snare 2"},
    {'P',   3,  58,  45, "SC-88Pro RHYTHM FX 2 Set", "Reverse Techno Snare 2"},
    {'P',   3,  58,  46, "SC-88Pro RHYTHM FX 2 Set", "Reverse TR-707 Snare"},
    {'P',   3,  58,  47, "SC-88Pro RHYTHM FX 2 Set", "Reverse TR-606 Snare 1"},
    {'P',   3,  58,  48, "SC-88Pro RHYTHM FX 2 Set", "Reverse TR-909 Snare 1"},
    {'P',   3,  58,  49, "SC-88Pro RHYTHM FX 2 Set", "Reverse Hip-Hop Snare 2"},
    {'P',   3,  58,  50, "SC-88Pro RHYTHM FX 2 Set", "Reverse Jungle Snare 1"},
    {'P',   3,  58,  51, "SC-88Pro RHYTHM FX 2 Set", "Reverse House Snare"},
    {'P',   3,  58,  52, "SC-88Pro RHYTHM FX 2 Set", "Reverse Closed Hi-Hat"},
    {'P',   3,  58,  53, "SC-88Pro RHYTHM FX 2 Set", "Reverse TR-606 Closed Hi-Hat"},
    {'P',   3,  58,  54, "SC-88Pro RHYTHM FX 2 Set", "Reverse TR-707 Closed Hi-Hat"},
    {'P',   3,  58,  55, "SC-88Pro RHYTHM FX 2 Set", "Reverse TR-808 Closed Hi-Hat"},
    {'P',   3,  58,  56, "SC-88Pro RHYTHM FX 2 Set", "Reverse Jungle Hi-Hat"},
    {'P',   3,  58,  57, "SC-88Pro RHYTHM FX 2 Set", "Reverse Tambourine 2"},
    {'P',   3,  58,  58, "SC-88Pro RHYTHM FX 2 Set", "Reverse Shake Tambourine"},
    {'P',   3,  58,  59, "SC-88Pro RHYTHM FX 2 Set", "Reverse TR-808 Open Hi-Hat"},
    {'P',   3,  58,  60, "SC-88Pro RHYTHM FX 2 Set", "Reverse TR-707 Open Hi-Hat"},
    {'P',   3,  58,  61, "SC-88Pro RHYTHM FX 2 Set", "Reverse Open Hi-Hat"},
    {'P',   3,  58,  62, "SC-88Pro RHYTHM FX 2 Set", "Reverse TR-606 Open Hi-Hat"},
    {'P',   3,  58,  63, "SC-88Pro RHYTHM FX 2 Set", "Reverse Hu Yin Luo"},
    {'P',   3,  58,  64, "SC-88Pro RHYTHM FX 2 Set", "Reverse TR-707 Crash Cymbal"},
    {'P',   3,  58,  65, "SC-88Pro RHYTHM FX 2 Set", "Voice One"},
    {'P',   3,  58,  66, "SC-88Pro RHYTHM FX 2 Set", "Reverse Voice One"},
    {'P',   3,  58,  67, "SC-88Pro RHYTHM FX 2 Set", "Voice Two"},
    {'P',   3,  58,  68, "SC-88Pro RHYTHM FX 2 Set", "Reverse Voice Two"},
    {'P',   3,  58,  69, "SC-88Pro RHYTHM FX 2 Set", "Voice Three"},
    {'P',   3,  58,  70, "SC-88Pro RHYTHM FX 2 Set", "Reverse Voice Three"},
    {'P',   3,  58,  71, "SC-88Pro RHYTHM FX 2 Set", "Voice Tah"},
    {'P',   3,  58,  72, "SC-88Pro RHYTHM FX 2 Set", "Reverse Voice Tah"},
    {'P',   3,  58,  73, "SC-88Pro RHYTHM FX 2 Set", "Voice Ou"},
    {'P',   3,  58,  74, "SC-88Pro RHYTHM FX 2 Set", "Voice Au"},
    {'P',   3,  58,  75, "SC-88Pro RHYTHM FX 2 Set", "Voice Whey"},
    {'P',   3,  58,  76, "SC-88Pro RHYTHM FX 2 Set", "Frog Vpoce"},
    {'P',   3,  58,  77, "SC-88Pro RHYTHM FX 2 Set", "Reverse Yyoo Dude"},
    {'P',   3,  58,  78, "SC-88Pro RHYTHM FX 2 Set", "Douby"},
    {'P',   3,  58,  79, "SC-88Pro RHYTHM FX 2 Set", "Reverse Douby"},
    {'P',   3,  58,  80, "SC-88Pro RHYTHM FX 2 Set", "Baert High"},
    {'P',   3,  58,  81, "SC-88Pro RHYTHM FX 2 Set", "Baert Low"},
    {'P',   3,  58,  82, "SC-88Pro RHYTHM FX 2 Set", "Bounce"},
    {'P',   3,  58,  83, "SC-88Pro RHYTHM FX 2 Set", "Reverse Bounce"},
    {'P',   3,  58,  84, "SC-88Pro RHYTHM FX 2 Set", "Dstortion Knock"},
    {'P',   3,  58,  85, "SC-88Pro RHYTHM FX 2 Set", "Guitar Slide"},
    {'P',   3,  58,  86, "SC-88Pro RHYTHM FX 2 Set", "Sub Marine"},
    {'P',   3,  58,  87, "SC-88Pro RHYTHM FX 2 Set", "Noise Attack"},
    {'P',   3,  58,  88, "SC-88Pro RHYTHM FX 2 Set", "Space Worms"},
    {'P',   3,  58,  89, "SC-88Pro RHYTHM FX 2 Set", "Emergency!"},
    {'P',   3,  58,  90, "SC-88Pro RHYTHM FX 2 Set", "Calculating..."},
    {'P',   3,  58,  91, "SC-88Pro RHYTHM FX 2 Set", "Saw LFO"},
    {'P',   4,   0,   0, "SC-8850 STANDARD 1 Set", "[88] Standard 1 Kick 1"},
    {'P',   4,   0,   1, "SC-8850 STANDARD 1 Set", "[88] Standard 1 Kick 2"},
    {'P',   4,   0,   2, "SC-8850 STANDARD 1 Set", "[88] Standard 2 Kick 1"},
    {'P',   4,   0,   3, "SC-8850 STANDARD 1 Set", "[88] Standard 2 Kick 2"},
    {'P',   4,   0,   4, "SC-8850 STANDARD 1 Set", "[55] Kick Drum 1"},
    {'P',   4,   0,   5, "SC-8850 STANDARD 1 Set", "[55] Kick Drum 2"},
    {'P',   4,   0,   6, "SC-8850 STANDARD 1 Set", "[88] Jazz Kick 1"},
    {'P',   4,   0,   7, "SC-8850 STANDARD 1 Set", "[88] Jazz Kick 2"},
    {'P',   4,   0,   8, "SC-8850 STANDARD 1 Set", "[88] Room Kick 1"},
    {'P',   4,   0,   9, "SC-8850 STANDARD 1 Set", "[88] Room Kick 2"},
    {'P',   4,   0,  10, "SC-8850 STANDARD 1 Set", "[88] Power Kick 1"},
    {'P',   4,   0,  11, "SC-8850 STANDARD 1 Set", "[88] Power Kick 2"},
    {'P',   4,   0,  12, "SC-8850 STANDARD 1 Set", "[88] Electric Kick 2"},
    {'P',   4,   0,  13, "SC-8850 STANDARD 1 Set", "[88] Electric Kick 1"},
    {'P',   4,   0,  14, "SC-8850 STANDARD 1 Set", "[88] TR-808 Kick"},
    {'P',   4,   0,  15, "SC-8850 STANDARD 1 Set", "[88] TR-909 Kick"},
    {'P',   4,   0,  16, "SC-8850 STANDARD 1 Set", "[88] Dance Kick"},
    {'P',   4,   0,  17, "SC-8850 STANDARD 1 Set", "[Pro] Voice One"},
    {'P',   4,   0,  18, "SC-8850 STANDARD 1 Set", "[Pro] Voice Two"},
    {'P',   4,   0,  19, "SC-8850 STANDARD 1 Set", "[Pro] Voice Three"},
    {'P',   4,   0,  20, "SC-8850 STANDARD 1 Set", "Room Kick 2"},
    {'P',   4,   0,  21, "SC-8850 STANDARD 1 Set", "Room Kick 1"},
    {'P',   4,   0,  22, "SC-8850 STANDARD 1 Set", "MC-500 Beep 1"},
    {'P',   4,   0,  23, "SC-8850 STANDARD 1 Set", "MC-500 Beep 2"},
    {'P',   4,   0,  24, "SC-8850 STANDARD 1 Set", "Concert SD"},
    {'P',   4,   0,  25, "SC-8850 STANDARD 1 Set", "Snare Roll"},
    {'P',   4,   0,  26, "SC-8850 STANDARD 1 Set", "Finger Snap 2"},
    {'P',   4,   0,  27, "SC-8850 STANDARD 1 Set", "High Q"},
    {'P',   4,   0,  28, "SC-8850 STANDARD 1 Set", "Slap"},
    {'P',   4,   0,  29, "SC-8850 STANDARD 1 Set", "Scratch Push"},
    {'P',   4,   0,  30, "SC-8850 STANDARD 1 Set", "Scratch Pull"},
    {'P',   4,   0,  31, "SC-8850 STANDARD 1 Set", "Sticks"},
    {'P',   4,   0,  32, "SC-8850 STANDARD 1 Set", "Square Click"},
    {'P',   4,   0,  33, "SC-8850 STANDARD 1 Set", "Metronome Click"},
    {'P',   4,   0,  34, "SC-8850 STANDARD 1 Set", "Metronome Bell"},
    {'P',   4,   0,  35, "SC-8850 STANDARD 1 Set", "Standard 1 Kick 2"},
    {'P',   4,   0,  36, "SC-8850 STANDARD 1 Set", "Standard 1 Kick 1"},
    {'P',   4,   0,  37, "SC-8850 STANDARD 1 Set", "Side Stick"},
    {'P',   4,   0,  38, "SC-8850 STANDARD 1 Set", "Standard 1 Snare 1"},
    {'P',   4,   0,  39, "SC-8850 STANDARD 1 Set", "TR-909 Hand Clap"},
    {'P',   4,   0,  40, "SC-8850 STANDARD 1 Set", "Standard 1 Snare 2"},
    {'P',   4,   0,  41, "SC-8850 STANDARD 1 Set", "Low Tom 2"},
    {'P',   4,   0,  42, "SC-8850 STANDARD 1 Set", "Closed Hi-Hat 1"},
    {'P',   4,   0,  43, "SC-8850 STANDARD 1 Set", "Low Tom 1"},
    {'P',   4,   0,  44, "SC-8850 STANDARD 1 Set", "Pedal Hi-Hat"},
    {'P',   4,   0,  45, "SC-8850 STANDARD 1 Set", "Mid Tom 2"},
    {'P',   4,   0,  46, "SC-8850 STANDARD 1 Set", "Open Hi-Hat 1"},
    {'P',   4,   0,  47, "SC-8850 STANDARD 1 Set", "Mid Tom 1"},
    {'P',   4,   0,  48, "SC-8850 STANDARD 1 Set", "High Tom 2"},
    {'P',   4,   0,  49, "SC-8850 STANDARD 1 Set", "Crash Cymbal 1"},
    {'P',   4,   0,  50, "SC-8850 STANDARD 1 Set", "High Tom 1"},
    {'P',   4,   0,  51, "SC-8850 STANDARD 1 Set", "Ride Cymbal 1"},
    {'P',   4,   0,  52, "SC-8850 STANDARD 1 Set", "Chinese Cymbal"},
    {'P',   4,   0,  53, "SC-8850 STANDARD 1 Set", "Ride Bell"},
    {'P',   4,   0,  54, "SC-8850 STANDARD 1 Set", "Tambourine"},
    {'P',   4,   0,  55, "SC-8850 STANDARD 1 Set", "Splash Cymbal"},
    {'P',   4,   0,  56, "SC-8850 STANDARD 1 Set", "Cowbell"},
    {'P',   4,   0,  57, "SC-8850 STANDARD 1 Set", "Crash Cymbal 2"},
    {'P',   4,   0,  58, "SC-8850 STANDARD 1 Set", "Vibra-slap"},
    {'P',   4,   0,  59, "SC-8850 STANDARD 1 Set", "Ride Cymbal 2"},
    {'P',   4,   0,  60, "SC-8850 STANDARD 1 Set", "High Bongo"},
    {'P',   4,   0,  61, "SC-8850 STANDARD 1 Set", "Low Bongo"},
    {'P',   4,   0,  62, "SC-8850 STANDARD 1 Set", "Mute High Conga"},
    {'P',   4,   0,  63, "SC-8850 STANDARD 1 Set", "Open High Conga"},
    {'P',   4,   0,  64, "SC-8850 STANDARD 1 Set", "Low Conga"},
    {'P',   4,   0,  65, "SC-8850 STANDARD 1 Set", "High Timbale"},
    {'P',   4,   0,  66, "SC-8850 STANDARD 1 Set", "Low Timbale"},
    {'P',   4,   0,  67, "SC-8850 STANDARD 1 Set", "High Agogo"},
    {'P',   4,   0,  68, "SC-8850 STANDARD 1 Set", "Low Agogo"},
    {'P',   4,   0,  69, "SC-8850 STANDARD 1 Set", "Cabasa"},
    {'P',   4,   0,  70, "SC-8850 STANDARD 1 Set", "Maracas"},
    {'P',   4,   0,  71, "SC-8850 STANDARD 1 Set", "Short High Whistle"},
    {'P',   4,   0,  72, "SC-8850 STANDARD 1 Set", "Long Low Whistle"},
    {'P',   4,   0,  73, "SC-8850 STANDARD 1 Set", "Short Guiro"},
    {'P',   4,   0,  74, "SC-8850 STANDARD 1 Set", "Long Guiro"},
    {'P',   4,   0,  75, "SC-8850 STANDARD 1 Set", "Claves"},
    {'P',   4,   0,  76, "SC-8850 STANDARD 1 Set", "High Wood Block"},
    {'P',   4,   0,  77, "SC-8850 STANDARD 1 Set", "Low Wood Block"},
    {'P',   4,   0,  78, "SC-8850 STANDARD 1 Set", "Mute Cuica"},
    {'P',   4,   0,  79, "SC-8850 STANDARD 1 Set", "Open Cuica"},
    {'P',   4,   0,  80, "SC-8850 STANDARD 1 Set", "Mute Triangle"},
    {'P',   4,   0,  81, "SC-8850 STANDARD 1 Set", "Open Triangle"},
    {'P',   4,   0,  82, "SC-8850 STANDARD 1 Set", "Shaker"},
    {'P',   4,   0,  83, "SC-8850 STANDARD 1 Set", "Jingle Bell"},
    {'P',   4,   0,  84, "SC-8850 STANDARD 1 Set", "Bell Tree"},
    {'P',   4,   0,  85, "SC-8850 STANDARD 1 Set", "Castanets"},
    {'P',   4,   0,  86, "SC-8850 STANDARD 1 Set", "Mute Surdo"},
    {'P',   4,   0,  87, "SC-8850 STANDARD 1 Set", "Open Surdo"},
    {'P',   4,   0,  88, "SC-8850 STANDARD 1 Set", "Applause 2"},
    {'P',   4,   0,  95, "SC-8850 STANDARD 1 Set", "Room Snare 1"},
    {'P',   4,   0,  96, "SC-8850 STANDARD 1 Set", "Room Snare 2"},
    {'P',   4,   0,  97, "SC-8850 STANDARD 1 Set", "[88] Standard 1 Snare1"},
    {'P',   4,   0,  98, "SC-8850 STANDARD 1 Set", "[88] Standard 1 Snare 2"},
    {'P',   4,   0,  99, "SC-8850 STANDARD 1 Set", "[88] Standard 2 Snare 1"},
    {'P',   4,   0, 100, "SC-8850 STANDARD 1 Set", "[88] Standard 2 Snare 2"},
    {'P',   4,   0, 101, "SC-8850 STANDARD 1 Set", "[55] Snare Drum 2"},
    {'P',   4,   0, 102, "SC-8850 STANDARD 1 Set", "[Pro] Standard 1 Snare 1"},
    {'P',   4,   0, 103, "SC-8850 STANDARD 1 Set", "[Pro] Standard 1 Snare 2"},
    {'P',   4,   0, 104, "SC-8850 STANDARD 1 Set", "[Pro] Standard 1 Snare 3"},
    {'P',   4,   0, 105, "SC-8850 STANDARD 1 Set", "[88] Jazz Snare 1"},
    {'P',   4,   0, 106, "SC-8850 STANDARD 1 Set", "[88] Jazz Snare 2"},
    {'P',   4,   0, 107, "SC-8850 STANDARD 1 Set", "[88] Room Snare 1"},
    {'P',   4,   0, 108, "SC-8850 STANDARD 1 Set", "[88] Room Snare 2"},
    {'P',   4,   0, 109, "SC-8850 STANDARD 1 Set", "[88] Power Snare 1"},
    {'P',   4,   0, 110, "SC-8850 STANDARD 1 Set", "[88] Power Snare 2"},
    {'P',   4,   0, 111, "SC-8850 STANDARD 1 Set", "[55] Gated Snare"},
    {'P',   4,   0, 112, "SC-8850 STANDARD 1 Set", "[88] Dance Snare 1"},
    {'P',   4,   0, 113, "SC-8850 STANDARD 1 Set", "[88] Dance Snare 2"},
    {'P',   4,   0, 114, "SC-8850 STANDARD 1 Set", "[88] Disco Snare"},
    {'P',   4,   0, 115, "SC-8850 STANDARD 1 Set", "[88] Electric Snare 2"},
    {'P',   4,   0, 116, "SC-8850 STANDARD 1 Set", "[55] Electric Snare"},
    {'P',   4,   0, 117, "SC-8850 STANDARD 1 Set", "[88] Electric Snare 3"},
    {'P',   4,   0, 118, "SC-8850 STANDARD 1 Set", "[Pro] TR-707 Snare 1"},
    {'P',   4,   0, 119, "SC-8850 STANDARD 1 Set", "[88] TR-808 Snare 1"},
    {'P',   4,   0, 120, "SC-8850 STANDARD 1 Set", "[88] TR-808 Snare 2"},
    {'P',   4,   0, 121, "SC-8850 STANDARD 1 Set", "[88] TR-909 Snare 1"},
    {'P',   4,   0, 122, "SC-8850 STANDARD 1 Set", "[88] TR-909 Snare 2"},
    {'P',   4,   0, 123, "SC-8850 STANDARD 1 Set", "[Pro] Rap Snare"},
    {'P',   4,   0, 124, "SC-8850 STANDARD 1 Set", "[Pro] Jungle Snare 1"},
    {'P',   4,   0, 125, "SC-8850 STANDARD 1 Set", "[Pro] House Snare 1"},
    {'P',   4,   0, 126, "SC-8850 STANDARD 1 Set", "[88] House Snare"},
    {'P',   4,   0, 127, "SC-8850 STANDARD 1 Set", "[Pro] House Snare 2"},
    {'P',   4,   1,   0, "SC-8850 STANDARD 2 Set", "[88] Standard 1 Kick 1"},
    {'P',   4,   1,   1, "SC-8850 STANDARD 2 Set", "[88] Standard 1 Kick 2"},
    {'P',   4,   1,   2, "SC-8850 STANDARD 2 Set", "[88] Standard 2 Kick 1"},
    {'P',   4,   1,   3, "SC-8850 STANDARD 2 Set", "[88] Standard 2 Kick 2"},
    {'P',   4,   1,   4, "SC-8850 STANDARD 2 Set", "[55] Kick Drum 1"},
    {'P',   4,   1,   5, "SC-8850 STANDARD 2 Set", "[55] Kick Drum 2"},
    {'P',   4,   1,   6, "SC-8850 STANDARD 2 Set", "[88] Jazz Kick 1"},
    {'P',   4,   1,   7, "SC-8850 STANDARD 2 Set", "[88] Jazz Kick 2"},
    {'P',   4,   1,   8, "SC-8850 STANDARD 2 Set", "[88] Room Kick 1"},
    {'P',   4,   1,   9, "SC-8850 STANDARD 2 Set", "[88] Room Kick 2"},
    {'P',   4,   1,  10, "SC-8850 STANDARD 2 Set", "[88] Power Kick 1"},
    {'P',   4,   1,  11, "SC-8850 STANDARD 2 Set", "[88] Power Kick 2"},
    {'P',   4,   1,  12, "SC-8850 STANDARD 2 Set", "[88] Electric Kick 2"},
    {'P',   4,   1,  13, "SC-8850 STANDARD 2 Set", "[88] Electric Kick 1"},
    {'P',   4,   1,  14, "SC-8850 STANDARD 2 Set", "[88] TR-808 Kick"},
    {'P',   4,   1,  15, "SC-8850 STANDARD 2 Set", "[88] TR-909 Kick"},
    {'P',   4,   1,  16, "SC-8850 STANDARD 2 Set", "[88] Dance Kick"},
    {'P',   4,   1,  17, "SC-8850 STANDARD 2 Set", "[Pro] Voice One"},
    {'P',   4,   1,  18, "SC-8850 STANDARD 2 Set", "[Pro] Voice Two"},
    {'P',   4,   1,  19, "SC-8850 STANDARD 2 Set", "[Pro] Voice Three"},
    {'P',   4,   1,  22, "SC-8850 STANDARD 2 Set", "MC-500 Beep 1"},
    {'P',   4,   1,  23, "SC-8850 STANDARD 2 Set", "MC-500 Beep 2"},
    {'P',   4,   1,  24, "SC-8850 STANDARD 2 Set", "Concert SD"},
    {'P',   4,   1,  25, "SC-8850 STANDARD 2 Set", "Snare Roll"},
    {'P',   4,   1,  26, "SC-8850 STANDARD 2 Set", "Finger Snap"},
    {'P',   4,   1,  27, "SC-8850 STANDARD 2 Set", "High Q"},
    {'P',   4,   1,  28, "SC-8850 STANDARD 2 Set", "Slap"},
    {'P',   4,   1,  29, "SC-8850 STANDARD 2 Set", "Scratch Push"},
    {'P',   4,   1,  30, "SC-8850 STANDARD 2 Set", "Scratch Pull"},
    {'P',   4,   1,  31, "SC-8850 STANDARD 2 Set", "Sticks"},
    {'P',   4,   1,  32, "SC-8850 STANDARD 2 Set", "Square Click"},
    {'P',   4,   1,  33, "SC-8850 STANDARD 2 Set", "Metronome Click"},
    {'P',   4,   1,  34, "SC-8850 STANDARD 2 Set", "Metronome Bell"},
    {'P',   4,   1,  35, "SC-8850 STANDARD 2 Set", "Standard 2 Kick 2"},
    {'P',   4,   1,  36, "SC-8850 STANDARD 2 Set", "Standard 2 Kick 1"},
    {'P',   4,   1,  37, "SC-8850 STANDARD 2 Set", "Side Stick"},
    {'P',   4,   1,  38, "SC-8850 STANDARD 2 Set", "Standard 2 Snare 1"},
    {'P',   4,   1,  39, "SC-8850 STANDARD 2 Set", "Hand Clap"},
    {'P',   4,   1,  40, "SC-8850 STANDARD 2 Set", "Standard 2 Snare 2"},
    {'P',   4,   1,  41, "SC-8850 STANDARD 2 Set", "Low Tom 2"},
    {'P',   4,   1,  42, "SC-8850 STANDARD 2 Set", "Closed Hi-Hat"},
    {'P',   4,   1,  43, "SC-8850 STANDARD 2 Set", "Low Tom 1"},
    {'P',   4,   1,  44, "SC-8850 STANDARD 2 Set", "Pedal Hi-Hat"},
    {'P',   4,   1,  45, "SC-8850 STANDARD 2 Set", "Mid Tom 2"},
    {'P',   4,   1,  46, "SC-8850 STANDARD 2 Set", "Open Hi-Hat"},
    {'P',   4,   1,  47, "SC-8850 STANDARD 2 Set", "Mid Tom 1"},
    {'P',   4,   1,  48, "SC-8850 STANDARD 2 Set", "High Tom 2"},
    {'P',   4,   1,  49, "SC-8850 STANDARD 2 Set", "Crash Cymbal1"},
    {'P',   4,   1,  50, "SC-8850 STANDARD 2 Set", "High Tom 1"},
    {'P',   4,   1,  51, "SC-8850 STANDARD 2 Set", "Ride Cymbal 1"},
    {'P',   4,   1,  52, "SC-8850 STANDARD 2 Set", "Chinese Cymbal"},
    {'P',   4,   1,  53, "SC-8850 STANDARD 2 Set", "Ride Bell"},
    {'P',   4,   1,  54, "SC-8850 STANDARD 2 Set", "Tambourine"},
    {'P',   4,   1,  55, "SC-8850 STANDARD 2 Set", "Splash Cymbal"},
    {'P',   4,   1,  56, "SC-8850 STANDARD 2 Set", "Cowbell"},
    {'P',   4,   1,  57, "SC-8850 STANDARD 2 Set", "Crash Cymbal 2"},
    {'P',   4,   1,  58, "SC-8850 STANDARD 2 Set", "Vibra-slap"},
    {'P',   4,   1,  59, "SC-8850 STANDARD 2 Set", "Ride Cymbal 2"},
    {'P',   4,   1,  60, "SC-8850 STANDARD 2 Set", "High Bongo"},
    {'P',   4,   1,  61, "SC-8850 STANDARD 2 Set", "Low Bongo"},
    {'P',   4,   1,  62, "SC-8850 STANDARD 2 Set", "Mute High Conga"},
    {'P',   4,   1,  63, "SC-8850 STANDARD 2 Set", "Open High Conga"},
    {'P',   4,   1,  64, "SC-8850 STANDARD 2 Set", "Low Conga"},
    {'P',   4,   1,  65, "SC-8850 STANDARD 2 Set", "High Timbale"},
    {'P',   4,   1,  66, "SC-8850 STANDARD 2 Set", "Low Timbale"},
    {'P',   4,   1,  67, "SC-8850 STANDARD 2 Set", "High Agogo"},
    {'P',   4,   1,  68, "SC-8850 STANDARD 2 Set", "Low Agogo"},
    {'P',   4,   1,  69, "SC-8850 STANDARD 2 Set", "Cabasa"},
    {'P',   4,   1,  70, "SC-8850 STANDARD 2 Set", "Maracas"},
    {'P',   4,   1,  71, "SC-8850 STANDARD 2 Set", "Short High Whistle"},
    {'P',   4,   1,  72, "SC-8850 STANDARD 2 Set", "Long Low Whistle"},
    {'P',   4,   1,  73, "SC-8850 STANDARD 2 Set", "Short Guiro"},
    {'P',   4,   1,  74, "SC-8850 STANDARD 2 Set", "Long Guiro"},
    {'P',   4,   1,  75, "SC-8850 STANDARD 2 Set", "Claves"},
    {'P',   4,   1,  76, "SC-8850 STANDARD 2 Set", "High Wood Block"},
    {'P',   4,   1,  77, "SC-8850 STANDARD 2 Set", "Low Wood Block"},
    {'P',   4,   1,  78, "SC-8850 STANDARD 2 Set", "Mute Cuica"},
    {'P',   4,   1,  79, "SC-8850 STANDARD 2 Set", "Open Cuica"},
    {'P',   4,   1,  80, "SC-8850 STANDARD 2 Set", "Mute Triangle"},
    {'P',   4,   1,  81, "SC-8850 STANDARD 2 Set", "Open Triangle"},
    {'P',   4,   1,  82, "SC-8850 STANDARD 2 Set", "Shaker"},
    {'P',   4,   1,  83, "SC-8850 STANDARD 2 Set", "Jingle Bell"},
    {'P',   4,   1,  84, "SC-8850 STANDARD 2 Set", "Bar Chimes"},
    {'P',   4,   1,  85, "SC-8850 STANDARD 2 Set", "Castanets"},
    {'P',   4,   1,  86, "SC-8850 STANDARD 2 Set", "Mute Surdo"},
    {'P',   4,   1,  87, "SC-8850 STANDARD 2 Set", "Open Surdo"},
    {'P',   4,   1,  88, "SC-8850 STANDARD 2 Set", "Applause 2"},
    {'P',   4,   1,  97, "SC-8850 STANDARD 2 Set", "[88] Standard 1 Snare1"},
    {'P',   4,   1,  98, "SC-8850 STANDARD 2 Set", "[88] Standard 1 Snare 2"},
    {'P',   4,   1,  99, "SC-8850 STANDARD 2 Set", "[88] Standard 2 Snare 1"},
    {'P',   4,   1, 100, "SC-8850 STANDARD 2 Set", "[88] Standard 2 Snare 2"},
    {'P',   4,   1, 101, "SC-8850 STANDARD 2 Set", "[55] Snare Drum 2"},
    {'P',   4,   1, 102, "SC-8850 STANDARD 2 Set", "[Pro] Standard 1 Snare 1"},
    {'P',   4,   1, 103, "SC-8850 STANDARD 2 Set", "[Pro] Standard 1 Snare 2"},
    {'P',   4,   1, 104, "SC-8850 STANDARD 2 Set", "[Pro] Standard 1 Snare 3"},
    {'P',   4,   1, 105, "SC-8850 STANDARD 2 Set", "[88] Jazz Snare 1"},
    {'P',   4,   1, 106, "SC-8850 STANDARD 2 Set", "[88] Jazz Snare 2"},
    {'P',   4,   1, 107, "SC-8850 STANDARD 2 Set", "[88] Room Snare 1"},
    {'P',   4,   1, 108, "SC-8850 STANDARD 2 Set", "[88] Room Snare 2"},
    {'P',   4,   1, 109, "SC-8850 STANDARD 2 Set", "[88] Power Snare 1"},
    {'P',   4,   1, 110, "SC-8850 STANDARD 2 Set", "[88] Power Snare 2"},
    {'P',   4,   1, 111, "SC-8850 STANDARD 2 Set", "[55] Gated Snare"},
    {'P',   4,   1, 112, "SC-8850 STANDARD 2 Set", "[88] Dance Snare 1"},
    {'P',   4,   1, 113, "SC-8850 STANDARD 2 Set", "[88] Dance Snare 2"},
    {'P',   4,   1, 114, "SC-8850 STANDARD 2 Set", "[88] Disco Snare"},
    {'P',   4,   1, 115, "SC-8850 STANDARD 2 Set", "[88] Electric Snare 2"},
    {'P',   4,   1, 116, "SC-8850 STANDARD 2 Set", "[55] Electric Snare"},
    {'P',   4,   1, 117, "SC-8850 STANDARD 2 Set", "[88] Electric Snare 3"},
    {'P',   4,   1, 118, "SC-8850 STANDARD 2 Set", "[Pro] TR-707 Snare 1"},
    {'P',   4,   1, 119, "SC-8850 STANDARD 2 Set", "[88] TR-808 Snare 1"},
    {'P',   4,   1, 120, "SC-8850 STANDARD 2 Set", "[88] TR-808 Snare 2"},
    {'P',   4,   1, 121, "SC-8850 STANDARD 2 Set", "[88] TR-909 Snare 1"},
    {'P',   4,   1, 122, "SC-8850 STANDARD 2 Set", "[88] TR-909 Snare 2"},
    {'P',   4,   1, 123, "SC-8850 STANDARD 2 Set", "[Pro] Rap Snare"},
    {'P',   4,   1, 124, "SC-8850 STANDARD 2 Set", "[Pro] Jungle Snare 1"},
    {'P',   4,   1, 125, "SC-8850 STANDARD 2 Set", "[Pro] House Snare 1"},
    {'P',   4,   1, 126, "SC-8850 STANDARD 2 Set", "[88] House Snare"},
    {'P',   4,   1, 127, "SC-8850 STANDARD 2 Set", "[Pro] House Snare 2"},
    {'P',   4,   2,  17, "SC-8850 STANDARD L/R Set", "[Pro] Voice One"},
    {'P',   4,   2,  18, "SC-8850 STANDARD L/R Set", "[Pro] Voice Two"},
    {'P',   4,   2,  19, "SC-8850 STANDARD L/R Set", "[Pro] Voice Three"},
    {'P',   4,   2,  22, "SC-8850 STANDARD L/R Set", "MC-500 Beep 1"},
    {'P',   4,   2,  23, "SC-8850 STANDARD L/R Set", "MC-500 Beep 2"},
    {'P',   4,   2,  24, "SC-8850 STANDARD L/R Set", "Concert SD"},
    {'P',   4,   2,  25, "SC-8850 STANDARD L/R Set", "Snare Roll"},
    {'P',   4,   2,  26, "SC-8850 STANDARD L/R Set", "Finger Snap 2"},
    {'P',   4,   2,  27, "SC-8850 STANDARD L/R Set", "High Q"},
    {'P',   4,   2,  28, "SC-8850 STANDARD L/R Set", "Slap"},
    {'P',   4,   2,  29, "SC-8850 STANDARD L/R Set", "Scratch Push"},
    {'P',   4,   2,  30, "SC-8850 STANDARD L/R Set", "Scratch Pull"},
    {'P',   4,   2,  31, "SC-8850 STANDARD L/R Set", "Sticks"},
    {'P',   4,   2,  32, "SC-8850 STANDARD L/R Set", "Square Click"},
    {'P',   4,   2,  33, "SC-8850 STANDARD L/R Set", "Metronome Click"},
    {'P',   4,   2,  34, "SC-8850 STANDARD L/R Set", "Metronome Bell"},
    {'P',   4,   2,  35, "SC-8850 STANDARD L/R Set", "[RND] Standard Kick 2"},
    {'P',   4,   2,  36, "SC-8850 STANDARD L/R Set", "[RND] Standard Kick 1"},
    {'P',   4,   2,  37, "SC-8850 STANDARD L/R Set", "Side Stick"},
    {'P',   4,   2,  38, "SC-8850 STANDARD L/R Set", "[RND] Standard Snare 1"},
    {'P',   4,   2,  39, "SC-8850 STANDARD L/R Set", "Hand Clap"},
    {'P',   4,   2,  40, "SC-8850 STANDARD L/R Set", "[RND] Standard Snare 2"},
    {'P',   4,   2,  41, "SC-8850 STANDARD L/R Set", "[RND] Low Tom 2"},
    {'P',   4,   2,  42, "SC-8850 STANDARD L/R Set", "[RND] Closed Hi-Hat"},
    {'P',   4,   2,  43, "SC-8850 STANDARD L/R Set", "[RND] Low Tom 1"},
    {'P',   4,   2,  44, "SC-8850 STANDARD L/R Set", "Pedal Hi-Hat"},
    {'P',   4,   2,  45, "SC-8850 STANDARD L/R Set", "[RND] Mid Tom 2"},
    {'P',   4,   2,  46, "SC-8850 STANDARD L/R Set", "[RND] Open Hi-Hat"},
    {'P',   4,   2,  47, "SC-8850 STANDARD L/R Set", "[RND] Mid Tom 1"},
    {'P',   4,   2,  48, "SC-8850 STANDARD L/R Set", "[RND] High Tom 2"},
    {'P',   4,   2,  49, "SC-8850 STANDARD L/R Set", "[RND] Crash Cymbal"},
    {'P',   4,   2,  50, "SC-8850 STANDARD L/R Set", "[RND] High Tom 1"},
    {'P',   4,   2,  51, "SC-8850 STANDARD L/R Set", "[RND] Ride Cymbal 1"},
    {'P',   4,   2,  52, "SC-8850 STANDARD L/R Set", "Chinese Cymbal"},
    {'P',   4,   2,  53, "SC-8850 STANDARD L/R Set", "[RND] Ride Bell 1"},
    {'P',   4,   2,  54, "SC-8850 STANDARD L/R Set", "Tambourine"},
    {'P',   4,   2,  55, "SC-8850 STANDARD L/R Set", "Splash Cymbal"},
    {'P',   4,   2,  56, "SC-8850 STANDARD L/R Set", "Cowbell"},
    {'P',   4,   2,  57, "SC-8850 STANDARD L/R Set", "Crash Cymbal 2"},
    {'P',   4,   2,  58, "SC-8850 STANDARD L/R Set", "Vibra-slap"},
    {'P',   4,   2,  59, "SC-8850 STANDARD L/R Set", "Ride Cymbal 2"},
    {'P',   4,   2,  60, "SC-8850 STANDARD L/R Set", "High Bongo"},
    {'P',   4,   2,  61, "SC-8850 STANDARD L/R Set", "Low Bongo"},
    {'P',   4,   2,  62, "SC-8850 STANDARD L/R Set", "Mute High Conga"},
    {'P',   4,   2,  63, "SC-8850 STANDARD L/R Set", "Open High Conga"},
    {'P',   4,   2,  64, "SC-8850 STANDARD L/R Set", "Low Conga"},
    {'P',   4,   2,  65, "SC-8850 STANDARD L/R Set", "High Timbale"},
    {'P',   4,   2,  66, "SC-8850 STANDARD L/R Set", "Low Timbale"},
    {'P',   4,   2,  67, "SC-8850 STANDARD L/R Set", "High Agogo"},
    {'P',   4,   2,  68, "SC-8850 STANDARD L/R Set", "Low Agogo"},
    {'P',   4,   2,  69, "SC-8850 STANDARD L/R Set", "Cabasa"},
    {'P',   4,   2,  70, "SC-8850 STANDARD L/R Set", "Maracas"},
    {'P',   4,   2,  71, "SC-8850 STANDARD L/R Set", "Short High Whistle"},
    {'P',   4,   2,  72, "SC-8850 STANDARD L/R Set", "Long Low Whistle"},
    {'P',   4,   2,  73, "SC-8850 STANDARD L/R Set", "Short Guiro"},
    {'P',   4,   2,  74, "SC-8850 STANDARD L/R Set", "Long Guiro"},
    {'P',   4,   2,  75, "SC-8850 STANDARD L/R Set", "Claves"},
    {'P',   4,   2,  76, "SC-8850 STANDARD L/R Set", "High Wood Block"},
    {'P',   4,   2,  77, "SC-8850 STANDARD L/R Set", "Low Wood Block"},
    {'P',   4,   2,  78, "SC-8850 STANDARD L/R Set", "Mute Cuica"},
    {'P',   4,   2,  79, "SC-8850 STANDARD L/R Set", "Open Cuica"},
    {'P',   4,   2,  80, "SC-8850 STANDARD L/R Set", "Mute Triangle"},
    {'P',   4,   2,  81, "SC-8850 STANDARD L/R Set", "Open Triangle"},
    {'P',   4,   2,  82, "SC-8850 STANDARD L/R Set", "Shaker"},
    {'P',   4,   2,  83, "SC-8850 STANDARD L/R Set", "Jingle Bell"},
    {'P',   4,   2,  84, "SC-8850 STANDARD L/R Set", "Bell Tree"},
    {'P',   4,   2,  85, "SC-8850 STANDARD L/R Set", "Castanets"},
    {'P',   4,   2,  86, "SC-8850 STANDARD L/R Set", "Mute Surdo"},
    {'P',   4,   2,  87, "SC-8850 STANDARD L/R Set", "Open Surdo"},
    {'P',   4,   2,  88, "SC-8850 STANDARD L/R Set", "Applause 2"},
    {'P',   4,   2,  95, "SC-8850 STANDARD L/R Set", "[L] Standard Kick 2"},
    {'P',   4,   2,  96, "SC-8850 STANDARD L/R Set", "[L] Standard Kick 1"},
    {'P',   4,   2,  97, "SC-8850 STANDARD L/R Set", "[L] Standard Crash Cymbal"},
    {'P',   4,   2,  98, "SC-8850 STANDARD L/R Set", "[L] Standard Snare 1"},
    {'P',   4,   2,  99, "SC-8850 STANDARD L/R Set", "[L] Standard Ride Cymbal"},
    {'P',   4,   2, 100, "SC-8850 STANDARD L/R Set", "[L] Standard Snare 2"},
    {'P',   4,   2, 101, "SC-8850 STANDARD L/R Set", "[L] Standard Low Tom"},
    {'P',   4,   2, 102, "SC-8850 STANDARD L/R Set", "[L] Standard Closed Hi-Hat"},
    {'P',   4,   2, 103, "SC-8850 STANDARD L/R Set", "[L] Standard Mid Tom"},
    {'P',   4,   2, 104, "SC-8850 STANDARD L/R Set", "[L] Standard Ride Bell"},
    {'P',   4,   2, 105, "SC-8850 STANDARD L/R Set", "[L] Standard High Tom"},
    {'P',   4,   2, 106, "SC-8850 STANDARD L/R Set", "[L] Standard Open Hi-Hat"},
    {'P',   4,   2, 107, "SC-8850 STANDARD L/R Set", "[R] Standard Kick 2"},
    {'P',   4,   2, 108, "SC-8850 STANDARD L/R Set", "[R] Standard Kick 1"},
    {'P',   4,   2, 109, "SC-8850 STANDARD L/R Set", "[R] Standard Crash Cymbal"},
    {'P',   4,   2, 110, "SC-8850 STANDARD L/R Set", "[R] Standard Snare 1"},
    {'P',   4,   2, 111, "SC-8850 STANDARD L/R Set", "[R] Standard Ride Cymbal"},
    {'P',   4,   2, 112, "SC-8850 STANDARD L/R Set", "[R] Standard Snare 2"},
    {'P',   4,   2, 113, "SC-8850 STANDARD L/R Set", "[R] Standard Low Tom"},
    {'P',   4,   2, 114, "SC-8850 STANDARD L/R Set", "[R] Standard Closed Hi-Hat"},
    {'P',   4,   2, 115, "SC-8850 STANDARD L/R Set", "[R] Standard Mid Tom"},
    {'P',   4,   2, 116, "SC-8850 STANDARD L/R Set", "[R] Standard Ride Bell"},
    {'P',   4,   2, 117, "SC-8850 STANDARD L/R Set", "[R] Standard High Tom"},
    {'P',   4,   2, 118, "SC-8850 STANDARD L/R Set", "[R] Standard Open Hi-Hat"},
    {'P',   4,   8,  20, "SC-8850 ROOM Set", "Standard Kick 2"},
    {'P',   4,   8,  21, "SC-8850 ROOM Set", "Standard Kick 1"},
    {'P',   4,   8,  26, "SC-8850 ROOM Set", "Finger Snap"},
    {'P',   4,   8,  29, "SC-8850 ROOM Set", "Scratch Push"},
    {'P',   4,   8,  30, "SC-8850 ROOM Set", "Scratch Pull"},
    {'P',   4,   8,  35, "SC-8850 ROOM Set", "Room Kick 2"},
    {'P',   4,   8,  36, "SC-8850 ROOM Set", "Room Kick 1"},
    {'P',   4,   8,  37, "SC-8850 ROOM Set", "Side Stick"},
    {'P',   4,   8,  38, "SC-8850 ROOM Set", "Room Snare 1"},
    {'P',   4,   8,  39, "SC-8850 ROOM Set", "Hand Clap"},
    {'P',   4,   8,  40, "SC-8850 ROOM Set", "Room Snare 2"},
    {'P',   4,   8,  41, "SC-8850 ROOM Set", "Room Low Tom 2"},
    {'P',   4,   8,  42, "SC-8850 ROOM Set", "Room Closed Hi-Hat 3"},
    {'P',   4,   8,  43, "SC-8850 ROOM Set", "Room Low Tom 1"},
    {'P',   4,   8,  44, "SC-8850 ROOM Set", "Pedal Hi-Hat"},
    {'P',   4,   8,  45, "SC-8850 ROOM Set", "Room Mid Tom 2"},
    {'P',   4,   8,  46, "SC-8850 ROOM Set", "Room Open Hi-Hat 3"},
    {'P',   4,   8,  47, "SC-8850 ROOM Set", "Room Mid Tom 1"},
    {'P',   4,   8,  48, "SC-8850 ROOM Set", "Room High Tom 2"},
    {'P',   4,   8,  49, "SC-8850 ROOM Set", "Room Crash Cymbal"},
    {'P',   4,   8,  50, "SC-8850 ROOM Set", "Room High Tom 1"},
    {'P',   4,   8,  51, "SC-8850 ROOM Set", "Room Ride Cymbal"},
    {'P',   4,   8,  52, "SC-8850 ROOM Set", "Chinese Cymbal"},
    {'P',   4,   8,  53, "SC-8850 ROOM Set", "Room Ride Bell"},
    {'P',   4,   8,  95, "SC-8850 ROOM Set", "Standard 1 Snare 1"},
    {'P',   4,   8,  96, "SC-8850 ROOM Set", "Standard 1 Snare 2"},
    {'P',   4,   9,   0, "SC-8850 HIP HOP Set", "[88] Electric Kick 2"},
    {'P',   4,   9,   1, "SC-8850 HIP HOP Set", "[88] Electric Kick 1"},
    {'P',   4,   9,   2, "SC-8850 HIP HOP Set", "[Pro] CR-78 Kick 1"},
    {'P',   4,   9,   3, "SC-8850 HIP HOP Set", "[Pro] CR-78 Kick 2"},
    {'P',   4,   9,   4, "SC-8850 HIP HOP Set", "[Pro] TR-606 Kick1"},
    {'P',   4,   9,   5, "SC-8850 HIP HOP Set", "[Pro] TR-707 Kick 1"},
    {'P',   4,   9,   6, "SC-8850 HIP HOP Set", "[55] TR-808 Kick"},
    {'P',   4,   9,   7, "SC-8850 HIP HOP Set", "[88] TR-808 Kick"},
    {'P',   4,   9,   8, "SC-8850 HIP HOP Set", "[Pro] TR-808 Kick 2"},
    {'P',   4,   9,   9, "SC-8850 HIP HOP Set", "[88] TR-909 Kick"},
    {'P',   4,   9,  10, "SC-8850 HIP HOP Set", "[88] Dance Kick"},
    {'P',   4,   9,  11, "SC-8850 HIP HOP Set", "[Pro] Hip-Hop Kick 2"},
    {'P',   4,   9,  12, "SC-8850 HIP HOP Set", "[Pro] TR-909 Kick 1"},
    {'P',   4,   9,  13, "SC-8850 HIP HOP Set", "[Pro] Hip-Hop Kick 3"},
    {'P',   4,   9,  14, "SC-8850 HIP HOP Set", "[Pro] Jungle Kick 1"},
    {'P',   4,   9,  15, "SC-8850 HIP HOP Set", "[Pro] Techno Kick 1"},
    {'P',   4,   9,  16, "SC-8850 HIP HOP Set", "[Pro] Bounce Kick"},
    {'P',   4,   9,  17, "SC-8850 HIP HOP Set", "[Pro] Voice One"},
    {'P',   4,   9,  18, "SC-8850 HIP HOP Set", "[Pro] Voice Two"},
    {'P',   4,   9,  19, "SC-8850 HIP HOP Set", "[Pro] Voice Three"},
    {'P',   4,   9,  20, "SC-8850 HIP HOP Set", "Jungle Kick 2"},
    {'P',   4,   9,  21, "SC-8850 HIP HOP Set", "Jungle Kick 1"},
    {'P',   4,   9,  22, "SC-8850 HIP HOP Set", "MC-500 Beep 1"},
    {'P',   4,   9,  23, "SC-8850 HIP HOP Set", "MC-500 Beep 2"},
    {'P',   4,   9,  24, "SC-8850 HIP HOP Set", "Concert SD"},
    {'P',   4,   9,  25, "SC-8850 HIP HOP Set", "Snare Roll"},
    {'P',   4,   9,  26, "SC-8850 HIP HOP Set", "Finger Snap 2"},
    {'P',   4,   9,  27, "SC-8850 HIP HOP Set", "High Q"},
    {'P',   4,   9,  28, "SC-8850 HIP HOP Set", "Slap"},
    {'P',   4,   9,  29, "SC-8850 HIP HOP Set", "Scratch Push 2"},
    {'P',   4,   9,  30, "SC-8850 HIP HOP Set", "Scratch Pull 2"},
    {'P',   4,   9,  31, "SC-8850 HIP HOP Set", "Sticks"},
    {'P',   4,   9,  32, "SC-8850 HIP HOP Set", "Square Click"},
    {'P',   4,   9,  33, "SC-8850 HIP HOP Set", "Metronome Click"},
    {'P',   4,   9,  34, "SC-8850 HIP HOP Set", "Metronome Bell"},
    {'P',   4,   9,  35, "SC-8850 HIP HOP Set", "Hip-Hop Kick 2"},
    {'P',   4,   9,  36, "SC-8850 HIP HOP Set", "Hip-Hop Kick 1"},
    {'P',   4,   9,  37, "SC-8850 HIP HOP Set", "TR-808 Rim Shot"},
    {'P',   4,   9,  38, "SC-8850 HIP HOP Set", "LoFi Snare 1"},
    {'P',   4,   9,  39, "SC-8850 HIP HOP Set", "TR-707 Claps"},
    {'P',   4,   9,  40, "SC-8850 HIP HOP Set", "LoFi Snare 2"},
    {'P',   4,   9,  41, "SC-8850 HIP HOP Set", "Jazz Low Tom 2"},
    {'P',   4,   9,  42, "SC-8850 HIP HOP Set", "Room Closed Hi-Hat"},
    {'P',   4,   9,  43, "SC-8850 HIP HOP Set", "Jazz Low Tom 1"},
    {'P',   4,   9,  44, "SC-8850 HIP HOP Set", "Pedal Hi-Hat"},
    {'P',   4,   9,  45, "SC-8850 HIP HOP Set", "Jazz Mid Tom 2"},
    {'P',   4,   9,  46, "SC-8850 HIP HOP Set", "Room Open Hi-Hat"},
    {'P',   4,   9,  47, "SC-8850 HIP HOP Set", "Jazz Mid Tom 1"},
    {'P',   4,   9,  48, "SC-8850 HIP HOP Set", "Jazz High Tom 2"},
    {'P',   4,   9,  49, "SC-8850 HIP HOP Set", "TR-909 Crash Cymbal"},
    {'P',   4,   9,  50, "SC-8850 HIP HOP Set", "Jazz High Tom 1"},
    {'P',   4,   9,  51, "SC-8850 HIP HOP Set", "Ride Cymbal1"},
    {'P',   4,   9,  52, "SC-8850 HIP HOP Set", "Reverse Cymbal"},
    {'P',   4,   9,  53, "SC-8850 HIP HOP Set", "Ride Bell"},
    {'P',   4,   9,  54, "SC-8850 HIP HOP Set", "Shake Tambourine"},
    {'P',   4,   9,  55, "SC-8850 HIP HOP Set", "Splash Cymbal"},
    {'P',   4,   9,  56, "SC-8850 HIP HOP Set", "TR-808 Cowbell"},
    {'P',   4,   9,  57, "SC-8850 HIP HOP Set", "Crash Cymbal 2"},
    {'P',   4,   9,  58, "SC-8850 HIP HOP Set", "Vibra-slap"},
    {'P',   4,   9,  59, "SC-8850 HIP HOP Set", "Ride Cymbal 2"},
    {'P',   4,   9,  60, "SC-8850 HIP HOP Set", "High Bongo"},
    {'P',   4,   9,  61, "SC-8850 HIP HOP Set", "Low Bongo"},
    {'P',   4,   9,  62, "SC-8850 HIP HOP Set", "Mute High Conga"},
    {'P',   4,   9,  63, "SC-8850 HIP HOP Set", "Open High Conga"},
    {'P',   4,   9,  64, "SC-8850 HIP HOP Set", "Low Conga"},
    {'P',   4,   9,  65, "SC-8850 HIP HOP Set", "High Timbale"},
    {'P',   4,   9,  66, "SC-8850 HIP HOP Set", "Low Timbale"},
    {'P',   4,   9,  67, "SC-8850 HIP HOP Set", "High Agogo"},
    {'P',   4,   9,  68, "SC-8850 HIP HOP Set", "Low Agogo"},
    {'P',   4,   9,  69, "SC-8850 HIP HOP Set", "Cabasa"},
    {'P',   4,   9,  70, "SC-8850 HIP HOP Set", "TR-808 Maracas"},
    {'P',   4,   9,  71, "SC-8850 HIP HOP Set", "Short High Whistle"},
    {'P',   4,   9,  72, "SC-8850 HIP HOP Set", "Long Low Whistle"},
    {'P',   4,   9,  73, "SC-8850 HIP HOP Set", "Short Guiro"},
    {'P',   4,   9,  74, "SC-8850 HIP HOP Set", "CR-78 Guiro"},
    {'P',   4,   9,  75, "SC-8850 HIP HOP Set", "TR-808 Claves"},
    {'P',   4,   9,  76, "SC-8850 HIP HOP Set", "High Wood Block"},
    {'P',   4,   9,  77, "SC-8850 HIP HOP Set", "Low Wood Block"},
    {'P',   4,   9,  78, "SC-8850 HIP HOP Set", "High Hoo"},
    {'P',   4,   9,  79, "SC-8850 HIP HOP Set", "Low Hoo"},
    {'P',   4,   9,  80, "SC-8850 HIP HOP Set", "Electric Mute Triangle"},
    {'P',   4,   9,  81, "SC-8850 HIP HOP Set", "Electric Open Triangle"},
    {'P',   4,   9,  82, "SC-8850 HIP HOP Set", "Shaker 2"},
    {'P',   4,   9,  83, "SC-8850 HIP HOP Set", "Jingle Bell"},
    {'P',   4,   9,  84, "SC-8850 HIP HOP Set", "Bell Tree"},
    {'P',   4,   9,  85, "SC-8850 HIP HOP Set", "Castanets"},
    {'P',   4,   9,  86, "SC-8850 HIP HOP Set", "Mute Surdo"},
    {'P',   4,   9,  87, "SC-8850 HIP HOP Set", "Open Surdo"},
    {'P',   4,   9,  88, "SC-8850 HIP HOP Set", "Small Club 1"},
    {'P',   4,   9,  89, "SC-8850 HIP HOP Set", "Hip-Hop Snare 2"},
    {'P',   4,   9,  90, "SC-8850 HIP HOP Set", "LoFi Snare Rim"},
    {'P',   4,   9,  91, "SC-8850 HIP HOP Set", "Hip-Hop Claps"},
    {'P',   4,   9,  92, "SC-8850 HIP HOP Set", "Stantard 1 Snare 1"},
    {'P',   4,   9,  93, "SC-8850 HIP HOP Set", "Standard 1 Snare 2"},
    {'P',   4,   9,  94, "SC-8850 HIP HOP Set", "Room Snare 1"},
    {'P',   4,   9,  95, "SC-8850 HIP HOP Set", "Room Snare 2"},
    {'P',   4,   9,  96, "SC-8850 HIP HOP Set", "Dance Snare"},
    {'P',   4,   9,  97, "SC-8850 HIP HOP Set", "[Pro] Techno Hit"},
    {'P',   4,   9,  98, "SC-8850 HIP HOP Set", "[Pro] Philly Hit"},
    {'P',   4,   9,  99, "SC-8850 HIP HOP Set", "[Pro] Impact Hit"},
    {'P',   4,   9, 100, "SC-8850 HIP HOP Set", "[Pro] Lo-Fi Rave"},
    {'P',   4,   9, 101, "SC-8850 HIP HOP Set", "[Pro] Bam Hit"},
    {'P',   4,   9, 102, "SC-8850 HIP HOP Set", "[Pro] Bim Hit"},
    {'P',   4,   9, 103, "SC-8850 HIP HOP Set", "[Pro] Tape Rewind"},
    {'P',   4,   9, 104, "SC-8850 HIP HOP Set", "[Pro] Phonograph Noise"},
    {'P',   4,   9, 105, "SC-8850 HIP HOP Set", "[88] Power Snare 1"},
    {'P',   4,   9, 106, "SC-8850 HIP HOP Set", "[88] Dance Snare 1"},
    {'P',   4,   9, 107, "SC-8850 HIP HOP Set", "[88] Dance Snare 2"},
    {'P',   4,   9, 108, "SC-8850 HIP HOP Set", "[88] Disco Snare"},
    {'P',   4,   9, 109, "SC-8850 HIP HOP Set", "[88] Electric Snare 2"},
    {'P',   4,   9, 110, "SC-8850 HIP HOP Set", "[55] Electric Snare"},
    {'P',   4,   9, 111, "SC-8850 HIP HOP Set", "[88] Electric Snare 3"},
    {'P',   4,   9, 112, "SC-8850 HIP HOP Set", "[Pro] TR-606 Snare 2"},
    {'P',   4,   9, 113, "SC-8850 HIP HOP Set", "[Pro] TR-707 Snare 1"},
    {'P',   4,   9, 114, "SC-8850 HIP HOP Set", "[88] TR-808 Snare 2"},
    {'P',   4,   9, 115, "SC-8850 HIP HOP Set", "[88] TR-808 Snare 1"},
    {'P',   4,   9, 116, "SC-8850 HIP HOP Set", "[Pro] TR-808 Snare 2"},
    {'P',   4,   9, 117, "SC-8850 HIP HOP Set", "[88] TR-909 Snare 1"},
    {'P',   4,   9, 118, "SC-8850 HIP HOP Set", "[88] TR-909 Snare 2"},
    {'P',   4,   9, 119, "SC-8850 HIP HOP Set", "[Pro] TR-909 Snare 1"},
    {'P',   4,   9, 120, "SC-8850 HIP HOP Set", "[Pro] TR-909 Snare 2"},
    {'P',   4,   9, 121, "SC-8850 HIP HOP Set", "[Pro] Rap Snare"},
    {'P',   4,   9, 122, "SC-8850 HIP HOP Set", "[Pro] Jungle Snare"},
    {'P',   4,   9, 123, "SC-8850 HIP HOP Set", "[Pro] House Snare 1"},
    {'P',   4,   9, 124, "SC-8850 HIP HOP Set", "[88] House Snare"},
    {'P',   4,   9, 125, "SC-8850 HIP HOP Set", "[Pro] House Snare 2"},
    {'P',   4,   9, 126, "SC-8850 HIP HOP Set", "[Pro] Voice Tah"},
    {'P',   4,   9, 127, "SC-8850 HIP HOP Set", "[88] Slappy"},
    {'P',   4,  10,  20, "SC-8850 JUNGLE Set", "HipHop Kick 2"},
    {'P',   4,  10,  21, "SC-8850 JUNGLE Set", "HipHop Kick 1"},
    {'P',   4,  10,  26, "SC-8850 JUNGLE Set", "Finger Snap 2"},
    {'P',   4,  10,  29, "SC-8850 JUNGLE Set", "Scratch Push 2"},
    {'P',   4,  10,  30, "SC-8850 JUNGLE Set", "Scratch Pull 2"},
    {'P',   4,  10,  35, "SC-8850 JUNGLE Set", "Jungle Kick 2"},
    {'P',   4,  10,  36, "SC-8850 JUNGLE Set", "Jungle Kick 1"},
    {'P',   4,  10,  37, "SC-8850 JUNGLE Set", "Jungle Snare Rim"},
    {'P',   4,  10,  38, "SC-8850 JUNGLE Set", "HipHop Snare 1"},
    {'P',   4,  10,  39, "SC-8850 JUNGLE Set", "R&B Claps"},
    {'P',   4,  10,  40, "SC-8850 JUNGLE Set", "Jungle Snare"},
    {'P',   4,  10,  41, "SC-8850 JUNGLE Set", "TR-909 Low Tom 2"},
    {'P',   4,  10,  42, "SC-8850 JUNGLE Set", "TR-606 Closed Hi-Hat"},
    {'P',   4,  10,  43, "SC-8850 JUNGLE Set", "TR-909 Low Tom 1"},
    {'P',   4,  10,  44, "SC-8850 JUNGLE Set", "Jungle Hi-Hat"},
    {'P',   4,  10,  45, "SC-8850 JUNGLE Set", "TR-909 Mid Tom 2"},
    {'P',   4,  10,  46, "SC-8850 JUNGLE Set", "TR-606 Open Hi-Hat"},
    {'P',   4,  10,  47, "SC-8850 JUNGLE Set", "TR-909 Mid Tom 1"},
    {'P',   4,  10,  48, "SC-8850 JUNGLE Set", "TR-909 High Tom 2"},
    {'P',   4,  10,  49, "SC-8850 JUNGLE Set", "Jungle Crash"},
    {'P',   4,  10,  50, "SC-8850 JUNGLE Set", "TR-909 High Tom 1"},
    {'P',   4,  10,  51, "SC-8850 JUNGLE Set", "Ride Cymbal 1"},
    {'P',   4,  10,  52, "SC-8850 JUNGLE Set", "Reverse Cymabal"},
    {'P',   4,  10,  53, "SC-8850 JUNGLE Set", "Ride Bell"},
    {'P',   4,  10,  54, "SC-8850 JUNGLE Set", "Shake Tambourine"},
    {'P',   4,  10,  55, "SC-8850 JUNGLE Set", "Splash Cymbal"},
    {'P',   4,  10,  56, "SC-8850 JUNGLE Set", "TR808 Cowbell"},
    {'P',   4,  10,  74, "SC-8850 JUNGLE Set", "CR-78 Guiro"},
    {'P',   4,  10,  75, "SC-8850 JUNGLE Set", "TR-808 Claves"},
    {'P',   4,  10,  78, "SC-8850 JUNGLE Set", "High Hoo"},
    {'P',   4,  10,  79, "SC-8850 JUNGLE Set", "Low Hoo"},
    {'P',   4,  10,  80, "SC-8850 JUNGLE Set", "Electric Mute Triangle"},
    {'P',   4,  10,  81, "SC-8850 JUNGLE Set", "Electric Open Triangle"},
    {'P',   4,  10,  82, "SC-8850 JUNGLE Set", "Jungle Shaker"},
    {'P',   4,  10,  88, "SC-8850 JUNGLE Set", "Small Club 1"},
    {'P',   4,  10,  89, "SC-8850 JUNGLE Set", "Jungle Kick Roll"},
    {'P',   4,  10,  90, "SC-8850 JUNGLE Set", "Jungle Snare Roll"},
    {'P',   4,  10,  91, "SC-8850 JUNGLE Set", "TR-606 Snare 2"},
    {'P',   4,  10,  92, "SC-8850 JUNGLE Set", "Dance Snare"},
    {'P',   4,  10,  93, "SC-8850 JUNGLE Set", "Techno Snare"},
    {'P',   4,  10,  94, "SC-8850 JUNGLE Set", "House Snare"},
    {'P',   4,  10,  95, "SC-8850 JUNGLE Set", "Rock Snare Dry"},
    {'P',   4,  10,  96, "SC-8850 JUNGLE Set", "LoFi Snare 1"},
    {'P',   4,  11,  20, "SC-8850 TECHNO Set", "TR-909 Kick 2"},
    {'P',   4,  11,  21, "SC-8850 TECHNO Set", "Fat Kick"},
    {'P',   4,  11,  26, "SC-8850 TECHNO Set", "Finger Snap 2"},
    {'P',   4,  11,  29, "SC-8850 TECHNO Set", "Scratch Push 2"},
    {'P',   4,  11,  30, "SC-8850 TECHNO Set", "Scratch Pull 2"},
    {'P',   4,  11,  35, "SC-8850 TECHNO Set", "TR-808 Kick"},
    {'P',   4,  11,  36, "SC-8850 TECHNO Set", "TR-909 Kick 1"},
    {'P',   4,  11,  37, "SC-8850 TECHNO Set", "TR-909 Snare Rim"},
    {'P',   4,  11,  38, "SC-8850 TECHNO Set", "TR-606 Snare 2"},
    {'P',   4,  11,  39, "SC-8850 TECHNO Set", "TR-909 Claps"},
    {'P',   4,  11,  40, "SC-8850 TECHNO Set", "Techno Snare"},
    {'P',   4,  11,  41, "SC-8850 TECHNO Set", "TR-606 Low Tom 2"},
    {'P',   4,  11,  42, "SC-8850 TECHNO Set", "TR-707 Closed Hi-Hat"},
    {'P',   4,  11,  43, "SC-8850 TECHNO Set", "TR-606 Low Tom 1"},
    {'P',   4,  11,  44, "SC-8850 TECHNO Set", "CR-78 Closed Hi-Hat"},
    {'P',   4,  11,  45, "SC-8850 TECHNO Set", "TR-606 Mid Tom 2"},
    {'P',   4,  11,  46, "SC-8850 TECHNO Set", "TR-909 Open Hi-Hat"},
    {'P',   4,  11,  47, "SC-8850 TECHNO Set", "TR-606 Mid Tom 1"},
    {'P',   4,  11,  48, "SC-8850 TECHNO Set", "TR-606 High Tom 2"},
    {'P',   4,  11,  49, "SC-8850 TECHNO Set", "TR-909 Crash Cymbal"},
    {'P',   4,  11,  50, "SC-8850 TECHNO Set", "TR-606 High Tom 1"},
    {'P',   4,  11,  51, "SC-8850 TECHNO Set", "Ride Cymbal 1"},
    {'P',   4,  11,  52, "SC-8850 TECHNO Set", "Reverse Cymbal"},
    {'P',   4,  11,  53, "SC-8850 TECHNO Set", "Ride Bell"},
    {'P',   4,  11,  54, "SC-8850 TECHNO Set", "Shake Tambourine"},
    {'P',   4,  11,  55, "SC-8850 TECHNO Set", "Splash Cymbal"},
    {'P',   4,  11,  56, "SC-8850 TECHNO Set", "TR-808Cowbell"},
    {'P',   4,  11,  57, "SC-8850 TECHNO Set", "TR-909 Crash Cymbal"},
    {'P',   4,  11,  60, "SC-8850 TECHNO Set", "CR-78 High Bongo"},
    {'P',   4,  11,  61, "SC-8850 TECHNO Set", "CR-78 Low Bongo"},
    {'P',   4,  11,  62, "SC-8850 TECHNO Set", "TR-808 High Conga"},
    {'P',   4,  11,  63, "SC-8850 TECHNO Set", "TR-808 Mute Conga"},
    {'P',   4,  11,  64, "SC-8850 TECHNO Set", "TR-808 Low Conga"},
    {'P',   4,  11,  70, "SC-8850 TECHNO Set", "TR-808 Maracas"},
    {'P',   4,  11,  74, "SC-8850 TECHNO Set", "CR-78 Guiro"},
    {'P',   4,  11,  75, "SC-8850 TECHNO Set", "TR-808 Claves"},
    {'P',   4,  11,  78, "SC-8850 TECHNO Set", "High Hoo"},
    {'P',   4,  11,  79, "SC-8850 TECHNO Set", "Low Hoo"},
    {'P',   4,  11,  80, "SC-8850 TECHNO Set", "Electric Mute Triangle"},
    {'P',   4,  11,  81, "SC-8850 TECHNO Set", "Electric Open Triangle"},
    {'P',   4,  11,  82, "SC-8850 TECHNO Set", "TR-626 Shaker"},
    {'P',   4,  11,  88, "SC-8850 TECHNO Set", "Applause 2"},
    {'P',   4,  11,  89, "SC-8850 TECHNO Set", "Dance Snare"},
    {'P',   4,  11,  90, "SC-8850 TECHNO Set", "House Snare"},
    {'P',   4,  11,  91, "SC-8850 TECHNO Set", "Rock Snare Dry"},
    {'P',   4,  11,  92, "SC-8850 TECHNO Set", "Jungle Snare"},
    {'P',   4,  11,  93, "SC-8850 TECHNO Set", "LoFi Snare 1"},
    {'P',   4,  11,  94, "SC-8850 TECHNO Set", "LoFi Snare 2"},
    {'P',   4,  11,  95, "SC-8850 TECHNO Set", "HipHop Snare 1"},
    {'P',   4,  11,  96, "SC-8850 TECHNO Set", "HipHop Snare 2"},
    {'P',   4,  12,  26, "SC-8850 ROOM L/R Set", "Finger Snap"},
    {'P',   4,  12,  29, "SC-8850 ROOM L/R Set", "Scratch Push"},
    {'P',   4,  12,  30, "SC-8850 ROOM L/R Set", "Scratch Pull"},
    {'P',   4,  12,  35, "SC-8850 ROOM L/R Set", "[RND] Room Kick 2"},
    {'P',   4,  12,  36, "SC-8850 ROOM L/R Set", "[RND] Room Kick 1"},
    {'P',   4,  12,  37, "SC-8850 ROOM L/R Set", "Side Stick"},
    {'P',   4,  12,  38, "SC-8850 ROOM L/R Set", "[RND] Room Snare 1"},
    {'P',   4,  12,  39, "SC-8850 ROOM L/R Set", "Hand Clap"},
    {'P',   4,  12,  40, "SC-8850 ROOM L/R Set", "[RND] Room Snare 2"},
    {'P',   4,  12,  41, "SC-8850 ROOM L/R Set", "Room Low Tom 2"},
    {'P',   4,  12,  42, "SC-8850 ROOM L/R Set", "[RND] Room Closed Hi-Hat"},
    {'P',   4,  12,  43, "SC-8850 ROOM L/R Set", "Room Low Tom 1"},
    {'P',   4,  12,  44, "SC-8850 ROOM L/R Set", "Pedal Hi-Hat"},
    {'P',   4,  12,  45, "SC-8850 ROOM L/R Set", "Room Mid Tom 2"},
    {'P',   4,  12,  46, "SC-8850 ROOM L/R Set", "[RND] Room Open Hi-Hat"},
    {'P',   4,  12,  47, "SC-8850 ROOM L/R Set", "Room Mid Tom 1"},
    {'P',   4,  12,  48, "SC-8850 ROOM L/R Set", "Room High Tom 2"},
    {'P',   4,  12,  49, "SC-8850 ROOM L/R Set", "[RND] Room Crash Cymbal"},
    {'P',   4,  12,  50, "SC-8850 ROOM L/R Set", "Room High Tom 1"},
    {'P',   4,  12,  51, "SC-8850 ROOM L/R Set", "[RND] Room Ride Cymbal"},
    {'P',   4,  12,  52, "SC-8850 ROOM L/R Set", "Chinese Cymbal"},
    {'P',   4,  12,  53, "SC-8850 ROOM L/R Set", "[RND] Room Ride Bell"},
    {'P',   4,  12,  54, "SC-8850 ROOM L/R Set", "Tambourine"},
    {'P',   4,  12,  55, "SC-8850 ROOM L/R Set", "Splash Cymbal"},
    {'P',   4,  12,  56, "SC-8850 ROOM L/R Set", "Cowbell"},
    {'P',   4,  12,  88, "SC-8850 ROOM L/R Set", "Applause 2"},
    {'P',   4,  12,  95, "SC-8850 ROOM L/R Set", "[L] Room Kick 2"},
    {'P',   4,  12,  96, "SC-8850 ROOM L/R Set", "[L] Room Kick 1"},
    {'P',   4,  12,  97, "SC-8850 ROOM L/R Set", "[L] Room Crash Cymbal"},
    {'P',   4,  12,  98, "SC-8850 ROOM L/R Set", "[L] Room Snare 1"},
    {'P',   4,  12,  99, "SC-8850 ROOM L/R Set", "[L] Room Ride Cymbal"},
    {'P',   4,  12, 100, "SC-8850 ROOM L/R Set", "[L] Room Snare 2"},
    {'P',   4,  12, 101, "SC-8850 ROOM L/R Set", "[L] Room Low Tom"},
    {'P',   4,  12, 102, "SC-8850 ROOM L/R Set", "[L] Room Closed Hi-Hat"},
    {'P',   4,  12, 103, "SC-8850 ROOM L/R Set", "[L] Room Mid Tom"},
    {'P',   4,  12, 104, "SC-8850 ROOM L/R Set", "[L] Room Ride Bell"},
    {'P',   4,  12, 105, "SC-8850 ROOM L/R Set", "[L] Room High Tom"},
    {'P',   4,  12, 106, "SC-8850 ROOM L/R Set", "[L] Room Open Hi-Hat"},
    {'P',   4,  12, 107, "SC-8850 ROOM L/R Set", "[R] Room Kick 2"},
    {'P',   4,  12, 108, "SC-8850 ROOM L/R Set", "[R] Room Kick 1"},
    {'P',   4,  12, 109, "SC-8850 ROOM L/R Set", "[R] Room Crash Cymbal"},
    {'P',   4,  12, 110, "SC-8850 ROOM L/R Set", "[R] Room Snare 1"},
    {'P',   4,  12, 111, "SC-8850 ROOM L/R Set", "[R] Room Ride Cymbal"},
    {'P',   4,  12, 112, "SC-8850 ROOM L/R Set", "[R] Room Snare 2"},
    {'P',   4,  12, 113, "SC-8850 ROOM L/R Set", "[R] Room Low Tom"},
    {'P',   4,  12, 114, "SC-8850 ROOM L/R Set", "[R] Room Closed Hi-Hat"},
    {'P',   4,  12, 115, "SC-8850 ROOM L/R Set", "[R] Room Mid Tom"},
    {'P',   4,  12, 116, "SC-8850 ROOM L/R Set", "[R] Room Ride Bell"},
    {'P',   4,  12, 117, "SC-8850 ROOM L/R Set", "[R] Room High Tom"},
    {'P',   4,  12, 118, "SC-8850 ROOM L/R Set", "[R] Room Open Hi-Hat"},
    {'P',   4,  13,  20, "SC-8850 HOUSE Set", "Fat Kick"},
    {'P',   4,  13,  21, "SC-8850 HOUSE Set", "Dance Kick"},
    {'P',   4,  13,  26, "SC-8850 HOUSE Set", "Finger Snap 2"},
    {'P',   4,  13,  29, "SC-8850 HOUSE Set", "Scratch Push 2"},
    {'P',   4,  13,  30, "SC-8850 HOUSE Set", "Scratch Pull 2"},
    {'P',   4,  13,  35, "SC-8850 HOUSE Set", "TR-909 Kick 2"},
    {'P',   4,  13,  36, "SC-8850 HOUSE Set", "TR-909 Kick 1"},
    {'P',   4,  13,  37, "SC-8850 HOUSE Set", "House Snare Rim"},
    {'P',   4,  13,  38, "SC-8850 HOUSE Set", "House Snare 1"},
    {'P',   4,  13,  39, "SC-8850 HOUSE Set", "TR-909 Claps"},
    {'P',   4,  13,  40, "SC-8850 HOUSE Set", "House Snare 2"},
    {'P',   4,  13,  41, "SC-8850 HOUSE Set", "TR-909 Low Tom 2"},
    {'P',   4,  13,  42, "SC-8850 HOUSE Set", "Room Closed Hi-Hat"},
    {'P',   4,  13,  43, "SC-8850 HOUSE Set", "TR-909 Low Tom 1"},
    {'P',   4,  13,  44, "SC-8850 HOUSE Set", "Pedal Hi-Hat"},
    {'P',   4,  13,  45, "SC-8850 HOUSE Set", "TR-909 Mid Tom 2"},
    {'P',   4,  13,  46, "SC-8850 HOUSE Set", "Room Open Hi-Hat"},
    {'P',   4,  13,  47, "SC-8850 HOUSE Set", "TR-909 Mid Tom 1"},
    {'P',   4,  13,  48, "SC-8850 HOUSE Set", "TR-909 High Tom 2"},
    {'P',   4,  13,  49, "SC-8850 HOUSE Set", "TR-909 Crash Cymbal"},
    {'P',   4,  13,  50, "SC-8850 HOUSE Set", "TR-909 High Tom 1"},
    {'P',   4,  13,  51, "SC-8850 HOUSE Set", "TR-909 Ride Cymbal"},
    {'P',   4,  13,  52, "SC-8850 HOUSE Set", "Reverse Cymbal"},
    {'P',   4,  13,  53, "SC-8850 HOUSE Set", "Ride Bell"},
    {'P',   4,  13,  54, "SC-8850 HOUSE Set", "Shake Tambourine"},
    {'P',   4,  13,  55, "SC-8850 HOUSE Set", "Splash Cymbal"},
    {'P',   4,  13,  56, "SC-8850 HOUSE Set", "TR-808 Cowbell"},
    {'P',   4,  13,  57, "SC-8850 HOUSE Set", "TR-909 Crash Cymbal"},
    {'P',   4,  13,  60, "SC-8850 HOUSE Set", "CR-78 High Bongo"},
    {'P',   4,  13,  61, "SC-8850 HOUSE Set", "CR-78 Low Bongo"},
    {'P',   4,  13,  62, "SC-8850 HOUSE Set", "TR-808 High Conga"},
    {'P',   4,  13,  63, "SC-8850 HOUSE Set", "TR-808 Mute Conga"},
    {'P',   4,  13,  64, "SC-8850 HOUSE Set", "TR-808 Low Conga"},
    {'P',   4,  13,  70, "SC-8850 HOUSE Set", "TR-808 Maracas"},
    {'P',   4,  13,  74, "SC-8850 HOUSE Set", "CR-78 Guiro"},
    {'P',   4,  13,  75, "SC-8850 HOUSE Set", "TR-808 Claves"},
    {'P',   4,  13,  78, "SC-8850 HOUSE Set", "High Hoo"},
    {'P',   4,  13,  79, "SC-8850 HOUSE Set", "Low Hoo"},
    {'P',   4,  13,  80, "SC-8850 HOUSE Set", "Electric Mute Triangle"},
    {'P',   4,  13,  81, "SC-8850 HOUSE Set", "Electric Open Triangle"},
    {'P',   4,  13,  82, "SC-8850 HOUSE Set", "TR-626 Shaker"},
    {'P',   4,  13,  88, "SC-8850 HOUSE Set", "Small Club 1"},
    {'P',   4,  13,  89, "SC-8850 HOUSE Set", "TR-606 Snare 2"},
    {'P',   4,  13,  90, "SC-8850 HOUSE Set", "Dance Snare"},
    {'P',   4,  13,  91, "SC-8850 HOUSE Set", "Techno Snare"},
    {'P',   4,  13,  92, "SC-8850 HOUSE Set", "Rock Snare Dry"},
    {'P',   4,  13,  93, "SC-8850 HOUSE Set", "Hip-Hop Snare 1"},
    {'P',   4,  13,  94, "SC-8850 HOUSE Set", "LoFi Snare 1"},
    {'P',   4,  13,  95, "SC-8850 HOUSE Set", "LoFi Snare 2"},
    {'P',   4,  13,  96, "SC-8850 HOUSE Set", "Jungle Snare"},
    {'P',   4,  16,  26, "SC-8850 POWER Set", "Finger Snap 2"},
    {'P',   4,  16,  29, "SC-8850 POWER Set", "Scratch Push"},
    {'P',   4,  16,  30, "SC-8850 POWER Set", "Scratch Pull"},
    {'P',   4,  16,  35, "SC-8850 POWER Set", "Power Kick 2"},
    {'P',   4,  16,  36, "SC-8850 POWER Set", "Power Kick 1"},
    {'P',   4,  16,  37, "SC-8850 POWER Set", "Side Stick"},
    {'P',   4,  16,  38, "SC-8850 POWER Set", "Power Snare 1"},
    {'P',   4,  16,  39, "SC-8850 POWER Set", "Hand Clap"},
    {'P',   4,  16,  40, "SC-8850 POWER Set", "Power Snare 2"},
    {'P',   4,  16,  41, "SC-8850 POWER Set", "Power Low Tom 2"},
    {'P',   4,  16,  42, "SC-8850 POWER Set", "Closed Hi-Hat 1"},
    {'P',   4,  16,  43, "SC-8850 POWER Set", "Power Low Tom 1"},
    {'P',   4,  16,  44, "SC-8850 POWER Set", "Pedal Hi-Hat"},
    {'P',   4,  16,  45, "SC-8850 POWER Set", "Power Mid Tom 2"},
    {'P',   4,  16,  46, "SC-8850 POWER Set", "Open Hi-Hat 1"},
    {'P',   4,  16,  47, "SC-8850 POWER Set", "Power Mid Tom 1"},
    {'P',   4,  16,  48, "SC-8850 POWER Set", "Power High Tom 2"},
    {'P',   4,  16,  49, "SC-8850 POWER Set", "Crash Cymbal 1"},
    {'P',   4,  16,  50, "SC-8850 POWER Set", "Power High Tom 1"},
    {'P',   4,  16,  51, "SC-8850 POWER Set", "Ride Cymbal 1"},
    {'P',   4,  16,  52, "SC-8850 POWER Set", "Chinese Cymbal"},
    {'P',   4,  16,  53, "SC-8850 POWER Set", "Ride Bell"},
    {'P',   4,  16,  54, "SC-8850 POWER Set", "Tambourine"},
    {'P',   4,  16,  55, "SC-8850 POWER Set", "Splash Cymbal"},
    {'P',   4,  16,  56, "SC-8850 POWER Set", "Cowbell"},
    {'P',   4,  16,  84, "SC-8850 POWER Set", "Bell Tree"},
    {'P',   4,  16,  88, "SC-8850 POWER Set", "Applause2"},
    {'P',   4,  24,   0, "SC-8850 ELECTRONIC Set", "[88] Electric Kick 2"},
    {'P',   4,  24,   1, "SC-8850 ELECTRONIC Set", "[88] Electric Kick 1"},
    {'P',   4,  24,   2, "SC-8850 ELECTRONIC Set", "CR-78 Kick 1"},
    {'P',   4,  24,   3, "SC-8850 ELECTRONIC Set", "CR-78 Kick 2"},
    {'P',   4,  24,   4, "SC-8850 ELECTRONIC Set", "TR-606 Kick1"},
    {'P',   4,  24,   5, "SC-8850 ELECTRONIC Set", "TR-707 Kick 1"},
    {'P',   4,  24,   6, "SC-8850 ELECTRONIC Set", "[55] TR-808 Kick"},
    {'P',   4,  24,   7, "SC-8850 ELECTRONIC Set", "[88] TR-808 Kick"},
    {'P',   4,  24,   8, "SC-8850 ELECTRONIC Set", "TR-808 Kick 2"},
    {'P',   4,  24,   9, "SC-8850 ELECTRONIC Set", "[88] TR-909 Kick"},
    {'P',   4,  24,  10, "SC-8850 ELECTRONIC Set", "[88] Dance Kick"},
    {'P',   4,  24,  11, "SC-8850 ELECTRONIC Set", "Hip-Hop Kick 2"},
    {'P',   4,  24,  12, "SC-8850 ELECTRONIC Set", "TR-909 Kick 1"},
    {'P',   4,  24,  13, "SC-8850 ELECTRONIC Set", "Hip-Hop Kick 3"},
    {'P',   4,  24,  14, "SC-8850 ELECTRONIC Set", "Jungle Kick 1"},
    {'P',   4,  24,  15, "SC-8850 ELECTRONIC Set", "Techno Kick 1"},
    {'P',   4,  24,  16, "SC-8850 ELECTRONIC Set", "Bounce Kick"},
    {'P',   4,  24,  26, "SC-8850 ELECTRONIC Set", "Finger Snap 2"},
    {'P',   4,  24,  29, "SC-8850 ELECTRONIC Set", "Scratch Push 2"},
    {'P',   4,  24,  30, "SC-8850 ELECTRONIC Set", "Scratch Pull 2"},
    {'P',   4,  24,  35, "SC-8850 ELECTRONIC Set", "Electric Kick 2"},
    {'P',   4,  24,  36, "SC-8850 ELECTRONIC Set", "Electric Kick 1"},
    {'P',   4,  24,  37, "SC-8850 ELECTRONIC Set", "Side Stick"},
    {'P',   4,  24,  38, "SC-8850 ELECTRONIC Set", "Electric Snare 1"},
    {'P',   4,  24,  39, "SC-8850 ELECTRONIC Set", "Hand Clap"},
    {'P',   4,  24,  40, "SC-8850 ELECTRONIC Set", "Electric Snare 2"},
    {'P',   4,  24,  41, "SC-8850 ELECTRONIC Set", "Electric Low Tom 2"},
    {'P',   4,  24,  42, "SC-8850 ELECTRONIC Set", "Closed Hi-Hat 2"},
    {'P',   4,  24,  43, "SC-8850 ELECTRONIC Set", "Electric Low Tom 1"},
    {'P',   4,  24,  44, "SC-8850 ELECTRONIC Set", "Pedal Hi-Hat"},
    {'P',   4,  24,  45, "SC-8850 ELECTRONIC Set", "Electric Mid Tom 2"},
    {'P',   4,  24,  46, "SC-8850 ELECTRONIC Set", "Open Hi-Hat 2"},
    {'P',   4,  24,  47, "SC-8850 ELECTRONIC Set", "Electric Mid Tom 1"},
    {'P',   4,  24,  48, "SC-8850 ELECTRONIC Set", "Electric High Tom 2"},
    {'P',   4,  24,  49, "SC-8850 ELECTRONIC Set", "Crash Cymbal 1"},
    {'P',   4,  24,  50, "SC-8850 ELECTRONIC Set", "Electric High Tom 1"},
    {'P',   4,  24,  51, "SC-8850 ELECTRONIC Set", "Ride Cymbal 1"},
    {'P',   4,  24,  52, "SC-8850 ELECTRONIC Set", "Reverse Cymbal"},
    {'P',   4,  24,  88, "SC-8850 ELECTRONIC Set", "Small Club 1"},
    {'P',   4,  24,  97, "SC-8850 ELECTRONIC Set", "Techno Hit"},
    {'P',   4,  24,  98, "SC-8850 ELECTRONIC Set", "Philly Hit"},
    {'P',   4,  24,  99, "SC-8850 ELECTRONIC Set", "Impact Hit"},
    {'P',   4,  24, 100, "SC-8850 ELECTRONIC Set", "Lo-Fi Rave"},
    {'P',   4,  24, 101, "SC-8850 ELECTRONIC Set", "Bam Hit"},
    {'P',   4,  24, 102, "SC-8850 ELECTRONIC Set", "Bim Hit"},
    {'P',   4,  24, 103, "SC-8850 ELECTRONIC Set", "Tape Rewind"},
    {'P',   4,  24, 104, "SC-8850 ELECTRONIC Set", "Phonograph Noise"},
    {'P',   4,  24, 105, "SC-8850 ELECTRONIC Set", "[88] Power Snare 1"},
    {'P',   4,  24, 106, "SC-8850 ELECTRONIC Set", "[88] Dance Snare 1"},
    {'P',   4,  24, 107, "SC-8850 ELECTRONIC Set", "[88] Dance Snare 2"},
    {'P',   4,  24, 108, "SC-8850 ELECTRONIC Set", "[88] Disco Snare"},
    {'P',   4,  24, 109, "SC-8850 ELECTRONIC Set", "[88] Electric Snare 2"},
    {'P',   4,  24, 110, "SC-8850 ELECTRONIC Set", "[55] Electric Snare"},
    {'P',   4,  24, 111, "SC-8850 ELECTRONIC Set", "[88] Electric Snare 3"},
    {'P',   4,  24, 112, "SC-8850 ELECTRONIC Set", "TR-606 Snare 2"},
    {'P',   4,  24, 113, "SC-8850 ELECTRONIC Set", "TR-707 Snare 1"},
    {'P',   4,  24, 114, "SC-8850 ELECTRONIC Set", "[88] TR-808 Snare 2"},
    {'P',   4,  24, 115, "SC-8850 ELECTRONIC Set", "[88] TR-808 Snare 1"},
    {'P',   4,  24, 116, "SC-8850 ELECTRONIC Set", "TR-808 Snare 2"},
    {'P',   4,  24, 117, "SC-8850 ELECTRONIC Set", "[88] TR-909 Snare 1"},
    {'P',   4,  24, 118, "SC-8850 ELECTRONIC Set", "[88] TR-909 Snare 2"},
    {'P',   4,  24, 119, "SC-8850 ELECTRONIC Set", "TR-909 Snare 1"},
    {'P',   4,  24, 120, "SC-8850 ELECTRONIC Set", "TR-909 Snare 2"},
    {'P',   4,  24, 121, "SC-8850 ELECTRONIC Set", "Rap Snare"},
    {'P',   4,  24, 122, "SC-8850 ELECTRONIC Set", "Jungle Snare"},
    {'P',   4,  24, 123, "SC-8850 ELECTRONIC Set", "House Snare 1"},
    {'P',   4,  24, 124, "SC-8850 ELECTRONIC Set", "[88] House Snare"},
    {'P',   4,  24, 125, "SC-8850 ELECTRONIC Set", "House Snare 2"},
    {'P',   4,  24, 126, "SC-8850 ELECTRONIC Set", "Voice Tah"},
    {'P',   4,  24, 127, "SC-8850 ELECTRONIC Set", "[88] Slappy"},
    {'P',   4,  25,  26, "SC-8850 TR-808 Set", "Finger Snap 2"},
    {'P',   4,  25,  29, "SC-8850 TR-808 Set", "Scratch Push 2"},
    {'P',   4,  25,  30, "SC-8850 TR-808 Set", "Scratch Pull 2"},
    {'P',   4,  25,  35, "SC-8850 TR-808 Set", "TR-808 Kick 2"},
    {'P',   4,  25,  36, "SC-8850 TR-808 Set", "TR-808 Kick 1"},
    {'P',   4,  25,  37, "SC-8850 TR-808 Set", "TR-808 Rim Shot"},
    {'P',   4,  25,  38, "SC-8850 TR-808 Set", "TR-808 Snare 1"},
    {'P',   4,  25,  39, "SC-8850 TR-808 Set", "Hand Clap"},
    {'P',   4,  25,  40, "SC-8850 TR-808 Set", "TR-808 Snare 2"},
    {'P',   4,  25,  41, "SC-8850 TR-808 Set", "TR-808 Low Tom 2"},
    {'P',   4,  25,  42, "SC-8850 TR-808 Set", "TR-808 Closed Hi-Hat 2"},
    {'P',   4,  25,  43, "SC-8850 TR-808 Set", "TR-808 Low Tom 1"},
    {'P',   4,  25,  44, "SC-8850 TR-808 Set", "TR-808 Closed Hi-Hat"},
    {'P',   4,  25,  45, "SC-8850 TR-808 Set", "TR-808 Mid Tom 2"},
    {'P',   4,  25,  46, "SC-8850 TR-808 Set", "TR-808 Open Hi-Hat"},
    {'P',   4,  25,  47, "SC-8850 TR-808 Set", "TR-808 Mid Tom 1"},
    {'P',   4,  25,  48, "SC-8850 TR-808 Set", "TR-808 High Tom 2"},
    {'P',   4,  25,  49, "SC-8850 TR-808 Set", "TR-808 Crash Cymbal"},
    {'P',   4,  25,  50, "SC-8850 TR-808 Set", "TR-808 High Tom 1"},
    {'P',   4,  25,  51, "SC-8850 TR-808 Set", "TR-606 Ride Cymbal"},
    {'P',   4,  25,  52, "SC-8850 TR-808 Set", "Chinese Cymbal"},
    {'P',   4,  25,  53, "SC-8850 TR-808 Set", "Ride Bell"},
    {'P',   4,  25,  54, "SC-8850 TR-808 Set", "CR-78 Tambourine"},
    {'P',   4,  25,  55, "SC-8850 TR-808 Set", "Splash Cymbal"},
    {'P',   4,  25,  56, "SC-8850 TR-808 Set", "TR-808 Cowbell"},
    {'P',   4,  25,  57, "SC-8850 TR-808 Set", "TR-909 Crash Cymbal"},
    {'P',   4,  25,  59, "SC-8850 TR-808 Set", "Ride Cymbal 2"},
    {'P',   4,  25,  60, "SC-8850 TR-808 Set", "CR-78 High Bongo"},
    {'P',   4,  25,  61, "SC-8850 TR-808 Set", "CR-78 Low Bongo"},
    {'P',   4,  25,  62, "SC-8850 TR-808 Set", "TR-808 High Conga"},
    {'P',   4,  25,  63, "SC-8850 TR-808 Set", "TR-808 Mute Conga"},
    {'P',   4,  25,  64, "SC-8850 TR-808 Set", "TR-808 Low Conga"},
    {'P',   4,  25,  70, "SC-8850 TR-808 Set", "TR-808 Maracas"},
    {'P',   4,  25,  74, "SC-8850 TR-808 Set", "CR-78 Guiro"},
    {'P',   4,  25,  75, "SC-8850 TR-808 Set", "TR-808 Claves"},
    {'P',   4,  25,  78, "SC-8850 TR-808 Set", "High Hoo"},
    {'P',   4,  25,  79, "SC-8850 TR-808 Set", "Low Hoo"},
    {'P',   4,  25,  80, "SC-8850 TR-808 Set", "Electric Mute Triangle"},
    {'P',   4,  25,  81, "SC-8850 TR-808 Set", "Electric Open Triangle"},
    {'P',   4,  25,  82, "SC-8850 TR-808 Set", "TR-626 Shaker"},
    {'P',   4,  25,  88, "SC-8850 TR-808 Set", "Small Club 1"},
    {'P',   4,  26,  20, "SC-8850 DANCE Set", "TR-909 Kick 2"},
    {'P',   4,  26,  21, "SC-8850 DANCE Set", "TR-909 Kick 1"},
    {'P',   4,  26,  26, "SC-8850 DANCE Set", "Finger Snap 2"},
    {'P',   4,  26,  29, "SC-8850 DANCE Set", "Scratch Push 2"},
    {'P',   4,  26,  30, "SC-8850 DANCE Set", "Scratch Pull 2"},
    {'P',   4,  26,  35, "SC-8850 DANCE Set", "Fat Kick"},
    {'P',   4,  26,  36, "SC-8850 DANCE Set", "Dance Kick"},
    {'P',   4,  26,  37, "SC-8850 DANCE Set", "Dance Rim Shot"},
    {'P',   4,  26,  38, "SC-8850 DANCE Set", "Dance Snare"},
    {'P',   4,  26,  39, "SC-8850 DANCE Set", "Comp Claps 2"},
    {'P',   4,  26,  40, "SC-8850 DANCE Set", "Rock SD Dry"},
    {'P',   4,  26,  41, "SC-8850 DANCE Set", "Electric Low Tom 2"},
    {'P',   4,  26,  42, "SC-8850 DANCE Set", "CR-78 Closed Hi-Hat"},
    {'P',   4,  26,  43, "SC-8850 DANCE Set", "Electric Low Tom 1"},
    {'P',   4,  26,  44, "SC-8850 DANCE Set", "TR-808 Closed Hi-Hat 2"},
    {'P',   4,  26,  45, "SC-8850 DANCE Set", "Electric Mid Tom 2"},
    {'P',   4,  26,  46, "SC-8850 DANCE Set", "CR-78 Open Hi-Hat"},
    {'P',   4,  26,  47, "SC-8850 DANCE Set", "Electric Mid Tom 1"},
    {'P',   4,  26,  48, "SC-8850 DANCE Set", "Electric High Tom 2"},
    {'P',   4,  26,  49, "SC-8850 DANCE Set", "TR-808 Crash Cymbal"},
    {'P',   4,  26,  50, "SC-8850 DANCE Set", "Electric High Tom 1"},
    {'P',   4,  26,  51, "SC-8850 DANCE Set", "TR-606 Ride Cymbal"},
    {'P',   4,  26,  52, "SC-8850 DANCE Set", "Reverse Cymbal"},
    {'P',   4,  26,  53, "SC-8850 DANCE Set", "Ride Bell"},
    {'P',   4,  26,  54, "SC-8850 DANCE Set", "Shake Tambourine"},
    {'P',   4,  26,  55, "SC-8850 DANCE Set", "Splash Cymbal"},
    {'P',   4,  26,  56, "SC-8850 DANCE Set", "TR-808 Cowbell"},
    {'P',   4,  26,  70, "SC-8850 DANCE Set", "Maracas"},
    {'P',   4,  26,  74, "SC-8850 DANCE Set", "Long Guiro"},
    {'P',   4,  26,  75, "SC-8850 DANCE Set", "Claves"},
    {'P',   4,  26,  78, "SC-8850 DANCE Set", "High Hoo"},
    {'P',   4,  26,  79, "SC-8850 DANCE Set", "Low Hoo"},
    {'P',   4,  26,  80, "SC-8850 DANCE Set", "Electric Mute Triangle"},
    {'P',   4,  26,  81, "SC-8850 DANCE Set", "Electric Open Triangle"},
    {'P',   4,  26,  82, "SC-8850 DANCE Set", "TR-626 Shaker"},
    {'P',   4,  26,  88, "SC-8850 DANCE Set", "Small Club 1"},
    {'P',   4,  26,  89, "SC-8850 DANCE Set", "TR-606 Snare 2"},
    {'P',   4,  26,  90, "SC-8850 DANCE Set", "Techno Snare"},
    {'P',   4,  26,  91, "SC-8850 DANCE Set", "House Snare"},
    {'P',   4,  26,  92, "SC-8850 DANCE Set", "Jungle Snare"},
    {'P',   4,  26,  93, "SC-8850 DANCE Set", "LoFi Snare 1"},
    {'P',   4,  26,  94, "SC-8850 DANCE Set", "LoFi Snare 2"},
    {'P',   4,  26,  95, "SC-8850 DANCE Set", "Hip-Hop Snare 1"},
    {'P',   4,  26,  96, "SC-8850 DANCE Set", "Hip-Hop Snare 2"},
    {'P',   4,  27,  26, "SC-8850 CR-78 Set", "Finger Snap 2"},
    {'P',   4,  27,  29, "SC-8850 CR-78 Set", "Scratch Push 2"},
    {'P',   4,  27,  30, "SC-8850 CR-78 Set", "Scratch Pull 2"},
    {'P',   4,  27,  35, "SC-8850 CR-78 Set", "CR-78 Kick 2"},
    {'P',   4,  27,  36, "SC-8850 CR-78 Set", "CR-78 Kick 1"},
    {'P',   4,  27,  37, "SC-8850 CR-78 Set", "CR-78 Rim Shot"},
    {'P',   4,  27,  38, "SC-8850 CR-78 Set", "CR-78 Snare 1"},
    {'P',   4,  27,  39, "SC-8850 CR-78 Set", "TR-707 Hand Clap"},
    {'P',   4,  27,  40, "SC-8850 CR-78 Set", "CR-78 Snare 2"},
    {'P',   4,  27,  41, "SC-8850 CR-78 Set", "CR-78 Low Tom 2"},
    {'P',   4,  27,  42, "SC-8850 CR-78 Set", "CR-78 Closed Hi-Hat"},
    {'P',   4,  27,  43, "SC-8850 CR-78 Set", "CR-78 Low Tom 1"},
    {'P',   4,  27,  44, "SC-8850 CR-78 Set", "TR-606 Closed Hi-Hat"},
    {'P',   4,  27,  45, "SC-8850 CR-78 Set", "CR-78 Mid Tom 2"},
    {'P',   4,  27,  46, "SC-8850 CR-78 Set", "CR-78 Open Hi-Hat"},
    {'P',   4,  27,  47, "SC-8850 CR-78 Set", "CR-78 Mid Tom 1"},
    {'P',   4,  27,  48, "SC-8850 CR-78 Set", "CR-78 High Tom 2"},
    {'P',   4,  27,  49, "SC-8850 CR-78 Set", "TR-808 Crash Cymbal"},
    {'P',   4,  27,  50, "SC-8850 CR-78 Set", "CR-78 High Tom 1"},
    {'P',   4,  27,  51, "SC-8850 CR-78 Set", "TR-606 Ride Cymbal"},
    {'P',   4,  27,  52, "SC-8850 CR-78 Set", "Chinese Cymbal"},
    {'P',   4,  27,  53, "SC-8850 CR-78 Set", "Ride Bell"},
    {'P',   4,  27,  54, "SC-8850 CR-78 Set", "CR-78 Tambourine"},
    {'P',   4,  27,  55, "SC-8850 CR-78 Set", "Splash Cymbal"},
    {'P',   4,  27,  56, "SC-8850 CR-78 Set", "CR-78 Cowbell"},
    {'P',   4,  27,  57, "SC-8850 CR-78 Set", "TR-909 Crash Cymbal"},
    {'P',   4,  27,  59, "SC-8850 CR-78 Set", "Ride Cymbal Edge"},
    {'P',   4,  27,  60, "SC-8850 CR-78 Set", "CR-78 High Bongo"},
    {'P',   4,  27,  61, "SC-8850 CR-78 Set", "CR-78 Low Bongo"},
    {'P',   4,  27,  62, "SC-8850 CR-78 Set", "TR-808 High Conga"},
    {'P',   4,  27,  63, "SC-8850 CR-78 Set", "TR-808 Mute Conga"},
    {'P',   4,  27,  64, "SC-8850 CR-78 Set", "TR-808 Low Conga"},
    {'P',   4,  27,  70, "SC-8850 CR-78 Set", "CR-78 Maracas"},
    {'P',   4,  27,  74, "SC-8850 CR-78 Set", "CR-78 Guiro"},
    {'P',   4,  27,  75, "SC-8850 CR-78 Set", "CR-78 Claves"},
    {'P',   4,  27,  78, "SC-8850 CR-78 Set", "High Hoo"},
    {'P',   4,  27,  79, "SC-8850 CR-78 Set", "Low Hoo"},
    {'P',   4,  27,  80, "SC-8850 CR-78 Set", "CR-78 Metalic Beat 1"},
    {'P',   4,  27,  81, "SC-8850 CR-78 Set", "CR-78 Metalic Beat 2"},
    {'P',   4,  27,  82, "SC-8850 CR-78 Set", "TR-626 Shaker"},
    {'P',   4,  27,  88, "SC-8850 CR-78 Set", "Small Club 1"},
    {'P',   4,  28,  26, "SC-8850 TR-606 Set", "Finger Snap 2"},
    {'P',   4,  28,  29, "SC-8850 TR-606 Set", "Scratch Push 2"},
    {'P',   4,  28,  30, "SC-8850 TR-606 Set", "Scratch Pull 2"},
    {'P',   4,  28,  35, "SC-8850 TR-606 Set", "CR-78 Kink 2"},
    {'P',   4,  28,  36, "SC-8850 TR-606 Set", "TR-606 Kick 1"},
    {'P',   4,  28,  37, "SC-8850 TR-606 Set", "CR-78 Rim Shot"},
    {'P',   4,  28,  38, "SC-8850 TR-606 Set", "TR-606 Snare 1"},
    {'P',   4,  28,  39, "SC-8850 TR-606 Set", "TR-707 Hand Clap"},
    {'P',   4,  28,  40, "SC-8850 TR-606 Set", "TR-606 Snare 2"},
    {'P',   4,  28,  41, "SC-8850 TR-606 Set", "TR-606 Low Tom 2"},
    {'P',   4,  28,  42, "SC-8850 TR-606 Set", "TR-606 Closed Hi-Hat"},
    {'P',   4,  28,  43, "SC-8850 TR-606 Set", "TR-606 Low Tom 1"},
    {'P',   4,  28,  44, "SC-8850 TR-606 Set", "TR-606 Closed Hi-Hat"},
    {'P',   4,  28,  45, "SC-8850 TR-606 Set", "TR-606 Mid Tom 2"},
    {'P',   4,  28,  46, "SC-8850 TR-606 Set", "TR-606 Open Hi-Hat"},
    {'P',   4,  28,  47, "SC-8850 TR-606 Set", "TR-606 Mid Tom 1"},
    {'P',   4,  28,  48, "SC-8850 TR-606 Set", "TR-606 High Tom 2"},
    {'P',   4,  28,  49, "SC-8850 TR-606 Set", "TR-808 Crash Cymbal"},
    {'P',   4,  28,  50, "SC-8850 TR-606 Set", "TR-606 High Tom 1"},
    {'P',   4,  28,  51, "SC-8850 TR-606 Set", "TR-606 Ride Cymbal"},
    {'P',   4,  28,  52, "SC-8850 TR-606 Set", "Chinese Cymbal"},
    {'P',   4,  28,  53, "SC-8850 TR-606 Set", "Ride Bell"},
    {'P',   4,  28,  54, "SC-8850 TR-606 Set", "CR-78 Tambourine"},
    {'P',   4,  28,  55, "SC-8850 TR-606 Set", "Splash Cymbal"},
    {'P',   4,  28,  56, "SC-8850 TR-606 Set", "CR-78 Cowbell"},
    {'P',   4,  28,  57, "SC-8850 TR-606 Set", "TR-909 Crash Cymbal"},
    {'P',   4,  28,  59, "SC-8850 TR-606 Set", "Ride Cymbal Edge"},
    {'P',   4,  28,  60, "SC-8850 TR-606 Set", "CR-78 High Bongo"},
    {'P',   4,  28,  61, "SC-8850 TR-606 Set", "CR-78 Low Bongo"},
    {'P',   4,  28,  62, "SC-8850 TR-606 Set", "TR-808 High Conga"},
    {'P',   4,  28,  63, "SC-8850 TR-606 Set", "TR-808 Mute Conga"},
    {'P',   4,  28,  64, "SC-8850 TR-606 Set", "TR-808 Low Conga"},
    {'P',   4,  28,  70, "SC-8850 TR-606 Set", "CR-78 Maracas"},
    {'P',   4,  28,  74, "SC-8850 TR-606 Set", "CR-78 Guiro"},
    {'P',   4,  28,  75, "SC-8850 TR-606 Set", "CR-78 Claves"},
    {'P',   4,  28,  78, "SC-8850 TR-606 Set", "High Hoo"},
    {'P',   4,  28,  79, "SC-8850 TR-606 Set", "Low Hoo"},
    {'P',   4,  28,  80, "SC-8850 TR-606 Set", "CR-78 Metalic Beat 1"},
    {'P',   4,  28,  81, "SC-8850 TR-606 Set", "CR-78 Metalic Beat 2"},
    {'P',   4,  28,  82, "SC-8850 TR-606 Set", "TR-626 Shaker"},
    {'P',   4,  28,  88, "SC-8850 TR-606 Set", "Small Club 1"},
    {'P',   4,  29,  26, "SC-8850 TR-707 Set", "Finger Snap 2"},
    {'P',   4,  29,  29, "SC-8850 TR-707 Set", "Scratch Push 2"},
    {'P',   4,  29,  30, "SC-8850 TR-707 Set", "Scratch Pull 2"},
    {'P',   4,  29,  35, "SC-8850 TR-707 Set", "TR-707 Kick 2"},
    {'P',   4,  29,  36, "SC-8850 TR-707 Set", "TR-707 Kick 1"},
    {'P',   4,  29,  37, "SC-8850 TR-707 Set", "TR-707 Rim Shot"},
    {'P',   4,  29,  38, "SC-8850 TR-707 Set", "TR-707 Snare 1"},
    {'P',   4,  29,  39, "SC-8850 TR-707 Set", "TR-707 Hand Clap"},
    {'P',   4,  29,  40, "SC-8850 TR-707 Set", "TR-707 Snare 2"},
    {'P',   4,  29,  41, "SC-8850 TR-707 Set", "TR-707 Low Tom 2"},
    {'P',   4,  29,  42, "SC-8850 TR-707 Set", "TR-707 Closed Hi-Hat"},
    {'P',   4,  29,  43, "SC-8850 TR-707 Set", "TR-707 Low Tom 1"},
    {'P',   4,  29,  44, "SC-8850 TR-707 Set", "TR-707 Closed Hi-Hat"},
    {'P',   4,  29,  45, "SC-8850 TR-707 Set", "TR-707 Mid Tom 2"},
    {'P',   4,  29,  46, "SC-8850 TR-707 Set", "TR-707 Open Hi-Hat"},
    {'P',   4,  29,  47, "SC-8850 TR-707 Set", "TR-707 Mid Tom 1"},
    {'P',   4,  29,  48, "SC-8850 TR-707 Set", "TR-707 High Tom 2"},
    {'P',   4,  29,  49, "SC-8850 TR-707 Set", "TR-909 Crash Cymbal"},
    {'P',   4,  29,  50, "SC-8850 TR-707 Set", "TR-707 High Tom 1"},
    {'P',   4,  29,  51, "SC-8850 TR-707 Set", "TR-909 Ride Cymbal"},
    {'P',   4,  29,  52, "SC-8850 TR-707 Set", "Chinese Cymbal"},
    {'P',   4,  29,  53, "SC-8850 TR-707 Set", "Ride Bell"},
    {'P',   4,  29,  54, "SC-8850 TR-707 Set", "Tambourine 2"},
    {'P',   4,  29,  55, "SC-8850 TR-707 Set", "Splash Cymbal"},
    {'P',   4,  29,  56, "SC-8850 TR-707 Set", "TR-808 Cowbell"},
    {'P',   4,  29,  57, "SC-8850 TR-707 Set", "Crash Cymbal 2"},
    {'P',   4,  29,  59, "SC-8850 TR-707 Set", "Ride Cymbal Edge"},
    {'P',   4,  29,  60, "SC-8850 TR-707 Set", "High Bongo"},
    {'P',   4,  29,  61, "SC-8850 TR-707 Set", "Low Bongo"},
    {'P',   4,  29,  62, "SC-8850 TR-707 Set", "Mute High Conga"},
    {'P',   4,  29,  63, "SC-8850 TR-707 Set", "Open High Conga"},
    {'P',   4,  29,  64, "SC-8850 TR-707 Set", "Low Conga"},
    {'P',   4,  29,  70, "SC-8850 TR-707 Set", "TR-808 Maracas"},
    {'P',   4,  29,  74, "SC-8850 TR-707 Set", "Long Guiro"},
    {'P',   4,  29,  75, "SC-8850 TR-707 Set", "Claves"},
    {'P',   4,  29,  78, "SC-8850 TR-707 Set", "High Hoo"},
    {'P',   4,  29,  79, "SC-8850 TR-707 Set", "Low Hoo"},
    {'P',   4,  29,  80, "SC-8850 TR-707 Set", "Electric Mute Triangle"},
    {'P',   4,  29,  81, "SC-8850 TR-707 Set", "Electric Open Triangle"},
    {'P',   4,  29,  82, "SC-8850 TR-707 Set", "TR-626 Shaker"},
    {'P',   4,  29,  88, "SC-8850 TR-707 Set", "Small Club 1"},
    {'P',   4,  30,  26, "SC-8850 TR-909 Set", "Finger Snap 2"},
    {'P',   4,  30,  29, "SC-8850 TR-909 Set", "Scratch Push 2"},
    {'P',   4,  30,  30, "SC-8850 TR-909 Set", "Scratch Pull 2"},
    {'P',   4,  30,  35, "SC-8850 TR-909 Set", "TR-909 Kick 2"},
    {'P',   4,  30,  36, "SC-8850 TR-909 Set", "TR-909 Kick 1"},
    {'P',   4,  30,  37, "SC-8850 TR-909 Set", "TR-909 Rim"},
    {'P',   4,  30,  38, "SC-8850 TR-909 Set", "TR-909 Snare 1"},
    {'P',   4,  30,  39, "SC-8850 TR-909 Set", "TR-909 Hand Clap"},
    {'P',   4,  30,  40, "SC-8850 TR-909 Set", "TR-909 Snare2"},
    {'P',   4,  30,  41, "SC-8850 TR-909 Set", "TR-909 Low Tom 2"},
    {'P',   4,  30,  42, "SC-8850 TR-909 Set", "TR-707 Closed Hi-Hat"},
    {'P',   4,  30,  43, "SC-8850 TR-909 Set", "TR-909 Low Tom 1"},
    {'P',   4,  30,  44, "SC-8850 TR-909 Set", "TR-707 Closed Hi-Hat"},
    {'P',   4,  30,  45, "SC-8850 TR-909 Set", "TR-909 Mid Tom 2"},
    {'P',   4,  30,  46, "SC-8850 TR-909 Set", "TR-909 Open Hi-Hat"},
    {'P',   4,  30,  47, "SC-8850 TR-909 Set", "TR-909 Mid Tom 1"},
    {'P',   4,  30,  48, "SC-8850 TR-909 Set", "TR-909 High Tom 2"},
    {'P',   4,  30,  49, "SC-8850 TR-909 Set", "TR-909 Crash Cymbal"},
    {'P',   4,  30,  50, "SC-8850 TR-909 Set", "TR-909 High Tom 1"},
    {'P',   4,  30,  51, "SC-8850 TR-909 Set", "TR-909 Ride Cymbal"},
    {'P',   4,  30,  52, "SC-8850 TR-909 Set", "Chinese Cymbal"},
    {'P',   4,  30,  53, "SC-8850 TR-909 Set", "Ride Bell"},
    {'P',   4,  30,  54, "SC-8850 TR-909 Set", "Tambourine 2"},
    {'P',   4,  30,  55, "SC-8850 TR-909 Set", "Splash Cymbal"},
    {'P',   4,  30,  56, "SC-8850 TR-909 Set", "TR-808 Cowbell"},
    {'P',   4,  30,  59, "SC-8850 TR-909 Set", "Ride Cymbal Edge"},
    {'P',   4,  30,  70, "SC-8850 TR-909 Set", "TR-808 Maracas"},
    {'P',   4,  30,  74, "SC-8850 TR-909 Set", "CR-78 Guiro"},
    {'P',   4,  30,  75, "SC-8850 TR-909 Set", "TR-808 Claves"},
    {'P',   4,  30,  78, "SC-8850 TR-909 Set", "High Hoo"},
    {'P',   4,  30,  79, "SC-8850 TR-909 Set", "Low Hoo"},
    {'P',   4,  30,  80, "SC-8850 TR-909 Set", "Electric Mute Triangle"},
    {'P',   4,  30,  81, "SC-8850 TR-909 Set", "Electric Open Triangle"},
    {'P',   4,  30,  82, "SC-8850 TR-909 Set", "TR-626 Shaker"},
    {'P',   4,  30,  88, "SC-8850 TR-909 Set", "Applause 2"},
    {'P',   4,  32,   0, "SC-8850 JAZZ Set", "[88] Standard 1 Kick 1"},
    {'P',   4,  32,   1, "SC-8850 JAZZ Set", "[88] Standard 1 Kick 2"},
    {'P',   4,  32,   2, "SC-8850 JAZZ Set", "[88] Standard 2 Kick 1"},
    {'P',   4,  32,   3, "SC-8850 JAZZ Set", "[88] Standard 2 Kick 2"},
    {'P',   4,  32,   4, "SC-8850 JAZZ Set", "[55] Kick Drum 1"},
    {'P',   4,  32,   5, "SC-8850 JAZZ Set", "[55] Kick Drum 2"},
    {'P',   4,  32,   6, "SC-8850 JAZZ Set", "[88] Jazz Kick 1"},
    {'P',   4,  32,   7, "SC-8850 JAZZ Set", "[88] Jazz Kick 2"},
    {'P',   4,  32,   8, "SC-8850 JAZZ Set", "[88] Room Kick 1"},
    {'P',   4,  32,   9, "SC-8850 JAZZ Set", "[88] Room Kick 2"},
    {'P',   4,  32,  10, "SC-8850 JAZZ Set", "[88] Power Kick 1"},
    {'P',   4,  32,  11, "SC-8850 JAZZ Set", "[88] Power Kick 2"},
    {'P',   4,  32,  12, "SC-8850 JAZZ Set", "[88] Electric Kick 2"},
    {'P',   4,  32,  13, "SC-8850 JAZZ Set", "[88] Electric Kick 1"},
    {'P',   4,  32,  14, "SC-8850 JAZZ Set", "[88] TR-808 Kick"},
    {'P',   4,  32,  15, "SC-8850 JAZZ Set", "[88] TR-909 Kick"},
    {'P',   4,  32,  16, "SC-8850 JAZZ Set", "[88] Dance Kick"},
    {'P',   4,  32,  17, "SC-8850 JAZZ Set", "[Pro] Voice One"},
    {'P',   4,  32,  18, "SC-8850 JAZZ Set", "[Pro] Voice Two"},
    {'P',   4,  32,  19, "SC-8850 JAZZ Set", "[Pro] Voice Three"},
    {'P',   4,  32,  20, "SC-8850 JAZZ Set", "Brush Kick 2"},
    {'P',   4,  32,  21, "SC-8850 JAZZ Set", "Brush Kick 1"},
    {'P',   4,  32,  22, "SC-8850 JAZZ Set", "MC-500 Beep 1"},
    {'P',   4,  32,  23, "SC-8850 JAZZ Set", "MC-500 Beep 2"},
    {'P',   4,  32,  24, "SC-8850 JAZZ Set", "Concert SD"},
    {'P',   4,  32,  25, "SC-8850 JAZZ Set", "Snare Roll"},
    {'P',   4,  32,  26, "SC-8850 JAZZ Set", "Finger Snap 2"},
    {'P',   4,  32,  27, "SC-8850 JAZZ Set", "High Q"},
    {'P',   4,  32,  28, "SC-8850 JAZZ Set", "Slap"},
    {'P',   4,  32,  29, "SC-8850 JAZZ Set", "Scratch Push"},
    {'P',   4,  32,  30, "SC-8850 JAZZ Set", "Scratch Pull"},
    {'P',   4,  32,  31, "SC-8850 JAZZ Set", "Sticks"},
    {'P',   4,  32,  32, "SC-8850 JAZZ Set", "Square Click"},
    {'P',   4,  32,  33, "SC-8850 JAZZ Set", "Metronome Click"},
    {'P',   4,  32,  34, "SC-8850 JAZZ Set", "Metronome Bell"},
    {'P',   4,  32,  35, "SC-8850 JAZZ Set", "Jazz Kick 2"},
    {'P',   4,  32,  36, "SC-8850 JAZZ Set", "Jazz Kick 1"},
    {'P',   4,  32,  37, "SC-8850 JAZZ Set", "Side Stick"},
    {'P',   4,  32,  38, "SC-8850 JAZZ Set", "Jazz Snare 1"},
    {'P',   4,  32,  39, "SC-8850 JAZZ Set", "Hand Clap 2"},
    {'P',   4,  32,  40, "SC-8850 JAZZ Set", "Jazz Snare 2"},
    {'P',   4,  32,  41, "SC-8850 JAZZ Set", "Jazz Low Tom 2"},
    {'P',   4,  32,  42, "SC-8850 JAZZ Set", "Closed Hi-Hat 2"},
    {'P',   4,  32,  43, "SC-8850 JAZZ Set", "Jazz Low Tom 1"},
    {'P',   4,  32,  44, "SC-8850 JAZZ Set", "Pedal Hi-Hat"},
    {'P',   4,  32,  45, "SC-8850 JAZZ Set", "Jazz Mid Tom 2"},
    {'P',   4,  32,  46, "SC-8850 JAZZ Set", "Open Hi-Hat 2"},
    {'P',   4,  32,  47, "SC-8850 JAZZ Set", "Jazz Mid Tom 1"},
    {'P',   4,  32,  48, "SC-8850 JAZZ Set", "Jazz High Tom 2"},
    {'P',   4,  32,  49, "SC-8850 JAZZ Set", "Jazz Crash Cymbal"},
    {'P',   4,  32,  50, "SC-8850 JAZZ Set", "Jazz High Tom 1"},
    {'P',   4,  32,  51, "SC-8850 JAZZ Set", "Jazz Ride Cymbal"},
    {'P',   4,  32,  52, "SC-8850 JAZZ Set", "Chinese Cymbal"},
    {'P',   4,  32,  53, "SC-8850 JAZZ Set", "Jazz Ride Bell"},
    {'P',   4,  32,  54, "SC-8850 JAZZ Set", "Tambourine"},
    {'P',   4,  32,  55, "SC-8850 JAZZ Set", "Splash Cymbal"},
    {'P',   4,  32,  56, "SC-8850 JAZZ Set", "Cowbell"},
    {'P',   4,  32,  57, "SC-8850 JAZZ Set", "Crash Cymbal 2"},
    {'P',   4,  32,  58, "SC-8850 JAZZ Set", "Vibra-slap"},
    {'P',   4,  32,  59, "SC-8850 JAZZ Set", "Ride Cymbal Edge"},
    {'P',   4,  32,  60, "SC-8850 JAZZ Set", "High Bongo"},
    {'P',   4,  32,  61, "SC-8850 JAZZ Set", "Low Bongo"},
    {'P',   4,  32,  62, "SC-8850 JAZZ Set", "Mute High Conga"},
    {'P',   4,  32,  63, "SC-8850 JAZZ Set", "Open High Conga"},
    {'P',   4,  32,  64, "SC-8850 JAZZ Set", "Low Conga"},
    {'P',   4,  32,  65, "SC-8850 JAZZ Set", "High Timbale"},
    {'P',   4,  32,  66, "SC-8850 JAZZ Set", "Low Timbale"},
    {'P',   4,  32,  67, "SC-8850 JAZZ Set", "High Agogo"},
    {'P',   4,  32,  68, "SC-8850 JAZZ Set", "Low Agogo"},
    {'P',   4,  32,  69, "SC-8850 JAZZ Set", "Cabasa"},
    {'P',   4,  32,  70, "SC-8850 JAZZ Set", "Maracas"},
    {'P',   4,  32,  71, "SC-8850 JAZZ Set", "Short High Whistle"},
    {'P',   4,  32,  72, "SC-8850 JAZZ Set", "Long Low Whistle"},
    {'P',   4,  32,  73, "SC-8850 JAZZ Set", "Short Guiro"},
    {'P',   4,  32,  74, "SC-8850 JAZZ Set", "Long Guiro"},
    {'P',   4,  32,  75, "SC-8850 JAZZ Set", "Claves"},
    {'P',   4,  32,  76, "SC-8850 JAZZ Set", "High Wood Block"},
    {'P',   4,  32,  77, "SC-8850 JAZZ Set", "Low Wood Block"},
    {'P',   4,  32,  78, "SC-8850 JAZZ Set", "Mute Cuica"},
    {'P',   4,  32,  79, "SC-8850 JAZZ Set", "Open Cuica"},
    {'P',   4,  32,  80, "SC-8850 JAZZ Set", "Mute Triangle"},
    {'P',   4,  32,  81, "SC-8850 JAZZ Set", "Open Triangle"},
    {'P',   4,  32,  82, "SC-8850 JAZZ Set", "Shaker"},
    {'P',   4,  32,  83, "SC-8850 JAZZ Set", "Jingle Bell"},
    {'P',   4,  32,  84, "SC-8850 JAZZ Set", "Bell Tree"},
    {'P',   4,  32,  85, "SC-8850 JAZZ Set", "Castanets"},
    {'P',   4,  32,  86, "SC-8850 JAZZ Set", "Mute Surdo"},
    {'P',   4,  32,  87, "SC-8850 JAZZ Set", "Open Surdo"},
    {'P',   4,  32,  88, "SC-8850 JAZZ Set", "Applause"},
    {'P',   4,  32,  98, "SC-8850 JAZZ Set", "Brush Tap 2"},
    {'P',   4,  32,  99, "SC-8850 JAZZ Set", "Brush Slap 2"},
    {'P',   4,  32, 100, "SC-8850 JAZZ Set", "[88] Brush Tap 1"},
    {'P',   4,  32, 101, "SC-8850 JAZZ Set", "[88] Brush Tap 2"},
    {'P',   4,  32, 102, "SC-8850 JAZZ Set", "[88] Brush Slap 1"},
    {'P',   4,  32, 103, "SC-8850 JAZZ Set", "[88] Brush Slap 2"},
    {'P',   4,  32, 104, "SC-8850 JAZZ Set", "[88] Brush Slap 3"},
    {'P',   4,  32, 105, "SC-8850 JAZZ Set", "[88] Brush Swirl 1"},
    {'P',   4,  32, 106, "SC-8850 JAZZ Set", "[88] Brush Swirl 2"},
    {'P',   4,  32, 107, "SC-8850 JAZZ Set", "[88] Brush Long Swirl"},
    {'P',   4,  32, 108, "SC-8850 JAZZ Set", "[88] Jazz Snare 1"},
    {'P',   4,  32, 109, "SC-8850 JAZZ Set", "[88] Jazz Snare 2"},
    {'P',   4,  32, 110, "SC-8850 JAZZ Set", "[88] Standard 1 Snare1"},
    {'P',   4,  32, 111, "SC-8850 JAZZ Set", "[88] Standard 1 Snare2"},
    {'P',   4,  32, 112, "SC-8850 JAZZ Set", "[88] Standard 2 Snare1"},
    {'P',   4,  32, 113, "SC-8850 JAZZ Set", "[88] Standard 2 Snare2"},
    {'P',   4,  32, 114, "SC-8850 JAZZ Set", "[55] Snare Drum 2"},
    {'P',   4,  32, 115, "SC-8850 JAZZ Set", "[Pro] Standard 1 Snare 1"},
    {'P',   4,  32, 116, "SC-8850 JAZZ Set", "[Pro] Standard 1 Snare 2"},
    {'P',   4,  32, 117, "SC-8850 JAZZ Set", "[Pro] Standard 1 Snare 3"},
    {'P',   4,  32, 118, "SC-8850 JAZZ Set", "[88] Room Snare 1"},
    {'P',   4,  32, 119, "SC-8850 JAZZ Set", "[88] Room Snare 2"},
    {'P',   4,  32, 120, "SC-8850 JAZZ Set", "[88] Power Snare 1"},
    {'P',   4,  32, 121, "SC-8850 JAZZ Set", "[88] Power Snare 2"},
    {'P',   4,  32, 122, "SC-8850 JAZZ Set", "[88] Gated Snare"},
    {'P',   4,  32, 123, "SC-8850 JAZZ Set", "[88] Dance Snare 1"},
    {'P',   4,  32, 124, "SC-8850 JAZZ Set", "[88] Dance Snare 2"},
    {'P',   4,  32, 125, "SC-8850 JAZZ Set", "[88] Disco Snare"},
    {'P',   4,  32, 126, "SC-8850 JAZZ Set", "[88] Electric Snare 2"},
    {'P',   4,  32, 127, "SC-8850 JAZZ Set", "[88] Electric Snare 3"},
    {'P',   4,  33,  26, "SC-8850 JAZZ L/R Set", "Finger Snap 2"},
    {'P',   4,  33,  29, "SC-8850 JAZZ L/R Set", "Scratch Push"},
    {'P',   4,  33,  30, "SC-8850 JAZZ L/R Set", "Scratch Pull"},
    {'P',   4,  33,  35, "SC-8850 JAZZ L/R Set", "[RND] Jazz Kick 2"},
    {'P',   4,  33,  36, "SC-8850 JAZZ L/R Set", "[RND] Jazz Kick 1"},
    {'P',   4,  33,  37, "SC-8850 JAZZ L/R Set", "Side Stick"},
    {'P',   4,  33,  38, "SC-8850 JAZZ L/R Set", "[RND] Jazz Snare 1"},
    {'P',   4,  33,  39, "SC-8850 JAZZ L/R Set", "Hand Clap 2"},
    {'P',   4,  33,  40, "SC-8850 JAZZ L/R Set", "[RND] Jazz Snare 2"},
    {'P',   4,  33,  41, "SC-8850 JAZZ L/R Set", "[RND] Jazz Low Tom 2"},
    {'P',   4,  33,  42, "SC-8850 JAZZ L/R Set", "[RND] Jazz Closed Hi-Hat"},
    {'P',   4,  33,  43, "SC-8850 JAZZ L/R Set", "[RND] Jazz Low Tom 1"},
    {'P',   4,  33,  44, "SC-8850 JAZZ L/R Set", "Pedal Hi-Hat"},
    {'P',   4,  33,  45, "SC-8850 JAZZ L/R Set", "[RND] Jazz Mid Tom 2"},
    {'P',   4,  33,  46, "SC-8850 JAZZ L/R Set", "[RND] Jazz Open Hi-Hat"},
    {'P',   4,  33,  47, "SC-8850 JAZZ L/R Set", "[RND] Jazz Mid Tom 1"},
    {'P',   4,  33,  48, "SC-8850 JAZZ L/R Set", "[RND] Jazz High Tom 2"},
    {'P',   4,  33,  49, "SC-8850 JAZZ L/R Set", "[RND] Jazz Crash Cymbal"},
    {'P',   4,  33,  50, "SC-8850 JAZZ L/R Set", "[RND] Jazz High Tom 1"},
    {'P',   4,  33,  51, "SC-8850 JAZZ L/R Set", "[RND] Jazz Ride Cymbal"},
    {'P',   4,  33,  52, "SC-8850 JAZZ L/R Set", "Chinese Cymbal"},
    {'P',   4,  33,  53, "SC-8850 JAZZ L/R Set", "[RND] Jazz Ride Bell"},
    {'P',   4,  33,  54, "SC-8850 JAZZ L/R Set", "Tambourine"},
    {'P',   4,  33,  55, "SC-8850 JAZZ L/R Set", "Splash Cymbal"},
    {'P',   4,  33,  56, "SC-8850 JAZZ L/R Set", "Cowbell"},
    {'P',   4,  33,  59, "SC-8850 JAZZ L/R Set", "Ride Cymbal Edge"},
    {'P',   4,  33,  88, "SC-8850 JAZZ L/R Set", "Applause"},
    {'P',   4,  33,  95, "SC-8850 JAZZ L/R Set", "[L] Jazz Kick 2"},
    {'P',   4,  33,  96, "SC-8850 JAZZ L/R Set", "[L] Jazz Kick 1"},
    {'P',   4,  33,  97, "SC-8850 JAZZ L/R Set", "[L] Jazz Crash Cymbal"},
    {'P',   4,  33,  98, "SC-8850 JAZZ L/R Set", "[L] Jazz Snare 1"},
    {'P',   4,  33,  99, "SC-8850 JAZZ L/R Set", "[L] Jazz Ride Cymbal"},
    {'P',   4,  33, 100, "SC-8850 JAZZ L/R Set", "[L] Jazz Snare 2"},
    {'P',   4,  33, 101, "SC-8850 JAZZ L/R Set", "[L] Jazz Low Tom"},
    {'P',   4,  33, 102, "SC-8850 JAZZ L/R Set", "[L] Jazz Closed Hi-Hat"},
    {'P',   4,  33, 103, "SC-8850 JAZZ L/R Set", "[L] Jazz Mid Tom"},
    {'P',   4,  33, 104, "SC-8850 JAZZ L/R Set", "[L] Jazz Ride Bell"},
    {'P',   4,  33, 105, "SC-8850 JAZZ L/R Set", "[L] Jazz High Tom"},
    {'P',   4,  33, 106, "SC-8850 JAZZ L/R Set", "[L] Jazz Open Hi-Hat"},
    {'P',   4,  33, 107, "SC-8850 JAZZ L/R Set", "[R] Jazz Kick 2"},
    {'P',   4,  33, 108, "SC-8850 JAZZ L/R Set", "[R] Jazz Kick 1"},
    {'P',   4,  33, 109, "SC-8850 JAZZ L/R Set", "[R] Jazz Crash Cymbal"},
    {'P',   4,  33, 110, "SC-8850 JAZZ L/R Set", "[R] Jazz Snare 1"},
    {'P',   4,  33, 111, "SC-8850 JAZZ L/R Set", "[R] Jazz Ride Cymbal"},
    {'P',   4,  33, 112, "SC-8850 JAZZ L/R Set", "[R] Jazz Snare 2"},
    {'P',   4,  33, 113, "SC-8850 JAZZ L/R Set", "[R] Jazz Low Tom"},
    {'P',   4,  33, 114, "SC-8850 JAZZ L/R Set", "[R] Jazz Closed Hi-Hat"},
    {'P',   4,  33, 115, "SC-8850 JAZZ L/R Set", "[R] Jazz Mid Tom"},
    {'P',   4,  33, 116, "SC-8850 JAZZ L/R Set", "[R] Jazz Ride Bell"},
    {'P',   4,  33, 117, "SC-8850 JAZZ L/R Set", "[R] Jazz High Tom"},
    {'P',   4,  33, 118, "SC-8850 JAZZ L/R Set", "[R] Jazz Open Hi-Hat"},
    {'P',   4,  40,   0, "SC-8850 BRUSH Set", "[88] Standard 1 Kick 1"},
    {'P',   4,  40,   1, "SC-8850 BRUSH Set", "[88] Standard 1 Kick 2"},
    {'P',   4,  40,   2, "SC-8850 BRUSH Set", "[88] Standard 2 Kick 1"},
    {'P',   4,  40,   3, "SC-8850 BRUSH Set", "[88] Standard 2 Kick 2"},
    {'P',   4,  40,   4, "SC-8850 BRUSH Set", "[55] Kick Drum 1"},
    {'P',   4,  40,   5, "SC-8850 BRUSH Set", "[55] Kick Drum 2"},
    {'P',   4,  40,   6, "SC-8850 BRUSH Set", "[88] Jazz Kick 1"},
    {'P',   4,  40,   7, "SC-8850 BRUSH Set", "[88] Jazz Kick 2"},
    {'P',   4,  40,   8, "SC-8850 BRUSH Set", "[88] Room Kick 1"},
    {'P',   4,  40,   9, "SC-8850 BRUSH Set", "[88] Room Kick 2"},
    {'P',   4,  40,  10, "SC-8850 BRUSH Set", "[88] Power Kick 1"},
    {'P',   4,  40,  11, "SC-8850 BRUSH Set", "[88] Power Kick 2"},
    {'P',   4,  40,  12, "SC-8850 BRUSH Set", "[88] Electric Kick 2"},
    {'P',   4,  40,  13, "SC-8850 BRUSH Set", "[88] Electric Kick 1"},
    {'P',   4,  40,  14, "SC-8850 BRUSH Set", "[88] TR-808 Kick"},
    {'P',   4,  40,  15, "SC-8850 BRUSH Set", "[88] TR-909 Kick"},
    {'P',   4,  40,  16, "SC-8850 BRUSH Set", "[88] Dance Kick"},
    {'P',   4,  40,  17, "SC-8850 BRUSH Set", "[Pro] Voice One"},
    {'P',   4,  40,  18, "SC-8850 BRUSH Set", "[Pro] Voice Two"},
    {'P',   4,  40,  19, "SC-8850 BRUSH Set", "[Pro] Voice Three"},
    {'P',   4,  40,  22, "SC-8850 BRUSH Set", "MC-500 Beep 1"},
    {'P',   4,  40,  23, "SC-8850 BRUSH Set", "MC-500 Beep 2"},
    {'P',   4,  40,  24, "SC-8850 BRUSH Set", "Concert SD"},
    {'P',   4,  40,  25, "SC-8850 BRUSH Set", "Snare Roll"},
    {'P',   4,  40,  26, "SC-8850 BRUSH Set", "Finger Snap 2"},
    {'P',   4,  40,  27, "SC-8850 BRUSH Set", "High Q"},
    {'P',   4,  40,  28, "SC-8850 BRUSH Set", "Slap"},
    {'P',   4,  40,  29, "SC-8850 BRUSH Set", "Scratch Push"},
    {'P',   4,  40,  30, "SC-8850 BRUSH Set", "Scratch Pull"},
    {'P',   4,  40,  31, "SC-8850 BRUSH Set", "Sticks"},
    {'P',   4,  40,  32, "SC-8850 BRUSH Set", "Square Click"},
    {'P',   4,  40,  33, "SC-8850 BRUSH Set", "Metronome Click"},
    {'P',   4,  40,  34, "SC-8850 BRUSH Set", "Metronome Bell"},
    {'P',   4,  40,  35, "SC-8850 BRUSH Set", "Jazz Kick 2"},
    {'P',   4,  40,  36, "SC-8850 BRUSH Set", "Jazz Kick 1"},
    {'P',   4,  40,  37, "SC-8850 BRUSH Set", "Side Stick"},
    {'P',   4,  40,  38, "SC-8850 BRUSH Set", "Brush Tap 1"},
    {'P',   4,  40,  39, "SC-8850 BRUSH Set", "Brush Slap 1"},
    {'P',   4,  40,  40, "SC-8850 BRUSH Set", "Brush Swirl 1"},
    {'P',   4,  40,  41, "SC-8850 BRUSH Set", "Brush Low Tom 2"},
    {'P',   4,  40,  42, "SC-8850 BRUSH Set", "Brush Closed Hi-Hat"},
    {'P',   4,  40,  43, "SC-8850 BRUSH Set", "Brush Low Tom 1"},
    {'P',   4,  40,  44, "SC-8850 BRUSH Set", "Pedal Hi-Hat"},
    {'P',   4,  40,  45, "SC-8850 BRUSH Set", "Brush Mid Tom 2"},
    {'P',   4,  40,  46, "SC-8850 BRUSH Set", "Brush Open Hi-Hat"},
    {'P',   4,  40,  47, "SC-8850 BRUSH Set", "Brush Mid Tom 1"},
    {'P',   4,  40,  48, "SC-8850 BRUSH Set", "Brush High Tom 2"},
    {'P',   4,  40,  49, "SC-8850 BRUSH Set", "Brush Crash Cymbal"},
    {'P',   4,  40,  50, "SC-8850 BRUSH Set", "Brush High Tom 1"},
    {'P',   4,  40,  51, "SC-8850 BRUSH Set", "Ride Cymbal Inner"},
    {'P',   4,  40,  52, "SC-8850 BRUSH Set", "Chinese Cymbal"},
    {'P',   4,  40,  53, "SC-8850 BRUSH Set", "Brush Ride Bell"},
    {'P',   4,  40,  54, "SC-8850 BRUSH Set", "Tambourine"},
    {'P',   4,  40,  55, "SC-8850 BRUSH Set", "Splash Cymbal"},
    {'P',   4,  40,  56, "SC-8850 BRUSH Set", "Cowbell"},
    {'P',   4,  40,  57, "SC-8850 BRUSH Set", "Crash Cymbal 2"},
    {'P',   4,  40,  58, "SC-8850 BRUSH Set", "Vibra-slap"},
    {'P',   4,  40,  59, "SC-8850 BRUSH Set", "Ride Cymbal Edge"},
    {'P',   4,  40,  60, "SC-8850 BRUSH Set", "High Bongo"},
    {'P',   4,  40,  61, "SC-8850 BRUSH Set", "Low Bongo"},
    {'P',   4,  40,  62, "SC-8850 BRUSH Set", "Mute High Conga"},
    {'P',   4,  40,  63, "SC-8850 BRUSH Set", "Open High Conga"},
    {'P',   4,  40,  64, "SC-8850 BRUSH Set", "Low Conga"},
    {'P',   4,  40,  65, "SC-8850 BRUSH Set", "High Timbale"},
    {'P',   4,  40,  66, "SC-8850 BRUSH Set", "Low Timbale"},
    {'P',   4,  40,  67, "SC-8850 BRUSH Set", "High Agogo"},
    {'P',   4,  40,  68, "SC-8850 BRUSH Set", "Low Agogo"},
    {'P',   4,  40,  69, "SC-8850 BRUSH Set", "Cabasa"},
    {'P',   4,  40,  70, "SC-8850 BRUSH Set", "Maracas"},
    {'P',   4,  40,  71, "SC-8850 BRUSH Set", "Short High Whistle"},
    {'P',   4,  40,  72, "SC-8850 BRUSH Set", "Long Low Whistle"},
    {'P',   4,  40,  73, "SC-8850 BRUSH Set", "Short Guiro"},
    {'P',   4,  40,  74, "SC-8850 BRUSH Set", "Long Guiro"},
    {'P',   4,  40,  75, "SC-8850 BRUSH Set", "Claves"},
    {'P',   4,  40,  76, "SC-8850 BRUSH Set", "High Wood Block"},
    {'P',   4,  40,  77, "SC-8850 BRUSH Set", "Low Wood Block"},
    {'P',   4,  40,  78, "SC-8850 BRUSH Set", "Mute Cuica"},
    {'P',   4,  40,  79, "SC-8850 BRUSH Set", "Open Cuica"},
    {'P',   4,  40,  80, "SC-8850 BRUSH Set", "Mute Triangle"},
    {'P',   4,  40,  81, "SC-8850 BRUSH Set", "Open Triangle"},
    {'P',   4,  40,  82, "SC-8850 BRUSH Set", "Shaker"},
    {'P',   4,  40,  83, "SC-8850 BRUSH Set", "Jingle Bell"},
    {'P',   4,  40,  84, "SC-8850 BRUSH Set", "Bell Tree"},
    {'P',   4,  40,  85, "SC-8850 BRUSH Set", "Castanets"},
    {'P',   4,  40,  86, "SC-8850 BRUSH Set", "Mute Surdo"},
    {'P',   4,  40,  87, "SC-8850 BRUSH Set", "Open Surdo"},
    {'P',   4,  40,  88, "SC-8850 BRUSH Set", "Applause"},
    {'P',   4,  40, 100, "SC-8850 BRUSH Set", "[88] Brush Tap 1"},
    {'P',   4,  40, 101, "SC-8850 BRUSH Set", "[88] Brush Tap 2"},
    {'P',   4,  40, 102, "SC-8850 BRUSH Set", "[88] Brush Slap 1"},
    {'P',   4,  40, 103, "SC-8850 BRUSH Set", "[88] Brush Slap 2"},
    {'P',   4,  40, 104, "SC-8850 BRUSH Set", "[88] Brush Slap 3"},
    {'P',   4,  40, 105, "SC-8850 BRUSH Set", "[88] Brush Swirl 1"},
    {'P',   4,  40, 106, "SC-8850 BRUSH Set", "[88] Brush Swirl 2"},
    {'P',   4,  40, 107, "SC-8850 BRUSH Set", "[88] Brush Long Swirl"},
    {'P',   4,  40, 108, "SC-8850 BRUSH Set", "[88] Jazz Snare 1"},
    {'P',   4,  40, 109, "SC-8850 BRUSH Set", "[88] Jazz Snare 2"},
    {'P',   4,  40, 110, "SC-8850 BRUSH Set", "[88] Standard 1 Snare1"},
    {'P',   4,  40, 111, "SC-8850 BRUSH Set", "[88] Standard 1 Snare2"},
    {'P',   4,  40, 112, "SC-8850 BRUSH Set", "[88] Standard 2 Snare1"},
    {'P',   4,  40, 113, "SC-8850 BRUSH Set", "[88] Standard 2 Snare2"},
    {'P',   4,  40, 114, "SC-8850 BRUSH Set", "[55] Snare Drum 2"},
    {'P',   4,  40, 115, "SC-8850 BRUSH Set", "Standard 1 Snare 1"},
    {'P',   4,  40, 116, "SC-8850 BRUSH Set", "Standard 1 Snare 2"},
    {'P',   4,  40, 117, "SC-8850 BRUSH Set", "Standard 1 Snare 3"},
    {'P',   4,  40, 118, "SC-8850 BRUSH Set", "[88] Room Snare 1"},
    {'P',   4,  40, 119, "SC-8850 BRUSH Set", "[88] Room Snare 2"},
    {'P',   4,  40, 120, "SC-8850 BRUSH Set", "[88] Power Snare 1"},
    {'P',   4,  40, 121, "SC-8850 BRUSH Set", "[88] Power Snare 2"},
    {'P',   4,  40, 122, "SC-8850 BRUSH Set", "[88] Gated Snare"},
    {'P',   4,  40, 123, "SC-8850 BRUSH Set", "[88] Dance Snare 1"},
    {'P',   4,  40, 124, "SC-8850 BRUSH Set", "[88] Dance Snare 2"},
    {'P',   4,  40, 125, "SC-8850 BRUSH Set", "[88] Disco Snare"},
    {'P',   4,  40, 126, "SC-8850 BRUSH Set", "[88] Electric Snare 2"},
    {'P',   4,  40, 127, "SC-8850 BRUSH Set", "[88] Electric Snare 3"},
    {'P',   4,  41,  20, "SC-8850 BRUSH 2 Set", "Jazz Kick 2"},
    {'P',   4,  41,  21, "SC-8850 BRUSH 2 Set", "Jazz Kick 1"},
    {'P',   4,  41,  26, "SC-8850 BRUSH 2 Set", "Finger Snap 2"},
    {'P',   4,  41,  29, "SC-8850 BRUSH 2 Set", "Scratch Push"},
    {'P',   4,  41,  30, "SC-8850 BRUSH 2 Set", "Scratch Pull"},
    {'P',   4,  41,  35, "SC-8850 BRUSH 2 Set", "Brush Kick 2"},
    {'P',   4,  41,  36, "SC-8850 BRUSH 2 Set", "Brush Kick 1"},
    {'P',   4,  41,  37, "SC-8850 BRUSH 2 Set", "Side Stick"},
    {'P',   4,  41,  38, "SC-8850 BRUSH 2 Set", "Brush Tap 2"},
    {'P',   4,  41,  39, "SC-8850 BRUSH 2 Set", "Brush Slap 2"},
    {'P',   4,  41,  40, "SC-8850 BRUSH 2 Set", "Brush Swirl 1"},
    {'P',   4,  41,  41, "SC-8850 BRUSH 2 Set", "Brush Low Tom 2"},
    {'P',   4,  41,  42, "SC-8850 BRUSH 2 Set", "Brush Closed Hi-Hat"},
    {'P',   4,  41,  43, "SC-8850 BRUSH 2 Set", "Brush Low Tom 1"},
    {'P',   4,  41,  44, "SC-8850 BRUSH 2 Set", "Pedal Hi-Hat"},
    {'P',   4,  41,  45, "SC-8850 BRUSH 2 Set", "Brush Mid Tom 2"},
    {'P',   4,  41,  46, "SC-8850 BRUSH 2 Set", "Brush Open Hi-Hat"},
    {'P',   4,  41,  47, "SC-8850 BRUSH 2 Set", "Brush Mid Tom 1"},
    {'P',   4,  41,  48, "SC-8850 BRUSH 2 Set", "Brush High Tom 2"},
    {'P',   4,  41,  49, "SC-8850 BRUSH 2 Set", "Brush Crash Cymbal"},
    {'P',   4,  41,  50, "SC-8850 BRUSH 2 Set", "Brush High Tom 1"},
    {'P',   4,  41,  51, "SC-8850 BRUSH 2 Set", "Brush Ride Cymbal"},
    {'P',   4,  41,  52, "SC-8850 BRUSH 2 Set", "Chinese Cymbal"},
    {'P',   4,  41,  53, "SC-8850 BRUSH 2 Set", "Brush Ride Bell"},
    {'P',   4,  41,  54, "SC-8850 BRUSH 2 Set", "Tambourine"},
    {'P',   4,  41,  55, "SC-8850 BRUSH 2 Set", "Splash Cymbal"},
    {'P',   4,  41,  56, "SC-8850 BRUSH 2 Set", "Cowbell"},
    {'P',   4,  41,  59, "SC-8850 BRUSH 2 Set", "Ride Cymbal Edge"},
    {'P',   4,  41,  88, "SC-8850 BRUSH 2 Set", "Applause"},
    {'P',   4,  41,  98, "SC-8850 BRUSH 2 Set", "Jazz Snare 1"},
    {'P',   4,  41,  99, "SC-8850 BRUSH 2 Set", "Jazz Snare 2"},
    {'P',   4,  42,  26, "SC-8850 BRUSH 2 L/R Set", "Finger Snap 2"},
    {'P',   4,  42,  29, "SC-8850 BRUSH 2 L/R Set", "Scratch Push"},
    {'P',   4,  42,  30, "SC-8850 BRUSH 2 L/R Set", "Scratch Pull"},
    {'P',   4,  42,  35, "SC-8850 BRUSH 2 L/R Set", "[RND] Brush Kick 2"},
    {'P',   4,  42,  36, "SC-8850 BRUSH 2 L/R Set", "[RND] Brush Kick 1"},
    {'P',   4,  42,  38, "SC-8850 BRUSH 2 L/R Set", "[RND] Brush Tap 2"},
    {'P',   4,  42,  39, "SC-8850 BRUSH 2 L/R Set", "[RND] Brush Slap 2"},
    {'P',   4,  42,  40, "SC-8850 BRUSH 2 L/R Set", "Brush Swirl 1"},
    {'P',   4,  42,  41, "SC-8850 BRUSH 2 L/R Set", "[RND] Brush Low Tom 2"},
    {'P',   4,  42,  42, "SC-8850 BRUSH 2 L/R Set", "[RND] Brush Closed Hi-Hat"},
    {'P',   4,  42,  43, "SC-8850 BRUSH 2 L/R Set", "[RND] Brush Low Tom 1"},
    {'P',   4,  42,  44, "SC-8850 BRUSH 2 L/R Set", "Pedal Hi-Hat"},
    {'P',   4,  42,  45, "SC-8850 BRUSH 2 L/R Set", "[RND] Brush Mid Tom 2"},
    {'P',   4,  42,  46, "SC-8850 BRUSH 2 L/R Set", "[RND] Brush Open Hi-Hat"},
    {'P',   4,  42,  47, "SC-8850 BRUSH 2 L/R Set", "[RND] Brush Mid Tom 1"},
    {'P',   4,  42,  48, "SC-8850 BRUSH 2 L/R Set", "[RND] Brush High Tom 2"},
    {'P',   4,  42,  49, "SC-8850 BRUSH 2 L/R Set", "[RND] Brush Crash Cymbal"},
    {'P',   4,  42,  50, "SC-8850 BRUSH 2 L/R Set", "[RND] Brush High Tom 1"},
    {'P',   4,  42,  51, "SC-8850 BRUSH 2 L/R Set", "[RND] Brush Ride Cymbal"},
    {'P',   4,  42,  52, "SC-8850 BRUSH 2 L/R Set", "Chinese Cymbal"},
    {'P',   4,  42,  53, "SC-8850 BRUSH 2 L/R Set", "[RND] Brush Ride Bell"},
    {'P',   4,  42,  54, "SC-8850 BRUSH 2 L/R Set", "Tambourine"},
    {'P',   4,  42,  55, "SC-8850 BRUSH 2 L/R Set", "Splash Cymbal"},
    {'P',   4,  42,  56, "SC-8850 BRUSH 2 L/R Set", "Cowbell"},
    {'P',   4,  42,  59, "SC-8850 BRUSH 2 L/R Set", "Ride Cymbal Edge"},
    {'P',   4,  42,  88, "SC-8850 BRUSH 2 L/R Set", "Applause"},
    {'P',   4,  42,  95, "SC-8850 BRUSH 2 L/R Set", "[L] Brush Kick 2"},
    {'P',   4,  42,  96, "SC-8850 BRUSH 2 L/R Set", "[L] Brush Kick 1"},
    {'P',   4,  42,  97, "SC-8850 BRUSH 2 L/R Set", "[L] Brush Crash Cymbal"},
    {'P',   4,  42,  98, "SC-8850 BRUSH 2 L/R Set", "[L] Brush Tap 2"},
    {'P',   4,  42,  99, "SC-8850 BRUSH 2 L/R Set", "[L] Brush Ride Cymbal"},
    {'P',   4,  42, 100, "SC-8850 BRUSH 2 L/R Set", "[L] Brush Slap 2"},
    {'P',   4,  42, 101, "SC-8850 BRUSH 2 L/R Set", "[L] Brush Low Tom"},
    {'P',   4,  42, 102, "SC-8850 BRUSH 2 L/R Set", "[L] Brush Closed Hi-Hat"},
    {'P',   4,  42, 103, "SC-8850 BRUSH 2 L/R Set", "[L] Brush Mid Tom"},
    {'P',   4,  42, 104, "SC-8850 BRUSH 2 L/R Set", "[L] Brush Ride Bell"},
    {'P',   4,  42, 105, "SC-8850 BRUSH 2 L/R Set", "[L] Brush High Tom"},
    {'P',   4,  42, 106, "SC-8850 BRUSH 2 L/R Set", "[L] Brush Open Hi-Hat"},
    {'P',   4,  42, 107, "SC-8850 BRUSH 2 L/R Set", "[R] Brush Kick 2"},
    {'P',   4,  42, 108, "SC-8850 BRUSH 2 L/R Set", "[R] Brush Kick 1"},
    {'P',   4,  42, 109, "SC-8850 BRUSH 2 L/R Set", "[R] Brush Crash Cymbal"},
    {'P',   4,  42, 110, "SC-8850 BRUSH 2 L/R Set", "[R] Brush Tap 2"},
    {'P',   4,  42, 111, "SC-8850 BRUSH 2 L/R Set", "[R] Brush Ride Cymbal"},
    {'P',   4,  42, 112, "SC-8850 BRUSH 2 L/R Set", "[R] Brush Slap 2"},
    {'P',   4,  42, 113, "SC-8850 BRUSH 2 L/R Set", "[R] Brush Low Tom"},
    {'P',   4,  42, 114, "SC-8850 BRUSH 2 L/R Set", "[R] Brush Closed Hi-Hat"},
    {'P',   4,  42, 115, "SC-8850 BRUSH 2 L/R Set", "[R] Brush Mid Tom"},
    {'P',   4,  42, 116, "SC-8850 BRUSH 2 L/R Set", "[R] Brush Ride Bell"},
    {'P',   4,  42, 117, "SC-8850 BRUSH 2 L/R Set", "[R] Brush High Tom"},
    {'P',   4,  42, 118, "SC-8850 BRUSH 2 L/R Set", "[R] Brush Open Hi-Hat"},
    {'P',   4,  48,   0, "SC-8850 ORCHESTRA Set", "[88] Standard 1 Kick 1"},
    {'P',   4,  48,   1, "SC-8850 ORCHESTRA Set", "[88] Standard 1 Kick 2"},
    {'P',   4,  48,   2, "SC-8850 ORCHESTRA Set", "[88] Standard 2 Kick 1"},
    {'P',   4,  48,   3, "SC-8850 ORCHESTRA Set", "[88] Standard 2 Kick 2"},
    {'P',   4,  48,   4, "SC-8850 ORCHESTRA Set", "[55] Kick Drum 1"},
    {'P',   4,  48,   5, "SC-8850 ORCHESTRA Set", "[55] Kick Drum 2"},
    {'P',   4,  48,   6, "SC-8850 ORCHESTRA Set", "[88] Jazz Kick 1"},
    {'P',   4,  48,   7, "SC-8850 ORCHESTRA Set", "[88] Jazz Kick 2"},
    {'P',   4,  48,   8, "SC-8850 ORCHESTRA Set", "[88] Room Kick 1"},
    {'P',   4,  48,   9, "SC-8850 ORCHESTRA Set", "[88] Room Kick 2"},
    {'P',   4,  48,  10, "SC-8850 ORCHESTRA Set", "[88] Power Kick 1"},
    {'P',   4,  48,  11, "SC-8850 ORCHESTRA Set", "[88] Power Kick 2"},
    {'P',   4,  48,  12, "SC-8850 ORCHESTRA Set", "[88] Electric Kick 2"},
    {'P',   4,  48,  13, "SC-8850 ORCHESTRA Set", "[88] Electric Kick 1"},
    {'P',   4,  48,  14, "SC-8850 ORCHESTRA Set", "[88] TR-808 Kick"},
    {'P',   4,  48,  15, "SC-8850 ORCHESTRA Set", "[88] TR-909 Kick"},
    {'P',   4,  48,  16, "SC-8850 ORCHESTRA Set", "[88] Dance Kick"},
    {'P',   4,  48,  17, "SC-8850 ORCHESTRA Set", "[Pro] Voice One"},
    {'P',   4,  48,  18, "SC-8850 ORCHESTRA Set", "[Pro] Voice Two"},
    {'P',   4,  48,  19, "SC-8850 ORCHESTRA Set", "[Pro] Voice Three"},
    {'P',   4,  48,  22, "SC-8850 ORCHESTRA Set", "MC-500 Beep 1"},
    {'P',   4,  48,  23, "SC-8850 ORCHESTRA Set", "MC-500 Beep 2"},
    {'P',   4,  48,  24, "SC-8850 ORCHESTRA Set", "Concert SD"},
    {'P',   4,  48,  25, "SC-8850 ORCHESTRA Set", "Snare Roll"},
    {'P',   4,  48,  26, "SC-8850 ORCHESTRA Set", "Finger Snap"},
    {'P',   4,  48,  27, "SC-8850 ORCHESTRA Set", "Closed Hi-Hat 2"},
    {'P',   4,  48,  28, "SC-8850 ORCHESTRA Set", "Pedal Hi-Hat"},
    {'P',   4,  48,  29, "SC-8850 ORCHESTRA Set", "Open Hi-Hat 2"},
    {'P',   4,  48,  30, "SC-8850 ORCHESTRA Set", "Ride Cymbal 1"},
    {'P',   4,  48,  31, "SC-8850 ORCHESTRA Set", "Sticks"},
    {'P',   4,  48,  32, "SC-8850 ORCHESTRA Set", "Square Click"},
    {'P',   4,  48,  33, "SC-8850 ORCHESTRA Set", "Metronome Click"},
    {'P',   4,  48,  34, "SC-8850 ORCHESTRA Set", "Metronome Bell"},
    {'P',   4,  48,  35, "SC-8850 ORCHESTRA Set", "Jazz Kick 1"},
    {'P',   4,  48,  36, "SC-8850 ORCHESTRA Set", "Concert BD 1"},
    {'P',   4,  48,  37, "SC-8850 ORCHESTRA Set", "Side Stick"},
    {'P',   4,  48,  38, "SC-8850 ORCHESTRA Set", "Concert SD"},
    {'P',   4,  48,  39, "SC-8850 ORCHESTRA Set", "Castanets"},
    {'P',   4,  48,  40, "SC-8850 ORCHESTRA Set", "Concert SD"},
    {'P',   4,  48,  41, "SC-8850 ORCHESTRA Set", "Timpani F"},
    {'P',   4,  48,  42, "SC-8850 ORCHESTRA Set", "Timpani F#"},
    {'P',   4,  48,  43, "SC-8850 ORCHESTRA Set", "Timpani G"},
    {'P',   4,  48,  44, "SC-8850 ORCHESTRA Set", "Timpani G#"},
    {'P',   4,  48,  45, "SC-8850 ORCHESTRA Set", "Timpani A"},
    {'P',   4,  48,  46, "SC-8850 ORCHESTRA Set", "Timpani A#"},
    {'P',   4,  48,  47, "SC-8850 ORCHESTRA Set", "Timpani B"},
    {'P',   4,  48,  48, "SC-8850 ORCHESTRA Set", "Timpani c"},
    {'P',   4,  48,  49, "SC-8850 ORCHESTRA Set", "Timpani c#"},
    {'P',   4,  48,  50, "SC-8850 ORCHESTRA Set", "Timpani d"},
    {'P',   4,  48,  51, "SC-8850 ORCHESTRA Set", "Timpani d#"},
    {'P',   4,  48,  52, "SC-8850 ORCHESTRA Set", "Timpani e"},
    {'P',   4,  48,  53, "SC-8850 ORCHESTRA Set", "Timpani f"},
    {'P',   4,  48,  54, "SC-8850 ORCHESTRA Set", "Tambourine"},
    {'P',   4,  48,  55, "SC-8850 ORCHESTRA Set", "Splash Cymbal"},
    {'P',   4,  48,  56, "SC-8850 ORCHESTRA Set", "Cowbell"},
    {'P',   4,  48,  57, "SC-8850 ORCHESTRA Set", "Concert Cymbal 2"},
    {'P',   4,  48,  58, "SC-8850 ORCHESTRA Set", "Vibra-slap"},
    {'P',   4,  48,  59, "SC-8850 ORCHESTRA Set", "Concert Cymbal 1"},
    {'P',   4,  48,  60, "SC-8850 ORCHESTRA Set", "High Bongo"},
    {'P',   4,  48,  61, "SC-8850 ORCHESTRA Set", "Low Bongo"},
    {'P',   4,  48,  62, "SC-8850 ORCHESTRA Set", "Mute High Conga"},
    {'P',   4,  48,  63, "SC-8850 ORCHESTRA Set", "Open High Conga"},
    {'P',   4,  48,  64, "SC-8850 ORCHESTRA Set", "Low Conga"},
    {'P',   4,  48,  65, "SC-8850 ORCHESTRA Set", "High Timbale"},
    {'P',   4,  48,  66, "SC-8850 ORCHESTRA Set", "Low Timbale"},
    {'P',   4,  48,  67, "SC-8850 ORCHESTRA Set", "High Agogo"},
    {'P',   4,  48,  68, "SC-8850 ORCHESTRA Set", "Low Agogo"},
    {'P',   4,  48,  69, "SC-8850 ORCHESTRA Set", "Cabasa"},
    {'P',   4,  48,  70, "SC-8850 ORCHESTRA Set", "Maracas"},
    {'P',   4,  48,  71, "SC-8850 ORCHESTRA Set", "Short High Whistle"},
    {'P',   4,  48,  72, "SC-8850 ORCHESTRA Set", "Long Low Whistle"},
    {'P',   4,  48,  73, "SC-8850 ORCHESTRA Set", "Short Guiro"},
    {'P',   4,  48,  74, "SC-8850 ORCHESTRA Set", "Long Guiro"},
    {'P',   4,  48,  75, "SC-8850 ORCHESTRA Set", "Claves"},
    {'P',   4,  48,  76, "SC-8850 ORCHESTRA Set", "High Wood Block"},
    {'P',   4,  48,  77, "SC-8850 ORCHESTRA Set", "Low Wood Block"},
    {'P',   4,  48,  78, "SC-8850 ORCHESTRA Set", "Mute Cuica"},
    {'P',   4,  48,  79, "SC-8850 ORCHESTRA Set", "Open Cuica"},
    {'P',   4,  48,  80, "SC-8850 ORCHESTRA Set", "Mute Triangle"},
    {'P',   4,  48,  81, "SC-8850 ORCHESTRA Set", "Open Triangle"},
    {'P',   4,  48,  82, "SC-8850 ORCHESTRA Set", "Shaker"},
    {'P',   4,  48,  83, "SC-8850 ORCHESTRA Set", "Jingle Bell"},
    {'P',   4,  48,  84, "SC-8850 ORCHESTRA Set", "Bell Tree"},
    {'P',   4,  48,  85, "SC-8850 ORCHESTRA Set", "Castanets"},
    {'P',   4,  48,  86, "SC-8850 ORCHESTRA Set", "Mute Surdo"},
    {'P',   4,  48,  87, "SC-8850 ORCHESTRA Set", "Open Surdo"},
    {'P',   4,  48,  88, "SC-8850 ORCHESTRA Set", "Applause"},
    {'P',   4,  48,  97, "SC-8850 ORCHESTRA Set", "Applause 2"},
    {'P',   4,  48,  98, "SC-8850 ORCHESTRA Set", "Small Club 1"},
    {'P',   4,  48,  99, "SC-8850 ORCHESTRA Set", "[55] Timpani D#"},
    {'P',   4,  48, 100, "SC-8850 ORCHESTRA Set", "[55] Timpani E"},
    {'P',   4,  48, 101, "SC-8850 ORCHESTRA Set", "[55] Timpani F"},
    {'P',   4,  48, 102, "SC-8850 ORCHESTRA Set", "[55] Timpani F#"},
    {'P',   4,  48, 103, "SC-8850 ORCHESTRA Set", "[55] Timpani G"},
    {'P',   4,  48, 104, "SC-8850 ORCHESTRA Set", "[55] Timpani G#"},
    {'P',   4,  48, 105, "SC-8850 ORCHESTRA Set", "[55] Timpani A"},
    {'P',   4,  48, 106, "SC-8850 ORCHESTRA Set", "[55] Timpani A#"},
    {'P',   4,  48, 107, "SC-8850 ORCHESTRA Set", "[55] Timpani B"},
    {'P',   4,  48, 108, "SC-8850 ORCHESTRA Set", "[55] Timpani c"},
    {'P',   4,  48, 109, "SC-8850 ORCHESTRA Set", "[55] Timpani c#"},
    {'P',   4,  48, 110, "SC-8850 ORCHESTRA Set", "[55] Timpani d"},
    {'P',   4,  48, 111, "SC-8850 ORCHESTRA Set", "[55] Timpani d#"},
    {'P',   4,  48, 112, "SC-8850 ORCHESTRA Set", "[55] Timpani e"},
    {'P',   4,  48, 113, "SC-8850 ORCHESTRA Set", "[55] Timpani f"},
    {'P',   4,  49,  25, "SC-8850 ETHNIC Set", "Finger Snap"},
    {'P',   4,  49,  26, "SC-8850 ETHNIC Set", "Tambourine"},
    {'P',   4,  49,  27, "SC-8850 ETHNIC Set", "Castanets"},
    {'P',   4,  49,  28, "SC-8850 ETHNIC Set", "Crash Cymbal 1"},
    {'P',   4,  49,  29, "SC-8850 ETHNIC Set", "Snare Roll"},
    {'P',   4,  49,  30, "SC-8850 ETHNIC Set", "Concert SD"},
    {'P',   4,  49,  31, "SC-8850 ETHNIC Set", "Concert Cymbal"},
    {'P',   4,  49,  32, "SC-8850 ETHNIC Set", "Concert BD 1"},
    {'P',   4,  49,  33, "SC-8850 ETHNIC Set", "Jingle Bell"},
    {'P',   4,  49,  34, "SC-8850 ETHNIC Set", "Bell Tree"},
    {'P',   4,  49,  35, "SC-8850 ETHNIC Set", "Bar Chimes"},
    {'P',   4,  49,  36, "SC-8850 ETHNIC Set", "Wadaiko"},
    {'P',   4,  49,  37, "SC-8850 ETHNIC Set", "Wadaiko Rim"},
    {'P',   4,  49,  38, "SC-8850 ETHNIC Set", "Shime Taiko"},
    {'P',   4,  49,  39, "SC-8850 ETHNIC Set", "Atarigane"},
    {'P',   4,  49,  40, "SC-8850 ETHNIC Set", "Hyoushigi"},
    {'P',   4,  49,  41, "SC-8850 ETHNIC Set", "Ohkawa"},
    {'P',   4,  49,  42, "SC-8850 ETHNIC Set", "High Kotsuzumi"},
    {'P',   4,  49,  43, "SC-8850 ETHNIC Set", "Low Kotsuzumi"},
    {'P',   4,  49,  44, "SC-8850 ETHNIC Set", "Ban Gu"},
    {'P',   4,  49,  45, "SC-8850 ETHNIC Set", "Big Gong"},
    {'P',   4,  49,  46, "SC-8850 ETHNIC Set", "Small Gong"},
    {'P',   4,  49,  47, "SC-8850 ETHNIC Set", "Bend Gong"},
    {'P',   4,  49,  48, "SC-8850 ETHNIC Set", "Thai Gong"},
    {'P',   4,  49,  49, "SC-8850 ETHNIC Set", "Rama Cymbal"},
    {'P',   4,  49,  50, "SC-8850 ETHNIC Set", "Gamelan Gong"},
    {'P',   4,  49,  51, "SC-8850 ETHNIC Set", "Udo Short"},
    {'P',   4,  49,  52, "SC-8850 ETHNIC Set", "Udo Long"},
    {'P',   4,  49,  53, "SC-8850 ETHNIC Set", "Udo Slap"},
    {'P',   4,  49,  54, "SC-8850 ETHNIC Set", "Bendir"},
    {'P',   4,  49,  55, "SC-8850 ETHNIC Set", "Req Dum"},
    {'P',   4,  49,  56, "SC-8850 ETHNIC Set", "Req Tik"},
    {'P',   4,  49,  57, "SC-8850 ETHNIC Set", "Tabla Te"},
    {'P',   4,  49,  58, "SC-8850 ETHNIC Set", "Tabla Na"},
    {'P',   4,  49,  59, "SC-8850 ETHNIC Set", "Tabla Tun"},
    {'P',   4,  49,  60, "SC-8850 ETHNIC Set", "Tabla Ge"},
    {'P',   4,  49,  61, "SC-8850 ETHNIC Set", "Tabla Ge Hi"},
    {'P',   4,  49,  62, "SC-8850 ETHNIC Set", "Talking Drum"},
    {'P',   4,  49,  63, "SC-8850 ETHNIC Set", "Bend Talking Drum"},
    {'P',   4,  49,  64, "SC-8850 ETHNIC Set", "Caxixi"},
    {'P',   4,  49,  65, "SC-8850 ETHNIC Set", "Djembe"},
    {'P',   4,  49,  66, "SC-8850 ETHNIC Set", "Djembe Rim"},
    {'P',   4,  49,  67, "SC-8850 ETHNIC Set", "Timbales Low"},
    {'P',   4,  49,  68, "SC-8850 ETHNIC Set", "Timbales Paila"},
    {'P',   4,  49,  69, "SC-8850 ETHNIC Set", "Timbales High"},
    {'P',   4,  49,  70, "SC-8850 ETHNIC Set", "Cowbell"},
    {'P',   4,  49,  71, "SC-8850 ETHNIC Set", "High Bongo"},
    {'P',   4,  49,  72, "SC-8850 ETHNIC Set", "Low Bongo"},
    {'P',   4,  49,  73, "SC-8850 ETHNIC Set", "Mute High Conga"},
    {'P',   4,  49,  74, "SC-8850 ETHNIC Set", "Open High Conga"},
    {'P',   4,  49,  75, "SC-8850 ETHNIC Set", "Mute Low Conga"},
    {'P',   4,  49,  76, "SC-8850 ETHNIC Set", "Conga Slap"},
    {'P',   4,  49,  77, "SC-8850 ETHNIC Set", "Open Low Conga"},
    {'P',   4,  49,  78, "SC-8850 ETHNIC Set", "Conga Slide"},
    {'P',   4,  49,  79, "SC-8850 ETHNIC Set", "Mute Pandiero"},
    {'P',   4,  49,  80, "SC-8850 ETHNIC Set", "Open Pandiero"},
    {'P',   4,  49,  81, "SC-8850 ETHNIC Set", "Open Surdo"},
    {'P',   4,  49,  82, "SC-8850 ETHNIC Set", "Mute Surdo"},
    {'P',   4,  49,  83, "SC-8850 ETHNIC Set", "Tamborim"},
    {'P',   4,  49,  84, "SC-8850 ETHNIC Set", "High Agogo"},
    {'P',   4,  49,  85, "SC-8850 ETHNIC Set", "Low Agogo"},
    {'P',   4,  49,  86, "SC-8850 ETHNIC Set", "Shaker"},
    {'P',   4,  49,  87, "SC-8850 ETHNIC Set", "High Whistle"},
    {'P',   4,  49,  88, "SC-8850 ETHNIC Set", "Low Whistle"},
    {'P',   4,  49,  89, "SC-8850 ETHNIC Set", "Mute Cuica"},
    {'P',   4,  49,  90, "SC-8850 ETHNIC Set", "Open Cuica"},
    {'P',   4,  49,  91, "SC-8850 ETHNIC Set", "Mute Triangle"},
    {'P',   4,  49,  92, "SC-8850 ETHNIC Set", "Open Triangle"},
    {'P',   4,  49,  93, "SC-8850 ETHNIC Set", "Short Guiro"},
    {'P',   4,  49,  94, "SC-8850 ETHNIC Set", "Long Guiro"},
    {'P',   4,  49,  95, "SC-8850 ETHNIC Set", "Cabasa Up"},
    {'P',   4,  49,  96, "SC-8850 ETHNIC Set", "Cabasa Down"},
    {'P',   4,  49,  97, "SC-8850 ETHNIC Set", "Claves"},
    {'P',   4,  49,  98, "SC-8850 ETHNIC Set", "High Wood Block"},
    {'P',   4,  49,  99, "SC-8850 ETHNIC Set", "Low Wood Block"},
    {'P',   4,  50,  25, "SC-8850 KICK & SNARE Set", "CR-78 Kick 1"},
    {'P',   4,  50,  26, "SC-8850 KICK & SNARE Set", "CR-78 Kick 2"},
    {'P',   4,  50,  27, "SC-8850 KICK & SNARE Set", "TR-606 Kick"},
    {'P',   4,  50,  28, "SC-8850 KICK & SNARE Set", "TR-707 Kick"},
    {'P',   4,  50,  29, "SC-8850 KICK & SNARE Set", "TR-808 Kick 1"},
    {'P',   4,  50,  30, "SC-8850 KICK & SNARE Set", "TR-909 Kick 1"},
    {'P',   4,  50,  31, "SC-8850 KICK & SNARE Set", "TR-909 Kick 2"},
    {'P',   4,  50,  32, "SC-8850 KICK & SNARE Set", "Hip-Hop Kick 2"},
    {'P',   4,  50,  33, "SC-8850 KICK & SNARE Set", "Hip-Hop Kick 1"},
    {'P',   4,  50,  34, "SC-8850 KICK & SNARE Set", "Jungle Kick 2"},
    {'P',   4,  50,  35, "SC-8850 KICK & SNARE Set", "Jungle Kick 1"},
    {'P',   4,  50,  36, "SC-8850 KICK & SNARE Set", "Techno Kick 2"},
    {'P',   4,  50,  37, "SC-8850 KICK & SNARE Set", "Techno Kick 1"},
    {'P',   4,  50,  38, "SC-8850 KICK & SNARE Set", "Standard 1 Kick 2"},
    {'P',   4,  50,  39, "SC-8850 KICK & SNARE Set", "Standard 1 Kick 1"},
    {'P',   4,  50,  40, "SC-8850 KICK & SNARE Set", "[88] Standard 1 Kick 1"},
    {'P',   4,  50,  41, "SC-8850 KICK & SNARE Set", "[88] Standard 1 Kick 2"},
    {'P',   4,  50,  42, "SC-8850 KICK & SNARE Set", "[88] Standard 2 Kick 1"},
    {'P',   4,  50,  43, "SC-8850 KICK & SNARE Set", "[88] Standard 2 Kick 2"},
    {'P',   4,  50,  44, "SC-8850 KICK & SNARE Set", "[55] Kick Drum1"},
    {'P',   4,  50,  45, "SC-8850 KICK & SNARE Set", "[55] Kick Drum 2"},
    {'P',   4,  50,  46, "SC-8850 KICK & SNARE Set", "[88] Soft Kick"},
    {'P',   4,  50,  47, "SC-8850 KICK & SNARE Set", "[88] Jazz Kick 1"},
    {'P',   4,  50,  48, "SC-8850 KICK & SNARE Set", "[88] Jazz Kick 2"},
    {'P',   4,  50,  49, "SC-8850 KICK & SNARE Set", "[55] Concert BD 1"},
    {'P',   4,  50,  50, "SC-8850 KICK & SNARE Set", "[88] Room Kick 1"},
    {'P',   4,  50,  51, "SC-8850 KICK & SNARE Set", "[88] Room Kick 2"},
    {'P',   4,  50,  52, "SC-8850 KICK & SNARE Set", "[88] Power Kick1"},
    {'P',   4,  50,  53, "SC-8850 KICK & SNARE Set", "[88] Power Kick2"},
    {'P',   4,  50,  54, "SC-8850 KICK & SNARE Set", "[88] Electric Kick 2"},
    {'P',   4,  50,  55, "SC-8850 KICK & SNARE Set", "[88] Electric Kick 1"},
    {'P',   4,  50,  56, "SC-8850 KICK & SNARE Set", "[55] Electric Kick"},
    {'P',   4,  50,  57, "SC-8850 KICK & SNARE Set", "[88] TR-808 Kick"},
    {'P',   4,  50,  58, "SC-8850 KICK & SNARE Set", "[88] TR-909 Kick"},
    {'P',   4,  50,  59, "SC-8850 KICK & SNARE Set", "[88] Dance Kick"},
    {'P',   4,  50,  60, "SC-8850 KICK & SNARE Set", "[88] Standard 1 Snare 1"},
    {'P',   4,  50,  61, "SC-8850 KICK & SNARE Set", "[88] Standard 1 Snare 2"},
    {'P',   4,  50,  62, "SC-8850 KICK & SNARE Set", "[88] Standard 2 Snare 1"},
    {'P',   4,  50,  63, "SC-8850 KICK & SNARE Set", "[88] Standard 2 Snare 2"},
    {'P',   4,  50,  64, "SC-8850 KICK & SNARE Set", "[55] Tight Snare"},
    {'P',   4,  50,  65, "SC-8850 KICK & SNARE Set", "[55] Concert Snare"},
    {'P',   4,  50,  66, "SC-8850 KICK & SNARE Set", "[88] Jazz Snare 1"},
    {'P',   4,  50,  67, "SC-8850 KICK & SNARE Set", "[88] Jazz Snare 2"},
    {'P',   4,  50,  68, "SC-8850 KICK & SNARE Set", "[88] Room Snare 1"},
    {'P',   4,  50,  69, "SC-8850 KICK & SNARE Set", "[88] Room Snare 2"},
    {'P',   4,  50,  70, "SC-8850 KICK & SNARE Set", "[88] Power Snare 1"},
    {'P',   4,  50,  71, "SC-8850 KICK & SNARE Set", "[88] Power Snare 2"},
    {'P',   4,  50,  72, "SC-8850 KICK & SNARE Set", "[55] Gated Snare"},
    {'P',   4,  50,  73, "SC-8850 KICK & SNARE Set", "[88] Dance Snare 1"},
    {'P',   4,  50,  74, "SC-8850 KICK & SNARE Set", "[88] Dance Snare 2"},
    {'P',   4,  50,  75, "SC-8850 KICK & SNARE Set", "[88] Disco Snare"},
    {'P',   4,  50,  76, "SC-8850 KICK & SNARE Set", "[88] Electric Snare 2"},
    {'P',   4,  50,  77, "SC-8850 KICK & SNARE Set", "[88] House Snare"},
    {'P',   4,  50,  78, "SC-8850 KICK & SNARE Set", "[55] Electric Snare 1"},
    {'P',   4,  50,  79, "SC-8850 KICK & SNARE Set", "[88] Electric Snare 3"},
    {'P',   4,  50,  80, "SC-8850 KICK & SNARE Set", "[88] TR-808 Snare 1"},
    {'P',   4,  50,  81, "SC-8850 KICK & SNARE Set", "[88] TR-808 Snare 2"},
    {'P',   4,  50,  82, "SC-8850 KICK & SNARE Set", "[88] TR-909 Snare 1"},
    {'P',   4,  50,  83, "SC-8850 KICK & SNARE Set", "[88] TR-909 Snare 2"},
    {'P',   4,  50,  84, "SC-8850 KICK & SNARE Set", "[88] Brush Tap 1"},
    {'P',   4,  50,  85, "SC-8850 KICK & SNARE Set", "[88] Brush Tap 2"},
    {'P',   4,  50,  86, "SC-8850 KICK & SNARE Set", "[88] Brush Slap 1"},
    {'P',   4,  50,  87, "SC-8850 KICK & SNARE Set", "[88] Brush Slap 2"},
    {'P',   4,  50,  88, "SC-8850 KICK & SNARE Set", "[88] Brush Slap 3"},
    {'P',   4,  50,  89, "SC-8850 KICK & SNARE Set", "[88] Brush Swirl 1"},
    {'P',   4,  50,  90, "SC-8850 KICK & SNARE Set", "[88] Brush Swirl 2"},
    {'P',   4,  50,  91, "SC-8850 KICK & SNARE Set", "[88] Brush Long Swirl"},
    {'P',   4,  50,  92, "SC-8850 KICK & SNARE Set", "Standard 1 Snare 1"},
    {'P',   4,  50,  93, "SC-8850 KICK & SNARE Set", "Standard 1 Snare 2"},
    {'P',   4,  50,  94, "SC-8850 KICK & SNARE Set", "Standard 1 Snare 3"},
    {'P',   4,  50,  95, "SC-8850 KICK & SNARE Set", "Rap Snare"},
    {'P',   4,  50,  96, "SC-8850 KICK & SNARE Set", "Hip-Hop Snare 2"},
    {'P',   4,  50,  97, "SC-8850 KICK & SNARE Set", "Jungle Snare 1"},
    {'P',   4,  50,  98, "SC-8850 KICK & SNARE Set", "Jungle Snare 2"},
    {'P',   4,  50,  99, "SC-8850 KICK & SNARE Set", "Techno Snare 1"},
    {'P',   4,  50, 100, "SC-8850 KICK & SNARE Set", "Techno Snare 2"},
    {'P',   4,  50, 101, "SC-8850 KICK & SNARE Set", "House Snare 2"},
    {'P',   4,  50, 102, "SC-8850 KICK & SNARE Set", "CR-78 Snare 1"},
    {'P',   4,  50, 103, "SC-8850 KICK & SNARE Set", "CR-78 Snare 2"},
    {'P',   4,  50, 104, "SC-8850 KICK & SNARE Set", "TR-606 Snare 1"},
    {'P',   4,  50, 105, "SC-8850 KICK & SNARE Set", "TR-606 Snare 2"},
    {'P',   4,  50, 106, "SC-8850 KICK & SNARE Set", "TR-707 Snare 1"},
    {'P',   4,  50, 107, "SC-8850 KICK & SNARE Set", "TR-707 Snare 2"},
    {'P',   4,  50, 108, "SC-8850 KICK & SNARE Set", "Standard 3 Snare 2"},
    {'P',   4,  50, 109, "SC-8850 KICK & SNARE Set", "TR-808 Snare 2"},
    {'P',   4,  50, 110, "SC-8850 KICK & SNARE Set", "TR-909 Snare 1"},
    {'P',   4,  50, 111, "SC-8850 KICK & SNARE Set", "TR-909 Snare 2"},
    {'P',   4,  51,  25, "SC-8850 KICK & SNARE 2 Set", "[Pro] CR-78 Kick 1"},
    {'P',   4,  51,  26, "SC-8850 KICK & SNARE 2 Set", "[Pro] CR-78 Kick 2"},
    {'P',   4,  51,  27, "SC-8850 KICK & SNARE 2 Set", "[Pro] TR-606 Kick"},
    {'P',   4,  51,  28, "SC-8850 KICK & SNARE 2 Set", "[Pro] TR-707 Kick"},
    {'P',   4,  51,  29, "SC-8850 KICK & SNARE 2 Set", "[Pro] TR-808 Kick 1"},
    {'P',   4,  51,  30, "SC-8850 KICK & SNARE 2 Set", "[Pro] TR-909 Kick 1"},
    {'P',   4,  51,  31, "SC-8850 KICK & SNARE 2 Set", "[Pro] TR-909 Kick 2"},
    {'P',   4,  51,  32, "SC-8850 KICK & SNARE 2 Set", "[Pro] Hip-Hop Kick 2"},
    {'P',   4,  51,  33, "SC-8850 KICK & SNARE 2 Set", "[Pro] Hip-Hop Kick 1"},
    {'P',   4,  51,  34, "SC-8850 KICK & SNARE 2 Set", "[Pro] Jungle Kick 2"},
    {'P',   4,  51,  35, "SC-8850 KICK & SNARE 2 Set", "[Pro] Jungle Kick 1"},
    {'P',   4,  51,  36, "SC-8850 KICK & SNARE 2 Set", "[Pro] Techno Kick 2"},
    {'P',   4,  51,  37, "SC-8850 KICK & SNARE 2 Set", "[Pro] Techno Kick 1"},
    {'P',   4,  51,  38, "SC-8850 KICK & SNARE 2 Set", "[Pro] Standard 1 Kick 2"},
    {'P',   4,  51,  39, "SC-8850 KICK & SNARE 2 Set", "[Pro] Standard 1 Kick 1"},
    {'P',   4,  51,  40, "SC-8850 KICK & SNARE 2 Set", "Standard 1 Kick 2"},
    {'P',   4,  51,  41, "SC-8850 KICK & SNARE 2 Set", "Standard 1 Kick 1"},
    {'P',   4,  51,  42, "SC-8850 KICK & SNARE 2 Set", "Brush Kick 2"},
    {'P',   4,  51,  43, "SC-8850 KICK & SNARE 2 Set", "Brush Kick 1"},
    {'P',   4,  51,  44, "SC-8850 KICK & SNARE 2 Set", "Jazz Kick 2"},
    {'P',   4,  51,  45, "SC-8850 KICK & SNARE 2 Set", "Jazz Kick 1"},
    {'P',   4,  51,  46, "SC-8850 KICK & SNARE 2 Set", "Hip-Hop Kick 2"},
    {'P',   4,  51,  47, "SC-8850 KICK & SNARE 2 Set", "Hip-Hop Kick 1"},
    {'P',   4,  51,  48, "SC-8850 KICK & SNARE 2 Set", "Concert BD 1 Mute"},
    {'P',   4,  51,  49, "SC-8850 KICK & SNARE 2 Set", "[55] Concert BD 1"},
    {'P',   4,  51,  50, "SC-8850 KICK & SNARE 2 Set", "Room Kick 2"},
    {'P',   4,  51,  51, "SC-8850 KICK & SNARE 2 Set", "Room Kick 1"},
    {'P',   4,  51,  52, "SC-8850 KICK & SNARE 2 Set", "Jungle Kick 2"},
    {'P',   4,  51,  53, "SC-8850 KICK & SNARE 2 Set", "Jungle Kick 1"},
    {'P',   4,  51,  54, "SC-8850 KICK & SNARE 2 Set", "Jungle Kick Roll"},
    {'P',   4,  51,  55, "SC-8850 KICK & SNARE 2 Set", "Fat Kick"},
    {'P',   4,  51,  56, "SC-8850 KICK & SNARE 2 Set", "Dance Kick"},
    {'P',   4,  51,  57, "SC-8850 KICK & SNARE 2 Set", "TR-808 Kick"},
    {'P',   4,  51,  58, "SC-8850 KICK & SNARE 2 Set", "TR-909 Kick 2"},
    {'P',   4,  51,  59, "SC-8850 KICK & SNARE 2 Set", "TR-909 Kick 1"},
    {'P',   4,  51,  60, "SC-8850 KICK & SNARE 2 Set", "Standard 1 Snare 1"},
    {'P',   4,  51,  61, "SC-8850 KICK & SNARE 2 Set", "Standard 1 Snare 2"},
    {'P',   4,  51,  62, "SC-8850 KICK & SNARE 2 Set", "[88] Standard 2 Snare 1"},
    {'P',   4,  51,  63, "SC-8850 KICK & SNARE 2 Set", "[88] Standard 2 Snare 2"},
    {'P',   4,  51,  64, "SC-8850 KICK & SNARE 2 Set", "[55] Tight Snare"},
    {'P',   4,  51,  65, "SC-8850 KICK & SNARE 2 Set", "[55] Concert Snare"},
    {'P',   4,  51,  66, "SC-8850 KICK & SNARE 2 Set", "Jazz Snare 1"},
    {'P',   4,  51,  67, "SC-8850 KICK & SNARE 2 Set", "Jazz Snare 2"},
    {'P',   4,  51,  68, "SC-8850 KICK & SNARE 2 Set", "Room Snare 1"},
    {'P',   4,  51,  69, "SC-8850 KICK & SNARE 2 Set", "Room Snare 2"},
    {'P',   4,  51,  70, "SC-8850 KICK & SNARE 2 Set", "LoFi Snare 1"},
    {'P',   4,  51,  71, "SC-8850 KICK & SNARE 2 Set", "LoFi Snare 2"},
    {'P',   4,  51,  72, "SC-8850 KICK & SNARE 2 Set", "[55] Gated Snare"},
    {'P',   4,  51,  73, "SC-8850 KICK & SNARE 2 Set", "LoFi Snare Rim"},
    {'P',   4,  51,  74, "SC-8850 KICK & SNARE 2 Set", "[88] Dance Snare 2"},
    {'P',   4,  51,  75, "SC-8850 KICK & SNARE 2 Set", "HipHop Snare 1"},
    {'P',   4,  51,  76, "SC-8850 KICK & SNARE 2 Set", "HipHop Snare 2"},
    {'P',   4,  51,  77, "SC-8850 KICK & SNARE 2 Set", "Dance Snare"},
    {'P',   4,  51,  78, "SC-8850 KICK & SNARE 2 Set", "TR-606 Snare 2"},
    {'P',   4,  51,  79, "SC-8850 KICK & SNARE 2 Set", "Techno Snare"},
    {'P',   4,  51,  80, "SC-8850 KICK & SNARE 2 Set", "House Snare"},
    {'P',   4,  51,  81, "SC-8850 KICK & SNARE 2 Set", "Rock Snare Dry"},
    {'P',   4,  51,  82, "SC-8850 KICK & SNARE 2 Set", "Jungle Snare"},
    {'P',   4,  51,  83, "SC-8850 KICK & SNARE 2 Set", "Jungle Snare Roll"},
    {'P',   4,  51,  84, "SC-8850 KICK & SNARE 2 Set", "[88] Brush Tap 1"},
    {'P',   4,  51,  85, "SC-8850 KICK & SNARE 2 Set", "[88] Brush Tap 2"},
    {'P',   4,  51,  86, "SC-8850 KICK & SNARE 2 Set", "Brush Tap 2"},
    {'P',   4,  51,  87, "SC-8850 KICK & SNARE 2 Set", "Brush Slap 2"},
    {'P',   4,  51,  88, "SC-8850 KICK & SNARE 2 Set", "[88] Brush Slap 3"},
    {'P',   4,  51,  89, "SC-8850 KICK & SNARE 2 Set", "[88] Brush Swirl 1"},
    {'P',   4,  51,  90, "SC-8850 KICK & SNARE 2 Set", "[88] Brush Swirl 2"},
    {'P',   4,  51,  91, "SC-8850 KICK & SNARE 2 Set", "[88] Brush Long Swirl"},
    {'P',   4,  51,  92, "SC-8850 KICK & SNARE 2 Set", "[Pro] Standard 1 Snare 1"},
    {'P',   4,  51,  93, "SC-8850 KICK & SNARE 2 Set", "[Pro] Standard 1 Snare 2"},
    {'P',   4,  51,  94, "SC-8850 KICK & SNARE 2 Set", "[Pro] Standard 1 Snare 3"},
    {'P',   4,  51,  95, "SC-8850 KICK & SNARE 2 Set", "[Pro] Rap Snare"},
    {'P',   4,  51,  96, "SC-8850 KICK & SNARE 2 Set", "[Pro] Hip-Hop Snare 2"},
    {'P',   4,  51,  97, "SC-8850 KICK & SNARE 2 Set", "[Pro] Jungle Snare 1"},
    {'P',   4,  51,  98, "SC-8850 KICK & SNARE 2 Set", "[Pro] Jungle Snare 2"},
    {'P',   4,  51,  99, "SC-8850 KICK & SNARE 2 Set", "[Pro] Techno Snare 1"},
    {'P',   4,  51, 100, "SC-8850 KICK & SNARE 2 Set", "[Pro] Techno Snare 2"},
    {'P',   4,  51, 101, "SC-8850 KICK & SNARE 2 Set", "[Pro] House Snare 2"},
    {'P',   4,  51, 102, "SC-8850 KICK & SNARE 2 Set", "[Pro] CR-78 Snare 1"},
    {'P',   4,  51, 103, "SC-8850 KICK & SNARE 2 Set", "[Pro] CR-78 Snare 2"},
    {'P',   4,  51, 104, "SC-8850 KICK & SNARE 2 Set", "[Pro] TR-606 Snare 1"},
    {'P',   4,  51, 105, "SC-8850 KICK & SNARE 2 Set", "[Pro] TR-606 Snare 2"},
    {'P',   4,  51, 106, "SC-8850 KICK & SNARE 2 Set", "[Pro] TR-707 Snare 1"},
    {'P',   4,  51, 107, "SC-8850 KICK & SNARE 2 Set", "[Pro] TR-707 Snare 2"},
    {'P',   4,  51, 108, "SC-8850 KICK & SNARE 2 Set", "[Pro] Standard 3 Snare 2"},
    {'P',   4,  51, 109, "SC-8850 KICK & SNARE 2 Set", "[Pro] TR-808 Snare 2"},
    {'P',   4,  51, 110, "SC-8850 KICK & SNARE 2 Set", "[Pro] TR-909 Snare 1"},
    {'P',   4,  51, 111, "SC-8850 KICK & SNARE 2 Set", "[Pro] TR-909 Snare 2"},
    {'P',   4,  52,  25, "SC-8850 ASIA Set", "Gamelan Gong 1"},
    {'P',   4,  52,  26, "SC-8850 ASIA Set", "Gamelan Gong 2"},
    {'P',   4,  52,  27, "SC-8850 ASIA Set", "Gamelan Gong 3"},
    {'P',   4,  52,  28, "SC-8850 ASIA Set", "Gamelan Gong 4"},
    {'P',   4,  52,  29, "SC-8850 ASIA Set", "Gamelan Gong 5"},
    {'P',   4,  52,  30, "SC-8850 ASIA Set", "Gamelan Gong 6"},
    {'P',   4,  52,  31, "SC-8850 ASIA Set", "Gamelan Gong 7"},
    {'P',   4,  52,  32, "SC-8850 ASIA Set", "Gamelan Gong 8"},
    {'P',   4,  52,  33, "SC-8850 ASIA Set", "Gamelan Gong 9"},
    {'P',   4,  52,  34, "SC-8850 ASIA Set", "Gamelan Gong 10"},
    {'P',   4,  52,  35, "SC-8850 ASIA Set", "Gender 1"},
    {'P',   4,  52,  36, "SC-8850 ASIA Set", "Gender 2"},
    {'P',   4,  52,  37, "SC-8850 ASIA Set", "Gender 3"},
    {'P',   4,  52,  38, "SC-8850 ASIA Set", "Gender 4"},
    {'P',   4,  52,  39, "SC-8850 ASIA Set", "Gender 5"},
    {'P',   4,  52,  40, "SC-8850 ASIA Set", "Bonang 1"},
    {'P',   4,  52,  41, "SC-8850 ASIA Set", "Bonang 2"},
    {'P',   4,  52,  42, "SC-8850 ASIA Set", "Bonang 3"},
    {'P',   4,  52,  43, "SC-8850 ASIA Set", "Bonang 4"},
    {'P',   4,  52,  44, "SC-8850 ASIA Set", "Bonang 5"},
    {'P',   4,  52,  45, "SC-8850 ASIA Set", "Rama Cymbal Low"},
    {'P',   4,  52,  46, "SC-8850 ASIA Set", "Rama Cymbal High"},
    {'P',   4,  52,  47, "SC-8850 ASIA Set", "Sagat Open"},
    {'P',   4,  52,  48, "SC-8850 ASIA Set", "Sagat Closed"},
    {'P',   4,  52,  49, "SC-8850 ASIA Set", "Jaws Harp"},
    {'P',   4,  52,  50, "SC-8850 ASIA Set", "Wadaiko"},
    {'P',   4,  52,  51, "SC-8850 ASIA Set", "Wadaiko Rim"},
    {'P',   4,  52,  52, "SC-8850 ASIA Set", "Small Taiko"},
    {'P',   4,  52,  53, "SC-8850 ASIA Set", "Shimetaiko"},
    {'P',   4,  52,  54, "SC-8850 ASIA Set", "Atarigane"},
    {'P',   4,  52,  55, "SC-8850 ASIA Set", "Hyoushigi"},
    {'P',   4,  52,  56, "SC-8850 ASIA Set", "Ohkawa"},
    {'P',   4,  52,  57, "SC-8850 ASIA Set", "High Kotsuzumi"},
    {'P',   4,  52,  58, "SC-8850 ASIA Set", "Low Kotsuzumi"},
    {'P',   4,  52,  59, "SC-8850 ASIA Set", "Yyoo Dude"},
    {'P',   4,  52,  60, "SC-8850 ASIA Set", "Buk"},
    {'P',   4,  52,  61, "SC-8850 ASIA Set", "Buk Rim"},
    {'P',   4,  52,  62, "SC-8850 ASIA Set", "Gengari p"},
    {'P',   4,  52,  63, "SC-8850 ASIA Set", "Gengari Mute Low"},
    {'P',   4,  52,  64, "SC-8850 ASIA Set", "Gengari f"},
    {'P',   4,  52,  65, "SC-8850 ASIA Set", "Gengari Mute High"},
    {'P',   4,  52,  66, "SC-8850 ASIA Set", "Gengari Samll"},
    {'P',   4,  52,  67, "SC-8850 ASIA Set", "Jang-Gu Che"},
    {'P',   4,  52,  68, "SC-8850 ASIA Set", "Jang-Gu Kun"},
    {'P',   4,  52,  69, "SC-8850 ASIA Set", "Jang-Gu Rim"},
    {'P',   4,  52,  70, "SC-8850 ASIA Set", "Jing p"},
    {'P',   4,  52,  71, "SC-8850 ASIA Set", "Jing f"},
    {'P',   4,  52,  72, "SC-8850 ASIA Set", "Jing Mute"},
    {'P',   4,  52,  73, "SC-8850 ASIA Set", "Asian Gong"},
    {'P',   4,  52,  74, "SC-8850 ASIA Set", "Big Gong"},
    {'P',   4,  52,  75, "SC-8850 ASIA Set", "Small Gong"},
    {'P',   4,  52,  76, "SC-8850 ASIA Set", "Pai Ban"},
    {'P',   4,  52,  77, "SC-8850 ASIA Set", "Ban Gu"},
    {'P',   4,  52,  78, "SC-8850 ASIA Set", "Tang Gu"},
    {'P',   4,  52,  79, "SC-8850 ASIA Set", "Tang Gu Mute"},
    {'P',   4,  52,  80, "SC-8850 ASIA Set", "Shou Luo"},
    {'P',   4,  52,  81, "SC-8850 ASIA Set", "Bend Gong"},
    {'P',   4,  52,  82, "SC-8850 ASIA Set", "Hu Yin Luo Low"},
    {'P',   4,  52,  83, "SC-8850 ASIA Set", "Hu Yin Luo Mid"},
    {'P',   4,  52,  84, "SC-8850 ASIA Set", "Hu Yin Luo Mid 2"},
    {'P',   4,  52,  85, "SC-8850 ASIA Set", "Hu Yin Luo High"},
    {'P',   4,  52,  86, "SC-8850 ASIA Set", "Hu Yin Luo High 2"},
    {'P',   4,  52,  87, "SC-8850 ASIA Set", "Nao Bo"},
    {'P',   4,  52,  88, "SC-8850 ASIA Set", "Xiao Bo"},
    {'P',   4,  52,  89, "SC-8850 ASIA Set", "Dholak 1"},
    {'P',   4,  52,  90, "SC-8850 ASIA Set", "Dholak 2"},
    {'P',   4,  53,  32, "SC-8850 CYMBAL&CLAPS Set", "Reverse Open Hi-Hat"},
    {'P',   4,  53,  33, "SC-8850 CYMBAL&CLAPS Set", "Reverse Closed Hi-Hat 1"},
    {'P',   4,  53,  34, "SC-8850 CYMBAL&CLAPS Set", "Reverse Closed Hi-Hat 2"},
    {'P',   4,  53,  35, "SC-8850 CYMBAL&CLAPS Set", "Jungle Hi-Hat"},
    {'P',   4,  53,  36, "SC-8850 CYMBAL&CLAPS Set", "[55] Closed Hi-Hat"},
    {'P',   4,  53,  37, "SC-8850 CYMBAL&CLAPS Set", "[88] Closed Hi-Hat 2"},
    {'P',   4,  53,  38, "SC-8850 CYMBAL&CLAPS Set", "[88] Closed Hi-Hat 3"},
    {'P',   4,  53,  39, "SC-8850 CYMBAL&CLAPS Set", "Closed Hi-Hat 4"},
    {'P',   4,  53,  40, "SC-8850 CYMBAL&CLAPS Set", "Closed Hi-Hat"},
    {'P',   4,  53,  41, "SC-8850 CYMBAL&CLAPS Set", "TR-707 Closed Hi-Hat"},
    {'P',   4,  53,  42, "SC-8850 CYMBAL&CLAPS Set", "TR-606 Closed Hi-Hat"},
    {'P',   4,  53,  43, "SC-8850 CYMBAL&CLAPS Set", "[88] TR-808 Closed Hi-Hat"},
    {'P',   4,  53,  44, "SC-8850 CYMBAL&CLAPS Set", "TR-808 Closed Hi-Hat"},
    {'P',   4,  53,  45, "SC-8850 CYMBAL&CLAPS Set", "CR-78 Closed Hi-Hat"},
    {'P',   4,  53,  46, "SC-8850 CYMBAL&CLAPS Set", "[55] Pedal Hi-Hat"},
    {'P',   4,  53,  47, "SC-8850 CYMBAL&CLAPS Set", "[88] Pedal Hi-Hat"},
    {'P',   4,  53,  48, "SC-8850 CYMBAL&CLAPS Set", "Pedal Hi-Hat"},
    {'P',   4,  53,  49, "SC-8850 CYMBAL&CLAPS Set", "Half-Open Hi-Hat 1"},
    {'P',   4,  53,  50, "SC-8850 CYMBAL&CLAPS Set", "Half-Open Hi-Hat 2"},
    {'P',   4,  53,  51, "SC-8850 CYMBAL&CLAPS Set", "[55] Open Hi-Hat"},
    {'P',   4,  53,  52, "SC-8850 CYMBAL&CLAPS Set", "[88] Open Hi-Hat 2"},
    {'P',   4,  53,  53, "SC-8850 CYMBAL&CLAPS Set", "[88] Open Hi-Hat 3"},
    {'P',   4,  53,  54, "SC-8850 CYMBAL&CLAPS Set", "Open Hi-Hat"},
    {'P',   4,  53,  55, "SC-8850 CYMBAL&CLAPS Set", "TR-909 Open Hi-Hat"},
    {'P',   4,  53,  56, "SC-8850 CYMBAL&CLAPS Set", "TR-707 Open Hi-Hat"},
    {'P',   4,  53,  57, "SC-8850 CYMBAL&CLAPS Set", "TR-606 Open Hi-Hat"},
    {'P',   4,  53,  58, "SC-8850 CYMBAL&CLAPS Set", "[88] TR-808 Open Hi-Hat"},
    {'P',   4,  53,  59, "SC-8850 CYMBAL&CLAPS Set", "TR-808 Open Hi-Hat"},
    {'P',   4,  53,  60, "SC-8850 CYMBAL&CLAPS Set", "CR-78 Open Hi-Hat"},
    {'P',   4,  53,  61, "SC-8850 CYMBAL&CLAPS Set", "Crash Cymbal 1"},
    {'P',   4,  53,  62, "SC-8850 CYMBAL&CLAPS Set", "Crash Cymbal 2"},
    {'P',   4,  53,  63, "SC-8850 CYMBAL&CLAPS Set", "Crash Cymbal 3"},
    {'P',   4,  53,  64, "SC-8850 CYMBAL&CLAPS Set", "Brush Crash Cymbal"},
    {'P',   4,  53,  65, "SC-8850 CYMBAL&CLAPS Set", "Hard Crash Cymbal"},
    {'P',   4,  53,  66, "SC-8850 CYMBAL&CLAPS Set", "TR-909 Crash Cymbal"},
    {'P',   4,  53,  67, "SC-8850 CYMBAL&CLAPS Set", "TR-808 Crash Cymbal"},
    {'P',   4,  53,  68, "SC-8850 CYMBAL&CLAPS Set", "Mute Crash Cymbal 1"},
    {'P',   4,  53,  69, "SC-8850 CYMBAL&CLAPS Set", "Mute Crash Cymbal 2"},
    {'P',   4,  53,  70, "SC-8850 CYMBAL&CLAPS Set", "Reverse Crash Cymbal 1"},
    {'P',   4,  53,  71, "SC-8850 CYMBAL&CLAPS Set", "Reverse Crash Cymbal 2"},
    {'P',   4,  53,  72, "SC-8850 CYMBAL&CLAPS Set", "Reverse Crash Cymbal 3"},
    {'P',   4,  53,  73, "SC-8850 CYMBAL&CLAPS Set", "Reverse TR-909 Crash Cymbal"},
    {'P',   4,  53,  74, "SC-8850 CYMBAL&CLAPS Set", "[55] Splash Cymbal"},
    {'P',   4,  53,  75, "SC-8850 CYMBAL&CLAPS Set", "Splash Cymbal"},
    {'P',   4,  53,  76, "SC-8850 CYMBAL&CLAPS Set", "[88] Ride Bell"},
    {'P',   4,  53,  77, "SC-8850 CYMBAL&CLAPS Set", "[88] Brush Ride Bell"},
    {'P',   4,  53,  78, "SC-8850 CYMBAL&CLAPS Set", "[88] Ride Cymbal 1"},
    {'P',   4,  53,  79, "SC-8850 CYMBAL&CLAPS Set", "[88] Ride Cymbal 2"},
    {'P',   4,  53,  80, "SC-8850 CYMBAL&CLAPS Set", "[88] Brush Ride Cymbal"},
    {'P',   4,  53,  81, "SC-8850 CYMBAL&CLAPS Set", "Ride Cymbal Low Inner"},
    {'P',   4,  53,  82, "SC-8850 CYMBAL&CLAPS Set", "Ride Cymbal Mid Inner"},
    {'P',   4,  53,  83, "SC-8850 CYMBAL&CLAPS Set", "Ride Cymbal High Inner"},
    {'P',   4,  53,  84, "SC-8850 CYMBAL&CLAPS Set", "Ride Cymbal Low Edge"},
    {'P',   4,  53,  85, "SC-8850 CYMBAL&CLAPS Set", "Ride Cymbal Mid Edge"},
    {'P',   4,  53,  86, "SC-8850 CYMBAL&CLAPS Set", "Ride Cymbal High Edge"},
    {'P',   4,  53,  87, "SC-8850 CYMBAL&CLAPS Set", "TR-606 Ride Cymbal"},
    {'P',   4,  53,  88, "SC-8850 CYMBAL&CLAPS Set", "TR-808 Ride Cymbal"},
    {'P',   4,  53,  89, "SC-8850 CYMBAL&CLAPS Set", "Chinese Cymbal"},
    {'P',   4,  53,  90, "SC-8850 CYMBAL&CLAPS Set", "Chinese Cymbal 2"},
    {'P',   4,  53,  91, "SC-8850 CYMBAL&CLAPS Set", "[55] Hand Clap"},
    {'P',   4,  53,  92, "SC-8850 CYMBAL&CLAPS Set", "[88] Hand Clap 2"},
    {'P',   4,  53,  93, "SC-8850 CYMBAL&CLAPS Set", "[88] Hand Clap"},
    {'P',   4,  53,  94, "SC-8850 CYMBAL&CLAPS Set", "Hand Clap"},
    {'P',   4,  53,  95, "SC-8850 CYMBAL&CLAPS Set", "Hand Clap 2"},
    {'P',   4,  53,  96, "SC-8850 CYMBAL&CLAPS Set", "TR-707 Hand Clap"},
    {'P',   4,  54,  36, "SC-8850 GAMELAN 1 Set", "Kendang Wadon"},
    {'P',   4,  54,  37, "SC-8850 GAMELAN 1 Set", "Kendang Lanang"},
    {'P',   4,  54,  38, "SC-8850 GAMELAN 1 Set", "Bebarongan"},
    {'P',   4,  54,  39, "SC-8850 GAMELAN 1 Set", "Pelegongan"},
    {'P',   4,  54,  40, "SC-8850 GAMELAN 1 Set", "Kelontuk"},
    {'P',   4,  54,  41, "SC-8850 GAMELAN 1 Set", "Kelontuk Mute"},
    {'P',   4,  54,  42, "SC-8850 GAMELAN 1 Set", "Kelontuk Side"},
    {'P',   4,  54,  43, "SC-8850 GAMELAN 1 Set", "Gamelan Gong Wadon"},
    {'P',   4,  54,  44, "SC-8850 GAMELAN 1 Set", "Gamelan Gong Lanang"},
    {'P',   4,  54,  45, "SC-8850 GAMELAN 1 Set", "Ceng-Ceng"},
    {'P',   4,  54,  46, "SC-8850 GAMELAN 1 Set", "Kopyak Open"},
    {'P',   4,  54,  47, "SC-8850 GAMELAN 1 Set", "Kopyak Mute"},
    {'P',   4,  54,  48, "SC-8850 GAMELAN 1 Set", "Kajar"},
    {'P',   4,  54,  49, "SC-8850 GAMELAN 1 Set", "Kempur"},
    {'P',   4,  54,  50, "SC-8850 GAMELAN 1 Set", "Jegogan"},
    {'P',   4,  54,  51, "SC-8850 GAMELAN 1 Set", "Jegogan"},
    {'P',   4,  54,  52, "SC-8850 GAMELAN 1 Set", "Jegogan"},
    {'P',   4,  54,  53, "SC-8850 GAMELAN 1 Set", "Jegogan"},
    {'P',   4,  54,  54, "SC-8850 GAMELAN 1 Set", "Jegogan"},
    {'P',   4,  54,  55, "SC-8850 GAMELAN 1 Set", "Jublag"},
    {'P',   4,  54,  56, "SC-8850 GAMELAN 1 Set", "Jublag"},
    {'P',   4,  54,  57, "SC-8850 GAMELAN 1 Set", "Jublag"},
    {'P',   4,  54,  58, "SC-8850 GAMELAN 1 Set", "Jublag"},
    {'P',   4,  54,  59, "SC-8850 GAMELAN 1 Set", "Jublag"},
    {'P',   4,  54,  60, "SC-8850 GAMELAN 1 Set", "Penyacah"},
    {'P',   4,  54,  61, "SC-8850 GAMELAN 1 Set", "Penyacah"},
    {'P',   4,  54,  62, "SC-8850 GAMELAN 1 Set", "Penyacah"},
    {'P',   4,  54,  63, "SC-8850 GAMELAN 1 Set", "Penyacah"},
    {'P',   4,  54,  64, "SC-8850 GAMELAN 1 Set", "Penyacah"},
    {'P',   4,  54,  65, "SC-8850 GAMELAN 1 Set", "Penyacah"},
    {'P',   4,  54,  66, "SC-8850 GAMELAN 1 Set", "Penyacah"},
    {'P',   4,  54,  67, "SC-8850 GAMELAN 1 Set", "Pemade"},
    {'P',   4,  54,  68, "SC-8850 GAMELAN 1 Set", "Pemade"},
    {'P',   4,  54,  69, "SC-8850 GAMELAN 1 Set", "Pemade"},
    {'P',   4,  54,  70, "SC-8850 GAMELAN 1 Set", "Pemade"},
    {'P',   4,  54,  71, "SC-8850 GAMELAN 1 Set", "Pemade"},
    {'P',   4,  54,  72, "SC-8850 GAMELAN 1 Set", "Pemade"},
    {'P',   4,  54,  73, "SC-8850 GAMELAN 1 Set", "Pemade"},
    {'P',   4,  54,  74, "SC-8850 GAMELAN 1 Set", "Pemade"},
    {'P',   4,  54,  75, "SC-8850 GAMELAN 1 Set", "Pemade"},
    {'P',   4,  54,  76, "SC-8850 GAMELAN 1 Set", "Pemade"},
    {'P',   4,  54,  77, "SC-8850 GAMELAN 1 Set", "Reyong"},
    {'P',   4,  54,  78, "SC-8850 GAMELAN 1 Set", "Reyong"},
    {'P',   4,  54,  79, "SC-8850 GAMELAN 1 Set", "Reyong"},
    {'P',   4,  54,  80, "SC-8850 GAMELAN 1 Set", "Reyong"},
    {'P',   4,  54,  81, "SC-8850 GAMELAN 1 Set", "Reyong"},
    {'P',   4,  54,  82, "SC-8850 GAMELAN 1 Set", "Reyong"},
    {'P',   4,  54,  83, "SC-8850 GAMELAN 1 Set", "Reyong"},
    {'P',   4,  54,  84, "SC-8850 GAMELAN 1 Set", "Reyong"},
    {'P',   4,  54,  85, "SC-8850 GAMELAN 1 Set", "Reyong"},
    {'P',   4,  54,  86, "SC-8850 GAMELAN 1 Set", "Reyong"},
    {'P',   4,  54,  87, "SC-8850 GAMELAN 1 Set", "Reyong"},
    {'P',   4,  54,  88, "SC-8850 GAMELAN 1 Set", "Reyong"},
    {'P',   4,  56,  21, "SC-8850 SFX Set", "MC-500 Beep 1"},
    {'P',   4,  56,  22, "SC-8850 SFX Set", "MC-500 Beep 2"},
    {'P',   4,  56,  23, "SC-8850 SFX Set", "Guitar Slide"},
    {'P',   4,  56,  24, "SC-8850 SFX Set", "Guitar Wah"},
    {'P',   4,  56,  25, "SC-8850 SFX Set", "Guitar Slap"},
    {'P',   4,  56,  26, "SC-8850 SFX Set", "Chord Stroke Down"},
    {'P',   4,  56,  27, "SC-8850 SFX Set", "Chord Stroke Up"},
    {'P',   4,  56,  28, "SC-8850 SFX Set", "Biwa FX"},
    {'P',   4,  56,  29, "SC-8850 SFX Set", "Phonograph Noise"},
    {'P',   4,  56,  30, "SC-8850 SFX Set", "Tape Rewind"},
    {'P',   4,  56,  31, "SC-8850 SFX Set", "Scratch Push 2"},
    {'P',   4,  56,  32, "SC-8850 SFX Set", "Scratch Pull 2"},
    {'P',   4,  56,  33, "SC-8850 SFX Set", "Cutting Noise 2 Up"},
    {'P',   4,  56,  34, "SC-8850 SFX Set", "Cutting Noise 2 Down"},
    {'P',   4,  56,  35, "SC-8850 SFX Set", "Distortion Guitar Cutting Noise Up"},
    {'P',   4,  56,  36, "SC-8850 SFX Set", "Distortion Guitar Cutting Noise Down"},
    {'P',   4,  56,  37, "SC-8850 SFX Set", "Bass Slide"},
    {'P',   4,  56,  38, "SC-8850 SFX Set", "Pick Scrape"},
    {'P',   4,  56,  39, "SC-8850 SFX Set", "High Q"},
    {'P',   4,  56,  40, "SC-8850 SFX Set", "Slap"},
    {'P',   4,  56,  41, "SC-8850 SFX Set", "Scratch Push"},
    {'P',   4,  56,  42, "SC-8850 SFX Set", "Scratch Pull"},
    {'P',   4,  56,  43, "SC-8850 SFX Set", "Sticks"},
    {'P',   4,  56,  44, "SC-8850 SFX Set", "Square Click"},
    {'P',   4,  56,  45, "SC-8850 SFX Set", "Metronome Click"},
    {'P',   4,  56,  46, "SC-8850 SFX Set", "Metronome Bell"},
    {'P',   4,  56,  47, "SC-8850 SFX Set", "Guitar Fret Noise"},
    {'P',   4,  56,  48, "SC-8850 SFX Set", "Guitar Cutting Noise Up"},
    {'P',   4,  56,  49, "SC-8850 SFX Set", "Guitar Cutting Noise Down"},
    {'P',   4,  56,  50, "SC-8850 SFX Set", "String Slap of Double Bass"},
    {'P',   4,  56,  51, "SC-8850 SFX Set", "Flute Key Click Noise"},
    {'P',   4,  56,  52, "SC-8850 SFX Set", "Laughing"},
    {'P',   4,  56,  53, "SC-8850 SFX Set", "Screaming"},
    {'P',   4,  56,  54, "SC-8850 SFX Set", "Punch"},
    {'P',   4,  56,  55, "SC-8850 SFX Set", "Heart Beat"},
    {'P',   4,  56,  56, "SC-8850 SFX Set", "Footsteps 1"},
    {'P',   4,  56,  57, "SC-8850 SFX Set", "Footsteps 2"},
    {'P',   4,  56,  58, "SC-8850 SFX Set", "Applause"},
    {'P',   4,  56,  59, "SC-8850 SFX Set", "Door Creaking"},
    {'P',   4,  56,  60, "SC-8850 SFX Set", "Door"},
    {'P',   4,  56,  61, "SC-8850 SFX Set", "Scratch"},
    {'P',   4,  56,  62, "SC-8850 SFX Set", "Wind Chimes"},
    {'P',   4,  56,  63, "SC-8850 SFX Set", "Car - Engine"},
    {'P',   4,  56,  64, "SC-8850 SFX Set", "Car - Stop"},
    {'P',   4,  56,  65, "SC-8850 SFX Set", "Car - Passing"},
    {'P',   4,  56,  66, "SC-8850 SFX Set", "Car - Crash"},
    {'P',   4,  56,  67, "SC-8850 SFX Set", "Siren"},
    {'P',   4,  56,  68, "SC-8850 SFX Set", "Train"},
    {'P',   4,  56,  69, "SC-8850 SFX Set", "Jetplane"},
    {'P',   4,  56,  70, "SC-8850 SFX Set", "Helicopter"},
    {'P',   4,  56,  71, "SC-8850 SFX Set", "Starship"},
    {'P',   4,  56,  72, "SC-8850 SFX Set", "Gun Shot"},
    {'P',   4,  56,  73, "SC-8850 SFX Set", "Machine Gun"},
    {'P',   4,  56,  74, "SC-8850 SFX Set", "Laser Gun"},
    {'P',   4,  56,  75, "SC-8850 SFX Set", "Explosion"},
    {'P',   4,  56,  76, "SC-8850 SFX Set", "Dog"},
    {'P',   4,  56,  77, "SC-8850 SFX Set", "Horse-Gallop"},
    {'P',   4,  56,  78, "SC-8850 SFX Set", "Birds"},
    {'P',   4,  56,  79, "SC-8850 SFX Set", "Rain"},
    {'P',   4,  56,  80, "SC-8850 SFX Set", "Thunder"},
    {'P',   4,  56,  81, "SC-8850 SFX Set", "Wind"},
    {'P',   4,  56,  82, "SC-8850 SFX Set", "Seashore"},
    {'P',   4,  56,  83, "SC-8850 SFX Set", "Stream"},
    {'P',   4,  56,  84, "SC-8850 SFX Set", "Bubble"},
    {'P',   4,  56,  85, "SC-8850 SFX Set", "Kitty"},
    {'P',   4,  56,  86, "SC-8850 SFX Set", "Bird 2"},
    {'P',   4,  56,  87, "SC-8850 SFX Set", "Growl"},
    {'P',   4,  56,  88, "SC-8850 SFX Set", "Applause 2"},
    {'P',   4,  56,  89, "SC-8850 SFX Set", "Telephone 1"},
    {'P',   4,  56,  90, "SC-8850 SFX Set", "Telephone 2"},
    {'P',   4,  56,  91, "SC-8850 SFX Set", "Small Club 1"},
    {'P',   4,  56,  92, "SC-8850 SFX Set", "Small Club 2"},
    {'P',   4,  56,  93, "SC-8850 SFX Set", "Applause Wave"},
    {'P',   4,  56,  94, "SC-8850 SFX Set", "Eruption"},
    {'P',   4,  56,  95, "SC-8850 SFX Set", "Big Shot"},
    {'P',   4,  56,  96, "SC-8850 SFX Set", "Percussion Bang"},
    {'P',   4,  57,  36, "SC-8850 RHYTHM FX Set", "Reverse Kick 1"},
    {'P',   4,  57,  37, "SC-8850 RHYTHM FX Set", "Reverse Concert Bass Drum"},
    {'P',   4,  57,  38, "SC-8850 RHYTHM FX Set", "Reverse Power Kick1"},
    {'P',   4,  57,  39, "SC-8850 RHYTHM FX Set", "Reverse Electric Kick 1"},
    {'P',   4,  57,  40, "SC-8850 RHYTHM FX Set", "Reverse Snare 1"},
    {'P',   4,  57,  41, "SC-8850 RHYTHM FX Set", "Reverse Snare 2"},
    {'P',   4,  57,  42, "SC-8850 RHYTHM FX Set", "Reverse Standard 1 Snare 1"},
    {'P',   4,  57,  43, "SC-8850 RHYTHM FX Set", "Reverse Tight Snare"},
    {'P',   4,  57,  44, "SC-8850 RHYTHM FX Set", "Reverse Dance Snare"},
    {'P',   4,  57,  45, "SC-8850 RHYTHM FX Set", "Reverse 808 Snare"},
    {'P',   4,  57,  46, "SC-8850 RHYTHM FX Set", "Reverse Tom 1"},
    {'P',   4,  57,  47, "SC-8850 RHYTHM FX Set", "Reverse Tom 2"},
    {'P',   4,  57,  48, "SC-8850 RHYTHM FX Set", "Reverse Sticks"},
    {'P',   4,  57,  49, "SC-8850 RHYTHM FX Set", "Reverse Slap"},
    {'P',   4,  57,  50, "SC-8850 RHYTHM FX Set", "Reverse Cymbal 1"},
    {'P',   4,  57,  51, "SC-8850 RHYTHM FX Set", "Reverse Cymbal 2"},
    {'P',   4,  57,  52, "SC-8850 RHYTHM FX Set", "Reverse Open Hi-Hat"},
    {'P',   4,  57,  53, "SC-8850 RHYTHM FX Set", "Reverse Ride Cymbal"},
    {'P',   4,  57,  54, "SC-8850 RHYTHM FX Set", "Reverse CR-78 Open Hi-Hat"},
    {'P',   4,  57,  55, "SC-8850 RHYTHM FX Set", "Reverse Closed Hi-Hat"},
    {'P',   4,  57,  56, "SC-8850 RHYTHM FX Set", "Reverse Gong"},
    {'P',   4,  57,  57, "SC-8850 RHYTHM FX Set", "Reverse Bell Tree"},
    {'P',   4,  57,  58, "SC-8850 RHYTHM FX Set", "Reverse Guiro"},
    {'P',   4,  57,  59, "SC-8850 RHYTHM FX Set", "Reverse Bendir"},
    {'P',   4,  57,  60, "SC-8850 RHYTHM FX Set", "Reverse Gun Shot"},
    {'P',   4,  57,  61, "SC-8850 RHYTHM FX Set", "Reverse Scratch"},
    {'P',   4,  57,  62, "SC-8850 RHYTHM FX Set", "Reverse Laser Gun"},
    {'P',   4,  57,  63, "SC-8850 RHYTHM FX Set", "Key Click"},
    {'P',   4,  57,  64, "SC-8850 RHYTHM FX Set", "Techno Thip"},
    {'P',   4,  57,  65, "SC-8850 RHYTHM FX Set", "Pop Drop"},
    {'P',   4,  57,  66, "SC-8850 RHYTHM FX Set", "Woody Slap"},
    {'P',   4,  57,  67, "SC-8850 RHYTHM FX Set", "Distortion Kick"},
    {'P',   4,  57,  68, "SC-8850 RHYTHM FX Set", "Syn. Drops"},
    {'P',   4,  57,  69, "SC-8850 RHYTHM FX Set", "Reverse Hi Q"},
    {'P',   4,  57,  70, "SC-8850 RHYTHM FX Set", "Pipe"},
    {'P',   4,  57,  71, "SC-8850 RHYTHM FX Set", "Ice Block"},
    {'P',   4,  57,  72, "SC-8850 RHYTHM FX Set", "Digital Tambourine"},
    {'P',   4,  57,  73, "SC-8850 RHYTHM FX Set", "Alias"},
    {'P',   4,  57,  74, "SC-8850 RHYTHM FX Set", "Modulated Bell"},
    {'P',   4,  57,  75, "SC-8850 RHYTHM FX Set", "Spark"},
    {'P',   4,  57,  76, "SC-8850 RHYTHM FX Set", "Metallic Percussion"},
    {'P',   4,  57,  77, "SC-8850 RHYTHM FX Set", "Velocity Noise FX"},
    {'P',   4,  57,  78, "SC-8850 RHYTHM FX Set", "Stereo Noise Clap"},
    {'P',   4,  57,  79, "SC-8850 RHYTHM FX Set", "Swish"},
    {'P',   4,  57,  80, "SC-8850 RHYTHM FX Set", "Slappy"},
    {'P',   4,  57,  81, "SC-8850 RHYTHM FX Set", "Voice Ou"},
    {'P',   4,  57,  82, "SC-8850 RHYTHM FX Set", "Voice Au"},
    {'P',   4,  57,  83, "SC-8850 RHYTHM FX Set", "Hoo"},
    {'P',   4,  57,  84, "SC-8850 RHYTHM FX Set", "Tape Stop 1"},
    {'P',   4,  57,  85, "SC-8850 RHYTHM FX Set", "Tape Stop 2"},
    {'P',   4,  57,  86, "SC-8850 RHYTHM FX Set", "Missile"},
    {'P',   4,  57,  87, "SC-8850 RHYTHM FX Set", "Space Birds"},
    {'P',   4,  57,  88, "SC-8850 RHYTHM FX Set", "Flying Monster"},
    {'P',   4,  58,  36, "SC-8850 RHYTHM FX 2 Set", "Reverse TR-707 Kick 1"},
    {'P',   4,  58,  37, "SC-8850 RHYTHM FX 2 Set", "Reverse TR-909 Kick 1"},
    {'P',   4,  58,  38, "SC-8850 RHYTHM FX 2 Set", "Reverse Hip-Hop Kick 1"},
    {'P',   4,  58,  39, "SC-8850 RHYTHM FX 2 Set", "Reverse Jungle Kick 2"},
    {'P',   4,  58,  40, "SC-8850 RHYTHM FX 2 Set", "Reverse Techno Kick 2"},
    {'P',   4,  58,  41, "SC-8850 RHYTHM FX 2 Set", "Reverse TR-606 Snare 2"},
    {'P',   4,  58,  42, "SC-8850 RHYTHM FX 2 Set", "Reverse CR-78 Snare 1"},
    {'P',   4,  58,  43, "SC-8850 RHYTHM FX 2 Set", "Reverse CR-78 Snare 2"},
    {'P',   4,  58,  44, "SC-8850 RHYTHM FX 2 Set", "Reverse Jungle Snare 2"},
    {'P',   4,  58,  45, "SC-8850 RHYTHM FX 2 Set", "Reverse Techno Snare 2"},
    {'P',   4,  58,  46, "SC-8850 RHYTHM FX 2 Set", "Reverse TR-707 Snare"},
    {'P',   4,  58,  47, "SC-8850 RHYTHM FX 2 Set", "Reverse TR-606 Snare 1"},
    {'P',   4,  58,  48, "SC-8850 RHYTHM FX 2 Set", "Reverse TR-909 Snare 1"},
    {'P',   4,  58,  49, "SC-8850 RHYTHM FX 2 Set", "Reverse Hip-Hop Snare 2"},
    {'P',   4,  58,  50, "SC-8850 RHYTHM FX 2 Set", "Reverse Jungle Snare 1"},
    {'P',   4,  58,  51, "SC-8850 RHYTHM FX 2 Set", "Reverse House Snare"},
    {'P',   4,  58,  52, "SC-8850 RHYTHM FX 2 Set", "Reverse Closed Hi-Hat"},
    {'P',   4,  58,  53, "SC-8850 RHYTHM FX 2 Set", "Reverse TR-606 Closed Hi-Hat"},
    {'P',   4,  58,  54, "SC-8850 RHYTHM FX 2 Set", "Reverse TR-707 Closed Hi-Hat"},
    {'P',   4,  58,  55, "SC-8850 RHYTHM FX 2 Set", "Reverse TR-808 Closed Hi-Hat"},
    {'P',   4,  58,  56, "SC-8850 RHYTHM FX 2 Set", "Reverse Jungle Hi-Hat"},
    {'P',   4,  58,  57, "SC-8850 RHYTHM FX 2 Set", "Reverse Tambourine 2"},
    {'P',   4,  58,  58, "SC-8850 RHYTHM FX 2 Set", "Reverse Shake Tambourine"},
    {'P',   4,  58,  59, "SC-8850 RHYTHM FX 2 Set", "Reverse TR-808 Open Hi-Hat"},
    {'P',   4,  58,  60, "SC-8850 RHYTHM FX 2 Set", "Reverse TR-707 Open Hi-Hat"},
    {'P',   4,  58,  61, "SC-8850 RHYTHM FX 2 Set", "Reverse Open Hi-Hat"},
    {'P',   4,  58,  62, "SC-8850 RHYTHM FX 2 Set", "Reverse TR-606 Open Hi-Hat"},
    {'P',   4,  58,  63, "SC-8850 RHYTHM FX 2 Set", "Reverse  Hu Yin Luo"},
    {'P',   4,  58,  64, "SC-8850 RHYTHM FX 2 Set", "Reverse TR-707 Crash Cymbal"},
    {'P',   4,  58,  65, "SC-8850 RHYTHM FX 2 Set", "Voice One"},
    {'P',   4,  58,  66, "SC-8850 RHYTHM FX 2 Set", "Reverse Voice One"},
    {'P',   4,  58,  67, "SC-8850 RHYTHM FX 2 Set", "Voice Two"},
    {'P',   4,  58,  68, "SC-8850 RHYTHM FX 2 Set", "Reverse Voice Two"},
    {'P',   4,  58,  69, "SC-8850 RHYTHM FX 2 Set", "Voice Three"},
    {'P',   4,  58,  70, "SC-8850 RHYTHM FX 2 Set", "Reverse Voice Three"},
    {'P',   4,  58,  71, "SC-8850 RHYTHM FX 2 Set", "Voice Tah"},
    {'P',   4,  58,  72, "SC-8850 RHYTHM FX 2 Set", "Reverse Voice Tah"},
    {'P',   4,  58,  73, "SC-8850 RHYTHM FX 2 Set", "Voice Ou"},
    {'P',   4,  58,  74, "SC-8850 RHYTHM FX 2 Set", "Voice Au"},
    {'P',   4,  58,  75, "SC-8850 RHYTHM FX 2 Set", "Voice Whey"},
    {'P',   4,  58,  76, "SC-8850 RHYTHM FX 2 Set", "Frog Vpoce"},
    {'P',   4,  58,  77, "SC-8850 RHYTHM FX 2 Set", "Reverse Yyoo Dude"},
    {'P',   4,  58,  78, "SC-8850 RHYTHM FX 2 Set", "Douby"},
    {'P',   4,  58,  79, "SC-8850 RHYTHM FX 2 Set", "Reverse Douby"},
    {'P',   4,  58,  80, "SC-8850 RHYTHM FX 2 Set", "Baert High"},
    {'P',   4,  58,  81, "SC-8850 RHYTHM FX 2 Set", "Baert Low"},
    {'P',   4,  58,  82, "SC-8850 RHYTHM FX 2 Set", "Bounce"},
    {'P',   4,  58,  83, "SC-8850 RHYTHM FX 2 Set", "Reverse bounce"},
    {'P',   4,  58,  84, "SC-8850 RHYTHM FX 2 Set", "Distortion Knock"},
    {'P',   4,  58,  85, "SC-8850 RHYTHM FX 2 Set", "Guitar Slide"},
    {'P',   4,  58,  86, "SC-8850 RHYTHM FX 2 Set", "Sub Marine"},
    {'P',   4,  58,  87, "SC-8850 RHYTHM FX 2 Set", "Noise Attack"},
    {'P',   4,  58,  88, "SC-8850 RHYTHM FX 2 Set", "Space Worms"},
    {'P',   4,  58,  89, "SC-8850 RHYTHM FX 2 Set", "Emergency !"},
    {'P',   4,  58,  90, "SC-8850 RHYTHM FX 2 Set", "Calculating..."},
    {'P',   4,  58,  91, "SC-8850 RHYTHM FX 2 Set", "Saw LFO Saw"},
    {'P',   4,  59,  17, "SC-8850 RHYTHM FX 3 Set", "Reverse Bass Mute Noise"},
    {'P',   4,  59,  18, "SC-8850 RHYTHM FX 3 Set", "Reverse Bass Touch Noise"},
    {'P',   4,  59,  19, "SC-8850 RHYTHM FX 3 Set", "Reverse Bass Attack Noise"},
    {'P',   4,  59,  20, "SC-8850 RHYTHM FX 3 Set", "Reverse Clean Guitar Cut Noise Up"},
    {'P',   4,  59,  21, "SC-8850 RHYTHM FX 3 Set", "Reverse Clean Guitar Cut Noise Down"},
    {'P',   4,  59,  22, "SC-8850 RHYTHM FX 3 Set", "Reverse Clean Guitar Mute Up"},
    {'P',   4,  59,  23, "SC-8850 RHYTHM FX 3 Set", "Reverse Clean Guitar Mute Down"},
    {'P',   4,  59,  24, "SC-8850 RHYTHM FX 3 Set", "Reverse Distortion Guitar Cut Noise Up"},
    {'P',   4,  59,  25, "SC-8850 RHYTHM FX 3 Set", "Reverse Distortion Guitar Cut Noise Down"},
    {'P',   4,  59,  26, "SC-8850 RHYTHM FX 3 Set", "Reverse Distortion Guitar Stroke Noise"},
    {'P',   4,  59,  27, "SC-8850 RHYTHM FX 3 Set", "Reverse Distortion Guitar Mute Noise"},
    {'P',   4,  59,  28, "SC-8850 RHYTHM FX 3 Set", "Reverse Steel Guitar Slide Noise 1"},
    {'P',   4,  59,  29, "SC-8850 RHYTHM FX 3 Set", "Reverse Steel Guitar Slide Noise 2"},
    {'P',   4,  59,  30, "SC-8850 RHYTHM FX 3 Set", "Reverse Steel Guitar Slide Noise 3"},
    {'P',   4,  59,  31, "SC-8850 RHYTHM FX 3 Set", "Reverse Steel Guitar Slide Noise 4"},
    {'P',   4,  59,  32, "SC-8850 RHYTHM FX 3 Set", "Reverse Steel Guitar Stroke Noise"},
    {'P',   4,  59,  33, "SC-8850 RHYTHM FX 3 Set", "Reverse Steel Guitar Stroke Noise Up 1"},
    {'P',   4,  59,  34, "SC-8850 RHYTHM FX 3 Set", "Reverse Steel Guitar Stroke Noise Down 1"},
    {'P',   4,  59,  35, "SC-8850 RHYTHM FX 3 Set", "Reverse Steel Guitar Stroke Noise Up 2"},
    {'P',   4,  59,  36, "SC-8850 RHYTHM FX 3 Set", "Reverse Steel Guitar Stroke Noise Down 2"},
    {'P',   4,  59,  37, "SC-8850 RHYTHM FX 3 Set", "Reverse Trombone Noise"},
    {'P',   4,  59,  38, "SC-8850 RHYTHM FX 3 Set", "Reverse Trumpet Noise"},
    {'P',   4,  59,  39, "SC-8850 RHYTHM FX 3 Set", "Reverse Standard Kick 2"},
    {'P',   4,  59,  40, "SC-8850 RHYTHM FX 3 Set", "Reverse Standard Kick 1"},
    {'P',   4,  59,  41, "SC-8850 RHYTHM FX 3 Set", "Reverse Room Kick 2"},
    {'P',   4,  59,  42, "SC-8850 RHYTHM FX 3 Set", "Reverse Room Kick 1"},
    {'P',   4,  59,  43, "SC-8850 RHYTHM FX 3 Set", "Reverse Jazz Kick 2"},
    {'P',   4,  59,  44, "SC-8850 RHYTHM FX 3 Set", "Reverse Jazz Kick 1"},
    {'P',   4,  59,  45, "SC-8850 RHYTHM FX 3 Set", "Reverse Brush Kick 2"},
    {'P',   4,  59,  46, "SC-8850 RHYTHM FX 3 Set", "Reverse Brush Kick 1"},
    {'P',   4,  59,  47, "SC-8850 RHYTHM FX 3 Set", "Reverse HipHop Kick 2"},
    {'P',   4,  59,  48, "SC-8850 RHYTHM FX 3 Set", "Reverse HipHop Kick 1"},
    {'P',   4,  59,  49, "SC-8850 RHYTHM FX 3 Set", "Reverse Jungle Kick 2"},
    {'P',   4,  59,  50, "SC-8850 RHYTHM FX 3 Set", "Reverse Jungle Kick 1"},
    {'P',   4,  59,  51, "SC-8850 RHYTHM FX 3 Set", "Reverse TR-808 Kick"},
    {'P',   4,  59,  52, "SC-8850 RHYTHM FX 3 Set", "Reverse TR-909 Kick 2"},
    {'P',   4,  59,  53, "SC-8850 RHYTHM FX 3 Set", "Reverse TR-909 Kick 1"},
    {'P',   4,  59,  54, "SC-8850 RHYTHM FX 3 Set", "Reverse Fat Kick"},
    {'P',   4,  59,  55, "SC-8850 RHYTHM FX 3 Set", "Reverse Dance Kick"},
    {'P',   4,  59,  56, "SC-8850 RHYTHM FX 3 Set", "Reverse Standard Snare 1"},
    {'P',   4,  59,  57, "SC-8850 RHYTHM FX 3 Set", "Reverse Standard Snare 2"},
    {'P',   4,  59,  58, "SC-8850 RHYTHM FX 3 Set", "Reverse Room Snare 1"},
    {'P',   4,  59,  59, "SC-8850 RHYTHM FX 3 Set", "Reverse Room Snare 2"},
    {'P',   4,  59,  60, "SC-8850 RHYTHM FX 3 Set", "Reverse Jazz Snare 1"},
    {'P',   4,  59,  61, "SC-8850 RHYTHM FX 3 Set", "Reverse Jazz Snare 2"},
    {'P',   4,  59,  62, "SC-8850 RHYTHM FX 3 Set", "Reverse Brush Snare 1"},
    {'P',   4,  59,  63, "SC-8850 RHYTHM FX 3 Set", "Reverse Brush Snare 2"},
    {'P',   4,  59,  64, "SC-8850 RHYTHM FX 3 Set", "Reverse Lo-Fi Snare 1"},
    {'P',   4,  59,  65, "SC-8850 RHYTHM FX 3 Set", "Reverse Lo-Fi Snare 2"},
    {'P',   4,  59,  66, "SC-8850 RHYTHM FX 3 Set", "Reverse HipHop Snare 1"},
    {'P',   4,  59,  67, "SC-8850 RHYTHM FX 3 Set", "Reverse HipHop Snare 2"},
    {'P',   4,  59,  68, "SC-8850 RHYTHM FX 3 Set", "Reverse House Snare 1"},
    {'P',   4,  59,  69, "SC-8850 RHYTHM FX 3 Set", "Reverse Jungle Snare"},
    {'P',   4,  59,  70, "SC-8850 RHYTHM FX 3 Set", "Reverse 606 Snare 2"},
    {'P',   4,  59,  71, "SC-8850 RHYTHM FX 3 Set", "Reverse Techno Snare"},
    {'P',   4,  59,  72, "SC-8850 RHYTHM FX 3 Set", "Reverse Dance Snare"},
    {'P',   4,  59,  73, "SC-8850 RHYTHM FX 3 Set", "Reverse Rock Snare Dry"},
    {'P',   4,  59,  74, "SC-8850 RHYTHM FX 3 Set", "Reverse Lo-Fi Snare Rim"},
    {'P',   4,  59,  75, "SC-8850 RHYTHM FX 3 Set", "Reverse 909 Snare Rim"},
    {'P',   4,  59,  76, "SC-8850 RHYTHM FX 3 Set", "Reverse Jungle Snare Rim"},
    {'P',   4,  59,  77, "SC-8850 RHYTHM FX 3 Set", "Reverse Dance Snare Rim"},
    {'P',   4,  59,  78, "SC-8850 RHYTHM FX 3 Set", "Reverse House Snare Rim"},
    {'P',   4,  59,  79, "SC-8850 RHYTHM FX 3 Set", "Reverse Brush Tom 1"},
    {'P',   4,  59,  80, "SC-8850 RHYTHM FX 3 Set", "Reverse Brush Tom 2"},
    {'P',   4,  59,  81, "SC-8850 RHYTHM FX 3 Set", "Reverse Brush Tom 3"},
    {'P',   4,  59,  82, "SC-8850 RHYTHM FX 3 Set", "Reverse 606 Tom"},
    {'P',   4,  59,  83, "SC-8850 RHYTHM FX 3 Set", "Reverse Jungle Crash Cymbal"},
    {'P',   4,  59,  84, "SC-8850 RHYTHM FX 3 Set", "Reverse Standard Closed Hi-Hat"},
    {'P',   4,  59,  85, "SC-8850 RHYTHM FX 3 Set", "Reverse Room Closed Hi-Hat"},
    {'P',   4,  59,  86, "SC-8850 RHYTHM FX 3 Set", "Reverse Jazz Closed Hi-Hat"},
    {'P',   4,  59,  87, "SC-8850 RHYTHM FX 3 Set", "Reverse Brush Closed Hi-Hat"},
    {'P',   4,  59,  88, "SC-8850 RHYTHM FX 3 Set", "Reverse 707 Claps"},
    {'P',   4,  59,  89, "SC-8850 RHYTHM FX 3 Set", "Reverse 909 Claps"},
    {'P',   4,  59,  90, "SC-8850 RHYTHM FX 3 Set", "Reverse R&B Claps 1"},
    {'P',   4,  59,  91, "SC-8850 RHYTHM FX 3 Set", "Reverse HipHop Claps"},
    {'P',   4,  59,  92, "SC-8850 RHYTHM FX 3 Set", "Reverse Comp Claps 2"},
    {'P',   4,  59,  93, "SC-8850 RHYTHM FX 3 Set", "Reverse Shaker 2"},
    {'P',   4,  59,  94, "SC-8850 RHYTHM FX 3 Set", "Reverse Jungle Shaker"},
    {'P',   4,  59,  95, "SC-8850 RHYTHM FX 3 Set", "Reverse Clap Hit"},
    {'P',   4,  59,  96, "SC-8850 RHYTHM FX 3 Set", "Reverse Boeeeen"},
    {'P',   4,  59,  97, "SC-8850 RHYTHM FX 3 Set", "Reverse Bounce"},
    {'P',   4,  59,  98, "SC-8850 RHYTHM FX 3 Set", "Reverse CD Tray"},
    {'P',   4,  59,  99, "SC-8850 RHYTHM FX 3 Set", "Reverse Drill"},
    {'P',   4,  59, 100, "SC-8850 RHYTHM FX 3 Set", "Reverse Glass Stir"},
    {'P',   4,  59, 101, "SC-8850 RHYTHM FX 3 Set", "Reverse Ice Ring"},
    {'P',   4,  59, 102, "SC-8850 RHYTHM FX 3 Set", "Reverse Industry Hit"},
    {'P',   4,  59, 103, "SC-8850 RHYTHM FX 3 Set", "Reverse Scratch 4"},
    {'P',   4,  59, 104, "SC-8850 RHYTHM FX 3 Set", "Reverse Scratch 5"},
    {'P',   4,  59, 105, "SC-8850 RHYTHM FX 3 Set", "Reverse Scratch 6"},
    {'P',   4,  59, 106, "SC-8850 RHYTHM FX 3 Set", "Reverse Scratch 7"},
    {'P',   4,  59, 107, "SC-8850 RHYTHM FX 3 Set", "Reverse Seal"},
    {'P',   4,  59, 108, "SC-8850 RHYTHM FX 3 Set", "Reverse Stab! 1"},
    {'P',   4,  59, 109, "SC-8850 RHYTHM FX 3 Set", "Reverse Stab! 2"},
    {'P',   4,  59, 110, "SC-8850 RHYTHM FX 3 Set", "Reverse Sword Boom"},
    {'P',   4,  59, 111, "SC-8850 RHYTHM FX 3 Set", "Reverse Sword Cross"},
    {'P',   4,  59, 112, "SC-8850 RHYTHM FX 3 Set", "Reverse Thrill Hit"},
    {'P',   4,  59, 113, "SC-8850 RHYTHM FX 3 Set", "Reverse Audio Switch"},
    {'P',   4,  59, 114, "SC-8850 RHYTHM FX 3 Set", "Reverse Keyboard Typing 1"},
    {'P',   4,  59, 115, "SC-8850 RHYTHM FX 3 Set", "Reverse Keyboard Typing 2"},
    {'P',   4,  59, 116, "SC-8850 RHYTHM FX 3 Set", "Reverse Keyboard Typing 3"},
    {'P',   4,  59, 117, "SC-8850 RHYTHM FX 3 Set", "Reverse Keyboard Typing 4"},
    {'P',   4,  59, 118, "SC-8850 RHYTHM FX 3 Set", "Reverse Keyboard Typing 5"},
    {'P',   4,  59, 119, "SC-8850 RHYTHM FX 3 Set", "Reverse Keyboard Typing 6"},
    {'P',   4,  60,  31, "SC-8850 SFX 2 Set", "Acoustic Bass Mute Noise"},
    {'P',   4,  60,  32, "SC-8850 SFX 2 Set", "Acoustic Bass Touch Noise"},
    {'P',   4,  60,  33, "SC-8850 SFX 2 Set", "Acoustic Bass Attack Noise"},
    {'P',   4,  60,  34, "SC-8850 SFX 2 Set", "Distortion Guitar Mute Noise"},
    {'P',   4,  60,  35, "SC-8850 SFX 2 Set", "Steel Guitar Slide Noise 1"},
    {'P',   4,  60,  36, "SC-8850 SFX 2 Set", "Steel Guitar Slide Noise 2"},
    {'P',   4,  60,  37, "SC-8850 SFX 2 Set", "Steel Guitar Slide Noise 3"},
    {'P',   4,  60,  38, "SC-8850 SFX 2 Set", "Steel Guitar Slide Noise 4"},
    {'P',   4,  60,  39, "SC-8850 SFX 2 Set", "Guitar Stroke Noise 1"},
    {'P',   4,  60,  40, "SC-8850 SFX 2 Set", "Guitar Stroke Noise 2"},
    {'P',   4,  60,  41, "SC-8850 SFX 2 Set", "Guitar Stroke Noise 3"},
    {'P',   4,  60,  42, "SC-8850 SFX 2 Set", "Guitar Stroke Noise 4"},
    {'P',   4,  60,  43, "SC-8850 SFX 2 Set", "Guitar Stroke Noise 5"},
    {'P',   4,  60,  44, "SC-8850 SFX 2 Set", "Open CD Tray"},
    {'P',   4,  60,  45, "SC-8850 SFX 2 Set", "Audio Switch"},
    {'P',   4,  60,  46, "SC-8850 SFX 2 Set", "Keyboard Typing 1"},
    {'P',   4,  60,  47, "SC-8850 SFX 2 Set", "Keyboard Typing 2"},
    {'P',   4,  60,  48, "SC-8850 SFX 2 Set", "Keyboard Typing 3"},
    {'P',   4,  60,  49, "SC-8850 SFX 2 Set", "Keyboard Typing 4"},
    {'P',   4,  60,  50, "SC-8850 SFX 2 Set", "Keyboard Typing 5"},
    {'P',   4,  60,  51, "SC-8850 SFX 2 Set", "Keyboard Typing 6"},
    {'P',   4,  60,  52, "SC-8850 SFX 2 Set", "Baby Laughing"},
    {'P',   4,  60,  53, "SC-8850 SFX 2 Set", "Clap Hit"},
    {'P',   4,  60,  54, "SC-8850 SFX 2 Set", "Stab! 1"},
    {'P',   4,  60,  55, "SC-8850 SFX 2 Set", "Stab! 2"},
    {'P',   4,  60,  56, "SC-8850 SFX 2 Set", "Bounce Hit"},
    {'P',   4,  60,  57, "SC-8850 SFX 2 Set", "Boeeeen"},
    {'P',   4,  60,  58, "SC-8850 SFX 2 Set", "Glass Stir"},
    {'P',   4,  60,  59, "SC-8850 SFX 2 Set", "Ice Ring"},
    {'P',   4,  60,  60, "SC-8850 SFX 2 Set", "Crack Bottle"},
    {'P',   4,  60,  61, "SC-8850 SFX 2 Set", "Pour Bottle"},
    {'P',   4,  60,  62, "SC-8850 SFX 2 Set", "Soda"},
    {'P',   4,  60,  63, "SC-8850 SFX 2 Set", "Car Engine 2"},
    {'P',   4,  60,  64, "SC-8850 SFX 2 Set", "Car - Horn"},
    {'P',   4,  60,  65, "SC-8850 SFX 2 Set", "Railroad Crossing"},
    {'P',   4,  60,  66, "SC-8850 SFX 2 Set", "SL 1"},
    {'P',   4,  60,  67, "SC-8850 SFX 2 Set", "SL 2"},
    {'P',   4,  60,  68, "SC-8850 SFX 2 Set", "Over Blow"},
    {'P',   4,  60,  69, "SC-8850 SFX 2 Set", "Sword Boom!"},
    {'P',   4,  60,  70, "SC-8850 SFX 2 Set", "Sword Cross"},
    {'P',   4,  60,  71, "SC-8850 SFX 2 Set", "Industry Hit"},
    {'P',   4,  60,  72, "SC-8850 SFX 2 Set", "Drill Hit"},
    {'P',   4,  60,  73, "SC-8850 SFX 2 Set", "Compresser"},
    {'P',   4,  60,  74, "SC-8850 SFX 2 Set", "Thrill Hit"},
    {'P',   4,  60,  75, "SC-8850 SFX 2 Set", "Explosion 2"},
    {'P',   4,  60,  76, "SC-8850 SFX 2 Set", "Seal"},
    {'P',   4,  60,  77, "SC-8850 SFX 2 Set", "Fancy Animal"},
    {'P',   4,  60,  78, "SC-8850 SFX 2 Set", "Cricket"},
    {'P',   4,  60,  79, "SC-8850 SFX 2 Set", "Bear"},
    {'P',   4,  60,  80, "SC-8850 SFX 2 Set", "Frog Vpoce"},
    {'P',   4,  60,  81, "SC-8850 SFX 2 Set", "Wind 2"},
    {'P',   4,  60,  82, "SC-8850 SFX 2 Set", "Scratch 3"},
    {'P',   4,  60,  83, "SC-8850 SFX 2 Set", "Scratch 4"},
    {'P',   4,  60,  84, "SC-8850 SFX 2 Set", "Scratch 5"},
    {'P',   4,  60,  85, "SC-8850 SFX 2 Set", "Scratch 6"},
    {'P',   4,  60,  86, "SC-8850 SFX 2 Set", "Scratch 7"},
    {'P',   4,  60,  87, "SC-8850 SFX 2 Set", "Noise Attack"},
    {'P',   4,  60,  88, "SC-8850 SFX 2 Set", "Bounce"},
    {'P',   4,  60,  89, "SC-8850 SFX 2 Set", "Dist Knock"},
    {'P',   4,  60,  90, "SC-8850 SFX 2 Set", "Bound"},
    {'P',   4,  61,  36, "SC-8850 VOICE Set", "Reverse Breath Slow"},
    {'P',   4,  61,  37, "SC-8850 VOICE Set", "Reverse Breath Short"},
    {'P',   4,  61,  38, "SC-8850 VOICE Set", "Reverse Breath Strong"},
    {'P',   4,  61,  39, "SC-8850 VOICE Set", "Reverse Woman's Breath"},
    {'P',   4,  61,  40, "SC-8850 VOICE Set", "Reverse Man's Breath"},
    {'P',   4,  61,  41, "SC-8850 VOICE Set", "Reverse Voice One"},
    {'P',   4,  61,  42, "SC-8850 VOICE Set", "Reverse Voice Two"},
    {'P',   4,  61,  43, "SC-8850 VOICE Set", "Reverse Voice Three"},
    {'P',   4,  61,  44, "SC-8850 VOICE Set", "Reverse Voice Tha"},
    {'P',   4,  61,  45, "SC-8850 VOICE Set", "Reverse Voice Come On"},
    {'P',   4,  61,  46, "SC-8850 VOICE Set", "Reverse Voice Kikit"},
    {'P',   4,  61,  47, "SC-8850 VOICE Set", "Reverse Voice Aou"},
    {'P',   4,  61,  48, "SC-8850 VOICE Set", "Reverse Voice Ooh"},
    {'P',   4,  61,  49, "SC-8850 VOICE Set", "Reverse Voice Hie 2"},
    {'P',   4,  61,  50, "SC-8850 VOICE Set", "Reverse Baby Laughing"},
    {'P',   4,  61,  51, "SC-8850 VOICE Set", "Reverse Yyooh"},
    {'P',   4,  61,  52, "SC-8850 VOICE Set", "Reverse Japanese Female Voice Lan"},
    {'P',   4,  61,  53, "SC-8850 VOICE Set", "Reverse Ooue!"},
    {'P',   4,  61,  54, "SC-8850 VOICE Set", "Flute Breath 1"},
    {'P',   4,  61,  55, "SC-8850 VOICE Set", "Flute Breath 2"},
    {'P',   4,  61,  56, "SC-8850 VOICE Set", "Flute Breath 3"},
    {'P',   4,  61,  57, "SC-8850 VOICE Set", "Voice Breath 1"},
    {'P',   4,  61,  58, "SC-8850 VOICE Set", "Voice Breath 2"},
    {'P',   4,  61,  59, "SC-8850 VOICE Set", "Voice One"},
    {'P',   4,  61,  60, "SC-8850 VOICE Set", "Voice Two"},
    {'P',   4,  61,  61, "SC-8850 VOICE Set", "Voice Three"},
    {'P',   4,  61,  62, "SC-8850 VOICE Set", "Voice Tah"},
    {'P',   4,  61,  63, "SC-8850 VOICE Set", "Voice ComeOn"},
    {'P',   4,  61,  64, "SC-8850 VOICE Set", "Voice Kikit"},
    {'P',   4,  61,  65, "SC-8850 VOICE Set", "Voice Aou"},
    {'P',   4,  61,  66, "SC-8850 VOICE Set", "Voice Oou"},
    {'P',   4,  61,  67, "SC-8850 VOICE Set", "Punch"},
    {'P',   4,  61,  68, "SC-8850 VOICE Set", "Screaming"},
    {'P',   4,  61,  69, "SC-8850 VOICE Set", "Laughing"},
    {'P',   4,  61,  70, "SC-8850 VOICE Set", "Voice Hie"},
    {'P',   4,  61,  71, "SC-8850 VOICE Set", "Baby Laughing"},
    {'P',   4,  61,  72, "SC-8850 VOICE Set", "Frog Vpoce"},
    {'P',   4,  61,  73, "SC-8850 VOICE Set", "Yyooh Dude"},
    {'P',   4,  61,  74, "SC-8850 VOICE Set", "Voice Ou"},
    {'P',   4,  61,  75, "SC-8850 VOICE Set", "Voice Au"},
    {'P',   4,  61,  76, "SC-8850 VOICE Set", "Jazz Voice Thum"},
    {'P',   4,  61,  77, "SC-8850 VOICE Set", "Jazz Voice Bap"},
    {'P',   4,  61,  78, "SC-8850 VOICE Set", "Jazz Voice Dat"},
    {'P',   4,  61,  79, "SC-8850 VOICE Set", "Jazz Voice Dow"},
    {'P',   4,  61,  80, "SC-8850 VOICE Set", "Voice Oohs 2"},
    {'P',   4,  61,  81, "SC-8850 VOICE Set", "Voice Oohs Chord Maj7 A"},
    {'P',   4,  61,  82, "SC-8850 VOICE Set", "Voice Oohs Chord Maj7 B"},
    {'P',   4,  61,  83, "SC-8850 VOICE Set", "Voice Oohs Chord Sus4 A"},
    {'P',   4,  61,  84, "SC-8850 VOICE Set", "Voice Oohs Chord Sus4 B"},
    {'P',   4,  61,  85, "SC-8850 VOICE Set", "Japanese Female Voice Lah"},
    {'P',   4,  61,  86, "SC-8850 VOICE Set", "Japanese Female Voice Lan"},
    {'P',   4,  61,  87, "SC-8850 VOICE Set", "Japanese Male Voice Wah"},
    {'P',   4,  61,  88, "SC-8850 VOICE Set", "Japanese Male Voice Woh"},
    {'P',   4,  62,  32, "SC-8850 CYM&CLAPS 2 Set", "Reverse Standard Closed Hi-Hat"},
    {'P',   4,  62,  33, "SC-8850 CYM&CLAPS 2 Set", "Reverse Room Closed Hi-Hat"},
    {'P',   4,  62,  34, "SC-8850 CYM&CLAPS 2 Set", "Reverse Jazz Closed Hi-Hat"},
    {'P',   4,  62,  35, "SC-8850 CYM&CLAPS 2 Set", "Reverse Brush Closed Hi-Hat"},
    {'P',   4,  62,  36, "SC-8850 CYM&CLAPS 2 Set", "Standard 1 Closed Hi-Hat"},
    {'P',   4,  62,  37, "SC-8850 CYM&CLAPS 2 Set", "Room Closed Hi-Hat"},
    {'P',   4,  62,  38, "SC-8850 CYM&CLAPS 2 Set", "Jazz Closed Hi-Hat"},
    {'P',   4,  62,  39, "SC-8850 CYM&CLAPS 2 Set", "Brush Closed Hi-Hat"},
    {'P',   4,  62,  40, "SC-8850 CYM&CLAPS 2 Set", "TR-707 Closed Hi-Hat"},
    {'P',   4,  62,  41, "SC-8850 CYM&CLAPS 2 Set", "TR-606 Closed Hi-Hat"},
    {'P',   4,  62,  42, "SC-8850 CYM&CLAPS 2 Set", "TR-808 Closed Hi-Hat"},
    {'P',   4,  62,  43, "SC-8850 CYM&CLAPS 2 Set", "CR-78 Closed Hi-Hat"},
    {'P',   4,  62,  44, "SC-8850 CYM&CLAPS 2 Set", "Pedal Hi-Hat"},
    {'P',   4,  62,  45, "SC-8850 CYM&CLAPS 2 Set", "Pedal Hi-Hat"},
    {'P',   4,  62,  46, "SC-8850 CYM&CLAPS 2 Set", "Pedal Hi-Hat"},
    {'P',   4,  62,  47, "SC-8850 CYM&CLAPS 2 Set", "Half-Open Hi-Hat 1"},
    {'P',   4,  62,  48, "SC-8850 CYM&CLAPS 2 Set", "Half-Open Hi-Hat 2"},
    {'P',   4,  62,  49, "SC-8850 CYM&CLAPS 2 Set", "Standard 1 Open Hi-Hat"},
    {'P',   4,  62,  50, "SC-8850 CYM&CLAPS 2 Set", "Room Open Hi-Hat"},
    {'P',   4,  62,  51, "SC-8850 CYM&CLAPS 2 Set", "Jazz Open Hi-Hat"},
    {'P',   4,  62,  52, "SC-8850 CYM&CLAPS 2 Set", "Brush Open Hi-Hat"},
    {'P',   4,  62,  53, "SC-8850 CYM&CLAPS 2 Set", "TR-909 Open Hi-Hat"},
    {'P',   4,  62,  54, "SC-8850 CYM&CLAPS 2 Set", "TR-707 Open Hi-Hat"},
    {'P',   4,  62,  55, "SC-8850 CYM&CLAPS 2 Set", "TR-606 Open Hi-Hat"},
    {'P',   4,  62,  56, "SC-8850 CYM&CLAPS 2 Set", "TR-808 Open Hi-Hat"},
    {'P',   4,  62,  57, "SC-8850 CYM&CLAPS 2 Set", "CR-78 Open Hi-Hat"},
    {'P',   4,  62,  58, "SC-8850 CYM&CLAPS 2 Set", "Standard 1 Crash Cymbal"},
    {'P',   4,  62,  59, "SC-8850 CYM&CLAPS 2 Set", "Room Crash Cymbal"},
    {'P',   4,  62,  60, "SC-8850 CYM&CLAPS 2 Set", "Jazz Crash Cymbal"},
    {'P',   4,  62,  61, "SC-8850 CYM&CLAPS 2 Set", "Brush Crash Cymbal"},
    {'P',   4,  62,  62, "SC-8850 CYM&CLAPS 2 Set", "Hard Crash Cymbal"},
    {'P',   4,  62,  63, "SC-8850 CYM&CLAPS 2 Set", "TR-909 Crash Cymbal"},
    {'P',   4,  62,  64, "SC-8850 CYM&CLAPS 2 Set", "Jungle Crash Cymbal"},
    {'P',   4,  62,  65, "SC-8850 CYM&CLAPS 2 Set", "TR-808 Crash Cymbal"},
    {'P',   4,  62,  66, "SC-8850 CYM&CLAPS 2 Set", "Standard 1 Mute Crash Cymbal"},
    {'P',   4,  62,  67, "SC-8850 CYM&CLAPS 2 Set", "Room Mute Crash Cymbal"},
    {'P',   4,  62,  68, "SC-8850 CYM&CLAPS 2 Set", "Jazz Mute Crash Cymbal"},
    {'P',   4,  62,  69, "SC-8850 CYM&CLAPS 2 Set", "Brush Mute Crash Cymbal"},
    {'P',   4,  62,  70, "SC-8850 CYM&CLAPS 2 Set", "Mute Crash Cymbal 1"},
    {'P',   4,  62,  71, "SC-8850 CYM&CLAPS 2 Set", "Mute Crash Cymbal 2"},
    {'P',   4,  62,  72, "SC-8850 CYM&CLAPS 2 Set", "Reverse Standard 1 Crash Cymbal"},
    {'P',   4,  62,  73, "SC-8850 CYM&CLAPS 2 Set", "Reverse Room Crash Cymbal"},
    {'P',   4,  62,  74, "SC-8850 CYM&CLAPS 2 Set", "Reverse Jazz Crash Cymbal"},
    {'P',   4,  62,  75, "SC-8850 CYM&CLAPS 2 Set", "Reverse Brush Crash Cymbal"},
    {'P',   4,  62,  76, "SC-8850 CYM&CLAPS 2 Set", "Splash Cymbal"},
    {'P',   4,  62,  77, "SC-8850 CYM&CLAPS 2 Set", "Standard Ride Bell"},
    {'P',   4,  62,  78, "SC-8850 CYM&CLAPS 2 Set", "Room Ride Bell"},
    {'P',   4,  62,  79, "SC-8850 CYM&CLAPS 2 Set", "Jazz Ride Bell"},
    {'P',   4,  62,  80, "SC-8850 CYM&CLAPS 2 Set", "Brush Ride Bell"},
    {'P',   4,  62,  81, "SC-8850 CYM&CLAPS 2 Set", "Standard Ride Cymbal"},
    {'P',   4,  62,  82, "SC-8850 CYM&CLAPS 2 Set", "Room Ride Cymbal"},
    {'P',   4,  62,  83, "SC-8850 CYM&CLAPS 2 Set", "Jazz Ride Cymbal"},
    {'P',   4,  62,  84, "SC-8850 CYM&CLAPS 2 Set", "Brush Ride Cymbal"},
    {'P',   4,  62,  85, "SC-8850 CYM&CLAPS 2 Set", "TR-606 Ride Cymbal"},
    {'P',   4,  62,  86, "SC-8850 CYM&CLAPS 2 Set", "TR-808 Ride Cymbal"},
    {'P',   4,  62,  87, "SC-8850 CYM&CLAPS 2 Set", "Chinese Cymbal"},
    {'P',   4,  62,  88, "SC-8850 CYM&CLAPS 2 Set", "Chinese Cymbal 2"},
    {'P',   4,  62,  89, "SC-8850 CYM&CLAPS 2 Set", "TR-707 Claps"},
    {'P',   4,  62,  90, "SC-8850 CYM&CLAPS 2 Set", "Hip-Hop Claps"},
    {'P',   4,  62,  91, "SC-8850 CYM&CLAPS 2 Set", "R&B Claps"},
    {'P',   4,  62,  92, "SC-8850 CYM&CLAPS 2 Set", "TR-909 Claps"},
    {'P',   4,  62,  93, "SC-8850 CYM&CLAPS 2 Set", "Comp Claps 2"},
    {'P',   4,  62,  94, "SC-8850 CYM&CLAPS 2 Set", "Hand Clap"},
    {'P',   4,  62,  95, "SC-8850 CYM&CLAPS 2 Set", "[Pro] Hand Clap 2"},
    {'P',   4,  62,  96, "SC-8850 CYM&CLAPS 2 Set", "[Pro] TR-707 Hand Clap"},
};

static const MidiProgram Gm2Set[] =
{
    {'M', 121,   0,   0, "GM2 Var #00", "Acoustic Grand Piano"},
    {'M', 121,   0,   1, "GM2 Var #00", "Bright Acoustic Piano"},
    {'M', 121,   0,   2, "GM2 Var #00", "Electric Grand Piano"},
    {'M', 121,   0,   3, "GM2 Var #00", "Honky-tonk Piano"},
    {'M', 121,   0,   4, "GM2 Var #00", "Electric Piano 1"},
    {'M', 121,   0,   5, "GM2 Var #00", "Electric Piano 2"},
    {'M', 121,   0,   6, "GM2 Var #00", "Harpsichord"},
    {'M', 121,   0,   7, "GM2 Var #00", "Clavi"},
    {'M', 121,   0,   8, "GM2 Var #00", "Celesta"},
    {'M', 121,   0,   9, "GM2 Var #00", "Glockenspiel"},
    {'M', 121,   0,  10, "GM2 Var #00", "Music Box"},
    {'M', 121,   0,  11, "GM2 Var #00", "Vibraphone"},
    {'M', 121,   0,  12, "GM2 Var #00", "Marimba"},
    {'M', 121,   0,  13, "GM2 Var #00", "Xylophone"},
    {'M', 121,   0,  14, "GM2 Var #00", "Tubular Bells"},
    {'M', 121,   0,  15, "GM2 Var #00", "Dulcimer"},
    {'M', 121,   0,  16, "GM2 Var #00", "Drawbar Organ"},
    {'M', 121,   0,  17, "GM2 Var #00", "Percussive Organ"},
    {'M', 121,   0,  18, "GM2 Var #00", "Rock Organ"},
    {'M', 121,   0,  19, "GM2 Var #00", "Church Organ"},
    {'M', 121,   0,  20, "GM2 Var #00", "Reed Organ"},
    {'M', 121,   0,  21, "GM2 Var #00", "Accordion"},
    {'M', 121,   0,  22, "GM2 Var #00", "Harmonica"},
    {'M', 121,   0,  23, "GM2 Var #00", "Tango Accordion"},
    {'M', 121,   0,  24, "GM2 Var #00", "Acoustic Guitar (nylon)"},
    {'M', 121,   0,  25, "GM2 Var #00", "Acoustic Guitar (steel)"},
    {'M', 121,   0,  26, "GM2 Var #00", "Electric Guitar (jazz)"},
    {'M', 121,   0,  27, "GM2 Var #00", "Electric Guitar (clean)"},
    {'M', 121,   0,  28, "GM2 Var #00", "Electric Guitar (muted)"},
    {'M', 121,   0,  29, "GM2 Var #00", "Overdriven Guitar"},
    {'M', 121,   0,  30, "GM2 Var #00", "Distortion Guitar"},
    {'M', 121,   0,  31, "GM2 Var #00", "Guitar Harmonics"},
    {'M', 121,   0,  32, "GM2 Var #00", "Acoustic Bass"},
    {'M', 121,   0,  33, "GM2 Var #00", "Electric Bass (finger)"},
    {'M', 121,   0,  34, "GM2 Var #00", "Electric Bass (pick)"},
    {'M', 121,   0,  35, "GM2 Var #00", "Fretless Bass"},
    {'M', 121,   0,  36, "GM2 Var #00", "Slap Bass 1"},
    {'M', 121,   0,  37, "GM2 Var #00", "Slap Bass 2"},
    {'M', 121,   0,  38, "GM2 Var #00", "Synth Bass 1"},
    {'M', 121,   0,  39, "GM2 Var #00", "Synth Bass 2"},
    {'M', 121,   0,  40, "GM2 Var #00", "Violin"},
    {'M', 121,   0,  41, "GM2 Var #00", "Viola"},
    {'M', 121,   0,  42, "GM2 Var #00", "Cello"},
    {'M', 121,   0,  43, "GM2 Var #00", "Contrabass"},
    {'M', 121,   0,  44, "GM2 Var #00", "Tremolo Strings"},
    {'M', 121,   0,  45, "GM2 Var #00", "Pizzicato Strings"},
    {'M', 121,   0,  46, "GM2 Var #00", "Orchestral Harp"},
    {'M', 121,   0,  47, "GM2 Var #00", "Timpani"},
    {'M', 121,   0,  48, "GM2 Var #00", "String Ensembles 1"},
    {'M', 121,   0,  49, "GM2 Var #00", "String Ensembles 2"},
    {'M', 121,   0,  50, "GM2 Var #00", "SynthStrings 1"},
    {'M', 121,   0,  51, "GM2 Var #00", "SynthStrings 2"},
    {'M', 121,   0,  52, "GM2 Var #00", "Choir Aahs"},
    {'M', 121,   0,  53, "GM2 Var #00", "Voice Oohs"},
    {'M', 121,   0,  54, "GM2 Var #00", "Synth Voice"},
    {'M', 121,   0,  55, "GM2 Var #00", "Orchestra Hit"},
    {'M', 121,   0,  56, "GM2 Var #00", "Trumpet"},
    {'M', 121,   0,  57, "GM2 Var #00", "Trombone"},
    {'M', 121,   0,  58, "GM2 Var #00", "Tuba"},
    {'M', 121,   0,  59, "GM2 Var #00", "Muted Trumpet"},
    {'M', 121,   0,  60, "GM2 Var #00", "French Horn"},
    {'M', 121,   0,  61, "GM2 Var #00", "Brass Section"},
    {'M', 121,   0,  62, "GM2 Var #00", "Synth Brass 1"},
    {'M', 121,   0,  63, "GM2 Var #00", "Synth Brass 2"},
    {'M', 121,   0,  64, "GM2 Var #00", "Soprano Sax"},
    {'M', 121,   0,  65, "GM2 Var #00", "Alto Sax"},
    {'M', 121,   0,  66, "GM2 Var #00", "Tenor Sax"},
    {'M', 121,   0,  67, "GM2 Var #00", "Baritone Sax"},
    {'M', 121,   0,  68, "GM2 Var #00", "Oboe"},
    {'M', 121,   0,  69, "GM2 Var #00", "English Horn"},
    {'M', 121,   0,  70, "GM2 Var #00", "Bassoon"},
    {'M', 121,   0,  71, "GM2 Var #00", "Clarinet"},
    {'M', 121,   0,  72, "GM2 Var #00", "Piccolo"},
    {'M', 121,   0,  73, "GM2 Var #00", "Flute"},
    {'M', 121,   0,  74, "GM2 Var #00", "Recorder"},
    {'M', 121,   0,  75, "GM2 Var #00", "Pan Flute"},
    {'M', 121,   0,  76, "GM2 Var #00", "Blown Bottle"},
    {'M', 121,   0,  77, "GM2 Var #00", "Shakuhachi"},
    {'M', 121,   0,  78, "GM2 Var #00", "Whistle"},
    {'M', 121,   0,  79, "GM2 Var #00", "Ocarina"},
    {'M', 121,   0,  80, "GM2 Var #00", "Lead 1 (square)"},
    {'M', 121,   0,  81, "GM2 Var #00", "Lead 2 (sawtooth)"},
    {'M', 121,   0,  82, "GM2 Var #00", "Lead 3 (calliope)"},
    {'M', 121,   0,  83, "GM2 Var #00", "Lead 4 (chiff)"},
    {'M', 121,   0,  84, "GM2 Var #00", "Lead 5 (charang)"},
    {'M', 121,   0,  85, "GM2 Var #00", "Lead 6 (voice)"},
    {'M', 121,   0,  86, "GM2 Var #00", "Lead 7 (fifths)"},
    {'M', 121,   0,  87, "GM2 Var #00", "Lead 8 (bass + lead)"},
    {'M', 121,   0,  88, "GM2 Var #00", "Pad 1 (new age)"},
    {'M', 121,   0,  89, "GM2 Var #00", "Pad 2 (warm)"},
    {'M', 121,   0,  90, "GM2 Var #00", "Pad 3 (polysynth)"},
    {'M', 121,   0,  91, "GM2 Var #00", "Pad 4 (choir)"},
    {'M', 121,   0,  92, "GM2 Var #00", "Pad 5 (bowed)"},
    {'M', 121,   0,  93, "GM2 Var #00", "Pad 6 (metallic)"},
    {'M', 121,   0,  94, "GM2 Var #00", "Pad 7 (halo)"},
    {'M', 121,   0,  95, "GM2 Var #00", "Pad 8 (sweep)"},
    {'M', 121,   0,  96, "GM2 Var #00", "FX 1 (rain)"},
    {'M', 121,   0,  97, "GM2 Var #00", "FX 2 (soundtrack)"},
    {'M', 121,   0,  98, "GM2 Var #00", "FX 3 (crystal)"},
    {'M', 121,   0,  99, "GM2 Var #00", "FX 4 (atmosphere)"},
    {'M', 121,   0, 100, "GM2 Var #00", "FX 5 (brightness)"},
    {'M', 121,   0, 101, "GM2 Var #00", "FX 6 (goblins)"},
    {'M', 121,   0, 102, "GM2 Var #00", "FX 7 (echoes)"},
    {'M', 121,   0, 103, "GM2 Var #00", "FX 8 (sci-fi)"},
    {'M', 121,   0, 104, "GM2 Var #00", "Sitar"},
    {'M', 121,   0, 105, "GM2 Var #00", "Banjo"},
    {'M', 121,   0, 106, "GM2 Var #00", "Shamisen"},
    {'M', 121,   0, 107, "GM2 Var #00", "Koto"},
    {'M', 121,   0, 108, "GM2 Var #00", "Kalimba"},
    {'M', 121,   0, 109, "GM2 Var #00", "Bag pipe"},
    {'M', 121,   0, 110, "GM2 Var #00", "Fiddle"},
    {'M', 121,   0, 111, "GM2 Var #00", "Shanai"},
    {'M', 121,   0, 112, "GM2 Var #00", "Tinkle Bell"},
    {'M', 121,   0, 113, "GM2 Var #00", "Agogo"},
    {'M', 121,   0, 114, "GM2 Var #00", "Steel Drums"},
    {'M', 121,   0, 115, "GM2 Var #00", "Woodblock"},
    {'M', 121,   0, 116, "GM2 Var #00", "Taiko Drum"},
    {'M', 121,   0, 117, "GM2 Var #00", "Melodic Tom"},
    {'M', 121,   0, 118, "GM2 Var #00", "Synth Drum"},
    {'M', 121,   0, 119, "GM2 Var #00", "Reverse Cymbal"},
    {'M', 121,   0, 120, "GM2 Var #00", "Guitar Fret Noise"},
    {'M', 121,   0, 121, "GM2 Var #00", "Breath Noise"},
    {'M', 121,   0, 122, "GM2 Var #00", "Seashore"},
    {'M', 121,   0, 123, "GM2 Var #00", "Bird Tweet"},
    {'M', 121,   0, 124, "GM2 Var #00", "Telephone Ring"},
    {'M', 121,   0, 125, "GM2 Var #00", "Helicopter"},
    {'M', 121,   0, 126, "GM2 Var #00", "Applause"},
    {'M', 121,   0, 127, "GM2 Var #00", "Gunshot"},
    {'M', 121,   1,   0, "GM2 Var #01", "Acoustic Grand Piano (wide)"},
    {'M', 121,   1,   1, "GM2 Var #01", "Bright Acoustic Piano (wide)"},
    {'M', 121,   1,   2, "GM2 Var #01", "Electric Grand Piano (wide)"},
    {'M', 121,   1,   3, "GM2 Var #01", "Honky-tonk Piano (wide)"},
    {'M', 121,   1,   4, "GM2 Var #01", "Detuned Electric Piano 1"},
    {'M', 121,   1,   5, "GM2 Var #01", "Detuned Electric Piano 2"},
    {'M', 121,   1,   6, "GM2 Var #01", "Harpsichord (octave mix)"},
    {'M', 121,   1,   7, "GM2 Var #01", "Pulse Clavi"},
    {'M', 121,   1,  11, "GM2 Var #01", "Vibraphone (wide)"},
    {'M', 121,   1,  12, "GM2 Var #01", "Marimba (wide)"},
    {'M', 121,   1,  14, "GM2 Var #01", "Church Bell"},
    {'M', 121,   1,  16, "GM2 Var #01", "Detuned Drawbar Organ"},
    {'M', 121,   1,  17, "GM2 Var #01", "Detuned Percussive Organ"},
    {'M', 121,   1,  19, "GM2 Var #01", "Church Organ (octave mix)"},
    {'M', 121,   1,  20, "GM2 Var #01", "Puff Organ"},
    {'M', 121,   1,  21, "GM2 Var #01", "Accordion 2"},
    {'M', 121,   1,  24, "GM2 Var #01", "Ukulele"},
    {'M', 121,   1,  25, "GM2 Var #01", "12-Strings Guitar"},
    {'M', 121,   1,  26, "GM2 Var #01", "Electric Guitar (pedal steel)"},
    {'M', 121,   1,  27, "GM2 Var #01", "Electric Guitar (detuned clean)"},
    {'M', 121,   1,  28, "GM2 Var #01", "Electric Guitar (funky cutting)"},
    {'M', 121,   1,  29, "GM2 Var #01", "Guitar Pinch"},
    {'M', 121,   1,  30, "GM2 Var #01", "Distortion Guitar (with feedback)"},
    {'M', 121,   1,  31, "GM2 Var #01", "Guitar Feedback"},
    {'M', 121,   1,  33, "GM2 Var #01", "Finger Slap Bass"},
    {'M', 121,   1,  38, "GM2 Var #01", "Synth Bass (warm)"},
    {'M', 121,   1,  39, "GM2 Var #01", "Synth Bass 4 (attack)"},
    {'M', 121,   1,  40, "GM2 Var #01", "Violin (slow attack)"},
    {'M', 121,   1,  46, "GM2 Var #01", "Yang Chin"},
    {'M', 121,   1,  48, "GM2 Var #01", "Strings and Brass"},
    {'M', 121,   1,  50, "GM2 Var #01", "SynthStrings 3"},
    {'M', 121,   1,  52, "GM2 Var #01", "Choir Aahs 2"},
    {'M', 121,   1,  53, "GM2 Var #01", "Humming"},
    {'M', 121,   1,  54, "GM2 Var #01", "Analog Voice"},
    {'M', 121,   1,  55, "GM2 Var #01", "Bass Hit Plus"},
    {'M', 121,   1,  56, "GM2 Var #01", "Dark Trumpet Soft"},
    {'M', 121,   1,  57, "GM2 Var #01", "Trombone 2"},
    {'M', 121,   1,  59, "GM2 Var #01", "Muted Trumpet 2"},
    {'M', 121,   1,  60, "GM2 Var #01", "French Horn 2 (warm)"},
    {'M', 121,   1,  61, "GM2 Var #01", "Brass Section 2 (octave mix)"},
    {'M', 121,   1,  62, "GM2 Var #01", "Synth Brass 3"},
    {'M', 121,   1,  63, "GM2 Var #01", "Synth Brass 4"},
    {'M', 121,   1,  80, "GM2 Var #01", "Lead 1a (square 2)"},
    {'M', 121,   1,  81, "GM2 Var #01", "Lead 2a (sawtooth 2)"},
    {'M', 121,   1,  84, "GM2 Var #01", "Lead 5a (wire lead)"},
    {'M', 121,   1,  87, "GM2 Var #01", "Lead 8a (soft wrl)"},
    {'M', 121,   1,  89, "GM2 Var #01", "Pad 2a (sine pad)"},
    {'M', 121,   1,  91, "GM2 Var #01", "Pad 4a (itopia)"},
    {'M', 121,   1,  98, "GM2 Var #01", "FX 3a (synth mallet)"},
    {'M', 121,   1, 102, "GM2 Var #01", "FX 7a (echo bell)"},
    {'M', 121,   1, 104, "GM2 Var #01", "Sitar 2 (bend)"},
    {'M', 121,   1, 107, "GM2 Var #01", "Taisho Koto"},
    {'M', 121,   1, 115, "GM2 Var #01", "Castanets"},
    {'M', 121,   1, 116, "GM2 Var #01", "Concert Bass Drum"},
    {'M', 121,   1, 117, "GM2 Var #01", "Melodic Tom 2 (power)"},
    {'M', 121,   1, 118, "GM2 Var #01", "Rhythm Box Tom"},
    {'M', 121,   1, 120, "GM2 Var #01", "Guitar Cutting Noise"},
    {'M', 121,   1, 121, "GM2 Var #01", "Flute Key Click"},
    {'M', 121,   1, 122, "GM2 Var #01", "Rain"},
    {'M', 121,   1, 123, "GM2 Var #01", "Dog"},
    {'M', 121,   1, 124, "GM2 Var #01", "Telephone Ring 2"},
    {'M', 121,   1, 125, "GM2 Var #01", "Car Engine"},
    {'M', 121,   1, 126, "GM2 Var #01", "Laughing"},
    {'M', 121,   1, 127, "GM2 Var #01", "Machine Gun"},
    {'M', 121,   2,   0, "GM2 Var #02", "Acoustic Grand Piano (dark)"},
    {'M', 121,   2,   4, "GM2 Var #02", "Electric Piano 1 (velocity mix)"},
    {'M', 121,   2,   5, "GM2 Var #02", "Electric Piano 2 (velocity mix)"},
    {'M', 121,   2,   6, "GM2 Var #02", "Harpsichord (wide)"},
    {'M', 121,   2,  14, "GM2 Var #02", "Carillon"},
    {'M', 121,   2,  16, "GM2 Var #02", "Italian 60's Organ"},
    {'M', 121,   2,  17, "GM2 Var #02", "Percussive Organ 2"},
    {'M', 121,   2,  19, "GM2 Var #02", "Detuned Church Organ"},
    {'M', 121,   2,  24, "GM2 Var #02", "Acoustic Guitar (nylon + key off)"},
    {'M', 121,   2,  25, "GM2 Var #02", "Mandolin"},
    {'M', 121,   2,  27, "GM2 Var #02", "Mid Tone Guitar"},
    {'M', 121,   2,  28, "GM2 Var #02", "Electric Guitar (muted velo-sw)"},
    {'M', 121,   2,  30, "GM2 Var #02", "Distorted Rhythm Guitar"},
    {'M', 121,   2,  38, "GM2 Var #02", "Synth Bass 3 (resonance)"},
    {'M', 121,   2,  39, "GM2 Var #02", "Synth Bass (rubber)"},
    {'M', 121,   2,  48, "GM2 Var #02", "60s Strings"},
    {'M', 121,   2,  55, "GM2 Var #02", "6th Hit"},
    {'M', 121,   2,  57, "GM2 Var #02", "Bright Trombone"},
    {'M', 121,   2,  62, "GM2 Var #02", "Analog Synth Brass 1"},
    {'M', 121,   2,  63, "GM2 Var #02", "Analog Synth Brass 2"},
    {'M', 121,   2,  80, "GM2 Var #02", "Lead 1b (sine)"},
    {'M', 121,   2,  81, "GM2 Var #02", "Lead 2b (saw + pulse)"},
    {'M', 121,   2, 102, "GM2 Var #02", "FX 7b (echo pan)"},
    {'M', 121,   2, 118, "GM2 Var #02", "Electric Drum"},
    {'M', 121,   2, 120, "GM2 Var #02", "Acoustic Bass String Slap"},
    {'M', 121,   2, 122, "GM2 Var #02", "Thunder"},
    {'M', 121,   2, 123, "GM2 Var #02", "Horse Gallop"},
    {'M', 121,   2, 124, "GM2 Var #02", "Door Creaking"},
    {'M', 121,   2, 125, "GM2 Var #02", "Car Stop"},
    {'M', 121,   2, 126, "GM2 Var #02", "Screaming"},
    {'M', 121,   2, 127, "GM2 Var #02", "Lasergun"},
    {'M', 121,   3,   4, "GM2 Var #03", "60's Electric Piano"},
    {'M', 121,   3,   5, "GM2 Var #03", "EP Legend"},
    {'M', 121,   3,   6, "GM2 Var #03", "Harpsichord (with key off)"},
    {'M', 121,   3,  16, "GM2 Var #03", "Drawbar Organ 2"},
    {'M', 121,   3,  24, "GM2 Var #03", "Acoustic Guitar (nylon 2)"},
    {'M', 121,   3,  25, "GM2 Var #03", "Steel Guitar with Body Sound"},
    {'M', 121,   3,  28, "GM2 Var #03", "Jazz Man"},
    {'M', 121,   3,  38, "GM2 Var #03", "Clavi Bass"},
    {'M', 121,   3,  39, "GM2 Var #03", "Attack Pulse"},
    {'M', 121,   3,  55, "GM2 Var #03", "Euro Hit"},
    {'M', 121,   3,  62, "GM2 Var #03", "Jump Brass"},
    {'M', 121,   3,  81, "GM2 Var #03", "Lead 2c (double sawtooth)"},
    {'M', 121,   3, 122, "GM2 Var #03", "Wind"},
    {'M', 121,   3, 123, "GM2 Var #03", "Bird Tweet 2"},
    {'M', 121,   3, 124, "GM2 Var #03", "Door"},
    {'M', 121,   3, 125, "GM2 Var #03", "Car Pass"},
    {'M', 121,   3, 126, "GM2 Var #03", "Punch"},
    {'M', 121,   3, 127, "GM2 Var #03", "Explosion"},
    {'M', 121,   4,   5, "GM2 Var #04", "EP Phase"},
    {'M', 121,   4,  38, "GM2 Var #04", "Hammer"},
    {'M', 121,   4,  81, "GM2 Var #04", "Lead 2d (sequenced analog)"},
    {'M', 121,   4, 122, "GM2 Var #04", "Stream"},
    {'M', 121,   4, 124, "GM2 Var #04", "Scratch"},
    {'M', 121,   4, 125, "GM2 Var #04", "Car Crash"},
    {'M', 121,   4, 126, "GM2 Var #04", "Heart Beat"},
    {'M', 121,   5, 122, "GM2 Var #05", "Bubble"},
    {'M', 121,   5, 124, "GM2 Var #05", "Wind Chime"},
    {'M', 121,   5, 125, "GM2 Var #05", "Siren"},
    {'M', 121,   5, 126, "GM2 Var #05", "Footsteps"},
    {'M', 121,   6, 125, "GM2 Var #06", "Train"},
    {'M', 121,   7, 125, "GM2 Var #07", "Jetplane"},
    {'M', 121,   8, 125, "GM2 Var #08", "Starship"},
    {'M', 121,   9, 125, "GM2 Var #09", "Burst Noise"},
    {'P', 120,   0,  27, "GM2 STANDARD Set", "High Q"},
    {'P', 120,   0,  28, "GM2 STANDARD Set", "Slap"},
    {'P', 120,   0,  29, "GM2 STANDARD Set", "Scratch Push"},
    {'P', 120,   0,  30, "GM2 STANDARD Set", "Scratch Pull"},
    {'P', 120,   0,  31, "GM2 STANDARD Set", "Sticks"},
    {'P', 120,   0,  32, "GM2 STANDARD Set", "Square Click"},
    {'P', 120,   0,  33, "GM2 STANDARD Set", "Metronome Click"},
    {'P', 120,   0,  34, "GM2 STANDARD Set", "Metronome Bell"},
    {'P', 120,   0,  35, "GM2 STANDARD Set", "Acoustic Bass Drum"},
    {'P', 120,   0,  36, "GM2 STANDARD Set", "Bass Drum 1"},
    {'P', 120,   0,  37, "GM2 STANDARD Set", "Side Stick"},
    {'P', 120,   0,  38, "GM2 STANDARD Set", "Acoustic Snare"},
    {'P', 120,   0,  39, "GM2 STANDARD Set", "Hand Clap"},
    {'P', 120,   0,  40, "GM2 STANDARD Set", "Electric Snare"},
    {'P', 120,   0,  41, "GM2 STANDARD Set", "Low Floor Tom"},
    {'P', 120,   0,  42, "GM2 STANDARD Set", "Closed Hi-hat"},
    {'P', 120,   0,  43, "GM2 STANDARD Set", "High Floor Tom"},
    {'P', 120,   0,  44, "GM2 STANDARD Set", "Pedal Hi-hat"},
    {'P', 120,   0,  45, "GM2 STANDARD Set", "Low Tom"},
    {'P', 120,   0,  46, "GM2 STANDARD Set", "Open Hi-hat"},
    {'P', 120,   0,  47, "GM2 STANDARD Set", "Low-Mid Tom"},
    {'P', 120,   0,  48, "GM2 STANDARD Set", "High Mid Tom"},
    {'P', 120,   0,  49, "GM2 STANDARD Set", "Crash Cymbal 1"},
    {'P', 120,   0,  50, "GM2 STANDARD Set", "High Tom"},
    {'P', 120,   0,  51, "GM2 STANDARD Set", "Ride Cymbal 1"},
    {'P', 120,   0,  52, "GM2 STANDARD Set", "Chinese Cymbal"},
    {'P', 120,   0,  53, "GM2 STANDARD Set", "Ride Bell"},
    {'P', 120,   0,  54, "GM2 STANDARD Set", "Tambourine"},
    {'P', 120,   0,  55, "GM2 STANDARD Set", "Splash Cymbal"},
    {'P', 120,   0,  56, "GM2 STANDARD Set", "Cowbell"},
    {'P', 120,   0,  57, "GM2 STANDARD Set", "Crash Cymbal 2"},
    {'P', 120,   0,  58, "GM2 STANDARD Set", "Vibra-slap"},
    {'P', 120,   0,  59, "GM2 STANDARD Set", "Ride Cymbal 2"},
    {'P', 120,   0,  60, "GM2 STANDARD Set", "High Bongo"},
    {'P', 120,   0,  61, "GM2 STANDARD Set", "Low Bongo"},
    {'P', 120,   0,  62, "GM2 STANDARD Set", "Mute Hi Conga"},
    {'P', 120,   0,  63, "GM2 STANDARD Set", "Open Hi Conga"},
    {'P', 120,   0,  64, "GM2 STANDARD Set", "Low Conga"},
    {'P', 120,   0,  65, "GM2 STANDARD Set", "High Timbale"},
    {'P', 120,   0,  66, "GM2 STANDARD Set", "Low Timbale"},
    {'P', 120,   0,  67, "GM2 STANDARD Set", "High Agogo"},
    {'P', 120,   0,  68, "GM2 STANDARD Set", "Low Agogo"},
    {'P', 120,   0,  69, "GM2 STANDARD Set", "Cabasa"},
    {'P', 120,   0,  70, "GM2 STANDARD Set", "Maracas"},
    {'P', 120,   0,  71, "GM2 STANDARD Set", "Short Whistle"},
    {'P', 120,   0,  72, "GM2 STANDARD Set", "Long Whistle"},
    {'P', 120,   0,  73, "GM2 STANDARD Set", "Short Guiro"},
    {'P', 120,   0,  74, "GM2 STANDARD Set", "Long Guiro"},
    {'P', 120,   0,  75, "GM2 STANDARD Set", "Claves"},
    {'P', 120,   0,  76, "GM2 STANDARD Set", "Hi Wood Block"},
    {'P', 120,   0,  77, "GM2 STANDARD Set", "Low Wood Block"},
    {'P', 120,   0,  78, "GM2 STANDARD Set", "Mute Cuica"},
    {'P', 120,   0,  79, "GM2 STANDARD Set", "Open Cuica"},
    {'P', 120,   0,  80, "GM2 STANDARD Set", "Mute Triangle"},
    {'P', 120,   0,  81, "GM2 STANDARD Set", "Open Triangle"},
    {'P', 120,   0,  82, "GM2 STANDARD Set", "Shaker"},
    {'P', 120,   0,  83, "GM2 STANDARD Set", "Jingle Bell"},
    {'P', 120,   0,  84, "GM2 STANDARD Set", "Bell Tree"},
    {'P', 120,   0,  85, "GM2 STANDARD Set", "Castanets"},
    {'P', 120,   0,  86, "GM2 STANDARD Set", "Mute Surdo"},
    {'P', 120,   0,  87, "GM2 STANDARD Set", "Open Surdo"},
    {'P', 120,   8,  41, "GM2 ROOM Set", "Room Low Tom 2"},
    {'P', 120,   8,  43, "GM2 ROOM Set", "Room Low Tom 1"},
    {'P', 120,   8,  45, "GM2 ROOM Set", "Room Mid Tom 2"},
    {'P', 120,   8,  47, "GM2 ROOM Set", "Room Mid Tom 1"},
    {'P', 120,   8,  48, "GM2 ROOM Set", "Room Hi Tom 2"},
    {'P', 120,   8,  50, "GM2 ROOM Set", "Room Hi Tom 1"},
    {'P', 120,  16,  36, "GM2 POWER Set", "Power Kick Drum"},
    {'P', 120,  16,  38, "GM2 POWER Set", "Power Snare Drum"},
    {'P', 120,  16,  41, "GM2 POWER Set", "Power Low Tom 2"},
    {'P', 120,  16,  43, "GM2 POWER Set", "Power Low Tom 1"},
    {'P', 120,  16,  45, "GM2 POWER Set", "Power Mid Tom 2"},
    {'P', 120,  16,  47, "GM2 POWER Set", "Power Mid Tom 1"},
    {'P', 120,  16,  48, "GM2 POWER Set", "Power Hi Tom 2"},
    {'P', 120,  16,  50, "GM2 POWER Set", "Power Hi Tom 1"},
    {'P', 120,  24,  36, "GM2 ELECTRONIC Set", "Electric Bass Drum"},
    {'P', 120,  24,  38, "GM2 ELECTRONIC Set", "Electric Snare 1"},
    {'P', 120,  24,  40, "GM2 ELECTRONIC Set", "Electric Snare 2"},
    {'P', 120,  24,  41, "GM2 ELECTRONIC Set", "Electric Low Tom 2"},
    {'P', 120,  24,  43, "GM2 ELECTRONIC Set", "Electric Low Tom 1"},
    {'P', 120,  24,  45, "GM2 ELECTRONIC Set", "Electric Mid Tom 2"},
    {'P', 120,  24,  47, "GM2 ELECTRONIC Set", "Electric Mid Tom 1"},
    {'P', 120,  24,  48, "GM2 ELECTRONIC Set", "Electric Hi Tom 2"},
    {'P', 120,  24,  50, "GM2 ELECTRONIC Set", "Electric Hi Tom 1"},
    {'P', 120,  24,  52, "GM2 ELECTRONIC Set", "Reverse Cymbal"},
    {'P', 120,  25,  36, "GM2 ANALOG Set", "Analog Bass Drum"},
    {'P', 120,  25,  37, "GM2 ANALOG Set", "Analog Rim Shot"},
    {'P', 120,  25,  38, "GM2 ANALOG Set", "Analog Snare 1"},
    {'P', 120,  25,  41, "GM2 ANALOG Set", "Analog Low Tom 2"},
    {'P', 120,  25,  42, "GM2 ANALOG Set", "Analog CHH 1"},
    {'P', 120,  25,  43, "GM2 ANALOG Set", "Analog Low Tom 1"},
    {'P', 120,  25,  44, "GM2 ANALOG Set", "Analog CHH 2"},
    {'P', 120,  25,  45, "GM2 ANALOG Set", "Analog Mid Tom 2"},
    {'P', 120,  25,  46, "GM2 ANALOG Set", "Analog OHH"},
    {'P', 120,  25,  47, "GM2 ANALOG Set", "Analog Mid Tom 1"},
    {'P', 120,  25,  48, "GM2 ANALOG Set", "Analog Hi Tom 2"},
    {'P', 120,  25,  49, "GM2 ANALOG Set", "Analog Cymbal"},
    {'P', 120,  25,  50, "GM2 ANALOG Set", "Analog Hi Tom 1"},
    {'P', 120,  25,  56, "GM2 ANALOG Set", "Analog Cowbell"},
    {'P', 120,  25,  62, "GM2 ANALOG Set", "Analog High Conga"},
    {'P', 120,  25,  63, "GM2 ANALOG Set", "Analog Mid Conga"},
    {'P', 120,  25,  64, "GM2 ANALOG Set", "Analog Low Conga"},
    {'P', 120,  25,  70, "GM2 ANALOG Set", "Analog Maracas"},
    {'P', 120,  25,  75, "GM2 ANALOG Set", "Analog Claves"},
    {'P', 120,  32,  35, "GM2 JAZZ Set", "Jazz Kick 2"},
    {'P', 120,  32,  36, "GM2 JAZZ Set", "Jazz Kick 1"},
    {'P', 120,  40,  35, "GM2 BRUSH Set", "Jazz Kick 2"},
    {'P', 120,  40,  36, "GM2 BRUSH Set", "Jazz Kick 1"},
    {'P', 120,  40,  38, "GM2 BRUSH Set", "Brush Tap"},
    {'P', 120,  40,  39, "GM2 BRUSH Set", "Brush Slap"},
    {'P', 120,  40,  40, "GM2 BRUSH Set", "Brush Swirl"},
    {'P', 120,  48,  27, "GM2 ORCHESTRA Set", "Closed Hi-hat 2"},
    {'P', 120,  48,  28, "GM2 ORCHESTRA Set", "Pedal Hi-hat"},
    {'P', 120,  48,  29, "GM2 ORCHESTRA Set", "Open Hi-hat 2"},
    {'P', 120,  48,  30, "GM2 ORCHESTRA Set", "Ride Cymbal 1"},
    {'P', 120,  48,  35, "GM2 ORCHESTRA Set", "Concert BD 2"},
    {'P', 120,  48,  36, "GM2 ORCHESTRA Set", "Concert BD 1"},
    {'P', 120,  48,  38, "GM2 ORCHESTRA Set", "Concert SD"},
    {'P', 120,  48,  39, "GM2 ORCHESTRA Set", "Castanets"},
    {'P', 120,  48,  40, "GM2 ORCHESTRA Set", "Concert SD"},
    {'P', 120,  48,  41, "GM2 ORCHESTRA Set", "Timpani F"},
    {'P', 120,  48,  42, "GM2 ORCHESTRA Set", "Timpani F#"},
    {'P', 120,  48,  43, "GM2 ORCHESTRA Set", "Timpani G"},
    {'P', 120,  48,  44, "GM2 ORCHESTRA Set", "Timpani G#"},
    {'P', 120,  48,  45, "GM2 ORCHESTRA Set", "Timpani A"},
    {'P', 120,  48,  46, "GM2 ORCHESTRA Set", "Timpani A#"},
    {'P', 120,  48,  47, "GM2 ORCHESTRA Set", "Timpani B"},
    {'P', 120,  48,  48, "GM2 ORCHESTRA Set", "Timpani c"},
    {'P', 120,  48,  49, "GM2 ORCHESTRA Set", "Timpani c#"},
    {'P', 120,  48,  50, "GM2 ORCHESTRA Set", "Timpani d"},
    {'P', 120,  48,  51, "GM2 ORCHESTRA Set", "Timpani d#"},
    {'P', 120,  48,  52, "GM2 ORCHESTRA Set", "Timpani e"},
    {'P', 120,  48,  53, "GM2 ORCHESTRA Set", "Timpani f"},
    {'P', 120,  48,  57, "GM2 ORCHESTRA Set", "Concert Cymbal 2"},
    {'P', 120,  48,  59, "GM2 ORCHESTRA Set", "Concert Cymbal 1"},
    {'P', 120,  48,  88, "GM2 ORCHESTRA Set", "Applause"},
    {'P', 120,  56,  39, "GM2 SFX Set", "High Q"},
    {'P', 120,  56,  40, "GM2 SFX Set", "Slap"},
    {'P', 120,  56,  41, "GM2 SFX Set", "Scratch Push"},
    {'P', 120,  56,  42, "GM2 SFX Set", "Scratch Pull"},
    {'P', 120,  56,  43, "GM2 SFX Set", "Sticks"},
    {'P', 120,  56,  44, "GM2 SFX Set", "Square Click"},
    {'P', 120,  56,  45, "GM2 SFX Set", "Metronome Click"},
    {'P', 120,  56,  46, "GM2 SFX Set", "Metronome Bell"},
    {'P', 120,  56,  47, "GM2 SFX Set", "Guitar Fret Noise"},
    {'P', 120,  56,  48, "GM2 SFX Set", "Guitar Cutting Noise Up"},
    {'P', 120,  56,  49, "GM2 SFX Set", "Guitar Cutting Noise Down"},
    {'P', 120,  56,  50, "GM2 SFX Set", "String Slap of Double Bass"},
    {'P', 120,  56,  51, "GM2 SFX Set", "Fl.Key Click"},
    {'P', 120,  56,  52, "GM2 SFX Set", "Laughing"},
    {'P', 120,  56,  53, "GM2 SFX Set", "Scream"},
    {'P', 120,  56,  54, "GM2 SFX Set", "Punch"},
    {'P', 120,  56,  55, "GM2 SFX Set", "Heart Beat"},
    {'P', 120,  56,  56, "GM2 SFX Set", "Footsteps 1"},
    {'P', 120,  56,  57, "GM2 SFX Set", "Footsteps 2"},
    {'P', 120,  56,  58, "GM2 SFX Set", "Applause"},
    {'P', 120,  56,  59, "GM2 SFX Set", "Door Creaking"},
    {'P', 120,  56,  60, "GM2 SFX Set", "Door"},
    {'P', 120,  56,  61, "GM2 SFX Set", "Scratch"},
    {'P', 120,  56,  62, "GM2 SFX Set", "Wind Chimes"},
    {'P', 120,  56,  63, "GM2 SFX Set", "Car-Engine"},
    {'P', 120,  56,  64, "GM2 SFX Set", "Car-Stop"},
    {'P', 120,  56,  65, "GM2 SFX Set", "Car-Pass"},
    {'P', 120,  56,  66, "GM2 SFX Set", "Car-Crash"},
    {'P', 120,  56,  67, "GM2 SFX Set", "Siren"},
    {'P', 120,  56,  68, "GM2 SFX Set", "Train"},
    {'P', 120,  56,  69, "GM2 SFX Set", "Jetplane"},
    {'P', 120,  56,  70, "GM2 SFX Set", "Helicopter"},
    {'P', 120,  56,  71, "GM2 SFX Set", "Starship"},
    {'P', 120,  56,  72, "GM2 SFX Set", "Gun Shot"},
    {'P', 120,  56,  73, "GM2 SFX Set", "Machine Gun"},
    {'P', 120,  56,  74, "GM2 SFX Set", "Lasergun"},
    {'P', 120,  56,  75, "GM2 SFX Set", "Explosion"},
    {'P', 120,  56,  76, "GM2 SFX Set", "Dog"},
    {'P', 120,  56,  77, "GM2 SFX Set", "Horse-Gallop"},
    {'P', 120,  56,  78, "GM2 SFX Set", "Birds"},
    {'P', 120,  56,  79, "GM2 SFX Set", "Rain"},
    {'P', 120,  56,  80, "GM2 SFX Set", "Thunder"},
    {'P', 120,  56,  81, "GM2 SFX Set", "Wind"},
    {'P', 120,  56,  82, "GM2 SFX Set", "Seashore"},
    {'P', 120,  56,  83, "GM2 SFX Set", "Stream"},
    {'P', 120,  56,  84, "GM2 SFX Set", "Bubble"},
};

static const MidiProgram Gm1Set[] =
{
    {'M',   0,   0,   0, "GM1 Melodic", "Acoustic Grand Piano"},
    {'M',   0,   0,   1, "GM1 Melodic", "Bright Acoustic Piano"},
    {'M',   0,   0,   2, "GM1 Melodic", "Electric Grand Piano"},
    {'M',   0,   0,   3, "GM1 Melodic", "Honky-tonk Piano"},
    {'M',   0,   0,   4, "GM1 Melodic", "Electric Piano 1"},
    {'M',   0,   0,   5, "GM1 Melodic", "Electric Piano 2"},
    {'M',   0,   0,   6, "GM1 Melodic", "Harpsichord"},
    {'M',   0,   0,   7, "GM1 Melodic", "Clavi"},
    {'M',   0,   0,   8, "GM1 Melodic", "Celesta"},
    {'M',   0,   0,   9, "GM1 Melodic", "Glockenspiel"},
    {'M',   0,   0,  10, "GM1 Melodic", "Music Box"},
    {'M',   0,   0,  11, "GM1 Melodic", "Vibraphone"},
    {'M',   0,   0,  12, "GM1 Melodic", "Marimba"},
    {'M',   0,   0,  13, "GM1 Melodic", "Xylophone"},
    {'M',   0,   0,  14, "GM1 Melodic", "Tubular Bells"},
    {'M',   0,   0,  15, "GM1 Melodic", "Dulcimer"},
    {'M',   0,   0,  16, "GM1 Melodic", "Drawbar Organ"},
    {'M',   0,   0,  17, "GM1 Melodic", "Percussive Organ"},
    {'M',   0,   0,  18, "GM1 Melodic", "Rock Organ"},
    {'M',   0,   0,  19, "GM1 Melodic", "Church Organ"},
    {'M',   0,   0,  20, "GM1 Melodic", "Reed Organ"},
    {'M',   0,   0,  21, "GM1 Melodic", "Accordion"},
    {'M',   0,   0,  22, "GM1 Melodic", "Harmonica"},
    {'M',   0,   0,  23, "GM1 Melodic", "Tango Accordion"},
    {'M',   0,   0,  24, "GM1 Melodic", "Acoustic Guitar (nylon)"},
    {'M',   0,   0,  25, "GM1 Melodic", "Acoustic Guitar (steel)"},
    {'M',   0,   0,  26, "GM1 Melodic", "Electric Guitar (jazz)"},
    {'M',   0,   0,  27, "GM1 Melodic", "Electric Guitar (clean)"},
    {'M',   0,   0,  28, "GM1 Melodic", "Electric Guitar (muted)"},
    {'M',   0,   0,  29, "GM1 Melodic", "Overdriven Guitar"},
    {'M',   0,   0,  30, "GM1 Melodic", "Distortion Guitar"},
    {'M',   0,   0,  31, "GM1 Melodic", "Guitar Harmonics"},
    {'M',   0,   0,  32, "GM1 Melodic", "Acoustic Bass"},
    {'M',   0,   0,  33, "GM1 Melodic", "Electric Bass (finger)"},
    {'M',   0,   0,  34, "GM1 Melodic", "Electric Bass (pick)"},
    {'M',   0,   0,  35, "GM1 Melodic", "Fretless Bass"},
    {'M',   0,   0,  36, "GM1 Melodic", "Slap Bass 1"},
    {'M',   0,   0,  37, "GM1 Melodic", "Slap Bass 2"},
    {'M',   0,   0,  38, "GM1 Melodic", "Synth Bass 1"},
    {'M',   0,   0,  39, "GM1 Melodic", "Synth Bass 2"},
    {'M',   0,   0,  40, "GM1 Melodic", "Violin"},
    {'M',   0,   0,  41, "GM1 Melodic", "Viola"},
    {'M',   0,   0,  42, "GM1 Melodic", "Cello"},
    {'M',   0,   0,  43, "GM1 Melodic", "Contrabass"},
    {'M',   0,   0,  44, "GM1 Melodic", "Tremolo Strings"},
    {'M',   0,   0,  45, "GM1 Melodic", "Pizzicato Strings"},
    {'M',   0,   0,  46, "GM1 Melodic", "Orchestral Harp"},
    {'M',   0,   0,  47, "GM1 Melodic", "Timpani"},
    {'M',   0,   0,  48, "GM1 Melodic", "String Ensembles 1"},
    {'M',   0,   0,  49, "GM1 Melodic", "String Ensembles 2"},
    {'M',   0,   0,  50, "GM1 Melodic", "SynthStrings 1"},
    {'M',   0,   0,  51, "GM1 Melodic", "SynthStrings 2"},
    {'M',   0,   0,  52, "GM1 Melodic", "Choir Aahs"},
    {'M',   0,   0,  53, "GM1 Melodic", "Voice Oohs"},
    {'M',   0,   0,  54, "GM1 Melodic", "Synth Voice"},
    {'M',   0,   0,  55, "GM1 Melodic", "Orchestra Hit"},
    {'M',   0,   0,  56, "GM1 Melodic", "Trumpet"},
    {'M',   0,   0,  57, "GM1 Melodic", "Trombone"},
    {'M',   0,   0,  58, "GM1 Melodic", "Tuba"},
    {'M',   0,   0,  59, "GM1 Melodic", "Muted Trumpet"},
    {'M',   0,   0,  60, "GM1 Melodic", "French Horn"},
    {'M',   0,   0,  61, "GM1 Melodic", "Brass Section"},
    {'M',   0,   0,  62, "GM1 Melodic", "Synth Brass 1"},
    {'M',   0,   0,  63, "GM1 Melodic", "Synth Brass 2"},
    {'M',   0,   0,  64, "GM1 Melodic", "Soprano Sax"},
    {'M',   0,   0,  65, "GM1 Melodic", "Alto Sax"},
    {'M',   0,   0,  66, "GM1 Melodic", "Tenor Sax"},
    {'M',   0,   0,  67, "GM1 Melodic", "Baritone Sax"},
    {'M',   0,   0,  68, "GM1 Melodic", "Oboe"},
    {'M',   0,   0,  69, "GM1 Melodic", "English Horn"},
    {'M',   0,   0,  70, "GM1 Melodic", "Bassoon"},
    {'M',   0,   0,  71, "GM1 Melodic", "Clarinet"},
    {'M',   0,   0,  72, "GM1 Melodic", "Piccolo"},
    {'M',   0,   0,  73, "GM1 Melodic", "Flute"},
    {'M',   0,   0,  74, "GM1 Melodic", "Recorder"},
    {'M',   0,   0,  75, "GM1 Melodic", "Pan Flute"},
    {'M',   0,   0,  76, "GM1 Melodic", "Blown Bottle"},
    {'M',   0,   0,  77, "GM1 Melodic", "Shakuhachi"},
    {'M',   0,   0,  78, "GM1 Melodic", "Whistle"},
    {'M',   0,   0,  79, "GM1 Melodic", "Ocarina"},
    {'M',   0,   0,  80, "GM1 Melodic", "Lead 1 (square)"},
    {'M',   0,   0,  81, "GM1 Melodic", "Lead 2 (sawtooth)"},
    {'M',   0,   0,  82, "GM1 Melodic", "Lead 3 (calliope)"},
    {'M',   0,   0,  83, "GM1 Melodic", "Lead 4 (chiff)"},
    {'M',   0,   0,  84, "GM1 Melodic", "Lead 5 (charang)"},
    {'M',   0,   0,  85, "GM1 Melodic", "Lead 6 (voice)"},
    {'M',   0,   0,  86, "GM1 Melodic", "Lead 7 (fifths)"},
    {'M',   0,   0,  87, "GM1 Melodic", "Lead 8 (bass + lead)"},
    {'M',   0,   0,  88, "GM1 Melodic", "Pad 1 (new age)"},
    {'M',   0,   0,  89, "GM1 Melodic", "Pad 2 (warm)"},
    {'M',   0,   0,  90, "GM1 Melodic", "Pad 3 (polysynth)"},
    {'M',   0,   0,  91, "GM1 Melodic", "Pad 4 (choir)"},
    {'M',   0,   0,  92, "GM1 Melodic", "Pad 5 (bowed)"},
    {'M',   0,   0,  93, "GM1 Melodic", "Pad 6 (metallic)"},
    {'M',   0,   0,  94, "GM1 Melodic", "Pad 7 (halo)"},
    {'M',   0,   0,  95, "GM1 Melodic", "Pad 8 (sweep)"},
    {'M',   0,   0,  96, "GM1 Melodic", "FX 1 (rain)"},
    {'M',   0,   0,  97, "GM1 Melodic", "FX 2 (soundtrack)"},
    {'M',   0,   0,  98, "GM1 Melodic", "FX 3 (crystal)"},
    {'M',   0,   0,  99, "GM1 Melodic", "FX 4 (atmosphere)"},
    {'M',   0,   0, 100, "GM1 Melodic", "FX 5 (brightness)"},
    {'M',   0,   0, 101, "GM1 Melodic", "FX 6 (goblins)"},
    {'M',   0,   0, 102, "GM1 Melodic", "FX 7 (echoes)"},
    {'M',   0,   0, 103, "GM1 Melodic", "FX 8 (sci-fi)"},
    {'M',   0,   0, 104, "GM1 Melodic", "Sitar"},
    {'M',   0,   0, 105, "GM1 Melodic", "Banjo"},
    {'M',   0,   0, 106, "GM1 Melodic", "Shamisen"},
    {'M',   0,   0, 107, "GM1 Melodic", "Koto"},
    {'M',   0,   0, 108, "GM1 Melodic", "Kalimba"},
    {'M',   0,   0, 109, "GM1 Melodic", "Bag pipe"},
    {'M',   0,   0, 110, "GM1 Melodic", "Fiddle"},
    {'M',   0,   0, 111, "GM1 Melodic", "Shanai"},
    {'M',   0,   0, 112, "GM1 Melodic", "Tinkle Bell"},
    {'M',   0,   0, 113, "GM1 Melodic", "Agogo"},
    {'M',   0,   0, 114, "GM1 Melodic", "Steel Drums"},
    {'M',   0,   0, 115, "GM1 Melodic", "Woodblock"},
    {'M',   0,   0, 116, "GM1 Melodic", "Taiko Drum"},
    {'M',   0,   0, 117, "GM1 Melodic", "Melodic Tom"},
    {'M',   0,   0, 118, "GM1 Melodic", "Synth Drum"},
    {'M',   0,   0, 119, "GM1 Melodic", "Reverse Cymbal"},
    {'M',   0,   0, 120, "GM1 Melodic", "Guitar Fret Noise"},
    {'M',   0,   0, 121, "GM1 Melodic", "Breath Noise"},
    {'M',   0,   0, 122, "GM1 Melodic", "Seashore"},
    {'M',   0,   0, 123, "GM1 Melodic", "Bird Tweet"},
    {'M',   0,   0, 124, "GM1 Melodic", "Telephone Ring"},
    {'M',   0,   0, 125, "GM1 Melodic", "Helicopter"},
    {'M',   0,   0, 126, "GM1 Melodic", "Applause"},
    {'M',   0,   0, 127, "GM1 Melodic", "Gunshot"},
    {'P',   0,   0,  35, "GM1 Percussion", "Acoustic Bass Drum"},
    {'P',   0,   0,  36, "GM1 Percussion", "Bass Drum 1"},
    {'P',   0,   0,  37, "GM1 Percussion", "Side Stick"},
    {'P',   0,   0,  38, "GM1 Percussion", "Acoustic Snare"},
    {'P',   0,   0,  39, "GM1 Percussion", "Hand Clap"},
    {'P',   0,   0,  40, "GM1 Percussion", "Electric Snare"},
    {'P',   0,   0,  41, "GM1 Percussion", "Low Floor Tom"},
    {'P',   0,   0,  42, "GM1 Percussion", "Closed Hi-hat"},
    {'P',   0,   0,  43, "GM1 Percussion", "High Floor Tom"},
    {'P',   0,   0,  44, "GM1 Percussion", "Pedal Hi-hat"},
    {'P',   0,   0,  45, "GM1 Percussion", "Low Tom"},
    {'P',   0,   0,  46, "GM1 Percussion", "Open Hi-hat"},
    {'P',   0,   0,  47, "GM1 Percussion", "Low-Mid Tom"},
    {'P',   0,   0,  48, "GM1 Percussion", "High Mid Tom"},
    {'P',   0,   0,  49, "GM1 Percussion", "Crash Cymbal 1"},
    {'P',   0,   0,  50, "GM1 Percussion", "High Tom"},
    {'P',   0,   0,  51, "GM1 Percussion", "Ride Cymbal 1"},
    {'P',   0,   0,  52, "GM1 Percussion", "Chinese Cymbal"},
    {'P',   0,   0,  53, "GM1 Percussion", "Ride Bell"},
    {'P',   0,   0,  54, "GM1 Percussion", "Tambourine"},
    {'P',   0,   0,  55, "GM1 Percussion", "Splash Cymbal"},
    {'P',   0,   0,  56, "GM1 Percussion", "Cowbell"},
    {'P',   0,   0,  57, "GM1 Percussion", "Crash Cymbal 2"},
    {'P',   0,   0,  58, "GM1 Percussion", "Vibra-slap"},
    {'P',   0,   0,  59, "GM1 Percussion", "Ride Cymbal 2"},
    {'P',   0,   0,  60, "GM1 Percussion", "High Bongo"},
    {'P',   0,   0,  61, "GM1 Percussion", "Low Bongo"},
    {'P',   0,   0,  62, "GM1 Percussion", "Mute Hi Conga"},
    {'P',   0,   0,  63, "GM1 Percussion", "Open Hi Conga"},
    {'P',   0,   0,  64, "GM1 Percussion", "Low Conga"},
    {'P',   0,   0,  65, "GM1 Percussion", "High Timbale"},
    {'P',   0,   0,  66, "GM1 Percussion", "Low Timbale"},
    {'P',   0,   0,  67, "GM1 Percussion", "High Agogo"},
    {'P',   0,   0,  68, "GM1 Percussion", "Low Agogo"},
    {'P',   0,   0,  69, "GM1 Percussion", "Cabasa"},
    {'P',   0,   0,  70, "GM1 Percussion", "Maracas"},
    {'P',   0,   0,  71, "GM1 Percussion", "Short Whistle"},
    {'P',   0,   0,  72, "GM1 Percussion", "Long Whistle"},
    {'P',   0,   0,  73, "GM1 Percussion", "Short Guiro"},
    {'P',   0,   0,  74, "GM1 Percussion", "Long Guiro"},
    {'P',   0,   0,  75, "GM1 Percussion", "Claves"},
    {'P',   0,   0,  76, "GM1 Percussion", "Hi Wood Block"},
    {'P',   0,   0,  77, "GM1 Percussion", "Low Wood Block"},
    {'P',   0,   0,  78, "GM1 Percussion", "Mute Cuica"},
    {'P',   0,   0,  79, "GM1 Percussion", "Open Cuica"},
    {'P',   0,   0,  80, "GM1 Percussion", "Mute Triangle"},
    {'P',   0,   0,  81, "GM1 Percussion", "Open Triangle"},
    {'P',   0,   0,  27, "GM Percussion Fallback", "High Q"},
    {'P',   0,   0,  28, "GM Percussion Fallback", "Slap"},
    {'P',   0,   0,  29, "GM Percussion Fallback", "Scratch Push"},
    {'P',   0,   0,  30, "GM Percussion Fallback", "Scratch Pull"},
    {'P',   0,   0,  31, "GM Percussion Fallback", "Sticks"},
    {'P',   0,   0,  32, "GM Percussion Fallback", "Square Click"},
    {'P',   0,   0,  33, "GM Percussion Fallback", "Metronome Click"},
    {'P',   0,   0,  34, "GM Percussion Fallback", "Metronome Bell"},
    {'P',   0,   0,  82, "GM Percussion Fallback", "Shaker"},
    {'P',   0,   0,  83, "GM Percussion Fallback", "Jingle Bell"},
    {'P',   0,   0,  84, "GM Percussion Fallback", "Bell Tree"},
    {'P',   0,   0,  85, "GM Percussion Fallback", "Castanets"},
    {'P',   0,   0,  86, "GM Percussion Fallback", "Mute Surdo"},
    {'P',   0,   0,  87, "GM Percussion Fallback", "Open Surdo"},
};
